/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date: 17/08/98                      */
/* Program: ACT_LOG                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This software is designed to indicate the length of a mains power        */
/* cut  or restart by SRT by periodically storing the latest time string    */
/* from the system clock.                                                   */
/* This information is continually written until a mains power failure      */
/* occurs or until a SRT is issued.                                         */
/* When power is resumed, or a STP/SRT is issued, the first time from the   */
/* system clock is stored in the /home/error/up1 file with a header.        */
/* This leaves the last time before the power failure or restart and the    */
/* first time after the failure or restart as stored records for reference  */
/* The new time stamps are stored in the following location in the file     */
/* until another power failure or restart occurs.                           */
/* When the file contains n records, the file is copied to /home/error/up2  */
/* and /home/error/up1 is removed and re-opened. This leaves a history of   */
/* n minutes at any time.                                                   */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include "filedef.h"
#include <time.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                          Date: 28/01/98    Version 7.00
------------------------------------------------------------------------------
Description: This routine is writing a time and date to file /home/error/up1
             whenever called. When file /home/error/up1 is full, the file is
             copied to /home/error/up2 and a new /home/error/up1 is opened.
             The program is activated from program cat_evt at restart and
             afterwards every minute. This way you get a log of uptime and
             downtime for the system for the last hours.

Usage:       act_log (0 - at restart, 1 - every minute)
#endif

FILE     *tims;                        /* Pointer to filename               */
int      set;                          /* argument from cat_evt             */
char     arg;
main(argc,argv)
int      argc;
char     **argv;
{
time_t time_of_day;
auto char buf[26];
long     filsize;                      /* filesize                          */
long int offset;                       /* filesize                          */
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"SEI_ACTI_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"SEI_ACTI_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
if(argc != 2)
{
    set=0;
}else{
    arg=*argv[1];
    switch(arg)
    {
        case '0':
        set=0;
        break;
        case '1':
        set=1;
        break;
    }
}
time_of_day=time(NULL);                /* get current cpu time              */
_ctime(&time_of_day,buf);
buf[24]=' ';
buf[25]=' ';

tims=fopen("/home/error/up1","a");     /* Open log-file                     */
filsize=ftell(tims);                   /* Get current filesize              */
offset=filsize;
if(filsize >=13000)
{
    fclose(tims);                      /* Yes, close file                   */
                                       /* Save file                         */
    system("cp /home/error/up1 /home/error/up2");
    system("rm -f /home/error/up1");   /* Delete old file                   */
}

tims=fopen("/home/error/up1","a");     /* Open log-file                     */
if(set==0)
    fprintf(tims,"SEISLOG started by SRT or power reset.\n");
fprintf(tims,"%s",_ctime(&time_of_day,buf));
fclose(tims);
}



