/******************************************************************************/
/*                          AUTO   PROGRAM                                    */
/*                Author:Jose Escobar && Jens Havskov                         */
/*                          UiB,UNAH                                          */
/*                       jjef@ns.unah.hondunet.net                            */
/*                       jose@ifjf.uib.no                                     */
/*                       jens.havskov@ifjf.uib.no                             */
/*                                                                            */
/*   This program read a file in SEISLOG format, find the phases for          */
/*   the event and print out a file containig the event information           */
/*   and phases in Nordic format.                                             */
/*                                                                            */
/*   It can take the input directly from the command line or open             */
/*   by default the file called "event".                                      */
/*                                                                            */
/*   INPUT: binary file in SEISLOG format                                     */
/*   OUTPUT: ascii file with the event information in Nordic Format           */
/*                                                                            */
/*                                                                            */
/*   date: 4.09.97                                                            */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/******************************************************************************/

#include<string.h>
#include<stdio.h>
#include<stdlib.h>
#include<errno.h>
#include<math.h>
#include<malloc.h>
#include<unistd.h>
#include "bpf.c"
#include "nr.c"

#define  NCHANNELS    50     /* define number of channel  */ 
#define  MAX_PHASES   10    /* define maximum number of phases per channel*/


void phase_time();
void wzero();
void werror();
void sta_lta();
float procoh1();
float cross();
void apply();
void design();
void wadati();
void mem();
float evlmem();
void three_comp();

/***************************************************************************/
/**********************Main Program*****************************************/

main(argc,argv)

 int argc;
 char *argv[];


{

 FILE *f1,*f2,*f3,*f4,*temp;

 int *data_block;     /*   Multiplexed data  (one block of data)  */

 int *data_int;

 int nheader,byte_length,nchan,nsamp,rlength;
 int unit,nblock,nhead,nsmp_buf,i,j,k,l;
 double rate;
 int samples_buffer_channel;
 char net_name[10];    /* name of the network */
 int year,dyear,day,month,hour,min;
 char tyear[10],tday[10],tmonth[10],thour[10],tmin[10],tsec[10];
 /* (up) dummy variables used mainly to read or write in formatted mode */

 int counter,status;  /* dummy variables */
 float ptime[10];     /* output of the phase_time subrotine */
 char wave;           /* dummy variable for write the output file */

 float sec;
 char staname[NCHANNELS*8];    /*  names and components of the stations */
 
 float threshold[NCHANNELS];  /* array of thresholds (one per channel) */
 float min_coda;              /* minimum coda duration in  seconds */
 float coda_detrigger[NCHANNELS];        /* array of de-trigger values */

 float iwindow,lwindow;  /* define window length in seconds */
                         /* for the coherence analisys */
                         /* length=lwindow+iwindow */

 float min_dur;    /* minimum duration of the trigger in seconds */ 

 int duration[NCHANNELS][MAX_PHASES+5];
 /* (up) duration of the phase */

 float maximum[NCHANNELS][MAX_PHASES+5];
/* (up) maximum amplitude of the phase */

 float min_rat_pow;  /* minimum ratio power */
 float ps_amp;       /* p s amplitude ratio */

 float pvel,psrat,min_coher,oprocoh[10];
/* (up) pvel: velocity of the P wave */
/* (up) psrat: Vp Vs ratio */
/* (up) min_coher :minimum coherence */
/* (up) oprocoh: output from procoh1 subrotine */

 float azimut[NCHANNELS],coher[NCHANNELS];
/* azimut and coherence after the three components analisys */

 float nsta[NCHANNELS],nlta[NCHANNELS],s_lta[NCHANNELS];
 float fc0[NCHANNELS],fc1[NCHANNELS];
/*(up) nsta[]: duration of the sort term average in seconds (one per channel)*/
/*(up) nlta[]: duration of the long term average in seconds (one per channel)*/
/*(up) s_lta[]: duration of the long trem average in seconds after the P */

/*(up) fc0[] : cut frequency (high pass) for the filter (one per channel) */
/*(up) fc1[] : cut frequency (low pass) for the filter (one per channel) */

 char ftr[NCHANNELS][5];   /* fileter*/
 char ftype[NCHANNELS][5];  /* filter type */
 int  cf[NCHANNELS]; /* coda filter */
 int  p_sh; /* filter phase shift */
 float fp_sh; /* dummy variable */

 int chasta[NCHANNELS]; /* array definig if the channel will be analized or not */
                    /* if chasta[]=1 analize channel, if chasta[]=0 not analize     */
                    /* if chasta[]=2 it will asume that it is the first component */
                    /* and it will also analize the next two components   */
                    /* like part of a three component station                */
                    /* if chasta[]=3 it will print the data in the screen */

 int npoles[NCHANNELS],phase[NCHANNELS][MAX_PHASES+5];
/* (up) npoles: number of poles for the filter (one per channel) */
/* (up) phases[][]: array containig the phases for [channel][fase_number] */

 char agency[10];
/* (up) agency name */

 char file_in[85];    /* name of the input file */
 char file_name[85];  /* name of the input file in SEISLOG format  */
                      /* WARNNING: if the name of the input file   */
                      /* is not in SEISLOG format, file_name going to */
                      /* write a name in SEISLOG format in the S-file */

 int checksum,gg;   
 char file_out[85];            /* output file name  */

 int asig_memory; /* flag for memory asignation */

 float *channel;   
/* (up)   data demultiplexed (component one if three stations)  */
 float *channel2, *channel3;
/* (up) components 2 and 3 for the three components analisys */
 
 int k_channel;           /*  current channel */

 int kk;   /*dummy variable*/
 char NO_S[30]; /* print possible S waves (y/n) */
 char aloc[30]; /* locate the event (y/n) */
 char prog[30]; /* locating program */
 char awad[30]; /* make wadati (y/n) */
 char wada_mes[50]; /* Wadati message */
 char cmd_line[200],tnchan[9];
 char asc_num[80]; /* dummy variable */

 char directory[80]; /* directory for ths s-files */
 char user[80]; /* mail to send the s-file */
 char quiet[30]; /* quiet mode (y/n) */
 char trans[30]; /* transfer file (y/n) */

 int SEISAN,jump,doy,num;
 char line_1[2000],stat1[10],stat2[10],stat3[10];
 float time_re[NCHANNELS],time_int[NCHANNELS];
 int mem_as;

 char machine[80],login[20],passw[20],direc[80];
 /* futuras variables para un ftp automatico */

mem_as=0;
SEISAN=0;
strcpy(directory,".");
strcpy(NO_S,"S-PHASES=NO");
strcpy(prog,"jepi");
strcpy(awad,"WADATI=NO");
strcpy(aloc,"LOCATE=NONE");
strcpy(trans,"MAIL=NO");
strcpy(quiet,"QUIET=YES");


for(i=0;i<NCHANNELS;i++){
    azimut[i]=0.0;
    coher[i]=0.0;
    for(j=0;j<MAX_PHASES+5;j++) {
        phase[i][j]=0;
        duration[i][j]=0;
        maximum[i][j]=0.0;
    };
        
};   /* reset variables */



/******************* Open the input file ***************************/

 status=0;


 if (argc==2){ 
    sprintf(cmd_line,"%s\0",argv[1]);
    f1=fopen(cmd_line,"rb");
    if(f1== NULL ){
       sprintf(cmd_line,"/home/events/%s",argv[1]); 
       f1=fopen(cmd_line,"rb");
       if(f1==NULL){
          printf("****Error openning file: %s****\n\n",cmd_line);
          exit(1);
       };
    };

    if(strcmp(quiet,"QUIET=NO")==0)
       printf(" Opening the input file : %s\n",argv[1]);
    strcpy(file_in,argv[1]);

 };
 if (argc!=2){
    if(strcmp(quiet,"QUIET=NO")==0) printf("Opennig the file :  EVENT");
    if ((f1=fopen("event","rb"))==(FILE *) NULL) {
      if ((f1=fopen("/home/events/event","rb"))==(FILE *) NULL ){
         printf("****Error openning file: /home/events/event****\n");
         exit(1);
      };
    };
 };



 if(status==0) { /* open file for error if s-file is open yet */
   
   strcpy(file_out,"auto.err");

   if ((f2=fopen(file_out,"w"))==(FILE *) NULL ){
       printf("error opening the error file\n Exiting to the system\n");  
       /* (up) open the parameter file */
       exit(1);
   };
 };


/* check file format */
 if(fread(&num,sizeof(int),1,f1)!=1 || num!=80) SEISAN=0;
 else SEISAN=1;

 rewind(f1);

/*************  Read the header of the input file ***************/
/*              if SEISLOG format */

if(SEISAN==0){

   fseek(f1,0L,0);
   if (fscanf(f1,"%s",net_name)!=1) 
       werror(1,file_out);
   fseek(f1,16L,0);
   if (fscanf(f1,"%2d %3d %2d %2d %2d %2d",&year,&dyear,&month,
      &day,&hour,&min)!=6) werror(1,file_out);
   fseek(f1,23L,0);
   if (fscanf(f1,"%2d",&month)!=1) werror(1,file_out);
   fseek(f1,38L,0);
   if (fscanf(f1,"%f",&sec)!=1) werror(1,file_out);   /* sec                           */
   fseek(f1,80L,0);
   if (fscanf(f1,"%8d",&nsamp)!=1) werror(1,file_out);/* no. of samples each channel   */
   fseek(f1,62L,0); 
   if (fscanf(f1,"%2d",&nchan)!=1) werror(1,file_out);/* number of channels            */
   fseek(f1,104L,0);
   if (fscanf(f1,"%8d",&rlength)!=1) werror(1,file_out);/* bytes per buffer              */
   fseek(f1,64L,0);
   if (fscanf(f1,"%7lf",&rate)!=1) werror(1,file_out); /* samplerate msecs              */
   fseek(f1,88L,0);
   if (fscanf(f1,"%1d",&byte_length)!=1) werror(1,file_out);/* bytes per sample              */
   fseek(f1,96L,0);
   if (fscanf(f1,"%2d",&unit)!=1) werror(1,file_out);  /* unit                          */
   fseek(f1,112L,0);
   if (fscanf(f1,"%5d",&nheader)!=1) werror(1,file_out); /* bytes in header               */
   fseek(f1,122L,0);
   if (fscanf(f1,"%6d",&samples_buffer_channel)!=1) werror(1,file_out); 
                                           /* samples per buffer per channel */
   fseek(f1,160L,0);


  if(fread(staname,sizeof(char)*8*nchan,1,f1)!=1) werror(1,file_out);

  /* Station names and components   */

 };


/*********** Read the header if SEISAN format ******************************/

if(SEISAN==1){

  net_name[29]='\0';
  stat1[8]='\0';
  stat2[8]='\0';
  stat3[8]='\0';
  line_1[1040]='\0';
  staname[0]='\0';


 
  if(fread(&num,sizeof(int),1,f1)!=1 ||
      fread(line_1,num*sizeof(char),1,f1)!=1 ||
      fread(&num,sizeof(int),1,f1)!=1 )
      werror(1,file_out);


  if(sscanf(line_1,"%*c%29c  %d %d %d %d %d %d %d %f",net_name,&nchan,&year,&doy,&month,
     &day,&hour,&min,&sec)!=9) werror(1,file_out); 


  /* read the second line */
  if(fread(&num,sizeof(int),1,f1)!=1) werror(1,file_out);
  if(fread(line_1,num*sizeof(char),1,f1)!=1) werror(1,file_out);
  if(fread(&num,sizeof(int),1,f1)!=1) werror(1,file_out);

  /* calculate the number of lines with information to read */
  if((float)(nchan)/3.0-(float)(nchan/3)!=0.0) jump=nchan/3+1;
  else jump=nchan/3;

  for(i=1;i<=jump;i++){

     time_re[i]=0.0;
     time_re[i+1]=0.0;
     time_re[i+2]=0.0;

     time_int[i]=0.0;
     time_int[i+1]=0.0;
     time_int[i+2]=0.0;


     if(fread(&num,sizeof(int),1,f1)!=1) werror(1,file_out);
     if(fread(line_1,num*sizeof(char),1,f1)!=1) werror(1,file_out);
     if(fread(&num,sizeof(int),1,f1)!=1) werror(1,file_out);

     if(sscanf(line_1,"%*c%8c %f %f%*c%8c %f %f%*c%8c %f %f",stat1,&time_re[i],&time_int[i],
            stat2,&time_re[i+1],&time_int[i+1],stat3,&time_re[i+2],&time_int[i+2])!=9)
            if(i!=jump) werror(1,file_out);
     strcat(staname,stat1);
     if(time_int[i+1]!=0.0) strcat(staname,stat2);
     if(time_int[i+2]!=0.0) strcat(staname,stat3);
  }; 


  /* jump the remaining lines */
  jump=jump+2;
  if(jump+2<=12) {
     jump=(12-jump);
     for(i=1;i<=jump;i++){
       if(fread(&num,sizeof(int),1,f1)!=1) werror(1,file_out);
       if(fread(line_1,num*sizeof(char),1,f1)!=1) werror(1,file_out);
       if(fread(&num,sizeof(int),1,f1)!=1) werror(1,file_out);
     };
  };

};


/************ Reading of the parameter file *********************************/


 if ((f3=fopen("parameter.def","r"))==(FILE *) NULL){
    if ((f3=fopen("/home/seislog/pr/parameter.def","r"))==(FILE *) NULL ){
        printf("error opening the parameter file\n Exiting to the system\n\n");  
        /* (up) open the parameter file */
        exit(1);
    };
 };

 /* Jump the header */
 while(strcmp(cmd_line,"HEADER_HELP")!=0) {
       if(fscanf(f3," %s ",cmd_line)!=1) werror(2,file_out);
 };

 if(fscanf(f3," %s ",directory)!=1) werror(2,file_out);
 sscanf(directory,"%*4c%s",directory);

 if(fscanf(f3," %s %s %s %s %s %s",agency,NO_S,awad,aloc,trans,quiet)!=6) 
    werror(2,file_out);

 if(strcmp(trans,"MAIL=NO")!=0) sscanf(trans,"%*5c%s",user);

 if(fscanf(f3," %f %f %f %f %f %f %f %f %f %f",&min_dur,&min_coda,&pvel,&psrat,
           &iwindow,&lwindow,&min_coher,&min_rat_pow,&ps_amp,&fp_sh)!=10) werror(2,file_out);

p_sh=(int)(fp_sh);

 /* Jump the header */
 while(strcmp(cmd_line,"SA")!=0) {
       if(fscanf(f3," %s ",cmd_line)!=1) werror(2,file_out);
 };

/* Reading of the frequencies an poles for the filter and the threshold       */
/* for the j-channel  */ 

 for (j=0;j<=nchan+1;j++){
    fc0[j]=1.0;
    fc1[j]=5.0;
    npoles[j]=4;
    threshold[j]=2.5;
    chasta[j]=0;                    /* reset all channels */
 };

 fscanf(f3,"%s",asc_num);
 j=atoi(asc_num);
 if(j<=0) werror(2,file_out);
 

 while(j>0) {
       if(fscanf(f3," %*s %*s %*s %f %f %f %f %f %f %f %d %s %s %d %d",
         &nsta[j],&nlta[j],&s_lta[j],&threshold[j],&coda_detrigger[j],&fc0[j],
         &fc1[j],&npoles[j], ftr[j],ftype[j],&cf[j],&chasta[j])!=12) 
               werror(2,file_out); 
       fscanf(f3,"%s",asc_num);
       j=atoi(asc_num);
 };


 if(chasta[nchan]==2 || chasta[nchan-1]==2 ){
    printf(" Last two channels can not be three components, check parameter file\n");
    exit(2);
 };

/****************************************************************************/

if(status==0) fclose(f2); /* close error file and continue writing errors */
                           /* in the s-file */


/**********************Open the output S-file   ******************************/


if(status==0){  /* Check if it is open  */

    wzero(year,tyear);
    wzero(day,tday);
    wzero(hour,thour);
    wzero(min,tmin);
    wzero((int)(sec),tsec);
    wzero(month,tmonth);

    sprintf(file_out,"%s/%2s-%2s%2s-%2sL.S%2s%2s",directory,tday,thour,tmin,tsec,tyear,tmonth);
    if(strcmp(quiet,"QUIET=NO")==0) printf(" Opening the output file:   %s\n\n",file_out);
    if ((f2=fopen(file_out,"w"))==(FILE *) NULL ){
        printf("****Error openning output file: %s\n****",file_out);  
        exit(1);
    };

};


/********************Get memory if SEISLOG format************/


if(SEISAN==0){
  /* Get memory to read a data block */

  data_block=(int *) calloc(nchan*(int)(1.0/rate)+100,sizeof(int));
  if(!data_block) {
     printf(" Not enougth memory to read a data block\n");
     printf(" Exiting the program\n");
     exit(1);
  };

  /* Get memory to read a data channel */

  channel=(float *) calloc(nsamp+100,sizeof(float));
  if(!channel) {
     printf(" Not enougth memory to read a data channel\n");
     printf(" Exiting the program\n");
     exit(1);
  };


  /*calculate the number of data blocks  */

   nhead=64/sizeof(int);
   nsmp_buf=rlength/sizeof(int);
   nblock=nsamp*nchan/(nsmp_buf-nhead); 

};

/*************************LOOP OVER ALL CHANNELS******************************/
/*************** Read and analize all the channels ***************************/


for (k_channel=1;k_channel<=nchan;k_channel++){ 
     /* analisys for the k_channel         */ 







  if(asig_memory==1){  /* free memory if it was asigned */
     free(channel2);
     free(channel3);
     asig_memory=0;
  };

  if(chasta[k_channel]==0) goto end_analisys; /* do not a analize */

  if(strcmp(quiet,"QUIET=NO")==0){
  printf("\n\n Analizing channel : %3d   ",k_channel);
  for(gg=8*(k_channel-1);gg<8*(k_channel-1)+8;gg=gg+1) printf("%c",staname[gg]);
  printf("\n");
  printf(" Filter : %4s   Filter tipe : %4s \n",ftr[k_channel],ftype[k_channel]);
  printf(" Low Pass frequency : %3.1f High Pass frequency : %3.1f",
          fc0[k_channel],fc1[k_channel]);
  printf(" Poles : %2d\n",npoles[k_channel]); 
  printf(" Time window for short term average : %5.2f sec \n",nsta[k_channel]);
  printf(" Time window for long term average  : %5.2f sec \n",nlta[k_channel]);
  printf(" Threshold level       : %8.2f \n",threshold[k_channel]);
  if(chasta[k_channel]==2) printf(" Minimum coherence : %5.2f\n",min_coher);
  printf(" Minimum power spectral ratio : %f\n",min_rat_pow);
  printf(" Coda de-trigger level : %8.2f \n",coda_detrigger[k_channel]);
  printf(" Minimum coda length   : %8.2f Seconds\n",min_coda);
  printf(" Total record length   : %8.2f Seconds\n",(float)(nsamp)*rate);
  };

  if(chasta[k_channel]==2){  /* check if this channel belong to a three component */
    if(strcmp(quiet,"QUIET=NO")==0)
       printf(" Analizing three components data with channels %d, %d and %d\n",
            k_channel,k_channel+1,k_channel+2); 

     /* Get memory to read the second component */

    channel2=(float *) calloc(nsamp+100,sizeof(float));
    if(!channel2) {
       printf(" Not enougth memory to read the second component\n");
       printf(" Exiting the program\n");
       exit(1);
    };

    /* Get memory to read a data channel */

    channel3=(float *) calloc(nsamp+100,sizeof(float));
    if(!channel3) {
       printf(" Not enougth memory to read the third component\n");
       printf(" Exiting the program\n");
       exit(1);
    };
    asig_memory=1;   /* memory asigned */
    chasta[k_channel+1]=0;
    chasta[k_channel+2]=0;  /* do not analize the next two components aguein  */
  };



/***************** Reading and demultiplexing of data if SEISLOG format       ****************/

if(SEISAN==0){

  fseek(f1,(long)(nheader),0); /* go to the begining of the data */


  l=0; 
  for (j=0;j<nblock;j++){

      /* Read the j-block of data*/
   
   if(fread(data_block,sizeof(int),nsmp_buf,f1)!=nsmp_buf) werror(3,file_out); 

     /* demultiplexing of the data block          */

    for (i=nhead;i<=nsmp_buf-1;i=i+nchan){

       channel[l]= (float)(data_block[i+k_channel-1]);

       if(chasta[k_channel]==2){ /* read the others two channels if three components */
         channel2[l]=(float)(data_block[i+k_channel]);
         channel3[l]=(float)(data_block[i+k_channel+1]);
       };

       if(chasta[k_channel]==3) printf("%d  %f\n",l,channel[l]);
       /* (up) output to the screen */

       l=l+1;

    };
  };

};

/***************** Reading and demultiplexing of data if SEISAN  format       ****************/

if(SEISAN==1){

     if(fread(&num,sizeof(int),1,f1)!=1) werror(1,file_out);
     if(fread(line_1,num*sizeof(char),1,f1)!=1) werror(1,file_out);
     if(fread(&num,sizeof(int),1,f1)!=1) werror(1,file_out);


     if(sscanf(line_1,"%*35c %lf %d",&rate,&nsamp)!=2)werror(1,file_out);
     rate=1.0/rate;

     /* get memory for data channel */


    channel=(float *) calloc(nsamp+100,sizeof(float));
    data_int=(int *) calloc(nsamp+100,sizeof(int));

     if(!channel || !data_int) {
        printf(" Not enougth memory to read a data channel\n");
        printf(" Exiting the program\n");
        exit(1);
     };

     if(fread(&num,sizeof(int),1,f1)!=1) werror(3,file_out);
     if(fread(data_int,sizeof(int),nsamp,f1)!=nsamp) werror(3,file_out);
     if(fread(&num,sizeof(int),1,f1)!=1) werror(3,file_out);

     for(i=0;i<nsamp;i++){
          channel[i]=(float)(data_int[i]);
          if(i<100) printf("%f\n",channel[i]);
     };
 
};

/***********************  Search for phases and coda   *********************/



   if(chasta[k_channel]==1 || chasta[k_channel]==4) 

      sta_lta(channel,k_channel,npoles[k_channel],ftr[k_channel],ftype[k_channel],
              fc0[k_channel],fc1[k_channel],cf[k_channel],nsamp,rate,
              nsta[k_channel],nlta[k_channel],s_lta[k_channel],min_dur,
              min_coda,threshold,coda_detrigger,min_rat_pow,
              ps_amp,p_sh,phase,duration,maximum,chasta[k_channel]);  
 
   if(chasta[k_channel]==2){  /* analize like three components  */

     three_comp(channel,channel2,channel3,k_channel,npoles,ftr,ftype,fc0,fc1,
                cf,nsamp,rate,nsta,nlta,s_lta,min_dur,min_coda,threshold,
                coda_detrigger,phase,duration,maximum,pvel,psrat,iwindow,
                lwindow,min_coher,min_rat_pow,ps_amp,p_sh,oprocoh);

     azimut[k_channel]=oprocoh[1];
     coher[k_channel]=oprocoh[0];

   };

   if(chasta[k_channel]==3) for(i=0;i<nsamp;i++) printf("%f\n",channel[i]);



   end_analisys: printf(""); 

 /* End of the analisys for the k_channel           */
 if(SEISAN==1){
    free(channel);
    free(data_int);
 };  

};

strcpy(wada_mes,"NONE");
if(strcmp(awad,"WADATI=YES")==0){
    /* make wadati analisys */
    wadati(phase,pvel,psrat,rate,wada_mes); 
};

/***********************Output**********************************************/ 

checksum=nheader+nblock*rlength*byte_length;

wzero(nchan,tnchan);
wzero(year,tyear);
wzero(day,tday);
wzero(hour,thour);
wzero(min,tmin);
wzero((int)(sec),tsec);
wzero(month,tmonth);


/* print the file name in SEISAN format */
file_name[0]='\0';

fprintf(f2," %4d %2d%2d %2d%2d %4.1f",year,month,day,hour,min,sec);

fprintf(f2," L                       %3s                               1\n",agency);
fprintf(f2," Phases found by the sta-lta routine   (auto program)                          3\n"); 

fprintf(f2," Wadati message : %35s                          3\n",wada_mes);

if(SEISAN==0) {
/* print the file name in SEISLOG format */
sprintf(file_name,"%2s%2s_%2s_%2s%2s_%2sT.%3s_%2s_1\0",tyear,tmonth,tday,thour,
        tmin,tsec,net_name,tnchan);

fprintf(f2," NET WAV CHK %3s  QNX %s        %7d   CHECKSUM      3\n",agency,file_name,checksum);
fprintf(f2," NET WAV EVE %3s  QNX %s        STA       REQUEST       3\n",agency,file_name,checksum);


fprintf(f2," %s                                                     6\n",
        file_name);

/* print the name in SEISAN format */
sprintf(file_name,"%2s%2s-%2s-%2s%2s-%2sS.%3s_%2s",tyear,tmonth,tday,thour,
        tmin,tsec,net_name,tnchan);
/* print the file name in SEISAN format */
fprintf(f2," %s                                                       6\n",
        file_name);

} else {
        fprintf(f2," %s",file_in);
        for(i=1;i<=78-strlen(file_in);i++) fprintf(f2," ");
        fprintf(f2,"6\n");
 };
 
/* print the file name  */



fprintf(f2," STAT SP IPHASW D HRMM SECON CODA AMPLIT PERI AZIMU VELO SNR AR TRES W  DIS CAZ7\n");
counter=0;
for(i=1;i<=nchan;i++){

  if(chasta[i]==0) counter=counter+8; /* use to print station name */

  if(chasta[i]!=0){  /* do not print it if it was not used */
  if(strcmp(NO_S,"S-PHASES=YES")==0) kk=2;
  else kk=1;
    for(k=0;k<kk;k++){
      if(phase[i][k]!=0){ /* do not print it if there is no phase */
         wave='P';
         if(k==1) wave='S';
         fprintf(f2," ");
         for(j=counter;j<4+counter;j++)fprintf(f2,"%c",staname[j]);
         /* print the station name */

         fprintf(f2," %c%c",staname[4+counter],staname[7+counter]);
         /* print component */

         phase_time(year,month,day,hour,min,sec,rate,phase[i][k],ptime);
         /* convert sample phase in year, month, day, hour, min, sec */


         fprintf(f2," I%c       %2.f%2.f %5.2f",wave,ptime[2],ptime[1],ptime[0]);
         /* print phase time */

         if(k==0){
           fprintf(f2," %4d",phase[i][MAX_PHASES]/(int)(1.0/rate));
           /* print coda */
           /* it is asumed that the coda is the last phase in the array */


           if(coher[i]>=0.5 && coher[i]<=1.0) 
              fprintf(f2,"             %5.1f\n",azimut[i]);
           else fprintf(f2,"\n");
           /* print azimut */

         };
         if(k==1) fprintf(f2,"\n"); 
      };
    };
    counter=counter+8;
  };
};

fprintf(f2,"                                                                                \n");
/* print blank line */

/* close files and free memory */
fclose(f3);
fclose(f2);
fclose(f1);
if(SEISAN==0) free(data_block);
free(channel);

if(strcmp(aloc,"LOCATE=NONE")!=0) {
   sscanf(aloc,"%*7c%s",prog);
   if(strcmp(quiet,"QUIET=NO")==0) 
      printf("\n\n ******* Locating event using %s *********\n",prog);
   /* locate event  */
   sprintf(cmd_line,"%s %s \n",prog,file_out);
   system(cmd_line); 
};

if(strcmp(trans,"MAIL=YES")==0) {
   if(strcmp(quiet,"QUIET=NO")==0) printf("\n\n ****** Mailing data *******\n");
   sprintf(cmd_line,"mail %s < %s ",user,file_out);
   system(cmd_line);
};

if(strcmp(quiet,"QUIET=NO")==0){
   sprintf(cmd_line,"\n\nmore %s",file_out);
   system(cmd_line);
};


}


/***************************************************************************/
/*********************subrutine cross***************************************/

/* this subrotine perform the cross correlation between two set of data(x,y) */


float   cross(x,y,l,ipts)

  float x[],y[];
  int l,ipts;

{

  float a=0.0;
  int i;
  
  for (i=ipts;i<=l+ipts;i++) a=a+x[i]*y[i];
      a=a+x[i]*y[i];
  return a;

}


/***************************************************************************/
/*********************subrutina procoh1*************************************/

float   procoh1(channel,channel2,channel3,ipts,lpts,pvel,psrat,oprocoh)

float channel[],channel2[],channel3[];
float pvel,psrat,oprocoh[10];
int ipts,lpts;


/*
  S/R to do 3-component azimuth and velocity estimation for a P-phase
  for a fixed user specified time window.
  Method: predicted coherence (R. Roberts, Uppsala)
*/

{

 
  float raddeg=57.2958;
  int lenwin=0,ii;
  float xx,xy,xz,yy,yx,yz,zz,zx,zy;
  float power,azi,zoverr,a,b,err,cc,ai;
  float svel;
  float coher,azim,velo,rmsamp;


  lenwin=lpts-ipts+1;
  xx=cross(channel2,channel2,lenwin,ipts);
  xy=cross(channel2,channel3,lenwin,ipts); 
  xz=cross(channel2,channel,lenwin,ipts);
  yy=cross(channel3,channel3,lenwin,ipts);
  yz=cross(channel3,channel,lenwin,ipts);
  zz=cross(channel,channel,lenwin,ipts);
  yx=xy;
  zx=xz;
  zy=yz;
  power=(xx+yy+zz)/(float)lenwin;
  rmsamp=sqrt(power);
  azi=atan2(-yz,-xz);
  azim=azi*raddeg;
  if (azim<0) azim=azim+360.0;
  zoverr=zz/sqrt(xz*xz+yz*yz);
  a=-zoverr*cos(azi);
  b=-zoverr*sin(azi);
   
  err=0.0; 
  for (ii=ipts;ii<=lpts;ii++){
      cc=channel[ii]-a*channel2[ii]-b*channel3[ii];
      err=err+cc*cc;
  };
  coher=1.0-err/zz;
  svel=pvel/psrat;
  ai=atan(1.0/zoverr);
  velo=svel/sin(ai/2.0);

  oprocoh[0]=coher;
  oprocoh[1]=azim;
  oprocoh[2]=velo;
  oprocoh[3]=rmsamp;

  return(coher);
} 


/****************************************************************************/
/***********************subrutine sta_lta************************************/


void sta_lta(channel,k_channel,npoles,ftr,ftype,fc0,fc1,cf,nsamp,rate,
             nsta,nlta,s_lta,min_dur,min_coda,threshold,coda_detrigger,
             min_rat_pow,ps_amp,p_sh,phase,dur,max,check_out)

char ftr[],ftype[];
double rate;
int k_channel,nsamp,phase[NCHANNELS][MAX_PHASES+5],npoles,cf,p_sh;
float nsta,nlta,min_dur,min_coda,threshold[],coda_detrigger[];
float s_lta,channel[],fc0,fc1,min_rat_pow,ps_amp; 
int dur[NCHANNELS][MAX_PHASES+5];
float max[NCHANNELS][MAX_PHASES+5];
int check_out;


{

   float *data_coda;
   float dx,w,z,n,keep_lta,fdt;
   float dxc,wc,zc,nc,average;
   /* variables ending with c are refered to the coda */
   int i,m,mm,nn,coda,trigger,k,ant;
   float power_ant,power_pos,pm,cof[20],amp_ratio;
   float *data;

   float xx,yy,s_ratio[MAX_PHASES+5],ss;

   FILE *f1,*f2,*f3;

/*  test output, it must be removed afterwards */
/*  check_out=number of the channel to print */
/*  if check_out=0 no channel to print */

   if(check_out==4) f1=fopen("sta_lta.dat","w");
    /* (up) open the output file for test, it must be removed afterwards */


   f2=fopen("wadati.dat","a+");
   if(f2==NULL){
      printf(" Unable to open temporary file : Wadati analisys\n");
      printf(" will not be possible or reliable \n");
   };

   /* get memory for filtered data for spectral analisys */
   data=(float *) calloc(nsamp+100,sizeof(float));
   if(!data) {
      printf(" Not enough memory for spectral analisys\n");
      return;
   }

   /* get memory for filtered data for coda search */
   data_coda=(float *) calloc(nsamp+100,sizeof(float));
   if(!data_coda) {
      printf(" Not enough memory search for  coda length\n");
      return;
   }
   else {

      /* filter data for coda */

      for(i=0;i<=nsamp+10;i++) data_coda[i]=channel[i];

      switch (cf){
        case 1:
           design(npoles,ftr,ftype,30.0,0.3,0.01,0.1,rate); 
           apply(data_coda,nsamp-1,p_sh);       
           break;
        case 2:
           design(npoles,ftr,ftype,30.0,0.3,0.1,1.0,rate); 
           apply(data_coda,nsamp-1,p_sh);       
           break;
        case 3:
           design(npoles,ftr,ftype,30.0,0.3,1.0,5.0,rate); 
           apply(data_coda,nsamp-1,p_sh);       
           break;
        case 4:
           design(npoles,ftr,ftype,30.0,0.3,2.0,4.0,rate); 
           apply(data_coda,nsamp-1,p_sh);       
           break;
        case 5:
           design(npoles,ftr,ftype,30.0,0.3,5.0,10.0,rate); 
           apply(data_coda,nsamp-1,p_sh);   
           break;    
        case 6:
           design(npoles,ftr,ftype,30.0,0.3,10.0,15.0,rate); 
           apply(data_coda,nsamp-1,p_sh);    
           break;   
        case 7:
           design(npoles,ftr,ftype,30.0,0.3,15.0,24.0,rate); 
           apply(data_coda,nsamp-1,p_sh);    
           break;   
        default:
           printf(" No filter for coda \n");
           break;
        };
   };



/*  filter the data */   

   if(npoles!=0){
     design(npoles,ftr,ftype,30.0,0.3,fc0,fc1,rate); 
     apply(channel,nsamp-1,p_sh);       
   };


   nn=(int)(nsta/rate);   /* number of samples for the short term average  */
   mm=(int)(nlta/rate);   /* number of samples for the long term average   */

   z=0.0;
   w=0.0;
   m=0;
   coda=0;   /* reset coda value */
   trigger=0;  /* de-trigger */
   keep_lta=1000.0;  /* high value in order to avoid coda trigger */
                     /* before the first phase */

   for(i=100;i<=350;i++) {
       z=z+fabs(channel[i]); 
   };

   average=0.0;
   for(i=100;i<=nsamp-100;i++) average=average+fabs(channel[i]);
   average=average/(float)(nsamp-200);

   /* (up) first 100 samples skipped due to the oscilation produced by the filter */
   /* (up) start to calculate the average with the next 250 points */
  
   zc=zc/250.0;
   wc=zc; 
   z=z/250.0;   /* initial value for the long term average  */
   w=z;   /* initial value for the short term average (same value) */ 

   if(check_out==4) 
   printf(" Trig_Time   Power_ant     Power_pos    Power_rat    Trig_dur\n"); 
 
   for(i=100;i<nsamp-100;i++){  

     dx=fabs(channel[i]-channel[i-1]);
     w=w+(dx-w)/(float)(nn);
        /* (up) calculate the new  short term average */
     z=z+(w-z)/(float)(mm);
        /* (up) calculate the new long term average */

     n=w/z; /* calculate the ratio */

     dxc=fabs(data_coda[i]-data_coda[i-1]);
     wc=wc+(dxc-wc)/(float)(nn);
        /* (up) calculate the new  short term average */
     zc=zc+(wc-zc)/(float)(mm);
        /* (up) calculate the new long term average */

     nc=wc/zc; /* calculate the ratio */

     if(check_out==4 ) 
        fprintf(f1,"%d %f %f %f %f %f\n",i,channel[i],w,z,n,w/keep_lta);
     /* (up) print output test, it must be removed afterwards */

     if(n<threshold[k_channel] && trigger==1) {
        trigger=0;
        /* (up) de-trigger */

        /* calculate espectral power  before the trigger */
        for(k=phase[k_channel][m-1]-200;k<=phase[k_channel][m-1];k++) 
            data[k-phase[k_channel][m-1]+200]=channel[k];
        memcof(data,200,10,&pm,cof);
        power_ant=0.0;
        for(fdt=fc0;fdt<=fc1; fdt += fc1*rate)
            power_ant=power_ant+evlmem(fdt,cof,10,pm);

 
        /* calculate espectral power  after the trigger */
        for(k=phase[k_channel][m-1];k<=phase[k_channel][m-1]+100;k++)
            data[k-phase[k_channel][m-1]]=channel[k];
        memcof(data,100,10,&pm,cof);
        power_pos=0.0;
        for(fdt=fc0;fdt<=fc1; fdt += fc1*rate)
            power_pos=power_pos+evlmem(fdt,cof,10,pm);

        if(check_out==4)
        printf(" %8.2f %12.4f  %12.4f     %8.3f       %5.2f",(float)(i)*rate,
               power_ant,power_pos,2.0*power_pos/power_ant,
               (float)(dur[k_channel][m-1])*rate); 

        if(m-2<0) {
           ant=0;
           amp_ratio=threshold[k_channel];
        }
        else {
           ant=phase[k_channel][m-2];
           amp_ratio=fabs(max[k_channel][m-1]/max[k_channel][m-2]);
        };

        if(dur[k_channel][m-1]<(int)(min_dur/rate) || 
           phase[k_channel][m-1]-ant<(int)(1.0/rate) ||
           2.0*power_pos/power_ant< min_rat_pow ||
           amp_ratio<ps_amp){ /* Spectrum check */
           s_ratio[m-1]=0.0;
           m=m-1;
           if(check_out==4) printf("\n");
           if(m==0){
              keep_lta=10000.0;
              coda=0;
           };
       /*    printf("\n"); */
           /* (up) if duration is less than the minimal duration */
           /* or the time diference between the current phase and */
           /* the previous phase is less than one second */
           /* do not use it like phase */
        }
        else {
            if(check_out==4) printf(" Phase assumed \n");
            if(m-1==0) mm=(int)(s_lta/rate);
            /* (up) change long term average after p */
        };
     };    

     if( wc/keep_lta>=coda_detrigger[k_channel]) {
        coda=coda+1; /* calculate coda */
     }

     if (n>=threshold[k_channel] ){

           if(trigger==0 ){
               phase[k_channel][m]=i; /* m phase */
               s_ratio[m]=n; /* keep sta-lta average */
               if(m==0){
                  keep_lta=zc; /* freeze long term average */
               };
               trigger=1;
               m=m+1;
           };
           if(fabs(channel[i]) >= fabs(max[k_channel][m-1])) 
              max[k_channel][m-1]=channel[i];
           /* (up) find the maximum amplitude */
           dur[k_channel][m-1]=i-phase[k_channel][m-1];
           /* (up) find the trigger duration */
     };
   };

   /* if coda is less than the minimal coda delete all the phases */
   if((float)(coda)*rate < min_coda){
       for(i=0;i<=MAX_PHASES+1;i++) phase[k_channel][i]=0;
       m=0;
       coda=0;
   };

   /* if phase is far than the coda, delete phase */
   if(m>=0){
      for(i=0;i<=m+1;i++) {
        if(phase[k_channel][i]>coda+phase[k_channel][0]) 
           phase[k_channel][i]=0;
      };
   /* if P is in the last 1/3 length of the record delete all phases */
      if(nsamp-phase[k_channel][0]<nsamp/3){
         for(i=0;i<=MAX_PHASES+1;i++) phase[k_channel][i]=0;
         m=0;
         coda=0;
      };
        
   };

   /* delete coda if there are no phases */
   if(m==0 && phase[k_channel][0]==0){
      coda=0;
      m=m-1;
   };

   ss=0.0; 
 
   if(m>=0 && phase[k_channel][0]!=0){
      i=1;
      do {
          xx=(float)(phase[k_channel][0]);
          yy=(float)(phase[k_channel][i])-xx;
          if(f2!=NULL) fprintf(f2,"%d %f %f\n",k_channel,xx,yy);
          if(i==1 && s_ratio[1]!=0.0) ss=yy;
          if(( s_ratio[i] > s_ratio[i-1]) && i >2 ) ss=yy;
          i=i+1;
      }while(phase[k_channel][i]!=0);

/*      if((f2!=NULL) && (ss!=0.0)) fprintf(f2,"%d %f %f\n",k_channel,xx,ss);*/
      fclose(f2);
   
      phase[k_channel][MAX_PHASES]=coda;  /* asign coda to the array */

 /*     printf(" Possible phases : %d Coda: %d\n\n",m+1,phase[k_channel][MAX_PHASES]); */
 
   };

   if(m==0 &&  phase[k_channel][0]==0)
/*      printf(" Possible phases : %d Coda: %d\n\n",m,phase[k_channel][MAX_PHASES]); */

   free(data);
   free(data_coda);

   if(check_out==4 ) fclose(f1); 
   /* (up) close the file for the output test, it must be removed afterwards */
}

void wzero(number,tnumber)

/* input: number ( a number less than 100) */
/* output: tnumber ( a string number with a zero in the left ) */  
/*         used to write in formatted mode  */


/*Esta subroutina recibe un numero menor que 100 y */
/* escribe un 0 delante del numero si este menor que 9 */
/* regresa el resultado en un "string" llamado tnumber */ 

int number;
char tnumber[];

{

if (number<10){

tnumber[0]='0';
tnumber[2]='\0';
switch(number)

{
 case 0:
      tnumber[1]='0';
      break;
 case 1:
      tnumber[1]='1';
      break;
 case 2:
      tnumber[1]='2';               
      break;
 case 3:
      tnumber[1]='3';                 
      break;
 case 4:
      tnumber[1]='4';                
      break;
 case 5:
      tnumber[1]='5';                  
      break;
 case 6:
      tnumber[1]='6';                
      break;
 case 7:
      tnumber[1]='7';                
      break;
 case 8:
      tnumber[1]='8';                 
      break;
 case 9:
      tnumber[1]='9';                 
      break;
 default:
      printf("eror ****");
      break;
};
};

if(number>99) {
   printf("\n\n Error in the wzero routine \n\n");
   exit(2);
};
if(number<100 && number>=10){
    sprintf(tnumber,"%2d",number);
    tnumber[3]='\0';
};

}
 
/******************************************************************************/
void phase_time(year,month,day,hour,min,sec,rate,sample_phase,ptime)

/* input: year, month, day, hour, min, */
/* sec, rate  and sample_phase. */

/* ouput time phase in
   year, month,day,hour, min,sec.                     
   ptime[5]=year, ptime[4]=month, ptime[3]=day, ptime[2]=hour
   ptime[1]=minute, ptime[0]=sec                   */
 
int month,day,hour,min,sample_phase;
float sec,rate,ptime[];
{
float time;
time=(float)(sample_phase)*rate;
ptime[2]=(float)((int)(time/3600.0));
ptime[1]=(float)((int)(time/60.0-(float)(ptime[2])*60.0));
ptime[0]=time-ptime[2]*3600.0-ptime[1]*60;
ptime[0]=ptime[0]+sec;
if (ptime[0]>=60.0){
    ptime[0]=ptime[0]-60.0;
    ptime[1]=ptime[1]+1.0;
};
ptime[1]=ptime[1]+min;
if (ptime[1]>=60.0){
    ptime[1]=ptime[1]-60.0;
    ptime[2]=ptime[2]+1.0;
};
ptime[2]=ptime[2]+hour;
ptime[3]=(float)(day);
if (ptime[2]>=24.0){
    ptime[2]=ptime[2]-24.0;
    ptime[3]=1.0+(float)(day);
};
ptime[4]=(float)(month);
if(year<100) year=year+1900;
ptime[5]=(float)(year);

if ((ptime[3]!=(float)(day)) && (ptime[3]>30.0)){
   switch (month)
   {
          case 1:
          if (ptime[3]>31.0){
              ptime[3]=1.0;
              ptime[4]=2.0;
          };
          break;
          case 3:
          if (ptime[3]>31.0){
              ptime[3]=1.0;
              ptime[4]=4.0;
          };
          break;
          case 5:
          if (ptime[3]>31.0){
              ptime[3]=1.0;
              ptime[4]=6.0;
          };
          break;
          case 7:
          if (ptime[3]>31.0){
              ptime[3]=1.0;
              ptime[4]=8.0;
          };
          break;
          case 8:
          if (ptime[3]>31.0){
              ptime[3]=1.0;
              ptime[4]=9.0;
          };
          break;
          case 10:
          if (ptime[3]>31.0){
              ptime[3]=1.0;
              ptime[4]=11.0;
          };
          break;
          case 12:
          if (ptime[3]>31.0){
              ptime[3]=1.0;
              ptime[4]=1.0;
              ptime[5]=ptime[5]+1.0;
          };
          break;
          default:
              ptime[3]=1.0;
              ptime[4]=ptime[4]+1.0;
          break;
   };
};
if (ptime[4]==2.0){
    if ((float)(4*(int)(ptime[5]/4.0))==ptime[5]){
       if (ptime[3]>29.0){
           ptime[3]=1.0;
           ptime[4]=3.0;
       };
    };
    if ((float)(4*(int)(ptime[5]/4.0))!=ptime[5]){ 
       if (ptime[3]>28.0){
           ptime[3]=1.0;
           ptime[4]=3.0;
       };
    };
};
}       

void werror(error_code,file_out)
/* input: error code */
/* output: exit the program */

int error_code;
char file_out[];

{

 FILE *f5;


   f5=fopen(file_out,"a+");
   if(f5==NULL){
      printf(" Unable to open error file %s\n",file_out);
      printf(" Exiting to the system\n");
      exit(0);
   };
 
  
 if(error_code==1){
   printf(" Error reading the header\n Exiting to the system\n");
   fprintf(f5," DETPHASE ERROR: 1    READING HEADER OF INPUT FILE                             3\n");
   exit(1);
 };
 if(error_code==2){
   printf(" Error reading the parameter file\n Exiting to the system\n");
   fprintf(f5," DETPHASE ERROR: 2    READING PARAMETER FILE                                  3\n");
   exit(2);
 };
 if(error_code==3){
   printf(" Error reading the data\n Exiting to the system\n");
   fprintf(f5," DETPHASE ERROR: 3    READING DATA                                             3\n");
   exit(3);
 };

}


void three_comp(c1,c2,c3,k_channel,npoles,ftr,ftype,fc0,fc1,cf,nsamp,rate,
                nsta,nlta,s_lta,min_dur,min_coda,threshold,coda_detrigger,
                phase,dur,maximum,pvel,psrat,iwindow,lwindow,min_coher,
                min_rat_pow,ps_amp,p_sh,oprocoh)

char  ftr[NCHANNELS][5],ftype[NCHANNELS][5];
float c1[],c2[],c3[],rate,nsta[],nlta[],s_lta[],min_dur,min_coda,threshold[];
float coda_detrigger[],fc0[],fc1[],ps_amp;
int k_channel,phase[NCHANNELS][MAX_PHASES+5],nsamp,npoles[],cf[];
int dur[NCHANNELS][MAX_PHASES+5],p_sh;
float maximum[NCHANNELS][MAX_PHASES+5],min_rat_pow;
float oprocoh[],pvel,psrat,iwindow,lwindow,min_coher;

{



  float *data,*data_coda1,*data_coda2,*data_coda3;
  float average1,average2,average3;
  float dx1,dx2,dx3,n1,n2,n3,n4,w1,w2,w3,z1,z2,z3,keep_lta1;
  float keep_lta2,keep_lta3,fdt,power_ant,power_pos;
  float dxc1,dxc2,dxc3,wc1,wc2,wc3,zc1,zc2,zc3,nc1,nc2,nc3;
  float nn1,nn2,nn3,mm1,mm2,mm3,amp_ratio;
  float p_coher,p_azim,pm,cof[20],freq1,freq2,keep_coher,keep_azim;
  int i,m,coda1,coda2,coda3,ipts,lpts;
  int trigger1,trigger2,trigger3,trigger4,trigger_duration;
  int sum,trigger_tot,ant,k,del;
  float xx,yy;
  FILE *f2;


   f2=fopen("wadati.dat","a+");
   if(f2==NULL){
      printf(" Unable to open temporary file : Wadati analisys\n");
      printf(" will not be possible or reliable \n");
   };

   /* get memory for filtered data for spectral analisys */
   data=(float *) calloc(nsamp+100,sizeof(float));
   if(!data) {
      printf(" Not enougth for spectral analisys\n");
      return;
   }

   /* get memory for filtered data for coda search */
   data_coda1=(float *) calloc(nsamp+100,sizeof(float));
   if(!data_coda1) {
      printf(" Not enougth memory search for  coda length\n");
      return;
   }
   else {

      /* filter data for coda */

      for(i=0;i<=nsamp+10;i++) data_coda1[i]=c1[i];

      switch (cf[k_channel]){
        case 1:
           design(npoles[k_channel],ftr[k_channel],ftype[k_channel],30.0,
                  0.3,0.01,0.1,rate); 
           apply(data_coda1,nsamp-1,p_sh);       
           break;
        case 2:
           design(npoles[k_channel],ftr[k_channel],ftype[k_channel],30.0,
                  0.3,0.1,1.0,rate); 
           apply(data_coda1,nsamp-1,p_sh);       
           break;
        case 3:
           design(npoles[k_channel],ftr[k_channel],ftype[k_channel],30.0,
                  0.3,1.0,5.0,rate); 
           apply(data_coda1,nsamp-1,p_sh);       
           break;
        case 4:
           design(npoles[k_channel],ftr[k_channel],ftype[k_channel],30.0,
                  0.3,2.0,4.0,rate); 
           apply(data_coda1,nsamp-1,p_sh);       
           break;
        case 5:
           design(npoles[k_channel],ftr[k_channel],ftype[k_channel],30.0,
                  0.3,5.0,10.0,rate); 
           apply(data_coda1,nsamp-1,p_sh);   
           break;    
        case 6:
           design(npoles[k_channel],ftr[k_channel],ftype[k_channel],30.0,
                  0.3,10.0,15.0,rate); 
           apply(data_coda1,nsamp-1,p_sh);    
           break;   
        case 7:
           design(npoles[k_channel],ftr[k_channel],ftype[k_channel],30.0,
                  0.3,15.0,24.0,rate); 
           apply(data_coda1,nsamp-1,p_sh);    
           break;   
        default:
           printf(" No filter for coda \n");
           break;
        };
   };

   /* get memory for filtered data for coda search */
   data_coda2=(float *) calloc(nsamp+100,sizeof(float));
   if(!data_coda2) {
      printf(" Not enougth memory search for  coda length\n");
      return;
   }
   else {

      /* filter data for coda */

      for(i=0;i<=nsamp+10;i++) data_coda2[i]=c2[i];

      switch (cf[k_channel+1]){
        case 1:
           design(npoles[k_channel+1],ftr[k_channel+1],ftype[k_channel+1],30.0,
                  0.3,0.01,0.1,rate); 
           apply(data_coda2,nsamp-1,p_sh);       
           break;
        case 2:
           design(npoles[k_channel+1],ftr[k_channel+1],ftype[k_channel+1],30.0,
                  0.3,0.1,1.0,rate); 
           apply(data_coda2,nsamp-1,p_sh);       
           break;
        case 3:
           design(npoles[k_channel+1],ftr[k_channel+1],ftype[k_channel+1],30.0,
                  0.3,1.0,5.0,rate); 
           apply(data_coda2,nsamp-1,p_sh);       
           break;
        case 4:
           design(npoles[k_channel+1],ftr[k_channel+1],ftype[k_channel+1],30.0,
                  0.3,2.0,4.0,rate); 
           apply(data_coda2,nsamp-1,p_sh);       
           break;
        case 5:
           design(npoles[k_channel+1],ftr[k_channel+1],ftype[k_channel+1],30.0,
                  0.3,5.0,10.0,rate); 
           apply(data_coda2,nsamp-1,p_sh);   
           break;    
        case 6:
           design(npoles[k_channel+1],ftr[k_channel+1],ftype[k_channel+1],30.0,
                  0.3,10.0,15.0,rate); 
           apply(data_coda2,nsamp-1,p_sh);    
           break;   
        case 7:
           design(npoles[k_channel+1],ftr[k_channel+1],ftype[k_channel+1],30.0,
                  0.3,15.0,24.0,rate); 
           apply(data_coda2,nsamp-1,p_sh);    
           break;   
        default:
           printf(" No filter for coda \n");
           break;
        };
   };

   /* get memory for filtered data for coda search */
   data_coda3=(float *) calloc(nsamp+100,sizeof(float));
   if(!data_coda3) {
      printf(" Not enougth memory search for  coda length\n");
      return;
   }
   else {

      /* filter data for coda */

      for(i=0;i<=nsamp+10;i++) data_coda3[i]=c3[i];

      switch (cf[k_channel+2]){
        case 1:
           design(npoles[k_channel+2],ftr[k_channel+2],ftype[k_channel+2],30.0,
                  0.3,0.01,0.1,rate); 
           apply(data_coda3,nsamp-1,p_sh);       
           break;
        case 2:
           design(npoles[k_channel+2],ftr[k_channel+2],ftype[k_channel+2],30.0,
                  0.3,0.1,1.0,rate); 
           apply(data_coda3,nsamp-1,p_sh);       
           break;
        case 3:
           design(npoles[k_channel+2],ftr[k_channel+2],ftype[k_channel+2],30.0,
                  0.3,1.0,5.0,rate); 
           apply(data_coda3,nsamp-1,p_sh);       
           break;
        case 4:
           design(npoles[k_channel+2],ftr[k_channel+2],ftype[k_channel+2],30.0,
                  0.3,2.0,4.0,rate); 
           apply(data_coda3,nsamp-1,p_sh);       
           break;
        case 5:
           design(npoles[k_channel+2],ftr[k_channel+2],ftype[k_channel+2],30.0,
                  0.3,5.0,10.0,rate); 
           apply(data_coda3,nsamp-1,p_sh);   
           break;    
        case 6:
           design(npoles[k_channel+2],ftr[k_channel+2],ftype[k_channel+2],30.0,
                  0.3,10.0,15.0,rate); 
           apply(data_coda3,nsamp-1,p_sh);    
           break;   
        case 7:
           design(npoles[k_channel+2],ftr[k_channel+2],ftype[k_channel+2],30.0,
                  0.3,15.0,24.0,rate); 
           apply(data_coda3,nsamp-1,p_sh);    
           break;   
        default:
           printf(" No filter for coda \n");
           break;
        };
   };

/*  filter the data */   

   if(npoles[k_channel]!=0){
     design(npoles[k_channel],ftr[k_channel],ftype[k_channel],30.0,0.3,
            fc0[k_channel],fc1[k_channel],rate); 
     apply(c1,nsamp-1,p_sh);       
   };

   if(npoles[k_channel+1]!=0){
     design(npoles[k_channel+1],ftr[k_channel+1],ftype[k_channel+1],30.0,0.3,
            fc0[k_channel+1],fc1[k_channel+1],rate); 
     apply(c2,nsamp-1,p_sh);       
   };

   if(npoles[k_channel+2]!=0){
     design(npoles[k_channel+2],ftr[k_channel+2],ftype[k_channel+2],30.0,0.3,
            fc0[k_channel+2],fc1[k_channel+2],rate); 
     apply(c3,nsamp-1,p_sh);       
   };

  /* calculate number of samples for the sort term average */
  nn1=nsta[k_channel]/rate; 
  nn2=nsta[k_channel+1]/rate; 
  nn3=nsta[k_channel+2]/rate; 

  /* calculate number of samples for the long term average */
  mm1=nlta[k_channel]/rate; 
  mm2=nlta[k_channel+1]/rate; 
  mm3=nlta[k_channel+2]/rate; 

  m=0;  /* start with phase zero */


  /* reset all variables */
  z1=0.0;
  w1=0.0;
  coda1=0;
  trigger1=0;

  z2=0.0;
  w2=0.0;
  coda2=0;
  trigger2=0;

  z3=0.0;
  w3=0.0;
  coda3=0;
  trigger3=0;

  /* high value in order to avoid initial trigger */
  keep_lta1=10000.0;
  keep_lta2=10000.0;
  keep_lta3=10000.0;

  /* calculate initial conditions */ 
  for(i=100;i<350;i++){
     z1=z1+fabs(c1[i]); 
     z2=z2+fabs(c2[i]);
     z3=z3+fabs(c3[i]);
     zc1=zc1+fabs(data_coda1[i]);
     zc2=zc2+fabs(data_coda2[i]);
     zc3=zc3+fabs(data_coda3[i]);
  };

  average1=0.0;
  average2=0.0;
  average3=0.0;
  for(i=100;i<=nsamp-100;i++){
      average1=average1+fabs(c1[i]);
      average2=average2+fabs(c2[i]);
      average3=average3+fabs(c3[i]);
  };
  average1=average1/(float)(nsamp-200);
  average1=average2/(float)(nsamp-200);
  average1=average3/(float)(nsamp-200);

  z1=z1/250.0;
  w1=z1;

  z2=z2/250.0;
  w2=z2;

  z3=z3/250.0;
  w3=z3;

  zc1=zc1/250;
  wc1=zc1;

  zc2=zc2/250.0;
  wc2=zc2;

  zc3=zc3/250.0;
  wc3=zc3;

  trigger_tot=0;

  /* loop locking for the phases */

 /* printf(" Trig_Time   Power_ant     Power_pos    Power_rat      Azim       Coher\n"); */

  for(i=100;i<nsamp-100;i++){

      dxc1=fabs(data_coda1[i]-data_coda1[i-1]);
      wc1=wc1+(dxc1-wc1)/nn1;
      zc1=zc1+(wc1-zc1)/mm1;
     
      dxc2=fabs(data_coda2[i]-data_coda2[i-1]);
      wc2=wc2+(dxc2-wc2)/nn2;
      zc2=zc2+(wc2-zc2)/mm2;

      dxc3=fabs(data_coda3[i]-data_coda3[i-1]);
      wc3=wc3+(dxc3-wc3)/nn3;
      zc3=zc3+(wc3-zc3)/mm3;

      dx1=fabs(c1[i]-c1[i-1]);
      w1=w1+(dx1-w1)/nn1;
      z1=z1+(w1-z1)/mm1;
      n1=w1/z1;

      dx2=fabs(c2[i]-c2[i-1]);
      w2=w2+(dx2-w2)/nn1;
      z2=z2+(w2-z2)/mm2;
      n2=w2/z2;

      dx3=fabs(c3[i]-c3[i-1]);
      w3=w3+(dx3-w3)/nn3;
      z3=z3+(w3-z3)/mm3;
      n3=w3/z3;


      if(n1>=threshold[k_channel] ) {
         trigger1=1;
      }
      else trigger1=0;


      if(n2>=threshold[k_channel+1] ) {
         trigger2=1;
      }
      else trigger2=0;
   
      if(n3>=threshold[k_channel+2] ) {
         trigger3=1;
      }
      else trigger3=0;
     
      
      ipts=i-(int)(iwindow/rate);
      if(ipts<100) ipts=100;
      lpts=i+(int)(lwindow/rate);

      n4=procoh1(c1,c2,c3,ipts,lpts,pvel,psrat,oprocoh);
      /* if the coherence is between 0.5 an 1.0 asume a trigger */
      if(n4>=min_coher && n4<=1.0){
          trigger4=1;
      }
      else trigger4=0;
     
      /* calculate coda duration */
      if(wc1/keep_lta1>=coda_detrigger[k_channel])coda1=coda1+1;
      if(wc2/keep_lta2>=coda_detrigger[k_channel+1])coda2=coda2+1;
      if(wc3/keep_lta3>=coda_detrigger[k_channel+2])coda3=coda3+1;


  /*    printf("%8.3f %8.3f  %8.3f %8.3f %9.3f\n",(float)(i)*rate,n1,n2,n3,n4);  */
     
      sum=trigger1+trigger2+trigger3+trigger4;

      if(sum>=1 && trigger_tot==0){
         phase[k_channel][m]=i;
         dur[k_channel][m]=0;
         keep_coher=n4;
         keep_azim=oprocoh[1];
         if(m==0){
            p_coher=n4;
            p_azim=oprocoh[1];
            keep_lta1=zc1;
            keep_lta2=zc2;
            keep_lta3=zc3;
         };
         m=m+1;

        freq1=fc0[k_channel];
        freq2=fc1[k_channel];

        /* calculate espectral power  before the trigger */
        for(k=phase[k_channel][m-1]-200;k<=phase[k_channel][m-1];k++) 
            data[k-phase[k_channel][m-1]+200]=c1[k];
        memcof(data,200,10,&pm,cof);
        power_ant=0.0;
        for(fdt=freq1;fdt<=freq2; fdt += freq1*rate)
            power_ant=power_ant+evlmem(fdt,cof,10,pm);

 
        /* calculate espectral power  after the trigger */
        for(k=phase[k_channel][m-1];k<=phase[k_channel][m-1]+100;k++)
            data[k-phase[k_channel][m-1]]=c1[k];
        memcof(data,100,10,&pm,cof);
        power_pos=0.0;
        for(fdt=freq1;fdt<=freq2; fdt += freq1*rate)
            power_pos=power_pos+evlmem(fdt,cof,10,pm);

  /*      printf(" %8.2f %12.4f  %12.4f     %8.3f    %6.2f    %8.2f",
               (float)(i)*rate,power_ant,power_pos,2.0*power_pos/power_ant,
                oprocoh[1],n4); */

         trigger_tot=1;
      };

      if(m-2<0) ant=0;
      else ant=phase[k_channel][m-2];

      if(sum>=1 && trigger_tot==1) {
         dur[k_channel][m-1]=dur[k_channel][m-1]+1;
         if(fabs(c1[i])>fabs(maximum[k_channel][m-1]))
            maximum[k_channel][m-1]=c1[i];
      };

      amp_ratio=0.0;
      if(sum<1 && trigger_tot==1) {
         trigger_tot=0;
         if(m-2 != 0) {
            amp_ratio=fabs(maximum[k_channel][m-1])/fabs(maximum[k_channel][m-2]);
         }
         else {
            amp_ratio=ps_amp;
         };
         if(dur[k_channel][m-1]<(int)(min_dur/rate) || 
            phase[k_channel][m-1]-ant<(int)(0.5/rate) ||
            2.0*power_pos/power_ant< min_rat_pow ||
            amp_ratio<threshold[k_channel] ){ /*Spectrum check */
            printf("\n");
            m=m-1;
            if(m==0 ){
               coda1=0;
               coda2=0;
               coda3=0;
               keep_lta1=10000.0;
               keep_lta2=10000.0;
               keep_lta3=10000.0;
            };
         }
         else{ 
              
          /*    printf(" %5.1f", (float)(dur[k_channel][m-1])*rate); */
                if(m-1==0){
                   mm1=s_lta[k_channel]/rate;
                   mm2=s_lta[k_channel+1]/rate;
                   mm3=s_lta[k_channel+2]/rate;
                   /* change lta after p */
                   /*printf(" P\n");*/
                };
          /*    else printf(" **\n"); */
         };
            
      };   
      sum=0;  /* reset variable */
   };



   if(m>=0 && phase[k_channel][0]!=0){
      phase[k_channel][MAX_PHASES]=(int)((float)(coda1+coda2+coda3)/3.0);  
   /* calculate the coda like the average of codas*/
   };

   /* if coda is less than the minimal coda delete all the phases */
   if((float)(phase[k_channel][MAX_PHASES])*rate < min_coda){
       for(i=0;i<=MAX_PHASES+1;i++) phase[k_channel][i]=0;
       m=0;
   };


   /* if phase is far than the coda, delete phase */
   if(m>=0){
      for(i=0;i<=m+1;i++) {
        if(phase[k_channel][i]>phase[k_channel][MAX_PHASES]+phase[k_channel][0]) 
           phase[k_channel][i]=0;
      };
   /* if P is in the last 1/3 length of the record delete all phases */
      if(nsamp-phase[k_channel][0]<nsamp/3){
         for(i=0;i<=MAX_PHASES+1;i++) phase[k_channel][i]=0;
         m=0;
      };
        
   };

   /* print information for wadati */



   if(m>=0 && phase[k_channel][0]!=0){
      i=1;
      do {
          xx=(float)(phase[k_channel][0]);
          yy=(float)(phase[k_channel][i])-xx;
          if(f2!=NULL && phase[k_channel][i]!=0) 
             fprintf(f2,"%d %f %f\n",k_channel,xx,yy);
          i=i+1;
      }while(phase[k_channel][i]!=0);

      fclose(f2);

   };

   /* delete coda if there are no phases */
   if(m==0 && phase[k_channel][0]==0) {
      phase[k_channel][MAX_PHASES]=0;
   };
  

   oprocoh[1]=p_azim;
   oprocoh[0]=p_coher;

   free(data);
   free(data_coda1);
   free(data_coda2);
   free(data_coda3);

   if(phase[k_channel][MAX_PHASES]==0) m=-1;
   printf(" Possible phases: %d coda: %d\n",m+1,phase[k_channel][MAX_PHASES]);
}

void wadati(phase,p_vel,ps_rat,rate,wada_mes)
/* this subroutine check the readings agueinst a wadati */
/* diagram, the input is an ascii file created by the sta_lta subroutine */

float p_vel,ps_rat,rate;
char wada_mes[];

int phase[NCHANNELS][MAX_PHASES+5];
{

 float x[100],y[100],prom,var_y[100],check,m,b;
 float prom_var,s_vel;
 int i,j,k,n_points,channel_num[100];
 FILE *f1;

   f1=fopen("wadati.dat","r");
   if(f1==NULL) {
    /*  printf(" Unable to open wadati.dat file\n");
      printf(" No wadati analisys, returning to main program\n"); */
      return;
   };

   i=1;

   s_vel=p_vel/ps_rat;  
   m=(p_vel/s_vel-1.0);

   check=0.0;
   /* read the phases from the ascii file */
   while(k=fscanf(f1,"%d  %f  %f",&channel_num[i],&x[i],&y[i])!=EOF){
        i=i+1;
   };

 n_points=i-1;

 if(n_points==2){
   b=y[1]-m*x[1];
   if(fabs(fabs(b)-fabs(y[2]-m*x[2]))>=1.0){
  /*    printf(" Warning S-phases are not consistent\n"); */
      strcpy(wada_mes,"Warning S-phases are not consistent");
   };
};

if(n_points<2 ){
  system("rm wadati.dat");
/*  printf(" Points = %d \n Not enough data for Wadati analysis\n",n_points); */
  strcpy(wada_mes,"Warning not enough data");
  return;
};

 prom=0.0;

 /* calculte the average of b */
 for(i=1;i<=n_points;i++){
     b=y[i]-m*x[i];
     prom=b+prom;
 };
 b=prom/n_points;
 /* printf(" average b=%f\n",b); */


/* calculate the variation of the data with respect to the average and
the average of the variaton */ 

prom_var=0.0;
for(i=1;i<=n_points;i++) {
    var_y[i]=fabs(y[i]-(m*x[i]+b));
    prom_var=prom_var+var_y[i];
   /*  printf("point %6.1f %6.1f variation =%5.2f \n",x[i],y[i],var_y[i]); */
};
prom_var=prom_var/n_points;

/*printf(" variation average =%5.2f\n",prom_var); */

/* re-calculate the b value without the points outside of the average
of the variation */

b=0.0;
k=0;
prom=0.0;
for(i=1;i<=n_points;i++){
    if(var_y[i]<=prom_var){
       b=y[i]-m*x[i];
       k=k+1;
    }
    else b=0;
    prom=prom+b;
};
b=prom/k;
/* printf(" new b=%f\n",b); */


/* calculate the new variation of the points */
prom_var=0.0;
for(i=1;i<=n_points;i++) {
    var_y[i]=fabs(y[i]-(m*x[i]+b));
    prom_var=prom_var+var_y[i];
  /*  printf("point %6.1f %6.1f  variation %5.2f \n",x[i],y[i],var_y[i]); */
};
prom_var=prom_var/n_points;
/* printf("average variation = %5.2f \n",prom_var); */

/* check the points */
for(i=1;i<=n_points;i++) {
    if(var_y[i]>prom_var && var_y[i] >=1.0){
  /*       printf("point %6.1f %6.1f for channel %d does not fit\n", 
                x[i],y[i],channel_num[i]);  */
        if(phase[channel_num[i]][1]-phase[channel_num[i]][0]==y[i]) 
                phase[channel_num[i]][1]=0;
    }
    else if(var_y[i]<1.0 && phase[channel_num[i]][MAX_PHASES]>(int)((y[i]+x[i])/rate))
         phase[channel_num[i]][1]=(int)(y[i]+x[i]);
};

   /* close the input file */
   fclose(f1);

/*   printf("End of Wadati analysis\n"); */
   strcpy(wada_mes,"Phases checked");

   /* delete the input file */

   system("rm wadati.dat");
} 

