/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                              Koki Saito                  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:30/12/91                       */
/* Program:AUTO_ASC                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/*       Program to compress and convert specified event to ASCII           */
/*                       for file transfer                                  */
/*                        AUTOMATIC VERSION                                 */
/*                                                                          */
/****************************************************************************/

#include <string.h>
#include <stdio.h>


#define LOGFILE "/home/buffer/event"    /* log-file name                    */
#define DATADIR "/home/events/"         /* event data directory             */
#define OS9LOG "/home/seislog/os9.log"


#define LOGSIZE 512                     /* Log-file size for 1 record(byte) */
#define LOGNO   1000                    /* Log-file size (Num of record)    */
#define FNOFF   208                     /* Filename offset in log file      */
#define MAXCH   48                      /* maximam chanel number            */
#define HEADER  1024                    /* header size (byte)               */

union   {   short   numb;               /* read buffer for event file       */
            char    chara[2];
        }readbuf;
union   {   int     lnumb;
            char    lchar[4];
        }lreadbuf;

long    input[8000];                    /* room for data                    */
int     ch;
int     pid;
int     unit;
int     dpx;                            /* detection number                 */
int     byt;
char    answer1;
char    answer2;
char    answer3;
int     OUTBUF;
int     bufsize;
int     block;
int     ii;
int     nchan;                          /* number of chanel                 */
long    nsamp;                          /* number of sample/chanel          */
long    hdrlgt;
int     displm;                         /* displacement                     */
int     reclength;
char    infile[80];                     /* Data file name                   */
char    syscall[80];
char    prbuf[20000];                   /* Outout buffer                    */
char    header[HEADER];                 /* header buffer                    */
FILE    *inp;
FILE    *inf;                           /*  Data file pointer               */
FILE    *cnv;
/******************************  main ***************************************/
/*              main routine of asc.c.  Read carefully.                     */
/*              argmunt : none                                              */
/*               return : none                                              */
/****************************************************************************/
main()
{

    int         ctr,blk;                 /* loop counter                    */
    long        lx,lc;
    char        event[128];              /* event data full path name       */
    int         i;
    hdrlgt=HEADER;
    displm=0;
    wait(0);

    answer1='C';                         /* Compression May be changed to A */
    if(answer1 == 13)
    {
        answer1='A';
    }else{
        if(answer1 !='A' && answer1 != 'a' && answer1 != 'C' && answer1 !='c')
        {
            printf("\n");
            printf("Input not allowed\n");
            exit(0);
        }
    }
    answer2='S';                       /* Screen May be changed to F     */
    if(answer2 == 13)
    {
        answer2 = 'S';
    }else{
        if(answer2!='S'&&answer2!='s'&&answer2!='F'&&answer2!='f')
        {
            printf("\n");
            printf("Input not allowed\n");
            exit(0);
        }
    }

    if(answer2 == 'F' || answer2 == 'f')
        cnv=fopen(OS9LOG,"a");
    inp=fopen("/home/events/filenr.lis","r");
    if(inp==0)
    {
        printf("No detections on /home/events\n");
        exit(0);
    }
    while(1)                                             /* end-less loop */
    {
        for(ii=0;(ii<80)&&((ch=fgetc(inp))!=EOF)&&(ch!='\n');ii++)
            infile[ii]=ch;
        infile[ii]='\0';        /* put end of string at end                */
        if(ii< 25)
        {
            fclose(inp);
            exit(0);
        }

        for(ii=0;ii<80;ii++) event[ii]=NULL;
        strcat(event,infile);
        printf("NO1 %s\n",event);
        error((inf = fopen(event,"r")) == NULL ? 2 : 0);  /* open data file */
        error(readheader());                /* get primary data from header */
        if(unit <= 36)
        {
            lx=block/2-32;
            bufsize=block/2;
        } else {
            lx=block/4-16;
            bufsize=block/4;
        }

        blk = (nchan  * nsamp);               /* calculate number of blocks */
        blk = ((blk % lx) == 0 ? 0 : 1) + blk / lx;

        i=0;
        if(answer2 == 'S' || answer2 == 's')
        {
            printf("START OF EVENT ***********************\n");
            }else{
                fprintf(cnv,"START OF EVENT ***********************\n");
            }
        if(answer2 == 'S' || answer2 == 's')
        {
        writeheader(header,hdrlgt);                            /* write header  */
        }else{
            writeheader_fil(header,hdrlgt);
        }
        for(ctr =0 ; ctr<blk;ctr++)             /* for all block             */
        {   lx = bufsize;                       /* set read buffer size      */
            bufclr(input);                      /* buffer clear              */
            error(readevnt(ctr));               /* read one block data       */
            if(answer1 == 'C' || answer1 == 'c')
            {
            error(difference(input,lx));        /* calc difference           */
            error(difference(input,lx));
            lc = block*2;                       /* lc has the length of cbuf */
            error(cmprs6(lx,input,&lc,prbuf));  /* translate to ASCII        */
            if(answer2 == 'S' || answer2 == 's')
            {
            printf("%3d      %4ld bytes\n",ctr+1,lc);
            dispdata(prbuf,lc);            /* display on the screen     */
            }else{
                fprintf(cnv,"%3d      %4ld bytes\n",ctr+1,lc);
                dispdata_fil(prbuf,lc);
            }
            }else{
                if(answer2 == 'S' || answer2 == 's')
            {
            lc=0;
            printf("%3d      %4ld bytes\n",ctr+1,lc);
            for(i=0;i<bufsize;i=i+13)
            printf("  %6d%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d\n",\
            input[i],input[i+1],input[i+2],input[i+3],input[i+4],\
            input[i+5],input[i+6],input[i+7],input[i+8],input[i+9],\
            input[i+10],input[i+11],input[i+12],input[i+13]);

            }else{

            lc=0;
            fprintf(cnv,"%3d      %4ld bytes\n",ctr+1,lc);
            for(i=0;i<bufsize;i=i+13)
            fprintf(cnv,"  %6d%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d\n",\
            input[i],input[i+1],input[i+2],input[i+3],input[i+4],\
            input[i+5],input[i+6],input[i+7],input[i+8],input[i+9],\
            input[i+10],input[i+11],input[i+12],input[i+13]);

            }
        }
    }
fclose(inf);
for(ii=0;ii<80;ii++) syscall[ii]=NULL;
strcpy(syscall,"mv -f ");
printf("NO2 %s\n",syscall);
strcat(syscall,event);
/*printf("NO3 %s\n",syscall);*/
strcat(syscall," /home/save");
/*printf("NO4 %s\n",syscall);*/

/*system(syscall);*/        /* copy file to save*/
/**/
/* now delete file from events*/
/**/
/*for(ii=0;ii<80;ii++) syscall[ii]=NULL;*/
/*strcpy(syscall,"rm -f ");*/
/*printf("NO5 %s\n",syscall);*/
/*strcat(syscall,event);*/
printf("NO6 %s\n",syscall);
system(syscall);
}
}
/*---------------------------------------------------- end of main ----------*/
/******************************** readlog ************************************/
/*                  Read log file to get data file name                      */
/*                            argments : none                                */
/*                            return   : if error 1                          */
/*                                       else     0                          */
/*         If this function can not open log file, program will be end.      */
/*****************************************************************************/
int readlog()
{   FILE    *ptr;                         /* log file pointer                */
    long    infposit;                     /* Infomation position on the file */
    int     err;                          /* error code for return           */

    if((ptr = fopen(LOGFILE,"r")) == NULL)    /* open log file               */
    {   printf("Error opening EVENT.\n");     /* if falure print message     */
        exit(1);                              /* and exit from program       */
    }

    infposit = (LOGSIZE * ((dpx ) % LOGNO )) + FNOFF;  /* Calc. position */
    if(fseek(ptr,infposit,0) == 0)         /* move file ptr to the position */
    {   fgets(infile,25,ptr);              /* read datafilename             */
        err = 0;                           /* success                       */
    }
    else
        err = 1;                           /* falure                        */
    fclose(ptr);                           /* close log file                */
    return(err);
}
/*-------------------------------------------- end of readlog --------------*/
/***************************  readheader  ***********************************/
/*         Read datafile header to get primary data which are               */
/*         number of chanel,data and displacement. These data               */
/*         are stared in global parameters.                                 */
/*                       arguments : none                                   */
/*                          return : if error 3                             */
/*                                   else     0                             */
/****************************************************************************/

int readheader()
{   int err;
    fread(header,1024,1,inf);              /* read header as char string */
    header[1023] = 90;                      /* ASCIIZ delimiter           */
    err = fseek(inf,62,0);                /* 'inf' decleared as global  */
    fscanf(inf,"%d",&nchan);              /* read chanel data as number */

    err += fseek(inf,80,0);               /* move file pointer          */
    fscanf(inf,"%d",&nsamp);              /* read sample number         */
    err += fseek(inf,104,0);
    fscanf(inf,"%d",&block);
    err += fseek(inf,96,0);
    fscanf(inf,"%d",&unit);
    return(err == 0 ? 0 : 3);             /* if err == 0 then return 0  */
}


/*------------------------------------------- end of readheader -------------*/
/******************************  bufclr  *************************************/
/*                         clear data buffer                                 */
/*                       argumet : buffer pointer (long)                     */
/*                        return : none                                      */
/*****************************************************************************/
bufclr(buf)
long    *buf;                      /* buffer pointer */
{   long i;
    for(i=0;i<bufsize;i++)
    buf[i] = 0;
}
/*-------------------------------------------- end of bufclr ----------------*/

/***************************** readevent *************************************/
/*          read event data(2byte) and write on the buffer(4byte)            */
/*          by defined BLOCK.                                                */
/*              data buffer has declared as global parameter                 */
/*                    argument : block number(from 0)                        */
/*                               pointer of input buffer length              */
/*                     return  : if error 4                                  */
/*                               else     0                                  */
/*             data file structure : see seislog manual                      */
/*****************************************************************************/
int readevnt(nth)
int     nth;                       /* 'n'th block */
{   int         err;
    long        c;                 /* utility                */
    long        head;              /* position of block head */
    long        ln;

    head=(nth+1+1024/block)*block;
    ln = bufsize;

    err = fseek(inf,head,0);        /* move file pointer                  */
    if(err == 0)
    {   for(c=0;c<ln;c++)          /* read data by each 2 byte.          */
            if(unit <= 36)
        {   fread(readbuf.chara,2,1,inf);    /* read as char*2           */
            input[c] = readbuf.numb;         /* write as short(union)    */
            if ( c < 32)
              input[c]=0;
        } else {
            fread(lreadbuf.lchar,4,1,inf);
            input[c] = lreadbuf.lnumb;
            if ( c < 16)
              input[c]=0;

        }

    }
    return  (err == 0 ? 0 : 4);     /* return 0 or 4                      */
}
/*------------------------------------------ end of readevnt -------------*/
/*************************** difference ***********************************/
/*       Data compress routine by culclate difference of each chanel      */
/*       But, data are still duplexed. Differencd data are stored in      */
/*       same array as original data. So, origiunal data is deleted       */
/*       except first data of each channel.                               */
/*                argument : original data array pointer                  */
/*                           number of data(length of data array)         */
/*                 return  : 0                                            */
/**************************************************************************/
int difference(data,l)
long    l;                 /* number of data(length of array */
long    *data;             /* data array pointer             */
{   long    temp[MAXCH];      /* temporary array */
    long    buf;              /* buffer          */
    long    c;                /* counter         */

    for(c=0;c<nchan;c++)      /* save current data in temp. by each chanel */
        temp[c] = data[c];
    for(c=nchan;c<l;c++)      /* culculate difference from one before data */
    {   buf = temp[c%nchan];  /* of same chanel. store in same data area   */
        temp[(c%nchan)] = data[c];
        data[c] = buf - data[c];
    }
    return(0);
}
/*-------------------------------------------- end of difference -----------*/

/******************************* dispdata ***********************************/
/*         write strings on screen with format(80char / line)               */
/*              argument :  pointer of ASCII converted data array           */
/*                          number of ASCII data(length of array)           */
/*               return  :  none                                            */
/****************************************************************************/
dispdata(cbuf,l)
char    *cbuf;
long    l;
{   long    i;
    long    c;
    i=0;
    while(cbuf[i] != 0)
    {   for(c=0;(c<80 && cbuf[i] != 0);c++)
        {   printf("%c",cbuf[i]);
            i++;
        }
        printf("\n");
    }
}
/*------------------------------------------------- end of disp data -------*/

/******************************* dispdata_fil********************************/
/*         write strings on screen with format(80char / line)               */
/*              argument :  pointer of ASCII converted data array           */
/*                          number of ASCII data(length of array)           */
/*               return  :  none                                            */
/****************************************************************************/
dispdata_fil(cbuf,l)
char    *cbuf;
long    l;
{   long    i;
    long    c;
    i=0;
    while(cbuf[i] != 0)
    {   for(c=0;(c<80 && cbuf[i] != 0);c++)
        {   fprintf(cnv,"%c",cbuf[i]);
            i++;
        }
        fprintf(cnv,"\n");
    }
}
/*------------------------------------------------- end of disp data -------*/

/*************************** writeheader ************************************/
/*                   display header on screen                               */
/*                 argument : pointer of header array                       */
/*                          : number of character to display                */
/*                   return : VOID                                          */
/****************************************************************************/
writeheader(c,l)
char    *c;
long    l;
{   int i,j,count;
    count = 0;
for (i=0;i<l;i++)
{
    printf("%c",c[i]);
    count=count+1;
    if(count == 80)
    {
        count=0;
        printf("\n");
    }
}
for(j=0;j<80;j++)
{
    printf("%c",'X');
    count=count+1;
    if(count==80)
        goto stop;
}
j=0;
stop:
printf("\n");
}
/*------------------------------------ end of writeheader -----------------*/

/*************************** writeheader_fil*********************************/
/*                   display header on screen                               */
/*                 argument : pointer of header array                       */
/*                          : number of character to display                */
/*                   return : VOID                                          */
/****************************************************************************/
writeheader_fil(c,l)
char    *c;
long    l;
{   int i,j,count;
    count = 0;
for (i=0;i<l;i++)
{
    fprintf(cnv,"%c",c[i]);
    count=count+1;
    if(count == 80)
    {
        count=0;
        fprintf(cnv,"\n");
    }
}
for(j=0;j<80;j++)
{
    fprintf(cnv,"%c",'X');
    count=count+1;
    if(count==80)
        goto stop;
}
j=0;
stop:
fprintf(cnv,"\n");
}
/*------------------------------------ end of writeheader -----------------*/

/******************************* error **************************************/
/*         display error message and exit from program                      */
/*                 argument : error number(return of each functions)        */
/*                  return  : none                                          */
/****************************************************************************/
error(i)
int i;
{   switch (i)
    {   case 0  :   break;
        case -1 :   printf("ERROR Cannot convert data to ASCII\n");
                    break;
        case 1  :   printf("ERROR Cannot read event log\n");
                    break;
        case 2  :   printf("ERROR Cannot open data file\n");
                    break;
        case 3  :   printf("ERROR Cannot read data header\n");
                    break;
        case 4  :   printf("ERROR Cannot read event data\n");
                    break;
        default :   printf("ERROR  %d\n",i);
                    break;
    }
        if(i!=0)
            exit(i);
}
/*------------------------------------------------ end of error ------------*/
/* end of program */



/****************************************************************************/
/*                                                                          */
/*                 function    cmprs6(lx,ix,lc,cbuff)                       */
/*                                                                          */
/*     Routine to compress integer data into printable ASCII characters     */
/*                                                                          */
/*             lx     : long         [ data array length ]                  */
/*             ix     : long pointer [ data array ]                         */
/*             lc     : long pointer [ output array length ]                */
/*             cbuff  : char pointer [ output array ]                       */
/*             RETURN : int          [ error -1 ,else 0 ]                   */
/*                                                                          */
/*   Inputs are 4 byte (long) integer ix of length lx.                      */
/*   Lc should contain the dimension of the array cbuff.                    */
/*   Outputs are char array cbuff containing lc characters.                 */
/*   If the array cbuff is not large enough to contain all of the data      */
/*   this function retuerns -1, otherwise return value is zero.             */
/*   Output is padded to a multiple of 80 characters using spaces.          */
/*   Method of compression is to use the 6 least significant six bits       */
/*   of an eight bit byte so that all data can be transmitted as ASCII      */
/*   characters.                                                            */
/*   Of these six bits, the most significant is used as a continuation      */
/*   bit. If it is set to one the following byte also forms part of         */
/*   the present sample. If zero, this is the least byte in the sample.     */
/*   The second least significant bit is used as a sign bit in the first    */
/*   byte and for data in subsequent bytes (original data is first conv-    */
/*   erted to sign and magnitude). All other bits of first and subsequent   */
/*   bytes form the magunitude of the number.                               */
/*                                                                          */
/*   To enable transmission over as many links as possible data is further  */
/*   trans formed to the characters +, -, 0..9, A..Z, a..z using a lookup   */
/*   table.                                                                 */
/*                                                                          */
/*   This function does not use any other functions and libraly functions.  */
/*                                                                          */
/*   Data may be decompressed using function decomp6.                       */
/*                                                                          */
/*   Program cmp6test can help you to understand the algorithm of this      */
/*   function.                                                              */
/*                                                                          */
/*                                         23rd Aug 1990    K.Saito         */
/****************************************************************************/

int cmprs6(lx,ix,lc,cbuff)
    long    lx;
    long    *ix;
    long    *lc;
    char    *cbuff;
{   char    achar[65];
    long    buf;
    long    jn;
    long    ioff;
    long    imax;
    long    mflag,nflag;
    long    counter;
    int     ierr;
    int     shiftctr;

     achar[0]='+'; achar[1]='-'; achar[2]='0'; achar[3]='1'; achar[4]='2';
     achar[5]='3'; achar[6]='4'; achar[7]='5'; achar[8]='6'; achar[9]='7';
    achar[10]='8';achar[11]='9';achar[12]='A';achar[13]='B';achar[14]='C';
    achar[15]='D';achar[16]='E';achar[17]='F';achar[18]='G';achar[19]='H';
    achar[20]='I';achar[21]='J';achar[22]='K';achar[23]='L';achar[24]='M';
    achar[25]='N';achar[26]='O';achar[27]='P';achar[28]='Q';achar[29]='R';
    achar[30]='S';achar[31]='T';achar[32]='U';achar[33]='V';achar[34]='W';
    achar[35]='X';achar[36]='Y';achar[37]='Z';achar[38]='a';achar[39]='b';
    achar[40]='c';achar[41]='d';achar[42]='e';achar[43]='f';achar[44]='g';
    achar[45]='h';achar[46]='i';achar[47]='j';achar[48]='k';achar[49]='l';
    achar[50]='m';achar[51]='n';achar[52]='o';achar[53]='p';achar[54]='q';
    achar[55]='r';achar[56]='s';achar[57]='t';achar[58]='u';achar[59]='v';
    achar[60]='w';achar[61]='x';achar[62]='y';achar[63]='z';achar[64]=0;
    mflag = 0x20;
    ierr = 0;
    imax = *lc;
    ioff = 0;

    for(counter=0;counter < lx; counter++)
    {   jn = ix[counter];
        nflag = 0x00;
        shiftctr = 5;       /* Always 5 times bit shift */

        if(jn < 0)          /* Convert sign and magunitude */
        {   nflag = 0x10;
            jn = -jn;
        }

        jn = ((jn & 0xe0000000) ? 0x1fffffff : jn);  /* Too big number */

        jn = (jn << 3);    /* At most 29 bits */

        while(((jn >> 27) == 0x00) && (shiftctr > 0))  /* Find first + bit */
        {   jn = (jn << 5);
            shiftctr--;
        }

        buf = (jn >> 28) & 0x0000000f;     /* Upper 4 bits to lower 4bits */
        buf = buf + nflag  + (shiftctr ? mflag : 0x00);  /* add flags */

        cbuff[ioff] = achar[buf];     /* 4bits number to character */
        jn = (jn << 4);               /* go to next 5 bits at Upper bits*/
        shiftctr--;
        ioff++;

        while((shiftctr >= 0) && (ioff < imax))
        {   buf = ((jn >> 27) & 0x0000001f);   /* Upper 5 bits to lower */
            buf = buf + (shiftctr ? mflag : 0x00);  /* add flag */
            cbuff[ioff] = achar[buf];
            jn = (jn << 5);
            shiftctr--;
            ioff++;
        }
        if(ioff >= (imax-1))  /* to save space for ASCIIZ delimiter */
        {   ierr = -1;
            *lc = imax;
            break;
        }
    }
    cbuff[ioff] = ' ';   /* Add at least one space */
    while(((((ioff++)+1) % 80) != 0) && (ioff < imax))
        cbuff[ioff] = ' ';
    cbuff[ioff] = 0;         /* ASCIIZ delimiter */
    *lc = ioff;
    return(ierr);
}
