/*************************************************************************/
/*                     VME-OS-9 SEISLOG Software                         */
/*************************************************************************/
/*                                                                       */
/*                                  University of Bergen                 */
/*                                  Institute of Solid Earth Physics     */
/*                                  Seismological Observatory            */
/*                                  Allegaten 41, 5000 Bergen            */
/*                                  Norway. Tlf.05-213419 Fax 05-320009  */
/*                                  Authors:                             */
/*                                  Koki Saito,Terje Utheim,Jens Havskov */
/*                                                                       */
/*                                  Date: 19/10/90                       */
/* Program: ASC                                                          */
/*                                                                       */
/* Last update: 19/10/90                                                 */
/*              apr 21 92 by jh   : fix bug with header size 32          */
/*              jun 10 92         : 4 byte nanometrics                   */
/*              jun 12-17         : bugs                                 */
/*              aug 12            : new header format                    */
/* 23.06.93 JE made it so that longer filenames can be used              */
/* 21.05.96 RJC path /dd/parameter changed to /dd/sl/pr                  */
/* Info:                                                                 */
/*                                                                       */
/*       Program to compress and convert specified event to ASCII        */
/*                       for file transfer                               */
/*                                                                       */
/*************************************************************************/
/*	Modified December 1995 by charlie to give the option to write        */
/*	to a file via the call    AUTOASC TOFILE <DPX_no>                    */
/*	Where:-		                                                         */
/*	   AUTOASC		is the current program	                             */
/*	   TOFILE		redirects output from screen to file                 */
/*	   <DPX_no>		Single DPX file number to transfer                   */
/*	The output file will automatically go to OS9.LOG                     */
/*  If no DPX number is supplied the program will prompt for it in the   */
/*  same way that the old AUTOASC program did                            */
/*  If AUTOASC is called on it's own it will respnd as the old system did*/
/*************************************************************************/
/*  Modified September 1997 to run on Seislog V7.0 (QNX), S. Flower      */
/*  Bug fix for 32 channel event files where BLOCK is a 5 digit int      */
/*    and the space delimiter is lost. Substituted an fread in           */
/*    readheader function.  D Scott, 26/03/98                            */
/*************************************************************************/
 
#include <string.h>
#include <stdio.h>
#include <ctype.h>
 
#define LOGFILE "/home/buffer/event"    /* log-file name                    */
#define DATADIR "/home/events/"         /* event data directory             */
#define OS9LOG 	"os9.log"
#define DETNR	"/home/seislog/pr/detnr"	/* file containing last detection # */
#define PDETNR	"/home/seislog/pr/pdetnr"	/* file containing last collected event */

#define LOGSIZE 512                   /* Log-file size for 1 record(byte) */
#define LOGNO   1000                  /* Log-file size (Num of record)    */
#define FNOFF   206                   /* Filename offset in log file      */
		/* changed FNOFF by -2 to accommodate 4-digit year filenames      */
        /*     for millenium compliance     28/01/99 DJS                  */
#define MAXCH   48                    /* maximam chanel number            */
#define HEADER  1024                  /* header size (byte)               */
 
union   {   short   numb;                /* read buffer for event file    */
            char    chara[2];
        }readbuf;
union   {   int     lnumb;
            char    lchar[4];
        }lreadbuf;

long    input[8000];                  /* room for data                 */
int		pid;
int     unit;
int     dpx;                             /* detection number              */
int		byt;
char	as_com[5];
char	fil_scr[5];
int		OUTBUF;
int		BUFSIZE;
int		BLOCK;
int     nchan;                           /* number of chanel              */
long    nsamp;                           /* number of sample/chanel       */
int     displm;                          /* displacement                  */
int		reclength;
char    infile[35];                      /* Data file name                */
char    prbuf[20000];                   /* Outout buffer                 */
char    header[HEADER];                  /* header buffer                 */
/***********  charlies variables  *******************/
char line[81];
int numbers[80];
int dpxnos[80];
int count;
int detnr,pdetnr;
int auto_flag=0;
int file_output=0;						/* file output flag */
int get_numbs(); 
FILE    *f1;                             /* file pointer for DETNR & PDETNR*/
FILE    *f2;                             /* file pointer for os9.log */
FILE    *inf;                            /*  Data file pointer            */
FILE	*cnv; 
/******************************  main ************************************/
/*              main routine of asc.c.  Read carefully.                  */
/*              argmunt : none                                           */
/*               return : none                                           */
/*************************************************************************/
get_numbs(numbers)
int numbers[80];
{
	char line[81];
	char space = ' ';
	int no_chars,i,j=0;
	gets(line);
	no_chars=strlen(line);
	if(no_chars==0)return(0);
	if(line[0]!=space)numbers[j++]=atoi(line);
	for(i=0;i<no_chars-1;i++)
	  if(line[i]==space && line[i+1]!=space)numbers[j++]=atoi(&line[i]);
	if(numbers[0]==0)j=0;
	return(j);
}
	
main(argc,argv)
int argc;
char *argv[];
{
char *ch;

    int                 ctr,blk;             /* loop counter */
    long                lx,lc;
    char                event[128];      /* event data full path name */
    int                 i,nofevents;
    displm=0;
    dpx=0;			/* clear the DPX number to help test for parameter*/
/* charlie says set up default for screen output and compressed ASCII*/
	as_com[0]  = 'A';
	fil_scr[0] = 'S';	
/*********************************************************************/
/* check arguments for file output and DPX number                       */
/* check for 'TOFILE' argument                                          */
/* Program can be called by     AUTOASC  TOFILE  DPX                    */
/*            where		AUTOASC is the program name             */
/*                      TOFILE  redirects output to OS9.LOG             */
/*                      dpx     The DPX file number to transfer eg 67   */ 
/*                 ie.  AUTOASC TOFILE 67                               */
      if(argc>1)
        {  ch=argv[1];
           while(*ch!='\0'){*ch=toupper(*ch);ch++;}       /* convert to upper case*/
           if(strcmp(argv[1],"TOFILE")==0)
             {  f2=fopen("os9.log","w");
                file_output = 1;
        printf("AUTOASC writes out events in ASCII to OS9.LOG  29-Apr-96\n");
        printf("=============================================  =========\n\n");
             }
        }
/* check for specific DPX number*/
      if(argc>2)
        {  sscanf(argv[2],"%d",&dpx);
           printf("DPX number is %d\n",dpx);
        }        
/*************************************************************************/
    if(file_output==0)
      {
        printf("AUTOASC transfers events in ASCII to terminal (2.4)  29-Apr-96\n");
        printf("===================================================  =========\n\n");
      }
    else
      {
        fprintf(f2,"AUTOASC transfers events in ASCII to OS9.LOG 29-Apr-96\n");
        fprintf(f2,"============================================ =========\n\n");
      }
/*    get_ts_nos(detnr,pdetnr);*/
/* manually read in detnr & pdetnr*/
	if((f1=fopen(DETNR,"r"))==NULL)
	  {
            printf("\nERROR opening %s", DETNR);exit(0);
          }
	fgets(line,10,f1);
	fclose(f1);
	detnr=atoi(line);
	if((f1=fopen(PDETNR,"r"))==NULL)
	  {
            printf("\nERROR opening %s", PDETNR);exit(0);
          }
	fgets(line,10,f1);
	fclose(f1);
	pdetnr=atoi(line);

        if(file_output==0||dpx==0)
          {
             printf("Latest event is %4d Last transferred was %4d",detnr,pdetnr); 
             printf("\nType in events to transfer or RETURN for AUTO mode\n");
          }
        else
          {
            fprintf(f2,"Latest event is %4d Last transferred was %4d",detnr,pdetnr); 
          }            

/* check if a DPX number has been passed as a parameter         */
        if(dpx>0)
          {
            nofevents=1;
            dpxnos[0]=dpx;
          }
        else
          {
	    nofevents=get_numbs(dpxnos); /* keep track of n events to do*/
          }
/* check nofevents to transfer if there are none then use detnr and pdetnr */
	if(nofevents==0)
          {
            if(file_output==0||dpx==0)
              {
              	printf("Auto-transfer of events from%4d to%4d\n",pdetnr+1,detnr);
	            nofevents=detnr-pdetnr;
	          }
	        if(nofevents==0)          /* if still no events then abort program */
	          {
                if(file_output==0)
                  {
                    printf("\nNo new events to tranfer");
	                printf("\nStop:    Normal termination of program\n\n\n");
		            exit(0);
                  }
                else
                  {
                    fprintf(f2,"Auto-transfer of events from%4d to%4d",pdetnr+1,detnr);
                    fprintf(f2,"\nNo new events to transfer");
                    fprintf(f2,"\nStop:   Normal termination of program\n\n\n");
                    close(f2);
                    exit(0);
                  }
                }
/* set auto transfer flag so we know to update the pdetnr number at the end*/
                auto_flag=1;
/* plant the dpx numbers in the numbers array */
		        for (i=0;i<nofevents;i++)dpxnos[i]=pdetnr+i+1;	
              }
/***************************************************************************/
/* Main loop for reading the events                                        */
		for(i=0;i<nofevents;i++)
		  {	
		    dpx=dpxnos[i];
		    if(dpx==0)continue;
		    if(file_output==0)
                      {
                        printf("\nTransferring event no%4d\n",dpx);
                      }
                    else
                      {
                        fprintf(f2,"\nTransferring event no%4d\n",dpx);
                      }
        	error(readlog());            /* read log file to get data file name */
        	strcpy(event,DATADIR);                    /* create mfull path name */
        	strcat(event,infile);
/* print out the event file name so charlies check_ok program can see it */
		if(file_output==0)
                  {
                    printf("%s\n",event);
                  }
                else
                  {
                    fprintf(f2,"%s\n",event);
                  }
			inf = fopen(event,"r");
        	error(inf == NULL ? 2 : 0);  /* open data file */
        	error(readheader());                /* get primary data from header */

			/* this sum works for 4 byte integers - in QNX no digitiser
			 * drivers use 2 byte integers - if you need 2 byte integers
			 * use 'lx=BLOCK/2-32;' & 'BUFSIZE=BLOCK/2' */
       		lx=BLOCK/4-16;
       		BUFSIZE=BLOCK/4;

        	blk = (nchan  * nsamp);               /* calculate number of blocks */
        	blk = ((blk % lx) == 0 ? 0 : 1) + blk / lx;
 
        	if(file_output==0)
                  {
                    printf("Number of buffers to write out: %4d",blk); /* write initial */
        	        printf("\nSTART OF EVENT ***************\n");
                  }
                else
                  {
                     fprintf(f2,"Number of buffers to write out: %4d",blk);
                     fprintf(f2,"\nSTART OF EVENT ***************\n");
                  }
        	writeheader(header,1024);                            /* write header  */
        	for(ctr =0 ; ctr<blk;ctr++)             /* for all block             */
        	{
        	    lx = BUFSIZE;                       /* set read buffer size      */
            	bufclr(input);                      /* buffer clear              */
            	error(readevnt(ctr));               /* read one block data       */
            	error(difference(input,lx));        /* calc difference           */
            	error(difference(input,lx));
            	lc=BLOCK*2;
            	error(cmprs6(lx,input,&lc,prbuf));  /* translate to ASCII        */
            	if(file_output==0)
            	  {
            	  	printf("%3d      %4ld bytes\n",ctr+1,lc);
                  } 
                else
                  {
                  	fprintf(f2,"%3d      %4ld bytes\n",ctr+1,lc);
                    printf("%3d      %4ld bytes\n",ctr+1,lc);
                  }
            	dispdata(prbuf,lc);            /* display on the screen     */
        	}
        	fclose(inf);
/* the last thing to do before exiting is to update the pdetnr number
   if it has been set due to auto transfer mode.
*/
		if(auto_flag)
		{
			f1=fopen(PDETNR,"w");
			fprintf(f1,"%4d",dpx);
			fclose(f1);   
		}
    	}

            printf("\nStop:    Normal termination of program\n\n");
            if(file_output!=0)
              {
                fprintf(f2,"\nStop:    Normal termination of program\n\n");
              }
}
/*---------------------------------------------------- end of main ----------*/
/******************************** readlog ************************************/
/*                  Read log file to get data file name                      */
/*                            argments : none                                */
/*                            return   : if error 1                          */
/*                                       else     0                          */
/*         If this function can not open log file, program will be end.      */
/*****************************************************************************/
int readlog()
{   FILE    *ptr;                         /* log file pointer                */
    long    infposit;                     /* Infomation position on the file */
    int     err;                          /* error code for return           */
    int     length;						  /* for stripping whitespace 		 */
 
    if((ptr = fopen(LOGFILE,"r")) == NULL)    /* open log file               */
    {  
      if(file_output==0)
      {
        printf("Error opening EVENT.\n");     /* if falure print message     */
      }
      else
      {
        fprintf(f2,"Error opening EVENT.\n");
      }
      exit(1);                              /* and exit from program       */
    }
 
    infposit = (LOGSIZE * ((dpx -1) % LOGNO )) + FNOFF;  /* Calc. position */
    if(fseek(ptr,infposit,0) == 0)         /* move file ptr to the position */
    {   fgets(infile,35,ptr);              /* read datafilename             */
        err = 0;                           /* success                       */
		/* strip whitespace from the end of the string */
        for (length = strlen (infile) -1; length >= 0; length --)
		{
			if (infile [length] == ' ') infile [length] = '\0';
			else break;
		}
    }
    else
        err = 1;                           /* falure                        */
    fclose(ptr);                           /* close log file                */

    return(err);
}
/*-------------------------------------------- end of readlog --------------*/
/***************************  readheader  ***********************************/
/*         Read datafile header to get primary data which are               */
/*         number of chanel,data and displacement. These data               */
/*         are stared in global parameters.                                 */
/*                       arguments : none                                   */
/*                          return : if error 3                             */
/*                                   else     0                             */
/****************************************************************************/
int readheader()
{   int err;
	char block_str[12];

    fread(header,1024,1,inf);              /* read header as char string */
    header[1023] = 90;                      /* ASCIIZ delimiter           */
    err = fseek(inf,62,0);                /* 'inf' decleared as global  */
    fscanf(inf,"%d",&nchan);              /* read chanel data as number */

    err += fseek(inf,80,0);               /* move file pointer          */
    fscanf(inf,"%d",&nsamp);              /* read sample number         */
    err += fseek(inf,104,0);
	fread(block_str,8,1,inf);             /* bug fix for 32 channels */
	block_str[8] = '\0';                  /* DJS 26/03/98            */
	BLOCK = atoi(block_str);
    err += fseek(inf,96,0);
    fscanf(inf,"%d",&unit);
    return(err == 0 ? 0 : 3);             /* if err == 0 then return 0  */
}
/*------------------------------------------- end of readheader -------------*/
/******************************  bufclr  *************************************/
/*                         clear data buffer                                 */
/*                       argumet : buffer pointer (long)                     */
/*                        return : none                                      */
/*****************************************************************************/
bufclr(buf)
long    *buf;                      /* buffer pointer */
{   long i;
    for(i=0;i<BUFSIZE;i++)
    buf[i] = 0;
}
/*-------------------------------------------- end of bufclr ----------------*/
/***************************** readevent *************************************/
/*          read event data(2byte) and write on the buffer(4byte)            */
/*          by defined BLOCK.                                                */
/*              data buffer has declared as global parameter                 */
/*                    argument : block number(from 0)                        */
/*                               pointer of input buffer length              */
/*                     return  : if error 4                                  */
/*                               else     0                                  */
/*             data file structure : see seislog manual                      */
/*****************************************************************************/
int readevnt(nth)
int     nth;                       /* 'n'th block */
{   int         err;
    long        c;                 /* utility                */
    long        head;              /* position of block head */
    long        ln;

    head=(nth+1+1024/BLOCK)*BLOCK;
    ln = BUFSIZE;
    err = fseek(inf,head,0);        /* move file pointer                  */
    if(err == 0)
    {
      for(c=0;c<ln;c++)          /* read data by each 2 byte.          */
	  {
		/* in QNX all data is 4 byte - the old two byte code is
		 * commented out below:
         *     fread(readbuf.chara,2,1,inf);
         *     input[c] = readbuf.numb;
         *     if ( c < 32) input[c]=0;  */
        fread(lreadbuf.lchar,4,1,inf);
        input[c] = lreadbuf.lnumb;
        if ( c < 16) input[c]=0;
	  }
    }
    return  (err == 0 ? 0 : 4);     /* return 0 or 4                      */
}
/*------------------------------------------ end of readevnt -------------*/
/*************************** difference ***********************************/
/*       Data compress routine by culclate difference of each chanel      */
/*       But, data are still duplexed. Differencd data are stored in      */
/*       same array as original data. So, origiunal data is deleted       */
/*       except first data of each channel.                               */
/*                argument : original data array pointer                  */
/*                           number of data(length of data array)         */
/*                 return  : 0                                            */
/**************************************************************************/  
int difference(data,l)
long    l;                 /* number of data(length of array */
long    *data;             /* data array pointer             */
{   long    temp[MAXCH];      /* temporary array */
    long    buf;              /* buffer          */
    long    c;                /* counter         */
 
    for(c=0;c<nchan;c++)      /* save current data in temp. by each chanel */
        temp[c] = data[c];  
    for(c=nchan;c<l;c++)      /* culculate difference from one before data */
    {   buf = temp[c%nchan];  /* of same chanel. store in same data area   */
        temp[(c%nchan)] = data[c];
        data[c] = buf - data[c];
    }
    return(0);
}
/*-------------------------------------------- end of difference -----------*/

/******************************* dispdata ***********************************/
/*         write strings on screen with format(80char / line)               */
/*              argument :  pointer of ASCII converted data array           */
/*                          number of ASCII data(length of array)           */
/*               return  :  none                                            */
/****************************************************************************/
dispdata(cbuf,l)
char    *cbuf;
long    l;
{   long    i;
    long    c;
    i=0;
    while(cbuf[i] != 0)
    {   for(c=0;(c<80 && cbuf[i] != 0);c++)
        {
          if(file_output==0)
          {
            printf("%c",cbuf[i]);
            i++;
          }
          else
          {
            fprintf(f2,"%c",cbuf[i]);
            i++;
          }
        }
        if(file_output==0)
        {
          printf("\n");
        }
        else
        {
          fprintf(f2,"\n");
        }
    }
}
/*------------------------------------------------- end of disp data -------*/

/******************************* dispdata_fil********************************/
/*         write strings on screen with format(80char / line)               */
/*              argument :  pointer of ASCII converted data array           */
/*                          number of ASCII data(length of array)           */
/*               return  :  none                                            */
/****************************************************************************/
dispdata_fil(cbuf,l)
char    *cbuf;
long    l;
{   long    i;
    long    c;
    i=0;
    while(cbuf[i] != 0)
    {   for(c=0;(c<80 && cbuf[i] != 0);c++)
        {   fprintf(cnv,"%c",cbuf[i]);
            i++;
        }
        fprintf(cnv,"\n");
    }
}
/*------------------------------------------------- end of disp data -------*/

/*************************** writeheader ************************************/
/*                   display header on screen                               */
/*                 argument : pointer of header array                       */
/*                          : number of character to display                */
/*                   return : VOID                                          */
/****************************************************************************/
writeheader(c,l)
char    *c;
int    l;
{   int i,j,count;
    count = 0;
for (i=0;i<l;i++)
{
	if(file_output==0)
        {
          printf("%c",c[i]);
        }
        else
        {
          fprintf(f2,"%c",c[i]);
        }
	count=count+1;
	if(count == 80)
	{
	  count=0;
	  if(file_output==0)printf("\n");
          if(file_output!=0)fprintf(f2,"\n");
	}
}
for(j=0;j<80;j++)
{
	if(file_output==0)printf("%c",'X');
        if(file_output!=0)fprintf(f2,"%c",'X');
	count=count+1;
	if(count==80)goto stop;
}
j=0;
stop:
if(file_output==0)printf("\n");
if(file_output!=0)fprintf(f2,"\n");
}
/*------------------------------------ end of writeheader -----------------*/

/*************************** writeheader_fil*********************************/
/*                   display header on screen                               */
/*                 argument : pointer of header array                       */
/*                          : number of character to display                */
/*                   return : VOID                                          */
/****************************************************************************/
writeheader_fil(c,l)
char    *c;
long    l;
{   int i,j,count;
    count = 0;
for (i=0;i<l;i++)
{
	fprintf(cnv,"%c",c[i]);
	count=count+1;
	if(count == 80)
	{
	  count=0;
	  fprintf(cnv,"\n");
	}
}
for(j=0;j<80;j++)
{
  fprintf(cnv,"%c",'X');
  count=count+1;
  if(count==80)goto stop;
}
j=0;
stop:
fprintf(cnv,"\n");
}
/*------------------------------------ end of writeheader -----------------*/

/******************************* error **************************************/
/*         display error message and exit from program                      */
/*                 argument : error number(return of each functions)        */
/*                  return  : none                                          */
/****************************************************************************/
error(i)
int i;
{   switch (i)
    {   case 0  :   break;
        case -1 :   if(file_output==0)printf("ERROR Cannot convert data to ASCII\n");
                    if(file_output!=0)fprintf(f2,"ERROR Cannot convert data to ASCII\n");
                    break;
        case 1  :   if(file_output==0)printf("ERROR Cannot read event log\n");
      
                    break;
        case 2  :   if(file_output==0)printf("ERROR Cannot open data file\n");
                    break;
        case 3  :   if(file_output==0)printf("ERROR Cannot read data header\n");
                    break;
        case 4  :   if(file_output==0)printf("ERROR Cannot read event data\n");
                    break;
        default :   if(file_output==0)printf("ERROR  %d\n",i);
                    break;
    }
        if(i!=0)
            exit(i);
}
/*------------------------------------------------ end of error ------------*/
/* end of program */



/****************************************************************************/
/*                                                                          */
/*                 function    cmprs6(lx,ix,lc,cbuff)                       */
/*                                                                          */
/*     Routine to compress integer data into printable ASCII characters     */
/*                                                                          */
/*             lx     : long         [ data array length ]                  */
/*             ix     : long pointer [ data array ]                         */
/*             lc     : long pointer [ output array length ]                */
/*             cbuff  : char pointer [ output array ]                       */
/*             RETURN : int          [ error -1 ,else 0 ]                   */
/*                                                                          */
/*   Inputs are 4 byte (long) integer ix of length lx.                      */
/*   Lc should contain the dimension of the array cbuff.                    */
/*   Outputs are char array cbuff containing lc characters.                 */
/*   If the array cbuff is not large enough to contain all of the data      */
/*   this function retuerns -1, otherwise return value is zero.             */
/*   Output is padded to a multiple of 80 characters using spaces.          */
/*   Method of compression is to use the 6 least significant six bits       */
/*   of an eight bit byte so that all data can be transmitted as ASCII      */
/*   characters.                                                            */
/*   Of these six bits, the most significant is used as a continuation      */
/*   bit. If it is set to one the following byte also forms part of         */
/*   the present sample. If zero, this is the least byte in the sample.     */
/*   The second least significant bit is used as a sign bit in the first    */
/*   byte and for data in subsequent bytes (original data is first conv-    */
/*   erted to sign and magnitude). All other bits of first and subsequent   */
/*   bytes form the magunitude of the number.                               */
/*                                                                          */
/*   To enable transmission over as many links as possible data is further  */
/*   trans formed to the characters +, -, 0..9, A..Z, a..z using a lookup   */
/*   table.                                                                 */
/*                                                                          */
/*   This function does not use any other functions and libraly functions.  */
/*                                                                          */
/*   Data may be decompressed using function decomp6.                       */
/*                                                                          */
/*   Program cmp6test can help you to understand the algorithm of this      */
/*   function.                                                              */
/*                                                                          */
/*                                         23rd Aug 1990    K.Saito         */
/****************************************************************************/

int	cmprs6(lx,ix,lc,cbuff)
	long 	lx;
	long 	*ix;
	long	*lc;
	char	*cbuff;
{	char	achar[65];
	long	buf;
	long	jn;
	long	ioff;
	long	imax;
	long	mflag,nflag;
	long	counter;
	int 	ierr;
	int		shiftctr;

	 achar[0]='+'; achar[1]='-'; achar[2]='0'; achar[3]='1'; achar[4]='2';
	 achar[5]='3'; achar[6]='4'; achar[7]='5'; achar[8]='6'; achar[9]='7';
	achar[10]='8';achar[11]='9';achar[12]='A';achar[13]='B';achar[14]='C';
	achar[15]='D';achar[16]='E';achar[17]='F';achar[18]='G';achar[19]='H';
	achar[20]='I';achar[21]='J';achar[22]='K';achar[23]='L';achar[24]='M';
	achar[25]='N';achar[26]='O';achar[27]='P';achar[28]='Q';achar[29]='R';
	achar[30]='S';achar[31]='T';achar[32]='U';achar[33]='V';achar[34]='W';
	achar[35]='X';achar[36]='Y';achar[37]='Z';achar[38]='a';achar[39]='b';
	achar[40]='c';achar[41]='d';achar[42]='e';achar[43]='f';achar[44]='g';
	achar[45]='h';achar[46]='i';achar[47]='j';achar[48]='k';achar[49]='l';
	achar[50]='m';achar[51]='n';achar[52]='o';achar[53]='p';achar[54]='q';
	achar[55]='r';achar[56]='s';achar[57]='t';achar[58]='u';achar[59]='v';
	achar[60]='w';achar[61]='x';achar[62]='y';achar[63]='z';achar[64]=0;
	mflag = 0x20;
	ierr = 0;
	imax = *lc;
	ioff = 0;

	for(counter=0;counter < lx; counter++)
	{	jn = ix[counter];
		nflag = 0x00;
		shiftctr = 5;       /* Always 5 times bit shift */

		if(jn < 0)          /* Convert sign and magunitude */
		{	nflag = 0x10;
			jn = -jn;
		}

		jn = ((jn & 0xe0000000) ? 0x1fffffff : jn);  /* Too big number */

		jn = (jn << 3);    /* At most 29 bits */

		while(((jn >> 27) == 0x00) && (shiftctr > 0))  /* Find first + bit */
		{	jn = (jn << 5);
			shiftctr--;
		}

		buf = (jn >> 28) & 0x0000000f;     /* Upper 4 bits to lower 4bits */
		buf = buf + nflag  + (shiftctr ? mflag : 0x00);  /* add flags */

		cbuff[ioff] = achar[buf];     /* 4bits number to character */
		jn = (jn << 4);               /* go to next 5 bits at Upper bits*/
		shiftctr--;
		ioff++;

		while((shiftctr >= 0) && (ioff < imax))
		{	buf = ((jn >> 27) & 0x0000001f);   /* Upper 5 bits to lower */
			buf = buf + (shiftctr ? mflag : 0x00);  /* add flag */
			cbuff[ioff] = achar[buf];
			jn = (jn << 5);
			shiftctr--;
			ioff++;
		}
		if(ioff >= (imax-1))  /* to save space for ASCIIZ delimiter */
		{	ierr = -1;
			*lc = imax;
			break;
		}
	}
	cbuff[ioff] = ' ';   /* Add at least one space */
	while(((((ioff++)+1) % 80) != 0) && (ioff < imax))
		cbuff[ioff] = ' ';
	cbuff[ioff] = 0;         /* ASCIIZ delimiter */
	*lc = ioff;
	return(ierr);
}

/* this function no longer used    DJS 29/01/99
get_ts_nos(detnr,pdetnr)
int *detnr,*pdetnr;
{
	char line[10];
	FILE *f1;
        printf("\nAbout to open DETNR file");
		if((f1=fopen(DETNR,"r"))==NULL)
		{
			printf("\nError opening %s", DETNR);
			*detnr=-99;
			return(0);
		}
*//* detnr file now open read in first line*//*
    printf("about to read in line of detnr");
	fgets(line,10,f1);
	fclose(f1);		
	*detnr=atoi(line);
*//* now try and open the pderner file*//*
	if((f1=fopen(PDETNR,"r"))==NULL)
		{
				printf("\nError opening %s\n", PDETNR);
				*pdetnr=-99;
				return(0);
		}
*//* PDETNR file now open read in first line*//*
	fgets(line,10,f1);
	fclose(f1);
	*pdetnr=atoi(line);
}		
*/