/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:11/08/95                       */
/* Program:BCK_STRU                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Make directory structure for backup, distribution and installation.      */
/*                                                                          */
/****************************************************************************/
#include <stdio.h>
#include <dirent.h>
#include <string.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                        Date: 28/01/99     Version 7.00
------------------------------------------------------------------------------
Description: Program to make a complete backup of SEISLOG directory structure
             under /home/backup. The backup does NOT contain special libraries
             like NCURSES or drivers for extra serial boards like Digiboard.
             From this backup you may make a distribution floppy of SEISLOG
             that can be installed on another computer with qnx_dist.

Usage:       bck_stru (will ask for version number)
#endif
main(argc,argv)
int argc;
char **argv;
{
DIR *dirp;                             /* pointer to directory              */
FILE*  buf;                            /* pointer to file                   */
FILE*  in;
int    ret;                            /* return value                      */
char   filename[100];                  /* names to backup                   */
char   syscall1[100];                  /* char storage                      */
char   syscall2[100];                  /* char storage                      */
#include "shar_mem.h"
#include <sys/mman.h>
#include <fcntl.h>
#include <stdlib.h>
int      cnt;
int      ix;
int      i;
int      all;
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */
if(argc !=2)
{
    all=0;
}else{
    all=1;
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"BCK_STRU_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"BCK_STRU_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
system("/home/seislog/version");       /* enter new version number          */
dirp=opendir("/home/backup");
if(dirp == NULL)
{
    printf("BCK_STRU: Backup directory tree does not exist, create it !\n");
/*--------------------------------------------------------------------------*/
/* make directory structure for backup, distribution, installation          */
/*--------------------------------------------------------------------------*/
    system("mkdir /home/backup");
    system("mkdir /home/backup/buffer");
    system("mkdir /home/backup/error");
    system("mkdir /home/backup/save");
    system("mkdir /home/backup/seismo");
    system("mkdir /home/backup/events");
    system("mkdir /home/backup/seislog");
    system("mkdir /home/backup/seislog/pr");
    system("mkdir /home/backup/seislog/digi1");
    system("mkdir /home/backup/seislog/fonts");
    system("mkdir /home/backup/work");
    printf("BCK_STRU: New directory tree created.\n");
/*--------------------------------------------------------------------------*/
/* change the access permitions for the new directory structure             */
/*--------------------------------------------------------------------------*/
    system("chmod a+rwx /home/backup");
    system("chmod a+rwx /home/backup/buffer");
    system("chmod a+rwx /home/backup/error");
    system("chmod a+rwx /home/backup/save");
    system("chmod a+rwx /home/backup/seismo");
    system("chmod a+rwx /home/backup/events");
    system("chmod a+rwx /home/backup/seislog");
    system("chmod a+rwx /home/backup/seislog/pr");
    system("chmod a+rwx /home/backup/seislog/digi1");
    system("chmod a+rwx /home/backup/seislog/fonts");
    system("chmod a+rwx /home/backup/work");
    printf("BCK_STRU: Access modes changed in new directory structure.\n");
}else{
    printf("BCK_STRU: Backup directory tree exists.\n");
}
/*--------------------------------------------------------------------------*/
/* copy  the original .profile files into the new directory structure       */
/*--------------------------------------------------------------------------*/
system("cp /home/seislog/.profile /home/backup/seislog/.profile");
system("cp /home/events/.profile  /home/backup/events/.profile");
system("cp /home/work/.profile    /home/backup/work/.profile");
printf("BCK_STRU: All .profile files copied.\n");
/*--------------------------------------------------------------------------*/
/* copy the configuration file for the editor to the user directories       */
/*--------------------------------------------------------------------------*/
system("cp /home/seislog/.vedit.cfg /home/backup/seislog");
system("cp /home/seislog/.vedit.cfg /home/backup/events");
system("cp /home/seislog/.vedit.cfg /home/backup/work");
printf("BCK_STRU: Editor configuration file copied.\n");
/*--------------------------------------------------------------------------*/
/* remove all old files in /home/src                                        */
/*--------------------------------------------------------------------------*/
system("rm -f /home/backup/seislog/*");
/*--------------------------------------------------------------------------*/
/* open file containing all SEISLOG files needed for upgrade                */
/*--------------------------------------------------------------------------*/
buf=fopen("/home/seislog/programs.lis","r");
if(buf==0)
{
    sprintf(ptr->err_msg,"BCK_STRU: Not able to open programs.lis.");
    errlog(ptr);
    printf("BCK_STRU: Not able to open programs.lis. No backup done.\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* read all filenames, make copy command and do the copy                    */
/*--------------------------------------------------------------------------*/
for(;;)
{
    ret=fscanf(buf,"%s",filename);
    if(ret==EOF) break;
    sprintf(syscall1,"cp /home/seislog/%s ",filename);
    sprintf(syscall2,"/home/backup/seislog\n");
    strcat(syscall1,syscall2);
//    printf("%s\n",syscall1);
    system(syscall1);
    if(all==1)
    {
        cnt=0;
        for(i=0;i<20;i++)
        {
            if(filename[i]=='.')
            {
                cnt++;
                ix=i;
                if(filename[i+1]=='c')
                    cnt++;
            }
        }
        if(cnt==2)
        {
            filename[ix]='\0';
            sprintf(syscall1,"/home/seislog/%s\0",filename);
            in=fopen(syscall1,"r");
            if(in==0)
            {
//                printf("%s is not there\n",filename);
            }else{
                fclose(in);
                sprintf(syscall1,"cp /home/seislog/%s ",filename);
                sprintf(syscall2,"/home/backup/seislog\n");
                strcat(syscall1,syscall2);
//            printf("%s\n",syscall1);
                system(syscall1);
            }
        }
        system("cp /home/seislog/slog.lib /home/backup/seislog/slog.lib");
    }
}
fclose(buf); 
printf("BCK_STRU: All   C-files copied.\n");
printf("BCK_STRU: All   h-files copied.\n");
printf("BCK_STRU: All lis-files copied.\n");
system("cp /home/seislog/pr/* /home/backup/seislog/pr");
printf("BCK_STRU: All parameter files copied.\n");
//system("cp /home/seislog/digi1/*.* /home/backup/seislog/digi1");
//printf("BCK_STRU: All DIGI board files copied.\n");
system("cp /home/seislog/fonts/* /home/backup/seislog/fonts");
printf("BCK_STRU: All font files copied.\n");
/*--------------------------------------------------------------------------*/
/* copy makefile for a/d driver to backup tree                              */
/*--------------------------------------------------------------------------*/
system("cp /home/seislog/Makefile /home/backup/seislog");
printf("BCK_STRU: Makefile for a/d copied.\n");
/*--------------------------------------------------------------------------*/
/* copy use file for a/d driver                                             */
/*--------------------------------------------------------------------------*/
system("cp /home/seislog/rbo_use.x /home/backup/seislog");
printf("BCK_STRU: Use file for a/d driver copied.\n");
printf("\n                            *** BACKUP FINISHED ***\n");
}