/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:23/01/95                       */
/* Program:BUF_PIP                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 11/05/00 t.u. Old Earth Data digitizer with internal timestamping,       */
/*               (Luis box): now update of CPU clock                        */
/* 31/05/00 djs+tu at BGS: support for 'new' Earth Data digitizer unit 64   */
/*                         preliminary version multiserial 'new' ED unit 34 */                         
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This routine receives data-buffers from the A/D driver supporting three  */
/* different A/D converter boards (VADC11,VADC21 and VADC36 manufactored by */
/* Oettle & Reichler in Augsburg), or it receives data-buffers from         */
/* different digitizers via serial line RS232. Supported digitizers are:    */
/* Nanometrics RD3, Nanometrics RD6, Nanometrics HRD and Earth Data 2400.   */
/* Each time a data buffer is ready it is time tagged and copied into a     */
/* memory pipe defined as a circular buffer.                                */
/* The time used is the CPU time which is updated continously by GPS clock  */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <math.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program starts up program to read from specified digitizer.
             When 1 second of data is ready, it time stamps the buffer and
             signals to other programs that one buffer is ready to process.

Usage:       buf_pip (no parameters)
#endif


//FILE *mtv;
int      secnew;
int      secold;
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
char     rec_msg[30];                  /* RECEIVE message                   */
char     rec_tim[35];
char     dum;
int      ii;
unsigned int  cop;
float    akk;
FILE     *crc;
struct timespec start;
struct timespec cputid;
main()
{
int      no_of_times;                  /* total number of buffers received  */
int      no_of_syncs;                  /* total number of synchronized times*/
char     tider[80];
char     sync;                         /* good time-bad time S-not S        */
int      snc;                          /* good time-bad time 1-0            */

double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
int msold;
int msnew;
int msdif;
float difakk;
int teller;
pid_t    proxy;
pid_t    proxy2;
int      pid;
unsigned msg;
struct   timespec ttag;
long     hrn;                          /* hours                             */
long     min;                          /* minutes                           */
long     day;                          /* days                              */
long     mon;                          /* months                            */
long     yrn;                          /* year                              */
long     doy;                          /* day of year                       */
float    sec;                          /* seconds                           */
long     dhrn;
long     dmin;
long     dday;
long     dmon;
long     dyrn;
long     ddoy;
float    dsec;
float    tck;                          /* ticks (CPU clock)                 */
float    tckrat;                       /* ticks per second (CPU clock)      */
float    milliseconds;
double   msecsn;                       /* total number of seconds           */
double   sectot;                       /* total number of seconds           */
int      times,date,days,ticks;        /* declaration  _SYSDATE system call */
unsigned short gain_table[MAXCHA];     /* Gain table                        */
int      i;                            /* Counter                           */
int      ip;                           /* Counter                           */
int      idd;                          /* Buffer indicator                  */
int      buf_to_tag;
int      buf_pip_id;                   /* id this process when attached     */
int      det_evt_id;
int      drv_id;                       /* id for driver reading digitizer   */
int      dpl_console_id;               /* id for console dpl program        */
int      dpl_serial_id;                /* id for serial  dpl program        */
int      dig_n24_id;                   /* id for driver reading HRD24       */
int      rtp_id;                       /* id for rtp program                */
int      rat_id;                       /* id for rat program                */
int      mon_id;                       /* id for mon program                */
int      tim_id;                       /* id for tim_chk program            */
int      avg_id;                       /* id for tim_avg program            */
int      trg_id;                       /* id for mon_trg program            */
char     rep_msg[10];                  /* REPLY   message                   */
char     snd_msg[10];                  /* SEND    message                   */
char     car;
float    drf;
float    cons1;
float    cons2;
float    diff;
int      cor;
struct   timespec start;               /* upgr. 8.21                        */
int 	 time_diff;            /* upgr. 8.22 for Earth Data digitiser */
//if((mtv=fopen("/home/seislog/times","w"))==NULL)
//  exit(0);

YR=1970;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSEC70);

buf_pip_id=qnx_name_attach(0,"buf_pip");
if(buf_pip_id == -1)
{
    printf("BUF_PIP attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"BUF_PIP_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"BUF_PIP_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
//system("/home/seislog/circle");
//if((crc=fopen("/home/buffer/CIRCLE","w"))==NULL)
//{
//    printf("BUF_PIP: Can't open CIRCLE\n");
//    exit(0);
//}
akk=0.0;

ptr->oktime = 0;                       /* Set indicator for good time       */
                                       /* received to 0 initially.          */
ptr->noktime = 0;                      /* Set number of correct times       */
                                       /* received to zero                  */
idd = 0;                               /* Bufferswitch where to put         */
                                       /* timestring                        */
ptr->kueue=-1;                         /* initialize pipe kueue             */
//for(i=0;i<2000;i++)                    /* initialize pipe monitoring list   */
//    ptr->list[i]='.';         
setprio(getpid(),12);
ptr->dplc_proxy=0;
ptr->dpls_proxy=0;
ptr->mon_proxy=0;
ptr->tim_proxy=0;
ptr->avg_proxy=0;
ptr->rat_proxy=0;
/****************************************************************************/
/* Do initialization according to type of unit (A/D-board or digitizer)     */
/****************************************************************************/
switch(ptr->unit)
{
/*--------------------------------------------------------------------------*/
/*               GeoSys/UiB GBV-116/316 3 LOWCOST DIGITIZER                 */
/*--------------------------------------------------------------------------*/
    case 10:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_low&");  /* read from GBV     */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   Metrabyte 16 CHANNEL DIGITIZER                         */
/*--------------------------------------------------------------------------*/
    case 30:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
//    system("on -t /dev/con6 /home/seislog/dig_met&");  /* read from DIG_MET                 */
    system("on -t /dev/con6 /home/seislog/dig_me2&");  /* read from DIG_MET                 */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   Terra IDS-24 3 CHANNEL DIGITIZER                             */
/*--------------------------------------------------------------------------*/
    case 31:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_ge1&");  /* read from DIG_GE1                 */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   GeoSys 12 CHANNEL DIGITIZER                             */
/*--------------------------------------------------------------------------*/
    case 32:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_ram&");  /* read from DIG_GE1                 */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   GeoSys/UiB GBV LOWCOST DIGITIZER Multiserial           */
/*--------------------------------------------------------------------------*/
    case 33:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_ser&");  /* read from Digiboard               */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                    Earth Data Multiserial                                */
/*--------------------------------------------------------------------------*/
    case 34:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_edm&");  /* read from Digiboard               */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   Terra IDS-24 3 CHANNEL DIGITIZER 1-SECOND BUFFERS      */
/*--------------------------------------------------------------------------*/
    case 35:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_ge2&");  /* read from DIG_GE1                 */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   Metrabyte 64 CHANNEL DIGITIZER                         */
/*--------------------------------------------------------------------------*/
    case 36:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_m18&");  /* read from DIG_MET                 */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*             A/D-board Royal Belgium Observatory                          */
/*--------------------------------------------------------------------------*/
    case 37:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_rbo&");  /* read from RBO A/D                 */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   Ramon Ortiz 1 CHANNEL DIGITIZER                        */
/*--------------------------------------------------------------------------*/
    case 38:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_ror&");  /* read from DIG_GE1                 */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*               Kinemetrics Everest                                        */
/*--------------------------------------------------------------------------*/
    case 39:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_evr&");  /* read from GBV     */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   NANOMETRICS 3 CHANNEL DIGITIZER HRD24 compressed       */
/*--------------------------------------------------------------------------*/
    case 40:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_n24&");  /* read from HRD3    */
    sleep(1);
    dig_n24_id=qnx_name_locate(0,"dig_n24",0,&cop);
    if(dig_n24_id == -1)
    {
          printf("BUF_PIP: Cant locate DIG_N24\n");
          exit(0);
    }
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    printf("DIG_N24 started\n");
    break;
/*--------------------------------------------------------------------------*/
/*                   GURALP 3 CHANNEL DIGITIZER compressed                  */
/*--------------------------------------------------------------------------*/
    case 41:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_g24&");  /* read from HRD3    */
    sleep(1);
    dig_n24_id=qnx_name_locate(0,"dig_g24",0,&cop);
    if(dig_n24_id == -1)
    {
          printf("BUF_PIP: Cant locate DIG_G24\n");
          exit(0);
    }
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    printf("DIG_G24 started\n");
    break;
/*--------------------------------------------------------------------------*/
/*                   IDS-24 DIGITIZER IRIS PASCAL FORMAT                    */
/*--------------------------------------------------------------------------*/
    case 42:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_i24&");  /* read from HRD3    */
    sleep(1);
    dig_n24_id=qnx_name_locate(0,"dig_i24",0,&cop);
    if(dig_n24_id == -1)
    {
          printf("BUF_PIP: Cant locate DIG_G24\n");
          exit(0);
    }
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    printf("DIG_I24 started\n");
    break;
/*--------------------------------------------------------------------------*/
/*                   NANOMETRICS 3 CHANNEL DIGITIZER                        */
/*--------------------------------------------------------------------------*/
    case 53:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_nan&");  /* read from RD3                     */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   NANOMETRICS 6 CHANNEL DIGITIZER                        */
/*--------------------------------------------------------------------------*/
    case 56:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_nan&");  /* read from RD6                     */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                      EARTH DATA 24 BIT DIGITIZER                         */
/*--------------------------------------------------------------------------*/
    case 63:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_eda&");  /* read from Earth Data              */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                      EARTH DATA 24 BIT DIGITIZER                         */
/*--------------------------------------------------------------------------*/
    case 64:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
//    system("on -t /dev/con6 /home/seislog/set_edn");
    system("on -t /dev/con6 /home/seislog/dig_edn&");  /* read from Earth Data              */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                      Digiboard Multiserial                               */
/*--------------------------------------------------------------------------*/
    case 65:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_ser&");  /* read from Digiboard               */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                      B & B converter                                     */
/*--------------------------------------------------------------------------*/
    case 66:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_bbc&");  /* read from B & B converter         */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;
/*--------------------------------------------------------------------------*/
/*                   Ramon Ortiz 3 CHANNEL DIGITIZER 24 bits 9600 baud      */
/*--------------------------------------------------------------------------*/
    case 70:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
//    system("on -t /dev/con6 /home/seislog/dig_324&");  /* read from DIG_324                 */
    system("/home/seislog/dig_324&");
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;

/*--------------------------------------------------------------------------*/
/*                            SYNTHETIC DATA                                */
/*--------------------------------------------------------------------------*/
    case 96:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_syn&");  /* make synthetic data               */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;

/*--------------------------------------------------------------------------*/
/*                            SYNTHETIC DATA                                */
/*--------------------------------------------------------------------------*/
    case 97:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("on -t /dev/con6 /home/seislog/dig_wav&");  /* make synthetic data               */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;

/*--------------------------------------------------------------------------*/
/*                            24 BIT ILI **DJS                                */
/*--------------------------------------------------------------------------*/
    case 99:
    ptr->sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    ptr->sec_in_crc=CRCSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
    system("/home/seislog/ili_seis &");  /* start 24 bit driver             */
    drv_id=Receive(0,&rec_msg,1);
    Reply(drv_id,&rep_msg,1);
    break;

}
ptr->crc_in=0;

no_of_times=0;                         /* total number of buffers received  */
no_of_syncs=0;                         /* total number of synchronized times*/

for (;;)                               /* Infinite loop                     */
{
    for(ptr->sec_in=0;ptr->sec_in<ptr->sec_in_pip;ptr->sec_in++) /*pipe loop*/
    {
        ptr->alive[0]=1;               /* tell I am alive                   */
/*--------------------------------------------------------------------------*/
/* Enter wait state and wait for "buffer0" or "buffer1" to be ready         */
/*--------------------------------------------------------------------------*/

switch(ptr->unit)
{
/*--------------------------------------------------------------------------*/
/*               GeoSys/UiB GBV-116/316 3 LOWCOST DIGITIZER                 */
/*--------------------------------------------------------------------------*/
    case 10:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* inform DIG_LOW that I'm ready     */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging. SYNC has been found      */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let DIG_LOW go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=(ptr->index+1)%2;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);

        drv_id=Receive(0,&rec_msg,1);  /* wait for DIG_LOW to finish buffer */
        Reply(drv_id,&rep_msg,1);      /* reply to DIG_LOW                  */
    }else{
        Reply(drv_id,&rep_msg,1);      /* let DIG_LOW go on                 */
        drv_id=Receive(0,&rec_msg,1);  /* wait for DIG_LOW to finish buffer */
        Reply(drv_id,&rep_msg,1);      /* reply to DIG_LOW                  */
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      Metrabyte 16 channel DIGITIZER                      */
/*--------------------------------------------------------------------------*/
    case 30:
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_me2 go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      Terra IDS-24 channel DIGITIZER                      */
/*--------------------------------------------------------------------------*/
    case 31:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_met go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_GE1 sequence                  */
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      GeoSys 3 channel DIGITIZER                          */
/*--------------------------------------------------------------------------*/
    case 32:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_met go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
YR=ptr->inpptid[0];
MTH=ptr->inpptid[1];
DAY=ptr->inpptid[2];
HR =ptr->inpptid[3];
MIN=ptr->inpptid[4];
SEC=ptr->inppsec;

TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
sectot=MSECS;
SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_GE1 sequence                  */
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      Terra IDS-24 channel DIGITIZER                      */
/*--------------------------------------------------------------------------*/
    case 33:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    milliseconds=ptr->itmde;           /* freeze counters immediately       */
    sectot=(float)ptr->icmde;          /* freeze counters immediately       */
//if(ptr->clktyp != 8)
//{
//        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
//        Reply(drv_id,&rep_msg,1);      /* let dig_synt go on                */
//        cor_cpu(ptr);
//        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
//        buf_to_tag=ptr->index;
//        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
//        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
//        Reply(drv_id,&rep_msg,1);      /* DIG_SYN sequence                  */
//}
    break;
/*--------------------------------------------------------------------------*/
/*                      Terra IDS-24 Multiserial                            */
/*--------------------------------------------------------------------------*/
    case 34:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    break;

/*--------------------------------------------------------------------------*/
/*                      Terra IDS-24 channel DIGITIZER 1-SECOND BUFFERS     */
/*--------------------------------------------------------------------------*/
    case 35:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_met go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_GE1 sequence                  */
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      Metrabyte 64 channel DIGITIZER                      */
/*--------------------------------------------------------------------------*/
    case 36:
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    milliseconds=ptr->itmde;           /* freeze counters immediately       */
    sectot=(float)ptr->icmde;          /* freeze counters immediately       */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {

        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_me2 go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);

    }
    break;

/*--------------------------------------------------------------------------*/
/*             A/D-board Royal Belgium Observatory                          */
/*--------------------------------------------------------------------------*/
    case 37:
    drv_id=Receive(0,&rec_msg,1);
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        milliseconds=ptr->itmde;
        sectot=(float)ptr->icmde;
        sectot=sectot+milliseconds/1000.0;
        Reply(drv_id,&rep_msg,1);      /* let rbo_3 go on                   */
        cor_cpu(ptr);
        sectot=sectot + ptr->delay;    /* add delay given in parameters     */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
        buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
        tagbuf(hrn,min,sec,yrn,mon,day,doy,buf_to_tag);
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      Ramon Ortiz 1 channel DIGITIZER                     */
/*--------------------------------------------------------------------------*/
    case 38:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_tim,32);      /* wait for right moment to tag data */
        Reply(drv_id,&rep_msg,1);      /* DIG_GE1 sequence                  */


    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_met go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_GE1 sequence                  */
    }else{
        sscanf(rec_tim,"%2d%c%2d%c%f6.3\n",&hrn,&dum,&min,&dum,&sec);
        for(ip=0;ip<14;ip++)
            rec_tim[ip]=rec_tim[ip+13];
        sscanf(rec_tim,"%2d%c%2d%c%4d\n",&day,&dum,&mon,&dum,&yrn);
//       printf("%2d:%2d:%6.3f\n",hrn,min,sec);

        YR=yrn;
        MTH=mon;
        DAY=day;
        HR=hrn;
        MIN=min;
        SEC=sec;
        TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
        if(yrn==1000)        // GPS out of sync ?
        {
          if(clock_gettime(CLOCK_REALTIME,&start)== -1) 
          {
            perror("clock gettime" );
            exit( EXIT_FAILURE);
          }
          MSECS=start.tv_sec;
        }

        sectot=MSECS;
        sectot=sectot+ptr->delay;
        MSECS=sectot;                  /* correction 10/10/01 JanMayen */
        if(yrn==1000)       // GPS out of sync ?
        {
          MSECS++;
          SECTIM(MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
          ptr->TimFlg[0]=2;
        }else{
          SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
          ptr->TimFlg[0]=1;
        }
        buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);

        start.tv_sec=(long)(MSECS-MSEC70);  /* upgr. 8.21                   */
        start.tv_nsec=0;                    /* upgr. 8.21                   */
        system("rtc -s hw");                /* upgr. 8.21                   */
        clock_settime(CLOCK_REALTIME,&start); /* upgr. 8.21                 */
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_GE1 sequence                  */

}


    break;

/*--------------------------------------------------------------------------*/
/*               Kinemetrics Everest                                        */
/*--------------------------------------------------------------------------*/
    case 39:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* inform DIG_LOW that I'm ready     */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging. SYNC has been found      */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let DIG_LOW go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=(ptr->index+1)%2;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);

        drv_id=Receive(0,&rec_msg,1);  /* wait for DIG_LOW to finish buffer */
        Reply(drv_id,&rep_msg,1);      /* reply to DIG_LOW                  */
    }else{
        Reply(drv_id,&rep_msg,1);      /* let DIG_LOW go on                 */
        drv_id=Receive(0,&rec_msg,1);  /* wait for DIG_LOW to finish buffer */
        Reply(drv_id,&rep_msg,1);      /* reply to DIG_LOW                  */
    }
    break;



/*--------------------------------------------------------------------------*/
/*                   NANOMETRICS 3 CHANNEL DIGITIZER HRD24 compressed       */
/*--------------------------------------------------------------------------*/
    case 40:
//    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell Nanometrics that I'm ready   */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
//    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
//    Reply(drv_id,&rep_msg,1);      /* let dig_nan go on                 */
    drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */

//    det_evt_id=Receive(0,&msg,sizeof(msg));
//    printf("Got proxy\n");
    ptr->alive[3]=1;                   /* internal timing, tell clock active*/
    break;
/*--------------------------------------------------------------------------*/
/*                   GURALP 3 CHANNEL DIGITIZER  compressed                 */
/*--------------------------------------------------------------------------*/
    case 41:
//    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell Nanometrics that I'm ready   */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
//    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
//    Reply(drv_id,&rep_msg,1);      /* let dig_nan go on                 */
    drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */

//    det_evt_id=Receive(0,&msg,sizeof(msg));
//    printf("Got proxy\n");
    ptr->alive[3]=1;                   /* internal timing, tell clock active*/
    break;
/*--------------------------------------------------------------------------*/
/*                   IDS-24 DIGITIZER  IRIS PASCAL FORMAT                   */
/*--------------------------------------------------------------------------*/
    case 42:
//    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell Nanometrics that I'm ready   */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
//    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
//    Reply(drv_id,&rep_msg,1);      /* let dig_nan go on                 */
    drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */

//    det_evt_id=Receive(0,&msg,sizeof(msg));
//    printf("Got proxy\n");
    ptr->alive[3]=1;                   /* internal timing, tell clock active*/
    break;
/*--------------------------------------------------------------------------*/
/*                   NANOMETRICS 3 CHANNEL DIGITIZER                        */
/*--------------------------------------------------------------------------*/
    case 53:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell Nanometrics that I'm ready   */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_nan go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_NAN sequence                  */
    }else{
        printf("BUF_PIP: You have to select another clock for this unit.\n");
    }
    break;
/*--------------------------------------------------------------------------*/
/*                   NANOMETRICS 6 CHANNEL DIGITIZER                        */
/*--------------------------------------------------------------------------*/
    case 56:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell Nanometrics that I'm ready   */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_nan go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_NAN sequence                  */
    }else{
        printf("BUF_PIP: You have to select another clock for this unit.\n");
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      EARTH DATA 24 BIT DIGITIZER                         */
/*--------------------------------------------------------------------------*/
    case 63:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)               /* check if data already tagged ?    */
    {                                  /* no, use frozen counters           */
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_eda go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_EDA sequence                  */
    }else{
        Reply(drv_id,&rep_msg,1);      /* let dig_eda go on                 */
        drv_id=Receive(0,&rec_msg,30); /* wait for message containing time  */        
        ptr->alive[3]=1;               /* tell SYS_WAT that the timing      */
                                       /* program is running. For clocktype */
                                       /* 8 there is no program, but this is*/
                                       /* only to keep SYS_WAT satisfied.   */
        sscanf(rec_msg,"%2d %2d %f6.3\n",&hrn,&min,&sec);
        for(ip=0;ip<8;ip++)
            rec_msg[ip]=rec_msg[ip+13];
        sscanf(rec_msg,"%2d %2d %2d\n",&day,&mon,&yrn);
/*       printf("%2d:%2d:%6.3f\n",hrn,min,sec);*/
        YR=yrn;
        MTH=mon;
        DAY=day;
        HR=hrn;
        MIN=min;
        SEC=sec;
        TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
        sectot=MSECS;
        sectot=sectot+ptr->delay;
        MSECS=sectot;                  /* correction 10/10/01 JanMayen */
        SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
        ptr->TimFlg[0]=1;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);

        start.tv_sec=(long)(MSECS-MSEC70);  /* upgr. 8.21                   */
        start.tv_nsec=0;                    /* upgr. 8.21                   */
        system("rtc -s hw");                /* upgr. 8.21                   */
        clock_settime(CLOCK_REALTIME,&start); /* upgr. 8.21                 */

        Reply(drv_id,&rep_msg,1);
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      EARTH DATA 24 BIT DIGITIZER                         */
/*--------------------------------------------------------------------------*/
    case 64:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    if(ptr->clktyp != 8)               /* check if data already tagged ?    */
    {                                  /* no, use frozen counters           */
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_eda go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        ptr->alive[3]=1;               /* tell SYS_WAT that the timing      */
        Reply(drv_id,&rep_msg,1);      /* DIG_EDA sequence                  */
    }else{
        Reply(drv_id,&rep_msg,1);      /* let dig_eda go on                 */
        drv_id=Receive(0,&rec_msg,30); /* wait for message containing time  */        
        ptr->alive[3]=1;               /* tell SYS_WAT that the timing      */
                                       /* program is running. For clocktype */
                                       /* 8 there is no program, but this is*/
                                       /* only to keep SYS_WAT satisfied.   */
        sscanf(rec_msg,"%2d %2d %6f %2d %2d %2d %d\n",&hrn,&min,&sec,&day,&mon,&yrn,&time_diff);
/*       printf("%2d:%2d:%6.3f\n",hrn,min,sec);*/
        YR=yrn;
        MTH=mon;
        DAY=day;
        HR=hrn;
        MIN=min;
        SEC=sec;
        TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
        sectot=MSECS;
        sectot=sectot+ptr->delay;
        if(time_diff > 1000000)
        {
          system("/home/seislog/clk_cpu &");
          ptr->clktyp=0;
        }

		if (time_diff > 0) 
        {
          MSECS += time_diff;
//          printf ("BUF_PIP: time_diff = %d\n",time_diff);
        }
        SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;

/*--------------------------------------------------------------------------*/
/* prepare seconds and nanoseconds in structure for CLOCK_SETIME            */
/*--------------------------------------------------------------------------*/
        start.tv_sec=(long)(MSECS-MSEC70);
        start.tv_nsec=0;

        system("rtc -s hw");
/*--------------------------------------------------------------------------*/
/* set CPU clock to time received from GPS                                  */
/*--------------------------------------------------------------------------*/
        clock_settime(CLOCK_REALTIME,&start);


//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        if(time_diff == 0)  /* GPS synchronised, fill Last Update string     */
        {
          for(ii=0;ii<27;ii++)
            ptr->Timsav[ii]=ptr->timcpy[ii];
        }
        Reply(drv_id,&rep_msg,1);
    }
    break;
/*--------------------------------------------------------------------------*/
/*                      Digi board multiserial                              */
/*--------------------------------------------------------------------------*/
    case 69:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
drv_id=Receive(0,&rec_msg,1);
    milliseconds=ptr->itmde;           /* freeze counters immediately       */
    sectot=(float)ptr->icmde;          /* freeze counters immediately       */
Reply(drv_id,&rep_msg,1);
drv_id=Receive(0,&rec_msg,30);
ptr->alive[3]=1;
sscanf(rec_msg,"%2d %2d %f6.3\n",&hrn,&min,&sec);
for(ip=0;ip<8;ip++)
    rec_msg[ip]=rec_msg[ip+13];
sscanf(rec_msg,"%2d %2d %2d\n",&day,&mon,&yrn);
/*       printf("%2d:%2d:%6.3f\n",hrn,min,sec);*/

YR=yrn;
MTH=mon;
DAY=day;
HR=hrn;
MIN=min;
SEC=sec;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
sectot=MSECS;
sectot=sectot+ptr->delay;
SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
tagbuf(hrn,min,sec,yrn,mon,day,doy,buf_to_tag);
Reply(drv_id,&rep_msg,1);
goto tid;
        sectot=sectot+milliseconds/1000.0;
        cor_cpu(ptr);
        sectot=sectot + ptr->delay;    /* add delay given in parameters     */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
        buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
        tagbuf(hrn,min,sec,yrn,mon,day,doy,buf_to_tag);
tid:
    i=0;
    break;
/*--------------------------------------------------------------------------*/
/*                      Digi board multiserial    old 65                    */
/*--------------------------------------------------------------------------*/
    case 65:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    milliseconds=ptr->itmde;           /* freeze counters immediately       */
    sectot=(float)ptr->icmde;          /* freeze counters immediately       */
        sectot=sectot+milliseconds/1000.0;
        cor_cpu(ptr);
        sectot=sectot + ptr->delay;    /* add delay given in parameters     */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
        buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
        tagbuf(hrn,min,sec,yrn,mon,day,doy,buf_to_tag);
    break;
/*--------------------------------------------------------------------------*/
/*                      B & B converter                                     */
/*--------------------------------------------------------------------------*/
    case 66:
    drv_id=Receive(0,&rec_msg,1);      /* wait for right moment to tag data */
    milliseconds=ptr->itmde;           /* freeze counters immediately       */
    sectot=(float)ptr->icmde;          /* freeze counters immediately       */
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        sectot=sectot+milliseconds/1000.0;
        Reply(drv_id,&rep_msg,1);      /* let dig_bbc go on                 */
        cor_cpu(ptr);
        sectot=sectot + ptr->delay;    /* add delay given in parameters     */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
        buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
        tagbuf(hrn,min,sec,yrn,mon,day,doy,buf_to_tag);
    }
    break;
/*--------------------------------------------------------------------------*/
/*            Ramon Ortiz 3 channel DIGITIZER 24 bits 9600 baud             */
/*--------------------------------------------------------------------------*/
    case 70:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell ed_24 that I'm ready         */
                                       /* when I receive the reply, I enter */
                                       /* RECEIVE mode to prepare for time- */
                                       /* tagging.                          */
    drv_id=Receive(0,&rec_tim,32);     /* wait for right moment to tag data */
    Reply(drv_id,&rep_msg,1);          /* DIG_GE1 sequence                  */


    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_met go on                 */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        secnew=(int)SEC;
//        printf("%3d %3d\n",secold,secnew);
//        if(secnew == secold)
//          printf("%d  %6.3f\n",ptr->icmde,SEC);
        secold=secnew;
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_GE1 sequence                  */
    }else{
        sscanf(rec_tim,"%2d%c%2d%c%f6.3\n",&hrn,&dum,&min,&dum,&sec);
        for(ip=0;ip<14;ip++)
            rec_tim[ip]=rec_tim[ip+13];
        sscanf(rec_tim,"%2d%c%2d%c%4d\n",&day,&dum,&mon,&dum,&yrn);
//       printf("%2d:%2d:%6.3f\n",hrn,min,sec);

        YR=yrn;
        MTH=mon;
        DAY=day;
        HR=hrn;
        MIN=min;
        SEC=sec;
        TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
        if(yrn==1000)        // GPS out of sync ?
        {
          if(clock_gettime(CLOCK_REALTIME,&start)== -1) 
          {
            perror("clock gettime" );
            exit( EXIT_FAILURE);
          }
          MSECS=start.tv_sec;
        }

        sectot=MSECS;
        sectot=sectot+ptr->delay;
        MSECS=sectot;                  /* correction 10/10/01 JanMayen */
        if(yrn==1000)       // GPS out of sync ?
        {
          MSECS++;
          SECTIM(MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
          ptr->TimFlg[0]=2;
        }else{
          SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
          ptr->TimFlg[0]=1;
        }
        buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);

        start.tv_sec=(long)(MSECS-MSEC70);  /* upgr. 8.21                   */
        start.tv_nsec=0;                    /* upgr. 8.21                   */
        system("rtc -s hw");                /* upgr. 8.21                   */
        clock_settime(CLOCK_REALTIME,&start); /* upgr. 8.21                 */
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_GE1 sequence                  */

}


    break;

/*--------------------------------------------------------------------------*/
/*                            SYNTHETIC DATA                                */
/*--------------------------------------------------------------------------*/
    case 96:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell synt_dat that I'm ready      */
    drv_id=Receive(0,&rec_msg,30);
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        MSECS=ptr->icmde+(float)ptr->itmde/1000.0+ptr->delay;
        Reply(drv_id,&rep_msg,1);      /* let dig_synt go on                */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        buf_to_tag=ptr->index;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_SYN sequence                  */
    }
    break;
/*--------------------------------------------------------------------------*/
/*                            SYNTHETIC DATA                                */
/*--------------------------------------------------------------------------*/
    case 97:
    Send(drv_id,&snd_msg,&rec_msg,1,1);/* tell synt_dat that I'm ready      */
    drv_id=Receive(0,&rec_msg,30);
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        milliseconds=ptr->itmde;
        sectot=(float)ptr->icmde;
        sectot=sectot+milliseconds/1000.0;
        Reply(drv_id,&rep_msg,1);      /* let dig_synt go on                */
        cor_cpu(ptr);
        sectot=sectot + ptr->delay;    /* add delay given in parameters     */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
        buf_to_tag=ptr->index;
        tagbuf(hrn,min,sec,yrn,mon,day,doy,buf_to_tag);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_SYN sequence                  */
    }
    break;

    case 197:
    drv_id=Receive(0,&rec_msg,30);
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        if(clock_gettime(CLOCK_REALTIME,&ttag)== -1) 
        {
            perror("clock gettime" );
            exit( EXIT_FAILURE);
        }
    }
    Reply(drv_id,&rep_msg,1);
    sectot=ttag.tv_sec;
    sectot=sectot+ ((float)ttag.tv_nsec / 1000000000.0);
    sectot=sectot +3600.0+ ptr->delay;           /* add delay given in parameters  */
    SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
    buf_to_tag=ptr->index;
//printf("BUF_PIP Tag buffer : %d %2d %2d\n",buf_to_tag,hrn,min);
    tagbuf(hrn,min,sec,yrn,mon,day,doy,buf_to_tag);
    break;


/*--------------------------------------------------------------------------*/
/*                           24 BIT ILI  **DJS                                */
/*--------------------------------------------------------------------------*/
    case 99:
    if(ptr->clktyp != 8)  /* for any clocktype except for direct tagging    */
    {
        milliseconds=ptr->itmde;
        sectot=(float)ptr->icmde;
        sectot=sectot+milliseconds/1000.0;
        cor_cpu(ptr);
        sectot=sectot + ptr->delay;    /* add delay given in parameters     */
        SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,idd);
        drv_id=Receive(0,&rec_msg,30); /* dummy RECEIVE & REPLY to satisfy  */
        Reply(drv_id,&rep_msg,1);      /* DIG_SYN sequence                  */
    }
    else                     /* direct time tagging */
    {
        drv_id = Receive(0, &rec_msg, 30);   /* message contains time */
        ptr->alive[3] = 1;                   /* tell I'm alive */
        sscanf(rec_msg,"%2d %2d %f6.3\n",&hrn,&min,&sec);
        for(ip=0;ip<8;ip++)
            rec_msg[ip]=rec_msg[ip+13];
        sscanf(rec_msg,"%2d %2d %2d\n",&day,&mon,&yrn);
//printf ("BUF_PIP: %2d %2d %6.3f %2d %2d %2d\n", hrn,min,sec,day,mon,yrn);
        YR=yrn;
        MTH=mon;
        DAY=day;
        HR=hrn;
        MIN=min;
        SEC=sec;
       
        TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
        sectot=MSECS;
        sectot = sectot + ptr->delay;
        SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
        yrn=YR;
        mon=MTH;
        day=DAY;
        hrn=HR;
        min=MIN;
        doy=DOY;
        sec=SEC;
        TAGBUF(HR,MIN,SEC,YR,MTH,DAY,DOY,idd);
        Reply (drv_id,&rep_msg,1);
    }
    break;

}

idd ^=1;                               /* Toggle buffer switch              */  
bf_chain();                            /* copy buffer to memory pipe        */
ptr->kueue=ptr->kueue+1;               /* incr. number of buffers in queue  */
if(ptr->unit==40)
{
    Reply(drv_id,&rep_msg,1);      /* DIG_NAN sequence                  */
}
if(ptr->unit==41)
{
    Reply(drv_id,&rep_msg,1);      /* DIG_GUR sequence                  */
}
if(ptr->unit==42)
{
    Reply(drv_id,&rep_msg,1);      /* DIG_IRS sequence                  */
}
/*--------------------------------------------------------------------------*/
/* send signal to processes waiting for message that a buffer is ready      */
/*--------------------------------------------------------------------------*/
ptr->buf_rdy=ptr->index;

dpl_console_id=qnx_name_locate(0,"dplc",0,&cop);
if(dpl_console_id != -1)
    Trigger(ptr->dplc_proxy);

dpl_serial_id=qnx_name_locate(0,"dpls",0,&cop);
if(dpl_serial_id != -1)
    Trigger(ptr->dpls_proxy);

rtp_id=qnx_name_locate(0,"rtp",0,&cop);
if(rtp_id != -1)
    Send(rtp_id,&snd_msg,&rec_msg,1,1);

ptr->ClkFlg_05=(short)ptr->sec_in;            /* info for mon_trg           */
trg_id=qnx_name_locate(0,"mon_trg",0,&cop);
if(trg_id != -1)
    Trigger(ptr->rat_proxy);

mon_id=qnx_name_locate(0,"mon",0,&cop);
if(mon_id != -1)
    Trigger(ptr->mon_proxy);


ptr->ClkFlg_05=(short)ptr->sec_in;            /* info for tim_chk           */
tim_id=qnx_name_locate(0,"tim_chk",0,&cop);
if(tim_id != -1)
    Trigger(ptr->tid_proxy);

    no_of_times++;                     /* total number of buffers received  */

/*         Extract the time-string of the k-th buffer                       */
    sprintf(tider,"%7d %s\n",ptr->sec_in,ptr->pipe+ptr->sec_in*((ptr->nchan*ptr->nsamp)+HEADER));
//printf("%s %x\n",tider,tider[36]);
    sync=tider[36];                    /* extract the sync/nosync indicator */
    if(sync=='S')                      /* check if synchronized or not      */
    {
        snc=1;                         /* clock synchronized                */
        no_of_syncs++;                 /* total number of synchronized times*/
    }else{
        snc=0;                         /* clock not synchronized            */
    }

/* meassure in percent reception of good GPS signal since last startup      */
//printf("No of syncs %d No of times %d\n",no_of_syncs,no_of_times);

    ptr->percent_gps=((float)no_of_syncs * 100.)/(float)no_of_times;


ptr->index=(ptr->index +1) %2;         /* Toggle bufferswitch (writing)     */
    }
}
}
bf_chain()
{
register int kp;
register int* bufptr;
register unsigned int* ch_ptr;
int      total;
unsigned int longv;
unsigned short shortv;
unsigned short swap;
int      sumtot;
int      sumdc;
int      kanal;
int      sample;
total=ptr->nsamp*ptr->nchan+HEADER;     
bufptr = ptr->ibuf[ptr->index];


//fprintf(mtv,"BC 1\n");
//fprintf(mtv,"%s\n",ptr->ibuf[ptr->index]);
ch_ptr=ptr->pipe+ptr->sec_in*((ptr->nchan*ptr->nsamp)+HEADER);

for(kp=0;kp<total;kp++)
    *ch_ptr++=*bufptr++;

//fprintf(mtv,"BC 2\n");
//fprintf(mtv,"%5d %s\n",ptr->sec_in,ptr->pipe+ptr->sec_in * ((ptr->nchan*ptr->nsamp)+HEADER));




//fseek(crc,ptr->crc_in*ptr->reclength,0);
ch_ptr=ptr->pipe+ptr->sec_in*((ptr->nchan*ptr->nsamp)+HEADER);
//fwrite(ch_ptr,ptr->reclength,1,crc);
ptr->crc_in++;
if(ptr->crc_in > ptr->sec_in_crc)
    ptr->crc_in=0;



for(kanal=0; kanal < ptr->nchan; kanal++)
{
    sumtot=0;
    sumdc=0;
    for(sample=0; sample < ptr->nsamp; sample++)
    {
       sumdc=sumdc+ptr->ibuf[ptr->index][kanal+sample*ptr->nchan+HEADER];
       sumtot=sumtot+abs(ptr->ibuf[ptr->index][kanal+sample*ptr->nchan+HEADER]-(int)ptr->dc[kanal]);
    }
    ptr->dc[kanal]=(ptr->dc[kanal]+(float)sumdc/ptr->nsamp)/2;  
    ptr->to[kanal]=(ptr->to[kanal]+(float)sumtot/ptr->nsamp)/2;
}
}
/*--------------------------------------------------------------------------*/
/* Function to update timestring for databuffers                            */
/*--------------------------------------------------------------------------*/

tagbuf(hrn,min,sec,yrn,mon,day,doy,idd)
long     hrn;
long     min;
float    sec;
long     yrn;
long     mon;
long     day;
long     doy;
int      idd;
{
register char *tmstr;                  /* pointer to time string in buffer  */
register int temp;
register unsigned int shift;
register unsigned int swap;
register unsigned short msw;
register unsigned short lsw;
register unsigned short sw;
int      k;
int      i;
int      tim_id;
char     tmpstr[80];
/*                  000000000011111111112222222222333333333344444444445555555*/
/*                  012345678901234567890123456789012345678901234567890123456*/
static   char ti[]="00:00:00.000 00/00/00 000 sts 00:00:00.000 00/00/00 000  ";
ti[55]=0x0d;
ti[56]=0x0a;
    
sprintf(tmpstr,"%2d:%2d:%6.3f %2d/%2d/%2d %3d",hrn,min,sec,day,mon,yrn,doy);

for(i=0;i<25;i++)                      /* copy current time into string     */
{
    ti[i]=tmpstr[i];
    if(i != 12 && i != 21)
    {
        if(ti[i]==' ')
            ti[i]='0';
    }
}
for(i=0;i<25;i++)                      /* Copy timestring to TIMCPY         */
    ptr->timcpy[i]=ti[i];              /* shared mem for other routines     */
    


for(i=0;i<25;i++)                      /* copy last update into string      */
    ti[i+30]=ptr->Timsav[i];
/*
switch(rec_msg[29])
{
    case '0':
    ti[26]='0';
    break;
    case '1':
    ti[26]='1';
    break;
    default:
    ti[26]='0';
    break;
}
*/
if(ptr->TimFlg[0]==-1)
{
    ti[26]=ptr->timcpy[26]='E';
    ti[27]=ptr->timcpy[27]='R';
    ti[28]=ptr->timcpy[28]='R';
}
if(ptr->TimFlg[0]==0)
{
    ti[26]=ptr->timcpy[26]='T';
    ti[27]=ptr->timcpy[27]='M';
    ti[28]=ptr->timcpy[28]='O';
}
if(ptr->TimFlg[0]==1)
{
    ti[26]=ptr->timcpy[26]='S';
    ti[27]=ptr->timcpy[27]='E';
    ti[28]=ptr->timcpy[28]='T';
}
if(ptr->TimFlg[0]==2)
{
    ti[26]=ptr->timcpy[26]='N';
    ti[27]=ptr->timcpy[27]='S';
    ti[28]=ptr->timcpy[28]='Y';
}
if(ptr->TimFlg[0]==3)
{
    ti[26]=ptr->timcpy[26]='C';
    ti[27]=ptr->timcpy[27]='P';
    ti[28]=ptr->timcpy[28]='U';
}
if(ptr->clk_chk==1)
    printf("%s",ti);
sprintf(ptr->timestamp,"%s",ti);
tim_id=qnx_name_locate(0,"tim",0,&cop);
if(tim_id != -1)
    Trigger(ptr->tim_proxy);

tmstr=ti;
k=0;
    for(temp=0;temp<56; temp=temp+4)        /* Copy timestring into the     */
    {                                       /* current shared buffer        */
        msw=*tmstr++;
        msw=msw<<8;
        msw=msw | *tmstr++;
        lsw=*tmstr++;
        lsw=lsw<<8; 
        lsw=lsw | *tmstr++;
        sw=msw;
        sw=sw<<8;
        msw=msw>>8;
        msw=msw | sw;
        sw=lsw;
        sw=sw<<8;
        lsw=lsw>>8;
        lsw=lsw | sw;        
        shift=lsw;
        shift=shift<<16;
        shift=shift | msw;
        ptr->ibuf[idd][k]=shift;
        k=k+1;
    }
}

TAGBUF(hrn,min,sec,yrn,mon,day,doy,idd)
int     hrn;
int     min;
float   sec;
int     yrn;
int     mon;
int     day;
int     doy;
int      idd;
{
register char *tmstr;                  /* pointer to time string in buffer  */
register int temp;
register unsigned int shift;
register unsigned int swap;
register unsigned short msw;
register unsigned short lsw;
register unsigned short sw;
int      k;
int      i;
int      tim_id;
char     tmpstr[80];
/* tut */
/*                00000000001111111111222222222233333333334444444444555555555*/
/*                01234567890123456789012345678901234567890123456789012345678*/
static char ti[]="00:00:00.000 00/00/0000 000 sts 00:00:00.000 00/00/0000 000";
ti[59]=0x0d;
ti[60]=0x0a;
    
sprintf(tmpstr,"%2d:%2d:%6.3f %2d/%2d/%4d %3d",hrn,min,sec,day,mon,yrn,doy);

for(i=0;i<27;i++)                      /* copy current time into string     */
{
    ti[i]=tmpstr[i];
    if(i != 12 && i != 23)
    {
        if(ti[i]==' ')
            ti[i]='0';
    }
}
for(i=0;i<27;i++)                      /* Copy timestring to TIMCPY         */
    ptr->timcpy[i]=ti[i];              /* shared mem for other routines     */

for(i=0;i<27;i++)                      /* copy last update into string      */
    ti[i+32]=ptr->Timsav[i];

//printf("TIMFLG=%d\n",ptr->TimFlg[0]);

if(ptr->TimFlg[0]==-1)
{
    ti[28]=ptr->timcpy[28]='E';
    ti[29]=ptr->timcpy[29]='R';
    ti[30]=ptr->timcpy[30]='R';
}
if(ptr->TimFlg[0]==0)
{
    ti[28]=ptr->timcpy[28]='T';
    ti[29]=ptr->timcpy[29]='M';
    ti[30]=ptr->timcpy[30]='O';
}
if(ptr->TimFlg[0]==1)
{
    ti[28]=ptr->timcpy[28]='S';
    ti[29]=ptr->timcpy[29]='E';
    ti[30]=ptr->timcpy[30]='T';
}
if(ptr->TimFlg[0]==2)
{
    ti[28]=ptr->timcpy[28]='N';
    ti[29]=ptr->timcpy[29]='S';
    ti[30]=ptr->timcpy[30]='Y';
}
if(ptr->TimFlg[0]==3)
{
    ti[28]=ptr->timcpy[28]='C';
    ti[29]=ptr->timcpy[29]='P';
    ti[30]=ptr->timcpy[30]='U';
}
//    printf("%s",ti);
//    printf("%s",ptr->timcpy);
sprintf(ptr->timestamp,"%s",ti);
tim_id=qnx_name_locate(0,"tim",0,&cop);
if(tim_id != -1)
    Trigger(ptr->tim_proxy);

tmstr=ti;
k=0;
    for(temp=0;temp<60; temp=temp+4)        /* Copy timestring into the     */
    {                                       /* current shared buffer        */
        msw=*tmstr++;
        msw=msw<<8;
        msw=msw | *tmstr++;
        lsw=*tmstr++;
        lsw=lsw<<8; 
        lsw=lsw | *tmstr++;
        sw=msw;
        sw=sw<<8;
        msw=msw>>8;
        msw=msw | sw;
        sw=lsw;
        sw=sw<<8;
        lsw=lsw>>8;
        lsw=lsw | sw;        
        shift=lsw;
        shift=shift<<16;
        shift=shift | msw;
        ptr->ibuf[idd][k]=shift;
        k=k+1;
    }
//fprintf(mtv,"%s",ti);
//fprintf(mtv,"BF \n");
//fprintf(mtv,"%s\n",ptr->ibuf[idd]);

}
cor_cpu(ptr)
smod *ptr;
{
int cor;
akk=akk+ptr->timerr;
//printf("%12.3f\n",akk);
if((fabs)(akk) >= 1.0)
{
    cor=(int)akk;
//printf("%5d ",ptr->itmde);
    ptr->itmde=ptr->itmde-cor;
//printf("%5d\n",ptr->itmde);
    akk=akk-cor;
}
}

