/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:22/12/94                       */
/* Program:C                                                                */
/*                                                                          */
/* Last update: october 96 by jh  add map and epi                           */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to compile all programs in application or single programs.       */
/* To compile all programs type 'c'                                         */
/* To compile single programs type 'c name'                                 */
/* If name is not part of the SEISLOG distribution you will have to do your */
/* own compile command based on the C manual.                               */
/*                                                                          */
/****************************************************************************/
#include <stdio.h>
#include "shar_mem.h"
#include "compile.h"
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                             Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program to compile all programs in SEISLOG (no arguments), or to
             compile program specified as argument.

Usage:       c (no arguments or c 'name of program)
#endif
struct entry
{
    char word[16];
    char definition[100];
};
/*--------------------------------------------------------------------------*/
/* Function to determine if two strings are equal                           */
/*--------------------------------------------------------------------------*/
int equal_strings(s1,s2)
char s1[],s2[];
{
    int i=0,answer;
    while(s1[i]==s2[i] && s1[i] != '\0' && s2[i] != '\0')
        ++i;
    if(s1[i]=='\0' && s2[i] == '\0')
        answer=1;                   /* strings equal                        */
    else
        answer=0;                   /* strings not equal                    */
    return(answer);
}
/*--------------------------------------------------------------------------*/
/* Function to look up in a table to check if the specified entry is present*/
/*--------------------------------------------------------------------------*/
int lookup(dictionary,search,number_of_entries)
struct  entry  dictionary[];
char           search[];
int            number_of_entries;
{
    int i;
    for(i=0; i < number_of_entries; ++i)
        if(equal_strings(search,dictionary[i].word))
            return(i);
return(-1);
}
main(argc,argv)
int argc;
char **argv;
{
/*--------------------------------------------------------------------------*/
/* Structure containing all valid arguments for program                     */
/*--------------------------------------------------------------------------*/
static struct entry dictionary[200]=
{ {"slog",          "SEISLOG library"                                  },
  {"clk_chk",       "check clock"                                      },
  {"melding",       "send mail file"                                   },
  {"trg",           "trigger event on simulated data"                  },
  {"buf_pip",       "time-stamp buffer and copy to pipe"               },
  {"cat_evt",       "save detected events on separate file"            },
  {"mkmail",        "make new mail file"                               },
  {"dig_met",       "driver DAS1200"                                   },
  {"auto",          "auto phase picking"                               },
  {"cls",           "clear screen"                                     },
  {"cpu",           "check cpu-active file"                            },
  {"src_dst",       "make upgrade flopy"                               },
  {"dig_syn",       "simulate n-channel Nanometrics"                   },
  {"rd_color",      "read color map"                                   },
  {"FREE",          "FREE"                                             },
  {"dly",           "force temporary delay in timestamp"               },
  {"dpl",           "plotting of selected channel in 'real time' "     },
  {"fpl",           "plot unfiltered and filtered of selected channel" },
  {"cat_pph",       "catlog p-phases"                                  },
  {"par_mrk",       "send minute pulse to par port"                    },
  {"fre",           "free space on hard disk"                          },
  {"help",          "print out general help file"                      },
  {"tape_sav",      "save ring buffers and events to tape"             },
  {"FREE",          "FREE"                                             },
  {"lgs",           "logg of detected events"                          },
  {"dig_wav",       "digitizer feeds real data"                        },
  {"mdc",           "monitor DC levels"                                },
  {"min_max",       "find min max in ringbuffersystem"                 },
  {"e_addr",        "edit mailing list, events"                        },
  {"ver",           "print version number"                             },
  {"cat_ext",       "make event of extracted data"                     },
  {"mon",           "monitor SEISLOG variables"                        },
  {"dig_ram",       "driver 12 channel Llolleo"                        },
  {"off",           "turn off manual trigger"                          },
  {"onn",           "turn on manual trigger"                           },
  {"par",           "modify parameters"                                },
  {"pic",           "plot catalogged events"                           },
  {"dig_m18",       "Metrabyte 1800"                                   },
  {"sei_act",       "record up-time for SEISLOG"                       },
  {"prl",           "list out all parameters"                          },
  {"pwr",           "list out file up-time SEISLOG"                    },
  {"co_msg",        "send msg INPP"                                    },
  {"min_mrk",       "send minute mark to parallell port"               },
  {"rd_param",      "read SEISLOG parameters"                          },
  {"det_evt",       "read pipe and do detection"                       },
  {"dig_nan",       "read data from Nanometrics"                       },
  {"re_initi",      "reinitialize dpx number"                          },
  {"plt",           "plot recorded events"                             },
  {"re_start",      "restart/reboot system after 24 hours"             },
  {"clk_ms1",       "read time from external msf clock"                },
  {"rtp",           "plot several channels in 'real time' "            },
  {"bck_stru",      "make bakup and directory structure"               },
  {"par_gps",       "send GPS ok to par port"                          },
  {"err_log",       "dump errorlog "                                   },
  {"srt",           "start SEISLOG"                                    },
  {"sys_wat",       "monitor SEISLOG"                                  },
  {"stp",           "stop SEISLOG"                                     },
  {"sts",           "print out current values (debugging)"             },
  {"cpu_acti",      "record file up time cpu"                          },
  {"dig_ge1",       "Terra IDS digitizer"                              },
  {"logg",          "check logg files"                                 },
  {"res_par",       "read resampling parameters"                       },
  {"smp_koef",      "filter koef alias"                                },
  {"lnk_dos",       "make dos filenames"                               },
  {"mem",           "memory on system"                                 },
  {"FREE",          "FREE"                                             },
  {"ext",           "extract data from ring buffer"                    },
  {"ascc"  ,        "convert to ascii"                                 },
  {"version",       "update version number"                            },
  {"destino",       "sound alarm on event"                             },
  {"dig_n24",       "driver HRD24"                                     },
  {"rng_ver",       "verify ringbuffer file system"                    },
  {"dig_gbv",       "lowcost recorder"                                 },
  {"gen_koef",      "generate filter coefficients"                     },
  {"tim_chk",       "monitor timing"                                   },
  {"tim_avg",       "average ms + gps"                                 },
  {"mds",           "autodelete of events"                             },
  {"rst_gps",       "Resets GPS clock"                                 },
  {"mon_trg",       "monitor sta/lta ratio"                            },
  {"clk_pps",       "maintain milliseconds, pps optional"              },
  {"dig_hhh",       "reader HRD24"                                     },
  {"mon_pip",       "monitor timestrings in memory pipe"               },
  {"auto_asc",      "convert to ascii all files"                       },
  {"sys_boot",      "force system to reboot"                           },
  {"hex",           "conversion utility hex-dec-bin"                   },
  {"act_log",       "write log every minute"                           },
  {"fmt_all",       "reformat events to av format"                     },
  {"tim_snt",       "monitor average timestamps"                       },
  {"dec_mat",       "resampling"                                       },
  {"resamp",        "plot resampled data"                              },
  {"pres",          "plot resampled data"                              },
  {"sng_res",       "plot resampled data"                              },
  {"lgs_res",       "overview dayfiles"                                },
  {"qnx_dist",      "make backup of SEISLOG on PC"                     },
  {"dig_eda",       "read ED digitizer"                                },
  {"dsk_all",       "allocate ringbuffers"                             },
  {"par_smp",       "edit parameters resampling"                       },
  {"map",           "plot epicenters     "                             },
  {"epi",           "epicenter location          "                     },
  {"copy_dos",      "copy events to dos partition"                     },
  {"chk_pps",       "check if PPS present"                             },
  {"clk_onn",       "turn on printout timestring"                      },
  {"clk_off",       "turn off printout timestring"                     },
  {"req_tid",       "request time from internal clock"                 },
  {"gar_min",       "check GARMIN clocks"                              },
  {"dig_me2",       "driver 2 1202 boards "                            },
  {"clk_cpu",       "driver CPU clock"                                 },
  {"rng_sts",       "status ringbuffer system"                         },
  {"dig_324",       "driver 3 channel 24 bits R.O.+C.A."               },
  {"dig_ser",       "reader multiserial board"                         },
  {"dbg_onn",       "turn on print flag"                               },
  {"dbg_off",       "turn off print flag"                              },
  {"rd_par_ser",    "read parameters multiserial board"                },
  {"sei_alm",       "alarm when a program stops"                       },
  {"clk_gnp",       "driver Garmin GPS No PPS"                         },
  {"set_pri",       "set priority of process",                         },
  {"clk_gpp",       "driver Garmin GPS 1 PPS"                          },
  {"mul_ser",       "read serial lines"                                },
  {"cnv_map",       "convert map files xy-yx"                          },
  {"plo",           "plot events with picks"                           },
  {"plt_chk",       "plot every event"                                 },
  {"sts_msg",       "status msg seislog"                               },
  {"cmd",           "transmit CO msg"                                  },
  {"par_ser",       "parameters INPP"                                  },
  {"cmd_gen",       "generate msg for radio"                           },
  {"chk_tag",       "check time ringbuffer system"                     },
  {"seislog",       "info seislog commands and programs"               },
  {"ili_seis",      "24 bit ILI driver"                                },
  {"autoasc",       "convert to ASCII BGS"                             },
  {"ts",            "reads PDETNR and DETNR BGS"                       },
  {"sav_ver",       "backups and distribution sets"                    },
  {"dig_low",       "driver lowcost recorder"                          },
  {"help",          "lists all modules with help files"                },
  {"FREE",          "FREE"                                             },
  {"FREE",          "FREE"                                             },
  {"src_dst",       "makes source upgrade floppy"                      },
  {"chk_par",       "check parameters"                                 },
  {"chk_snc",       "check for sync characters"                        },
  {"pk",            "pick phases"                                      },
  {"clear",         "clear screen (for serial access)"                 },
  {"dsk_inf",       "check for space left"                             },
  {"active",        "check if SEISLOG is active"                       },
  {"map2",          " draw map with azimuth"                           },
  {"map3",          "draw map with epicenter"                          },
  {"dig_ge2",       "Terra IDS digitizer 1-second buffers"             },
  {"dig_evr",       "driver Kinemetrics Altus/Everest"                 },
  {"prt_onn",       "turn on output option"                            },
  {"prt_off",       "turn off output option"                           },
  {"mul_ser2",      "multiserial IDS-24 1 second buffers"              },
  {"dig_ser2",      "multiserial IDS-24 1 second buffers"              },
  {"cln_lnk",       "clean links to ringbuffersystem"                  },
  {"min_pls",       "minute pulse on parallell port"                   },
  {"dig_edn",       "earth data digitizer (new)"                       },
  {"dig_gur",       "Guralp digitizer"                                 },
  {"dig_g24",       "Guralp driver"                                    },
  {"fil_dpx",       "make link dpx filename"                           },
  {"dig_ror",       "driver R.O. 1 comp. 50Hz 16 bits"                 },
  {"mg2",           "monitor Guralp pipe"                              },
  {"gur_rst",       "check if active Guralp driver"                    },
  {"pro_all",       "process data in work"                             },
  {"map3ps",        "map all locations ps file"                        },
  {"slog2sac",      "SEISLOG-SAC"                                      },
  {"dig_irs",       "IDS-24 IRIS PASCAL"                               },
  {"dig_i24",       "IDS-24 IRIS PASCAL"                               },
  {"e_mail_adr",    "edit file for addresse to send diagnostics"       },
  {"message",       "send e-mail with diagnostics"                     },
  {"watchdog",      "watch all processes, and send diagnostic message" },
  {"pltbgs",        "plt version BGS"                                  },
  {"dig_edm",       "multiserial ED"                                   },
  {"mul_eda",       "multiserial ED driver"                            },
  {"avslutt",       "melding1"                                         },
  {"begynn",        "melding2"                                         },
  {"receiving",     "melding3"                                         },
  {"vel",           "setup ED2400"                                     },
  {"chk_edn",       "check ED2400"                                     },
  {"web_koef",      "filter koeficients"                               },
  {"plt_o",         "old plot program"                                 },
  {"bck_web",       "backup web"                                       },
  {"sei_acw",       "same as sei_act but for web"                      },
  {"edn_hdr",       "read header & set new samplerate"                 }


};
int entries=180;     /* ADD 1 HERE EACH TIME YOU ADD A NEW PROGRAM           */
int entry_number;
int k;
FILE *in;
/*in=fopen("/r0/BUFFER","d");*/           /* check if BUFFER is allocated       */
/*if(in != 0)*/
/*{*/
/*    fclose(in);*/
/*    system("deldir -q /r0/BUFFER");*/   /* if YES, delete whole stucture      */
/*}*/
if(argc !=2)
{
    printf("No argument typed on the command line, COMPILE ALL\n");

    for(k=0;k<entries;k++)
        compile(k);

    exit(1);
}
entry_number=lookup(dictionary,argv[1],entries);
if(entry_number != -1)
{
    printf("%s\n", dictionary[entry_number].definition);
    compile(entry_number);
}
else{
    printf("Sorry %s is not in my dictionary.\n",argv[1]);

}
}
compile(number)
int number;
{
switch(number)
{
    case 0:
        system("cc -c slog.c");
        system("cc -A slog.lib slog.o");
    break;
    case 1:
        system("cc clk_chk.c -l slog -o clk_chk");
        system("chmod a+rwx clk_chk");
        break;
    case 2:
        system("cc melding.c -l slog -o melding");
        break;
    case 3:
        system("cc trg.c -l slog -o trg");
        break;
    case 4:
        system("cc buf_pip.c -l slog -o buf_pip");
        system("usemsg buf_pip buf_pip.c");
    break;
    case 5:
        system("cc cat_evt.c -l slog -o cat_evt");
        system("usemsg cat_evt cat_evt.c");
        break;
    case 6:
        system("cc mkmail.c -l slog -o mkmail");
        break;
    case 7:
        system("cc dig_met.c -T1 -o dig_met");
        break;
    case 8:
        system("cc auto.c -l slog -o auto -N 2000000");
        system("usemsg auto auto.c");
        break;
    case 9:
		if (__WATCOMC__ > 1000)
    	{
/* the link to termlib in next line needed for link under version 10.6      */
        	system("cc cls.c -l slog -l termlib -o cls");
    	}else{
        	system("cc cls.c -l slog -o cls");
    	}
        system("usemsg cls cls.c");
        break;
    case 10:
        system("cc cpu.c -l slog -o cpu");
        break;
    case 11:
        system("cc src_dst.c -l slog -o src_dst");
        system("usemsg src_dst src_dst.c");
        break;
    case 12:
        system("cc dig_syn.c -l slog -o dig_syn");
        break;
    case 13:
        system("cc rd_color.c -l slog -o rd_color");
        break;
    case 14:
        printf("Not supported\n");
//        system("cc eev.c -l slog -o eev -N 2000000");
        break;
    case 15:
        system("cc dly.c -l slog -o dly");
        system("usemsg dly dly.c");
        break;
    case 16:
        system("cc dpl.c -l slog -D_CGRAPH=_loadds -T1 -o dpl");
        system("usemsg dpl dpl.c");
        break;
    case 17:
        system("cc fpl.c -l slog -D_CGRAPH=_loadds -T1 -o fpl");
        break;
    case 18:
        system("cc cat_pph.c -l slog -o cat_pph");
        break;
    case 19:
    	system("cc par_mrk.c -l slog -T1 -o par_mrk");
        break;
    case 20:
        system("cc fre.c -l slog -o fre");
        break;
    case 21:
        system("cc help.c -l slog -o help");
        break;
    case 22:
        system("cc tape_sav.c -l slog -o tape_sav");
        break;
    case 23:
        printf("Not supported\n");
//        system("cc jepi.c -l slog -o jepi -N 2000000");
        break;
    case 24:
        system("cc lgs.c -l slog -o lgs");
        break;
    case 25:
        system("cc dig_wav.c -l slog -o dig_wav");
        break;
    case 26:
		if (COPT == "NCURSES" && __WATCOMC__ > 1000)
    	{
        	system("cc mdc_c.c -l ncurses -o mdc");
    	}else{
        	system("cc mdc.c -l slog -l termlib -o mdc");
    	}
        break;
    case 27:
        system("cc min_max.c -l slog -o min_max");
        break;
    case 28:
        system("cc e_addr.c -l slog  -o e_addr");
        break;
    case 29:
        system("cc ver.c -l slog -o ver");
        break;
    case 30:
        system("cc cat_ext.c -l slog -o cat_ext");
        system("usemsg cat_ext cat_ext.c");
        break;
    case 31:
		if (COPT == "NCURSES" && __WATCOMC__ > 1000)
    	{
        	system("cc mon_c.c -l ncurses -l slog -o mon ");
    	}else{
        	system("cc mon.c -l slog -l termlib -o mon ");
    	}
        break;
    case 32:
        system("cc dig_ram.c -l slog -o dig_ram");
        break;
    case 33:
        system("cc off.c -o off");
        system("usemsg off off.c");
        break;
    case 34:
        system("cc onn.c -o onn");
        system("usemsg onn onn.c");
        break;
    case 35:
		if (__WATCOMC__ > 1000)
    	{
/* the link to termlib in next line needed for link under version 10.6      */
        	system("cc par.c -l slog -l termlib -o par");
    	}else{
        	system("cc par.c -l slog -o par");
    	}
        break;
    case 36:
    printf("Not supported\n");     
//    system("cc pic.c -l slog -D_CGRAPH=_loadds -T1 -o pic");
        break;
    case 37:
        system("cc dig_m18.c -l slog -T1 -o dig_m18");
        break;
    case 38:
        system("cc sei_act.c -l slog -o sei_act");
        break;
    case 39:
        system("cc prl.c -l slog -o prl");
        break;
    case 40:
        system("cc pwr.c -l slog -o pwr");
        break;
    case 41:
//        system("cc co_msg.c -l slog -T1 -o co_msg");
        break;
    case 42:
        system("cc min_mrk.c -l slog -T1 -o min_mrk");
    	break;
    case 43:
        system("cc rd_param.c -l slog -o rd_param");
        break;
    case 44:
        system("cc det_evt.c -l slog -o det_evt");
        break;
    case 45:
        system("cc dig_nan.c -l slog -o dig_nan");
        system("usemsg dig_nan dig_nan.c");
        break;
    case 46:
        system("cc re_initi.c  -l slog -o re_initi");
        break;
    case 47:
		if (__WATCOMC__ > 1000)
    	{
/* the link to termlib in next line needed for link under version 10.6      */
        	system("cc plt.c -l slog -l termlib -D_CGRAPH=_loadds -T1 -o plt");
    	}else{
        	system("cc plt.c -l slog -D_CGRAPH=_loadds -T1 -o plt");
    	}
        break;
    case 48:
//        system("cc re_start.c -l slog -o re_start");
        break;
    case 49:
        system("cc clk_ms1.c -l slog -T1 -o clk_ms1");
    break;
    case 50:
        system("cc rtp.c -l slog -D_CGRAPH=_loadds -T1 -o rtp");
        break;
    case 51:
        system("cc bck_stru.c -l slog -o bck_stru");
        system("usemsg bck_stru bck_stru.c");
        break;
    case 52:
        system("cc par_gps.c -l slog -T1 -o par_gps");
    	break;
    case 53:
        system("cc err_log.c -l slog -o err_log");
        break;
    case 54:
        system("cc srt.c -l slog -o srt");
        break;
    case 55:
        system("cc sys_wat.c -l slog -T1 -o sys_wat");
        break;
    case 56:
        system("cc stp.c -l slog -o stp");
        break;
    case 57:
        system("cc sts.c -l slog -o sts");
        break;
    case 58:
        system("cc cpu_acti.c -l slog -o cpu_acti");
        break;
    case 59:
        system("cc dig_ge1.c -l slog -T1 -o dig_ge1");
        break;
    case 60:
        system("cc logg.c -l slog -o logg");
        system("usemsg logg logg.c");
        break;
    case 61:
        system("cc res_par.c -l slog -o res_par");
        break;
    case 62:
        system("cc smp_koef.c -l slog -o smp_koef");
    	break;
    case 63:
        system("cc lnk_dos.c -l slog -o lnk_dos");
        break;
    case 64:
        system("cc mem.c -l slog -o mem");
        break;
    case 65:
        printf("Not supported\n");
//        system("cc ccollect.c -l slog -N 2000000 -o ccollect");
        break;
    case 66:
        system("cc ext.c -l slog -o ext");
        system("usemsg ext ext.c");
        break;
    case 67:
        system("cc ascc.c -l slog -o ascc");
        system("usemsg ascc ascc.c");
        break;
    case 68:
        system("cc version.c -l slog -o version");
        break;
    case 69:
        system("cc destino.c -T1 -o destino");
        break;
    case 70:
        system("cc dig_n24.c -l slog -o dig_n24");
        break;
    case 71:
        system("cc rng_ver.c -l slog -o rng_ver");
        break;
    case 72:
        system("cc dig_gbv.c -l slog -o dig_gbv");
        break;
    case 73:
        system("cc gen_koef.c -l slog -o gen_koef");
        break;
    case 74:
        system("cc tim_chk.c -l slog -D_CGRAPH=_loadds -T1 -o tim_chk");
        break;
    case 75:
        system("cc tim_avg.c -l slog -o tim_avg");
        break;
    case 76:
        system("cc mds.c -l slog -o mds");
        break;
    case 77:
        system("cc rst_gps.c -T1 -o rst_gps");
        break;
    case 78:
        system("cc mon_trg.c -l slog -D_CGRAPH=_loadds -T1 -o mon_trg");
        break;
    case 79:
        system("cc clk_pps.c -l slog -T1 -o clk_pps");
        break;
    case 80:
        system("cc dig_hhh.c -l slog -o dig_hhh");
        system("usemsg dig_hhh dig_hhh.c");
        break;
    case 81:
        system("cc mon_pip.c -l slog -o mon_pip");
        break;
    case 82:
        system("cc auto_asc.c -l slog -o auto_asc");
        break;
    case 83:
        system("cc sys_boot.c -l slog -o sys_boot");
        break;
    case 84:
        system("cc hex.c -l slog -o hex");
        break;
    case 85:
        system("cc act_log.c -l slog -o act_log");
        system("usemsg act_log act_log.c");
        break;
    case 86:
        system("cc fmt_all.c -l slog -o fmt_all");
        break;
    case 87:
        system("cc tim_snt.c -l slog -D_CGRAPH=_loadds -T1 -o tim_snt");
        break;
    case 88:
        system("cc dec_mat.c -l slog -o dec_mat");
        system("usemsg dec_mat dec_mat.c");
        break;
    case 89:
        system("cc resamp.c -l slog -D_CGRAPH=_loadds -T1 -o resamp");
        break;
    case 90:
        system("cc pres.c -l slog -D_CGRAPH=_loadds -T1 -o pres");
        break;
    case 91:
        system("cc sng_res.c -l slog -D_CGRAPH=_loadds -T1 -o sng_res");
        break;
    case 92:
        system("cc lgs_res.c -l slog -o lgs_res");
        break;
    case 93:
          system("cc qnx_dist.c -l slog -o qnx_dist");
        break;
    case 94:
        system("cc dig_eda.c -l slog -o dig_eda");
        break;
    case 95:
        system("cc dsk_all.c -l slog -o dsk_all");
        break;
    case 96:
        system("cc par_smp.c -l slog -o par_smp");
        system("usemsg par_smp par_smp.c");
        break;
    case 97:
        system("cc map.c -l slog -D_CGRAPH=_loadds -T1 -o map");
        break;
    case 98:
        system("cc epi.c -l slog -o epi");
        break;
    case 99:
        system("cc copy_dos.c -l slog -o copy_dos");
        break;
    case 100:
        system("cc chk_pps.c -T1 -o chk_pps");
        system("usemsg chk_pps chk_pps.c");
        break;
    case 101:
        system("cc clk_onn.c -l slog -o clk_onn");
        break;
    case 102:
        system("cc clk_off.c -l slog -o clk_off");
        break;
    case 103:
        system("cc req_tid.c -l slog -T1 -o req_tid");
        break;
    case 104:
        system("cc gar_min.c -l slog -o gar_min");
        break;
    case 105:
        system("cc dig_me2.c -l slog -T1 -o dig_me2");
        system("usemsg dig_me2 dig_me2.c");
        break;
    case 106:
        system("cc clk_cpu.c -l slog -T1 -o clk_cpu");
        system("usemsg clk_cpu clk_cpu.c");
        break;
    case 107:
        system("cc rng_sts.c -l slog -o rng_sts");
        break;
    case 108:
        system("cc dig_324.c -l slog -o dig_324");
        break;
    case 109:
        system("cc dig_ser.c -l slog -o dig_ser");
        break;
    case 110:
        system("cc dbg_onn.c -l slog -o dbg_onn");
        break;
    case 111:
        system("cc dbg_off.c -l slog -o dbg_off");
        break;
    case 112:
        system("cc rd_par_ser.c -l slog -o rd_par_ser");
        break;
    case 113:
        system("cc sei_alm.c -l slog -T1 -o sei_alm");
        break;
    case 114:
        system("cc clk_gnp.c -l slog -T1 -o clk_gnp");
        system("usemsg clk_gnp clk_gnp.c");
        break;
    case 115:
        system("cc set_pri.c -l slog -o set_pri");
        break;
    case 116:
        system("cc clk_gpp.c -l slog -T1 -o clk_gpp");
        system("usemsg clk_gpp clk_gpp.c");
        break;
    case 117:
        system("cc mul_ser.c -l slog -o mul_ser");
        break;
    case 118:
        system("cc cnv_map.c -l slog -o cnv_map");
        break;
    case 119:
		if (__WATCOMC__ > 1000)
    	{
/* the link to termlib in next line needed for link under version 10.6      */
        	system("cc plo.c -l slog -l termlib -D_CGRAPH=_loadds -T1 -o plo");
    	}else{
        	system("cc plo.c -l slog -D_CGRAPH=_loadds -T1 -o plo");
    	}
        break;
    case 120:
		if (__WATCOMC__ > 1000)
    	{
/* the link to termlib in next line needed for link under version 10.6      */
        	system("cc plt_chk.c -l slog -l termlib -D_CGRAPH=_loadds -T1 -o plt_chk -N 2000000");
    	}else{
        	system("cc plt_chk.c -l slog -D_CGRAPH=_loadds -T1 -o plt_chk -N 2000000");
    	}
        break;
    case 121:
        system("cc sts_msg.c -l slog -o sts_msg");
        break;
    case 122:
        system("cc cmd.c -l slog -o cmd");
        break;
    case 123:
        system("cc par_ser.c -l slog -o par_ser");
        break;
    case 124:
        system("cc cmd_gen.c -l slog -o cmd_gen");
        break;
    case 125:
        system("cc chk_tag.c -l slog -o chk_tag");
        break;
    case 126:
        printf("Program not supported this version.\n");
        break;
    case 127:                    /* 24 bit ILI option added 16/02/98 DJS */
        system("cc -c -g ili_seis.c");
        system("cc -Zu -Wc -s -c int.c");
        system("cc -c -g date_change.c");
        system("cc -g -T1 -o ili_seis ili_seis.o int.o date_change.o");
        break;
    case 128:
        system("cc autoasc.c -l slog -o autoasc");
        break;
    case 129:
        system("cc ts.c -l slog -o ts");
        break;
    case 130:
        system("cc sav_ver.c -l slog -o sav_ver");
        break;
    case 131:
        system("cc dig_low.c -l slog -o dig_low");
        break;
    case 132:
        system("cc help.c -l slog -o help");
        break;
    case 133:
        printf("Not supported\n");
//        system("cc bck_slg.c -l slog -o bck_slg");
//        system("usemsg bck_slg bck_slg.c");
        break;
    case 134:
        printf("Not supported\n");
//        system("cc bck_src.c -l slog -o bck_src");
//        system("usemsg bck_src bck_src.c");
        break;
    case 135:
        system("cc src_dst.c -l slog -o src_dst");
        system("usemsg bck_src src_dst.c");
        break;
    case 136:
        system("cc chk_par.c -o chk_par");
        system("usemsg chk_par chk_par.c");
        break;
    case 137:
        system("cc chk_snc.c -o chk_snc");
        system("usemsg chk_snc chk_snc.c");
        break;
    case 138:
        if (__WATCOMC__ > 1000)
        {
/* the link to termlib in next line needed for link under version 10.6      */
            system("cc pk.c -l slog -l termlib -D_CGRAPH=_loadss -T1 -o pk -N 2000000");
        }else{
            system("cc pk.c -l slog -D_GRAPH=_loadss -T1 -o pk -N 2000000");
        }
        break;
    case 139:
        system("cc clear.c -l slog -o clear");
        break;
     case 140:
        system("cc dsk_inf.c -l slog -o dsk_inf");
        break;
    case 141:
        system("cc active.c -l slog -o active");
        break;
    case 142:
		if (__WATCOMC__ > 1000)
    	{
/* the link to termlib in next line needed for link under version 10.6      */
        	system("cc map2.c -l slog -l termlib -D_CGRAPH=_loadds -T1 -o map2 -N 2000000");
    	}else{
        	system("cc map2.c -l slog -D_CGRAPH=_loadds -T1 -o map2 -N 2000000");
    	}
        break;
    case 143:
		if (__WATCOMC__ > 1000)
    	{
/* the link to termlib in next line needed for link under version 10.6      */
        	system("cc map3.c -l slog -l termlib -D_CGRAPH=_loadds -T1 -o map3 -N 2000000");
    	}else{
        	system("cc map3.c -l slog -D_CGRAPH=_loadds -T1 -o map3 -N 2000000");
    	}
        break;
     case 144:
        system("cc dig_ge2.c -l slog -o dig_ge2");
        break;
     case 145:
        system("cc dig_evr.c -l slog -o dig_evr");
        break;
    case 146:
        system("cc prt_onn.c -o prt_onn");
        system("usemsg prt_onn prt_onn.c");
        break;
    case 147:
        system("cc prt_off.c -o prt_off");
        system("usemsg prt_off prt_off.c");
        break;
    case 148:
        system("cc mul_ser2.c -l slog -o mul_ser2");
        break;
    case 149:
        system("cc dig_ser2.c -l slog -o dig_ser2");
        break;
    case 150:
        system("cc cln_lnk.c -l slog -o cln_lnk");
        break;
    case 151:
        system("cc min_pls.c -l slog -T1 -o min_pls");
        break;
    case 152:
        system("cc dig_edn.c -l slog -o dig_edn");
        break;
    case 153:
        system("cc dig_gur.c -l slog -o dig_gur");
        break;
    case 154:
        system("cc dig_g24.c -l slog -o dig_g24");
        break;
    case 155:
            system("cc fil_dpx.c -l slog -o fil_dpx");
        break;
    case 156:
            system("cc dig_ror.c -l slog -o dig_ror");
        break;
    case 157:
		if (__WATCOMC__ > 1000)
        {
            system("cc mg2.c -l slog -l termlib -o mg2");
        }else{
        	system("cc mg2.c -l slog -o mg2 ");
        }
        break;
    case 158:
        	system("cc gur_rst.c -l slog -o gur_rst ");
        break;
    case 159:
        	system("cc pro_all.c -l slog -o pro_all ");
        break;
    case 160:
        	system("cc map3ps.c -l slog -D_CGRAPH=_loadds -T1 -o map3ps");
        break;
    case 161:
        	system("cc slog2sac.c -l slog -o slog2sac");
        break;
    case 162:
        	system("cc dig_irs.c -l slog -o dig_irs");
        break;
    case 163:
        	system("cc dig_i24.c -l slog -o dig_i24");
        break;
    case 164:
        	system("cc e_mail_adr.c -l slog -o e_mail_adr");
        break;
    case 165:
        	system("cc message.c -l slog -o message");
        break;
    case 166:
        	system("cc watchdog.c -l slog -o watchdog");
        break;
    case 167:
    printf("Not supported\n");
        break;
    case 168:
            system("cc dig_edm.c -l slog -o dig_edm");
    break;
    case 169:
            system("cc mul_eda.c -l slog -o mul_eda");
    break;
    case 170:
            system("cc avslutt.c -o avslutt");
    break;
    case 171:
            system("cc begynn.c -o begynn");
    break;
    case 172:
            system("cc receiving.c -o receiving");
    break;
    case 173:
            system("cc vel.c -l slog -o vel");
    break;
    case 174:
            system("cc chk_edn.c -l slog -o chk_edn");
    break;
    case 175:
            system("cc web_koef.c -o web_koef");
    break;
    case 176:
		if (__WATCOMC__ > 1000)
    	{
/* the link to termlib in next line needed for link under version 10.6      */
        	system("cc plt_o.c -l slog -l termlib -D_CGRAPH=_loadds -T1 -o plt_o");
    	}else{
        	system("cc plt_o.c -l slog -D_CGRAPH=_loadds -T1 -o plt_o");
    	}
        break;
    case 177:
            system("cc bck_web.c -l slog -o bck_web");
    break;
    case 178:
            system("cc sei_acw.c -l slog -o sei_acw");
    break;
    case 179:
            system("cc edn_hdr.c -l slog -o edn_hdr");
    break;

}
}


