/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:26/09/97                       */
/* Program:CAT_EVT                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 16/09/97 included help file                                              */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program saves data onto separate files when triggered by the        */
/* detection algorithm.                                                     */
/* Depending on the SEISLOG parameters, CAT_EVT updates the detection-      */
/* record-file /home/buffer/event and saves the actual seismic event as a   */
/* separate file in directory /home/events with a filename based on the     */
/* time of detection.                                                       */
/* The seismic data are stored continously on several circular disk-files   */
/* that can hold data for several minutes (hours) depending on the defined  */
/* sizes set in the parameter file.                                         */
/* Several seismic events may be queued up by the detection algorithm if the*/
/* seismic activity is high. CAT_EVT will pick out events from this queue   */
/* (circular disk-files), and process the events one by one until there are */
/* no more events in the queue. CAT_EVT will then enter a 'wait' state      */
/* until new events are detected.                                           */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "filedef.h"                   /* Include file for application      */
#include "timexrou.h"                  /* Structure def. of date and time   */
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: This program saves data onto separate files when triggered by the
             detection algorithm. Depending on the SEISLOG parameters,
             CAT_EVT updates the detection-record-file /home/buffer/event and
             saves the actual seismic event as a separate file in directory 
             /home/events with a filename based on the time of detection.
             The seismic data are stored continously on several circular 
             disk-files that can hold data for several minutes (hours),
             depending on the defined sizes set in the parameter file. Several
             seismic events may be queued up by the detection algorithm if the
             seismic activity is high. CAT_EVT will pick out events from this
             queue (circular disk-files), and process the events one by one 
             until there are no more events in the queue. CAT_EVT will then 
             enter a 'wait' state until new events are detected.    

Usage:       started by srt command (no parameters)
#endif


int      id;
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

int      datal[BUFFER];
short    datas[BUFFER];
char     syscall[200];
main()
{
static int  kanal[50000];              // SEISANFORMAT array for demultiplexed data
int      SEISAN;                       // SEISANFORMAT Switch to write
                                       // SEISANFORMAT SEISAN-format.
                                       // SEISANFORMAT SEISAN=0, QNX-format
                                       // SEISANFORMAT SEISAN=1, SEISAN-format
char     SDIR[85];                     // SEISANFORMAT Directory S-files
char     WDIR[85];                     // SEISANFORMAT Directory W-files
char     detection[85];                // SEISANFORMAT Detection time
char     snam[85];                     // SEISANFORMAT S-file name
float    duration;                     // SEISANFORMAT number seconds event
int      tophdr;
int      type_of_evt;
int      cnt_frq;                      /* seconds passed                    */
int      cnt_ant;                      /* number of time tags               */
int      rett;
int      yrn;                          /* year                              */
int      mon;                          /* month                             */
int      day;                          /* day                               */
int      hrn;                          /* hour                              */
int      min;                          /* minute                            */
float    sec;                          /* second                            */
double   sectot1;                      /* number of seconds for a date      */
int      i;                            /* counter                           */
int      konst;                        /* constant                          */
int      par;                          /* parameter set                     */
int      dpx;                          /*J DPX number for this event        */
int      inblk;                        /*J Block number for fseek to find   */
                                       /*  start of event                   */ 
int      pipblk;                       /* Block number in pipe to find      */
                                       /*  start of event                   */ 
int      start_blk;                    /*  Block number for first block in  */
                                       /*  event                            */
float    dur;                          /*J Duration of event in seconds     */
int      no_of_buf;                    /*J Total number of buffers for event*/
char     fnam[EVNAMLEN+10];            /*  Event filename                   */
char     evnam[45];                    /* Event name incl. directory path   */
int      point;                        /* Place of event in queue           */
int      fidx;                         /* index of the ring filename        */
char     ringname[DIRNAMLEN+EVNAMLEN]; /* name of ringbuffer to read        */
int      ret_val;                      /* return value from functions       */
static   char post[120];
static   char head[SIZ_EV_HDR+1];      /*J Space for det. header (record)   */
static   char output[EVTREC+1];        /*J Space for SEISNOR standard header*/
static   int input[BUFFER];            /*J Space to hold one data buffer    */
struct   date  event_time;             /*J Time of the event being processed*/
struct   date  rec_time;               /*J Time the event file is to start  */
struct   date  start_time;             /*  Start time of recording          */
struct   date  begin_time;             /*  Stop time of recording           */


SEISAN=0;                              // SEISANFORMAT 1-SEISAN,0-QNX

/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"CAT_EVT_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CAT_EVT_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
tophdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
konst=ptr->nchan * ptr->nsamp + HEADER;
/*--------------------------------------------------------------------------*/
/*  initialise all necessary shared variables                               */
/*--------------------------------------------------------------------------*/

ptr->ntick = 0;                        /* no. of buffers to catlog          */
point = 0;                             /* start with the first in the queue */

cnt_frq=0;                             /* seconds passed                    */
cnt_ant=0;                             /* number of time tags               */
system("/home/seislog/act_log 0");
cnt_ant++;
/*--------------------------------------------------------------------------*/
/* CHECK 'QUEUE' AND ENTER WAIT-STATE IF 'QUEUE' IS EMPTY (-1)              */
/* PROGRAM WILL BE STARTED BY 'DET' WHEN A NEW EVENT IS DETECTED            */
/* IF THERE ARE EVENT IN THE QUEUE, GOON AND PROCESS IT !!                  */
/*--------------------------------------------------------------------------*/
wait:
if(ptr->queue < 0 )                    /* Check if there is any events      */
{                                      /* queued up >=0 ?                   */
    ptr->alive[4]=1;                   /* tell I am alive                   */
    sleep(1);                          /* if queue empty goto sleep         */
    cnt_frq++;
    if(cnt_frq == 60)
    {
        cnt_frq=0;    
        system("/home/seislog/act_log 1");
        cnt_ant++;
    }
    if(ptr->man_que == 1)
    {
//        printf(" TRIGGER !\n");
        ptr->man_que = 0;
        goto manual;
    }
    goto wait;                         /* check queue again                 */
}
type_of_evt=0;
goto record;
manual:
type_of_evt=1;
record:
/****************************************************************************/
/* Event in queue, goon and process it                                      */
/****************************************************************************/

/*--------------------------------------------------------------------------*/
/* Assign DPX number, DPX number is returned in dpx                         */
/*--------------------------------------------------------------------------*/
if(Assign_dpx(&dpx,DETNUMBER) == ERROR)
{
    sprintf(ptr->err_msg,"CAT_EVT :Unable to open dpx file.");
    errlog(ptr);
    exit(0);
}
ptr->last_dpx=dpx;
/*--------------------------------------------------------------------------*/
/* Put blanks into Detection header initially                               */
/*--------------------------------------------------------------------------*/
Blank(head, SIZ_EV_HDR);
/*--------------------------------------------------------------------------*/
/* Put blanks into Detection Eventfile header initially                     */
/*--------------------------------------------------------------------------*/
Blank(output, EVTREC);
/*--------------------------------------------------------------------------*/
/* Find out where abouts in the CIRCLE buffer the event is                  */
/*--------------------------------------------------------------------------*/
//pos_in_ring_buf(ptr,&inblk, point, ringname, &fidx,type_of_evt);
/****************************************************************************/
/*  update to extract data from pipe                                        */
/****************************************************************************/
pos_in_pipe_buf(ptr,&pipblk,point);        /* second in pipe for detection      */
read_pipe_buf(pipblk,input,ptr);       /* read corresponding time-tag       */
//printf("PIPBLK=%d INPUT=%s\n",pipblk,input);

//printf("CAT Block of detection INBLK=%d\n",inblk);
//printf("CAT Ring buffer of detection=%s\n",ringname);
//printf("CAT Ring buffer number detection FIDX=%d\n",fidx);
/*--------------------------------------------------------------------------*/
/* Read in a record, (which will contain the time)                          */
/*    start at inblk, and read into input                                   */
/*--------------------------------------------------------------------------*/
////rett=read_ring_buf(ptr,inblk, input, ringname);
////if(rett == -1)
////    goto que_full;
printf("\nTime of detection=%s\n",input);
sprintf(detection,"%s",input);         // SEISANFORMAT Detection time
/*--------------------------------------------------------------------------*/
/* Edit and convert time-string for detection record                        */
/*   event time is returned by reading it from the input                    */
/*--------------------------------------------------------------------------*/
Time_Conv(input, &event_time,ptr);
/*--------------------------------------------------------------------------*/
/* Compute the duration of the event in seconds                             */
/*--------------------------------------------------------------------------*/
Compute_duration(ptr,&dur, point,type_of_evt);
//printf("Duration of detection:%d\n",(int)dur);
/*--------------------------------------------------------------------------*/
/*  Compute the start of the event for recording - ie add on pre-event
    time and allow for wrap around                                          */
/*--------------------------------------------------------------------------*/
if(type_of_evt == 0)                   /* normal event ?                    */
{
    Compute_start_pipe(ptr,dur, &pipblk, &no_of_buf, point);
}else{
    no_of_buf=(int)dur;
}
//printf("PIPE: no_of_buf=%d Address:%d\n",no_of_buf,pipblk);
//ret_val=Compute_start(ptr,dur, &inblk, &no_of_buf, point, ringname, &fidx,type_of_evt,tophdr);
//if(ret_val == -1)
//{
//    sprintf(ptr->err_msg,"CAT_EVT: Not able to catalog an event at this time");
//    errlog(ptr);
//    goto que_full;
//}
//printf("RING: no_of_buf=%d Address:%d\n",no_of_buf,inblk);
ptr->ntick=no_of_buf;                       /* Number of buffers to write   */
/*--------------------------------------------------------------------------*/
/* Get the start time for the filename - ie the time the event will be      */
/* recorded from                                                            */
/*--------------------------------------------------------------------------*/
//rett=read_ring_buf(ptr,inblk, input, ringname);
//printf("Time of recording=%s\n",input);
//if(rett==-1)
//    goto que_full;

read_pipe_buf(pipblk,input,ptr);       /* read corresponding time-tag       */

/*--------------------------------------------------------------------------*/
/* Convert the input string into an event recording time
/*--------------------------------------------------------------------------*/
Time_Conv(input, &rec_time,ptr);
/*--------------------------------------------------------------------------*/
/* Now make up the event file name using the start of recording time, the   */
/* name is returned in fnam                                                 */
/*--------------------------------------------------------------------------*/
if(type_of_evt==1)
{
    par=MANUAL;
}else{
    par=ptr->qfsmp[point];
}

//printf("Parameter set %d\n",par);

Event_Name(&rec_time, fnam,par+1,par,ptr);
/*--------------------------------------------------------------------------*/
/*  Add directory name to front of event name                               */
/*--------------------------------------------------------------------------*/
strcpy(evnam, ptr->Eventdir);
strcat(evnam, fnam);
sprintf(ptr->thisevent,"%s\n",fnam);
if(SEISAN==1)                          // SEISANFORMAT
{                                      // SEISANFORMAT
  conv_name(fnam,snam);                // SEISANFORMAT Convert QNX-name to
                                       // SEISANFORMAT S-file name
}                                      // SEISANFORMAT
/*--------------------------------------------------------------------------*/
/* Make up event header, write the additional information in it,            */
/*    and append the filename                                               */
/*--------------------------------------------------------------------------*/
Event_Header(&rec_time, no_of_buf, head,par,ptr);
Write_info(head+688, dpx, dur, &rec_time, point, ptr);
strncpy(head+688+208, fnam, EVNAMLEN);
remove_null(head, SIZ_EV_HDR);
/*--------------------------------------------------------------------------*/
/* Make up detection record in output                                       */
/*--------------------------------------------------------------------------*/
Write_info(output, dpx, dur, &event_time, point, ptr);
if(ptr->print_on_event == 1)
{
    chek_post(dpx,dur,&event_time,point,ptr,fidx,fnam,input);
    printf("\nOptional e-mail message:\n");
    printf("\n%s\n",post);
}
strncpy(output+206 , fnam, EVNAMLEN);
remove_null(output, EVTREC);
if(ptr->queue <= MAXQUE && (int)dur > ptr->durlim[par])
{
    ret_val=Det_record(output, dpx, DETECTION, ptr);
    if(ret_val==-1)
    {
        ptr->ntick=0;
        goto que_full;
    }
}else{
    ret_val=Det_rec_only(output, dpx, DETECTION, ptr);
//    if(ret_val==-1)
//    {
        ptr->ntick=0;
        goto que_full;
//    }
}
/*--------------------------------------------------------------------------*/
/*  Write event into evnam                                                  */
/*--------------------------------------------------------------------------*/
if(SEISAN==1)                          // SEISANFORMAT
{
  ret_val=Write_seisan_file_pip(ptr,evnam,head,pipblk,no_of_buf,par,point,(int)dur,kanal);
  sprintf(WDIR,"/home/w-files/");      // SEISANFORMAT Directory W-files
  duration=(float)no_of_buf;           // SEISANFORMAT
  W_HDR(WDIR,snam,head,duration,kanal); // SEISANFORMAT
}else{
  ret_val=Write_event_file_pip(ptr,evnam,head,pipblk,no_of_buf,par,point,(int)dur);
}
//ret_val=Write_event_file(ptr,evnam,head,inblk,no_of_buf,par,point,(int)dur,ringname,&fidx);
if(ret_val == -1)
{
     ptr->qfblk[point]=0;   /* This event will be lost */
     ptr->ntick=0;
     sprintf(ptr->err_msg,"CAT_EVT: a. Unable to write event to file.");
     errlog(ptr);
     sprintf(ptr->err_msg,"CAT_EVT: b. Event at this time will be lost.");
     errlog(ptr);
     goto que_full;
}
printf("%s stored on disk.\n",evnam);
if(ptr->autop==1)
{
    sprintf(syscall,"/home/seislog/auto %s\n",evnam);
    system(syscall);
}
if(ptr->autop==2)
{
    sprintf(syscall,"/home/seislog/auto_uch %s\n",evnam);
    system(syscall);
}
if(ptr->autop==3)
{
    sprintf(syscall,"/home/seislog/slog2sac %s\n",evnam);
    system(syscall);
}
if(ptr->autop==23)
{
    sprintf(syscall,"/home/seislog/auto_uch %s\n",evnam);
    system(syscall);
    sprintf(syscall,"/home/seislog/slog2sac %s\n",evnam);
    system(syscall);
}
if(SEISAN==1)                          // SEISANFORMAT
{                                      // SEISANFORMAT
  sprintf(SDIR,"/home/s-files/");      // SEISANFORMAT Directory S-files
  S_File(fnam,SDIR,detection,snam);    // SEISANFORMAT Write S-file
}                                      // SEISANFORMAT
/*--------------------------------------------------------------------------*/
/* Decrement queue, and go back into wait mode                              */
/*--------------------------------------------------------------------------*/
que_full:
if(type_of_evt==0)
{
    ptr -> queue = ptr->queue-1;       /* One event processed --            */
    point = point + 1;                 /* Point to next event               */
    if(point ==  MAXQUE) point = 0;    /* Check for wrap                    */
}
switch(ptr->mail)
{
    case 0:
    break;
    case 1:
    make_post(post,dpx,dur,&event_time,point,ptr,fidx);
    system("/home/seislog/melding");
    break;
    case 2:
    make_post(post,dpx,dur,&event_time,point,ptr,fidx);
    sprintf(syscall,"/home/seislog/pk %s\n",evnam);
    system(syscall);
    system("/home/seislog/melding");
}
goto wait;                                  /* Check queue                  */
}
/****************************************************************************/
/*      END OF CATLOG MAIN                                                  */
/****************************************************************************/
/*-----------------------------Start: chek_post()----------------------*/
chek_post( dpx, dur,rec, point,ptr,rng,name,tag)
  int dpx;
  float dur;
  struct date* rec;
  int point;
  smod* ptr;
  int rng;
  char* name;
  int* tag;  
{
char time_tag[100];
char dummy[100];
 float rsta = 0;
 float rlta = 0;
 float rampl = 0;
 int detsmp = 0;
 int detblk = 0;
 float SNR;
if(point > -1){
 rsta = ptr->qsta[point];
 rlta = ptr->qlta[point];
 rampl = ptr->qamp[point];
 detsmp = ptr->qfsmp[point];
 detblk = ptr->qfblk[point]/ptr->reclength;
 }
/* next statement sends out a sound alarm */
system("/home/seislog/destino&");
system("/home/seislog/cls");
sprintf(time_tag,"%s\n",tag);
printf("                              Station: %s\n",ptr->name);
printf("                              ------------\n");
printf("Detection number     : %5d     Recorded as: %s\n",dpx,name);
printf("\n");
printf("Date                 : %2d/%2d - %4d\n",rec->day,rec->month,rec->year);
printf("Time                 : %2d:%2d:%6.3f\n",rec->hour,rec->min,rec->secs);
if(time_tag[28] == 'S' && time_tag[29] == 'E' && time_tag[30] == 'T')
printf("GPS                  : Synchronized\n");
if(time_tag[28] == 'N' && time_tag[29] == 'S' && time_tag[30] == 'Y')
printf("GPS                  : Not Synchronized\n");
if(time_tag[28] == 'T' && time_tag[29] == 'M' && time_tag[30] == 'O')
printf("GPS                  : Not Transmitting or Serial line dead\n");
if(time_tag[28] == 'C' && time_tag[29] == 'P' && time_tag[30] == 'U')
printf("GPS                  : No GPS, CPU timing only\n");
printf("\n");
printf("Recording Duration   : %6.1f seconds\n",dur);
sprintf(dummy,"RSTA=%f RLTA=%f\n",rsta,rlta);
SNR=rsta/(rlta+0.0001);
printf("Avg.Max.Trigger Ratio: %6.1f\n",SNR);

//sprintf(string,"%5d %4d/%2d/%2d %2d:%2d:%6.3f %3d %6.1f %10.1f %10.1f %5d %s\n",
//dpx,rec->year,rec->month,rec->day,rec->hour,rec->min,rec->secs,rec->doy,
//dur,rsta,rlta,rng,ptr->name);
}
S_File(name,sdir,detec,snam)
char*   name;
char*   sdir;
char*   detec;
char*   snam;
{
FILE    *sf;
char    file_out[85];                  // name of S-file
char    dummy[80];

char    tday[5];
char    thour[5];
char    tmin[5];
char    tsec[5];
char    tyear[5];
char    tmonth[5];

int     hour;
int     minute;
float   sec;
int     day;
int     month;
int     year;

printf("%s\n",name);
sscanf(name,"%4s",tyear);
sscanf(name,"%5s%2s",dummy,tmonth);
sscanf(name,"%8s%2s",dummy,tday);
sscanf(name,"%11s%2s",dummy,thour);
sscanf(name,"%13s%2s",dummy,tmin);
sscanf(name,"%16s%2s",dummy,tsec);

sscanf(detec,"%2d",&hour);
sscanf(detec,"%3s%2d",dummy,&minute);
sscanf(detec,"%6s%f",dummy,&sec);
sscanf(detec,"%13s%2d",dummy,&day);
sscanf(detec,"%16c%2d",dummy,&month);
sscanf(detec,"%19c%4d",dummy,&year);

sprintf(file_out,"%s%2s-%2s%2s-%2sL.S%4s%2s",sdir,tday,thour,tmin,tsec,tyear,tmonth);
printf("S-filename: %s\n",file_out);
printf("Det time  : %2d:%2d:%6.3f %2d/%2d/%4d\n",hour,minute,sec,day,month,year);
sf=fopen(file_out,"w");
if(sf == NULL)
{
  printf("Can't open S-file\n");
}
fprintf(sf," %4d %2d%2d %2d%2d %4.1f",year,month,day,hour,minute,sec);
fprintf(sf," L                       BER                               1\n");
fprintf(sf," %s",snam);
fprintf(sf,"                                                 6\n");
fprintf(sf," STAT SP IPHASW D HRMM SECON CODA AMPLIT PERI AZIMU VELO");
fprintf(sf," SNR AR TRES W  DIS CAZ7\n");
fclose(sf);
}
conv_name(fnam,snam)
char* fnam;
char* snam;
{
char dummy[85];
char year[5];
char month[5];
char day[5];
char hour[5];
char minute[5];
char sec[5];
char net[5];
char nc[5];
int  chan;
int  l;
  sscanf(fnam,"%4s",year);
  sscanf(fnam,"%5s%2s",dummy,month);
  sscanf(fnam,"%8s%2s",dummy,day);
  sscanf(fnam,"%11s%2s",dummy,hour);
  sscanf(fnam,"%13s%2s",dummy,minute);
  sscanf(fnam,"%16s%2s",dummy,sec);
  sscanf(fnam,"%20s%3s",dummy,net);
  sscanf(fnam,"%24s%2d",dummy,&chan);
  sprintf(nc,"%3d",chan);
  sprintf(snam,"%4s-%2s-%2s-%2s%2s-%2sS.%3s___%3s",year,month,day,hour,minute,sec,net,nc);
  for(l=0;l<29;l++)
  {
    if(snam[l]==' ')
      snam[l]='0';
  }
}

/*--------------------------Start: Write_seisan_file_pip---------------------*/
int Write_seisan_file_pip(ptr,evnam, head, inblk, nbuf,par, point,idur,kanal)
smod*    ptr;
char*    evnam;
char*    head;
int      inblk;
int      nbuf;
int      par;
int      point;
int      idur;
int      kanal[];
{
FILE*    new;
int      bytes;
int      i;                            /* counters                          */
int      j;
static   int datal[BUFFER];
//static   int kanal[50000];
int      newblk;                       /* find new position after each read */
int      comp;
int      ix;
int      t;
int      idd;
int      fast;
int      tophdr;
int*     in_ptr_l;
int*     pk_ptr_l;
int      scans;
int      kans;
int      kj;
int      lr;
int      nxt_rbf;
int      secpip;
char     detnam[400];
FILE     *idx;


new=fopen(evnam,"w");                  /* Open new event-file               */
if(new == 0)
{
    sprintf(ptr->err_msg,"CAT_EVT: a. Unable to open new eventfile with name:");
    errlog(ptr);
    sprintf(ptr->err_msg,"CAT_EVT: b. %s.",evnam);
    errlog(ptr);
    return(-1);
}

/*--------------------------------------------------------------------------*/
/* Write event header                                                       */
/*--------------------------------------------------------------------------*/
fseek(new,0,0);
bytes=fwrite(head,SIZ_EV_HDR,1,new);
if(bytes==0)
    clearerr(new);
/****************************************************************************/
/* Copy NBUF data buffers from data to new eventfile                        */
/****************************************************************************/
if(par != EXTRACT)
{
    fast=idur+ptr->prevnt[par];
}else{
    fast=idur;
}
//printf("NBUF=%d FAST=%d PREVNT[%d]=%d\n",nbuf,fast,par,ptr->prevnt[par]);
secpip=inblk;
for(j=1;j<nbuf+1;j++)                  /* Pick out nbuf buffers             */
{
    ptr->alive[4]=1;                   /* tell I am alive                   */
    while((secpip-ptr->sec_ut) == 0) sleep(1);
    read_pipe_buf2(ptr,secpip,datal);
    next_block(ptr,&newblk,j,par);
/****************************************************************************/
/* Position for first data record is computed so that the length of the     */
/* header is always a multiple of the recordlength >= 1024                  */
/****************************************************************************/
    fseek(new,newblk,0);                /* Position eventfile               */
    if(ptr->nchstr[par] == ptr-> nchan || par == EXTRACT)
    {
      bytes=fwrite(datal,ptr->reclength,1,new);
    }else{
      in_ptr_l=datal+HEADER;
      pk_ptr_l=datal+HEADER;
      for(scans=0;scans<ptr->nsamp;scans++)
      {
        for(kans=0;kans<ptr->nchan;kans++)
        {
          if(ptr->store[par][kans]==1)
          {
            *(in_ptr_l)=*(pk_ptr_l);
            in_ptr_l+=1;
            pk_ptr_l+=1;
          }else{
            pk_ptr_l+=1;
          }
        }
      }
      bytes=fwrite(datal,ptr->newrl[par],1,new);
    }
    if(bytes==0)
      clearerr(new);
    ptr->ntick=ptr->ntick-1;
    if(ptr->print_on_event == 1)
    {
      if(j<=ptr->prevnt[par])
        printf(".");
      if(j>ptr->prevnt[par] && j<= fast)
        printf("E");
      if(j> fast)
        printf("_");
      fflush(stdin);
      fflush(stdout);
    }
    secpip=secpip+1;
    if(secpip >= ptr->sec_in_pip)
      secpip=0;
}   /* end of for loop writing the data */
printf("\n");
fclose(new);

ix=0;
for(comp=0;comp<ptr->nchan;comp++)
{
printf("COMP: %2d\n",comp);
  secpip=inblk;
  for(j=1;j<nbuf+1;j++)                  /* Pick out nbuf buffers             */
  {
    ptr->alive[4]=1;                   /* tell I am alive                   */
    read_pipe_buf2(ptr,secpip,datal);
    next_block(ptr,&newblk,j,par);
/****************************************************************************/
/* Position for first data record is computed so that the length of the     */
/* header is always a multiple of the recordlength >= 1024                  */
/****************************************************************************/
//    fseek(new,newblk,0);                /* Position eventfile               */
    if(ptr->nchstr[par] == ptr-> nchan || par == EXTRACT)
    {
//      bytes=fwrite(datal,ptr->reclength,1,new);

for(t=0;t<ptr->nsamp;t++)
{
  kanal[ix]=datal[t*ptr->nchan+HEADER+comp];
  ix++;
}


    }else{
      in_ptr_l=datal+HEADER;
      pk_ptr_l=datal+HEADER;
      for(scans=0;scans<ptr->nsamp;scans++)
      {
        for(kans=0;kans<ptr->nchan;kans++)
        {
          if(ptr->store[par][kans]==1)
          {
            *(in_ptr_l)=*(pk_ptr_l);
            in_ptr_l+=1;
            pk_ptr_l+=1;
          }else{
            pk_ptr_l+=1;
          }
        }
      }
//      bytes=fwrite(datal,ptr->newrl[par],1,new);
    }
    secpip=secpip+1;
    if(secpip >= ptr->sec_in_pip)
      secpip=0;
  }   /* end of for loop writing the data */
}


return(0);
}
/*-----------------------------------------End: Write_event_file_pip--------*/

/*----------------------------------------Start: W_HDR----------------------*/
/* Function to write event file in SEISAN format                            */
/* Arguments:                                                               */
/* wdir   : directory path, f.ex.: /home/w-files/                           */
/* snam   : SEISAN filename                                                 */
/* hd     : SEISLOG event header                                            */
/* dur    : duration in seconds (float)                                     */
/* kanal  : array to hold demultiplexed channels                            */
/*--------------------------------------------------------------------------*/
W_HDR(wdir,snam,hd,dur,kanal)
char*   wdir;
char*   snam;
char*   hd;
float   dur;
int     kanal[];
{
FILE    *sf;
int     lengde[10];
int     seismo[5000];
char    file_out[85];                  // name of W-file
char    line01[90];                    // header line
char    dummy[200];
int     kanaler;
char    century[1];
char    yr[2];
char    nam[64][6];
char    cmp[64][6];
char    na1[6];
char    na2[6];
char    na3[6];
char    comp1[6];
char    comp2[6];
char    comp3[6];
int     doy;
float   zero;
int     hour;
int     minute;
float   sec;
int     day;
int     month;
int     year;
int     cnt;
int     ch;
int     jj;
int     kk;
int     total_no_samples;
int     samples_channel;
unsigned int verdi;
unsigned char b1,b2,b3,b4;
int val;
int recl;
int ret;
int kt;
int counter;
century[0]='1';
//
// read number of channels from SEISLOG header and initialise some tables
//
sscanf(hd,"%62c%d",dummy,&kanaler);    // extract number of channels

for(ch=0;ch<kanaler;ch++)              // initialise station and component tables
{
  for(cnt=0;cnt<6;cnt++)
  {
    nam[ch][cnt]=' ';
    cmp[ch][cnt]=' ';
  }
}
sprintf(file_out,"%s%s",wdir,snam);    // make complete filename
//
// read SEISLOG header to extract information for new SEISAN header
//
sscanf(snam,"%2c%d",dummy,&year);
sscanf(hd,"%19c%d",dummy,&doy);
sscanf(hd,"%23c%d",dummy,&month);
sscanf(hd,"%26c%d",dummy,&day);
sscanf(hd,"%32c%d",dummy,&hour);
sscanf(hd,"%35c%d",dummy,&minute);
sscanf(hd,"%38c%f",dummy,&sec);

jj=160;
for(ch=0;ch<kanaler;ch++)
{
  for(cnt=0;cnt<4;cnt++)
  {
    nam[ch][cnt]=hd[jj+cnt];
    cmp[ch][cnt]=hd[jj+cnt+4];
  }
  nam[ch][4]='\0';
  cmp[ch][4]='\0';
  jj=jj+8;  
}
total_no_samples=(int)dur*ptr->nsamp*ptr->nchan; // total number of samples event
samples_channel=(int)dur*ptr->nsamp;   // number of samples per channel
//
// do byte swapping on all samples in demultiplexed channel array
//
for(jj=0;jj<total_no_samples;jj++)
{
  verdi=kanal[jj];
  b4=verdi;
  verdi=verdi >> 8;
  b3=verdi;
  verdi=verdi >> 8;
  b2=verdi;
  verdi=verdi >> 8;
  b1=verdi;
  verdi=b4;
  verdi=verdi << 8;
  verdi=verdi | b3;
  verdi=verdi << 8;
  verdi=verdi | b2;
  verdi=verdi << 8;
  verdi=verdi | b1;
  kanal[jj]=verdi;
}
//
// do byte swapping of the recordlength in bytes
//
verdi=samples_channel*4;
b4=verdi;
verdi=verdi >> 8;
b3=verdi;
verdi=verdi >> 8;
b2=verdi;
verdi=verdi >> 8;
b1=verdi;
verdi=b4;
verdi=verdi << 8;
verdi=verdi | b3;
verdi=verdi << 8;
verdi=verdi | b2;
verdi=verdi << 8;
verdi=verdi | b1;
recl=verdi;
lengde[0]=verdi;

sf=fopen(file_out,"w");
if(sf == NULL)
{
  printf("Can't open W-file\n");
}
//------------------------LINE 1----------------------------------
fprintf(sf,"%c%c%c%c",0x00,0x00,0x00,'P');
fprintf(sf,"                              ");
fprintf(sf,"%3d%c%2d %3d %2d %2d %2d %2d %6.3f %9.3f",
kanaler,century[0],year,doy,month,day,hour,minute,sec,dur);
fprintf(sf,"           ");
fprintf(sf,"%c%c%c%c",0x00,0x00,0x00,'P');
//------------------------LINE 2----------------------------------
fprintf(sf,"%c%c%c%c",0x00,0x00,0x00,'P');
fprintf(sf,"                                        ");
fprintf(sf,"                                        ");
fprintf(sf,"%c%c%c%c",0x00,0x00,0x00,'P');
//------------------------LINE 3 and onwards----------------------
counter=0;
cnt=0;
for(ch=0;ch<kanaler;ch++)
{
  if(counter==0)
    fprintf(sf,"%c%c%c%c",0x00,0x00,0x00,'P');
  printf("%2d %4s %4s %8.2f\n",ch,nam[ch],cmp[ch],dur);
  fprintf(sf," %4s%4s     0.0 %8.2f",nam[ch],cmp[ch],dur);
  cnt++;
  counter++;
  if(counter == 3)
  {
    counter=0;
    fprintf(sf,"  ");
    fprintf(sf,"%c%c%c%c",0x00,0x00,0x00,'P');
  }
}
for(ch=0;ch<(10*3-cnt);ch++)
{
  if(counter == 0)
    fprintf(sf,"%c%c%c%c",0x00,0x00,0x00,'P');
  fprintf(sf,"                          ");
  counter++;
  if(counter == 3)
  {
    counter=0;
    fprintf(sf,"  ");
    fprintf(sf,"%c%c%c%c",0x00,0x00,0x00,'P');
  }
}
//------------------------Finished Main Header-----------------------
//------------------------Start Event Header-------------------------
for(ch=0;ch<kanaler;ch++)
{
  fprintf(sf,"%c%c%c%c",0x00,0x00,0x04,0x10);
  fprintf(sf,"%4s %4s%c%2d %3d %2d %2d %2d %2d %6.3f %7.4f %6d",
nam[ch],cmp[ch],century[0],year,doy,month,day,hour,minute,sec,(float)ptr->nsamp,ptr->nsamp*(int)dur);
  fprintf(sf,"                          4   ");
  for(cnt=0;cnt<24;cnt++)
    fprintf(sf,"                                        ");
  fprintf(sf,"%c%c%c%c",0x00,0x00,0x04,0x10);
  ret=fwrite(lengde,4,1,sf);
  for(kt=0;kt<samples_channel;kt++)
    seismo[kt]=kanal[kt+ch * samples_channel];
  ret=fwrite(seismo,samples_channel * 4,1,sf);
  ret=fwrite(lengde,4,1,sf);
}

fclose(sf);
}
