/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:16/02/98                       */
/* Program:CAT_EXT                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 01/06/00 t.u. 8.22 variable To Catlog now counts down to 0 as for CAT_EVT*/
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program saves data onto separate files when activated by the        */
/* extract program.                                                         */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include "filedef.h"                   /* Include file for application      */
#include "timexrou.h"                  /* Structure def. of date and time   */
#ifdef __USAGE
-----------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
-----------------------------------------------------------------------------
Description: Make file in /home/events of extracted data. Filename of
             extracted data has the extension _X.

Usage:       cat_ext (no parameters)
#endif

int      id;
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

int      datal[BUFFER];
short    datas[BUFFER];
char     syscall[200];
int      tophdr;
main()
{
int      type_of_evt;
int      yrn;                          /* year                              */
int      mon;                          /* month                             */
int      day;                          /* day                               */
int      hrn;                          /* hour                              */
int      min;                          /* minute                            */
float    sec;                          /* second                            */
double   sectot1;                      /* number of seconds for a date      */
int      i;                            /* counter                           */
int      konst;                        /* constant                          */
int      par;                          /* parameter set                     */
int      dpx;                          /*J DPX number for this event        */
int      inblk;                        /*J Block number for fseek to find   */
                                       /*  start of event                   */ 
int      start_blk;                    /*  Block number for first block in  */
                                       /*  event                            */
float    dur;                          /*J Duration of event in seconds     */
int      no_of_buf;                    /*J Total number of buffers for event*/
char     fnam[EVNAMLEN];               /*  Event filename                   */
char     evnam[60];                    /* Event name incl. directory path   */
int      point;                        /* Place of event in queue           */
int      idur;                         /* integer duration                  */
int      fidx;                         /* index of the ring filename        */
char     ringname[DIRNAMLEN+EVNAMLEN]; /* name of ringbuffer to read        */
static   char head[SIZ_EV_HDR+1];      /*J Space for det. header (record)   */
static   char output[EVTREC+1];        /*J Space for SEISNOR standard header*/
static   int input[BUFFER];            /*J Space to hold one data buffer    */
struct   date  event_time;             /*J Time of the event being processed*/
struct   date  rec_time;               /*J Time the event file is to start  */
struct   date  start_time;             /*  Start time of recording          */
struct   date  begin_time;             /*  Stop time of recording           */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"CAT_EXT_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CAT_EXT_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
tophdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
konst=ptr->nchan * ptr->nsamp + HEADER;
type_of_evt=2;                         /* extracted event                   */
/*--------------------------------------------------------------------------*/
/*  initialise all necessary shared variables                               */
/*--------------------------------------------------------------------------*/
point = 0;                             /* start with the first in the queue */
/*--------------------------------------------------------------------------*/
/* Assign DPX number, DPX number is returned in dpx                         */
/*--------------------------------------------------------------------------*/
if(Assign_dpx(&dpx,DETNUMBER) == ERROR)
{
    printf("CAT_EXT Unable to open dpx file\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Put blanks into Detection header initially                               */
/*--------------------------------------------------------------------------*/
Blank(head, SIZ_EV_HDR);
/*--------------------------------------------------------------------------*/
/* Put blanks into Detection Eventfile header initially                     */
/*--------------------------------------------------------------------------*/
Blank(output, EVTREC);
/*--------------------------------------------------------------------------*/
/* Find out where abouts in the CIRCLE buffer the event is                  */
/*--------------------------------------------------------------------------*/
pos_in_ring_buf(ptr,&inblk,point, ringname, &fidx,type_of_evt);
/*--------------------------------------------------------------------------*/
/* Read in a record, (which will contain the time)                          */
/*    start at inblk, and read into input                                   */
/*--------------------------------------------------------------------------*/
read_ring_buf(ptr,inblk, input, ringname);
/*--------------------------------------------------------------------------*/
/* Edit and convert time-string for detection record                        */
/*   event time is returned by reading it from the input                    */
/*--------------------------------------------------------------------------*/
Time_Conv(input, &event_time,ptr);
/*--------------------------------------------------------------------------*/
/* Compute the duration of the event in seconds                             */
/*--------------------------------------------------------------------------*/
idur=ptr->dur_ext;
dur=(float)idur;
no_of_buf=ptr->dur_ext;                /* Number of buffers to write        */
ptr->ntick=no_of_buf;
/*--------------------------------------------------------------------------*/
/* Get the start time for the filename - ie the time the event will be      */
/* recorded from                                                            */
/*--------------------------------------------------------------------------*/
read_ring_buf(ptr,inblk, input, ringname);
/*--------------------------------------------------------------------------*/
/* Convert the input string into an event recording time                    */
/*--------------------------------------------------------------------------*/
Time_Conv(input, &rec_time,ptr);
/*--------------------------------------------------------------------------*/
/* Now make up the event file name using the start of recording time, the   */
/* name is returned in fnam                                                 */
/*--------------------------------------------------------------------------*/
par=EXTRACT;
Event_Name(&rec_time, fnam,par+1,par,ptr);
/*--------------------------------------------------------------------------*/
/*  Add directory name to front of event name                               */
/*--------------------------------------------------------------------------*/
strcpy(evnam, ptr->Eventdir);
strcat(evnam, fnam);
printf("Filename %s\n",evnam);
/*--------------------------------------------------------------------------*/
/* Make up event header, write the additional information in it,            */
/*    and append the filename                                               */
/*--------------------------------------------------------------------------*/
Event_Header(&rec_time, no_of_buf, head,par,ptr);
Write_info(head+688, dpx, dur, &rec_time, point, ptr);
strncpy(head+688+208, fnam, EVNAMLEN);
remove_null(head, SIZ_EV_HDR);
/*--------------------------------------------------------------------------*/
/* Make up detection record in output                                       */
/*--------------------------------------------------------------------------*/
Write_info(output, dpx, dur, &event_time, point, ptr);
strncpy(output+206 , fnam, EVNAMLEN);
remove_null(output, EVTREC);
Det_record(output, dpx, DETECTION, ptr);
/*--------------------------------------------------------------------------*/
/*  Write event into evnam                                                  */
/*--------------------------------------------------------------------------*/
if(Write_event_file(ptr,evnam,head,inblk,no_of_buf,par,point,idur,
                    ringname,&fidx) == ERROR)
i=0;
}
/****************************************************************************/
/*      END OF CATLOG MAIN                                                  */
/****************************************************************************/
