/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:18/07/95                       */
/* Program:CAT_P_PH                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/****************************************************************************/
#include    "shar_mem.h"
#include    <sys/mman.h>
#include    <fcntl.h>
#include    <errno.h>
#include    <stdlib.h>
#include    <string.h>
#include    "filedef.h"           /* Include file definition for application*/
#include    "timexrou.h"

int fd;
smod  *ptr;

int   datal[BUFFER];
short datas[BUFFER];

main()
{
int i;
int konst;                          /* parameter for READ_PIPE_BUF           */
   int par;
   int det_id;                      /*J id for linking to event signal      */
   int cat_id;
   int dpx;                         /*J DPX number for this event           */
   static char head[SIZ_EV_HDR+1]; /*J Space for detection header (record)       */
   static char output[EVTREC+1];  /*J Space for SEISNOR standard header         */
   int inblk;                 /*J Block number for fseek to find start of 
                                  event                                     */
   static int input[BUFFER];/*J Space to hold one data buffer             */  
   struct date  event_time;   /*J Time of the event being processed         */
   struct date  rec_time;     /*J Time that the event file is to start      */
   float dur;                 /*J Duration of event in seconds              */
   int no_of_buf;             /*J Total number of buffers for event         */
   char fnam[EVNAMLEN];       /*  Event filename                            */
   char evnam[39];            /* Event name including directory path        */
   int point;                 /* Place of event in queue                    */
   int idur;                  /* integer duration                           */
   char ringname[DIRNAMLEN+EVNAMLEN]; /*holds name of ring buffer to read   */
   int fidx;                  /* index of the ring filename                 */

/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"CAT_P_PH_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CAT_P_PH_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

/*--------------------------------------------------------------------------*/
/* Get process id for CATLOG. The ids is stored in the parameter            */
/* module for later use by STPACQ.                                          */
/*--------------------------------------------------------------------------*/
/*ptr->proc_id_catlog=getpid();*/
    konst=ptr->nchan * ptr->nsamp+HEADER;   /* for READ_PIPE_BUF          */
/*--------------------------------------------------------------------------*/
/*  initialise all necessary shared variables                               */
/*--------------------------------------------------------------------------*/
ptr->ptick = 0;
ptr->pqueue= -1;
point = 0;                 /* start with the first in the queue */
/*--------------------------------------------------------------------------*/
/* CHECK 'QUEUE' AND ENTER WAIT-STATE IF 'QUEUE' IS EMPTY (-1)              */
/* PROGRAM WILL BE STARTED BY 'DET' WHEN A NEW EVENT IS DETECTED            */  
/* IF THERE ARE EVENT IN THE QUEUE, GOON AND PROCESS IT !!                  */
/*--------------------------------------------------------------------------*/
wait:
if(ptr->pqueue < 0 )                         /* Check if there is any events */
{                                           /* queued up >=0 ?              */
    sleep(1);
    goto wait;
}
/* Event in queue, goon and process it                                      */
/*--------------------------------------------------------------------------*/
/* Assign DPX number, DPX number returned in dpx                            */
/*--------------------------------------------------------------------------*/
if(Assign_dpx(&dpx,DETNUMBER) == ERROR)
{
    printf("CAT_P_PH Unable to open dpx file\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Put blanks into Detection header initially                               */
/*--------------------------------------------------------------------------*/
Blank(head,SIZ_EV_HDR);
/*--------------------------------------------------------------------------*/
/* Put blanks into Detection Event file header initially                    */
/*--------------------------------------------------------------------------*/
Blank(output,EVTREC);
/*--------------------------------------------------------------------------*/
/* Find out where abouts in the CIRCLE buffer the event is                  */
/*--------------------------------------------------------------------------*/
pos_in_pipe_buf(&inblk,point);
/*printf("CAT_P_PH:Position in pipe:%d seconds\n",inblk);*/
/*--------------------------------------------------------------------------*/
/* Read in a record, (which will contain the time)                          */
/* start at inblk, and read into input                                      */
/*--------------------------------------------------------------------------*/
    read_pipe_buf(inblk,input,konst);
/*--------------------------------------------------------------------------*/
/* Edit and convert time-string for detection record                        */
/*   event time is returned by reading it from the input                    */
/*--------------------------------------------------------------------------*/
Time_Conv(input, &event_time,ptr);
/*--------------------------------------------------------------------------*/
/* Compute the duration of the event in seconds                             */
/*--------------------------------------------------------------------------*/
Compute_duration(&dur, point);
idur=(int)dur;
/*--------------------------------------------------------------------------*/
/*  Compute the start of the event for recording - ie add on pre-event
    time and allow for wrap around                                          */
/*--------------------------------------------------------------------------*/
    Compute_start_pipe(dur,&inblk,&no_of_buf,point);
ptr->ptick=no_of_buf;                       /* Number of buffers to write   */
/*printf("CAT_P_PH:ptick:%d\n",no_of_buf);*/
/*printf("CAT_P_PH:start second in pipe:%d\n",inblk);*/
/*--------------------------------------------------------------------------*/
/* Get the start time for the filename - ie the time the event will be      */
/* recorded from                                                            */
/*--------------------------------------------------------------------------*/
read_pipe_buf(inblk,input,konst);
/*--------------------------------------------------------------------------*/
/* Convert the input string into an event recording time
/*--------------------------------------------------------------------------*/
Time_Conv(input, &rec_time,ptr);
/*--------------------------------------------------------------------------*/
/* Now make up the event file name using the start of recording time, the   */
/* name is returned in fnam                                                 */
/*--------------------------------------------------------------------------*/
par=ptr->qpsmp[point];
Event_Name_p(&rec_time, fnam,par+1,par,ptr);
/*printf("CAT_P_PH: EVENTNAME:    %s\n", fnam);*/
/*--------------------------------------------------------------------------*/
/*  Add directory name to front of event name                               */
/*--------------------------------------------------------------------------*/
    strcpy(evnam, ptr->Eventdir);
    strcat(evnam, fnam);
/*--------------------------------------------------------------------------*/
/* Make up event header, write the additional information in it,
    and append the filename                                                 */
/*--------------------------------------------------------------------------*/
Event_Header(&rec_time, no_of_buf, head,par,ptr);
Write_info(head+688, dpx, dur, &rec_time, point, ptr);

strncpy(head+688+208, fnam, EVNAMLEN);
remove_null(head, SIZ_EV_HDR);

/*--------------------------------------------------------------------------*/
/* Make up detection record in output
/*--------------------------------------------------------------------------*/
Write_info(output, dpx, dur, &event_time, point, ptr);

strncpy(output+208 , fnam, EVNAMLEN);

remove_null(output, EVTREC);
if(ptr->pqueue <= MAXQUE && idur >= ptr->durlim[par])
{
    Det_record(output, dpx, DETECTION, ptr);
}else{
    Det_rec_only(output, dpx);
    ptr->ptick=0;
    goto que_full;
}

/*--------------------------------------------------------------------------*/
/*  Write event into evnam
/*--------------------------------------------------------------------------*/
if(wrt_evt_fil_pip(evnam,head,inblk,no_of_buf,cat_id,par,point,idur)==ERROR)
/*     ptr->qpblk[point]=0; */  /* This event will be lost */

/*printf("Here 10\n");*/
/*--------------------------------------------------------------------------*/
/* Decrement queue, and go back into wait mode
/*--------------------------------------------------------------------------*/
que_full:
idur=0;
/*printf("CAT_P_PH:Before decrement pqueue=%d\n",ptr->pqueue);*/
ptr -> pqueue = ptr->pqueue-1;                /* One event processed --       */
                                            /* decrement queue              */
/*printf("CAT_P_PH:After  decrement pqueue=%d\n",ptr->pqueue);*/

point = point + 1;                          /* Point to next event          */
if(point ==  MAXQUE) point = 0;         /* Check for wrap               */
goto wait;
}
/**************************************************************************/
/*      END OF CATLOG MAIN
/**************************************************************************/

/*--------------------------Start: Write_event_file()--------------------*/
int wrt_evt_fil_pip(evnam, head, inblk, nbuf, cat_id,par, point,idur)

char* evnam;
char* head;
int inblk, nbuf, cat_id;
int par;
int point,idur;  
{
FILE* new;
int bytes;
int i,j;                        /* counters */

int newblk;                     /* to find new position after each read */
int idd;
int fast;
int tophdr;

short* in_ptr_s;
short* pk_ptr_s;
int* in_ptr_l;
int* pk_ptr_l;
int scans,kans;

new=fopen(evnam,"w");            /* Open new event-file          */
if(new == 0)
{
    printf("Unable to open eventfile: %s\n",evnam);
    for(i=0;i<30;i++)
        printf("%x",evnam[i]);
    printf("\n");
    return ERROR;
}
/*--------------------------------------------------------------------------*/
/* Write event header                                                       */
/*--------------------------------------------------------------------------*/
fseek(new,0,0);
bytes=fwrite(head,SIZ_EV_HDR,1,new);    
if(bytes==0)
    clearerr(new);
/****************************************************************************/
/* Copy NBUF data buffers from data to new eventfile                        */  
/****************************************************************************/
fast=idur+ptr->prevnt[par];

for(j=1;j<nbuf+1;j++)                       /* Pick out nbuf buffers        */
{
    read_ring_buf_pip(inblk);
    next_block(ptr,&newblk,j,par);
    fseek(new,newblk,0);                    /* Position eventfile           */   
    if(ptr->nchstr[par] == ptr-> nchan)     /* Save all channels ?          */
    {
        bytes=fwrite(datal,ptr->reclength,1,new); /* YES, all channels      */
    }else{
        in_ptr_l=datal+HEADER;
        pk_ptr_l=datal+HEADER;
        for(scans=0;scans<ptr->nsamp;scans++)
        {
            for(kans=0;kans<ptr->nchan;kans++)
            {
                if(ptr->store[par][kans]==1)
                {
                    *(in_ptr_l)=*(pk_ptr_l);
                    in_ptr_l+=1;
                    pk_ptr_l+=1;
                }else{
                    pk_ptr_l+=1;
                }
            }
        }
    }
                bytes=fwrite(datal,ptr->newrl[par],1,new);
if(bytes==0)
   clearerr(new);

ptr->ptick=ptr->ptick-1;
inblk=inblk+1;                           /* Compute new address for      */
                                            /* for inputfile (data)         */
        /* Check for forward wrap       */

if(inblk >= ptr->sec_in_pip)
    inblk=0;

}   /* end of for loop writing the data */
        fclose(new);
return(0);
}
/*-----------------------------------------End: wrt_evt_fil_pipe------------*/

/*-----------------------------Start: pos_in_pipe_buf-----------------*/

pos_in_pipe_buf(inblk, point)
    int* inblk;
    int point;
{
*inblk       =   ptr-> qpsec[point];         /* Get detection pipe-address   */
                                             /* fresh from data-module       */
}
  
/*-----------------------------End: pos_in_pipe_buf-----------------*/

/*-----------------------------Start: read_pipe_buf-----------------*/

read_pipe_buf(inblk, input, konst)
  int inblk;     /* beginning of event */
  int* input;    /* buffer for input   */
  int konst;  
{
unsigned int* ch_ptr;    /* pointer to memory pipe */
unsigned int longv;
unsigned short shortv;
int kp;

ch_ptr=ptr->pipe + inblk * konst;  /* point to correct second in pipe */

for(kp=0;kp<16;kp++)
{
    longv=*(ch_ptr);
    input[kp]=longv;
    ch_ptr+=1;
}
}
/*-----------------------------End: read_pipe_buf-----------------*/

read_ring_buf_pip(inblk)
  int inblk;     /* beginning of event */
 {

register int kp;
unsigned int longv;
unsigned short shortv;
unsigned short* ch_ptr;
unsigned int* chi_ptr;
short* sh_ptr;
int* lg_ptr;
int konst;
int total;

    konst=ptr->nchan * ptr->nsamp+HEADER;
    total=ptr->nsamp * ptr->nchan + HEADER;
    chi_ptr=ptr->pipe+inblk*konst;
    lg_ptr=datal;
    for(kp=0;kp<total;kp++)
        *lg_ptr++=*chi_ptr++;

}
/*-----------------------------Start: Compute_duration()------------------*/
Compute_duration(dur, point)
   float* dur;
   int point;
{
int ival1, ival2;
float val1, val2;
float rsamp;

rsamp = ptr -> nsamp;
ival1 = ptr -> qdur[point];                /* Compute duration in seconds  */
ival2 = ptr -> rate;
val1=ival1;
val2=ival2;

*dur=val1*rsamp/val2;                        /* Duration station-type 1      */
    
}
/*-----------------------------End: Compute_duration()------------------*/

/*-----------------------------Start: Compute_start_pipe()--------------------*/

Compute_start_pipe(dur, inblk, no_of_buf, point)
   float dur;              /* duration of event in seconds                    */
   int *inblk;             /* set at the position in the ring buffer at which */
                           /* trigger occurred. To be returned allowing for   */
                           /* wrap around, and for some pre-event time        */
   int *no_of_buf;         /* to be returned as the number of event buffers   */
                           /* that should be written into the event file      */
   int point;              /* number of the event in the queue                */
{
    int ttotal;            /* total number of seconds for event               */
    int par;
    par= ptr->qpsmp[point];
/*printf("CAT_EVTS:par=%d\n",par);*/
 ttotal=2+ptr->prevnt[par]; /* Total number of seconds  */
/*printf("CAT_P_PH:dur=%f prevnt=%d\n",dur,ptr->prevnt[par]);*/
/*printf("CAT_P_PH:%d\n",ttotal);*/
    *no_of_buf=ttotal;                        /* Number of buffers for event  */

    if(*no_of_buf > ptr->maxbuf[par])             /* Check for wrap-around        */
        *no_of_buf=ptr->maxbuf[par];
                                            /* Disk-address incl. pre-event */
/*printf("PTICK2:%d\n",*no_of_buf);*/
   *inblk=*inblk-ptr->prevnt[par];
    if(*inblk < 0)                           /* Check for backward wrap      */
    {
/*printf("BACKWARD WRAP\n");*/
        *inblk=*inblk+ptr->sec_in_pip;  
}
}
/*-----------------------------End: Compute_start_pipe()-------------------*/
/*------------------------------Start:Det_rec_only--------------------------*/
Det_rec_only(output, dpx)
char*  output;                         /* detection record                  */
int    dpx;                            /* detection number                  */
  
{
FILE*  dets;                           /* pointer to detection file         */
int    bytes;                          /* return value after write          */
int    lt;                             /* counter                           */
int    outblk;                         /* disk adress detection record      */

outblk = (dpx % EVTSIZ) * EVTREC;      /* compute disk adress detection rec */

for(lt=208;lt<233;lt++)                /* put blanks in filename            */
    output[lt]=' ';   

/*--------------------------------------------------------------------------*/
/* Write detection record                                                   */
/*--------------------------------------------------------------------------*/

dets=fopen(DETECTION,"r+");
fseek(dets,outblk,0);
bytes=fwrite(output,EVTREC,1,dets);
if(bytes==0)
    clearerr(dets);
fclose(dets);
}
/****************************************************************************/
/*                    VME-OS-9    SEISLOG Software           VERSION 6.0    */
/*                    PC -OS-9000 SEISLOG Software           VERSION 6.0    */
/****************************************************************************/
/*                                                                          */
/*                                  University of Bergen                    */
/*                                  Institute of Solid Earth Physics        */
/*                                  Seismological Observatory               */
/*                                  Allegaten 41, 5000 Bergen               */
/*                                  Norway. Tlf.55213419 Fax 55320009       */
/*                                  E-mail: terje.utheim@ifjf.uib.no        */
/*                                          jens.havskov@ifjf.uib.no        */
/*                                  Authors:                                */
/*                                  Terje Utheim,Jens Havskov               */
/*                                                                          */
/*                                  Date:8.3.95                             */
/* Procedures common to detec_01, catlog etc                                */
/*                                                                          */
/* Last update: 8.3.95                                                      */
/*                                                                          */
/* Info: The following procedures are contained in this file, they are      */
/*       listed here along with the programs which use them                 */
/*                                                                          */
/* Event_Header: writes out an event header given the information           */
/*               that is needed.                                            */
/*       Used by: Detec_01.c to write headers into each ring buffer         */
/*                Cat_evts.c to write headers into event files              */
/* Event_Name:  makes up the name of an event file                          */
/*       Used by: Dsk_all.c in allocating the ring buffers for cont. record */
/*                Cat_evts.c to make up evnt file names                     */
/*                Detec_01.c to make the names of new continuous records    */ 
/****************************************************************************/

/*#include    <string.h>*/
/*#include    "shar_mem.h"*/
/*#include    <errno.h>*/
/*#include    "filedef.h"*/           /* Include file definition for application*/





/*--------------------------------------------Start: Event_name-------------*/
/* Make file name from time-string                                          */
/*--------------------------------------------------------------------------*/
Event_Name(rec_time, fnam,point,par,ptr)
  struct date* rec_time;
  char* fnam;
  int point;
  int par;
  smod* ptr;
{   
int i;
/* The format of this event file is as follows:
   YYMM_DD_HHMM_SST.CCC_NN_P
   year,month,day,hour,mins,secs,T=temporary,station code, num of channels 
   P parameter set */
   
   char cyear[4];   /* help variable in cutting the century off year        */
   char* namptr= fnam;    /* index in fnam string                           */
   
 
sprintf(cyear, "%4d", rec_time->year);

sprintf(namptr, "%2s", cyear+2);
namptr+=2;
sprintf(namptr, "%2d_", rec_time->month);
namptr+=3;
sprintf(namptr, "%2d_", rec_time->day);
namptr+=3;
sprintf(namptr, "%2d%2d_", rec_time->hour, rec_time->min);
namptr+=5;
sprintf(namptr, "%2dT.", (int) rec_time->secs);
namptr+=4;
strncpy(namptr, ptr->name, 3);                   /* station code            */
namptr+=3;
*namptr++ = '_';

sprintf(namptr, "%2d", ptr->nchstr[par]);   

namptr+=2;
/*switch(point)*/
switch(par)
{
    case MANUAL:
    sprintf(namptr, "_M");
    break;
    case EXTRACT:
    sprintf(namptr, "_X");
    break;
    case CONTIN:
    sprintf(namptr, "_C");
    break;
    default:
    sprintf(namptr, "_%1d", point);
    break;
}
/* fill fnam with zeros in all blank spaces */
zero_fill(fnam);


}
/*------------------------------------------End: Event_Name-----------------*/

/*--------------------------------------------Start: Event_name_p-------------*/
/* Make file name from time-string                                          */
/*--------------------------------------------------------------------------*/
Event_Name_p(rec_time, fnam,point,par,ptr)
  struct date* rec_time;
  char* fnam;
  int point;
  int par;
  smod* ptr;
{   
int i;
/* The format of this event file is as follows:
   YYMM_DD_HHMM_SST.CCC_NN_P
   year,month,day,hour,mins,secs,T=temporary,station code, num of channels 
   P parameter set */

   char cyear[4];   /* help variable in cutting the century off year        */
   char* namptr= fnam;    /* index in fnam string                           */

sprintf(cyear, "%4d", rec_time->year);
sprintf(namptr, "%2s", cyear+2);
namptr+=2;
sprintf(namptr, "%2d_", rec_time->month);
namptr+=3;
sprintf(namptr, "%2d_", rec_time->day);
namptr+=3;
sprintf(namptr, "%2d%2d_", rec_time->hour, rec_time->min);
namptr+=5;
sprintf(namptr, "%2dP.", (int) rec_time->secs);
namptr+=4;
strncpy(namptr, ptr->name, 3);                   /* station code            */
namptr+=3;
*namptr++ = '_';

sprintf(namptr, "%2d", ptr->nchstr[par]);   

namptr+=2;
/*switch(point)*/
switch(par)
{
    case MANUAL:
    sprintf(namptr, "_M");
    break;
    case EXTRACT:
    sprintf(namptr, "_X");
    break;
    case CONTIN:
    sprintf(namptr, "_C");
    break;
    default:
    sprintf(namptr, "_%1d", point);
    break;
}
/* fill fnam with zeros in all blank spaces */
zero_fill(fnam);
}
/*------------------------------------------End: Event_Name_p-----------------*/

/*--------------------------------Start: zero_fill-----------------------*/

zero_fill(string)
  char* string;
  
{
    int i;
    for(i=0; i< strlen(string); i++) 
     if(string[i] == ' ') string[i] = '0';
     
}

/*-----------------------------------End: zero_fill----------------------*/

/*---------------------------------------Start: Event_Header----------------*/
/* Create event-header                                                      */
/*  event time is passed in the date structure rec_time, and the header
    is returned in head                                                     */
/*--------------------------------------------------------------------------*/
Event_Header(rec_time, nbuf, head,par,ptr)
  struct date* rec_time;
  int nbuf;                       /* number of buffers                     */
  char* head;
  int par;
  smod* ptr;
{
int         start_index;            /* Index building headers               */
char* hdptr = head;                 /* text pointer to header   */  
int ll,jj,kk;                            /* counters */
int newblk;
/*printf("PAR=%d\n",par);*/
strncpy(hdptr, ptr->name, 3);       /* Put in station code    */
hdptr+=16;

sprintf(hdptr, "%2d ", rec_time->year);
hdptr+=3;

sprintf(hdptr,"%3d ", rec_time->doy);
hdptr+=4;

sprintf(hdptr,"%2d %2d     ",rec_time->month, rec_time->day);
hdptr+=9;
sprintf(hdptr,"%2d %2d %6.3f",rec_time->hour,rec_time->min,
                              rec_time->secs);

hdptr = head + 62;
if(ptr->nchstr[par] == ptr-> nchan)
{
    sprintf(hdptr, "%2d", ptr->nchan);
}else{
    sprintf(hdptr, "%2d", ptr->nchstr[par]);
}
/*printf("Here 1a\n");*/
/* Now need to put in the sample rate information */

/* SAMPLERATE start in position 64  */
hdptr = head+64;                                        
sprintf(hdptr, "%16.7f", ptr->srate);


/* find out the data type SHORT or INT */
/* NUMBER OF SAMPLES EACH CHANNEL PER EVENT + 2 OR 4 BYTE                   */
hdptr = head + 80;
if (find_data_type(ptr) == SHORT)
  sprintf(hdptr, "%8d       %1d",ptr->nsamp*nbuf, 2);
else
  sprintf(hdptr, "%8d       %1d",ptr->nsamp*nbuf, 4);



/* switch to indicate PC version */

hdptr=head+93;
sprintf(hdptr,"%1d",2);

/* UNIT NUMBER + BYTES PER BUFFER                                           */  
hdptr = head + 96;
if(ptr->nchstr[par] == ptr-> nchan)
{
    sprintf(hdptr,"%2d      %8d",ptr->unit,ptr->reclength);
}else{
    sprintf(hdptr,"%2d      %8d",ptr->unit,ptr->newrl[par]);
}           
/*printf("Here 1b\n");*/
/* BYTES IN HEADER  + NUMBER OF SAMPLES PER CHANNEL                         */
hdptr = head + 112;
/*printf("nchstr[par] %d nchan %d\n",ptr->nchstr[par],ptr->nchan);*/
if(ptr->nchstr[par] == ptr->nchan)
{
    newblk=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
}else{
    newblk=(SIZ_EV_HDR/ptr->newrl[par]+1)*ptr->newrl[par];
}   
sprintf(hdptr,"%5d     %6d",newblk,ptr->nsamp);
if(ptr->nchstr[par] == ptr->nchan)     /* all channels ?                    */
{
    kk=160;
    for(jj=0;jj<ptr->nchan;jj++)
    {
        for(ll=jj*4;ll<(jj*4)+4;ll++)
        {
            head[kk]=ptr -> statio[ll];
            head[kk+4]=ptr->compon[ll];
            kk=kk+1;
        }
    kk=kk+4;
    }
}else{                                 /* selected channels                 */
    kk=160;
    for(jj=0;jj<ptr->nchan;jj++)
    {
        if(ptr->store[par][jj] == 1)
        {
            for(ll=jj*4;ll<(jj*4)+4;ll++)
            {
                head[kk]=ptr -> statio[ll];
                head[kk+4]=ptr->compon[ll];
                kk=kk+1;
            }
            kk=kk+4;
        }
    }
}
/*printf("Here 1d\n");*/
/* remove all the NULL characters that have been put in the string by the 
   sprintf statements */
   
remove_null( head, SIZ_EV_HDR); 
}
/*----------------------------------------------End: Event_Header-----------*/


/*----------------------------------------------Start: Time_Conv------------*/
/* Convert and edit timestring                                              */
/*--------------------------------------------------------------------------*/
Time_Conv(input, event_time, ptr)
  int* input;
  struct date* event_time;
  smod* ptr;
{
char    time[15],date[15],doys[5],car;
int data_type;        /* whether to interpret the data as short or int */

data_type = find_data_type(ptr);
if(data_type == SHORT)
{
/*   sscanf((short*)input,"%s %s %s",time,date,doys);*/
}else{
   sscanf((char*)input,"%s %s %s",time,date,doys);
}
sscanf(time,"%2d%c%2d%c%f6.3\n",&(event_time->hour),&car,
                                &(event_time->min),&car,
                                &(event_time->secs));
sscanf(date,"%2d%c%2d%c%2d\n",&(event_time->day),&car,
                              &(event_time->month),&car,
                              &(event_time->year));
sscanf(doys,"%3d\n",&(event_time->doy));
}
/*------------------------------------------------------Stop: Time_Conv-----*/

/*-----------------------------Start: find_data_type()------------------*/
int find_data_type(ptr)
  smod *ptr;
{
     return LONG;
}
/*-----------------------------End: find_data_type()------------------*/

/*-----------------------------------Start: remove_null()-----------------*/
/* makes sure that all intermediate nulol characters are removed          */

remove_null(string, length)
 char* string;
 int length;
{
    int i;
    for(i=0; i<length; i++)
      if(string[i] == NULL) string[i] = ' ';
    string[length] = NULL;
    
}

/*--------------------------------------End: remove_null()----------------*/


/*------start: first_block(ptr, newblk)-------------------------------------*/

/****************************************************************************/
/* Position for first data record is computed so that the length of the     */
/* header is always a multiple of the recordlength >= 1024                  */
/****************************************************************************/
next_block(ptr, newblk, blocknum, par)
smod*    ptr;
int*     newblk;
int      blocknum;
int      par;                          /* current parameter set             */
{ 
if(ptr->nchstr[par] == ptr-> nchan)
{
    *newblk=(SIZ_EV_HDR/ptr->reclength+blocknum)*ptr->reclength;
}else{
    *newblk=(SIZ_EV_HDR/ptr->newrl[par]+blocknum)*ptr->newrl[par];
    }
}
/*------------------------------------------------------Start:Assign_dpx----*/

int Assign_dpx(dpx_no,detfile)
   int* dpx_no;
   char* detfile;
{
FILE        *in;                  /* Pointer to detnr-file                  */

if((int)(in = fopen(detfile,"r+")) == ERROR)
         return (ERROR);                    /* Open detnr-file              */
fscanf(in,"%d",dpx_no);                     /* Read old detection number    */
*dpx_no = *dpx_no + 1;                      /* Update detection number      */
rewind(in);                                 /* Rewind file                  */
fprintf(in,"%8d",*dpx_no);                  /* Write new detection-number   */
fclose(in);                                 /* Close file                   */
return (*dpx_no);
}
/*------------------------------------------------------End:Assign_dpx-----*/







/*-----------------------------Start: Write_info()----------------------*/
Write_info(string, dpx, dur,rec, point,ptr)
  char* string;
  int dpx;
  float dur;
  struct date* rec;
  int point;
  smod* ptr;
{
 int beamnr = 99;                         /* Dummy                        */
 int filternr= 99;                       /* Dummy                        */
 float freq  =   1.0;                        /* Dummy                        */
 float vel   =   2.0;                        /* Dummy                        */
 float azi   =   3.0;                        /* Dummy                        */
 float pwr   =   4.0;                        /* Dummy                        */ 

 float rsta = 0;
 float rlta = 0;
 float rampl = 0;
 int detsmp = 0;
 int detblk = 0;

if(point>-1){
 rsta = ptr->qsta[point];
 rlta = ptr->qlta[point];
 rampl = ptr->qamp[point];
 detsmp = ptr->qfsmp[point];
 detblk = ptr->qfblk[point]/ptr->reclength;
 }

sprintf(string,"%16d%4d%4d%4d%4d%4d%4d%8.3f%8d%8d%16.3f\
%16.3f%16.3f%16.3f%8.3f%8.3f%8.3f%8.3f%16d%8d%8d",\
dpx,rec->year,rec->doy,rec->month,rec->day,rec->hour,
rec->min,rec->secs,beamnr,\
filternr,dur,rsta,rlta,rampl,freq,vel,azi,pwr,detblk,0,detsmp);

string[192]=32;                         /* Set NUL byte to blank after write*/
}
/*-----------------------------End: Write_info()----------------------*/

/*---------------------------------Start: Det_record()----------------------*/

Det_record(output, dpx, detfile, ptr)
char*  output;                         /* detection record                  */
int    dpx;                            /* detection number                  */
char*  detfile;                        /* detection file name               */
smod*  ptr;                            /* pointer to shared region          */

{
FILE*  dets;                           /* pointer to detection file         */
int    bytes;                          /* return value after write          */
int    outblk;                         /* disk adress detection record      */

outblk = (dpx % EVTSIZ) * EVTREC;      /* compute disk adress detection rec */

/*--------------------------------------------------------------------------*/
/* Write detection record                                                   */
/*--------------------------------------------------------------------------*/

dets=fopen(detfile,"r+");
/*printf("\ndet_record: outblk is: %d", outblk);*/
fseek(dets,outblk,0);
bytes=fwrite(output,EVTREC,1,dets);
if(bytes==0)
    clearerr(dets);
fclose(dets);
}
/*-----------------------------------End: Det_record()----------------------*/

/*--------------------------------------------Start:Blank-------------------*/
/* Puts the number of blanks specified by length in the string argument     */
Blank(string, length)
  char *string;
  int length;
{
    int i;
    for(i=0;i<length;i++){
        string[i] = 32;
    }
    string[length] = 0;
}
/*--------------------------------------------Stop:Blank--------------------*/
  

