/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:31/05/00                       */
/* Program:DIG_EDN                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 01/06/00 8.22 t.u. new driver for Earth Data PS2400, new model           */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from ED24. Determine start of buffers   */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>


/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
#define numfwrite(a1,a2) numwrite= fwrite(a1,1,a2,in)
int           kval;
int           count;
int chk;
unsigned char *bp;
unsigned char *byt_ptr;
unsigned char *buf_ptr;
int block_length;
int      dec12,dec10;
unsigned char byt[1500];               /* Input buffer                      */
unsigned char cmd1[20];
unsigned char cmd2[20];
unsigned char cmd3[20];
unsigned char cmd4[20];
unsigned char cmd5[20];
unsigned char cmd6[20];
int           vent;
int           ba;
int           retu;
int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           numwrite;                /* write definition                  */
int           err_cnt;                 /* error counter                     */
int           fd;                      /* return value open share           */
smod          *ptr;                    /* pointer to shared region          */
main()
{
int			  ret_val;				   /* return value for ED3 function	    */
int           slow;
count=0;
err_cnt=0;                             /* set error counter to zero         */
slow=0;

/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_EDN_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_EDN_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
n1:
system("stty +raw -ihflow -osflow baud=19200 </dev/ser1");
in=fopen("/dev/ser1","r+");
if(in == 0)
{
  printf("Cant open serial line at 19200\n");
  exit(0);
}
retu=Sync();
if(retu==1)
{
  printf("DIG_EDN   : Receiving data from digitizer !\n");
  goto found;
}else{
  printf("Not receiving sync\n");
  exit(0);
}

found:
retu=Sync();
if(retu==1)
{
  printf("DIG_EDN   : Receiving data from digitizer !\n");
  ret_val = ED_SR();
}else{
  printf("cant find correct sync\n");
  exit(0);
}

  retu=Sync();
  ret_val = ED_NC();

  retu=Sync();
  ret_val = ED_BS();

  retu=Sync();
  ret_val = ED_BB();


}


COM()
{
  switch(ptr->rate)
  {
    case 50:
    cmd1[0]=0x24; // $
    cmd1[1]=0x53; // S
    cmd1[2]=0x46; // F
    cmd1[3]=0x3;  // 3
    cmd1[4]=0xc;  // 12

    cmd2[0]=0x24; // $
    cmd2[1]=0x53; // S
    cmd2[2]=0x46; // F
    cmd2[3]=0x4;  // 4
    cmd2[4]=0x5;  // 5

    break;

    case 100:
    cmd1[0]=0x24; // $
    cmd1[1]=0x53; // S
    cmd1[2]=0x46; // F
    cmd1[3]=0x3;  // 3
    cmd1[4]=0x3;  // 3

    cmd2[0]=0x24; // $
    cmd2[1]=0x53; // S
    cmd2[2]=0x46; // F
    cmd2[3]=0x4;  // 4
    cmd2[4]=0xa;  // 10

    break;

    case 75:
    cmd1[0]=0x24; // $
    cmd1[1]=0x53; // S
    cmd1[2]=0x46; // F
    cmd1[3]=0x3;  // 3
    cmd1[4]=0x4;  // 4

    cmd2[0]=0x24; // $
    cmd2[1]=0x53; // S
    cmd2[2]=46;   // F
    cmd2[3]=0x4;  // 4
    cmd2[4]=0xa;  // 10
    break;
  }
  cmd3[0]=0x24; // $ Write settings to ROM
  cmd3[1]=0x52; // R
  cmd3[2]=0x57; // W

  cmd4[0]=0x24; // $ Restore Default ROM
  cmd4[1]=0x52; // R
  cmd4[2]=0x44; // D

  cmd5[0]=0x24; // $ Set Baud Rate
  cmd5[1]=0x42; // B
  cmd5[2]=0x41; // A
  cmd5[3]=0x0;  // 0
  cmd5[4]=0x4;  // 4

  cmd6[0]=0x24; // $ Confirm Baud Rate Settings
  cmd6[1]=0x4f; // O
  cmd6[2]=0x4b; // K
}

ED_SR()
{

byt_ptr=byt;
block_length = 200;                    /* assume this for now */
numfread(byt_ptr,block_length);         /* read in the header = 192 bytes
									    *   less the 1st 2 bytes (already read)
										*   plus the data header 8 bytes
										*   plus the data (75Hz, 3C, 3bytes/samp) = 675 bytes
									    *   plus the checksum 12 bytes */
if(numread<block_length)               /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
dec12=(int)0;
dec12  = (byt[53] << 8) | byt[52];

dec10=(int)0;
dec10  = (byt[55] << 8) | byt[54];
chk=-1;
if(byt[50]==12 && byt[52]==5)
  chk=50;
if(byt[50]==4 && byt[52]==10)
  chk=75;
if(byt[50]==3 && byt[52]==10)
  chk=100;
printf("Samplerate is: %4d\n",chk);
return(1);
}

ED_NC()
{
int nch;
byt_ptr=byt;
block_length = 200; /* assume this for now */

numfread(byt_ptr,block_length);       
if(numread<block_length)               /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
nch = (byt[43] << 8) | byt[42];
	printf(" no of components in digitiser %d\n", nch);
return(0);
}
int    ED_BS()
{
static int block_length;
int bs;
unsigned char *byt_ptr;
char time_inc[20];
int chk;
byt_ptr=byt;
block_length = 200; /* assume this for now */


numfread(byt_ptr,block_length);         /* read in the header = 192 bytes
									    *   less the 1st 2 bytes (already read)
										*   plus the data header 8 bytes
										*   plus the data (75Hz, 3C, 3bytes/samp) = 675 bytes
									    *   plus the checksum 12 bytes */
if(numread<block_length)               /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
bs = (byt[47] << 8) | byt[46];
printf("Bytes per sample: %4d\n",bs);
return 0;
}
int    ED_BB()
{
static int block_length;
int bs;
unsigned char *byt_ptr;
long DAT_s;
int dbl;
byt_ptr=byt;
block_length = 200; /* assume this for now */
dbl=ptr->nchan * ptr->rate *3;

numfread(byt_ptr,block_length);         /* read in the header = 192 bytes
									    *   less the 1st 2 bytes (already read)
										*   plus the data header 8 bytes
										*   plus the data (75Hz, 3C, 3bytes/samp) = 675 bytes
									    *   plus the checksum 12 bytes */
if(numread<block_length)               /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
DAT_s = (long)0;
DAT_s = (byt[197] << 24) | (byt[196] << 16) | (byt[195] << 8) | byt[194];
printf("Bytes per block %4d\n",DAT_s);
return(0);
}
ED_BA()
{
  system("stty +raw -ihflow -osflow baud=19200 </dev/ser1");
  in=fopen("/dev/ser1","r+");
  if(in == 0)
  {
    printf("Cant open serial line at 19200\n");
    exit(0);
  }
  bp=cmd5;
  numfwrite(bp,5);
  printf("NUMWRITE= %d\n",numwrite);
  fflush(in);
  fclose(in);

}
ED_RD()
{
  system("stty +raw -ihflow -osflow baud=19200 </dev/ser1");
  in=fopen("/dev/ser1","r+");
  if(in == 0)
  {
    printf("Cant open serial line at 19200\n");
    exit(0);
  }
  bp=cmd4;
  numfwrite(bp,3);
  printf("NUMWRITE= %d\n",numwrite);
  fflush(in);
  fclose(in);
}
ED_RW()
{
  system("stty +raw -ihflow -osflow baud=19200 </dev/ser1");
  in=fopen("/dev/ser1","r+");
  if(in == 0)
  {
    printf("Cant open serial line at 19200\n");
    exit(0);
  }
  bp=cmd3;
  numfwrite(bp,3);
  printf("NUMWRITE= %d\n",numwrite);
  fflush(in);
  fclose(in);
}
ED_SF()
{
  bp=cmd1;
  numfwrite(bp,5);
//  printf("NUMWRITE= %d\n",numwrite);
  fflush(in);
  bp=cmd2;
  numfwrite(bp,5);
//  printf("NUMWRITE= %d\n",numwrite);
  fflush(in);
}
ED_OK()
{
  system("stty +raw -ihflow -osflow baud=19200 </dev/ser1");
  in=fopen("/dev/ser1","r+");
  if(in == 0)
  {
    printf("Cant open serial line at 19200\n");
    exit(0);
  }
  bp=cmd6;
  numfwrite(bp,3);
  printf("NUMWRITE= %d\n",numwrite);
  fflush(in);
  fclose(in);
}
Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
        } while(sync_value != 0x4d4f);
return(1);
/*putchar(0x07);*/
}
