/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:12/01/98                       */
/* Program:CHK_PAR                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program checks for basic errors in the parameter setting            */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                          Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program check for basic errors in parameter setting.

Usage:       chk_par (no parameters)
#endif
main ()
{
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */
int      errors;                            /* error counter                */
int      warnings;                          /* warning counter              */
float    ids_del;                           /* default delay IDS-24         */
float    das1200_del;                       /* default delay DAS1202        */
float    das1800_del;                       /* default delay DAS1202        */
float    rd3_del;                           /* default delay RD-3           */
float    rd6_del;                           /* default delay RD-6           */
float    gbv316_del;                        /* default delay GBV-316        */
float    gbv316m_del;                       /* default delay GBV-316 multis */
float    ed_del;
float    ro_ca_324;                         /* default delay RO+CA+324      */
int      ans;
int      i;
int      tot_rec;
ids_del     = -0.120;
das1200_del = -0.95;
das1800_del = -0.95;
rd3_del     = -2.2;
rd6_del     = -2.2;
gbv316_del  = -2.01;
gbv316m_del = -2.01;
ed_del      = 0.0;
ro_ca_324   =-1.0;
errors=0;
warnings=0;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"CHK_PAR_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CHK_PAR_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
//system("/home/seislog/cls");
switch(ptr->unit)
{
/****************************************************************************/
/*                        GeoSys/UiB GBV-316                                */
/****************************************************************************/
    case 10:
    printf("SEISLOG set up to use GeoSys/UiB GBV-316 digitizer.\n");
    printf("---------------------------------------------------\n");
    if(ptr->nchan > 3 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1,2 or 3\n");
        errors++;
    }
    if(ptr->mintrg[0] > 3)
    {
        printf("Warning: With minimum number of triggers set to more than 3,\n"); 
        printf("         SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("Warning: Check that GPS is connected\n");
        warnings++;
    }
    if(ptr->clktyp != 8)
    {
        if(ptr->delay != gbv316_del)
        {
            printf("Warning: Delay for GBV-316 is normally -x.x seconds\n");
            warnings++;
        }
    }
    if(ptr->bd_dig != 38400)
    {
        printf("ERROR  : Baudrate for GBV-316 should normally be 38400\n");
        errors++;
    }
    break;
/****************************************************************************/
/*                  DAS1202 16/32 Metrabyte (1 or 2 boards)                 */
/****************************************************************************/
    case 30:
    printf("SEISLOG set up to use DAS1202 A/D board(s).\n");
    printf("-------------------------------------------\n");
    if(ptr->nchan > 32 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1-32\n");
        errors++;
    }
    if(ptr->nchan > 16)
    {
        printf("Warning: You have specified more than 16 channels.\n");
        printf("         Be sure that you have 2 boards installed.\n");
        warnings++;
    }
    if(ptr->nchan <= 16)
    {
        if(ptr->mintrg[0] > 16)
        {
            printf("Warning: With minimum number of triggers set to more than 16,\n"); 
            printf("         SEISLOG will never trigger\n");
            warnings++;
        }
    }else{
        if(ptr->mintrg[0] > 32)
        {
            printf("Warning: With minimum number of triggers set to more than 32,\n"); 
            printf("         SEISLOG will never trigger\n");
            warnings++;
        }
    }
    if(ptr->clktyp == 8)
    {
        printf("ERROR  : DAS1200 does not support timing by digitizer\n");
        errors++;
    }
    if(ptr->delay == 0.0)
    {
        ptr->delay=das1200_del;
//        printf("Warning: Delay for DAS1202 is normally -0.95 seconds\n");
        printf("Time delay: %f Default set by system\n",ptr->delay);
        warnings++;
    }
    if(ptr->prt_clk < 1 || ptr->prt_clk > 8)
    {
        printf("Warning: Port number must be from 1-8\n");
        warnings++;
    }
    switch(ptr->bd_clk)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    break;
/****************************************************************************/
/*                  DAS1802 16/32 Metrabyte                                 */
/****************************************************************************/
    case 36:
    printf("SEISLOG set up to use DAS1802 A/D board(s).\n");
    printf("-------------------------------------------\n");
    if(ptr->nchan > 64 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1-64\n");
        errors++;
    }
    if(ptr->mintrg[0] > ptr->nchan)
    {
            printf("Warning: With minimum number of triggers set to more than tot channel,\n"); 
            printf("         SEISLOG will never trigger\n");
            warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("ERROR  : DAS1800 does not support timing by digitizer\n");
        errors++;
    }
    if(ptr->delay == 0.0)
    {
        ptr->delay=das1800_del;
//        printf("Warning: Delay for DAS1802 is normally -0.95 seconds\n");
        printf("Time delay: %f Default set by system\n",ptr->delay);
        warnings++;
    }
    if(ptr->prt_clk < 1 || ptr->prt_clk > 8)
    {
        printf("Warning: Port number must be from 1-8\n");
        warnings++;
    }
    switch(ptr->bd_clk)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    break;
/****************************************************************************/
/*                        IDS-24 Terra Technology                           */
/****************************************************************************/
    case 31:
    printf("SEISLOG set up to use IDS-24 digitizer from Terra Technology.\n");
    printf("-------------------------------------------------------------\n");
    if(ptr->nchan > 3 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1,2 or 3\n");
        errors++;
    }
    if(ptr->mintrg[0] > 3)
    {
        printf("Warning: With minimum number of triggers set to more than 3,\n"); 
        printf("         SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("ERROR  : IDS-24 supported by SEISLOG does not support\n");
        printf("         digitizer timing.\n");
        errors++;
    }
    if(ptr->delay == 0.0)
    {
        ptr->delay=ids_del;
//        printf("Warning: Delay for IDS-24 is normally -0.120 seconds\n");
        printf("Time delay: %f Default set by system\n",ptr->delay);
        warnings++;
    }
    if(ptr->prt_clk < 1 || ptr->prt_clk > 8)
    {
        printf("Warning: Port number must be from 1-8\n");
        warnings++;
    }
    switch(ptr->bd_clk)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    switch(ptr->bd_dig)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    break;
/****************************************************************************/
/*                  GeoSys 12 channel digitizer Llolleo                     */
/****************************************************************************/
    case 32:
    printf("SEISLOG set up to use GeoSys 12 channel (Llolleo).\n");
    printf("--------------------------------------------------\n");
    if(ptr->nchan > 12 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1-12\n");
        errors++;
    }
    if(ptr->mintrg[0] > 12)
    {
        printf("Warning: With minimum number of triggers set to more than 12,\n"); 
        printf("         SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->prt_clk < 1 || ptr->prt_clk > 8)
    {
        printf("Warning: Port number must be from 1-8\n");
        warnings++;
    }
    switch(ptr->bd_dig)
    {
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 38400\n");
        warnings++;
        break;
    }
    break;
/****************************************************************************/
/*                  Multiserial GeoSys/UiB GBV-316                          */
/****************************************************************************/
    case 33:
    printf("SEISLOG set up to use 1 or more GeoSys/UiB GBV-316 digitizers.\n");
    printf("--------------------------------------------------------------\n");
    if(ptr->nchan != ptr->totchan)
    {
        printf("ERROR  : Number of channels MUST be equal to total number\n");
        printf("         specified in parameterfile for multiserial support\n");
        errors++;
    }
    if(ptr->mintrg[0] > ptr->totchan)
    {
        printf("Warning: With minimum number of triggers set to more than %2d,\n",ptr->totchan); 
        printf("         SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("Warning: Check that GPS is connected\n");
        warnings++;
    }
    if(ptr->clktyp == 1 || ptr->clktyp == 2)
    {
        if(ptr->delay != gbv316m_del)
        {
            printf("Warning: Delay for GBV-316 is normally -2.1 seconds\n");
            warnings++;
        }
    }
    if(ptr->bd_dig != 38400)
    {
        printf("ERROR  : Baudrate for GBV-316 should normally be 38400\n");
        errors++;
    }
    break;
/****************************************************************************/
/*                  Multiserial Earth Data PS2400                           */
/****************************************************************************/
    case 34:
    printf("SEISLOG set up to use 1 or more Earth Data PS2400 digitizers.\n");
    printf("-------------------------------------------------------------\n");
    if(ptr->nchan != ptr->totchan)
    {
        printf("ERROR  : Number of channels MUST be equal to total number\n");
        printf("         specified in parameterfile for multiserial support\n");
        errors++;
    }
    if(ptr->mintrg[0] > ptr->totchan)
    {
        printf("Warning: With minimum number of triggers set to more than %2d,\n"); 
        printf("         SEISLOG will never trigger\n",ptr->totchan);
        warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("Warning: Check that GPS is connected\n");
        warnings++;
    }
    if(ptr->bd_dig != 19200)
    {
        printf("ERROR  : Baudrate for IDS-24 should normally be 19200\n");
        errors++;
    }
    break;
/****************************************************************************/
/*                        Nanometrics HRD-24                                */
/****************************************************************************/
    case 40:
    printf("SEISLOG set up to use Nanometrics HRD-24.\n");
    printf("-----------------------------------------\n");
    if(ptr->nchan > 3 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1-3\n");
        errors++;
    }
    if(ptr->mintrg[0] > 3)
    {
        printf("Warning: With minimum number of triggers set to more than 3,\n"); 
        printf("         SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->clktyp != 8)
    {
        printf("ERROR  : Nanometrics HRD-24 always use timing by digitizer\n");
        errors++;
    }
    if(ptr->delay != 0.0)
    {
        printf("Warning: Time correction for Nanometrics HRD-24 has no meaning\n");
        warnings++;
    }
        switch(ptr->rate)
        {
            case 50:
            if(ptr->bd_dig != 9600)
            {
                printf("Warning: Baudrate for HRD-24 should normally be 9600 with 50 Hz sampling\n");
                warnings++;
            }
            break;
            case 100:
            if(ptr->bd_dig != 19200)
            {
                printf("Warning: Baudrate for HRD-24 should normally be 19200 with 100 Hz sampling\n");
                warnings++;
            }
            break;
            default:
            printf("ERROR  : Samplerate must be 50 Hz or 100 Hz\n");
            errors++;
        }           
    break;
/****************************************************************************/
/*                        Nanometrics RD-3                                  */
/****************************************************************************/
    case 53:
    printf("SEISLOG set up to use Nanometrics RD-3.\n");
    printf("---------------------------------------\n");
    if(ptr->nchan > 3 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1-3\n");
        errors++;
    }
    if(ptr->mintrg[0] > 3)
    {
        printf("Warning: With minimum number of triggers set to more than 3,\n"); 
        printf("         SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("ERROR  : Nanometrics does not support timing by digitizer\n");
        errors++;
    }
    if(ptr->delay == 0.0)
    {
        ptr->delay=rd3_del;
//        printf("Warning: Delay for Nanometrics RD-3 is normally -2.2 seconds\n");
        printf("Time delay: %f Default set by system\n",ptr->delay);
        warnings++;
    }
    if(ptr->bd_dig != 9600)
    {
        printf("ERROR  : Baudrate for RD-6 should normally be 9600\n");
        errors++;
    }
    if(ptr->prt_clk < 1 || ptr->prt_clk > 8)
    {
        printf("Warning: Port number must be from 1-8\n");
        warnings++;
    }
    switch(ptr->bd_clk)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    switch(ptr->bd_dig)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    break;
/****************************************************************************/
/*                        Nanometrics RD-6                                  */
/****************************************************************************/
    case 56:
    printf("SEISLOG set up to use Nanometrics RD-6.\n");
    printf("---------------------------------------\n");
    if(ptr->nchan > 6 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1-6\n");
        errors++;
    }
    if(ptr->mintrg[0] > 6)
    {
        printf("Warning: With minimum number of triggers set to more than 6,\n"); 
        printf("         SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("ERROR  : Nanometrics does not support timing by digitizer\n");
        errors++;
    }
    if(ptr->delay == 0.0)
    {
        ptr->delay=rd6_del;
//        printf("Warning: Delay for Nanometrics RD-6 is normally -2.2 seconds\n");
        printf("Time delay: %f Default set by system\n",ptr->delay);
        warnings++;
    }
    if(ptr->bd_dig != 9600)
    {
        printf("ERROR  : Baudrate for RD-6 should normally be 9600\n");
        errors++;
    }
    if(ptr->prt_clk < 1 || ptr->prt_clk > 8)
    {
        printf("Warning: Port number must be from 1-8\n");
        warnings++;
    }
    switch(ptr->bd_clk)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    switch(ptr->bd_dig)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    break;
/****************************************************************************/
/*                        Earth Data                                        */
/****************************************************************************/
    case 63:
    printf("SEISLOG set up to use Earth Data Digitizer.\n");
    printf("-------------------------------------------\n");
    if(ptr->nchan > 3 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1-3\n");
        errors++;
    }
    if(ptr->mintrg[0] > 3)
    {
        printf("Warning: With minimum number of triggers set to more than 3,\n"); 
        printf("         SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("Warning: SEISLOG assumes timing by digitizer\n");
        warnings++;
    }
    if(ptr->delay != ed_del)
    {
        printf("Warning: Delay for Earth Data is normally xx seconds\n");
        warnings++;
    }
    if(ptr->prt_clk < 1 || ptr->prt_clk > 8)
    {
        printf("Warning: Port number must be from 1-8\n");
        warnings++;
    }
    switch(ptr->bd_clk)
    {
        case 9600:
        break;
        default:
        printf("Warning: Baudrate must be 9600\n");
        warnings++;
        break;
    }
    switch(ptr->bd_dig)
    {
        case 19200:
        break;
        default:
        printf("Warning: Baudrate must be 19200 \n");
        warnings++;
        break;
    }
    break;
/****************************************************************************/
/*                        Synthetic Data (testing)                          */
/****************************************************************************/
    case 96:
    printf("SEISLOG set up to use Synthetic Data.\n");
    printf("-------------------------------------\n");
    if(ptr->nchan > 64 || ptr->nchan < 1)
    {
        printf("ERROR  : Number of channels has to be 1-54\n");
        errors++;
    }
    if(ptr->mintrg[0] > ptr->nchan)
    {
        printf("Warning: With minimum number of triggers set to more than\n"); 
        printf("         total number of channels, SEISLOG will never trigger\n");
        warnings++;
    }
    if(ptr->clktyp == 8)
    {
        printf("ERROR  : This driver does not support digitizer timing\n");
        errors++;
    }
    if(ptr->prt_clk < 1 || ptr->prt_clk > 8)
    {
        printf("Warning: Port number must be from 1-8\n");
        warnings++;
    }
    switch(ptr->bd_clk)
    {
        case 2400:
        case 4800:
        case 9600:
        case 19200:
        case 38400:
        break;
        default:
        printf("Warning: Baudrate must be 2400,4800,9600,19200 or 38400\n");
        warnings++;
        break;
    }
    break;
}

tot_rec=PIPSIZ*4/ptr->reclength;
for(i=0;i<ptr->active;i++)
{
    if(ptr->maxbuf[i] > tot_rec)
    {
        ptr->maxbuf[i]=tot_rec;
        printf("Your MAXIMUM RECORDING in parameter set %d is set to %5d\n",
        i,tot_rec);
    }    
}

printf("\n");
printf("Summary parameter check:\n");
printf("------------------------\n");
printf("Check encountered: %2d ERRORS and %2d Warnings.\n",errors,warnings);
if(errors==0 && warnings==0)
{
    printf("Parameters look OK, you may start the system.\n");
    goto slutt;
}
if(errors == 0 && warnings != 0)
{
    printf("Encountered 1 or more Warnings. You may start the system,\n");
    printf("but check once more to be sure.\n");
    goto slutt;
}
if(errors != 0 && warnings == 0)
{
    printf("Encountered 1 or more ERRORS !\n");
    printf("If you start the system, it will give unpredictable results !\n");
    putchar(0x07);
    goto slutt;
}
if(errors != 0 && warnings != 0)
{
    printf("Encountered both ERRORS and Warnings.\n");
    printf("If you start the system, it will give unpredictable results !\n");
    putchar(0x07);
}
slutt:
errors=0;
system("/home/seislog/dsk_inf");
if(ptr->reinit_flag==1)
{
    printf("\n                                WARNING\n");
 
    printf("\nParameters that affect the ringbuffersystem have been changed.\n");
    if(ptr->rngsystem > ptr->freespace)
    {
    printf("Your current parameters will try to allocate: %8d Mb on the disk\n",ptr->rngsystem);
    printf("You have available: %8d Mb\n",ptr->freespace);
    printf("Run PAR again and reduce the size of the ringbuffersystem !\n");
    exit(0);
}   
    printf("Its recommended to do a re-initialization now.\n");

    printf("\nDo you want to run RE_INITI now ?\n");
    printf("Enter YES or NO\n");
    ans=getch();
    if(ans == 'y' || ans == 'Y')
        system("/home/seislog/re_initi");
}
}
