/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:04/12/98                       */
/* Program:CHK_PPS                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Check if 1 PPS is present.                                               */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/name.h>
#include <i86.h>
#include <time.h>
#include <conio.h>
#include <sys/irqinfo.h>               /* needed for interrupt-routines     */
#include <sys/proxy.h>
#include <sys/kernel.h>                /* needed for interrupt-routines     */
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program checks if the 1 PPS pulse is present on the parallell port.

Usage:       chk_pps (no parameters)
#endif

pid_t             proxy;
pid_t             pxy;
unsigned int      cop;
volatile unsigned counter;             /* variable for interrupt routine    */
volatile int      cnt;                 /* for debugging                     */
volatile int      ms;                  /* for debugging                     */
int               fd;                  /* return value open share           */
volatile smod     *ptr;                /* pointer to shared region          */
/****************************************************************************/
/*      The Hardware Interrupt Handler 1PPS signal from clock               */
/*--------------------------------------------------------------------------*/
/* This interrupt handler is triggered by the 1PPS signal from the Garmin   */
/* GPS receiver through the line printer port. This interrupt is enabled    */
/* just before we can expect a 1 PPS, and then disabled. Each time triggered*/
/* the second counter is set to the correct second and the millisecond      */
/* counter is set to 0. A proxy is returned to the main program to disable  */
/* the interrupt.                                                           */
/****************************************************************************/
#pragma off( check_stack );
pid_t far handler()
{
    ptr->icmde=ptr->corr_sec;          /* adjust clock to correct second    */
    ptr->itmde=0;                      /* reset millisecond counter         */
    ms=0;                              /* for debugging                     */
    cnt++;                             /* for debugging                     */
    return(proxy);                     /* signal main to disable interrupt  */
}
#pragma on( check_stack);
/****************************************************************************/
/*      The Hardware Interrupt Handler counting milliseconds                */
/*--------------------------------------------------------------------------*/
/* This interrupt handler is connected up with the systems own timer, and   */
/* is not more accurate than the systems CPU time. However, it is accurate  */
/* enough to count to 1000 milliseconds.                                    */
/****************************************************************************/
#pragma off( check_stack );
pid_t far mil_handler()
{
    ms++;                              /* for debugging                     */
    ptr->itmde++;                      /* increment millisecond counter     */
    if(ptr->itmde == 1000)
    {
        ptr->itmde=0;                  /* reset millisecond counter to 0    */
        ptr->icmde++;                  /* increment second counter          */
    }
return(0);
}
#pragma on( check_stack);

main()
{
int      id;                           /* id for the 1 PPS interrupt handler*/
int      idd;                          /* id for the 1msec interrupt handler*/
int      k;
cnt=0;                                 /* set to zero initially             */
system("ticksize 1");                  /* set ticksize of system to 1 msec  */
setprio(getpid(),15);                  /* set the priority this process=14  */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    printf("CLK_PPS: Can't open shared memory.\n");
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    printf("CLK_PPS: Can't map shared memory.\n");
    exit(0);
}
if(ptr->clktyp==2 || ptr->clktyp==4)       /* do we have 1 pps support ?        */
    outp(0x37a,0x00);                      /* disable interrupt at PAR port */

/*--------------------------------------------------------------------------*/
/* set up a proxy for the interrupt handler to kick                         */
/*--------------------------------------------------------------------------*/
if((proxy = qnx_proxy_attach(0,0,0,0)) == -1)
{
    printf("CLK_PPS: Can't attach proxy.\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Attach the interrupt from the 1PPS from GPS                              */
/*--------------------------------------------------------------------------*/
if(ptr->clktyp==2 || ptr->clktyp==4)     /* if we have 1 pps support          */
{
    if(( id=qnx_hint_attach(7,&handler,FP_SEG(&counter)))==-1)
    {
        printf("CLK_PPS: Can't attach the 1 PPS interrupt.\n");
        exit(0);
    }
}
/*--------------------------------------------------------------------------*/
/* Attach the interrupt to count milliseconds                               */
/*--------------------------------------------------------------------------*/
if(( idd=qnx_hint_attach(0,&mil_handler,FP_SEG(&counter)))==-1)
{
    printf("CLK_PPS: Can't attach to the millisecond interrupt.\n");
    exit(0);
}

if(ptr->clktyp == 2 || ptr->clktyp==4)        /* 1 PPS support ?                   */
{
    for(k=0;k<10;k++)
    {
        outp(0x37a,0x10);              /* enable interrupt at par port      */
        Receive(proxy,0,0);            /* wait for signal from interrupt    */
        outp(0x37a,0x00);              /* disable interrupt at par port     */
        printf("The 1 PPS is there !\n");
//        printf("cnt=%4d ms=%4d\n",cnt,ms);
    }
    exit(0);
}
printf("Clock type: %d do not have 1 PPS support.\n");
}


