/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:13/01/99                       */
/* Program:CHK_SNC                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Check for synchronization characters from different digitizers           */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 13/01/99     Version 7.70
------------------------------------------------------------------------------
Description: Program to be used in trouble shooting. Looks for sync character
             on digitizer specified in parameter file. SEISLOG must be stopped
             before activated.

Usage:       chk_snc (no parameters)     
#endif

char stty_str[100];
char port_str[12];

unsigned char       byt[1500];         /* Input buffer                      */
unsigned char       *buf_ptr;
int                 i;
int                 n;
int                 fh;                /* i/o handler serial line           */
int                 sts_snc;           /* return value from Sync1           */
int                 fd;                /* return value open share           */
smod                *ptr;              /* pointer to shared region          */
main()
{
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_NAN_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CHK_SNC_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* open serial line where digitizer is connected                            */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_clk);
sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_clk,port_str);
system (stty_str);

if ((fd = open(port_str,O_RDONLY)) == ERROR)
{
    printf("CHK_SNC: Can't open serial line.\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Look for 10 sync characters in input data                                */
/*--------------------------------------------------------------------------*/
for(i=0;i<10;i++)
{
    sts_snc=Sync1();
    switch(sts_snc)
    {
        case 1:
        printf("SYNC found !\n");
        break;
        case 0:
        printf("TIMEOUT Nothing received from serial port\n");
        break;
        case -1:
        printf("ERROR reading data from serial port\n");
        break;
    }
}
}
Sync1()
{
register unsigned short sync_value = 0;
unsigned short sy;
switch(ptr->unit)
{
    case 10:
    sy=0x04742;
    break;

    case 31:
    sy=0x0a0d;
    break;

    case 53:
    case 56:
    sy=0xAA16;
    break;
}
buf_ptr = byt;
do
{
    n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
    switch(n)
    {
        case -1:
        return(-1);
        break;
        case 0:
        return(0);
        break;
        case 1:       
        sync_value = sync_value << 8 | *buf_ptr;
        break;
    }
} while(sync_value != sy);
return(1);
}
