/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:17/02/98                       */
/* Program:CHK_TAG                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program is started by EXT to find the current time span in the       */
/* ringbuffer system.                                                       */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <dirent.h>
#include <string.h>

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
char     names[32];              /* storage all ringbuffer names      */
char     ringb[6];
char     sts[2];
double   totsec;
int      reset[50];
int      nrec=0;
int      reco=0;
int      nfin=0;
int      nrst=0;
int      indx=0;
int      start,stopp;
int      cnt;
main()
{
FILE     *idx;
FILE     *rng;
DIR      *dirp;                        /* pointer to directory              */
struct   dirent *direntp;              /* structure for directory structure */
char     rngnam[50];                   /* complete ringbuffer name          */
int      tophdr;                       /* size of header                    */
int      bytes;
int      ind_min;                      /* index oldest ringbuffer           */
int      ind_max;                      /* index newest ringbuffer           */
int      first;
double   DIFF;
float    diff;
double   OLD;
double   MSECS;                        /* total seconds ringbuffer filename */
double   mint;                         /* smallest value indicating rngfile */
double   maxt;                         /* largest value indicating rngfile  */
double   save;
float    SEC;                          /* seconds floating point            */
int      YR,DOY,MTH,DAY,HR,MIN,SECL;   /* ringbuffer file date              */
char     idx_log[400];
int      hu;
int      lk;                           /* index                             */
int      buf[5000];
char     lager[5000];
char     tid[13];                      /* string for uptime                 */
char     minutes[10];                  /* storage for minutes from uptime   */
char     hours[10];                    /* storage for hours from uptime     */
char     days[10];                     /* storage for days from uptime      */
char     name[50];                     /* storage file name                 */
char     name1[50];                    /* storage oldest file name          */
char     fil_min[50];                  /* storage oldest file name          */
char     fil_max[50];                  /* storage newest file name          */
char     syscall[100];                 /* string to hold system command     */
char     timestring[200];
int      sw;                           /* switch to indicate type of file   */
char     dum;
char     arg;
int      no_files;                     /* number of files in directory      */
FILE     *up;                          /* pointer to file                   */
int      min;                          /* minutes uptime                    */
int      hr;                           /* hours uptime                      */
int      dg;                           /* days uptime                       */
char     se[30];
int      dur;                          /* uptime in minutes                 */
int      fr;
int      i;                            /* index                             */
int      newblk;
int      k;
int      opt;
char     num[10];
first=0;
OLD=0.0;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"CHK_TAG_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CHK_TAG_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
tophdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
/*--------------------------------------------------------------------------*/
/* Read all filenames in ringbuffer directory and find the oldest           */
/*--------------------------------------------------------------------------*/
idx=fopen("/home/buffer/idx_log","r+");    

for(i=0;i<ptr->nringbuf[0];i++)
{
    fseek(idx,i*320,SEEK_SET);
    fread(idx_log,320,1,idx);
    for(lk=0;lk<29;lk++)
        names[lk]=idx_log[lk+19];
    names[29]='\0';
    sts[0]=idx_log[87];
    sts[1]=idx_log[167];
    if(sts[0]=='1' && sts[1]=='3') sw=1;     /* Recording & Reset      */
    if(sts[0]=='2' && sts[1]=='3') sw=2;     /* Finished  & Reset      */
    if(sts[0]=='0' && sts[1]!='3') sw=3;     /* Empty                  */
    if(sts[0]=='1' && sts[1]!='3') sw=4;     /* Recording              */
    if(sts[0]=='2' && sts[1]!='3') sw=5;     /* Finished               */
    
    for(lk=0;lk<5;lk++)
        ringb[lk]=idx_log[lk+13];
    ringb[5]='\0';
    sscanf(names,"%4d%c%2d%c%2d%c%2d%2d%c%2d",
    &YR,&dum,&MTH,&dum,&DAY,&dum,&HR,&MIN,&dum,&SECL);
    SEC=(float)SECL;
    TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
    totsec=MSECS;
    switch(sw)
    {
        case 1:            /* RECORDING & RESET                 */
        printf("Status R %4d : RECORDING & RESET\n",i);
        first=0;
        goto next;
        break;
        case 2:
        printf("Status R %4d : FINISHED  & RESET\n",i);
        first=0;
        break;
        case 3:
        printf("Status R %4d : EMPTY\n",i);
        first=0;
        goto next;
        break;
        case 4:
        printf("Status R %4d : RECORDING\n",i);
        first=0;
        goto next;
        break;
        case 5:
//        printf("Status R %4d : FINISHED\n",i);
        break;
    }

    printf("%5d %c %c %s %s %12.1f\n",i,sts[0],sts[1],names,ringb,MSECS);
    sprintf(rngnam,"/home/buffer/%s\n",ringb);
    rngnam[18]='\0';
    rng=fopen(rngnam,"r");
    if(rng == NULL)
    {
        printf("Error opening ringbuffer\n");
        exit(0);
    }
    for(k=0;k<ptr->nmin_per_ring[0]*60;k++) /* check all time tags each buffer */
    {
        newblk=ptr->reclength*k+tophdr;
        fseek(rng,newblk,0);
        bytes=fread(buf,200,1,rng);
        if(bytes==0)
        {
            printf("Error reading RXXXX\n");
            exit(0);
        }
        sprintf(timestring,"%s\n",buf);
        se[0]=timestring[19];
        se[1]=timestring[20];
        se[2]=timestring[21];
        se[3]=timestring[22];
        sscanf(se,"%d",&YR);
        sscanf(timestring,"%2d%c%2d%c%f6.3%c%2d%c%2d%c",
        &HR,&dum,&MIN,&dum,&SEC,&dum,&DAY,&dum,&MTH);
        TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
        DIFF=MSECS-OLD;
        diff=(float)DIFF;
        if(diff > 1.3 || diff < 0.0)
        {
            if(first == 1)
            {
                switch(sw)
                {
                    case 1:            /* RECORDING & RESET                 */
                    printf("Status ringbuffer: RECORDING & RESET Time error\n");
                    break;
                    case 2:
                    printf("Status ringbuffer: FINISHED  & RESET Time error\n");
                    break;
                    case 3:
                    printf("Status ringbuffer: EMPTY Time error\n");
                    break;
                    case 4:
                    printf("Status ringbuffer: RECORDING  Time error\n");
                    break;
                    case 5:
                    printf("Status ringbuffer: FINISHED Time error\n");
                    break;
                }
                first=1;
            }
        }
        OLD=MSECS;                       
        first=1;
//getch();
//        printf("%s %10.1f\n",timestring,MSECS);
//        getch();
    }
    fclose(rng);
next:
lk=0;
//getch();
}
fclose(idx);
}



