/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:25/03/96                       */
/* Program:CLK_CPU                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program starts up the routine that controls the software clock CLK_PPS   */
/* sets the initial values seconds and milliseconds in the software clock   */
/* and keeps track of how long SEISLOG has been active.                     */
/*                                                                          */
/****************************************************************************/


#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <i86.h>
#include <time.h>
#include <conio.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program to supply timing when no GPS is availlable. Not accurate.

Usage:       clk_cpu (no parameters)
#endif

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

main()
{
int      cpu_id;
double   MSECS;
struct timespec start,sstart, stopp;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"CLK_CPU_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CLK_CPU_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
cpu_id=qnx_name_attach(0,"clk_cpu");
if(cpu_id == -1)
{
    printf("CLK_CPU attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* start up program to count milliseconds to keep software clock updated    */
/* If 1 PPS is supported, the clock will be synchronized by this puls       */
/*--------------------------------------------------------------------------*/
system("on -t /dev/con6 /home/seislog/clk_pps&");
if(ptr->min_dur != 0)                  /* activate minute mark on par port  */
{
//    system("on -t /dev/con6 /home/seislog/par_mrk &"); /* start minute marker process       */
//    system("on -t /dev/con6 /home/seislog/par_gps &"); /* start gps ok indicator par port   */
//    sleep(1);
}

ptr->displm=0;                         /* to zero initially for correct %   */
ptr->var05=0;                          /* to zero initially for correct %   */

sprintf(ptr->err_msg,"CLK_CPU: CPU clock selected");
errlog(ptr);

/*--------------------------------------------------------------------------*/
/* use current CPU time as initial value for GPS controlled internal clock  */
/*--------------------------------------------------------------------------*/
if(clock_gettime(CLOCK_REALTIME,&start)== -1) 
{
    perror("clock gettime" );
    exit( EXIT_FAILURE);
}
ptr->icmde=start.tv_sec;
ptr->itmde=start.tv_nsec/1000000;
MSECS=start.tv_sec+(float)ptr->itmde/1000.0;
/****************************************************************************/
/*                     entry point every nn second                          */
/****************************************************************************/
if(clock_gettime(CLOCK_REALTIME,&sstart)== -1)
{
    perror("clock gettime");
    exit(0);
}
ptr->TimFlg[0]=3;                  /* indicates CPU time                */

Next:
if(clock_gettime(CLOCK_REALTIME,&stopp)== -1)
{
    perror("clock gettime");
    exit(0);
}
ptr->oktime=(stopp.tv_sec - sstart.tv_sec);
ptr->alive[3]=1;                       /* tell I'm alive !                  */
if(ptr->clktyp==0)                     /* CPU clock                         */
{
   sleep(1);
   goto Next;
}
}
