/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:25/03/96                       */
/* Program:CLK_GNP                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 16/09/97 including new header info                                       */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/*                                                                          */
/****************************************************************************/


#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <i86.h>
#include <time.h>
#include <conio.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program reads timestring ($GPRMC) from Garmin GPS, internal CPU
             clock is updated when GPS has satellite fix. Program does not
             support synchronizing with 1 pulse per second.

Usage:       clk_gnp (no parameters)
#endif

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

unsigned char buffer[64];              /* Receive buffer for time-string    */
unsigned char buffer2[64];
char     stty_str[100];
char     port_str[12];
main()
{
int      k;
int      gps_id;
double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
struct   timespec sstart, stopp;
int      fh;                           /* file descriptor for serial device */
char     STS;
int      nor;                          /* return value from SYNC            */
                                       /* negative->ERROR                   */
                                       /* zero    ->timeout                 */
                                       /* positive->number of bytes recived */
struct   timespec start;               /* structure for setting system time */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"CLK_GNP_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CLK_GNP_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* compute number of seconds since 01/01/1970 00:00:00.00                   */
/*--------------------------------------------------------------------------*/
YR=1970;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSEC70);
/*--------------------------------------------------------------------------*/
/* start up program to count milliseconds to keep software clock updated    */
/* If 1 PPS is supported, the clock will be synchronized by this puls       */
/*--------------------------------------------------------------------------*/
system("on -t /dev/con6 /home/seislog/clk_pps&");

for(k=0;k<64;k++)                      /* initialize timestring buffers     */
{
    buffer[k]='.';
}
ptr->displm=0;                         /* to zero initially for correct %   */
ptr->var05=0;                          /* to zero initially for correct %   */
/*--------------------------------------------------------------------------*/
/* Set line /dev/ser2 to the correct baudrate for this particular clock.    */
/* The serial line used for the clock can NOT be started as a user terminal */
/* in the SYSINIT.1 file.                                                   */
/*--------------------------------------------------------------------------*/
sprintf(ptr->err_msg,"CLK_GNP: Garmin GPS 9600 baud, only time. No PPS");
errlog(ptr);
//system("stty baud=9600 < /dev/ser2");  /* set baudrate to 9600              */
sprintf(port_str,"/dev/ser%d",ptr->prt_clk);
sprintf(stty_str,"stty +raw -osflow baud=%d < %s\n",ptr->bd_clk,port_str);
system(stty_str);
if((fh=open(port_str,O_RDONLY))==ERROR) /* open /dev/ser2 for read only  */
{
    sprintf(ptr->err_msg,"CLK_GNP: Can't open /dev/ser2, clock:%d",ptr->clktyp);
    errlog(ptr);
    exit(0);
}
setprio(getpid(),15);                  /* set the priority this process=15  */
/*--------------------------------------------------------------------------*/
/* use current CPU time as initial value for GPS controlled internal clock  */
/*--------------------------------------------------------------------------*/
if(clock_gettime(CLOCK_REALTIME,&start)== -1) 
{
    perror("clock gettime" );
    exit( EXIT_FAILURE);
}
ptr->icmde=start.tv_sec;
ptr->itmde=start.tv_nsec/1000000;
//MSECS=start.tv_sec+(float)ptr->itmde/1000.0;
//ptr->corr_sec=ptr->icmde;              /* correct second on next 1PPS       */
/*--------------------------------------------------------------------------*/
/* convert floating point value of sectot since 1970 to date and time       */
/*--------------------------------------------------------------------------*/
//SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
//if(ptr->dbg==1)
//    printf("%2d:%2d:%6.3f %2d/%2d/%2d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
/****************************************************************************/
/*                     entry point every nn second                          */
/****************************************************************************/
if(clock_gettime(CLOCK_REALTIME,&sstart)== -1)
{
    perror("clock gettime");
    exit(0);
}
ptr->TimFlg[0]=0;                      /* initially CPU                     */
ptr->TimFlg[1]=0;                      /* initially counter for not reading */

Next:
if(clock_gettime(CLOCK_REALTIME,&stopp)== -1)
{
    perror("clock gettime");
    exit(0);
}
ptr->oktime=(stopp.tv_sec - sstart.tv_sec);
ptr->alive[3]=1;                       /* tell I'm alive !                  */
do
{
    nor=Sync(fh,buffer);               /* Find "MC" string in word "$GPRMC" */
    if(nor==0)
    {
        system("/home/seislog/rst_gps"); /* If supported, turn power off/on */
        sprintf(ptr->err_msg,"CLK_GNP: Timeout reading timestring");
        errlog(ptr);
        sprintf(ptr->err_msg,"CLK_GNP: Reset GPS");
        errlog(ptr);
    }        
}   while(nor <= 0);                   /* in data recived from clock        */

nor=read57(fh,buffer,&STS);            /* read rest of $GPRMC msg from clock*/
switch(nor)                            /* check return value                */
{
    case -1:                           /* ERROR reading time string         */
    goto Next;                         /* re-synchronize                    */
    break;

    case 0:                            /* TIMEOUT                           */
    system("/home/seislog/rst_gps");   /* If supported: turn power off/on   */
    system("/home/seislog/rst_gps"); /* If supported, turn power off/on */
    sprintf(ptr->err_msg,"CLK_GNP: Timeout reading timestring");
    errlog(ptr);
    sprintf(ptr->err_msg,"CLK_GNP: Reset GPS");
    errlog(ptr);
    sleep(1);
    goto Next;                         /* re-synchronize                    */
    break;

    case 1:                            /* characters received, and accepted */
    if(STS=='A')
    {
        ptr->TimFlg[0]=1;              /* SYNCHRONIZED                      */
        ptr->displm++;
        if(ptr->displm>=1000000)ptr->displm=0;
    }else{
        ptr->TimFlg[0]=2;              /* NOT SYNCHRONIZED                  */
    }
    break;                         

    case 2:                            /* characters received, NOT accepted */
    goto Next;                         /* re-synchronize                    */
    break;
}
/*--------------------------------------------------------------------------*/
/* if correct message, goon and interpret message                           */
/*--------------------------------------------------------------------------*/
rd_msg(ptr,buffer,&HR,&MIN,&SECL,&DAY,&MTH,&YR);
//printf("GPS: %4d/%2d/%2d %2d:%2d:%6.3f\n",YR,MTH,DAY,HR,MIN,(float)SECL);

// Patch for TROMS 31/01/00
//ptr->whole_second=SECL;
//if(ptr->whole_second == 59)
//    system("/home/seislog/minutt &");

SEC=(float)SECL;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
//ptr->corr_sec=(int)(MSECS-MSEC70);           /* correct second on next 1PPS       */
ptr->icmde=(int)(MSECS-MSEC70);
ptr->itmde=0;
/*--------------------------------------------------------------------------*/
/* prepare seconds and nanoseconds in structure for CLOCK_SETIME            */
/*--------------------------------------------------------------------------*/
start.tv_sec=(long)(MSECS-MSEC70);
start.tv_nsec=0;
start.tv_nsec=ptr->itmde*1000000;
system("rtc -s hw");
ptr->noktime++;
/*--------------------------------------------------------------------------*/
/* set CPU clock to time received from GPS                                  */
/*--------------------------------------------------------------------------*/
clock_settime(CLOCK_REALTIME,&start);
SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
/*--------------------------------------------------------------------------*/
/* save new update of CPU in Timsav for mon utility Last Update             */
/*--------------------------------------------------------------------------*/
SavTim(HR,MIN,SEC,DAY,MTH,YR,DOY);   /* make copy of timestring           */
goto Next;
}
rd_msg(ptr,buffer,HR,MIN,SECL,DAY,MTH,YR)
smod*    ptr;
unsigned char *buffer;
int*    HR;
int*    MIN;
int*    SECL;
int*    DAY;
int*    MTH;
int*    YR;
{
int      i;
int      cnt;
unsigned char car[1];
int     ptrhrn;
int     ptrmin;
int     ptrsec;
int     ptrday;
int     ptrmon;
int     ptryrn;

sscanf(buffer,"%c%2d%2d%2d%2d",car,&ptrhrn,&ptrmin,&ptrsec);
*HR=ptrhrn;
*MIN=ptrmin;
*SECL=ptrsec;
cnt=0;
for(i=0;i<50;i++)
{
    if(buffer[i]==',')
        cnt=cnt+1;
    if(cnt==9)
        goto fnd;
}
fnd:
cnt=i;
for(i=0;i<6;i++)
    buffer[i]=buffer[cnt+1+i];
sscanf(buffer,"%2d%2d%2d",&ptrday,&ptrmon,&ptryrn);
*DAY=ptrday;
*MTH=ptrmon;
*YR=ptryrn;
}
/****************************************************************************/
/* Copy timestring into save area                                           */
/****************************************************************************/
SavTim(HR,MIN,SEC,DAY,MTH,YR,DOY)
int     HR;
int     MIN;
float   SEC;
int     DAY;
int     MTH;
int     YR;
int     DOY;
{
int      kr;
char     tm[40];

sprintf(tm,"%2d:%2d:%6.3f %2d/%2d/%2d %3d\n",
HR,MIN,SEC,DAY,MTH,YR,DOY);
zero_pad(tm);
for(kr=0;kr<28;kr++)
    ptr->Timsav[kr]=tm[kr];
}
/****************************************************************************/
/* Read timestring from clock                                               */
/****************************************************************************/
read57(fh,buffer,STS)
int      fh;
unsigned char *buffer;
char     *STS;
{
register unsigned char *buf_ptr;
int      n;                            /* status indicator                  */
int      sts;                          /* A-valid, V-Not valid              */
int      t;
buf_ptr=buffer;
n=dev_read(fh,buf_ptr,57,57,0,10,0,0); /* read 57 first bytes,              */
                                       /* Garmin   ",hhmmss,S,"             */
                                       /* Lowrance ",hhmmss.ssssss,S"       */
if(n==-1)                              /* check for ERROR                   */
{
//    printf("ERROR\n");
//    ptr->TimFlg[0]=-1;
    sprintf(ptr->err_msg,"CLK_GEN: Error reading timestring, clock:%d",ptr->clktyp);
    errlog(ptr);
    return(n);                         /* return error indication           */
}
if(n==0)                               /* check for TIMEOUT                 */
{
//    printf("TIMEOUT\n");
//    ptr->TimFlg[0]=0;
    sprintf(ptr->err_msg,"CLK_GEN: Timeout reading timestring, clock:%d",ptr->clktyp);
    errlog(ptr);
    return(n);                         /* return timeout indication         */
}    
*STS=buffer[8];
return(1);
}
/*--------------------------------------------------------------------------*/
/* search for "MC" in $GPRMC string from GARMIN GPS clock                   */
/*--------------------------------------------------------------------------*/
Sync(fh,buffer)
int      fh;
unsigned char *buffer;
{
register unsigned char *buf_ptr;
register unsigned short sync_value=0;
int      n;
buf_ptr=buffer;                              /* set pointer to buffer       */
do
{
    n=dev_read(fh,buf_ptr,1,1,0,15,0,0); /* read one byte                   */
    if(n==-1)                          /* check for ERROR                   */
    {
//        printf("SNC ERROR\n");
//        ptr->TimFlg[0]=-1;
        sprintf(ptr->err_msg,"CLK_GEN: Error reading sync character from clock:%d",ptr->clktyp);
        errlog(ptr);
        return(n);                     /* return ERROR indication           */
    }
    if(n==0)                           /* check for timeout                 */
    {
//        printf("SNC TIMEOUT\n");
        ptr->TimFlg[0]=0;
        ptr->TimFlg[1]++;              /* count timeouts                    */
        sprintf(ptr->err_msg,"CLK_GEN: Timeout reading sync character from clock:%d",ptr->clktyp);
        errlog(ptr);
        if(ptr->TimFlg[1]>5)           /* more than 5 in sequence, action   */
        {
            sprintf(ptr->err_msg,"CLK_GEN: MORE THAN 5 SYNC TIMEOUTS");
            errlog(ptr);
        }
        system("cp /home/seislog/pr/PGRMI /dev/ser1");

        ptr->corr_sec++;
        return(n);                     /* return timeout indication         */
    }
//    ptr->TimFlg[0]=1;
//    ptr->TimFlg[1]=0;                  /* zero timeout counter              */
//    ptr->flg_RAM=1;                      /* Initially CPU NOT SET       */
    sync_value=sync_value << 8 | *buf_ptr;
}while(sync_value != 0x4d43);            /* read and search until "MC"  */
//    Trigger(ptr->req_proxy);
      ptr->it_freeze++;
      if(ptr->it_freeze >= 1000000) ptr->it_freeze=0;
return(n);
}


par_gps()
{   
/*outp(0x378,0x00);*/
outp(0x378,inp(0x378) ^ 0x04);
delay(500);                              /* duration of pulse in msec   */
outp(0x378,inp(0x378) ^ 0x04);
/*outp(0x378,inp(0x378) ^ 0x04);*/
printf("gps ok\n");
}   /* End of main */




rd_msg_low(ptr,buffer,buffer2,HR,MIN,sec,DAY,MTH,YR)
smod*    ptr;
unsigned char *buffer;
unsigned char *buffer2;
int*      HR;
int*      MIN;
float*    sec;
int*      DAY;
int*      MTH;
int*      YR;
{
int      i;
int      lps;
unsigned char car[1];
int       ptrhrn;
int       ptrmin;
float     ptrsec;
int       ptrday;
int       ptrmon;
int       ptryrn;
/* read time and date                                                       */
for(i=0;i<57;i++)
    buffer[i]=buffer[i+1];
    sscanf(buffer,"%2d%2d%f9.6",&ptrhrn,&ptrmin,&ptrsec);
    for(i=0;i<6;i++)
        buffer[i]=buffer[50+i];
    sscanf(buffer,"%2d%2d%2d",&ptrday,&ptrmon,&ptryrn);
*HR=ptrhrn;
*MIN=ptrmin;
*sec=ptrsec;
*DAY=ptrday;
*MTH=ptrmon;
*YR=ptryrn;
}



