/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      Author: David Scott                 */
/*                                              BGS, Murchison House,       */
/*                                              Edinburgh                   */
/*                                                                          */
/*                                      Date:26/06/97                       */
/* Program: CLK_MS1                                                         */
/*                                                                          */
/* Last update:   Included Bergen's new timing routines to overcome         */
/*    extra day at start of year problem.  DJS 03/02/98                     */
/*                                                                          */
/* Program description: Program to handle timing using the MSF time from    */
/*    the RCC 8000A Radiocode clock.  The program receives a timestring     */
/*    a timestring every second on the serial port, and a 1Hz input on      */
/*    the parallel port.  When the 1PPS is received, the CPU clock is       */
/*    updated from the MSF timestring.                                      */
/*                                                                          */
/****************************************************************************/     

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/irqinfo.h>
#include <i86.h>
#include <unistd.h>
#include <time.h>
#include <sys/kernel.h>
//#include "msf_hint.h"
#include <sys/types.h>
#include <sys/irqinfo.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include "/home/seislog/shar_mem.h"

pid_t far 			pps_handler (void);
pid_t far 			mil_handler (void);
pid_t 				proxy;
volatile unsigned 	counter;
volatile smod 		*ptr;


extern int 		errno;
/*volatile unsigned counter;
extern volatile long int pulses;
volatile int icmde;
volatile int itmde;
*/


int			fd;                /* file descriptor for shared memory        */
int 		ser_fd;            /* file descriptor for serial device        */
int 		pps_handler_id;    /* id for ipps interrupt handler            */
int 		mil_handler_id;    /* id for millisecond counting int handler  */

unsigned char buffer[64];      /* buffer for timestring                    */
unsigned char *buf_ptr;        /* ptr to buffer                            */
int 		errors[64];        /* errors in timestring                     */
int 		errstring;         /* switch to indicate timestring error      */
int 		kl;                /* array counting type dude                 */  
//#include "msf_hint.h"

main ()
{
  struct timespec sstart,stopp;
  int count; /* temp for loop */
  char key;  /* temp for loop */
  int old_pulses;  /* temp for loop */

  int 		nor;               /* return status of sync operation        */
  int 		stat_read;         /* return status of reading timestring    */
  struct timespec start;       /* to hold CPU time values                */
  long 		yrn,
       		monn,
	   		dayn,
       		hrn,
       		minn,
       		secl,
	   		msecn,
       		doyn;              /* integer values from timestring         */
  float 	secn;              /* seconds from timestring as float       */
  double 	sectot;            /* total no of seconds in epoch           */
/*  int nanosecs; */
  char stty_str[100];
  char port_str[12];


  int       YR,
            DOY,
            MTH,
            DAY,
            HR,
            MIN;
  float     SEC;
  int       SECL;
  double    MSECS,
            MSEC70;  



	system ("ticksize 1");
	/*---------------------------------------------------------------------*/
	/* open shared memory region                                           */
	/*---------------------------------------------------------------------*/
	fd = shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
	if (fd == -1) 
	{
		exit1 (strcat("failed to open shared memory, ", strerror(errno)));
	}
	/*---------------------------------------------------------------------*/
	/* set size of memory region                                           */
	/*---------------------------------------------------------------------*/
	ltrunc (fd,sizeof(smod),SEEK_SET);
	/*---------------------------------------------------------------------*/
	/* Compute pointer to shared memory region                             */
	/*---------------------------------------------------------------------*/
	ptr = mmap (0, sizeof(smod), PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if ((int)ptr == -1) 
	{
	    exit1 (strcat("mmap failed, ", strerror(errno)));
	}
	ptr->itmde = 0;                  /* set milliseconds to zero initially */
	ptr->icmde = 0;                  /* set seconds to zero initially      */
	ptr->pid_timing = getpid();      /* set process id this process        */
	for (kl = 0; kl < 64; kl++)      /* initialize timestring buffers      */
    	buffer[kl] = '.';
/*--------------------------------------------------------------------------*/
/* start up program to count milliseconds to keep software clock updated    */
/* If 1 PPS is supported, the clock will be synchronized by this puls       */
/*--------------------------------------------------------------------------*/
system("on -t /dev/con6 /home/seislog/clk_pps&");

	/*---------------------------------------------------------------------*/
	/* set up serial port                                                  */
	/*---------------------------------------------------------------------*/

	sprintf(port_str,"/dev/ser%d",ptr->prt_clk);
	sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",
			ptr->bd_clk,port_str);
	system (stty_str);

	if ((ser_fd = open(port_str,O_RDONLY)) == ERROR)
		exit1 ("failed to open serial device");

	setprio (getpid(), 14);

	/*---------------------------------------------------------------------*/
	/* use current CPU time as initial value for internal clock            */
	/*---------------------------------------------------------------------*/

    YR = 1970;
    MTH = 1;
    DAY = 1;
    HR = 0;
    MIN = 0;
    SEC = 0.0;
    TIMSEC (YR, MTH, DAY, HR, MIN, SEC, &MSEC70);     /* new! */

	if (clock_gettime (CLOCK_REALTIME, &start) == -1)
		exit1("clock_gettime failed");
	/* and set up initial values */
	ptr->icmde = start.tv_sec;
	ptr->itmde = start.tv_nsec/1000000;
	MSECS = start.tv_sec + (float)ptr->itmde/1000.0;
    SECTIM (MSECS+MSEC70, &YR, &DOY, &MTH, &DAY, &HR, &MIN, &SEC);    /* new! */

	if (ptr->clk_chk == 1)
	{
	    /* print out initial values */
		printf ("seconds = %d\nmillisec = %d\n", ptr->icmde, ptr->itmde);
//		sectim (&yrn, &monn, &dayn, &hrn, &minn, &doyn, sectot, &secn);
		printf ("%2d:%2d:%6.3f %2d/%2d/%2d\n", hrn,minn,secn,dayn,monn,yrn);
	}


/***************************************************************************/
/*    get in sync                                                          */
/***************************************************************************/
	if (clock_gettime (CLOCK_REALTIME, &sstart) == -1)
    {
		perror("clock_gettime failed");
        exit(0);
    }

Resync: 
	while (dev_ischars (ser_fd) != 0)
		read (ser_fd, buffer, 64);          /* drain input buffer          */
	ptr->TimFlg[0] = -3;                    /* sync not found              */
    do
    {
		nor = Sync();                       /* find CRLF                   */
    } while (nor <= 0);
	for (kl = 0; kl < 64; kl++)      /* re-initialize timestring buffers   */
    	buffer[kl] = '.';

/***************************************************************************/
/*    main loop entry point                                                */
/***************************************************************************/

 for (;;)
 {
	if (clock_gettime (CLOCK_REALTIME, &stopp) == -1)
    {
		perror("clock_gettime failed");
        exit(0);
    }
    ptr->oktime=(stopp.tv_sec - sstart.tv_sec);
//printf("stopp %d start %d\n",stopp.tv_sec,sstart.tv_sec);
	ptr->alive[3] = 1;                      /* tell I'm alive              */
	stat_read = read31();                   /* read the timestring         */
	switch (stat_read)                      /* and check the return value  */
	{
		case -1:                   /* error reading timestring             */
			goto Resync;           /* resynchronise                        */
			break;
		case 0:                    /* timeout                              */
			goto Resync;           /* resynchronise                        */
			break;
		case 1:                    /* timestring received and accepted     */
			break;
		case 2:                    /* timestring received but not accepted */
			if (ptr->clk_chk == 1)
				printf ("CLK_MS1: Time received but not valid\n");
			goto Resync;           /* resynchronise                        */
			break;
	}
	/*---------------------------------------------------------------------*/
	/* if correct timestring received, interpret it                        */
	/*---------------------------------------------------------------------*/
//	Convert_Integer (&yrn, &monn, &dayn, &hrn, &minn, &secl, &msecn, &doyn);
//	secn = (float)secl;
//	timsec (yrn, monn, dayn, hrn, minn, secn, &sectot);  /* convert to secs */
//	sectot++;                               /* will set time on NEXT second */
//	ptr->corr_sec = (int)sectot;


    Convert_Integer (&YR, &MTH, &DAY, &HR, &MIN, &SECL, &MSECS, &DOY);
    SEC = (float) SECL;
    TIMSEC (YR, MTH, DAY, HR, MIN, SEC, &MSECS);
    MSECS++;                                /* will set time on NEXT second */
    ptr->corr_sec = (int) (MSECS - MSEC70);                   /* new! */

	/*----------------------------------------------------------------------*/
	/* prepare structure for CLOCK_SETTIME                                  */
	/*----------------------------------------------------------------------*/
	start.tv_sec = (long) (MSECS - MSEC70);           /* new! */
	start.tv_nsec = 0;

	/* fix for timing error if 1 pps disappears */
	if (dev_ischars(ser_fd) > 0)
		goto Resync;

	clock_settime(CLOCK_REALTIME, &start);
    system("rtc -s hw");
//	ptr->TimFlg[0] = 3;
	ptr->noktime++;

	/*----------------------------------------------------------------------*/
	/* make copy of updated CPU time                                        */
	/*----------------------------------------------------------------------*/
//	sectim (&yrn, &monn, &dayn, &hrn, &minn, &doyn, sectot, &secn);
    SECTIM (MSECS, &YR, &DOY, &MTH, &DAY, &HR, &MIN, &SEC);     /* new! */
	SavTim (HR, MIN, SEC, DAY, MTH, YR, DOY);

	if (ptr->clk_chk == 1)  
    	printf ("CLK_MS1: %2d:%2d:%7.3f %2d/%2d/%2d\n",hrn,minn,secn,dayn,monn,yrn);

	/*----------------------------------------------------------------------*/
	/* is the min_mrk and par_gps active ?                                  */
	/* if they are, send pulse to par port to signal that GPS is receiving  */
	/*----------------------------------------------------------------------*/
	if(ptr->min_dur != 0)              /* activate minute mark on par port ?*/
    	Trigger(ptr->gps_proxy);
 }                                     /* end of main loop                  */


	/* this is never reached now */
	printf ("end of program reached\n");
    tidy_up();
    exit (0);
}
/***************************************************************************/
/* end of main                                                             */
/***************************************************************************/


tidy_up(void)
{
    /* tidy up for exit */
    close (ser_fd);
    /* disable IRQs on parallel ports */
    outp (0x03be, inp (0x03bd) & 0xef);
    outp (0x037a, inp (0x0379) & 0xef);
    outp (0x027a, inp (0x0279) & 0xef);
    qnx_hint_detach (pps_handler_id);
}


exit1(err_message)
char 		*err_message;
{
    fprintf (stderr, "msf: %s\n", err_message);
	tidy_up();
	exit (1);
}



/****************************************************************************/
/* Read timestring from clock                                               */
/****************************************************************************/
read31()
{
  int 	n;                     /* status of dev_read                   */

    errstring = 0;
	for (kl = 0; kl < 64; kl++)
		errors[kl] = 0;
  
    buf_ptr = buffer;
    n = dev_read (ser_fd, buf_ptr, 31, 31, 0, 10, 0, 0);
    if (n == -1)
	{
        ptr->TimFlg[0]=-1;
		if (ptr->clk_chk == 1)		printf ("ERROR\n");
  		return (n);              /* error on read                          */
	}
	if (n == 0)
	{
        ptr->TimFlg[0]=0;
		if (ptr->clk_chk == 1)		printf ("TIMEOUT\n");
		return (n);              /* data not received                      */
	}
    if(buffer[28] >='8')
    {
        ptr->TimFlg[0]=1;
    }else{
        ptr->TimFlg[0]=2;
    }
	ChkTim();
	if (errstring == 0)
		return (1);              /* data received, timestring accepted     */

	return (2);                  /* data received, timestring not accepted */
}



/*--------------------------------------------------------------------------*/
/* Convert time string and put into integers                                */
/*--------------------------------------------------------------------------*/
Convert_Integer(yrn, monn, dayn, hrn, minn, secl, msecn, doyn)
int	*yrn;
int	*monn;
int	*dayn;
int	*hrn;
int	*minn;
int	*secl;
double    *msecn;
int	*doyn;
{
	*yrn   = atoi (&buffer[19]);
	*monn  = atoi (&buffer[16]);
	*dayn  = atoi (&buffer[13]);
	*hrn   = atoi (&buffer[0]);
	*minn  = atoi (&buffer[3]);
	*secl  = atoi (&buffer[6]);
	*msecn = atoi (&buffer[9]);
	*doyn  = atoi (&buffer[22]);
}



/***************************************************************************/
/* get in sync by searching for CRLF                                       */
/***************************************************************************/
Sync()
{
  register unsigned char *buf_ptr;
  register unsigned short sync_value = 0;
  int n;

    buf_ptr = buffer;
    do
	{
		n = dev_read (ser_fd, buf_ptr, 1, 1, 0, 10, 0, 0); /* read one byte */
		if (n == -1)
		{
			printf ("SYNC ERROR\n");
            ptr->TimFlg[0]=-1;
			return (n);
		}
		if (n == 0)
		{
			printf ("SYNC Timeout\n");
            ptr->TimFlg[0]=0;
			return (n);
		}
        ptr->TimFlg[0]=1;     
		sync_value = sync_value << 8 | *buf_ptr;
	} while (sync_value != 0x0d0a);
	return (n);
}


void zero_fill2(string)
char* string;
{
  int i;

    for (i=0; i< 12; i++)
        if (string[i] == ' ') string[i] = '0';
    for (i=13; i< 23; i++)
        if (string[i] == ' ') string[i] = '0';
    for (i=24; i< 27; i++)
        if (string[i] == ' ') string[i] = '0';
}



/****************************************************************************/
/* Copy timestring into save area                                           */
/****************************************************************************/
SavTim(hrn,min,sec,day,mon,yrn,doy)
int     hrn;
int     min;
float   sec;
int     day;
int     mon;
int     yrn;
int     doy;
{
  int      kr;
  char     tm[40];

	sprintf (tm,"%2d:%2d:%6.3f %2d/%2d/%4d %3d\n",hrn,min,sec,day,mon,yrn,doy);
	zero_fill2 (tm);
	for (kr = 0; kr < 27; kr++)
	    ptr->Timsav[kr] = tm[kr];
}




/*--------------------------------------------------------------------------*/
/*  Routine to check if timestring is valid.                                */
/*  If any errors are discovered, a flag is incremented and datamodule      */
/*  value ERRIND is set to 1 at the corresponding position.                 */
/*  When an error occur, a message is written to ERRLOG indicating          */
/*  which character in the timestring that is wrong.                        */
/*--------------------------------------------------------------------------*/
ChkTim()
{
/*--------------------------------------------------------------------------*/
/*  10's of HOURS should be from 0 to 2                                     */
/*--------------------------------------------------------------------------*/
if(buffer[0] < '0' || buffer[0] > '2')
{
    errors[0]=1;
/*    ptr->errind[0]=1;*/
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  1's of HOURS should be from 0 to 9                                      */
/*--------------------------------------------------------------------------*/
if(buffer[1] < '0' || buffer[1] > '9')
{
    errors[1]=1;
/*    ptr->errind[1]=1;*/
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  Position 2 should be :                                                  */
/*--------------------------------------------------------------------------*/
if(buffer[2] != ':')
{
    errors[2]=1;
/*    ptr->errind[2]=1;*/
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  10's of MINUTES should be from 0 to 5                                   */
/*--------------------------------------------------------------------------*/
if(buffer[3] < '0' || buffer[3] > '5')
{
    errors[3]=1;
/*    ptr->errind[3]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  1's of MINUTES should be from 0 to                                      */
/*--------------------------------------------------------------------------*/
if(buffer[4] < '0' || buffer[4] > '9')
{
    errors[4]=1;
/*    ptr->errind[4]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  Position 5 should be :                                                  */
/*--------------------------------------------------------------------------*/
if(buffer[5] != ':')
{
    errors[5]=1;
/*    ptr->errind[5]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  10's of SECONDS should be from 0 to 5                                   */
/*--------------------------------------------------------------------------*/
if(buffer[6] < '0' || buffer[6] > '5')
{
    errors[6]=1;
/*    ptr->errind[6]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  1's of SECONDS should be from 0 to 9                                    */
/*--------------------------------------------------------------------------*/
if(buffer[7] < '0' || buffer[7] > '9')
{
    errors[7]=1;
/*    ptr->errind[7]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  Position 8 should be .                                                  */
/*--------------------------------------------------------------------------*/
if(buffer[8] != '.')
{
    errors[8]=1;
/*    ptr->errind[8]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  100's of MSEC should be from 0 to 9                                     */
/*--------------------------------------------------------------------------*/
if(buffer[9] < '0' || buffer[9] > '9')
{
    errors[9]=1;
/*    ptr->errind[9]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  10's of MSEC should be from 0 to 9                                      */
/*--------------------------------------------------------------------------*/
if(buffer[10] < '0' || buffer[10] > '9')
{
    errors[10]=1;
/*    ptr->errind[10]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  1's of MSEC should be from 0 to 9                                       */
/*--------------------------------------------------------------------------*/
if(buffer[11] < '0' || buffer[11] > '9')
{
    errors[11]=1;
/*    ptr->errind[11]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  Position 12 should be from ' '                                          */
/*--------------------------------------------------------------------------*/
if(buffer[12] != ' ')
{
    errors[12]=1;
/*    ptr->errind[12]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  10's of DAY should be from 0 to 3                                       */
/*--------------------------------------------------------------------------*/
if(buffer[13] < '0' || buffer[13] > '3')
{
    errors[13]=1;
/*    ptr->errind[13]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  1's of DAY should be from 0 to 9                                        */
/*--------------------------------------------------------------------------*/
if(buffer[14] < '0' || buffer[14] > '9')
{
    errors[14]=1;
/*    ptr->errind[14]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  Position 15 should be /                                                 */
/*--------------------------------------------------------------------------*/
if(buffer[15] != '/')
{
    errors[15]=1;
/*    ptr->errind[15]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  10's of MONTH should be from 0 to 1                                     */
/*--------------------------------------------------------------------------*/
if(buffer[16] < '0' || buffer[16] > '1')
{
    errors[16]=1;
/*    ptr->errind[16]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  1's of MONTH should be from 0 to 9                                      */
/*--------------------------------------------------------------------------*/
if(buffer[17] < '0' || buffer[17] > '9')
{
    errors[17]=1;
/*    ptr->errind[17]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  Position 18 should be /                                                 */
/*--------------------------------------------------------------------------*/
if(buffer[18] != '/')
{
    errors[18]=1;
/*    ptr->errind[18]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  10's of YEAR should be 9                                                */
/*--------------------------------------------------------------------------*/
if(buffer[19] < '0' || buffer[19] > '9')
{
    errors[19]=1;
/*    ptr->errind[19]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  1's of YEAR should be from 0 to 9                                       */
/*--------------------------------------------------------------------------*/
if(buffer[20] < '0' || buffer[20] > '9')
{
    errors[20]=1;
/*    ptr->errind[20]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  Position 21 should be ' '                                               */
/*--------------------------------------------------------------------------*/
if(buffer[21] != ' ')
{
    errors[21]=1;
/*    ptr->errind[21]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/* 100's of DOY should be from 0 to 3                                       */
/*--------------------------------------------------------------------------*/
if(buffer[22] < '0' || buffer[22] > '3')
{
    errors[22]=1;
/*    ptr->errind[22]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  10's of DOY should be from 0 to 9                                       */
/*--------------------------------------------------------------------------*/
if(buffer[23] < '0' || buffer[23] > '9')
{
    errors[23]=1;
/*    ptr->errind[23]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  1's of DOY should be from 0 to 9                                        */
/*--------------------------------------------------------------------------*/
if(buffer[24] < '0' || buffer[24] > '9')
{
    errors[24]=1;
/*    ptr->errind[24]=1; */
    errstring=errstring+1;
}
/*--------------------------------------------------------------------------*/
/*  Position 25 should be ' '                                               */
/*--------------------------------------------------------------------------*/
if(buffer[25] != ' ')
{
    errors[25]=1;
/*    ptr->errind[25]=1; */
    errstring=errstring+1;
}
if(errstring != 0)
{
/*for(kp=0;kp<25;kp++)*/
/*    printf(" %d",errors[kp]);*/
/*printf("\n");*/
/*for(kp=0;kp<24;kp++)*/
/*    printf("%2x",buffer[kp]);*/
/*printf("\n");*/
}

}
/***************************************************************************/
/*  end of ChkTim routine												   */
/***************************************************************************/
/***************************************************************************/


