/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:13/08/98                       */
/* Program:CLK_PPS                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* 09/06/00 8.23 t.u. enable interrupt at par port after while(1)           */
/* 09/06/00 8.24 t.u. modification above ONLY for clktyp=4                  */
/* Program description:                                                     */
/*                                                                          */
/* Program maintain milliseconds free running or optionally synchronized to */
/* 1 PPS from GPS clock. The millisecond counter counts at a rate of one    */
/* millisecond up to 1000 and then begins at 0. If the option of a 1 PPS is */
/* the millisecond counter is reset to 0 at each 1 PPS.                     */
/* The correct second (since 1.january 1970) is set in the interrup routine */
/* at reception of 1 PPS.                                                   */
/* The program is started from within the CLK_GEN program.                  */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/name.h>
#include <i86.h>
#include <time.h>
#include <conio.h>
#include <sys/irqinfo.h>               /* needed for interrupt-routines     */
#include <sys/proxy.h>
#include <sys/kernel.h>                /* needed for interrupt-routines     */

pid_t             proxy;
pid_t             pxy;
unsigned int      cop;
int               dig_ge1_id;
volatile unsigned counter;             /* variable for interrupt routine    */
volatile int      cnt;                 /* for debugging                     */
volatile int      ms;                  /* for debugging                     */
int               fd;                  /* return value open share           */
volatile smod     *ptr;                /* pointer to shared region          */
/****************************************************************************/
/*      The Hardware Interrupt Handler 1PPS signal from clock               */
/*--------------------------------------------------------------------------*/
/* This interrupt handler is triggered by the 1PPS signal from the Garmin   */
/* GPS receiver through the line printer port. This interrupt is enabled    */
/* just before we can expect a 1 PPS, and then disabled. Each time triggered*/
/* the second counter is set to the correct second and the millisecond      */
/* counter is set to 0. A proxy is returned to the main program to disable  */
/* the interrupt.                                                           */
/****************************************************************************/
#pragma off( check_stack );
pid_t far handler()
{
//    if(ptr->TimFlg[0] != 0)
        ptr->icmde=ptr->corr_sec;      /* adjust clock to correct second    */
    ptr->itmde=0;                      /* reset millisecond counter         */
    ms=0;                              /* for debugging                     */
    cnt++;                             /* for debugging                     */
    return(proxy);                     /* signal main to disable interrupt  */
}
#pragma on( check_stack);
/****************************************************************************/
/*      The Hardware Interrupt Handler counting milliseconds                */
/*--------------------------------------------------------------------------*/
/* This interrupt handler is connected up with the systems own timer, and   */
/* is not more accurate than the systems CPU time. However, it is accurate  */
/* enough to count to 1000 milliseconds.                                    */
/****************************************************************************/
#pragma off( check_stack );
pid_t far mil_handler()
{
    ms++;                              /* for debugging                     */
    ptr->itmde++;                      /* increment millisecond counter     */
    if(ptr->itmde == 1000)
    {
        ptr->itmde=0;                  /* reset millisecond counter to 0    */
        ptr->icmde++;                  /* increment second counter          */
    }
return(0);
}
#pragma on( check_stack);

main()
{
int      id;                           /* id for the 1 PPS interrupt handler*/
int      idd;                          /* id for the 1msec interrupt handler*/
int      clk_pps_id;
int      drv_id;
char     rec_msg[30];                  /* RECEIVE message                   */
char     rep_msg[30];
char     snd_msg[30];
//unsigned int  cop;
struct timespec t1;
cnt=0;                                 /* set to zero initially             */
system("ticksize 1");                  /* set ticksize of system to 1 msec  */
setprio(getpid(),15);                  /* set the priority this process=14  */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    sprintf(ptr->err_msg,"CLK_PPS: Can't open shared memory.");
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    sprintf(ptr->err_msg,"CLK_PPS: Can't map shared memory.");
    errlog(ptr);
    exit(0);
}
if(ptr->clktyp==2 || ptr->clktyp==4)   /* do we have 1 pps support ?        */
    outp(0x37a,0x00);                  /* disable interrupt at PAR port     */

clk_pps_id=qnx_name_attach(0,"clk_pps");
if(clk_pps_id == -1)
{
    printf("CLK_PPS attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* set up a proxy for the interrupt handler to kick                         */
/*--------------------------------------------------------------------------*/
if((proxy = qnx_proxy_attach(0,0,0,0)) == -1)
{
    sprintf(ptr->err_msg,"CLK_PPS: Can't attach proxy.");
    errlog(ptr);
    exit(0);
}
ptr->min_proxy=0;
/*--------------------------------------------------------------------------*/
/* Attach the interrupt from the 1PPS from GPS                              */
/*--------------------------------------------------------------------------*/
if(ptr->clktyp==2 || ptr->clktyp==4)   /* do we have 1 pps support ?        */
{
    if(( id=qnx_hint_attach(7,&handler,FP_SEG(&counter)))==-1)
    {
        sprintf(ptr->err_msg,"CLK_PPS: Can't attach the 1 PPS interrupt.");
        errlog(ptr);
        exit(0);
    }
}
/*--------------------------------------------------------------------------*/
/* Attach the interrupt to count milliseconds                               */
/*--------------------------------------------------------------------------*/
if(( idd=qnx_hint_attach(0,&mil_handler,FP_SEG(&counter)))==-1)
{
    sprintf(ptr->err_msg,"CLK_PPS: Can't attach to the millisecond interrupt.");
    errlog(ptr);
    exit(0);
}
ms=0;
if(ptr->clktyp==2 || ptr->clktyp==4)   /* do we have 1 pps support ?        */
{
    while(1)
    {
        if(ptr->clktyp==4)
          outp(0x37a,0x10);            /* enable interrupt at par port      */
        Receive(proxy,0,0);            /* wait for signal from interrupt    */
        outp(0x37a,0x00);              /* disable interrupt at par port     */
        ptr->average1++;
//clock_gettime(CLOCK_REALTIME,&t1);
//printf("cnt=%4d ms=%4d nano=%d\n",cnt,ms,t1.tv_nsec);
        if(ptr->whole_second==59)
        {
            if(ptr->unit==31)
            {
                dig_ge1_id=qnx_name_locate(0,"dig_ge1",0,&cop);
                if(dig_ge1_id != -1)
                {
                    Trigger(ptr->min_proxy);
                }else{
                    printf("Cant locate dig_ge1\n");
                }
            }else{
                dig_ge1_id=qnx_name_locate(0,"dig_m18",0,&cop);
                if(dig_ge1_id != -1)
                {
                    Trigger(ptr->min_proxy);
                }else{
//                    printf("Cant locate dig_m18\n");
                }
            }
            system("/home/seislog/min_pls&");
        }
    }
}


while(1)
{
    sleep(10);
}
}


