/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:29/09/95                       */
/* Program:MDS                                                              */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program monitors the disk space available at all time.               */
/* In the parameter file you may specify a disk space level in parameter    */
/* number 18. Whenever SEISLOG uses more than this percentage, the program  */
/* will start deleting old files from /home/save and /home/events.          */
/* All the files in /home/save directory will be deleted in one go. After   */
/* all files have been deleted in /home/save and the usage of disk still    */
/* exceeds the level specified, eventfiles from /home/events will be deleted*/
/* one by one. The corresponding detection record in /home/buffer/event is  */
/* marked with a '*' in the column before the filename.                     */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <dirent.h>
#include "filedef.h"

long     position;                     /* variable for disk seek            */
int      place;                        /* variable for disk seek            */
int      eq;                           /* return after string-compare       */
char     input[512];                   /* array to hold detection record    */
unsigned char line[200];               /* input buffer for filename         */
char     names[30];                    /* string to hold filename           */
char     syscall[100];                 /* string to hold system command     */
int      lt;                           /* counter det numbers               */
int      bytes;                        /* return value read/write           */
int      rec_adr2;                     /* record adress                     */
int      i;                            /* Counter                           */
int      j;                            /* counter                           */
int      k;                            /* counter                           */
int      percent;                      /* percent free space                */
float    sec_nes;                      /* seconds of next filename          */
float    sec_min;                      /* seconds of oldest filename        */
int      eldst_index;                  /* index of oldest filename          */
int      no_files;                     /* number of files in directory      */
char     filenames[EVTSIZ][EVNAMLEN+1];/* table of filenames                */
char     streng[10];                   /* help string                       */
char     eldst_fil[30];                /* string for oldest file            */
char     neste_fil[30];                /* string for next file              */
int      yrn;                          /* year                              */
int      mon;                          /* month                             */
int      day;                          /* day                               */
int      hrn;                          /* hour                              */
int      min;                          /* minute                            */
float    sec;                          /* second                            */
int      yrn1;                         /* year                              */
int      mon1;                         /* month                             */
int      day1;                         /* day                               */
int      hrn1;                         /* hour                              */
int      min1;                         /* minute                            */
float    sec1;                         /* second                            */
double   sectot;                       /* number of seconds for a date      */
double   sectot1;                      /* number of seconds for a date      */
long     fre;                          /* return value for disk_space       */
int      ser_prt;
char     devinfo[50];
long     total;                        /* return value for disk_space       */
DIR      *dirp;                        /* pointer to directory              */
FILE     *eve;                         /* Pointer to detection file         */
FILE     *cmd;
struct   dirent *direntp;              /* structure for filenames           */

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

main()
{
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MDS_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MDS_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
restart:
save();                            /* get table of files in /home/save  */
//printf("Number of files in /home/fil4 =%d\n",no_files);
/*    for(i=0;i<no_files;i++)*/
/*        printf("%2d %s\n",i,filenames[i]);*/
if(no_files == 0)                   /* check if there are files in save  */
{
    sleep(1);
    goto restart;
}
sprintf(syscall,"/home/fil4/"); /* make remove command           */
sprintf(line,filenames[0]); /* make remove command        */
strcat(syscall,line);                 /* complete the remove command*/
if((cmd=fopen(syscall, "r"))==NULL)
{
    fprintf(stderr,"Could not open DUMP file, error=%d\n",errno);
    exit(1);
}
fread(devinfo,22,1,cmd);
fclose(cmd);
for(i=0;i<22;i++)
    devinfo[i]=devinfo[i+4];
sscanf(devinfo,"%2d",&ser_prt);
//printf("ser_prt=%2d\n",ser_prt);
for(i=0;i<22;i++)
    ptr->cmds[ser_prt-1][i]=devinfo[i];

sprintf(syscall,"rm /home/fil4/"); /* make remove command           */
sprintf(line,filenames[0]); /* make remove command        */
strcat(syscall,line);                 /* complete the remove command*/
printf("Remove this file: %s\n",syscall);
system(syscall);               /* remove the file                   */
sleep(1);
goto restart;                  /* do another check in x seconds     */
}


save()
{
int lk;
no_files=0;
dirp=opendir("/home/fil4");
if(dirp != NULL)
{
    for(;;)
    {
lop:
        direntp = readdir(dirp);
        if(direntp != 0)
        {
//            printf("%s\n",direntp->d_name);
            if(direntp->d_name[0]=='1' || direntp->d_name[4]=='2')
            {
                for(lk=0;lk<17;lk++)
                    filenames[no_files][lk]=direntp->d_name[lk];
                filenames[no_files][17]='\0';

                no_files=no_files+1;
            }else{
                goto lop;
            }            
        }else{
            closedir(dirp);
            goto clo_1;
        }
    }
}
clo_1:
lk=0;
}
/*--------------------------------------------------------------------------*/
/* Compute disk adress for given detection record                           */
/*--------------------------------------------------------------------------*/
Get_Adr_Rec(record)
int record;
{
    return(record*EVTREC);
}
/*--------------------------------------------------------------------------*/
/* Read specified detection record                                          */
/*--------------------------------------------------------------------------*/
Det_rec_2(disc_adr)
int disc_adr;
{
    position=fseek(eve,disc_adr,place);
    bytes=fread(input,EVTREC,1,eve);
    for(j=0;j<25;j++)
        names[j]=input[208+j];
}

/*--------------------------------------------------------------------------*/
/* Read specified detection record                                          */
/*--------------------------------------------------------------------------*/
Det_Rec_3(disc_adr)
int disc_adr;
{
    position=fseek(eve,disc_adr,0);
    bytes=fread(input,EVTREC,1,eve);
/*    printf("%d %s %s\n",lt,line,names);*/
}
/*--------------------------------------------------------------------------*/
/* compute total number of seconds.milliseconds                             */
/* input : yrn     year                                                     */
/*         mon     month                                                    */
/*         day     day                                                      */
/*         hrn     hours                                                    */
/*         min     minutes                                                  */
/*         sec     seconds                                                  */
/* output: sectot  total number of seconds                                  */
/*--------------------------------------------------------------------------*/
void timsec(yrn,mon,day,hrn,min,sec,seconds)
int   yrn;
int   mon;
int   day;
int   hrn;
int   min;
float  sec;
double *seconds;
{
    int   dyr;
    int   iyr;
    int   ydy;

    ydy=0;
    if(yrn >=1900) yrn=yrn-1900;
    dyr=0;
    if(yrn%4==0)dyr=1;
    iyr=yrn/4-dyr+1;
    *seconds=iyr*366+(yrn-iyr)*365;
    *seconds=*seconds*86400.0;
    if(mon==1)ydy=day-1;
    if(mon==2)ydy=day+30;
    if(mon==3)ydy=day+dyr+58;
    if(mon==4)ydy=day+dyr+89;
    if(mon==5)ydy=day+dyr+119;
    if(mon==6)ydy=day+dyr+150;
    if(mon==7)ydy=day+dyr+180;
    if(mon==8)ydy=day+dyr+211;
    if(mon==9)ydy=day+dyr+242;
    if(mon==10)ydy=day+dyr+272;
    if(mon==11)ydy=day+dyr+303;
    if(mon==12)ydy=day+dyr+333;
    *seconds=*seconds+ydy*86400.0+hrn*3600+min*60+sec;
}
