/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:23/01/95                       */
/* Program:STS_MSG                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Generate ststus files for SEISLOG in directory /home/fil3                */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <conio.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
FILE *in;
char direc1[11]={'/','h','o','m','e','/','f','i','l','4','/'};
struct date {
             int year;         
             int month;
             int day;
             int hour;
             int min;
             int doy;                  /* day of year                       */
             float secs;               /* seconds                           */
             double abs_secs;          /* seconds from 1/1/1900             */
            };
char     devinfo[50];
char     stsfil2[50];
char     linje[20];
int      ii;
time_t time_of_day;
int      kp;
char     fname[50];
long     hrn;                          /* hours                             */
long     min;                          /* minutes                           */
long     day;                          /* days                              */
long     mon;                          /* months                            */
long     yrn;                          /* years                             */
long     doy;                          /* day of year                       */
long     secl;                         /* seconds from GARMIN               */
float    sec,se1;                      /* seconds x.xxx                     */
int      lt;
int sp;
char     arg;
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */
main (argc,argv)
int argc;
char **argv;
{
struct   timespec start;               /* structure for setting system time */
double   sectot;                       /* total seconds x.xxx of date & time*/
if(argv[1] != '\0')
{
    strcpy(linje,argv[1]);
}else{
    exit(0);
}
sscanf(linje,"%d",&kp);
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"ONN_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"ONN_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
//for(;;)
//{
//    for(kp=1;kp<2;kp++)
//    {
/*--------------------------------------------------------------------------*/
/* use current CPU time as initial value for GPS controlled internal clock  */
/*--------------------------------------------------------------------------*/
    if(clock_gettime(CLOCK_REALTIME,&start)== -1) 
    {
        perror("clock gettime" );
        exit( EXIT_FAILURE);
    }
    sectot=start.tv_sec;
/*--------------------------------------------------------------------------*/
/* convert floating point value of sectot since 1970 to date and time       */
/*--------------------------------------------------------------------------*/
    sectim(&yrn,&mon,&day,&hrn,&min,&doy,sectot,&sec); /* convert back      */
    if(ptr->dbg==1)
        printf("%2d:%2d:%6.3f %2d/%2d/%2d\n",hrn,min,sec,day,mon,yrn);
    for(ii=0;ii<50;ii++)
        stsfil2[ii]='\0';
    bufopen();                         /* generate file name                */
    for(ii=0;ii<11;ii++)
        stsfil2[ii]=direc1[ii];
    strcat(stsfil2,fname);
//printf("FILENAME: %s\n",stsfil2);
    if((in=fopen(stsfil2, "w"))==NULL)
    {
        fprintf(stderr,"Could not open status file, error=%d\n",errno);
        exit(1);
    }
    sprintf(devinfo,"CO16%2d",kp);
    devinfo[6]='&';
    devinfo[7]='I';
    devinfo[8]='D';
    devinfo[9]='C';
    devinfo[10]='1';
    devinfo[11]='2';
    devinfo[12]='3';
    devinfo[13]='4';
    devinfo[14]='5';
    devinfo[15]='6';
    devinfo[16]='7';
    devinfo[17]='8';
    devinfo[18]='9';
    devinfo[19]='0';
    devinfo[20]='*';
    devinfo[21]='*';
    if(devinfo[4]==' ') devinfo[4]='0';
    fwrite(devinfo,22,1,in);
    fclose(in);            
//    sleep(6);
//    }
//}
}
/*---------start: rename_buf------------------------------------------------*/

rename_buf(ptr, start)
smod *ptr;
struct date* start;
{         
char tid[30];
char dag[5],moned[5],dato[5],tiden[9];
int aaret;
time_of_day=time(NULL);
sprintf(tid,"%s\n",ctime(&time_of_day));
sscanf(tid,"%s %s %s %s %d",dag,moned,dato,tiden,&aaret);
start->hour=hrn;
start->min =min;
start->secs=(int)sec;
start->day =day;
start->month=mon;
start->year=aaret;
Event_Name(start,CONTIN,CONTIN,ptr);
}
/*------------end: rename_buf------------------------------------------------*/



/*-----------start: bufopen(ptr)--------------------------------------------*/
/* opens the next ring buffer file for writing and puts a header on at the  */
/* beginning so that the files have the same format as the files in the     */
/* events directory                                                         */
/* the file is left positioned for the start of writing                     */
/*--------------------------------------------------------------------------*/
bufopen()
{ 
char head[SIZ_EV_HDR+1];
struct date start;
char* fnam;
int bytes;
int newblk;
int cpx;
char cpxrec[EVTREC+1];
/*--------------------------------------------------------------------------*/       
/* find the start time of the new buffer, and rename it                     */
/*--------------------------------------------------------------------------*/
rename_buf(ptr, &start);
}

/*-------------------------end: bufopen()---------------------------------*/
/*--------------------------------------------Start: Event_name-------------*/
/* Make file name from time-string                                          */
/*--------------------------------------------------------------------------*/
Event_Name(rec_time,point,par,ptr)
  struct date* rec_time;
  int point;
  int par;
  smod* ptr;
{   
int i;
/* The format of this event file is as follows:
   YYMM_DD_HHMM_SST.CCC_NN_P
   year,month,day,hour,mins,secs,T=temporary,station code, num of channels 
   P parameter set */
   
   char cyear[4];   /* help variable in cutting the century off year        */
   char* namptr= fname;    /* index in fnam string                           */
   
 
sprintf(cyear, "%4d", rec_time->year);
sprintf(namptr, "%4s", cyear);
namptr+=4;
sprintf(namptr, "%2d", rec_time->month);
namptr+=2;
sprintf(namptr, "%2d", rec_time->day);
namptr+=2;
sprintf(namptr, "%2d%2d", rec_time->hour, rec_time->min);
namptr+=4;
sprintf(namptr, "%2d_", (int) rec_time->secs);
namptr+=3;
sprintf(namptr, "%2d", kp);
//sprintf(namptr, "01");
/* fill fnam with zeros in all blank spaces */
zero_fill();

}
/*------------------------------------------End: Event_Name-----------------*/

/****************************************************************************/
/* set blanks to zero in some positions                                     */
/****************************************************************************/
zero_fill()

{
    int i;
    for(i=0;i< 46; i++)
        if(fname[i] == ' ') fname[i]='0';
//    for(i=13;i< 21; i++)
//        if(fname[i] == ' ') fname[i]='0';
//    for(i=22;i< 25; i++)
//        if(fname[i] == ' ') fname[i]='0';
    }
/* SECTIM                                                                   */
/* convert total number of seconds back to year,month,day,etc...            */
/* input : sectot     total number of seconds                               */
/* output: yrn        year                                                  */
/*         mon        month                                                 */
/*         day        day                                                   */
/*         hrn        hours                                                 */
/*         min        minutes                                               */
/*         doy        day of year                                           */
/*         sec        seconds                                               */
/*--------------------------------------------------------------------------*/
sectim(yrn,mon,day,hrn,min,doy,sectot,secs)
long   *yrn;
long   *mon;
long   *day;
long   *hrn;
long   *min;
long   *doy;
double sectot;
float  *secs;
{
    double secday;
    double secyr;
    long year;
    long day_of_year;
    long month;
    long days;
    long   i;
    float  ind;

    secday=86400.0;
    secyr=31536000.0;

    for(i=70;i< 201;i+=1)
    {
        ind=0.0;
        if(i%4==0)ind=1.0;
        if(sectot < (secyr+ind*secday))
        {
           year=i;
           goto found_year;
        }
        else
            sectot=sectot-(secyr+ind*secday);
    }
    year=-1;
found_year:
    *yrn=year;
    day_of_year=sectot/secday+1;
    *doy=day_of_year;
    dte(year,day_of_year,&month,&days);
    *mon=month;
    *day=days;
    sectot=sectot-(day_of_year-1)*secday;
    *hrn=sectot/3600;
    sectot=sectot-*hrn*3600.0;
    *min=sectot/60;
    *secs=sectot-*min*60.0;
}
/*--------------------------------------------------------------------------*/
/* called by: BUF_PIPE                                                      */
/*--------------------------------------------------------------------------*/
/* DTE                                                                      */
/* Convert Day Of Year to Date                                              */
/* Input : doy                                                              */
/* Output: mon,day                                                          */
/*--------------------------------------------------------------------------*/
dte(yrn,doy,mon,day)
long yrn;
long doy;
long *mon;
long *day;
{
int   mth[13];                         /* days in each month                */
int   j,m,n;                           /* counters                          */
    
/* assign days of month to array                                            */

for (j=1;j< 8; j+=2)
{
    mth[j] = 31;
}
for(j=8;j<13;j+=2)
{
    mth[j] = 31;
}
mth[2]=28;
mth[4]=30;
mth[6]=30;
mth[9]=30;
mth[11]=30;
if(yrn%4==0)
    mth[2]=29;
m=0;
for(j=1;j<13;j++)
{
    m=m+mth[j];
    n=doy-m;
    if(n<=0)
    {
        *mon=j;
        *day=n + mth[j];
        break;
    }
}
}       
