/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:01/09/98                       */
/* Program:DEC_MAT                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
//  april 6, jh : remove test print out
//                remove "Event in resampled data"
/* Program description:                                                     */
/*                                                                          */
/* Receives a proxy signal when an aliased filtered buffer is ready         */
/* Resamples data and create files of n minutes                             */
/*                                                                          */
/****************************************************************************/     
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <graph.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <sys/name.h>
#include "timexrou.h"
#define  TIME 60                       /* collect data for n seconds each re*/
#define  TOTAL 1440                      /* minutes per file                  */ 
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                             Date: 17/09/98    Version 7.50
------------------------------------------------------------------------------
Description: Program that do resampling of specified channels. The routine
             writes resampled data files in directory /home/resamp. The data-
             format is described in the SEISLOG manual. The files are named
             as for events, but with the last letter 'R' to indicate
             resampling. Resampling parameters are specified with program
             PAR_SMP. 

Usage:       Routine started according to parameter settings by SEISLOG
------------------------------------------------------------------------------
#endif
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
char     line0[80];
main()
{
double   MSECS;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
double   MS_START;
double   MS_STOPP;
double   DIFF;
double   SAMPLES;
double   TRUESR;
float    SR_AVG;
pid_t    proxy;
unsigned msg;
int      bryter;
float    ssttaa;
float    llttaa;
int      step;
int      reclength;
long     int pos;
int      counter;
int      cur_min;
int      cur_rcl;
int      lc;                        /* Counter                              */
int      ll;                        /* Counter                              */
int      l;
int      j;
int      k;
int      kl;                        /* Counter                              */
int      id;
int      fpl_id;
int      rchan[MAXCHA];
int      det_evt_id;
unsigned int     cop;
unsigned int* ch_ptr;
int      sec_no;                    /* buffer number in pipe just processed */
int      smp;
int      i;                            /* Counter                           */
int      lt;
int      scan;
int      tel;
int      num_cha;
int      skip;           /* Sample skip factor                           */
int      sw;             /* Switch to indicate plotting of station       */
unsigned char head[10000];
char*    hdptr = head;
int      yy[5000];
int      input[100];
int      records;
int      seconds;
FILE*    day;
char     evnam[100];
char     fname[100];
struct   date rec_time;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DEC_MAT_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DEC_MAT_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* locate DET_EVT                                                           */
/*--------------------------------------------------------------------------*/
det_evt_id=qnx_name_locate(0,"det_evt",0,&cop);
if(det_evt_id == -1)
{
    printf("DEC_MAT: Can't find DET_EVT. SEISLOG not active\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* attach proxy for the terminal in use                                     */
/*--------------------------------------------------------------------------*/
    proxy=qnx_proxy_attach(0,0,0,-1);
    ptr->dec_mat_proxy=proxy;
/*--------------------------------------------------------------------------*/
/* attach this process under the name DEC_MAT                               */
/*--------------------------------------------------------------------------*/
    fpl_id=qnx_name_attach(0,"dec_mat");
    if(fpl_id == -1)
    {
        printf("DEC_MAT: CONSOLE attach failed. DEC_MAT already active on console ?\n");
        exit(0);
    }
tel=0;
num_cha=0;
ptr->stop[0]=0;
//printf("Original SampleRate           : %3d\n",ptr->rate);
//printf("Resample SampleRate           : %3d\n",ptr->rsrate);
for(i=0;i<MAXCHA;i++)
{
    if(ptr->ch_res[i]==1)
    {
        num_cha++;
        rchan[tel]=i;
        tel++;
    }
}
//printf("Number of channels to resample: %3d\n",num_cha);
//printf("Channels to resample          :");
//for(i=0;i<num_cha;i++)
//    printf("%3d",rchan[i]);
//printf("\n");
k=(ptr->rate % ptr->rsrate);
if(k != 0)
{
    printf("Samplerate for resampling not valid\n");
    exit(0);
}
//printf("%3d Every %3d recl: %6d\n",ptr->rsrate,ptr->rate/ptr->rsrate,num_cha*4*ptr->rsrate*60);
reclength=num_cha * 4 * ptr->rsrate * 60;
//printf("Recordlength in bytes header:%d\n",reclength);
step=ptr->rate/ptr->rsrate;
//printf("Extract every                 :%3d sample\n",step);
//exit(0);
seconds=0;
smp=0;
scan=TOTAL;
records=0;
sw=0;
counter=0;
cur_min=0;
cur_rcl=0;
ssttaa=0;
llttaa=0;
bryter=1;
for (;;)                               /* enter infinite loop               */
{
    det_evt_id=Receive(0,&msg,sizeof(msg)); /* wait for buffer ready       */
    sec_no=ptr->buf_rdy_fpl;
    id=sec_no;
//printf("%s\n",ptr->rsmp_time);
    for(lt=0;lt<HEADER;lt++)
        input[lt]=ptr->rsmp_time[lt];
    if(ptr->stop[0]==1)
    {
        fclose(day);
        printf("Terminating DEC_MAT smoothly\n");
        exit(0);
    }
/*--------------------------------------------------------------------------*/
/* first time, and thereafter every TOTAL minutes, open a new file.         */
/* Make the header same size as recordlength of data and write it.          */
/*--------------------------------------------------------------------------*/
    if(scan==TOTAL)                    /* do we open a new file ?           */
    {
//printf("Open new file : %s\n",input);
        for(i=0;i<2000;i++)
            head[i]=' ';
        Time_Conv1(input,&rec_time,ptr);
        YR=rec_time.year;
        MTH=rec_time.month;
        DAY=rec_time.day;
        HR=rec_time.hour;
        MIN=rec_time.min;
        SEC=rec_time.secs;
        TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MS_START);
        hdptr = head;
        sprintf(hdptr,"%2d ",rec_time.hour);
        hdptr+=3;
        sprintf(hdptr,"%2d ",rec_time.min);
        hdptr+=3;
        sprintf(hdptr,"%6.3f ",rec_time.secs);
        hdptr+=10;
        sprintf(hdptr,"%2d ",rec_time.day);
        hdptr+=3;
        sprintf(hdptr,"%2d ",rec_time.month);
        hdptr+=3;
        sprintf(hdptr,"%4d ",rec_time.year);
        hdptr+=10;
        Event_Name1(&rec_time,fname,ptr,num_cha);
        sprintf(evnam,"/home/resamp/");
        strcat(evnam,fname);
//printf("EVNAM %s\n",evnam);
        scan=0;

        sprintf(hdptr,"%8d",reclength);
        hdptr+=8;
        sprintf(hdptr,"%8d",num_cha);
        hdptr+=8;
        sprintf(hdptr,"%8d",TOTAL);
        hdptr+=8;
        sprintf(hdptr,"%7d",ptr->rsrate);
        hdptr+=8;
        sprintf(hdptr,"%10.8f",ptr->srate);
        hdptr+=11;
        sprintf(hdptr,"%5d",((reclength/4)/num_cha)*TOTAL);
        hdptr+=5;
        sprintf(hdptr,"%8d",ptr->order2);
        hdptr+=14;
        sprintf(hdptr,"%s",ptr->ftype);
        hdptr+=2;
        sprintf(hdptr,"%8.3f",ptr->fltlo2[0][0]);
        hdptr+=8;
        sprintf(hdptr,"%8.3f",ptr->flthi2[0][0]);
        hdptr+=8;
        switch(num_cha)
        {
            case 1:
            line0[0]=ptr->cmp1[0];
            line0[1]=ptr->cmp1[1];
            line0[2]=ptr->cmp1[2];
            line0[3]=ptr->cmp1[3];
            line0[4]='\0';
            sprintf(hdptr,"%s",line0);
            hdptr+=5;
            hdptr+=5;
            hdptr+=5;
            break;
            case 2:
            line0[0]=ptr->cmp1[0];
            line0[1]=ptr->cmp1[1];
            line0[2]=ptr->cmp1[2];
            line0[3]=ptr->cmp1[3];
            line0[4]='\0';
            sprintf(hdptr,"%s",line0);
            hdptr+=5;
            line0[0]=ptr->cmp2[0];
            line0[1]=ptr->cmp2[1];
            line0[2]=ptr->cmp2[2];
            line0[3]=ptr->cmp2[3];
            line0[4]='\0';
            sprintf(hdptr,"%s",line0);
            hdptr+=5;
            hdptr+=5;
            break;
            case 3:
            line0[0]=ptr->cmp1[0];
            line0[1]=ptr->cmp1[1];
            line0[2]=ptr->cmp1[2];
            line0[3]=ptr->cmp1[3];
            line0[4]='\0';
            sprintf(hdptr,"%s",line0);
            hdptr+=5;
            line0[0]=ptr->cmp2[0];
            line0[1]=ptr->cmp2[1];
            line0[2]=ptr->cmp2[2];
            line0[3]=ptr->cmp2[3];
            line0[4]='\0';
            sprintf(hdptr,"%s",line0);
            hdptr+=5;
            line0[0]=ptr->cmp3[0];
            line0[1]=ptr->cmp3[1];
            line0[2]=ptr->cmp3[2];
            line0[3]=ptr->cmp3[3];
            line0[4]='\0';
            sprintf(hdptr,"%s",line0);
            break;
        }
        k=128;
        for(j=0;j<num_cha;j++)
        {
            for(l=rchan[j]*4;l<(rchan[j]*4)+4;l++)
            {
                head[k]=ptr->statio[l];
                k=k+1;
            }
            k++;
        }
        hdptr = head + 144;
        sprintf(hdptr,"%8d",cur_min);
        hdptr+=8;
        sprintf(hdptr,"%8d",cur_rcl);
        for(i=0;i<165;i++)
        {
            if(head[i]==0x0)
                head[i]=' ';
        }
        day=fopen(evnam,"w+");
        if(day == 0)
        {
            printf("DEC_MAT: a. Can not open dayfile\n");
            exit(0);
        }
        fwrite(head,reclength,1,day);
        fflush(day);
        pos=ftell(day);
        fclose(day);
//        printf("POS after new header %d\n",pos);
    }
    switch(sw)
    {
        case 0:
        for(lt=0;lt<ptr->rate;lt=lt+step)
        {
            for(k=0;k<num_cha;k++)
            {
                yy[smp]=(int)ptr->rsmp1[lt*num_cha+rchan[k]];
                if(bryter==1)
                {
                    ssttaa=(float)abs(yy[smp]);
                    llttaa=ssttaa;
                    bryter=0;
                }
                if(k==0)
                {   
                    ssttaa=ssttaa+((float)abs(yy[smp])-ssttaa)/100.;
                    llttaa=llttaa+((float)abs(yy[smp])-llttaa)/500.;
//                    if(ssttaa/llttaa >= 1.5)
//                        printf("Event in resampled data\n");
//printf("%10.3f %10.3f %10.3f\n",ssttaa,llttaa,ssttaa/llttaa);
                }
                smp++;
            }
        }
        break;
        case 1:
        for(lt=0;lt<ptr->rate;lt=lt+step)
        {
            for(k=0;k<num_cha;k++)
            {
                yy[smp]=(int)ptr->rsmp2[lt*num_cha+rchan[k]];
                if(k==0)
                {   
                    ssttaa=ssttaa+((float)abs(yy[smp])-ssttaa)/100.;
                    llttaa=llttaa+((float)abs(yy[smp])-llttaa)/500.;
//                   if(ssttaa/llttaa >= 1.5)
//                        printf("Event in resampled data\n");
//printf("%10.3f %10.3f %10.3f\n",ssttaa,llttaa,ssttaa/llttaa);
                }
                smp++;
            }
        }
        break;
    }
    id=id % 2;
    sw=(sw+1) % 2;
//    printf("%3d %6d %6d %6d\n",scan,yy[scan*3+0],yy[scan*3+1],yy[scan*3+2]);
    counter++;
/*--------------------------------------------------------------------------*/
/* when we have collected 1 minute of data in local storage, write to file  */
/*--------------------------------------------------------------------------*/
    if(counter== TIME)
    {
//("%5d seconds : %s\n",counter,input);
        
        day=fopen(evnam,"r+");
        if(day == 0)
        {
            printf("DEC_MAT: b. Can not open dayfile\n");
            exit(0);
        }
        fseek(day,pos,SEEK_SET);
//        printf("POS before writing record %d\n",pos);
        fwrite(yy,reclength,1,day);
        fflush(day);
        pos=ftell(day);
        fclose(day);
//        printf("POS after writing record %d\n",pos);
        counter=0;
        smp=0;
        cur_min++;
        cur_rcl=((reclength/4)/num_cha)*cur_min;
        hdptr=head+144;
        sprintf(hdptr,"%8d",cur_min);
        hdptr+=8;
        sprintf(hdptr,"%8d",cur_rcl);
        for(i=0;i<165;i++)
        {
            if(head[i]==0x0)
                head[i]=' ';
        }
        day=fopen(evnam,"r+");
        if(day == 0)
        {
            printf("DEC_MAT: c. Can not open dayfile\n");
            exit(0);
        }
        fseek(day,0,SEEK_SET);
        fwrite(head,reclength,1,day);
        fflush(day);
            Time_Conv1(input,&rec_time,ptr);
            YR=rec_time.year;
            MTH=rec_time.month;
            DAY=rec_time.day;
            HR=rec_time.hour;
            MIN=rec_time.min;
            SEC=rec_time.secs;
            TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MS_STOPP);
            DIFF=MS_STOPP-MS_START;
            SR_AVG=DIFF/(float)(TOTAL*60);
            SAMPLES=cur_rcl;
            TRUESR=DIFF/SAMPLES;
//printf("%4d %2d/%2d   %2d:%2d:%6.3f DIFF %8.3f MIN %5d SMP %5d True %11.8f\n",
//YR,MTH,DAY,HR,MIN,SEC,DIFF,cur_min,cur_rcl,TRUESR);
            fseek(day,0,SEEK_SET);
            fflush(day);
            fread(head,reclength,1,day);
            fflush(day);
            hdptr=head;
            hdptr+=64;
            sprintf(hdptr,"%10.8f",TRUESR);
            fseek(day,0,SEEK_SET);
fflush(day);
            fwrite(head,reclength,1,day);
fflush(day);
        fclose(day);
        records++;

/*--------------------------------------------------------------------------*/
/* when we have written TOTAL minutes of data to file, close it and open new*/
/*--------------------------------------------------------------------------*/
        if(records==TOTAL)
        {
            day=fopen(evnam,"r+");
            if(day == 0)
            {
                printf("DEC_MAT: a. Can not open dayfile\n");
                exit(0);
            }
//            Time_Conv1(input,&rec_time,ptr);
//            YR=rec_time.year;
//            MTH=rec_time.month;
//            DAY=rec_time.day;
//            HR=rec_time.hour;
//            MIN=rec_time.min;
//            SEC=rec_time.secs;
//            TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MS_STOPP);
//            DIFF=MS_STOPP-MS_START;
//            SR_AVG=DIFF/(float)(TOTAL*60);
//printf("SR_AVG: %10.8f\n",SR_AVG);
//            fseek(day,0,SEEK_SET);
//            fflush(day);
//            fread(head,reclength,1,day);
//            fflush(day);
//            hdptr=head;
//            hdptr+=64;
//            sprintf(hdptr,"%10.8f",SR_AVG/(float)ptr->rate);
//            fseek(day,0,SEEK_SET);
//fflush(day);
//            fwrite(head,reclength,1,day);
//fflush(day);
//            fclose(day);
            scan=TOTAL;
            records=0;
            cur_min=0;
            cur_rcl=0;
            pos=0;
        }
    }
}                                /* End of indefinite loop              */
}                                    /* End of main                         */
Time_Conv1(input,event_time,ptr)
int* input;
struct date* event_time;
smod* ptr;
{
char time[15],date[15],doys[5],car;
sscanf((char*)input,"%s %s %s",time,date,doys);
sscanf(time,"%2d%c%2d%c%f6.3\n",&(event_time->hour),&car,
                                &(event_time->min),&car,
                                &(event_time->secs));
sscanf(date,"%2d%c%2d%c%4d\n",  &(event_time->day),&car,
                                &(event_time->month),&car,
                                &(event_time->year));
sscanf(doys,"%3d\n",&(event_time->doy));
}
Event_Name1(rec_time, fnam,ptr,kan)
  struct date* rec_time;
  char* fnam;
  smod* ptr;
  int kan;
{   
int i;
int res;
/* The format of this event file is as follows:
   YYMM_DD_HHMM_SST.CCC_NN_P
   year,month,day,hour,mins,secs,T=temporary,station code, num of channels 
   P parameter set */
   
   char cyear[4];   /* help variable in cutting the century off year        */
   char* namptr= fnam;    /* index in fnam string                           */
   
 
sprintf(cyear, "%4d", rec_time->year);

sprintf(namptr, "%4s_", cyear);
namptr+=5;
sprintf(namptr, "%2d_", rec_time->month);
namptr+=3;
sprintf(namptr, "%2d_", rec_time->day);
namptr+=3;
sprintf(namptr, "%2d%2d_", rec_time->hour, rec_time->min);
namptr+=5;
sprintf(namptr, "%2dT.", (int) rec_time->secs);
namptr+=4;
strncpy(namptr, ptr->name, 3);                   /* station code            */
namptr+=3;
*namptr++ = '_';

res=3;
//sprintf(namptr, "%2d", ptr->nchstr[par]);   
sprintf(namptr, "%2d", kan);   

namptr+=2;
    sprintf(namptr, "_R");
/********************JUST FOR TESTING**********************/
/*sprintf(fnam,"9504_15_1230_30T.WNN_06_C");*/
/**********************************************************/
/* fill fnam with zeros in all blank spaces */
zero_fill1(fnam);

}
zero_fill1(string)
  char* string;
  
{
    int i;
    for(i=0; i< strlen(string); i++) 
     if(string[i] == ' ') string[i] = '0';
     
}





