/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:05/02/02                       */
/* Program:DIG_324                                                          */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from R.O.+C.A. digitiser 3 channels,    */
/* 24 bit and 50 Hz.                                                        */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>

unsigned char byt[3000];               /* Input buffer                      */
unsigned char byt2[200];

unsigned char one[20];
unsigned char *buf_ptr;
unsigned char *buf_ptr2;
unsigned char two[10];
unsigned char *two_ptr;

int           n;
int           lst_val1;
int           lst_val2;
int           lst_val3;
int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
//int           s;
int           i;                       /* counter                           */
int           fd;                      /* return value open share           */
int           nbyt;                    /* number of bytes to read           */
int           fh;                      /* input handler                     */
int           sts_snc;                 /* status after reading sync         */
int           sts_buf;                 /* status after reading data buffer  */
int           sec_ptr;
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
char          snd_tim[35];

char          msg[10];
char          stty_str[100];
char          port_str[12];

main()
{
pid_t         proxy;
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
int           clk_pps_id;
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
unsigned int  cop;
setprio(getpid(),12);                  /* set the priority of this process  */
dig_eda_id=qnx_name_attach(0,"dig_324"); /* attach this process as "dig_324"*/
if(dig_eda_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_324 attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_324_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_324_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
if(ptr->clktyp !=8)
{
  clk_pps_id=qnx_name_locate(0,"clk_pps",0,&cop);
  if(clk_pps_id == -1)
  {
    printf("DIG_324:can't locate CLK_PPS\n");
    exit(0);
  }
}

ptr->min_proxy=proxy;

ptr -> index=0;                        /* Initial buffer pointer            */
/*--------------------------------------------------------------------------*/
/* Open device for reading data from RO+CA digitizer                        */
/*--------------------------------------------------------------------------*/

sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -ohflow -lkhflow -ihpaged -osflow -ihflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);

/*--------------------------------------------------------------------------*/
/* open serial line where digitizer is connected                            */
/*--------------------------------------------------------------------------*/
if((fh=open(port_str,O_RDONLY | O_CACHE))==ERROR)
{
    printf("DIG_324: Can't open serial line %s\n",port_str);
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Read data from R.O+C.A. digitiser until 3 sync character is detected.    */
/*--------------------------------------------------------------------------*/

for(i=0;i<3;i++)
{
  ptr->alive[1]=1;                     /* tell I am alive                   */
  sts_snc=Sync3();
  switch(sts_snc)                      /* test if return value is ok        */
  {
    case 1:                            /* reading byte ok                   */
    break;
    case 0:                            /* timeout looking for sync          */
    printf("Not receiving data from digitiser\n");
    printf("Check if digitiser connected to correct port with correct \n");
    printf("baudrate\n");
    printf("Also check if output at all from digitiser.\n");
    exit(0);
    break;
    case -1:                           /* error reading data                */
    printf("Error reading data from digitiser\n");
    printf("Check if digitiser connected to correct port with correct \n");
    printf("baudrate\n");
    printf("Also check if output at all from digitiser.\n");
    exit(0);
    break;
  }
}
printf("DIG_324   : Receiving data from digitizer !\n");
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
  ptr->alive[1]=1;                   /* tel I am alive                    */
  Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
  Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                     /* At this point both processes are  */
                                     /* ready to proceed.                 */
                                     /* BUF_PIP is waiting to tag data    */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
  sts_buf=ED_3();
//printf("sts_buf= %d\n",sts_buf);
  switch(sts_buf)                      /* test if return value is ok        */
  {
    case 1:                            /* reading byte ok                   */
    Send(buf_pip_id,&snd_tim,&rec_msg,32,1); /* let buf_pip get the time-tag */
    break;
    case 0:                        /* timeout reading data buffer       */
    sec_ptr=HEADER;
//    ErrBuf();
//    goto goon;
    break;
    case -1:                       /* error reading data                */
//    sec_ptr=HEADER;
//    ErrBuf();
//    goto goon;
    break;
  }
/*--------------------------------------------------------------------------*/
/* convert sample values into right format and store in correct buffer      */
/*--------------------------------------------------------------------------*/
  sec_ptr=HEADER;
  Convert1();
/*--------------------------------------------------------------------------*/
/* send time info to BUF_PIP                                                */
/*--------------------------------------------------------------------------*/
//goon:
i=0;
//printf("DIG_ROR finished buffer: %d\n",ptr->index);
  Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
}
}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3()
{
register unsigned short sync_value = 0;
register unsigned short tst_val = 0;
unsigned short facit[30];
int lo;
int p;
int num_byt;
int cnt;
facit[0]=0xaa00;
facit[1]=0xaa01;
facit[2]=0xaa02;
facit[3]=0xaa03;
facit[4]=0xaa04;
facit[5]=0xaa05;
facit[6]=0xaa10;
facit[7]=0xaa20;
facit[8]=0xaa80;
cnt=0;
num_byt=ptr->rate * 3 * ptr->nchan;
buf_ptr=one;
for(;;)
{
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  n=dev_read(fh,buf_ptr,1,1,0,11,0,0);       /* read 1 byte from serial port*/
  byt[cnt]=one[0];
  cnt++;
  tst_val = tst_val << 8 | one[0];
  for(p=0;p<9;p++)
  {
//    if(tst_val == facit[p])
//      printf("control data in sample %2x\n",facit[p]);
  }
  if(cnt>=(num_byt))
  {
    sts_snc=Sync();
    switch(sts_snc)                    /* test if return value is ok        */
    {
      case 1:                          /* reading byte ok                   */
      break;
      case 0:                          /* timeout looking for sync          */
      break;
      case -1:                         /* error reading data                */
      break;
    }
    return(1);
  }
  sts_snc=Sync();
  switch(sts_snc)                      /* test if return value is ok        */
  {
    case 1:                            /* reading byte ok                   */
    break;
    case 0:                            /* timeout looking for sync          */
    break;
    case -1:                           /* error reading data                */
    break;
  }
  
}
return(-1);
}

/****************************************************************************/
/* Function to convert data buffers from RO+CA format                       */
/****************************************************************************/
void Convert1()
{
int tt;
register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register unsigned short vals;
register short verdis;
register int verdi;
register int v1,v2,v3;
static union CHAR_INTE
{
  char cadena[4];
  int numero;
  unsigned long numerol;
}INTEGER_CHAR;

int xx,yy;

unsigned short smp;
int num_byt;
short sample;
tt=0;
num_byt=ptr->rate * 3 * ptr->nchan;
buf_ptr= &byt[0];                     /* point to first data byte          */
xx=0;
yy=HEADER;
while(buf_ptr < &byt[num_byt])
{
  val=0;
  lo=(*buf_ptr++);
  me=(*buf_ptr++);
  hi=(*buf_ptr++);
  INTEGER_CHAR.cadena[2]=hi;
  INTEGER_CHAR.cadena[1]=me;
  INTEGER_CHAR.cadena[0]=lo;
  v1=INTEGER_CHAR.numerol;
  v1-=0x00800000L;
  ptr->ibuf[ptr->index][sec_ptr]=v1;
  sec_ptr++;
  lo=(*buf_ptr++);
  me=(*buf_ptr++);
  hi=(*buf_ptr++);
  INTEGER_CHAR.cadena[2]=hi;
  INTEGER_CHAR.cadena[1]=me;
  INTEGER_CHAR.cadena[0]=lo;
  v2=INTEGER_CHAR.numerol;
  v2-=0x00800000L;
  ptr->ibuf[ptr->index][sec_ptr]=v2;
  sec_ptr++;
  lo=(*buf_ptr++);
  me=(*buf_ptr++);
  hi=(*buf_ptr++);
  INTEGER_CHAR.cadena[2]=hi;
  INTEGER_CHAR.cadena[1]=me;
  INTEGER_CHAR.cadena[0]=lo;
  v3=INTEGER_CHAR.numerol;
  v3-=0x00800000L;
  ptr->ibuf[ptr->index][sec_ptr]=v3;
  sec_ptr++;

}
//lst_val1=v1;
//lst_val2=v2;
//lst_val3=v3;
}
/****************************************************************************/
/* Function to make zero buffer when error                                  */
/****************************************************************************/
void ErrBuf()
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register int verdi;
register int v1,v2,v3;


unsigned short smp;
short sample;
buf_ptr= &byt[0];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v1=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v2=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v3=verdi/256;
buf_ptr++;
buf_ptr++;

    switch(ptr->nchan)
    {
        case 1:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val2;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
}


Sync3()
{
  register unsigned short sync_value = 0;
  buf_ptr = byt;
  do
  {
    n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
    switch(n)
    {
      case -1:
      return(-1);
      break;
      case 0:
      return(0);
      break;
      case 1:       
      sync_value = sync_value << 8 | *buf_ptr;
      break;
    }
  } while(sync_value != 0xaa04);
//printf("snc dig_324\n");
  return(1);
}
Sync()
{
  register unsigned short sync_value = 0;
  buf_ptr2 = byt2;
  do
  {
    n=dev_read(fh,buf_ptr2,1,1,0,11,0,0);
    switch(n)
    {
      case -1:
      return(-1);
      break;
      case 0:
      return(0);
      break;
      case 1:       
      sync_value = sync_value << 8 | *buf_ptr2;
      break;
    }
  } while(sync_value != 0xaa04);
//printf("snc dig_324\n");
  return(1);
}
