/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:14/02/95                       */
/* Program:DIG_EDA                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from ED24. Determine start of buffers   */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
int           kval;
unsigned char byt[1500];               /* Input buffer                      */
char tidgps[100];
int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           err_cnt;                 /* error counter                     */
int           fd;                      /* return value open share           */
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
main()
{
char          stty_str[100];
char          port_str[12];
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
unsigned int  cop;
setprio(getpid(),12);                  /* set the priority of this process  */
err_cnt=0;                             /* set error counter to zero         */
dig_eda_id=qnx_name_attach(0,"dig_eda"); /* attach this process as "dig_eda"*/
if(dig_eda_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_EDA attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_EDA_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_EDA_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);

ptr -> index=0;                        /* Initial buffer pointer            */

/*--------------------------------------------------------------------------*/
/* Open device for reading data from ED_24                                  */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);

in=fopen(port_str,"r");
/*--------------------------------------------------------------------------*/
/* Read data from ED_24 until one 5 sync character is detected.             */
/*--------------------------------------------------------------------------*/
for(i=0;i<5;i++)
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Sync();
}
printf("DIG_EDA   : Receiving data from digitizer !\n");
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
    ptr->alive[1]=1;                   /* tel I am alive                    */
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
    Sync();                            /* find sync char. in received data  */
                                       
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
                                       /* signal.                           */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
    ED_3();
/*--------------------------------------------------------------------------*/
/* convert sample values into right format and store in correct buffer      */
/*--------------------------------------------------------------------------*/
    ConED_3();
/*--------------------------------------------------------------------------*/
/* send time info to BUF_PIP                                                */
/*--------------------------------------------------------------------------*/
    Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
}
}
/****************************************************************************/
/* Function to data buffers from a RD3                                      */
/****************************************************************************/
void    ED_3()
{
int l1,i1;
char tid[30];

numfread(byt,873);                     /* read in the header and data block */
if(numread<873)                        /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
for(l1=0;l1<68;l1++)
    tidgps[l1]=byt[l1+118];
tidgps[l1]='\0';
//printf("%s\n",tidgps);
//exit(0);
l1=0;
for(i1=190;i1<210;i1++)                  /* move time/date string into buffer */
{
    tid[l1]=byt[i1];
//    printf("%c",byt[i1]);

    l1=l1+1;
}
//printf("\n");
/* print time string into the send-message to be sent to buf_pipe           */
/*printf("%c%c %c%c %c%c.%c%c %c%c%c %c%c %c%c\n",*/
/*tid[0],tid[1],tid[3],tid[4],tid[6],tid[7],tid[9],tid[10],tid[11],*/
/*tid[14],tid[15],tid[17],tid[18],tid[20],tid[21]);*/

sprintf(snd_msg,"%c%c %c%c %c%c.%c%c %c%c%c %c%c %c%c\n",
tid[0],tid[1],tid[3],tid[4],tid[6],tid[7],tid[9],tid[10],tid[11],
tid[14],tid[15],tid[17],tid[18],tid[20],tid[21]);
kval=0;                                /* initially data is ok              */
snd_msg[29]='0';                       /* initially data is ok              */
if(byt[69] == '1')                     /* check received data for error     */
{
    kval=1;                            /* set error indicator to zero buffer*/
    snd_msg[29]='1';                   /* set error indicator               */
    err_cnt=err_cnt+1;
    printf("Error count= %d\n",err_cnt);
    if(err_cnt > 10000)
        err_cnt=0;
}

}
/****************************************************************************/
/* Function to convert data buffers from ED_24                               /
/****************************************************************************/
void ConED_3()
{
register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register int verdi;
register int v1,v2,v3;
indx=0;
buf_ptr= &byt[198];         /* point to first data byte */
while(buf_ptr < &byt[873])
{
val=0;
lo=(*buf_ptr++);
me=(*buf_ptr++);
hi=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v1=verdi/256;

val=0;
lo=(*buf_ptr++);
me=(*buf_ptr++);
hi=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v2=verdi/256;

val=0;
lo=(*buf_ptr++);
me=(*buf_ptr++);
hi=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v3=verdi/256;
if(kval==1)
{
    v1=0.0;
    v2=0.0;
    v3=0.0;
}
switch(ptr->nchan)
{
    case 1:
    ptr->ibuf[ptr->index][indx+HEADER]=v1;
    break;

    case 2:
    ptr->ibuf[ptr->index][indx+HEADER]=v1;
    ptr->ibuf[ptr->index][indx+1+HEADER]=v2;
    break;

    case 3:
    ptr->ibuf[ptr->index][indx+HEADER]=v1;
    ptr->ibuf[ptr->index][indx+1+HEADER]=v2;
    ptr->ibuf[ptr->index][indx+2+HEADER]=v3;
    break;
}
indx=indx+ptr->nchan;
}
}

void    Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
        } while(sync_value != 0x4d4f);
//printf("sync\n");
/*putchar(0x07);*/
}
