/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:31/05/00                       */
/* Program:DIG_EDN                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 01/06/00 8.22 t.u. new driver for Earth Data PS2400, new model           */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from ED24. Determine start of buffers   */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
#include "dig_edn.h"				/* DJS 29/08/00 */


/* define DEBUG to print useful info to debug file dig_eda.dbg */
/* #define DEBUG */
/* read a2 bytes from in and store them at a1                               */
#define numfread(a1,a2) numread = fread(a1,1,a2,in)

int           kval;
unsigned char byt[1500];               /* Input buffer                      */
int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           hold;                    /* switch to hold recording until GPS fix */
int           fd;                      /* return value open share           */
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
FILE		  *dbg_fp = (FILE *) 0;
int 		  first_time;			   /* initialisation or not 			*/
long 		  data_block_size;		   /* size of DAT block from ED24		*/

void main (int argc, char *argv[])
{
  int           dig_eda_id;              /* Process id for ED_24              */
  int           buf_pip_id;              /* Process id for BUF_PIP            */
  char          rep_msg[10];             /* reply message                     */
  char          rec_msg[10];             /* receive message                   */
  int			status;                  /* return value for ED3 function	    */
  unsigned int  cop;

  char stty_string[300];    /* string for serial port setup command DJS */
  char port_string[50];     /* string for serial port open command DJS */
  struct Earth_data_2400_235 header;  /* ED24 header structure DJS */
  int header_found;         /* have we found the start of an ED24 header DJS */
  

#ifdef DEBUG
dbg_fp = fopen ("dig_eda.dbg", "a");
#endif

  setprio(getpid(),12);                  /* set the priority of this process  */
  dig_eda_id=qnx_name_attach(0,"dig_eda"); /* attach this process as "dig_eda"*/
  if(dig_eda_id == -1)                   /* check if OK ?                     */
  {
    printf("DIG_EDN attach failed\n"); /* its not, EXIT                     */
    exit(0);
  }
  /*--------------------------------------------------------------------------*/
  /* open shared memory region                                                */
  /*--------------------------------------------------------------------------*/
  fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
  if(fd== -1) 
  {
    fprintf(stderr,"DIG_EDN_01 open failed:%s\n",strerror(errno));
    exit(1);
  }
  /*--------------------------------------------------------------------------*/
  /* set size of memory region                                                */
  /*--------------------------------------------------------------------------*/
  ltrunc(fd,sizeof(smod),SEEK_SET);
  /*--------------------------------------------------------------------------*/
  /* Compute pointer to shared memory region                                  */
  /*--------------------------------------------------------------------------*/
  ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
  if((int)ptr == -1) 
  {
    fprintf(stderr,"DIG_EDN_02 mmap failed : %s\n",strerror(errno));
    exit(1);
  }

  /*--------------------------------------------------------------------------*/
  /* locate process BUF_PIP and get its process id                            */
  /*--------------------------------------------------------------------------*/
  buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);

  ptr -> index=0;                        /* Initial buffer pointer            */

  /*** end of Seislog startup bumf ***/

  first_time = TRUE;
  header_found = FALSE;

  /*** open serial port using parameters from par and 
       hard_coded stty settings DJS ***/
  sprintf (port_string, "/dev/ser%d", ptr->prt_dig);
  sprintf (stty_string,
          "stty +raw +clocal -ihflow -ohflow -isflow -osflow -lkhflow baud=%d <%s",
          ptr->bd_dig, port_string);
  system (stty_string);
  in = fopen (port_string, "r");


  /* start receiving from the digitiser and try to find a start of header */
  do 
  {
	status = read_ED2400 (&header, &header_found);
	if (status != RB_OK)
	{
		printf ("DIG_EDN: Error in input stream\n");
		exit (1);
	}
  } while (header_found == FALSE);
  printf("DIG_EDN   : Receiving data from digitizer !\n");

  /*** read the rest of a block ***/
  /*** loop if GPS has not sync'ed; other error conditions are tested ***/
  do
  { 
	status = read_ED2400 (&header, &header_found);
  } while ((status != RB_OK) || (header_found == TRUE));
  Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */

  /*** main loop ***/
  for(;;) 
  {
    ptr->alive[1]=1;                   /* tel I am alive                    */
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
	do 
	{
		status = read_ED2400 (&header, &header_found);
		if (status != RB_OK)
		{
			printf ("DIG_EDN: Error in input stream\n");
			exit (1);
		}
	} while (header_found == FALSE);                                       
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
                                       		/* signal.                      */
	/* header found == TRUE here always */
	do   /* this loop only executes once, unless there has been an error */
	{
		status = read_ED2400 (&header, &header_found);
	} while (header_found == TRUE); 
	if (status == RB_OK)
    	Send(buf_pip_id,&snd_msg,&rec_msg,30,1);   /* send time info to BUF_PIP */
	else
    	Send(buf_pip_id,&snd_msg,&rec_msg,1,1);
  }
}


/************************************************************************
 * read_ED2400
 *
 * Description: read a one second block of ED24 data
 *
 * Input parameters: header - pointer to the one second header
 * Output parameters: header_found - pointer to true/false indicator
 * Returns: RB_END_OF_FILE - an end of file was found
 *			RB_FILE_ERROR - there was an error on the input stream
 *			RB_ERROR - unexpected data was found in the input stream
 *			RB_DATA_TOO_BIG - there is too much data
 *		    RB_OK - the block was found OK
 *			RB_NOT_YET_IN_SYNC - GPS not yet in sync
 *
 * Comments: A mixture of Terje's ED_3 function and Simon's code
 * to allow for prevention of data loss. DJS 30/08/00.
 * 
 * Can return OK from two possibilities:
 *   (a) start of header located, ready to go on and read;
 *   (b) finished reading block for this second.
 *
 ************************************************************************/
int read_ED2400 (struct Earth_data_2400_235 *header, int *header_found)
{
  int error_flag, count, status, byte;
  int checksum, checksum_found, low_byte, high_byte;
  static int time_diff, time_diff2;		/* for when GPS not in sync */
  char time_inc[7];
  unsigned char *byt_ptr;

  /* this array is the largest data block you could ever get from an
   * ED 2400 - that is 4 bytes/sample, 3 components, 1000Hz sample rate-
   * we add a little extra just to be on the safe side */
  static char data_bytes [15000];

  if (first_time == TRUE)
  {
	data_block_size = 0l;
  }

  if (*header_found == FALSE)
  {
	do
	{
		status = get_next_byte (in, data_block_size, &byte);
		if (status == GNB_EOF)	return RB_END_OF_FILE;
		if (status == GNB_ERROR) return RB_FILE_ERROR;
	} while (status != GNB_FOUND_HEADER);
    *header_found = TRUE;
    return RB_OK;
  }

  *header_found = FALSE;

  /* read the header */
  for (count = 0, byt_ptr = (char *) header;
       count < sizeof (struct Earth_data_2400_235);
	   count ++, byt_ptr ++)
  {
	status = get_next_byte (in, data_block_size, &byte);
	switch (status)
	{
	case GNB_EOF:
	  return RB_END_OF_FILE;
	case GNB_ERROR:
	  return RB_FILE_ERROR;
	case GNB_FOUND_BYTE:
	  *byt_ptr = byte;
	  break;
	case GNB_FOUND_HEADER:
	  *header_found = TRUE;
	  return RB_ERROR;
	default:
	  return RB_ERROR;
	}
  }

  /* check for errors first time around */
  if (first_time == TRUE)
  {
    if(ptr->clktyp!=0)                     //TU 21/04/01
    {
	  if (header->block_at_last_sync <= 0)
	  {
		printf ("Waiting for GPS to lock . . .\n");
		return RB_NOT_YET_IN_SYNC;
	  }
    }
    if (header->bytes_per_sample < 1 || header->bytes_per_sample > 4)
	{
		printf("DIG_EDN Error: bytes per sample outside possible limits\n");
		system ("/home/seislog/stp");
		exit(1);
	}
    if (header->n_components <= 0 || header->n_components > 4) 
	{
		printf("DIG_EDN Error: number of components outside possible limits\n");
		system ("/home/seislog/stp");
		exit(1);
	}
	if (header->n_components != ptr->nchan)
	{
		printf("DIG_EDN Error: mismatch in number of channels\n");
		printf("\tNo of channels in parameter file %d", ptr->nchan);
		printf(" no of components in digitiser %d\n", header->n_components);
		system ("/home/seislog/stp");
		exit(1);
	}
	if (header->samples_per_block != (ptr->rate * ptr->nchan))
	{
		printf("DIG_EDN Error: mismatch in sample rate\n");
		printf("\tSample rate in parameter file = %d Hz * %d channels = %d\n",
				 ptr->rate, ptr->nchan, ptr->rate * ptr->nchan);
		printf("\tSample rate in digitiser %d\n", header->samples_per_block);
		printf("\tAdjust sample rate in par to %f\n", 
				1.0 / ((float)header->samples_per_block / (float)ptr->nchan));
		system ("/home/seislog/stp");
		exit (1);
	}
	data_block_size = (long)(header->samples_per_block * header->bytes_per_sample);	/* expected */
	first_time = FALSE;
  }

  /* check for bad data */
  error_flag = FALSE;
  kval = 0;							/* Terje's global */
  if (header->test_pattern[0] != 0x22) error_flag = TRUE;
  if (header->test_pattern[1] != 0x22) error_flag = TRUE;
  if (header->test_pattern[2] != 0x00) error_flag = TRUE;
  if (header->test_pattern[3] != 0x55) error_flag = TRUE;
  if (header->test_pattern[4] != 0x55) error_flag = TRUE;
  if (header->test_pattern[5] != 0x00) error_flag = TRUE;
  if (header->test_pattern[6] != 0xff) error_flag = TRUE;
  if (header->test_pattern[7] != 0xff) error_flag = TRUE;

  if (error_flag == TRUE)
  {
    kval=1;                            /* set error indicator to zero buffer*/
    return RB_ERROR;
  }

  /* extract GPS message */
  byt_ptr = header->gps_message;
if(ptr->clktyp==8)
{
  for (count = 0; count < 9; count++)
  {
  	byt_ptr = strchr (byt_ptr, ',') + 1;
  	switch (count)
  	{
  	case 0:               
    	strcpy (snd_msg, "xx xx xx.000 xx xx xx 0      0");
    	snd_msg [0] = *byt_ptr;
    	snd_msg [1] = *(byt_ptr +1);
    	snd_msg [3] = *(byt_ptr +2);
    	snd_msg [4] = *(byt_ptr +3);
    	snd_msg [6] = *(byt_ptr +4);
    	snd_msg [7] = *(byt_ptr +5);
    	break;
  	case 8:
    	snd_msg [13] = *byt_ptr;
    	snd_msg [14] = *(byt_ptr +1);
    	snd_msg [16] = *(byt_ptr +2);
    	snd_msg [17] = *(byt_ptr +3);
    	snd_msg [19] = *(byt_ptr +4);
    	snd_msg [20] = *(byt_ptr +5);
    	break;
  	}
  }
}

  /* check if GPS is in sync */
  if (header->block_count == header->block_at_last_sync)
  {
    ptr->TimFlg[0]=1;		/* flag as in sync */
    ptr->clktyp=8;
  }
  else
  {
	if(ptr->clktyp==8)
	  ptr->TimFlg[0]=2;		/* flag as not in sync */
	if(ptr->clktyp==0)       
	  ptr->TimFlg[0]=3;
	if (header->block_count > header->block_at_last_sync)
	{
		time_diff = header->block_count - header->block_at_last_sync;
    	if (time_diff < 1000000) sprintf(time_inc,"%6d",time_diff);
	}
	else 				/* block count has rolled over since last valid GPS */
	{
		time_diff2 = time_diff + header->block_count;
    	if (time_diff2 < 1000000) sprintf(time_inc,"%6d",time_diff2);
	}
	memcpy (&snd_msg[22], time_inc, 6);
    snd_msg[29]='1';                   /* set error indicator               */
  }

  /* try to find the data block */
  for (count = 0; count < 8; count++)
  {
    status = get_next_byte (in, data_block_size, &byte);
    switch (status)
    {
    case GNB_EOF:
	  return RB_END_OF_FILE;
    case GNB_ERROR:
	  return RB_FILE_ERROR;
    case GNB_FOUND_BYTE:
	  if (count >= 7) return RB_ERROR;
	  break;
    case GNB_FOUND_HEADER:
	  *header_found = TRUE; 
	  return RB_ERROR;
	case GNB_FOUND_DATA:
	  if (count < 7) return RB_ERROR;
	  break;
    default:
	  return RB_ERROR;
    }
  }

  /* get the data */
  for (count = 0; count < data_block_size; count++)
  {
	status = get_next_byte (in, data_block_size, &byte);
    switch (status)
    {
    case GNB_EOF:
	  return RB_END_OF_FILE;
    case GNB_ERROR:
	  return RB_FILE_ERROR;
    case GNB_FOUND_BYTE:
	  data_bytes [count] = byte;
	  break;
    case GNB_FOUND_HEADER:
	  *header_found = TRUE; 
	  return RB_ERROR;
    default:
	  return RB_ERROR;
    }
  }

  byt_ptr = data_bytes;
  convert_ED2400 (byt_ptr);

  /* look for a checksum block */
  checksum_found = FALSE;
  for (count = 0; count < 8; count ++)
  {
    status = get_next_byte (in, data_block_size, &byte);
    switch (status)
    {
    case GNB_EOF:
	  return RB_END_OF_FILE;
    case GNB_ERROR:
	  return RB_FILE_ERROR;
    case GNB_FOUND_BYTE:
	  if (count >= 7) return RB_ERROR;
	  break;
    case GNB_FOUND_HEADER:
	  *header_found = TRUE; 
	  break;
	case GNB_FOUND_CHECKSUM:
	  if (count < 7) return RB_ERROR;
	  checksum_found = TRUE;
	  break;
    default:
	  return RB_ERROR;
    }
  }

  /* read the checksum block */
  if (checksum_found)
  {
    for (count=0; count<4; count++)
    {
      status = get_next_byte (in, data_block_size, &byte);
      switch (status)
      {
      case GNB_EOF:
        return RB_END_OF_FILE;
      case GNB_ERROR:
	    return RB_FILE_ERROR;
      case GNB_FOUND_BYTE:
	    switch (count)
	    {
	    case 0: break;
	    case 1: break;
	    case 2: low_byte = byte; break;
	    case 3: high_byte = byte; break;
	    }
	    break;
      case GNB_FOUND_HEADER:
	    *header_found = TRUE; 
		return RB_ERROR;
      default:
	    return RB_ERROR;
      }
	}
  }

  /* calculate the checksum */
  if (checksum_found)
    checksum = (int) (low_byte | (high_byte < 8));
  else
    checksum = -1;

  /* normal successful completion */
  return RB_OK;

}

/************************************************************************
 * convert_ED2400
 *
 * Description: read the data block into Seislog buffer
 *
 * Input parameters: data_bytes - pointer to the data
 * Output parameters: none
 * Returns: none
 *
 * Comments: Terje's ConED_3 function, unchanged except that
 * the data is passed in via a parameter
 * 
 ************************************************************************/
void convert_ED2400 (unsigned char *data_bytes)
{
  register unsigned char *buf_ptr;
  register unsigned char lo,me,hi;
  register unsigned int val;
  register int count;
  register int v1,v2,v3;
  indx=0;

  buf_ptr = data_bytes;         /* point to first data byte */

#ifdef DEBUG
fprintf (dbg_fp, "Buffer start\n");
#endif

  for (count = 0; count < ptr->rate; count++)
  {
	val=0;
	lo=(*buf_ptr++);
	me=(*buf_ptr++);
	hi=(*buf_ptr++);
	val = (hi << 16) | (me << 8) | lo;
	if (hi & 0x80) v1 = val | 0xff000000;
	else v1 = val;

	val=0;
	lo=(*buf_ptr++);
	me=(*buf_ptr++);
	hi=(*buf_ptr++);
	val = (hi << 16) | (me << 8) | lo;
	if (hi & 0x80) v2 = val | 0xff000000;
	else v2 = val;

	val=0;
	lo=(*buf_ptr++);
	me=(*buf_ptr++);
	hi=(*buf_ptr++);
	val = (hi << 16) | (me << 8) | lo;
	if (hi & 0x80) v3 = val | 0xff000000;
	else v3 = val;

	if(kval==1)
	{
    	v1=0;
    	v2=0;
    	v3=0;
	}

#ifdef DEBUG
fprintf (dbg_fp, "%d\n", v1);
#endif

	switch(ptr->nchan)
	{
    case 1:
    	ptr->ibuf[ptr->index][indx+HEADER]=v1;
    	break;

    case 2:
    	ptr->ibuf[ptr->index][indx+HEADER]=v1;
    	ptr->ibuf[ptr->index][indx+1+HEADER]=v2;
    	break;

    case 3:
    	ptr->ibuf[ptr->index][indx+HEADER]=v1;
    	ptr->ibuf[ptr->index][indx+1+HEADER]=v2;
    	ptr->ibuf[ptr->index][indx+2+HEADER]=v3;
    	break;
	}
	indx=indx+ptr->nchan;
  }
}



/************************************************************************
 * get_next_byte
 *
 * Description: get the next byte from the input stream, checking for
 *				one of the header synchronisation sequences
 *
 * Input parameters: fp - the stream to read from
 *					 sizeof_data - expected size of the next data block
 * Output parameters: byte - the byte read (if any)
 * Returns: GNB_ERROR - if there was an error on the stream
 *			GNB_EOF - if the stream returned end of file
 *			GNB_FOUND_HEADER - found "MOD" in the data stream
 *			GNB_FOUND_DATA - found "DAT" in the data stream
 *			GNB_FOUND_CHECKSUM - found "SUM" in the data stream
 *			GNB_FOUND_BYTE - the next byte
 *
 * Comments: byte contains the most recently read byte for GNB_FOUND_HEADER,
 * GNB_FOUND_DATA, GNB_FOUND_CHECKSUM and GNB_FOUND_BYTE
 *
 * NB Added to Seislog driver 29/08/00, based on Simon's code, DJS
 ************************************************************************/
int get_next_byte (FILE *fp, long sizeof_data, int *byte)
{

  int count;
  static char history [9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
  
  
  /* get the next byte */
  *byte = fgetc (fp);
  if (ferror (fp)) return GNB_ERROR;
  if (feof (fp)) return GNB_EOF;
  
  /* shuffle the history data down */
  for (count=1; count<8; count++) history [count -1] = history [count];
  history [7] = *byte;
  
  /* check for one of the special sequences */
  if (! strcmp (history, "MOD"))
  {
    if ((long) (*((long *) (&history[4]))) == (long) sizeof (struct Earth_data_2400_235))
	  return GNB_FOUND_HEADER;
  }
  if (! strcmp (history, "DAT"))
  {
    if ((long) (*((long *) (&history[4]))) == sizeof_data)
      return GNB_FOUND_DATA;
  }
  if (! strcmp (history, "SUM"))
  {
    if ((long) (*((long *) (&history[4]))) == 4l)
      return GNB_FOUND_CHECKSUM;
  }
  return GNB_FOUND_BYTE;
  
}
