/*************************************************************************
 * dig_edn.h - include file for dig_edn.c - earth data digitiser driver
 *************************************************************************/
 
/* use this structure to overlay the Earth Data 2400-235 digitiser header
 * NOTE - none of the strings are NULL terminated */ 
struct Earth_data_2400_235
{
  char device_id [12];			/* ASCII device identifier */
  char version [7];				/* device version number, followed by ' ' */
  char serial_number [5];		/* device serial number, followed by ' ' */
  char test_pattern [8];		/* 0x22, 0x22, 0x00, 0x55, 0x55, 0x00,
  								 * 0xff, 0xff */
  long block_count;				/* count of (1 sec.) blocks since power up */
  short n_components;			/* number of data components: 1, 2 or 3 */
  short samples_per_block;		/* number of samples in a block */
  short bytes_per_sample;		/* number of bytes / samples: 1, 2, 3 or 4 */
  char filter_type [2];			/* 'LP' or 'MP' */
  short filter_3_deci;			/* decimation factor for filter 3 */
  short filter_4_deci;			/* decimation factor for filter 4 */
  short pll_phase_error;		/* phase error in 1 sec. phase-locked-loop */
  char gain_name [2];			/* 'Hi' or 'Lo' */
  long gain [3];				/* gain for component 1-3 */
  long offset [3];				/* 0 offset for component 1-3:
  								 * sample[n] = (raw[n] - offset[n]) * gain[n] */
  short low_res_adc [8];		/* adc[0] - V(in) = (adc[0]*10.9)/2700)+5
  								 * adc[1] - Supply current = adc[1]*22/170
								 * adc[2] - Temperature = adc[2]/10-50
								 * adc[3] - V(supply out) = (adc[3]*10.9)/2700)+5
								 * adc[4] - User in 1 = adc[4]/1000
								 * adc[5] - User in 2 = adc[5]/1000
								 * adc[6] - User in 3 = adc[6]/1000
								 * adc[7] - V(ref) = adc[7]/1000 */
  char coupling [2];			/* 'AC' or 'DC' */
  long max_val [3];				/* not used */
  short gps_flags;				/* not used */
  long block_at_last_sync;		/* block_count value at the last GPS sync */
  char gps_message [72];		/* NMEA $GPRMC message */
};

/* standard Boolean definitions */
#define TRUE				1
#define FALSE				0

/* return codes for read_block() */
#define RB_OK				0
#define RB_FILE_ERROR		-1
#define RB_END_OF_FILE		-2
#define RB_DATA_TOO_BIG		-3
#define RB_ERROR			-4
#define RB_NOT_YET_IN_SYNC	-5

/* return codes for get_next_byte () */
#define GNB_ERROR			-1
#define GNB_EOF				-2
#define GNB_FOUND_HEADER	1
#define GNB_FOUND_DATA		2
#define GNB_FOUND_CHECKSUM	3
#define GNB_FOUND_BYTE		4

/* forward declarations */
void main (int argc, char *argv[]);
int read_ED2400 (struct Earth_data_2400_235 *header, int *header_found);
void convert_ED2400 (unsigned char *data_bytes);
int get_next_byte (FILE *fp, long sizeof_data, int *byte);
