/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:07/03/98                       */
/* Program:DIG_EVR                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from Everest.Determine start of buffers */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
int           k;
int           cn;
int           ix;
int           sp;
int           start;
int           kval;
int           tel;
int           stream;
int           prt;
int           drop;
int           drop2;
int           signal;
int           bytes;
int teller;
unsigned short sn = 0;
unsigned short chk;
unsigned short bt;
unsigned short chkd;
unsigned short sn;
unsigned short st;
unsigned short stt;
unsigned int sequence;
unsigned int sss;
unsigned short mss;
unsigned char chk_hdr;
unsigned short chk_dat;
unsigned char byt[3000];               /* Input buffer                      */
unsigned char vb[3000];
unsigned char ts[50];

int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
int           p;
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           err_cnt;                 /* error counter                     */
int           fd;                      /* return value open share           */
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
int           speed;
int           sr;
int           rdbyt;
int           strom;
int           skip;
main()
{
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
unsigned int  cop;
prt=0;
signal=1;
skip=0;
setprio(getpid(),12);                  /* set the priority of this process  */
err_cnt=0;                             /* set error counter to zero         */
if(signal==0)
{
dig_eda_id=qnx_name_attach(0,"dig_evr"); /* attach this process as "dig_eda"*/
if(dig_eda_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_EVR attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_GBV_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_EVR_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
if(signal==1)
  buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);

ptr -> index=0;                        /* Initial buffer pointer            */

/*--------------------------------------------------------------------------*/
/* Open device for reading data from GeoSys digitizer                       */
/*--------------------------------------------------------------------------*/
ptr->baud=38400;
printf("BAUD=%d\n",ptr->baud);

switch(ptr->bd_dig)
{
case 9600:
system("stty +raw -osflow baud=9600 </dev/ser1");
break;
case 14400:
system("stty +raw -osflow baud=14400 </dev/ser1");
break;
case 19200:

system("stty +raw -ohflow -lkhflow -ihpaged -osflow baud=19200 </dev/ser1");
break;
case 38400:
system("stty +raw -osflow baud=38400 </dev/ser1");
system("stty +raw -ohflow -lkhflow -ihpaged -osflow baud=38400 </dev/ser1");

break;
case 28800:
system("stty +raw -osflow baud=28800 </dev/ser1");
break;
}
in=fopen("/dev/ser1","r");
if(in==0)
{
  printf("Cant open serial port\n");
  exit(0);
}
printf("Serial line opened\n");
/*--------------------------------------------------------------------------*/
/* Read data from GeoSys until one 5 sync character is detected.            */
/*--------------------------------------------------------------------------*/
for(i=0;i<2;i++)
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Sync();
}
strom=0;
printf("DIG_EVR   : Receiving data from digitizer !\n");
if(signal==1)
  Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
    ptr->alive[1]=1;                   /* tel I am alive                    */
if(signal==1)
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
if(signal==1)
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
//    Sync();                            /* find sync char. in received data  */
//    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
                                       /* signal.                           */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
    for(strom=0;strom<3;strom++)
    {
      do
      {
        Sync();
        skip=ED_3();
        if(skip==1)
        {
          printf("checsum error loop for sync\n");
          strom=2;
          goto zero;
        }
      }while(stream != strom);
zero:
      if(signal==1)
      {
        if(strom==2)
          Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
      }
/*--------------------------------------------------------------------------*/
/* convert sample values into right format and store in correct buffer      */
/*--------------------------------------------------------------------------*/
printf("%d\n",stream);
      ConED_3();
    }

/*--------------------------------------------------------------------------*/
/* send time info to BUF_PIP                                                */
/*--------------------------------------------------------------------------*/
//printf("DIG_GE1 finished buffer: %d\n",ptr->index);
if(signal==1)
    Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
}
}
/****************************************************************************/
/* Function to data buffers from a RD3                                      */
/****************************************************************************/
ED_3()
{
int l1,i1;
char tid[30];
numfread(byt,9);                       /* read in the header block          */
if(numread<(9))                        /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
chk_hdr=0;                             /* compute checksum header           */
for(l1=0;l1<8;l1++)
  chk_hdr=chk_hdr+byt[l1];
chk=byt[8];
if(chk_hdr != chk)
{
  printf("checksum error header\n");
  skip=1;
  return(skip);
}else{
  skip=0;
}


bt=byt[6];                             /* extract number of bytes in data   */
bt=bt<<8 | byt[7];
bytes=bt;                              /* number of bytes to read           */

numfread(vb,16);                      /* read header in data block         */
if(numread<(16))                       /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
sn=0;
ix=0;
for(k=0;k<16;k++)
{
  sn = sn << 8 | vb[k];
  if(sn==0x5cdd || sn==0x5cdc)
  {
    printf("Escape sequence in data header at %d %2x %2x %2x %2x\n",k,vb[k-1],vb[k],vb[k+1]);
    numfread(byt,1);               /* read data block                   */

    if(sn==0x5cdd)
    {
      ix=ix-1;
      ts[ix]=0xc0;
      ix=ix+1;
    }
    if(sn==0x5cdc)
    {
      ix=ix-1;
      ts[ix]=0x5c;
      ix=ix+1;
    }
  }else{
    ts[ix]=vb[k];
    ix=ix+1;
  }
}

st=ts[2];                             /* extract stream number             */
//printf("stream: %d nbytes: %d\n",st,bytes);
if(bytes != 320)
{
  printf("BYTES=%d\n",bytes);
}

numfread(vb,(bytes-16));               /* read data block                   */
if(numread<(bytes-16))                 /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
chkd=vb[30];
chk_dat=0;
for(ix=0;ix<300;ix++)
  chk_dat=chk_dat+vb[ix];

for(ix=295;ix<306;ix++)
  printf("%2x ",vb[ix]);
printf("   %4x",chk_dat);
printf("\n");
chkd=chkd << 8 | vb[331];
sn=0;
ix=0;
teller=0;
for(k=0;k<(bytes-16);k++)
{
  sn = sn << 8 | vb[k];
  if(sn==0x5cdd || sn==0x5cdc)
  {
    printf("Escape sequence data block at %d %2x %2x %2x %2x\n",k,vb[k-1],vb[k],vb[k+1]);
    teller=teller+1;
    if(sn==0x5cdd)
    {
      ix=ix-1;
      byt[ix]=0xc0;
      ix=ix+1;
    }
    if(sn==0x5cdc)
    {
      ix=ix-1;
      byt[ix]=0x5c;
      ix=ix+1;
    }
  }else{
    byt[ix]=vb[k];
    ix=ix+1;
  }
}
k=0;
//for(ix=0;ix<1;ix++)
//{
//  printf("%3d",ix);
//  printf(" %2x%2x%2x %2x%2x%2x %2x%2x%2x %2x%2x%2x %2x%2x%2x\n",
//byt[k],byt[k+1],byt[k+2],
//byt[k+3],byt[k+4],byt[k+5],
//byt[k+6],byt[k+7],byt[k+8],
//byt[k+9],byt[k+10],byt[k+11],
//byt[k+12],byt[k+13],byt[k+14]);
//k=k+15;

//}
//exit(0);
//printf("%5d %2d %2d %5d %8d %3d\n",sn,st,stt,sequence,sss,mss);


//chk_dat=0;
//for(l1=0;l1<(bytes-16);l1++)
//  chk_dat=chk_dat+vb[l1];
//chkd=vb[302];
//printf("%2x %2x\n",chk_hdr,chk);
//chkd=chkd << 8 | vb[303];


//if(chk==chk_hdr)
//{
//if(st==0)
//  printf("%5d %2d %2d %5d %8d %3d ok %4x  %4x  %2x %2x %2x %2x\n",
//sn,st,stt,sequence,sss,mss,chk_dat,chkd,vb[328],byt[329],byt[330],byt[331]);
//  drop2=0;
//}else{
//  printf("%5d %2d %2d %5d %8d %3d --\n",sn,st,stt,sequence,sss,mss);
//  drop2=1;
//}
//printf("%8d %8d   %2x %2x\n",chk,chk_dat,chk,chk_dat);
stream=st;
//if(prt==1)
//  printf("Stream: %2d\n",stream);
  return(skip);

}
/****************************************************************************/
/* Function to convert data buffers from ED_24                               /
/****************************************************************************/
void ConED_3()
{
register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
unsigned int smp;
int sample;
register int verdi;
register int v1,v2,v3;
int t;
int ch;
int ix;
//for(t=0;t<3;t++)
//{
//  for(ix=0;ix<100;ix++)
//  {
//    ptr->ibuf[ptr->index][3*sp+t+HEADER]=ptr->dc[t];
//  }
//}

ix=0;
indx=0;
if(skip==0)
{
  start=0;
  for(sp=0;sp<100;sp++)
  {

    val=0;
    hi=byt[start];
    start++;
    me=byt[start];
    start++;
    lo=byt[start];
    start++;

    smp=hi;
    smp=smp<<8 | me;
    smp=smp<<8 | lo;
    smp=smp<<8;
    smp=smp<<1;
    sample=smp;
    sample=sample/512;
    if(prt==1)
      printf("Sample: %d\n",sample);
    v1=sample;
    ptr->ibuf[ptr->index][3*sp+stream+HEADER]=v1;
//    if(ix<5)
//      printf("%3d %2d index: %6d %2x%2x%2x %8d\n",
//sp,stream,3*sp+stream+HEADER,hi,me,lo,v1);
ix++;
  }

}else{
  for(ix=0;ix<3;ix++)
  {
    start=0;
    for(sp=0;sp<100;sp++)
    {
      v1=ptr->dc[ix];
      ptr->ibuf[ptr->index][3*sp+ix+HEADER]=v1;
    }
  }
}
ix=16;
for(t=0;t<10;t++)
{
//  printf("%3d %8d %8d %8d\n",t,
//  ptr->ibuf[ptr->index][ix+HEADER],
//  ptr->ibuf[ptr->index][ix+HEADER+1],
//  ptr->ibuf[ptr->index][ix+HEADER+2]);
  ix=ix+3;
}
//exit(0);
}

void    Sync()
{
int cn=0;
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
unsigned short no_bytes=0;
register unsigned int snc=0;
buf_ptr = byt;
do
{
  numfread(buf_ptr,1);
//printf("%2x\n",byt[0]);

  if(numread == 0)
  {
    clearerr(in);
  }else{
    snc = snc << 8 | *buf_ptr;
    snc=snc & 0x00ffffff;
  }
} while(snc != 0x00c0c0c0);
//printf("snc\n");
}
