/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:07/03/98                       */
/* Program:DIG_GBV-116                                                      */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from Geosys. Determine start of buffers */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
int           kval;
unsigned char byt[3000];               /* Input buffer                      */
int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           err_cnt;                 /* error counter                     */
int           fd;                      /* return value open share           */
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
int           speed;
int           sr;
int           rdbyt;
main()
{
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
unsigned int  cop;
setprio(getpid(),12);                  /* set the priority of this process  */
err_cnt=0;                             /* set error counter to zero         */
dig_eda_id=qnx_name_attach(0,"dig_gbv"); /* attach this process as "dig_eda"*/
if(dig_eda_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_GBV attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_GBV_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_GBV_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);

rdbyt=ptr->rate*3*ptr->nchan+57;
printf("rdbyt:%d rate:%d\n",rdbyt,ptr->rate);
//exit(0);
ptr -> index=0;                        /* Initial buffer pointer            */

/*--------------------------------------------------------------------------*/
/* Open device for reading data from GeoSys digitizer                       */
/*--------------------------------------------------------------------------*/
printf("BAUD=%d\n",ptr->baud);
switch(ptr->bd_dig)
{
case 9600:
system("stty +raw -osflow baud=9600 </dev/ser1");
break;
case 14400:
system("stty +raw -osflow baud=14400 </dev/ser1");
break;
case 19200:
system("stty +raw -osflow baud=19200 </dev/ser1");
break;
case 38400:
system("stty +raw -osflow baud=38400 </dev/ser1");
break;
case 28800:
system("stty +raw -osflow baud=28800 </dev/ser1");
break;
}
in=fopen("/dev/ser1","r");
/*--------------------------------------------------------------------------*/
/* Read data from GeoSys until one 5 sync character is detected.            */
/*--------------------------------------------------------------------------*/
for(i=0;i<5;i++)
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Sync();
}
printf("DIG_GBV   : Receiving data from digitizer !\n");
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
    ptr->alive[1]=1;                   /* tel I am alive                    */
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
    Sync();                            /* find sync char. in received data  */
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
                                       /* signal.                           */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
    ED_3();
/*--------------------------------------------------------------------------*/
/* convert sample values into right format and store in correct buffer      */
/*--------------------------------------------------------------------------*/
    ConED_3();
/*--------------------------------------------------------------------------*/
/* send time info to BUF_PIP                                                */
/*--------------------------------------------------------------------------*/
//printf("DIG_GE1 finished buffer: %d\n",ptr->index);
    Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
}
}
/****************************************************************************/
/* Function to data buffers from a RD3                                      */
/****************************************************************************/
void    ED_3()
{
int l1,i1;
char tid[30];

numfread(byt,rdbyt);                     /* read in the header and data block */
if(numread<rdbyt)                        /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
//for(l1=40;l1<60;l1++)
//{
//    printf("%3d %x %d\n",l1,byt[l1],byt[l1]);
//    getch();
//}
//exit(0);
}
/****************************************************************************/
/* Function to convert data buffers from ED_24                               /
/****************************************************************************/
void ConED_3()
{
register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
unsigned short smp;
short sample;
register int verdi;
register int v1,v2,v3;
indx=0;
buf_ptr= &byt[57];         /* point to first data byte */
while(buf_ptr < &byt[rdbyt])
{
val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++); /* in this case dummy */
smp=hi;
smp=smp<<8 | me;
sample=smp;
v1=sample;

hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
smp=hi;
smp=smp<<8 | me;
sample=smp;
//val=val<<8 | lo;
//val=val<<8;
//verdi=val;
v2=sample;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
smp=hi;
smp=smp<<8 | me;
//val=val<<8 | lo;
//val=val<<8;
sample=smp;
//verdi=val;
v3=sample;
//buf_ptr++;
//buf_ptr++;
switch(ptr->nchan)
{
    case 1:
    ptr->ibuf[ptr->index][indx+HEADER]=v1;
//printf("%3d %5d\n",indx,v1);
    break;

    case 2:
    ptr->ibuf[ptr->index][indx+HEADER]=v1;
    ptr->ibuf[ptr->index][indx+1+HEADER]=v2;
    break;

    case 3:
    ptr->ibuf[ptr->index][indx+HEADER]=v1;
    ptr->ibuf[ptr->index][indx+1+HEADER]=v2;
    ptr->ibuf[ptr->index][indx+2+HEADER]=v3;
    break;
}
indx=indx+ptr->nchan;
}
}

void    Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
//printf("%2x\n",sync_value);
        } while(sync_value != 0x4742);
//        } while(sync_value != 0x0a0d);
//putchar(0x07);
}
