/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:14/02/95                       */
/* Program:DIG_IDS                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from Geosys. Determine start of buffers */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
#define MAXTMO 5

int           kval;
unsigned char byt[3000];               /* Input buffer                      */
unsigned char *buf_ptr;
int           n;
int           lst_val1;
int           lst_val2;
int           lst_val3;
int           indx;                    /* index current channel             */
int           only_one;
FILE          *in;                     /* input file pointer                */
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           err_cnt;                 /* error counter                     */
int           fd;                      /* return value open share           */
int           nbyt;                    /* number of bytes to read           */
int           fh;                      /* input handler                     */
int           sts_snc;                 /* status after reading sync         */
int           sts_buf;                 /* status after reading data buffer  */
int           sec_ptr;
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
char          msg[10];
int           speed;
int           sr;
int           rdbyt;
char          stty_str[100];
char          port_str[12];
int           TIMEOUTS;
main()
{
pid_t         proxy;
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
int           clk_pps_id;
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
unsigned int  cop;
setprio(getpid(),12);                  /* set the priority of this process  */
err_cnt=0;                             /* set error counter to zero         */
only_one=0;
dig_eda_id=qnx_name_attach(0,"dig_ge1"); /* attach this process as "dig_eda"*/
if(dig_eda_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_GE1 attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_GE1_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_GE1_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
TIMEOUTS=0;
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
clk_pps_id=qnx_name_locate(0,"clk_pps",0,&cop);
if(clk_pps_id == -1)
{
    printf("DIG_GE1:cant locate CLK_PPS\n");
    exit(0);
}
proxy=qnx_proxy_attach(0,0,0,-1);
ptr->min_proxy=proxy;
nbyt=ptr->rate*11-2;

ptr -> index=0;                        /* Initial buffer pointer            */
/*--------------------------------------------------------------------------*/
/* Open device for reading data from GeoSys digitizer                       */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);

/*--------------------------------------------------------------------------*/
/* open serial line where digitizer is connected                            */
/*--------------------------------------------------------------------------*/
if((fh=open(port_str,O_RDONLY | O_CACHE))==ERROR)
{
    printf("DIG_GE1: Can't open serial line %s\n",port_str);
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Read data from IDS until sync character is detected.                     */
/*--------------------------------------------------------------------------*/
do
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    sts_snc=Sync1();
    if(sts_snc <= 0)
        TIMEOUTS++;
    if(TIMEOUTS >= MAXTMO)
    {
        TIMEOUTS=0;
//        close(fh);
        sprintf(ptr->err_msg,"DIG_GE1: Not receiving data from digitizer");
        errlog(ptr);
//        if(ptr->err_mail == 1)
//            system("/home/seislog/message 8");
//        goto reopen;
    }
}while(sts_snc <= 0);
TIMEOUTS=0;
printf("-------------------------------------------\n");
printf("DIG_GE1   : Receiving data from digitizer !\n");
printf("-------------------------------------------\n");
sprintf(ptr->err_msg,"DIG_GE1: Receiving data from digitizer");
errlog(ptr);
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
    ptr->alive[1]=1;                   /* tel I am alive                    */
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
    do
    {
        sts_snc=Sync1();                   /* find sync char. in received data  */
        if(sts_snc <= 0)                    /* test if return value is ok        */
            TIMEOUTS++;
        if(TIMEOUTS > MAXTMO)
        {
            TIMEOUTS=0;
//            close(fh);
            sprintf(ptr->err_msg,"DIG_GE1: Not receiving data from digitizer");
            errlog(ptr);
//            if(ptr->err_mail == 1)
//                system("/home/seislog/message 8");
//            goto reopen;
        }
    }while(sts_snc <= 0);
    TIMEOUTS=0;
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
                                       /* signal.                           */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
    do
    {
        sts_buf=ED_3();
        if(sts_buf <= 0)                    /* test if return value is ok        */
            TIMEOUTS++;
        if(TIMEOUTS == 10)
        {
            close(fh);
            if(ptr->err_mail == 1)
                system("/home/seislog/message 8");
            exit(0);
        }
    }while(sts_buf <= 0);
    TIMEOUTS=0;
/*--------------------------------------------------------------------------*/
/* convert sample values into right format and store in correct buffer      */
/*--------------------------------------------------------------------------*/
    sec_ptr=HEADER;
    Convert();
/*--------------------------------------------------------------------------*/
/* send time info to BUF_PIP                                                */
/*--------------------------------------------------------------------------*/
//printf("DIG_GE1 finished buffer: %d\n",ptr->index);
    Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
}
}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3()
{
buf_ptr=byt;
n=dev_read(fh,buf_ptr,nbyt,nbyt,0,11,0,0); /* read 'nbyt' bytes from serial port*/
switch(n)
{
    case -1:                           /* ERROR reading data                */
    return(-1);
    break;

    case 0:                            /* TIMEOUT reading data              */
    return(0);
    break;

    case 1:                            /* Reading data ok                   */
    break;
}
return(1);
}

/****************************************************************************/
/* Function to convert data buffers from IDS-24                              /
/****************************************************************************/
void Convert()
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register int verdi;
register int v1,v2,v3;


unsigned short smp;
short sample;
buf_ptr= &byt[0];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v1=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v2=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v3=verdi/256;
buf_ptr++;
buf_ptr++;

    switch(ptr->nchan)
    {
        case 1:
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=v3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
lst_val1=v1;
lst_val2=v2;
lst_val3=v3;
}
/****************************************************************************/
/* Function to make zero buffer when error                                  */
/****************************************************************************/
void ErrBuf()
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register int verdi;
register int v1,v2,v3;


unsigned short smp;
short sample;
buf_ptr= &byt[0];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v1=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v2=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v3=verdi/256;
buf_ptr++;
buf_ptr++;

    switch(ptr->nchan)
    {
        case 1:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val2;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
}


Sync1()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0x0a0d);
//printf("snc\n");
return(1);
}

