/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date: 16/03/99                      */
/* Program: DIG_GE2                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from IDS-24 1-second buffers.           */
/* Determine start of buffers and send request to clock.                    */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
#define HDR 24
#define BPS 4
unsigned char byt[3000];               /* Input buffer                      */
unsigned char *buf_ptr;
int           n;
int           lst_val1;
int           lst_val2;
int           lst_val3;
int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
int           lp;                      /* index                             */
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           fd;                      /* return value open share           */
int           nbyt;                    /* number of bytes to read           */
int           fh;                      /* input handler                     */
int           sts_snc;                 /* status after reading sync         */
int           sts_buf;                 /* status after reading data buffer  */
int           sec_ptr;
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
char          msg[10];
char          stty_str[100];           /* port setup                        */
char          port_str[12];            /* port setup                        */
main()
{
int           ok;                      /* number of ok channels             */
int           err_ind;                 /* error indicator                   */
int           next_channel;            /* next channel to read              */
int           SEQ;                     /* sequence number                   */
int           KAN;                     /* channel current packet            */
int           CHK;                     /* checksum from unit                */
int           BPP;                     /* bytes per packet                  */
int           SPP;                     /* samples per packet                */
int           FMT;                     /* data format 16 or 32              */
unsigned char CHK_SUM;                 /* checksum computed                 */
pid_t         proxy;
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
int           clk_pps_id;
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
unsigned int  cop;
setprio(getpid(),12);                  /* set the priority of this process  */
dig_eda_id=qnx_name_attach(0,"dig_ge2");
if(dig_eda_id == 1)
{
    printf("DIG_GE2 attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_GE2_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_GE2_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
clk_pps_id=qnx_name_locate(0,"clk_pps",0,&cop);
if(clk_pps_id == -1)
{
    printf("DIG_GE2 cant locate CLK_PPS\n");
    exit(0);
}
proxy=qnx_proxy_attach(0,0,0,-1);
ptr->min_proxy=proxy;
nbyt=HDR-2+(ptr->rate * BPS);          /* number of bytes to read           */
printf("NBYT=%d\n",nbyt);
ptr -> index=0;                        /* Initial buffer pointer            */

/*--------------------------------------------------------------------------*/
/* Open device for reading data from IDS digitizer                          */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);
/*--------------------------------------------------------------------------*/
/* open serial line where digitizer is connected                            */
/*--------------------------------------------------------------------------*/
if((fh=open(port_str,O_RDONLY | O_CACHE))==ERROR)
{
    printf("DIG_GE2: Can't open serial line %s\n",port_str);
    exit(0);
}
err_ind=0;                             /* initially no errors               */
ok=0;
/*--------------------------------------------------------------------------*/
/* Read data from IDS until one 3 sync character is detected.               */
/*--------------------------------------------------------------------------*/
for(i=0;i<3;i++)
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    sts_snc=Sync1();
    switch(sts_snc)                    /* test if return value is ok        */
    {
        case 1:                        /* reading byte ok                   */
        break;
        case 0:                        /* timeout looking for sync          */
        printf("DIG_GE2: TIMEOUT reading data from line %s\n",port_str);
        break;
        case -1:                       /* error reading data                */
        printf("DIG_GE2: ERROR reading data from line %s\n",port_str);
        break;
    }
}
printf("DIG_GE2   : Receiving data from digitizer !\n");
/*--------------------------------------------------------------------------*/
/* search in data until buffer last channel has been read. Next channel to  */
/* read will then be channel 1. This is to get synchronized at startup.     */
/*--------------------------------------------------------------------------*/
next_channel=ptr->nchan;               /* set next_channel to last channel  */
next:
sts_snc=Sync1();                       /* look for sync characters          */
switch(sts_snc)                        /* test if return value is ok        */
{
    case 1:                            /* reading byte ok                   */
    break;
    case 0:                            /* timeout looking for sync          */
    printf("DIG_GE2: TIMEOUT (Sync1) reading data from line %s\n",port_str);
    goto next;
    break;
    case -1:                           /* error reading data                */
    printf("DIG_GE2: ERROR (Sync1) reading data from line %s\n",port_str);
    goto next;
    break;
}
sts_buf=ED_3();                        /* read 1 second of data, one channel*/
switch(sts_buf)                        /* test if return value is ok        */
{
    case 0:                            /* timeout reading data buffer       */
    printf("DIG_GE2: TIMEOUT (ED_3) reading data from line %s\n",port_str);
    goto next;
    break;
    case -1:                           /* error reading data                */
    printf("DIG_GE2: ERROR (ED_3) reading data from line %s\n",port_str);
    goto next;
    break;
    default:
    if(sts_buf != nbyt)
    {
        printf("DIG_GE2: Less than 1 second of data has been read on %s\n",port_str);
        goto next;
    }
    break;
}
CHKSUM(nbyt,&CHK_SUM);                 /* compute checksum of received data */
HDR_INFO(&SEQ,&KAN,&CHK,&BPP,&SPP,&FMT); /* extract header information      */
if(KAN != next_channel)                  /* if not next channel             */
    goto next;
if(CHK != CHK_SUM)                     /* test check sum                    */
    goto next;
/*--------------------------------------------------------------------------*/
/* we are now receiving correct data, and we found last channel Tell BUF_PIP*/
/*--------------------------------------------------------------------------*/
Send(buf_pip_id,&snd_msg,&rec_msg,1,1);

for(;;)                                /* indefinite loop collecting data   */
{
top:
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Receive(buf_pip_id,&rec_msg,1);
    Reply(buf_pip_id,&rep_msg,1);
/*--------------------------------------------------------------------------*/
/* find error-free sequence of 1->nchan buffers                             */
/* if not error-free, insert zero-buffer                                    */
/*--------------------------------------------------------------------------*/
    for(next_channel=1;next_channel<ptr->nchan+1;next_channel++)
    {
        sts_snc=Sync1();               /* find sync char. in received data  */
        switch(sts_snc)                /* test if return value is ok        */
        {
            case 1:                    /* reading byte ok                   */
            break;
   
            case 0:                    /* timeout looking for sync          */
            printf("DIG_GE2: TIMEOUT (Sync1) reading data from line %s\n",port_str);
            err_ind++;                 /* set error                         */
            break;

            case -1:                   /* error reading data                */
            printf("DIG_GE2: ERROR (Sync1) reading data from line %s\n",port_str);
            err_ind++;                 /* set error                         */
            break;
        }
        if(next_channel==1)            /* sync found, signal BUF_PIP to tag */
            Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* signal BUF_PIP       */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
        sts_buf=ED_3();
        switch(sts_buf)                /* test if return value is ok        */
        {
            case 0:                    /* timeout reading data buffer       */
            printf("DIG_GE2: TIMEOUT (ED_3) reading data from line %s\n",port_str);
            err_ind++;                 /* set error                         */
            break;
            case -1:                   /* error reading data                */
            printf("DIG_GE2: ERROR (ED_3) reading data from line %s\n",port_str);
            err_ind++;                 /* set error                         */
            break;
            default:
            if(sts_buf != nbyt)
            {
                printf("DIG_GE2: Less than 1 second of data has been read on %s\n",port_str);
                err_ind++;             /* set error                         */
            }
            break;
        }
        CHKSUM(nbyt,&CHK_SUM);         /* compute checksum received data    */
/************************************************************/
/*    NEXT STATEMENT MUST BE DELETED ON FINAL VERSION       */
/************************************************************/
//        byt[22]=byt[23]=byt[24]=byt[25]=0;
        HDR_INFO(&SEQ,&KAN,&CHK,&BPP,&SPP,&FMT); /* extract header information  */
        if(next_channel != KAN)        /* do we have the right channel ?    */
        {
            printf("Wrong channel sequence\n");
            err_ind++;                 /* set error                         */
        }
        if(CHK_SUM != CHK)
        {
            printf("Checksum error\n");
            err_ind++;                 /* set error                         */
        }
//    printf("SEQ=%d KANAL=%d BPP=%d SPP=%d FMT=%d CHK=%8d %8d\n",SEQ,KAN,BPP,SPP,FMT,CHK,CHK_SUM);
        sec_ptr=HEADER;                /* pointer output buffer             */
        Convert(next_channel);         /* convert and store data current ch */
    }   /* nchan channels read, but do we have errors ? */

/*--------------------------------------------------------------------------*/
/* nchan channels are now stored in outputbuffer and the buffer is tagged   */
/* Now check data integrity of the buffer. If errors, insert zero buffer    */
/*--------------------------------------------------------------------------*/
    if(err_ind != 0)                   /* any errors ?                      */
    {
        printf("ErrorBuffer\n");
        ErrorBuffer();                 /* yes, insert zero buffer           */
        err_ind=0;                     /* reset error flag                  */
        Send(buf_pip_id,&snd_msg,&rec_msg,30,1); /* signal buffer ready     */
/*--------------------------------------------------------------------------*/
/* now re-synchronize input to channel 1->nchan by looking for last channel */
/*--------------------------------------------------------------------------*/
        next_channel=ptr->nchan;       /* set next channel to last channel  */
resync:
        sts_snc=Sync1();
        switch(sts_snc)                /* test if return value is ok        */
        {
            case 1:                    /* reading byte ok                   */
            break;
            case 0:                    /* timeout looking for sync          */
            printf("DIG_GE2: TIMEOUT (Sync1 resync) reading data from line %s\n",port_str);
            goto resync;
            break;
            case -1:                   /* error reading data                */
            printf("DIG_GE2: ERROR (Sync1 resync) reading data from line %s\n",port_str);
            goto resync;
            break;
        }

        sts_buf=ED_3();                /* read 1 second of data             */
        switch(sts_buf)                /* test if return value is ok        */
        {
            case 0:                    /* timeout reading data buffer       */
            break;
            case -1:                   /* error reading data                */
            break;
            default:
            if(sts_buf != nbyt)
            {
                printf("DIG_GE2: Less than 1 second of data has been read on %s\n",port_str);
            }
            break;
        }







        CHKSUM(nbyt,&CHK_SUM);         /* compute checksum of received data */
        HDR_INFO(&SEQ,&KAN,&CHK,&BPP,&SPP,&FMT); /* extract header information      */
        if(KAN != next_channel)        /* if not next channel               */
        {
            goto resync;
        }else{
Receive(buf_pip_id,&rec_msg,1);
Reply(buf_pip_id,&rep_msg,1);
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* signal BUF_PIP       */
ErrorBuffer();                 /* yes, insert zero buffer           */
err_ind=0;                     /* reset error flag                  */
Send(buf_pip_id,&snd_msg,&rec_msg,30,1); /* signal buffer ready     */
        }
        if(CHK != CHK_SUM)             /* test check sum                    */
            goto resync;
        goto top;                      /* we are in business again          */
    }else{
/*--------------------------------------------------------------------------*/
/* end of error check, no errors, signal BUF_PIP that we have error free buf*/
/*--------------------------------------------------------------------------*/
            Send(buf_pip_id,&snd_msg,&rec_msg,30,1); /* signal buffer ready     */
    }
}
}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3()
{
buf_ptr=byt;
n=dev_read(fh,buf_ptr,nbyt,nbyt,0,11,0,0); /* read 'nbyt' bytes from serial port*/
/*
for(i=0;i<39;i++)
    printf("%2d",i);
printf("\n");
for(i=0;i<39;i++)
    printf("%2x",byt[i]);
printf("\n");
printf(" %2x%2x%2x%2x %2x%2x%2x%2x %2x%2x%2x%2x",
byt[22],byt[23],byt[24],byt[25],
byt[26],byt[27],byt[28],byt[29],
byt[30],byt[31],byt[32],byt[33]);
printf("\n");
*/
//printf("N %d\n",n);
//exit(0);

switch(n)
{
    case -1:                           /* ERROR reading data                */
    return(-1);
    break;

    case 0:                            /* TIMEOUT reading data              */
    return(0);
    break;

    default:                           /* Reading data ok, return no. read  */
    return(n);
    break;
}
}

/****************************************************************************/
/* Function to convert data buffers from IDS-24                              /
/****************************************************************************/
void Convert(cur_kan)
int cur_kan;
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned char b1,b2,b3,b4;
register unsigned int val;
register int verdi;
register int v1,v2,v3;
int v[10];

unsigned short smp;
short sample;
buf_ptr= &byt[22];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
    val=0;
    b1=(*buf_ptr++);
    b2=(*buf_ptr++);
    b3=(*buf_ptr++);
    b4=(*buf_ptr++);
    val=b1;
    val=val<<8 | b2;
    val=val<<8 | b3;
    val=val<<8 | b4;
    verdi=val;
    v1=verdi;

    ptr->ibuf[ptr->index][sec_ptr+cur_kan-1]=v1;
    sec_ptr=sec_ptr+ptr->nchan;
}
}
/****************************************************************************/
/* Function to make zero buffer when error                                  */
/****************************************************************************/
void ErrBuf()
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register int verdi;
register int v1,v2,v3;


unsigned short smp;
short sample;
buf_ptr= &byt[22];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v1=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v2=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v3=verdi/256;
buf_ptr++;
buf_ptr++;

    switch(ptr->nchan)
    {
        case 1:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val2;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
}


Sync1()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);                /* ERROR reading data                */
            break;
            case 0:
            return(0);                 /* TIMEOUT reading data              */
            break;
            case 1:                    /* Data ok, 1 byte received          */
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0x4454);
//printf("snc\n");
return(1);
}

CHKSUM(no_byt,chksum)
int  no_byt;
unsigned char* chksum;
{
int t;
unsigned char sum;
sum=68+84;
for(t=0;t<no_byt;t++)
{
    if(t!=20)
        sum=sum+byt[t];
}

*chksum=sum;
}

HDR_INFO(seq,ch,chk,bpp,spp,fmt)
int* seq;
int* ch;
int* chk;
int* bpp;
int* spp;
int* fmt;
{
int add;
unsigned short ss;
unsigned int ss1;
unsigned short save;
int sequence;
int k;
int tusen;
int hundre;
int ti;
int en;
/*--------------------------------------------------------------------------*/
/*                            SEQUENCE NUMBER                               */
/*--------------------------------------------------------------------------*/
    save=byt[12];                      /* extract thousand in sequence      */
    ss=save;
    ss=ss & 0x00f0;
    ss=ss >> 4;
    tusen=ss*1000;

    ss=save;                           /* extract hundreds in sequence      */
    ss=ss & 0x000f;
    hundre=ss*100;

    save=byt[13];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in sequence          */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in sequence          */

    *seq=tusen+hundre+ti+en;
/*--------------------------------------------------------------------------*/
/*                            CHANNEL NUMBER                                */
/*--------------------------------------------------------------------------*/
    save=byt[17];
    *ch=save;                          /* extract channel number            */

/*--------------------------------------------------------------------------*/
/*                            CHECKSUM                                      */
/*--------------------------------------------------------------------------*/
    *chk=byt[20];                      /* extract checksum                  */

/*--------------------------------------------------------------------------*/
/*                            BYTES PER PACKET                              */
/*--------------------------------------------------------------------------*/
    save=byt[10];                      /* extract thousand in bpp           */

    ss=save;                           /* extract bytes per packet          */
    ss=ss & 0x00f0;
    ss=ss >> 4;
    tusen=ss*1000;

    ss=save;                           /* extract hundreds in bpp           */
    ss=ss & 0x000f;
    hundre=ss*100;

    save=byt[11];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in bpp               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in bpp               */

    *bpp=tusen+hundre+ti+en;

/*--------------------------------------------------------------------------*/
/*                            SAMPLES PER PACKET                            */
/*--------------------------------------------------------------------------*/
    save=byt[18];                      /* extract thousand in spp           */

    ss=save;                           /* extract samples per packet        */
    ss=ss & 0x00f0;
    ss=ss >> 4;
    tusen=ss*1000;

    ss=save;                           /* extract hundreds in spp           */
    ss=ss & 0x000f;
    hundre=ss*100;

    save=byt[19];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in spp               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in spp               */

    *spp=tusen+hundre+ti+en;


/*--------------------------------------------------------------------------*/
/*                            DATA FORMAT                                   */
/*--------------------------------------------------------------------------*/
    save=byt[21];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in fmt               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in fmt               */

    *fmt=ti+en;
printf("%2x %2x %2x %2x %2x %2x %2x\n",byt[1],byt[4],byt[5],byt[6],byt[7],byt[8],byt[9]);
}
/****************************************************************************/
/* generate error buffer                                                    */
/****************************************************************************/
void ErrorBuffer()
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned char b1,b2,b3,b4;
register unsigned int val;
register int verdi;
register int v1,v2,v3;
int v[10];
int ch;
int sp;
unsigned short smp;
short sample;
v1=0;
buf_ptr= &byt[22];                     /* point to first data byte          */
for(ch=0;ch<ptr->nchan;ch++)
{
    sec_ptr=HEADER;
    for(sp=0;sp<ptr->rate;sp++)
    {
        ptr->ibuf[ptr->index][sec_ptr+ch]=v1;
        sec_ptr=sec_ptr+ptr->nchan;
    }
}
}
