/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:12/07/99                       */
/* Program:DIG_GUR                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)


double              MSECS;             /* variables for time routines       */
double              MSEC70;
double              MSEC89;
float               SEC;
float               MILSEC;
int                 YR,DOY,MTH,DAY,HR,MIN;
int                 SECL;
float               sec;               /* seconds x.xxx                     */

FILE                *gpslog;
FILE                *adrlog;
FILE                *sizes;
int           kval;
unsigned char byt[1500];               /* Input buffer                      */
unsigned char chk[10];
char          status_block[1000];
int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
int           i;                       /* counter                           */
unsigned int  chk_in;                  /* checksum input                    */
int           numread;                 /* read definition                   */
int           fd;                      /* return value open share           */
smod          *ptr;                    /* pointer to shared region          */
main()
{
int lp;
int k;
int j;
int           seqold;
int           uncomp[2000];            /* uncompressed values               */
int           SEQNO;                   /* sequence number 0-255             */
int           SIZE;                    /* block size                        */
char          SYS_ID[10];              /* system id                         */
char          STR_ID[10];              /* stream id                         */
int           DAG;                     /* day from 17 Nov. 1989             */
int           SECND;                   /* seconds after midnight            */
int           SR;                      /* samplerate                        */
int           CMP;                     /* compression code                  */
int           XFIRST;                  /* first value in block              */
int           IX;                      /* start index data buffer           */
int           ADR;                     /* second in minute, start of block  */
char          COMPONENT;               /* component identifier              */
int           NE;                      /* samples per block                 */
int           SB;                      /* seconds per block                 */
int           PHADR;                   /* physical address first sample     */
int           MAXADR;                  /* max address in pipe               */
int           CH;                      /* component identifier 0,1,2        */
int           CHECK_READ;              /* checksum read from data           */
int           ticks;                   /* copy of SECND                     */
int           adr_ticks;
int           ret;
int           old_seq;                 /* old sequence number               */
char          stty_str[100];
char          port_str[12];
setprio(getpid(),12);                  /* set the priority of this process  */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_GUR_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_GUR_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* Open device for reading data from ED_24                                  */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);

in=fopen(port_str,"r");
system("rm -f /home/work/gps");
/*--------------------------------------------------------------------------*/
/* compute number of seconds since this date. Start of QNX internal clock   */
/*--------------------------------------------------------------------------*/
YR=1970;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSEC70);
/*--------------------------------------------------------------------------*/
/* compute number of seconds since this date. Start of GURALP internal clock*/
/*--------------------------------------------------------------------------*/
YR=1989;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSEC89);

/*--------------------------------------------------------------------------*/
/* Read data from ED_24 until one 5 sync character is detected.             */
/*--------------------------------------------------------------------------*/
for(i=0;i<3;i++)
{
    for(j=0;j<90000;j++)
        ptr->snc_pip[i][j]=0;
}
for(i=0;i<(60*15);i++)
{
    for(k=0;k<3;k++)
        ptr->rdy[i][k]=0;
}
for(i=0;i<(60*15);i++)
{
    for(k=0;k<6;k++)
        ptr->clr[i][k]=0;
}

seqold=0;
for(i=0;i<5;i++)
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Sync();
    SEQNO=find_seqno();                /* find sequence number 0-255        */
    SIZE=find_size();                  /* find block size                   */
    if(SEQNO >= 0 && SEQNO <= 255)
    {
        if(SIZE > 0 && SIZE <= 1024)
        {
            read_block(SIZE);          /* read data block current component */
//            printf("%5d %5d\n",SEQNO,SIZE);
            seqold=SEQNO;
        }
    }
}
printf("DIG_GUR   : Receiving data from digitizer !\n");
search:
for(;;)                                /* indefinite loop collecting data   */
{
    chk_in=0x47;                       /* zero checksum akumulator          */
    ptr->alive[1]=1;                   /* tel I am alive                    */
    Sync();                            /* find sync char. in received data  */
    SEQNO=find_seqno();                /* find sequence number 0-255        */
    SIZE=find_size();                  /* find block size                   */
    if(SEQNO >= 0 && SEQNO <= 255)
    {
        if(SIZE > 0 && SIZE <= 1024)
        {
//            sizes=fopen("/home/work/sis","a");
//            if(sizes==0)
//                printf("ERROR opening SIS\n");
//            fprintf(sizes," %5d %5d\n",SEQNO,SIZE);       
//            fclose(sizes);
            read_block(SIZE);          /* read data block current component */
//            printf("%5d %5d\n",SEQNO,SIZE);
            goto block;
        }
    }
    goto search;

block:
/*--------------------------------------------------------------------------*/
/* find out if we have a valid buffersize                                   */
/*--------------------------------------------------------------------------*/
    SR=100;
    for(lp=SR;lp<1001; lp=lp+SR)
    {
//printf("lp+SR:%5d SIZE:%5d\n",lp+24,SIZE);
//getch();
        if(SIZE==(lp+24))              /* data block ?                      */
        {
            if(SIZE != 424)
                goto found;
        }
    }
/*--------------------------------------------------------------------------*/
/* its a status buffer                                                      */
/*--------------------------------------------------------------------------*/

    if(SEQNO >= 0  && SIZE <= 1000)
    {
//        printf("STATUS BLOCK: %4d %5d\n",SEQNO,SIZE);       
        gpslog=fopen("/home/work/gps","a");
        if(gpslog==0)
            printf("ERROR opening GPSLOG\n");
        fprintf(gpslog,"STATUS BLOCK: %4d %5d\n",SEQNO,SIZE);       
        fclose(gpslog);
        byt[SIZE-16]='\0';
        for(i=0;i<SIZE;i++)
            status_block[i]=byt[i+16];
//        printf("%s\n",status_block);
        gpslog=fopen("/home/work/gps","a");
        if(gpslog==0)
            printf("ERROR opening GPSLOG\n");
        fprintf(gpslog,"%s\n",status_block);
        fclose(gpslog);
        goto search;
    }
printf("Its not a data buffer and not a status buffer\n");
    goto search;
    
found:
        CHECK_READ=read_checksum();    /* read and interpret checksum       */
        if((chk_in % 65536)!= CHECK_READ)
        {
            gpslog=fopen("/home/work/gps","a");
            if(gpslog==0)
                printf("ERROR opening GPSLOG\n");
            fprintf(gpslog,"Checksum error\n");
            fclose(gpslog);
            goto search;
        }
        find_system_id(SYS_ID);        /* find system id                    */
        find_stream_id(STR_ID);        /* find stream id                    */
        find_day_sec(&DAG,&SECND);     /* find day & seconds                */
        DAG=DAG-44;                    /* 45 days left to new year          */
        ret=GUR_DAG (DAG,&YR,&DOY);
        DATO (DOY,&DAY,&MTH,YR);
        SEC_SEC (YR,MTH,DAY,SECND,&MSECS);
        SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);

        find_sr_cmp(&SR,&CMP);         /* sample rate & compression code    */
        find_first_val(&XFIRST);       /* first absolute value in block     */
        IX=20;                         /* first byte in compressed data     */
        ADR=(MIN % 15)*60+(int)SEC;       /* second in 15 minute data buffer   */
                                       /* start data block                  */
        NE=(SIZE-24)/CMP;              /* samples per block                 */
        SB=NE/SR;                      /* seconds per block                 */
        PHADR= ADR * SR;               /* physical address first sample     */
        MAXADR=SR * 60 * 15;           /* max address in pipe               */
        COMPONENT=STR_ID[4];           /* component identifier              */
        switch(COMPONENT)              /* test for component to convert     */
        {
            case 'Z':
            CH=0;
//printf("MIN:%2d SEC:%2d ADR:%6d PHADR:%6d\n",MIN,(int)SEC,ADR,PHADR);
            ticks=SECND;
            adr_ticks=ADR;
            for(i=0;i<SB;i++)
            {
                ptr->clr[adr_ticks][CH*2]=DAG;          /* store day number in correct second*/
                ptr->clr[adr_ticks][CH*2+1]=ticks;        /* store second                      */
                ticks++;
                adr_ticks++;
                if(adr_ticks > (60*15)-1)
                    adr_ticks=0;
            }
            decomp(&uncomp,IX,NE,CMP,XFIRST);
            to_pipe(uncomp,NE,CH,PHADR,MAXADR,SR,ADR);
            break;
            case 'N':
            CH=1;
            ticks=SECND;
            adr_ticks=ADR;
            for(i=0;i<SB;i++)
            {
                ptr->clr[adr_ticks][CH*2]=DAG;          /* store day number in correct second*/
                ptr->clr[adr_ticks][CH*2+1]=ticks;        /* store second                      */
                ticks++;
                adr_ticks++;
                if(adr_ticks > (60*15)-1)
                    adr_ticks=0;
            }
            decomp(&uncomp,IX,NE,CMP,XFIRST);
            to_pipe(uncomp,NE,CH,PHADR,MAXADR,SR,ADR);
            break;
            case 'E':
            CH=2;
            ticks=SECND;
            adr_ticks=ADR;
            for(i=0;i<SB;i++)
            {
                ptr->clr[adr_ticks][CH*2]=DAG;          /* store day number in correct second*/
                ptr->clr[adr_ticks][CH*2+1]=ticks;        /* store second                      */
                ticks++;
                adr_ticks++;
                if(adr_ticks > (60*15)-1)
                    adr_ticks=0;
            }
            decomp(&uncomp,IX,NE,CMP,XFIRST);
            to_pipe(uncomp,NE,CH,PHADR,MAXADR,SR,ADR);
            break;
        }
if(COMPONENT=='k')
        printf("%4d %5d %s %s %5d %5d %2d %3d %1d %8d %c %3d %1d %2d:%2d:%6.3f\n",
SEQNO,SIZE,SYS_ID,STR_ID,DAG,SECND,ADR,SR,CMP,XFIRST,COMPONENT,NE,SB,HR,MIN,SEC);
//    }
}
}

void    Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
        } while(byt[0] != 0x47);
//printf("sync\n");
/*putchar(0x07);*/
}
find_seqno()
{
unsigned char b1;
int seqno;
    numfread(byt,1);                   /* read in the header and data block */
    if(numread<1)                      /* read OK ?                         */
    clearerr(in);                      /* clear error                       */
    b1=byt[0];
    chk_in=chk_in+b1;                  /* akumulate checksum                */
    seqno=b1;
    return(seqno);
}
find_size()
{
unsigned char b1,b2;
unsigned short noofb;
int size;
    numfread(byt,2);                   /* read in the header and data block */
    if(numread<2)                      /* read OK ?                         */
        clearerr(in);                  /* clear error                       */
    b1=byt[0];
    b2=byt[1];
    chk_in=chk_in+b1;                  /* akumulate checksum                */
    chk_in=chk_in+b2;                  /* akumulate checksum                */
    noofb=b1;
    noofb=noofb << 8 | b2;
    size=noofb;
    return(size);
}
read_block(size)
int size;
{
int k;
    numfread(byt,size);                /* read in the header and data block */
    if(numread<size)                   /* read OK ?                         */
        clearerr(in);                      /* clear error                       */
    for(k=0;k<size;k++)
        chk_in=chk_in+byt[k];          /* akumulate checksum                */
}
find_system_id(sys_id)
char sys_id[];
{
unsigned char b1,b2,b3,b4;
unsigned int sysid;
int system_id;
int imed;
int space;
int k;
    b1=byt[0];
    b2=byt[1];
    b3=byt[2];
    b4=byt[3];
    sysid=b1;
    sysid=sysid << 8 | b2;
    sysid=sysid << 8 | b3;
    sysid=sysid << 8 | b4;
    system_id=sysid;
    space=6;
    for(k=space-1; k >=0; k--)
    {
        imed=system_id % 36;
        if(imed > 9)
            imed +=7;
        sys_id[k]=imed + '0';
        system_id /=36;
    }
    sys_id[6]='\0';
}
find_stream_id(str_id)
char str_id[];
{
unsigned char b1,b2,b3,b4;
unsigned int streamid;
int stream;
int space;
int imed;
int k;
    b1=byt[4];
    b2=byt[5];
    b3=byt[6];
    b4=byt[7];
    streamid=b1;
    streamid=streamid << 8 | b2;
    streamid=streamid << 8 | b3;
    streamid=streamid << 8 | b4;
    stream=streamid;
    space=6;
    for(k=space-1; k >=0; k--)
    {
        imed=stream % 36;
        if(imed > 9)
            imed +=7;
        str_id[k]=imed + '0';
        stream /=36;
    }
    str_id[6]='\0';
}
find_day_sec(dag,secnd)
int *dag;
int *secnd;
{
unsigned char b1,b2,b3;
unsigned short dagg;
unsigned int sec;

    b1=byt[8];
    b2=byt[9];
    dagg=b1;
    dagg=dagg<<8 | b2;
    *dag=dagg;
    *dag=*dag/2;
    b1=byt[9];
    b2=byt[10];
    b3=byt[11];
    b1=b1 & 0x01;
    sec=b1;
    sec=sec<<8 | b2;
    sec=sec<<8 | b3;
    *secnd=sec;
}
find_sr_cmp(sr,cmp)
int *sr;
int *cmp;
{
unsigned char b1,b2,b3,b4;
unsigned int srat;
unsigned int cmpr;
int ccode;
    b1=byt[12];
    b2=byt[13];
    b3=byt[14];
    b4=byt[15];
    srat=b2;
    *sr=srat;
    cmpr=b3;
    cmpr=cmpr & 0x07;
    ccode=cmpr;
    switch(ccode)
    {
        case 1:
        *cmp=3;
        break;
        case 2:
        *cmp=2;
        break;
        case 4:
        *cmp=1;
    }
}
find_first_val(xfirst)                 /* first absolute value in block     */
int *xfirst;
{
unsigned char b1,b2,b3,b4;
unsigned int val;
int val2;
    b1=byt[16];
    b2=byt[17];
    b3=byt[18];
    b4=byt[19];
    val=b1;
    val=val << 8 | b2;
    val=val << 8 | b3;
    val=val << 8 | b4;
    val2=val;
    *xfirst=val2;
}
decomp(uk,ix,ne,code,FIC)
int *uk;
int ix;
int ne;
int code;
int FIC;
{
int k;
int y;
int idx;
unsigned char b1,b2,b3;
unsigned int v1;
int vv1;
unsigned short v2;
short vv2;
int x1;
    idx=ix;
    switch(code)
    {
        case 3:
        for(k=0;k<ne;k++)
        {
            b1=byt[idx];
            idx++;
            b2=byt[idx];
            idx++;
            b3=byt[idx];
            idx++;
            v1=b1;
            v1=v1<<8 | b2;
            v1=v1<<8 | b3;
            v1=v1<<8;
            vv1=v1;
            vv1=vv1/256;
//printf("%4d %6d\n",k,vv1);
            uk[k]=vv1;
        }
        break;
        case 2:
        for(k=0;k<ne;k++)
        {
            b1=byt[idx];
            idx++;
            b2=byt[idx];
            idx++;
            v2=b1;
            v2=v2<<8 | b2;
            vv2=v2;
//printf("%4d %6d\n",k,vv1);
            uk[k]=vv2;
        }
        break;
        case 1:
        break;
    }
    y=FIC;
    for(k=0;k<ne;k++)
    {
        y+=uk[k];
        uk[k]=y;
//        uk[k]=x1-uk[k];
//        x1=uk[k];
    }
}
to_pipe(uk,ne,ch,phadr,maxadr,sr,adr)
int *uk;
int ne;
int ch;
int phadr;
int maxadr;
int sr;
int adr;
{
int k;
int adress;
int sec_lim;
int start_second;
start_second=adr;
sec_lim=0;
adress=phadr;
//("ne:%4d ch:%3d phadr:%6d maxadr:%6d sr:%4d adr:%5d\n",ne,ch,phadr,maxadr,sr,adr);
    for(k=0;k<ne;k++)
    {
        ptr->snc_pip[ch][adress]=uk[k];
        adress++;
        if(adress == maxadr)
            adress=0;
        sec_lim++;
//printf("adress %6d sec_lim %6d\n",adress,sec_lim);
        if(sec_lim >= sr)
        {
            sec_lim=0;
//printf("whole second\n");
            ptr->rdy[start_second][ch]=1;
            start_second++;
            if(start_second > (60*15)-1)
                start_second=0;
        }
    }
}
read_checksum()
{
unsigned char b1,b2;
unsigned short noofb;
int size;
    numfread(chk,2);                   /* read in the header and data block */
    if(numread<2)                      /* read OK ?                         */
        clearerr(in);                  /* clear error                       */
    b1=chk[0];
    b2=chk[1];
//printf("%2x%2x\n",b1,b2);
    noofb=b1;
    noofb=noofb << 8 | b2;
    size=noofb;
    return(size);
}
GUR_DAG (DAG,YR,DOY)                      
int DAG;
int *YR;
int *DOY;
{
int    I;
int    LP;
int    DAGER;
DAGER=DAG;
//*-------------------------------------------------------------------------      
//*  Find year:                                                                   
//*                                                                               
for(I=1990;I<2010;I++)
{
    if(DAGER<365)
    {
//        printf("%5d %5d\n",I,DAGER);
//        printf("YEAR= %d DAGER= %d\n",I,DAGER);
        *YR=I;
        *DOY=DAGER;
        return(0);
    }
        
//               -- Leap year                 
    if ((I%4) == 0)
    {
        LP=1;
        DAGER=DAGER-365-1;
    }else{
        LP=0;
        DAGER=DAGER-365;
    }
}
return(1);
}
SEC_SEC (YR,MTH,DAY,SECND,MSECS)
int YR;
int MTH;
int DAY;
int SECND;
double *MSECS;                         /* total seconds to be returned      */
{
int DYR;                               /* flag for leap year                */
int IYR;                               /* number of leap-years since 1900   */
int YDY;                               /* number of days in current year    */

if(YR <= 50) YR=YR+100;                /* check for 2000 or more in 2 digits*/

if(YR >= 1900) YR=YR - 1900;
DYR = 0;
if((YR % 4) == 0) DYR = 1;
IYR = YR/4 - DYR;
//                -- Seconds to beginning of     
*MSECS = (float)(IYR*366) + (float)((YR-IYR)*365);
//                -- current year                
*MSECS = *MSECS*86400.0;
//                -- January                     
if (MTH == 1) YDY = DAY;
//                -- February                    
if (MTH == 2) YDY = DAY + 31;
//                -- ....                        
if (MTH == 3) YDY = DAY + DYR + 59;       
if (MTH == 4) YDY = DAY + DYR + 90;                                     
if (MTH == 5) YDY = DAY + DYR + 120;                                    
if (MTH == 6) YDY = DAY + DYR + 151;                                    
if (MTH == 7) YDY = DAY + DYR + 181;                                    
if (MTH == 8) YDY = DAY + DYR + 212;                                    
if (MTH == 9) YDY = DAY + DYR + 243;                                    
if (MTH ==10) YDY = DAY + DYR + 273;                                    
if (MTH ==11) YDY = DAY + DYR + 304;                                    
if (MTH ==12) YDY = DAY + DYR + 334;                                    
*MSECS = *MSECS + (float)(YDY*86400) + SECND;
}

//*=========================================================================      
//*                                                                               
//* Routine to convert from day-of-year to date                                   
//*                                                         Leif Kvamme 12-1-85   
DATO (DOY,DAY,MON,YR)                                          
int DOY;
int *DAY;
int *MON;
int YR;
{
int MTH[14],J,M,N;
for(J=1;J<8;J=J+2)
    MTH[J] = 31;
for(J=8;J<13;J=J+2)
    MTH[J] = 31;
MTH[2] = 28;
MTH[4] = 30;
MTH[6] = 30;
MTH[9] = 30;
MTH[11]= 30;
if ((YR%4) == 0) MTH[2] = 29;
M = 0;                                                                    
for(J=1;J<13;J++)
{
    M = M + MTH[J];
    N = DOY - M;                                                              
    if (N <= 0)
    {                                                       
        *MON = J;
        *DAY = N + MTH[J];                                                     
        goto four;                                                               
    }
}
four:                                                                   
J=0;
}
