/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:04/02/99                       */
/* Program:DIG_HHH                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program reads data from the Nanometrics HRD-24 digitizer through the */
/* serial port. Data from the three channels are timetagged by an internal  */
/* GPS clock. Data from each channel comes in "bundles" of 17 bytes each.   */
/* Depending on signal variations on each channel and between channels,     */
/* data "bundles" is output on the serial port from different times. Data   */
/* is therefor stored in a memory pipe of 60 seconds according to the time. */
/* DIG_N24 reads from this pipe and puts data into a standard SEISLOG       */
/* 1-second buffer.                                                         */
/* The program assumes that the HRD-24 is set up for 59 bundles, no radio   */
/* The sample rate may be 50 or 100 Hz.                                     */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                     Date: 04/02/99     Version 7.00
------------------------------------------------------------------------------
Description: Program reads data from serial port on HRD-24, decompresses the
             data, and store data in a temporary memory pipe, later read by
             DIG_N24.

Usage:       started by DIG_N24 (no parameters)     
#endif


/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)

char      stty_str[100];
char      port_str[12];
int       p;
int       go_on;
int       typ;
int       new_minute[3];
int       bryter;
int       buffer_switch;
float     ms;
double    MSECS;
double    MSEC70;
float     SEC;
float     MILSEC;
int       HELE_SEKUNDER;
int       YR,DOY,MTH,DAY,HR,MIN;
int       SECL;
int       adr;
int       GPS_STS;
int       old_adr;
unsigned  int seq_no;
unsigned  int smp_rat;
unsigned  int channel;
unsigned  int lseconds;
unsigned  short sseconds;
unsigned  int smple;
unsigned int lange[3];
unsigned int korte[3];
int past_end[3];
double big[3];
double small[3];
double fl_val1;
double fl_val2;
int       cnt;
int       verdi;
int       new_seq[3];
int       lst_adr[3];
int       lst_ls_cnt[3];
int       lst_ss_cnt[3];
int       verdier[3][1000];
int       decomp[3][1000];
unsigned  int ls_cnt;
unsigned  int ss_cnt;
int       ty;
int       nr;
int       dset;
int       ds[5];
int       x0;
int       bundle;
unsigned  char data_set_desc;
unsigned  char desc1;
unsigned  char desc2;
unsigned  char desc3;
unsigned  char desc4;
unsigned  char byt[1500];              /* Input buffer                      */
int       indx;
FILE      *in;
int       lp,k,i,ll,numread,numwrite;
int       inc_smp;
int       inc_buf;
int       t;
int       pp;
int       typ2;
int       id;                          /* Buffer indicator                  */
int       fd;                          /* return value open share           */
smod      *ptr;                        /* pointer to shared region          */
main()
{
bryter=0;
buffer_switch=0;
YR=1970;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSEC70);
//printf("%f\n",MSEC70);
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_HHH_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_HHH_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
switch(ptr->rate)
{
    case 50:
    inc_smp=200;
    inc_buf=200;
    break;
    case 100:
    inc_smp=100;
    inc_buf=100;
    break;
}
ptr -> min_idx=1;
ptr -> index=0;                        /* Initial buffer pointer            */
old_adr=0;
ptr->sts_gps=-1;
/*--------------------------------------------------------------------------*/
/* Open device for reading data from RD3                                    */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);

in=fopen(port_str,"r");


//if ((fd = open(port_str,O_RDONLY)) == ERROR)
//{
//    printf("CHK_SNC: Can't open serial line.\n");
//    exit(0);
//}
/*--------------------------------------------------------------------------*/
/* Read data from RD3 until one sync character is detected.                 */
/* Read the data block from RD3, Convert the header info and save the       */
/* current number of seconds for this block.                                */
/*--------------------------------------------------------------------------*/
//***********************
//**** START AT TOP OF BUFFER
//***********************
adr=0;
ptr->ixd[0]=0;
ptr->ixd[1]=0;
ptr->ixd[2]=0;
past_end[0]=0;
past_end[1]=0;
past_end[2]=0;
ptr->stopp_idx[0]=0;
ptr->stopp_idx[1]=0;
ptr->stopp_idx[2]=0;
ptr->old_seq[0]=0;
ptr->old_seq[1]=0;
ptr->old_seq[2]=0;
new_minute[0]=1;
new_minute[1]=1;
new_minute[2]=1;
go_on=0;
top:
for(;;)
{
  Sync();
  numfread(byt,21);
  bryter=1;
//printf("     OLD   F/R   SECONDS  SUBSEC    M-S/N  SEQUENCE  S/S-CH   VALUE\n");

//printf("%2x%2x%2x%2x  ",byt[0],byt[1],byt[2],byt[3]); /* oldest packet */
//printf("  %2x  ",byt[4]);                               /* first time restrans */
//printf("%2x%2x%2x%2x  ",byt[5],byt[6],byt[7],byt[8]); /* long seconds      */
//printf("  %2x%2x  ",byt[9],byt[10]);                    /* subseconds        */
//printf("   %2x%2x  ",byt[11],byt[12]);                   /* model serno.      */
//printf("%2x%2x%2x%2x  ",byt[13],byt[14],byt[15],byt[16]); /* sequence     */
//printf("    %2x  ",byt[17]);                              /* s per s  cha #   */
//printf("%2x%2x%2x\n",byt[18],byt[19],byt[20]);        /* first sample     */

  typ=byt[4];

  seq_no=byt[16];
  seq_no=seq_no<<8 | byt[15];
  seq_no=seq_no<<8 | byt[14];
  seq_no=seq_no<<8 | byt[13];

  smp_rat=byt[17];
  smp_rat=smp_rat>>3;

  channel=byt[17];
  channel=channel & 0x00000007;

  lseconds=byt[8];
  lseconds=lseconds<<8 | byt[7];
  lseconds=lseconds<<8 | byt[6];
  lseconds=lseconds<<8 | byt[5];

  sseconds=byt[10];
  sseconds=sseconds<<8 | byt[9];
//if(channel==0)
//    printf("CHANNEL %2d LSECONDS %8d SSECONDS %8d\n",channel,lseconds,sseconds);
  ms=(float)sseconds;
  ms=ms/10000.;
  smple=byt[20];
  smple=smple<<8 | byt[19];
  smple=smple<<8 | byt[18];
  smple=smple<<8;
  verdi=smple;
  verdi=verdi/256;
  x0=verdi;
  if(typ==2)
  {
    numfread(byt,17);
    typ2=byt[0];
    if(typ2==7)
    {
//        printf("STATUS PACKET: ");
//        printf("%2x ",byt[0]);
//        printf("%2x%2x%2x%2x ",byt[1],byt[2],byt[3],byt[4]);
//        printf("%2x%2x ",byt[5],byt[6]);
//        printf("%2x%2x ",byt[7],byt[8]); /* time diff */
//        printf("%2x%2x ",byt[9],byt[10]); /* time error */
//        printf("%2x%2x ",byt[11],byt[12]); /* frequency */
//        printf("%2x%2x ",byt[13],byt[14]); /* crystal temp */
//        printf("PLL %2x ",byt[15]);  /* PLL */
//        printf("GPS %2x ",byt[16]); /* GPS */
//        printf("\n");
      GPS_STS=byt[16];
      ptr->sts_gps=GPS_STS;
    }
  }
  if(typ==1)
  {
    if(channel==0 || channel==1 || channel==2)
    {
      MSECS=lseconds;
      SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
      if(go_on==0)
      {
        if(SEC<40.0 && SEC>10.0)
        {
          go_on=1;
        }else{
          printf(".");
          fflush(stdin);
          fflush(stdout);
          goto top;
        }
      }
//switch(channel)
//{
//  case 0:
//  printf("%8d\n",seq_no);
//  break;
//  case 1:
//  printf("          %8d\n",seq_no);
//  break;
//  case 2:
//  printf("                    %8d\n",seq_no);
//  break;
//}
      MILSEC=SEC+ms;
      HELE_SEKUNDER=(int)SEC;        /* whole seconds within minute       */
      if(channel==10)
      {
        printf("CH%2d LSEC %8d SSEC %5d ",channel,lseconds,sseconds);
        printf("%2d:%2d:%6.3f %2d/%2d/%2d %3d %8d GPS %1d\n",HR,MIN,SEC+ms,DAY,MTH,YR,DOY,seq_no,GPS_STS);
      }
//        printf("            %12d                 %10d  %3d%3d%8d\n",lseconds,seq_no,smp_rat,channel,verdi);
      numfread(byt,17*59);
    
      nr=0;
//        verdier[channel][nr]=verdi;
//        nr++;
      for(bundle=0;bundle<59;bundle++)
      {
           
        k=bundle*17;
        data_set_desc=byt[k];
        desc1=data_set_desc;
        desc1=desc1>>6;
        desc2=data_set_desc;
        desc2=desc2<<2;
        desc2=desc2>>6;
        desc3=data_set_desc;
        desc3=desc3<<4;
        desc3=desc3>>6;
        desc4=data_set_desc;
        desc4=desc4<<6;
        desc4=desc4>>6;
        ds[0]=desc1;
        ds[1]=desc2;
        ds[2]=desc3;
        ds[3]=desc4;
        for(dset=0;dset<4;dset++)
        {
          switch(ds[dset])
          {
            case 0:
            if(bryter==0)
              printf("Not Valid Type: %d\n",typ);
            bryter=1;
            break;
            case 1:                            /* 1 byte difference */
            for(t=(bundle*17+dset*4+1);t<(bundle*17+dset*4+5);t++)
            {
              smple=byt[t];
              smple=smple<<24;
              verdi=smple;
              verdi=verdi>>24;
              verdier[channel][nr]=verdi;
              nr++;
            }
            break;
            case 2:                            /* 2 byte difference */
            smple=byt[bundle*17+dset*4+2];
            smple=smple<<8 | byt[bundle*17+dset*4+1];
            smple=smple<<16;
            verdi=smple;
            verdi=verdi>>16;
            verdier[channel][nr]=verdi;
            nr++;
  
            smple=byt[bundle*17+dset*4+4];
            smple=smple<<8 | byt[bundle*17+dset*4+3];
            smple=smple<<16;
            verdi=smple;
            verdi=verdi>>16;
            verdier[channel][nr]=verdi;
            nr++;            
            break;
            case 3:                            /* 4 byte difference */
            smple=byt[bundle*17+dset*4+4];
            smple=smple<<8 | byt[bundle*17+dset*4+3];
            smple=smple<<8 | byt[bundle*17+dset*4+2];
            smple=smple<<8 | byt[bundle*17+dset*4+1];
            verdi=smple;
            verdier[channel][nr]=verdi;
            nr++;
            break;                
          }                      /* end of switch */     
        }                          /* end of dset */
      }                              /* end of bundles */
      decomp[channel][0]=verdier[channel][0];
//        for(ty=1;ty<nr;ty++)
//            decomp[channel][ty]=decomp[channel][ty-1]+verdier[channel][ty];


      for(ty=0;ty<nr;ty++)
      {
        decomp[channel][ty]=x0+verdier[channel][ty];
        x0=decomp[channel][ty];
      }

      MILSEC=MILSEC*100;            /* compute adress based on 50 Hz     */
         adr=(int)MILSEC;
      if(channel==8)
        printf("ch: %3d ADR:%10d NR:%10d\n",channel,adr,nr);
      ls_cnt=lseconds;
      ss_cnt=sseconds;

      if(ptr->old_seq[channel] != 0)
      {
        new_seq[channel]=seq_no;
        if((new_seq[channel]-ptr->old_seq[channel]) > 1)
        {
          printf("lost data on channel %d\n",channel);
          lost_data();        
        }
      }
      for(ty=0;ty<nr;ty++)
      {
        if(ptr->stopp_idx[channel]==0)
        {
          ptr->snc_pip0[channel][adr]=decomp[channel][ty];
          ptr->lsec0[channel][adr]=ls_cnt;
          ptr->ssec0[channel][adr]=ss_cnt;
        }else{
          ptr->snc_pip1[channel][adr]=decomp[channel][ty];
          ptr->lsec1[channel][adr]=ls_cnt;
          ptr->ssec1[channel][adr]=ss_cnt;

        }
//        ptr->all[channel][adr]=1;
        ss_cnt=ss_cnt+inc_buf;         /* add 20 millisec for 50 Hz         */
        if(ss_cnt > 9999)
        {
          ss_cnt=ss_cnt-10000;
          ls_cnt++;
        }             
        adr++;
        if(adr>5999)
        {
          adr=0;
          ptr->stopp_idx[channel]=(ptr->stopp_idx[channel]+1) % 2;
printf("switch buffer channel %d\n",channel);
        if(ptr->stopp_idx[0]==ptr->stopp_idx[1] && ptr->stopp_idx[1]==ptr->stopp_idx[2])
          ptr->min_idx=(ptr->min_idx+1) % 2;
        }
      } /* end of ty */
      lst_adr[channel]=adr;
      lst_ls_cnt[channel]=ls_cnt;
      lst_ss_cnt[channel]=ss_cnt;
      ptr->old_seq[channel]=seq_no;
//printf("%10d %10d %10d %10d %10d %10d\n",lange[0],korte[0],lange[1],korte[1],lange[2],korte[2]);
    }    /* end of if */
  }
}
    ptr->alive[1]=1;                   /* tell I am alive                   */
}
void    Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
        } while(sync_value != 0xAABB);
//printf("sync\n");
/*putchar(0x07);*/
}

void lost_data()
{
int diff;
      diff=adr-lst_adr[channel];
      if(diff < 0)              // do we have a wrap ?
      {
        diff=diff+6000;
        printf("lost data,   wrap, %d samples missing\n",diff);
      }else{
        printf("lost data, nowrap, %d samples missing\n",diff);
      }    
      ty=diff;
      adr=lst_adr[channel];
      ls_cnt=lst_ls_cnt[channel];
      ss_cnt=lst_ss_cnt[channel];
      for(ty=0;ty<nr;ty++)
      {
        if(ptr->stopp_idx[channel]==0)
        {
          ptr->snc_pip0[channel][adr]=0;
          ptr->lsec0[channel][adr]=ls_cnt;
          ptr->ssec0[channel][adr]=ss_cnt;
        }else{
          ptr->snc_pip1[channel][adr]=0;
          ptr->lsec1[channel][adr]=ls_cnt;
          ptr->ssec1[channel][adr]=ss_cnt;

        }
//        ptr->all[channel][adr]=1;
        ss_cnt=ss_cnt+inc_buf;         /* add 20 millisec for 50 Hz         */
        if(ss_cnt > 9999)
        {
          ss_cnt=ss_cnt-10000;
          ls_cnt++;
        }             
        adr++;
        if(adr>5999)
        {
          adr=0;
          ptr->stopp_idx[channel]=(ptr->stopp_idx[channel]+1) % 2;
printf("switch buffer channel %d\n",channel);
        if(ptr->stopp_idx[0]==ptr->stopp_idx[1] && ptr->stopp_idx[1]==ptr->stopp_idx[2])
          ptr->min_idx=(ptr->min_idx+1) % 2;
        }
      } /* end of ty */
}