/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:25/09/98                       */
/* Program:DIG_I24                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from RD3 through 1 minute buffer.       */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program reads 1-second buffers of digitized data from RD3 or RD6.
             Data are stored into buffers in shared memory for processing.

Usage:       started by buf_pip (no parameters)     
#endif
int k;

double              MSECS;             /* variables for time routines       */
double              MSEC70;
float               SEC;
float               MILSEC;
int                 YR,DOY,MTH,DAY,HR,MIN;
int                 SECL;
float               sec;               /* seconds x.xxx                     */
int                 SECND;
int                 DAG;
int                 ret;
int                 indx;
int                 i;
int                 sw;
int                 j;
int                 id;                /* Buffer indicator                  */
int                 pid;               /* Process id for READER_RD3         */
char                tid[50];           /* char string formatted time & date */
int                 kp;                /* counter                           */
int                 fd;                /* return value open share           */
smod                *ptr;              /* pointer to shared region          */
main()
{
int                 teller;            /* counter 1 second of samples       */
int                 ty;                /* counter samples in 1-minute buffer*/
int                 chan;              /* master channel                    */
unsigned int        slang;             /* current lon second                */
unsigned int        subs;              /* current subsecond                 */
int                 dig_n24_id;        /* process id dig_n24                */
int                 buf_pip_id;        /* process id buf_pip                */
int                 tell;
int                 inc_smp;
int                 first_second;
int                 inc_buf;
int                 cnt;
char                snd_msg[30];       /* send message                      */
char                rec_msg[10];       /* receive message                   */
unsigned int        cop;               /* variable for Send message         */
/*--------------------------------------------------------------------------*/
/* compute number of seconds since this date. Start of QNX internal clock   */
/*--------------------------------------------------------------------------*/
YR=1970;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSEC70);

dig_n24_id=qnx_name_attach(0,"dig_i24");  /* attach process as dig_n24      */
if(dig_n24_id == -1)
{
    printf("DIG_I24 attach failed\n"); /* exit                              */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_I24_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_I24_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
tell=0;
sw=0;
ptr -> index=0;                        /* Initial buffer pointer            */
teller = 0;                            /* start retreiving data at top      */
indx = 0;                              /* start at channel 1                */
/*--------------------------------------------------------------------------*/
/* start program that reads data from HRD24 and puts them into 1-minute buf */
/*--------------------------------------------------------------------------*/
system("/home/seislog/dig_irs&");
cnt=0;
sleep(5);
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* signal buf_pip that we can start */
for(;;)
{
for(ty=0;ty<60;ty++)
{
    if(ptr->rdy[ty][0] == 1 && ptr->rdy[ty][1] == 1 && ptr->rdy[ty][2] == 1)
    {        
        first_second=ty;
        goto data1;
    }else{
        ptr->alive[5]=1;   /* tell I am alive                   */
        delay(2);
    }
}
}
data1:
printf("FIRST SECOND %d\n",first_second);
for(;;)                                /* Enter indefinite loop reading data*/
{
    ptr->alive[5]=1;                   /* tell I am alive                   */
    chan=0;                            /* channel 0 is master channel       */
/*--------------------------------------------------------------------------*/
/* go through 1-minute buffer and retreive data with same time.             */
/* step 2 is hardcoded for 50 Hz samplingrate.                              */
/*--------------------------------------------------------------------------*/
newloop:
    for(ty=first_second;ty<60;ty++)          
    {
wait1:
        ptr->alive[5]=1;               /* tell I am alive                   */
/*--------------------------------------------------------------------------*/
/* check if seconds and subseconds are equal on all channel for this 'ty'   */
/*--------------------------------------------------------------------------*/
//printf("%3d %3d %3d %3d\n",ty,ptr->rdy[ty][0],ptr->rdy[ty][1],ptr->rdy[ty][2]);
        if(ptr->rdy[ty][0] == 1 && ptr->rdy[ty][1] == 1 && ptr->rdy[ty][2] == 1)
        {        
if(cnt != 0)
    printf("WAIT: %6d\n",cnt);
            cnt=0;
            goto data;
        }else{
            ptr->alive[5]=1;   /* tell I am alive                   */
            delay(100);
            cnt++;
//printf("WAIT: %6d\n",cnt);
            if(cnt > 75)
            {
                cnt=0;
                ty++;
                if(ty< 60 )
                {
                    goto wait1;
                }else{
                    goto newloop;
                }
            }
//            if(cnt>=10)
//            {
//                if(ptr->rdy[ty][0] != 1) ptr->rdy[ty][0]=1;
//                if(ptr->rdy[ty][1] != 1) ptr->rdy[ty][1]=1;
//                if(ptr->rdy[ty][2] != 1) ptr->rdy[ty][2]=1;
//            }
            goto wait1;
        }
data:
        if(sw==0)
        {
            if(ty==0)
                printf("\ndata at beginning of minute buffer\n");
            sw=1;
        }
        if(indx==0 )
        {


//            DAG=ptr->clr[ty][0];                    /* 45 days left to new year          */
//            SECND=ptr->clr[ty][1];
//            ret=GUR_DAG (DAG,&YR,&DOY);
//            DATO (DOY,&DAY,&MTH,YR);
//            SEC_SEC (YR,MTH,DAY,SECND,&MSECS);
        MSECS=ptr->klr[ty][0];
            SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
//            printf("%2d:%2d:%6.3f %2d/%2d/%2d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
            sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d",HR,MIN,SEC,DAY,MTH,YR,DOY);
            zero();            /* zeroes into blanks                */
            sprintf(ptr->timcpy,"%s",tid);
            sprintf(ptr->ibuf[ptr->index],"%s",tid);
        }
        for(k=0;k<ptr->rate;k++)
        {
            ptr->ibuf[ptr->index][indx+HEADER]=ptr->snc_pip[0][ptr->clr[ty][0]+k];
            ptr->ibuf[ptr->index][indx+1+HEADER]=ptr->snc_pip[1][ptr->clr[ty][0]+k];
            ptr->ibuf[ptr->index][indx+2+HEADER]=ptr->snc_pip[2][ptr->clr[ty][0]+k];
            indx=indx+ptr->nchan;
        }
//printf("                                       sec %3d %10d\n",ty,ptr->clr[ty][0]);
        ptr->rdy[ty][0]=0;
        ptr->rdy[ty][1]=0;
        ptr->rdy[ty][2]=0;
        indx=0;
//            printf("buffer ready\n");
            Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
            delay(900);
    }    /* end of buffer loop  */
    first_second=0;
}      /* end indefinite loop */


}
zero()
{
        for(kp=0;kp<12;kp++)
        {
            if(tid[kp]==' ')
                tid[kp]='0';
        }
        for(kp=13;kp<23;kp++)
        {
            if(tid[kp]==' ')
                tid[kp]='0';
        }
        if(tid[24]==' ')
            tid[24]='0';
        if(tid[25]==' ')
            tid[25]='0';        
}

double              MSECS;             /* variables for time routines       */
double              MSEC70;
double              MSEC89;
float               SEC;
float               MILSEC;
int                 YR,DOY,MTH,DAY,HR,MIN;
int                 SECL;
float               sec;               /* seconds x.xxx                     */




GUR_DAG (DAG,YR,DOY)                      
int DAG;
int *YR;
int *DOY;
{
int    I;
int    LP;
int    DAGER;
DAGER=DAG;
//*-------------------------------------------------------------------------      
//*  Find year:                                                                   
//*                                                                               
for(I=1990;I<2010;I++)
{
    if(DAGER<365)
    {
//        printf("%5d %5d\n",I,DAGER);
//        printf("YEAR= %d DAGER= %d\n",I,DAGER);
        *YR=I;
        *DOY=DAGER;
        return(0);
    }
        
//               -- Leap year                 
    if ((I%4) == 0)
    {
        LP=1;
        DAGER=DAGER-365-1;
    }else{
        LP=0;
        DAGER=DAGER-365;
    }
}
return(1);
}
SEC_SEC (YR,MTH,DAY,SECND,MSECS)
int YR;
int MTH;
int DAY;
int SECND;
double *MSECS;                         /* total seconds to be returned      */
{
int DYR;                               /* flag for leap year                */
int IYR;                               /* number of leap-years since 1900   */
int YDY;                               /* number of days in current year    */

if(YR <= 50) YR=YR+100;                /* check for 2000 or more in 2 digits*/

if(YR >= 1900) YR=YR - 1900;
DYR = 0;
if((YR % 4) == 0) DYR = 1;
IYR = YR/4 - DYR;
//                -- Seconds to beginning of     
*MSECS = (float)(IYR*366) + (float)((YR-IYR)*365);
//                -- current year                
*MSECS = *MSECS*86400.0;
//                -- January                     
if (MTH == 1) YDY = DAY;
//                -- February                    
if (MTH == 2) YDY = DAY + 31;
//                -- ....                        
if (MTH == 3) YDY = DAY + DYR + 59;       
if (MTH == 4) YDY = DAY + DYR + 90;                                     
if (MTH == 5) YDY = DAY + DYR + 120;                                    
if (MTH == 6) YDY = DAY + DYR + 151;                                    
if (MTH == 7) YDY = DAY + DYR + 181;                                    
if (MTH == 8) YDY = DAY + DYR + 212;                                    
if (MTH == 9) YDY = DAY + DYR + 243;                                    
if (MTH ==10) YDY = DAY + DYR + 273;                                    
if (MTH ==11) YDY = DAY + DYR + 304;                                    
if (MTH ==12) YDY = DAY + DYR + 334;                                    
*MSECS = *MSECS + (float)(YDY*86400) + SECND;
}

//*=========================================================================      
//*                                                                               
//* Routine to convert from day-of-year to date                                   
//*                                                         Leif Kvamme 12-1-85   
DATO (DOY,DAY,MON,YR)                                          
int DOY;
int *DAY;
int *MON;
int YR;
{
int MTH[14],J,M,N;
for(J=1;J<8;J=J+2)
    MTH[J] = 31;
for(J=8;J<13;J=J+2)
    MTH[J] = 31;
MTH[2] = 28;
MTH[4] = 30;
MTH[6] = 30;
MTH[9] = 30;
MTH[11]= 30;
if ((YR%4) == 0) MTH[2] = 29;
M = 0;                                                                    
for(J=1;J<13;J++)
{
    M = M + MTH[J];
    N = DOY - M;                                                              
    if (N <= 0)
    {                                                       
        *MON = J;
        *DAY = N + MTH[J];                                                     
        goto four;                                                               
    }
}
four:                                                                   
J=0;
}




