/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:14/02/95                       */
/* Program:DIG_LOW                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 11/05/00 t.u. CPU clock updated by digitizer GPS clock                   */
/* 27/06/00 t.u.  8.25 set port to -ihflow to avoid blocking                */
/* 28/06/00 t.u.  8.26 set port to -osflow to avoid blocking                */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from Geosys. Determine start of buffers */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
#define  BPS 3                         /* number of bytes per sample        */
#define  RHDR 57                       /* number of bytes left of header    */
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
int           kval;
unsigned char byt[3000];               /* Input buffer                      */
int           indx;                    /* index current channel             */
unsigned char *buf_ptr;
int           nbyt;                    /* total number of bytes to read     */
int           n;
int           only_one;
FILE          *in;                     /* input file pointer                */
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           err_cnt;                 /* error counter                     */
int           fd;                      /* return value open share           */
int           fh;                      /* input handler                     */
int           sts_snc;                 /* status after reading sync         */
int           sts_buf;                 /* status after reading data buffer  */
int           sec_ptr;
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
char          msg[10];
int           speed;
int           sr;
int           rdbyt;
main()
{
char          stty_str[100];
char          port_str[12];
pid_t         proxy;
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
int           clk_pps_id;
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
int      address;
int      sekvens;
char     timestring[100];
int      TIMEOUTS;
int      NOTSYNCS;
struct   timespec start;
unsigned int  cop;
setprio(getpid(),12);                  /* set the priority of this process  */
err_cnt=0;                             /* set error counter to zero         */
only_one=0;
dig_eda_id=qnx_name_attach(0,"dig_low"); /* attach this process as "dig_eda"*/
if(dig_eda_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_LOW attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_LOW_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_LOW_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* compute number of seconds since 01/01/1970 00:00:00.000                  */
/*--------------------------------------------------------------------------*/
YR=1970;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC(YR,MTH,DAY,HR,MIN,SEC,&MSEC70);




TIMEOUTS=0;
NOTSYNCS=0;
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
if(buf_pip_id == -1)
{
    printf("cant locate BUF_PIP\n");
    exit(0);
}
proxy=qnx_proxy_attach(0,0,0,-1);
ptr->min_proxy=proxy;
nbyt=ptr->rate*ptr->nchan*BPS+RHDR;             /* number of bytes to read  */

ptr -> index=0;                        /* Initial buffer pointer            */

/*--------------------------------------------------------------------------*/
/* Open device for reading data from GeoSys digitizer                       */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -ihflow -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);
/*--------------------------------------------------------------------------*/
/* open serial line where digitizer is connected                            */
/*--------------------------------------------------------------------------*/
if((fh=open(port_str,O_RDONLY | O_CACHE))==ERROR)
{
    printf("LOW_LOW: Can't open serial line %s\n",port_str);
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Read data from GeoSys until one 5 sync character is detected.            */
/*--------------------------------------------------------------------------*/
do
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    if(ptr->clktyp==0 || ptr->clktyp==8) /* indicate that timing system ok  */
        ptr->alive[3]=1;
    sts_snc=Sync1();
    if(sts_snc <= 0)
        TIMEOUTS++;
    if(TIMEOUTS >= 5)
    {
        close(fh);
        if(ptr->err_mail == 1)
            system("/home/seislog/message 8");
        exit(0);
    }        
}while(sts_snc <= 0);
TIMEOUTS=0;
printf("DIG_LOW   : Receiving data from digitizer !\n");
system("on -t /dev/con1 /home/seislog/receiving");
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
    ptr->alive[1]=1;                   /* tel I am alive                    */
    if(ptr->clktyp==0 || ptr->clktyp==8) /* indicate that timing system ok  */
        ptr->alive[3]=1;
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
    do
    {
        sts_snc=Sync1();               /* find SYNC in received data        */
        if(sts_snc <= 0)
            TIMEOUTS++;
        if(TIMEOUTS >= 5)
        {
            close(fh);
            if(ptr->err_mail == 1)
                system("/home/seislog/message 8");
            exit(0);
        }        
    }while(sts_snc <= 0);
    TIMEOUTS=0;
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
                                       /* signal.                           */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
    do
    {
        sts_buf=ED_3();
        if(sts_buf <= 0)
            TIMEOUTS++;
        if(TIMEOUTS >= 10)
        {
            close(fh);
            if(ptr->err_mail == 1)
                system("/home/seislog/message 8");
            exit(0);
        }        
    }while(sts_buf <=0 );
    TIMEOUTS=0;
    Get_Adr_GBV(&address,&sekvens,&YR,&MTH,&DAY,&HR,&MIN,&SEC);
    TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
//    SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
    SECTIM (MSECS,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
    sprintf(timestring,
    "%2d:%2d:%6.3f %2d/%2d/%4d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
//    printf("%2d:%2d:%6.3f %2d/%2d/%4d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
    if(ptr->clktyp==8)
        Mark_Time(timestring,address);
    if(ptr->clktyp==8)
    {
        start.tv_sec=(long)(MSECS-MSEC70);
        start.tv_nsec=0;
        system("rtc -s hw");
        clock_settime(CLOCK_REALTIME,&start);
    }
/*--------------------------------------------------------------------------*/
/* convert sample values into right format and store in correct buffer      */
/*--------------------------------------------------------------------------*/
    sec_ptr=HEADER;
    Convert();
    
/*--------------------------------------------------------------------------*/
/* send time info to BUF_PIP                                                */
/*--------------------------------------------------------------------------*/
//printf("DIG_GE1 finished buffer: %d\n",ptr->index);
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1);
}
}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3()
{
unsigned char check[5];
unsigned short chk_sum;
short chk_val;
int j;
buf_ptr=byt;
n=dev_read(fh,buf_ptr,nbyt+2,nbyt+2,0,11,0,0); /* read 'nbyt' bytes from serial port*/
//printf("nbyt=%5d n=%5d\n",nbyt,n);
switch(n)
{
    case -1:
    return(-1);
    break;

    case 0:
    return(0);
    break;

    case 509:       
    chk_sum=check[0];
    chk_sum=chk_sum << 8 || check[1];
    chk_val=chk_sum;
//for(j=500;j<510;j++)
//    printf("%6d ",byt[j]);
//printf("\n");
//printf("%2x %2x %2x\n",byt[47],byt[48],byt[49]);
    break;
}
return(1);
}
/****************************************************************************/
/* Function to convert data buffers from Lowcost                            /
/****************************************************************************/
void Convert()
{
register unsigned char lo,me,hi;
register unsigned int val;
unsigned short smp;
short sample;
register int verdi;
register int v1,v2,v3;
buf_ptr= &byt[57];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
    val=0;
    hi=(*buf_ptr++);
    me=(*buf_ptr++);
    lo=(*buf_ptr++);                   /* in this case dummy                */
    smp=hi;
    smp=smp<<8 | me;
    sample=smp;
    v1=sample;

    val=0;
    hi=(*buf_ptr++);
    me=(*buf_ptr++);
    lo=(*buf_ptr++);
    smp=hi;
    smp=smp<<8 | me;
    sample=smp;
//    v2=sample;
    v3=sample;

    val=0;
    hi=(*buf_ptr++);
    me=(*buf_ptr++);
    lo=(*buf_ptr++);
    smp=hi;
    smp=smp<<8 | me;
    sample=smp;
//    v3=sample;
    v2=sample;
    switch(ptr->nchan)
    {
        case 1:
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=v3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
}

void    Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
        } while(sync_value != 0x0a0d);
/**putchar(0x07);*/
}
Sync1()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0x4742);
//printf("snc\n");
return(1);
}

Get_Adr_GBV(address,seq,ad1,ad2,ad3,ad4,ad5,ad6)
int* address;
int* seq;
int* ad1;
int* ad2;
int* ad3;
int* ad4;
int* ad5;
float* ad6;
{
int add;
unsigned short ss;
unsigned int ss1;
unsigned short save;
int sequence;
int k;
int tusen;
int hundre;
int ti;
int en;
    

    ss1=byt[38];
    ss1=ss1<<8 | byt[39];
    ss1=ss1<<8 | byt[40];
    ss1=ss1<<8 | byt[41];
    sequence=ss1;
    *seq=sequence;                     /* extract sequence number           */
//printf("%5d\n",sequence);
//for(k=38;k<60;k++)
//    printf("%2x ",byt[k]);
//printf("\n");

    save=byt[54];
    ss=save;
    ss=ss & 0x000f;
    *address= ss;
    ss=save;
    ss=ss & 0x00f0;
    ss = ss >> 4;
    *address=*address+ss*10;
    add=*address;                      /* extract second to use as address  */

    save=byt[48];                      /* extract thousand in year          */
    ss=save;
    ss=ss & 0x00f0;
    ss=ss >> 4;
    tusen=ss*1000;

    ss=save;                           /* extract hundreds in year          */
    ss=ss & 0x000f;
    hundre=ss*100;

    save=byt[49];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in year              */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in year              */

    *ad1=tusen+hundre+ti+en;

    save=byt[50];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in months            */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in months            */

    *ad2=ti+en;

    save=byt[51];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in day               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in day               */

    *ad3=ti+en;

    save=byt[52];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in hour              */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in hour              */

    *ad4=ti+en;

    save=byt[53];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in minute            */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in minute            */

    *ad5=ti+en;

    save=byt[54];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in seconds           */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in seconds           */

    *ad6=ti+en;


//    ptr->mulpip[ser_idx][ptr->cha_def[0][7]*add]=ss1;

//    ptr->mulpip[ser_idx][(ptr->cha_def[0][7]*add)+1]=ss1;

//    ptr->mulpip[ser_idx][(ptr->cha_def[0][7]*add)+2]=ss1;

}
Mark_Time(tstring,add)
char* tstring;
int  add;
{
char timestamp[100];
int  k;
int  sec_ptr;
char str[100];

register char *tmstr;                  /* pointer to time string in buffer  */
register int temp;
register unsigned int shift;
register unsigned int swap;
register unsigned short msw;
register unsigned short lsw;
register unsigned short sw;
int      i;
char     tmpstr[80];

/*           000000000011111111112222222222333333333344444444445555555*/
/*           012345678901234567890123456789012345678901234567890123456*/
/*           00:00:00.000 00/00/0000 000 sts 00:00:00.000 00/00/00 000  "*/

//ti[55]=0x0d;
//ti[56]=0x0a;
sec_ptr=0;
for(k=0;k<60;k++)
    timestamp[k]=' ';
timestamp[55]=0x0d;
timestamp[56]=0x0a;
for(k=0;k<27;k++)
    timestamp[k]=tstring[k];
if(ptr->clktyp==8)
{
    timestamp[28]='S';
    timestamp[29]='E';
    timestamp[30]='T';
}
if(ptr->clktyp==0)
{
    timestamp[28]='C';
    timestamp[29]='P';
    timestamp[30]='U';
}
timestamp[31]='\0';
if(timestamp[0]==' ')
    timestamp[0]='0';
if(timestamp[3]==' ')
    timestamp[3]='0';
if(timestamp[6]==' ')
    timestamp[6]='0';
if(timestamp[9]==' ')
    timestamp[9]='0';
if(timestamp[10]==' ')
    timestamp[10]='0';
if(timestamp[13]==' ')
    timestamp[13]='0';
if(timestamp[16]==' ')
    timestamp[16]='0';
if(timestamp[24]==' ')
    timestamp[24]='0';
if(timestamp[25]==' ')
    timestamp[25]='0';
for(k=0;k<32;k++)
    ptr->timcpy[k]=timestamp[k];
//printf("MUL_SERb: %s\n",timestamp);
k=0;
shift=timestamp[3];
shift=shift << 8;
shift=shift | timestamp[2];
shift=shift << 8;
shift=shift | timestamp[1];
shift=shift << 8;
shift=shift | timestamp[0];
ptr->ibuf[ptr->index][sec_ptr]=shift;
sec_ptr=sec_ptr+1;
k++;

shift=timestamp[7];
shift=shift << 8;
shift=shift | timestamp[6];
shift=shift << 8;
shift=shift | timestamp[5];
shift=shift << 8;
shift=shift | timestamp[4];
ptr->ibuf[ptr->index][sec_ptr]=shift;
sec_ptr=sec_ptr+1;
k++;

shift=timestamp[11];
shift=shift << 8;
shift=shift | timestamp[10];
shift=shift << 8;
shift=shift | timestamp[9];
shift=shift << 8;
shift=shift | timestamp[8];
ptr->ibuf[ptr->index][sec_ptr]=shift;
sec_ptr=sec_ptr+1;
k++;

shift=timestamp[15];
shift=shift << 8;
shift=shift | timestamp[14];
shift=shift << 8;
shift=shift | timestamp[13];
shift=shift << 8;
shift=shift | timestamp[12];
ptr->ibuf[ptr->index][sec_ptr]=shift;
sec_ptr=sec_ptr+1;
k++;

shift=timestamp[19];
shift=shift << 8;
shift=shift | timestamp[18];
shift=shift << 8;
shift=shift | timestamp[17];
shift=shift << 8;
shift=shift | timestamp[16];
ptr->ibuf[ptr->index][sec_ptr]=shift;
sec_ptr=sec_ptr+1;
k++;

shift=timestamp[23];
shift=shift << 8;
shift=shift | timestamp[22];
shift=shift << 8;
shift=shift | timestamp[21];
shift=shift << 8;
shift=shift | timestamp[20];
ptr->ibuf[ptr->index][sec_ptr]=shift;
sec_ptr=sec_ptr+1;
k++;

shift=timestamp[27];
shift=shift << 8;
shift=shift | timestamp[26];
shift=shift << 8;
shift=shift | timestamp[25];
shift=shift << 8;
shift=shift | timestamp[24];
ptr->ibuf[ptr->index][sec_ptr]=shift;
sec_ptr=sec_ptr+1;
k++;

shift=timestamp[31];
shift=shift << 8;
shift=shift | timestamp[30];
shift=shift << 8;
shift=shift | timestamp[29];
shift=shift << 8;
shift=shift | timestamp[28];
ptr->ibuf[ptr->index][sec_ptr]=shift;
sec_ptr=sec_ptr+1;
k++;

}
