/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:05/04/96                       */
/* Program:DIG_ME2                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 16/09/97 include help file                                               */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program controls analog-digital converter type Keithly 1202 16 channels  */
/* 12 bit resolution. 2 boards can be installed to support up to 32 channels*/
/* 1 second of data are stored in shared memory buffers for processing.     */
/*                                                                          */
/****************************************************************************/     
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <conio.h>
#include <sys/irqinfo.h>               /* needed for interrupt-routines     */
#include <sys/proxy.h>
#include <sys/kernel.h>                /* needed for interrupt-routines     */
#include <sys/name.h>
//#define B1 0x380
//#define  BASE 0x200                    /* base address for DAS1202 board    */
//#define  IRQ  0x5                      /* interrupt level                   */
//#define  BASE2 0x300
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program controls analog-digital converter type Keithly 1202
             16 channels 12 bit resolution. 2 boards can be installed to 
             support up to 32 channels. 1 second of data are stored in shared
             memory buffers for processing. 

Usage:       started by buf_pip (no parameters)
#endif


char     snd_msg[30];
char     rec_msg[30];
pid_t    proxy;
volatile unsigned counter;
volatile unsigned cntr;
int      fd;                           /* return value open share           */
volatile smod     *ptr;                /* pointer to shared region          */
volatile unsigned char  mux;
volatile unsigned char  val1;
volatile unsigned char  val2;
volatile unsigned char  ch;
volatile unsigned short cnt;
volatile unsigned short temp;
volatile unsigned char  irqen;
volatile BASE;
volatile BASE2;
volatile IRQ;
volatile int            sample;
volatile int            i;
volatile int            scan;
volatile unsigned char  sts;
volatile int            totkan;
volatile int            merkan;
volatile int            index;
volatile int            tel;
volatile int            samp;
volatile int            irq_block;
volatile int            mil;
volatile int            sc_sec;
volatile int            ms;
volatile int            ms1;
volatile int            chan;
volatile int            chan2;
volatile int            teller;
volatile int            bank;
volatile unsigned char status1;
volatile unsigned char status2;
/****************************************************************************/
/*      The Hardware Interrupt handler for DAS 1200 board                   */
/*--------------------------------------------------------------------------*/
/****************************************************************************/
#pragma off( check_stack );
pid_t far handler()
{
    if(totkan > 16)                    /* more than one board ?             */
    {
        for(i=0;i<16;i++)
        {
            val1=inp(BASE);            /* read LSB                          */
            val2=inp(BASE+0x1);        /* read MSB                          */
            outp(BASE+0x8,0x00);       /* clear interrupt                   */
            if(i <= 14)
                outp(BASE,0x00);       /* start conversion first board      */
            cnt=val1;
            temp=val2;
            temp=temp<<8;
            cnt=cnt | temp;
            cnt=cnt>>4;                    /* shift out channel number          */
            sample=cnt;                    /* signed long integer               */
            sample=sample-2048;            /* from binary offset to +/- 2048    */
            ptr->ibuf[irq_block][scan+i+HEADER]=sample; /* into shared memory          */
            if(i <= 14)                    /* first board                       */
            {
                do
                {
                    teller++;
                }while ( (inp(BASE+0x8) & 0x80) != 0); /* AD busy ?             */
            }
        }
        for(i=16;i<(merkan+16);i++)
        {
            outp(BASE2+0x8,0x00);
            outp(BASE2,0x00);       /* start conversion second board      */
            do
            {
                teller++;
            }while ( (inp(BASE2+0x8) & 0x80) != 0); /* AD busy ?             */
            val1=inp(BASE2);            /* read LSB                          */
            val2=inp(BASE2+0x1);        /* read MSB                          */
            cnt=val1;
            temp=val2;
            temp=temp<<8;
            cnt=cnt | temp;
            cnt=cnt>>4;                    /* shift out channel number          */
            sample=cnt;                    /* signed long integer               */
            sample=sample-2048;            /* from binary offset to +/- 2048    */
            ptr->ibuf[irq_block][scan+i+HEADER]=sample; /* into shared memory          */
        }
    }else{
        for(i=0;i<ptr->nchan;i++)
        {
            val1=inp(BASE);            /* read LSB                          */
            val2=inp(BASE+0x1);        /* read MSB                          */
            outp(BASE+0x8,0x00);       /* clear interrupt                   */
            if(i <= chan)
                outp(BASE,0x00);       /* start conversion first board      */
            cnt=val1;
            temp=val2;
            temp=temp<<8;
            cnt=cnt | temp;
            cnt=cnt>>4;                    /* shift out channel number          */
            sample=cnt;                    /* signed long integer               */
            sample=sample-2048;            /* from binary offset to +/- 2048    */
            ptr->ibuf[irq_block][scan+i+HEADER]=sample; /* into shared memory          */
            if(i <= chan)                    /* first board                       */
            {
                do
                {
                    teller++;
                }while ( (inp(BASE+0x8) & 0x80) != 0); /* AD busy ?             */
            }
        }
    }

        tel++;                             /* increment scan counter            */
        scan=scan+ptr->nchan;              /* increment buffer index            */
        if(tel==sc_sec)                    /* check for 1 second                */
        {
            irq_block=(irq_block + 1) % 2; /* switch buffer                     */
            scan=0;                        /* zero index                        */
            tel=0;                         /* zero index                        */
            outp(BASE+0x8,0x00);           /* clear interrupt                   */
            return(proxy);             /* signal main to save 1 sec buffer  */
        }
        outp(BASE+0x8,0x00);               /* clear interrupt                   */
        return(0);
}
#pragma on( check_stack);
/****************************************************************************/
/*      The Hardware Interrupt Handler counting milliseconds                */
/*--------------------------------------------------------------------------*/
/* This interrupt handler is connected up with the systems own timer, and   */
/* is not more accurate than the systems CPU time. However, it is accurate  */
/* enough to count to 1000 milliseconds.                                    */
/****************************************************************************/
#pragma off( check_stack );
pid_t far mil_handler()
{
    mil++;
    if(mil == ms)
    {
        mil=0;
        outp(BASE,0x00);
    }
return(0);
}
#pragma on( check_stack);
main()
{
int      dig_met_id;                   /* id this process                   */
int      buf_pip_id;                   /* id BUF_PIP                        */
char     rep_msg[10];
char     rec_msg[10];
unsigned int cop;
unsigned char kanaler;
int      j;
int      k;
int      gps_garm_id;
int      id;                           /* id for the 1 PPS interrupt handler*/
int      idd;                          /* id for the 1msec interrupt handler*/
int      fh;                           /* file descriptor for serial device */
struct   timespec start;               /* structure for setting system time */
dig_met_id=qnx_name_attach(0,"dig_met");
if(dig_met_id == 1)
{
    printf("DIG_MET attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_MET_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_MET_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
setprio(getpid(),14);                  /* set the priority this process=14  */
printf("rate:%d\n",ptr->rate);
sc_sec=ptr->rate;
ms=(int)(ptr->srate * 1000.0 + 0.000001);
printf("Samplerate: %d Scans per second: %d\n",ms,sc_sec);
BASE=ptr->AD_BASE_1;
BASE2=ptr->AD_BASE_2;

IRQ=ptr->ADIRQ;
chan=ptr->nchan-2;

/*--------------------------------------------------------------------------*/
/* locate BUF_PIP and get its process id                                    */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
/*--------------------------------------------------------------------------*/
/* set up a proxy for the interrupt handler to kick                         */
/*--------------------------------------------------------------------------*/
if((proxy = qnx_proxy_attach(0,0,0,0)) == -1)
{
    printf("DIG_MET Unable to attach proxy. ");
    return;
}
/*--------------------------------------------------------------------------*/
/* Attach the interrupt for the DAS1200 board                               */
/*--------------------------------------------------------------------------*/
if(( id=qnx_hint_attach(IRQ,&handler,FP_SEG(&counter)))==-1)
{
    printf("Unable to attach the a/d interrupt.");
    return;
}
system("ticksize 1");                  /* set ticksize of system to 1 msec  */
    irqen=IRQ<<4 | 0x0;                /* IRQ + software start a/d          */
irqen=irqen | 0x80;                    /* enable interrupt                  */
samp=0;
tel=0;
cnt=0;
scan=0;
mil=0;
irq_block=0;                           /* initial buffer index              */
totkan=ptr->nchan;
merkan=totkan-16;
chan2=merkan-2;
outp(BASE+0x8,0x0);
outp(BASE+0x9,0x0);
outp(BASE2+0x8,0x0);
outp(BASE2+0x9,0x0);
if(ptr->nchan > 16)                    /* more than 16 channels => 2 boards */
{
    kanaler=15;                        /* set first board to max (0-15 ch)  */
    kanaler = kanaler << 4;            /* shift left into high nibble       */
    outp(BASE+0x2,kanaler);            /* write to multiplex register       */
    kanaler=(char)ptr->nchan-16-1;     /* set up channels for second board  */
    kanaler = kanaler << 4;            /* shift left into high nibble       */
    outp(BASE2+0x2,kanaler);           /* write to multiplex register       */
    outp(BASE2+0x8,0x00);              /* clear interrupt first board       */
    outp(BASE+0x8,0x00);               /* clear interrupt second board      */
}else{
    kanaler=(char)ptr->nchan-1;        /* last channel number to digitize   */
    kanaler=kanaler << 4;              /* shift left into high nibble       */
    outp(BASE+0x2,kanaler);            /* write to multiplex register       */
    outp(BASE+0x8,0x00);               /* clear interrupt                   */
}
status1=inp(BASE+0x8);
status2=inp(BASE2+0x8);
//printf("  8h %4x %4x\n",status1,status2);
status1=inp(BASE+0x9);
status2=inp(BASE2+0x9);
//printf("  9h %4x %4x\n",status1,status2);
status1=inp(BASE+0xa);
status2=inp(BASE2+0xa);
//printf("  ah %4x %4x\n",status1,status2);
status1=inp(BASE+0xb);
status2=inp(BASE2+0xb);
//printf("  bh %4x %4x\n",status1,status2);
status1=inp(BASE+0xc);
status2=inp(BASE2+0xc);
//printf("  ch %4x %4x\n",status1,status2);
status1=inp(BASE+0xd);
status2=inp(BASE2+0xd);
//printf("  dh %4x %4x\n",status1,status2);
    outp(BASE+0xe,0x00);          
    outp(BASE2+0xe,0xff);          
status1=inp(BASE+0xe);
status2=inp(BASE2+0xe);
//printf("  eh %4x %4x\n",status1,status2);
status1=inp(BASE+0x403);
status2=inp(BASE2+0x403);
//printf("403h %4x %4x\n",status1,status2);
status1=inp(BASE+0x404);
status2=inp(BASE2+0x404);
//printf("404h %4x %4x\n",status1,status2);
status1=inp(BASE+0x407);
status2=inp(BASE2+0x407);
//printf("407h %4x %4x\n",status1,status2);
ptr->index=0;                          /* pointer to first buffer to fill   */
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pip to give id       */
/*--------------------------------------------------------------------------*/
/* Attach the interrupt to count milliseconds                               */
/*--------------------------------------------------------------------------*/
if(( idd=qnx_hint_attach(0,&mil_handler,FP_SEG(&cntr)))==-1)
{
    printf("Unable to attach the interrupt.");
    return;
}
    outp(BASE+0x9,irqen);              /* enable IRQ + external trigger     */
    outp(BASE2+0x9,0x0);              /* disable IRQ + external trigger     */
for(;;)
{
    outp(BASE2+0x9,0x0);              /* disable IRQ + external trigger     */
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Receive(proxy,0,0);                /* value ready                       */
//printf("         bank=%d\n",bank);
    bank=teller;
    teller=0;
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1);
}
}
