/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:25/09/98                       */
/* Program:DIG_N24                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from RD3 through 1 minute buffer.       */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program reads 1-second buffers of digitized data from RD3 or RD6.
             Data are stored into buffers in shared memory for processing.

Usage:       started by buf_pip (no parameters)     
#endif


double              MSECS;             /* variables for time routines       */
double              MSEC70;
float               SEC;
float               MILSEC;
int                 YR,DOY,MTH,DAY,HR,MIN;
int                 SECL;
int                 dummy;
float               sec;               /* seconds x.xxx                     */
int                 indx;
int                 i;
int                 sw;
int                 j;
int                 id;                /* Buffer indicator                  */
int                 pid;               /* Process id for READER_RD3         */
char                tid[50];           /* char string formatted time & date */
int                 kp;                /* counter                           */
int                 fd;                /* return value open share           */
smod                *ptr;              /* pointer to shared region          */
main()
{
//unsigned            msg;
//int idd;
int                 teller;            /* counter 1 second of samples       */
//int drv_id;
int                 ty;                /* counter samples in 1-minute buffer*/
//unsigned int oldsec;
//unsigned int oldssec;
int                 chan;              /* master channel                    */
unsigned int        slang;             /* current lon second                */
unsigned int        subs;              /* current subsecond                 */
int                 dig_n24_id;        /* process id dig_n24                */
int                 buf_pip_id;        /* process id buf_pip                */
int                 tell;
int                 ix;
int                 inc_smp;
int                 inc_buf;
char                snd_msg[30];       /* send message                      */
char                rec_msg[10];       /* receive message                   */
unsigned int        cop;               /* variable for Send message         */
/*--------------------------------------------------------------------------*/
/* compute number of seconds since this date. Start of QNX internal clock   */
/*--------------------------------------------------------------------------*/
YR=1970;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSEC70);

dig_n24_id=qnx_name_attach(0,"dig_n24");  /* attach process as dig_n24      */
if(dig_n24_id == -1)
{
    printf("DIG_N24 attach failed\n"); /* exit                              */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_N24_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_N24_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
for(i=0;i<6000;i++)
{
    for(j=0;j<ptr->nchan;j++)
    {
        ptr->lsec0[j][i]=0;
        ptr->ssec0[j][i]=0;
        ptr->lsec1[j][i]=0;
        ptr->ssec1[j][i]=0;
        ptr->snc_pip0[j][i]=0;
        ptr->snc_pip1[j][i]=0;
        ptr->all[j][i]=0;
    }
}
tell=0;
sw=0;
ptr -> index=0;                        /* Initial buffer pointer            */
teller = 0;                            /* start retreiving data at top      */
indx = 0;                              /* start at channel 1                */
switch(ptr->rate)
{
    case 50:
    inc_smp=20;
    inc_buf=20;
    break;
    case 100:
    inc_smp=10;
    inc_buf=10;
    break;
}
/*--------------------------------------------------------------------------*/
/* start program that reads data from HRD24 and puts them into 1-minute buf */
/*--------------------------------------------------------------------------*/
system("/home/seislog/dig_hhh&");
sleep(5);
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* signal buf_pip that we can start */
do
{
  sleep(1);
}while(ptr->min_idx==1);
do
{
  sleep(1);
}while(ptr->min_idx==0);
ix=ptr->min_idx;
for(;;)                                /* Enter indefinite loop reading data*/
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    chan=0;                            /* channel 0 is master channel       */
/*--------------------------------------------------------------------------*/
/* go through 1-minute buffer and retreive data with same time.             */
/* step 2 is hardcoded for 50 Hz samplingrate.                              */
/*--------------------------------------------------------------------------*/
    for(ty=0;ty<6000;ty++)          
    {
wait1:
        ptr->alive[1]=1;               /* tell I am alive                   */
/*--------------------------------------------------------------------------*/
/* check if seconds and subseconds are equal on all channel for this 'ty'   */
/*--------------------------------------------------------------------------*/
//        if(ptr->all[0][ty] == 1 && ptr->all[1][ty]==1 && ptr->all[2][ty]==1)
//        {        
          if(ix==0)
          {
            slang=ptr->lsec0[0][ty];
            subs=ptr->ssec0[0][ty];
          }else{
            slang=ptr->lsec1[0][ty];
            subs=ptr->ssec1[0][ty];
          }
            if(sw==0)
            {
                if(ty==0)
                    printf("\ndata at beginning of minute buffer\n");
                sw=1;
            }
            if(indx==0 )
            {
                MSECS=slang;
//printf("%f %f\n",MSECS,MSEC70);
                SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
//                printf("%2d:%2d:%6.3f %2d/%2d/%2d %3d",HR,MIN,SEC,DAY,MTH,YR,DOY);
//printf("SUBS %8d\n",subs);

                MILSEC=SEC+((float)subs)/10000.0;
//                printf("%2d:%2d:%6.3f %2d/%2d/%2d %3d",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                switch(ptr->sts_gps)
                {
                case -1: // no status from GPS so far
                sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                break;
                case 0:
                sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d SET",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                break;
                case 1:
                sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d SET",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                break;
                case 2:
                sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d NSY",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                break;
                case 3:
                sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d NSY",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                break;
                case 4:
                sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d NSY",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                break;
                case 5:
                sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d NSY",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                break;
                case 6:
                sprintf(tid,"%2d:%2d:%6.3f %2d/%2d/%2d %3d NSY",HR,MIN,MILSEC,DAY,MTH,YR,DOY);
                break;
                }
                zero();            /* zeroes into blanks                */
                sprintf(ptr->timcpy,"%s",tid);
                sprintf(ptr->ibuf[ptr->index],"%s",tid);
            }
          if(ix==0)
          {
            ptr->ibuf[ptr->index][indx+HEADER]=ptr->snc_pip0[0][ty];
//            ptr->snc_pip0[0][ty]=0;
            ptr->ibuf[ptr->index][indx+1+HEADER]=ptr->snc_pip0[1][ty];
//            ptr->snc_pip0[1][ty]=0;
            ptr->ibuf[ptr->index][indx+2+HEADER]=ptr->snc_pip0[2][ty];
//            ptr->snc_pip0[2][ty]=0;
          }else{
            ptr->ibuf[ptr->index][indx+HEADER]=ptr->snc_pip1[0][ty];
//            ptr->snc_pip1[0][ty]=0;
            ptr->ibuf[ptr->index][indx+1+HEADER]=ptr->snc_pip1[1][ty];
//            ptr->snc_pip1[1][ty]=0;
            ptr->ibuf[ptr->index][indx+2+HEADER]=ptr->snc_pip1[2][ty];
//            ptr->snc_pip1[2][ty]=0;

          }
            ptr->all[0][ty]=0;
            ptr->all[1][ty]=0;
            ptr->all[2][ty]=0;
            indx=indx+ptr->nchan;
            teller++;
            if(teller==ptr->rate)
            {
                teller=0;
                indx=0;
//                printf("buffer ready\n");
                Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
                    delay(900);
            } 
            ptr->alive[1]=1;           /* tell I am alive                   */
//        }else{
//            delay(10);
//            goto wait1;
//        }
    }    /* end of buffer loop  */
    do
    {
      sleep(1);
      printf("Waiting...\n");
    }while(ix==ptr->min_idx);
    ix=ptr->min_idx;
}      /* end indefinite loop */


}
zero()
{
        for(kp=0;kp<12;kp++)
        {
            if(tid[kp]==' ')
                tid[kp]='0';
        }
        for(kp=13;kp<23;kp++)
        {
            if(tid[kp]==' ')
                tid[kp]='0';
        }
        if(tid[24]==' ')
            tid[24]='0';
        if(tid[25]==' ')
            tid[25]='0';        
}
