/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:22/12/94                       */
/* Program:DIG_NAN                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 16/09/97 included help file                                              */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from RD3. Determine start of buffers    */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                         Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program reads 1-second buffers of digitized data from RD3 or RD6.
             Data are stored into buffers in shared memory for processing.

Usage:       started by buf_pip (no parameters)     
#endif


/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
int lrec;
unsigned char       byt[1500];         /* Input buffer                      */
short               bytcnt;            /* HEADER number of bytes in data    */
short               idnum;             /* HEADER id-number for unit         */
short               datacode;          /* HEADER type of data               */
short               headertype;        /* HEADER type of header             */
short               timemarkcount;     /* HEADER time indicator             */
int                 minute;            /* HEADER minutes                    */
short               second;            /* HEADER seconds                    */
short               mes1;              /* HEADER message                    */
short               mes2;              /* HEADER message                    */
short               mes3;              /* HEADER message                    */
short               mescnt;            /* HEADER number of messages         */
short               sample;            /* HEADER                            */
short               tik;               /* HEADER                            */
unsigned int        byte0;
unsigned int        byte1;
unsigned int        byte2;
unsigned int        byte3;
unsigned short      b1,b2;

unsigned char       *buf_ptr;
int                 nbyt;
int                 n;
int                 fh;
int                 sts_snc;
int                 sts_buf;
int                 sec_ptr;

short               old;
short               diff;
int                 gainconst[4];
short               g1,g2,g3,g4,g5,g6;
int                 ch1,ch2,ch3,ch4,ch5,ch6;
int                 indx;
int                 lost_sec;
FILE                *in;
int                 lp,k,i,ll,numread,numwrite;
int                 id;                /* Buffer indicator                  */
int                 pid;               /* Process id for READER_RD3         */
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
main()
{
char      stty_str[100];
char      port_str[12];
int dig_nan_id;
int buf_pip_id;
char rep_msg[10];
char snd_msg[30];
char rec_msg[10];
int   TIMEOUTS;
int   NOTSYNCS;
unsigned int cop;
dig_nan_id=qnx_name_attach(0,"dig_nano");  /* attach process as dig_nan     */
if(dig_nan_id == -1)
{
    printf("DIG_NAN attach failed\n"); /* exit                              */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_NAN_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_NAN_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
TIMEOUTS=0;
NOTSYNCS=0;
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);

ptr -> index=0;                        /* Initial buffer pointer            */

gainconst[0]=1;                        /* Initialize gain table constants   */
gainconst[1]=8;                        /* -----------------"--------------  */
gainconst[2]=64;                       /* -----------------"--------------  */
gainconst[3]=512;                      /* -----------------"--------------  */

if(ptr->unit==53)
{
    lrec=7*ptr->rate+20;
}else{
    lrec=14*ptr->rate+20;
}
//printf("LENGTH %d\n",lrec);
nbyt=lrec;
/*--------------------------------------------------------------------------*/
/* open serial line where digitizer is connected                            */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);


if((fh=open(port_str,O_RDONLY | O_CACHE))==ERROR)
{
    printf("DIG_NAN: Can't open serial line %s\n",port_str);
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Read data from RD3 until one sync character is detected.                 */
/* Read the data block from RD3, Convert the header info and save the       */
/* current number of seconds for this block.                                */
/*--------------------------------------------------------------------------*/
do
{
    ptr->alive[1]=1;
    sts_snc=Sync1();
    if(sts_snc <= 0)
        TIMEOUTS++;
    if(TIMEOUTS >= 5)
    {
        close(fh);
        if(ptr->err_mail == 1)
            system("/home/seislog/message 8");
        exit(0);
    }
}while(sts_snc <= 0);
TIMEOUTS=0;
printf("DIG_NAN: Receiving data from digitizer !\n");
system("on -t /dev/con1 /home/seislog/receiving");
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pip to give id       */
for(;;)                                /* Enter indefinite loop reading data*/
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pip to be ready      */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send from buf_pip    */ 
    do
    {
        ptr->alive[1]=1;
        sts_snc=Sync1();
        if(sts_snc <= 0)
            TIMEOUTS++;
        if(TIMEOUTS >= 5)
        {
            close(fh);
            if(ptr->err_mail == 1)
                system("/home/seislog/message 8");
            exit(0);
        }
    }while(sts_snc <= 0);
    TIMEOUTS=0;
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time     */
    switch(ptr->unit)
    {
        case 40:
        break;
        case 53:
    do
    {
        ptr->alive[1]=1;
        sts_buf=ED_3();                /* read 1 sec data from 3-chan unit  */ 
        if(sts_buf <= 0)
            TIMEOUTS++;
        if(TIMEOUTS >= 5)
        {
            close(fh);
            if(ptr->err_mail == 1)
                system("/home/seislog/message 8");
            exit(0);
        }
    }while(sts_buf <= 0);
    TIMEOUTS=0;
//Conv_Head();
        sec_ptr=HEADER;
        Convert_3();                   /* convert 1 sec data 6-chan unit    */
        break;
        case 56:                       /* read 1 sec data from 6-chan unit  */
    do
    {
        ptr->alive[1]=1;
        sts_buf=ED_3();                /* read 1 sec data from 3-chan unit  */ 
        if(sts_buf <= 0)
            TIMEOUTS++;
        if(TIMEOUTS >= 5)
        {
            close(fh);
            if(ptr->err_mail == 1)
                system("/home/seislog/message 8");
            exit(0);
        }
    }while(sts_buf <= 0);
    TIMEOUTS=0;
        sec_ptr=HEADER;
        Convert_6();
        break;
    }
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1);
}
}
/****************************************************************************/
/* Function to convert data buffers from Lowcost                            /
/****************************************************************************/
void Convert_3()
{
register unsigned char lo,me,hi;
register unsigned int val;
register short val1,val2,val3;
register unsigned char gain;
unsigned short smp;
short sample;
register int verdi;
register int v1,v2,v3;
buf_ptr= &byt[20];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
    val1 = (*buf_ptr++)<<8;
    val1 += *buf_ptr++;
    val2 = (*buf_ptr++)<<8;
    val2 += *buf_ptr++;
    val3 = (*buf_ptr++)<<8;
    val3 += *buf_ptr++;

    gain = *buf_ptr++;
    g1 = gain>>4;
    g2 = (gain & 12)>>2;
    g3 = gain & 3;

    switch(ptr->nchan)
    {
        case 1:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val2*gainconst[g2];
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val2*gainconst[g2];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val3*gainconst[g3];
        sec_ptr=sec_ptr+1;
        break;
    }
}
}
/****************************************************************************/
/* Function to convert data buffers from Lowcost                            /
/****************************************************************************/
void Convert_6()
{
register unsigned char lo,me,hi;
register unsigned int val;
register short val1,val2,val3,val4,val5,val6;
register unsigned char gain;
unsigned short smp;
short sample;
register int verdi;
register int v1,v2,v3;
buf_ptr= &byt[20];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
    val1 = (*buf_ptr++)<<8;
    val1 += *buf_ptr++;
    val2 = (*buf_ptr++)<<8;
    val2 += *buf_ptr++;
    val3 = (*buf_ptr++)<<8;
    val3 += *buf_ptr++;

    gain = *buf_ptr++;
    g1 = gain>>4;
    g2 = (gain & 12)>>2;
    g3 = gain & 3;

    val4 = (*buf_ptr++)<<8;
    val4 += *buf_ptr++;
    val5 = (*buf_ptr++)<<8;
    val5 += *buf_ptr++;
    val6 = (*buf_ptr++)<<8;
    val6 += *buf_ptr++;

    gain = *buf_ptr++;
    g4 = gain>>4;
    g5 = (gain & 12)>>2;
    g6 = gain & 3;

    switch(ptr->nchan)
    {
        case 1:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val2*gainconst[g2];
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val2*gainconst[g2];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val3*gainconst[g3];
        sec_ptr=sec_ptr+1;
        break;

        case 4:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val2*gainconst[g2];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val3*gainconst[g3];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val4*gainconst[g4];
        sec_ptr=sec_ptr+1;
        break;

        case 5:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val2*gainconst[g2];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val3*gainconst[g3];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val4*gainconst[g4];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val5*gainconst[g5];
        sec_ptr=sec_ptr+1;
        break;

        case 6:
        ptr->ibuf[ptr->index][sec_ptr]=val1*gainconst[g1];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val2*gainconst[g2];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val3*gainconst[g3];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val4*gainconst[g4];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val5*gainconst[g5];
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=val6*gainconst[g6];
        sec_ptr=sec_ptr+1;
        break;
    }
}
}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3()
{
buf_ptr=byt;
n=dev_read(fh,buf_ptr,nbyt+2,nbyt+2,0,11,0,0); /* read 'nbyt' bytes from serial port*/
switch(n)
{
    case -1:
    return(-1);
    break;

    case 0:
    return(0);
    break;

    case 1:       
    break;
}
return(1);
}
void    Convert_Zero()
{
    register unsigned char *buf_ptr;
    indx=0;

    buf_ptr = &byt[20];                /* point to first data byte          */

    while(buf_ptr < &byt[370])         /* while ptr < address of byt[370]   */
    {
        ptr->ibuf[ptr->index][indx+HEADER]=0;
        ptr->ibuf[ptr->index][indx+1+HEADER]=0;
        ptr->ibuf[ptr->index][indx+2+HEADER]=0;
        indx=indx+3;
    }
}
void    Conv_Head()
{
    register unsigned char *buf_ptr;
    buf_ptr = byt;

    bytcnt = *buf_ptr++;
    bytcnt = bytcnt<<8;
    bytcnt = bytcnt + *buf_ptr++;
    idnum  = *buf_ptr++;
    datacode = *buf_ptr++;
    headertype=*buf_ptr++;
    timemarkcount=*buf_ptr++;
    byte0 = *buf_ptr++;
    byte0 = byte0 << 24;
    byte1 = *buf_ptr++;
    byte1 = byte1 << 16;
    byte2 = *buf_ptr++;
    byte2 = byte2 << 8;
    byte3 = *buf_ptr++;
    minute = byte0 | byte1 | byte2 | byte3;
    b1 = *buf_ptr++;
    b1 = b1 << 8;
    b2 = *buf_ptr++;
    second = b1 | b2;
    b1 = *buf_ptr++;
    b1 = b1 << 8;
    b2 = *buf_ptr++;
    sample = b1 | b2;
    b1 = *buf_ptr++;
    b1 = b1 << 8;
    b2 = *buf_ptr++;
    tik= b1 | b2;
    mes1=*buf_ptr++;
    mes2=*buf_ptr++;
    mes3=*buf_ptr++;
    mescnt=*buf_ptr++;
printf("%8d\n",headertype);
}
Sync1()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0xAA16);
//printf("snc\n");
return(1);
}
