/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:14/02/95                       */
/* Program:DIG_RAM                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from Geosys. Determine start of buffers */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <math.h>
#include <sys/sched.h>
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)
#define RHDR 57
FILE *new;
int           gam;
int           old[12];
int           diff;
float           news;
double           cons;
int           kval;
int           sts_snc;
unsigned char byt[4000];               /* Input buffer                      */
int           tiden[10];
int           seq;
int           seqold;
unsigned int  sequence;
int           indx;                    /* index current channel             */
FILE          *in;                     /* input file pointer                */
float         ssek;
float         mmil;
int           sttt;
int           aarr;
int           mont;
int           dage;
int           timm;
int           minu;
int           seku;
int           mill;
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           err_cnt;                 /* error counter                     */
int           fd;                      /* return value open share           */
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
int           speed;
int           sr;
int           rdbyt;
main()
{
char          stty_str[100];
char          port_str[12];
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
unsigned int  cop;
setprio(getpid(),12);                  /* set the priority of this process  */
err_cnt=0;                             /* set error counter to zero         */
dig_eda_id=qnx_name_attach(0,"dig_ram"); /* attach this process as "dig_eda"*/
if(dig_eda_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_RAM attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_RAM_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_RAM_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);

rdbyt=ptr->rate*3*12+57;               /* total number of bytes to read     */
//rdbyt=1865;
ptr -> index=0;                        /* Initial buffer pointer            */

/*--------------------------------------------------------------------------*/
/* Open device for reading data from GeoSys digitizer                       */
/*--------------------------------------------------------------------------*/
sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -osflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);

in=fopen(port_str,"r");
/*--------------------------------------------------------------------------*/
/* Read data from GeoSys until one 5 sync character is detected.            */
/*--------------------------------------------------------------------------*/
for(i=0;i<5;i++)
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Sync();

}
printf("DIG_RAM   : Receiving data from digitizer !\n");
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
    ptr->alive[1]=1;                   /* tel I am alive                    */
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
    Sync();                            /* find sync char. in received data  */
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
                                       /* signal.                           */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
    ED_3();
/*--------------------------------------------------------------------------*/
/* convert sample values into right format and store in correct buffer      */
/*--------------------------------------------------------------------------*/
    ConED_3();
//exit(0);
/*--------------------------------------------------------------------------*/
/* send time info to BUF_PIP                                                */
/*--------------------------------------------------------------------------*/
    Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
}
}
/****************************************************************************/
/* Function to data buffers from the digitizer                              */
/****************************************************************************/
void    ED_3()
{
unsigned int m1000;
unsigned int m100;
unsigned int m10;
unsigned int m1; 
int l1,i1;
char tid[30];

numfread(byt,rdbyt);                     /* read in the header and data block */
if(numread<rdbyt)                        /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}
//printf("numread: %d\n",numread);
//new=fopen("/home/seislog/dmp","w+");
//fprintf(new,"%4d\n",ptr->iblk);
//fwrite(byt,rdbyt,1,new);
//fclose(new);
//exit(0);
sequence=0;
sequence=byt[38];                      /* extract sequence number bytes and */
sequence=sequence<<8 | byt[39];        /* convert to integer value          */
sequence=sequence << 8 | byt[40];
sequence=sequence << 8 | byt[41];
seq=(int)sequence;
//printf("%d\n",seq);
if(seq-seqold >= 2)
{
    putchar(0x7);
}
for(i1=39;i1<65;i1++)
    printf("%2x ",byt[i1]);
printf("\n");
//printf("%x\n",byt[47]);
seqold=seq;
seq=0;
m10=byt[49];
m10=m10>>4;
m1=byt[49];
m1=m1 & 0x000f;
seq=seq+(int)m10*10;
seq=seq+(int)m1;
tiden[0]=seq;

seq=0;
    m10=byt[50];                       /* get month                         */
    m10=m10>>4;
    m1=byt[50];
    m1=m1 & 0x000f;
    seq=(int)m10*10;
    seq=seq+(int)m1;
tiden[1]=seq;
    sequence=seq;

seq=0;
    m10=byt[51];                       /* get day                           */
    m10=m10>>4;
    m1=byt[51];
    m1=m1 & 0x000f;
    seq=(int)m10*10;
    seq=seq+(int)m1;
tiden[2]=seq;
    sequence=seq;

seq=0;
    m10=byt[52];                       /* get hour                          */
    m10=m10>>4;
    m1=byt[52];
    m1=m1 & 0x000f;
    seq=(int)m10*10;
    seq=seq+(int)m1;
tiden[3]=seq;
    sequence=seq;

seq=0;
    m10=byt[53];                       /* get minute                        */
    m10=m10>>4;
    m1=byt[53];
    m1=m1 & 0x000f;
    seq=(int)m10*10;
    seq=seq+(int)m1;
tiden[4]=seq;
    sequence=seq;

seq=0;
    m10=byt[54];                       /* get second                        */
    m10=m10>>4;
    m1=byt[54];
    m1=m1 & 0x000f;
    seq=(int)m10*10;
    seq=seq+(int)m1;
tiden[5]=seq;
    sequence=seq;

seq=0;
    m1000=byt[55];                       /* get milliseconds                  */
    m1000=m1000>>4;
    m100=byt[55];
    m100=m100 & 0x000f;
    seq=(int)m1000*1000;
    seq=seq+(int)m100*100;
    m10=byt[56];                            /* get years                         */
    m10=m10>>4;
    m1=byt[56];
    m1=m1 & 0x000f;
    seq=seq+(int)m10*10;
    seq=seq+(int)m1;
tiden[6]=seq;
    sequence=seq;       /* OR in the year                    */

seku=tiden[5];
mill=tiden[6];
            ssek=(float)seku;
            mmil=(float)mill;
            mmil=mmil/1000.;
            ssek=ssek+mmil;
            ptr->inpptid[0]=tiden[0];
            ptr->inpptid[1]=tiden[1];
            ptr->inpptid[2]=tiden[2];
            ptr->inpptid[3]=tiden[3];
            ptr->inpptid[4]=tiden[4];
            ptr->inppsec=ssek;
}
/****************************************************************************/
/* Function to convert data buffers from ED_24                               /
/****************************************************************************/
void ConED_3()
{
register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
int smp;
int verdi;
int ix;
int verdi2;
unsigned int val;
int kanal;
int iy;
indx=0;

buf_ptr= &byt[RHDR];         /* point to first data byte */
while(buf_ptr < &byt[rdbyt])
{
    for(kanal=0;kanal < ptr->nchan;kanal++)
    {
        hi=(*buf_ptr++);
        me=(*buf_ptr++);
        lo=(*buf_ptr++);
//        hi=0x7f;
//        me=0xff;
//        lo=0xfc;
//        hi=0xfd;
//        me=0x36;
//        lo=0x41;
    val=0;
    val=hi;
    val=val<<8 | me;
    val=val<<8 | lo;
    val=val<<8 | 0x00;
    verdi=val;
//verdi=verdi/512;
verdi=verdi/1024;
//if(kanal==1)
//{
    if(verdi > 1800000 || verdi < -1800000)
    {
        printf("%8d %2x %2x %2x\n",verdi,hi,me,lo);
        verdi=old[kanal];
    }
//}
old[kanal]=verdi;
verdi2=verdi;
        ptr->ibuf[ptr->index][indx+kanal+HEADER]=verdi;
    }
    indx=indx+ptr->nchan;
}
goto jmp;
for(kanal=0;kanal < ptr->nchan;kanal++)
{
    for(iy=0;iy < ptr->nsamp-1;iy++)
    {
        gam=ptr->ibuf[ptr->index][0+kanal+HEADER];
            if(verdi > 1900000 || verdi < -1900000)
                ptr->ibuf[ptr->index][0+kanal+HEADER]=0;
    }                
}        

goto jmp;
for(kanal=0;kanal < ptr->nchan;kanal++)
{
    gam=ptr->ibuf[ptr->index][0+kanal+HEADER];
    for(iy=0;iy<ptr->nsamp-1;iy++)
    {
        verdi=ptr->ibuf[ptr->index][iy+kanal+HEADER];
        diff=abs(verdi-gam);
if(kanal==1)
    printf("%8d\n",diff);
        if(diff >= 60000)
        {
if(kanal==1)
    printf("%3d verdi:%8d gam:%8d diff:%8d",iy,verdi,gam,diff);
            verdi=(gam+ptr->ibuf[ptr->index][iy+1+HEADER])/2;
            ptr->ibuf[ptr->index][iy+kanal+HEADER]=verdi;
if(kanal==1)
    printf(" avg:%8d nxt:%8d\n",verdi,ptr->ibuf[ptr->index][iy+1+HEADER]);
//            ptr->ibuf[ptr->index][iy+kanal+HEADER]=-2000000;
            gam=verdi;
        }else{
            gam=verdi;
        }
    }
}
jmp:
indx=0;
}

void    Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
        } while(sync_value != 0x4742);
/*putchar(0x07);*/
}
void conv_smp(hi,me,lo,smp)
unsigned hi;
unsigned me;
unsigned lo;
int      *smp;
{
unsigned int val;
int verdi;
    val=0;
    val=hi;
    val=val<<8 | me;
    val=val<<8 | lo;
    val=val<<8;
    verdi=val;
//    *smp=verdi/256;
*smp=verdi;
    *smp=*smp/4;
}
