/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:14/02/95                       */
/* Program:DIG_IDS                                                          */
/*                                                                          */
/* Last update:                                                             */
/* 09/06/00 8.23 t.u. disable flowcontrol on serial port                    */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from Geosys. Determine start of buffers */
/* and send request to clock.                                               */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
/* read a2 bytes from in and store them at a1                               */

#define numfread(a1,a2) numread = fread(a1,1,a2,in)

union GPS_SECUNDOS
{
  char cadena[4];
  float numero;
}SEGUND_GPS;

union GPS_OFFSET
{
  char cadena[4];
  float numero;
}OFFSET_GPS;

union GPS_SEMANA
{
  char cadenab[2];
  short int semana;
}SEMANA_GPS;

div_t sx;
int semana;
int segundos;
int numero_dias;
int hora_GPS;
int minuto_GPS;
int segundo_GPS;
int DiaHoraGPS;

double   MSECS;
double   MSEC70;
double   MSEC80;
double   MSEC99;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;

int      kj;
int           kval;
unsigned char byt[3000];               /* Input buffer                      */
unsigned char bytes[30];               /* Input buffer                      */

unsigned char one[20];
unsigned char *buf_ptr;
unsigned char two[10];
unsigned char *two_ptr;

int           n;
int           lst_val1;
int           lst_val2;
int           lst_val3;
int           indx;                    /* index current channel             */
int           only_one;
FILE          *in;                     /* input file pointer                */
int           s;
int           i;                       /* counter                           */
int           numread;                 /* read definition                   */
int           err_cnt;                 /* error counter                     */
int           fd;                      /* return value open share           */
int           nbyt;                    /* number of bytes to read           */
int           fh;                      /* input handler                     */
int           sts_snc;                 /* status after reading sync         */
int           sts_buf;                 /* status after reading data buffer  */
int           sec_ptr;
smod          *ptr;                    /* pointer to shared region          */
char          snd_msg[30];             /* return message containing time    */
char          snd_tim[35];

char          msg[10];
int           speed;
int           sr;
int           ro;
int           rdbyt;
char          stty_str[100];
char          port_str[12];
int BIT;
int MODEM;
int BAUD;
int TIME;
int KAN;
int GPS;
int UNIT;                              /* 1-16 bit 1 ch  50 Hz              */
                                       /* 2-16 bit 1 ch 100 Hz              */
                                       /* 3-16 bit 3 ch  50 Hz              */
                                       /* 4-16 bit 3 ch 100 Hz              */
                                       /* 5-24 bit 1 ch  50 Hz              */
                                       /* 6-24 bit 1 ch 100 Hz              */
                                       /* 7-24 bit 3 ch  50 Hz              */
                                       /* 8-24 bit 3 ch 100 Hz              */
                                       /* 9-16 bit 1 ch  50 Hz GPS          */
                                       /*10-16 bit 1 ch 100 Hz GPS          */
                                       /*11-16 bit 3 ch  50 Hz GPS          */
                                       /*12-16 bit 3 ch 100 Hz GPS          */
                                       /*13-24 bit 1 ch  50 Hz GPS          */
                                       /*14-24 bit 1 ch 100 Hz GPS          */
                                       /*15-24 bit 3 ch  50 Hz GPS          */
                                       /*16-24 bit 3 ch 100 Hz GPS          */
                                       /*17-16 bit 1 ch  50 Hz MODEM        */
                                       /*18-16 bit 1 ch 100 Hz MODEM        */
                                       /*19-24 bit 1 ch  50 Hz MODEM        */
                                       /*20-24 bit 1 ch 100 Hz MODEM        */

main()
{
pid_t         proxy;
int           dig_eda_id;              /* Process id for ED_24              */
int           buf_pip_id;              /* Process id for BUF_PIP            */
int           clk_pps_id;
char          rep_msg[10];             /* reply message                     */
char          rec_msg[10];             /* receive message                   */
unsigned int  cop;
int teller;
YR=1970;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSEC70);
YR=1980;
MTH=1;
DAY=6;
TIMSEC(YR,MTH,DAY,HR,MIN,SEC,&MSEC80);
YR=1999;
MTH=8;
DAY=22;
TIMSEC(YR,MTH,DAY,HR,MIN,SEC,&MSEC99);
setprio(getpid(),12);                  /* set the priority of this process  */
BIT=MODEM=BAUD=TIME=KAN=0;
err_cnt=0;                             /* set error counter to zero         */
only_one=0;
dig_eda_id=qnx_name_attach(0,"dig_ror"); /* attach this process as "dig_eda"*/
if(dig_eda_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_ROR attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_ROR_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_ROR_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
if(ptr->clktyp !=8)
{
  clk_pps_id=qnx_name_locate(0,"clk_pps",0,&cop);
  if(clk_pps_id == -1)
  {
    printf("DIG_ROR:cant locate CLK_PPS\n");
    exit(0);
  }
}

ptr->min_proxy=proxy;

ptr -> index=0;                        /* Initial buffer pointer            */
reinit:
ptr->bd_dig=2400;

reopen:
/*--------------------------------------------------------------------------*/
/* Open device for reading data from GeoSys digitizer                       */
/*--------------------------------------------------------------------------*/

sprintf(port_str,"/dev/ser%d",ptr->prt_dig);
sprintf(stty_str,"stty +raw -ohflow -lkhflow -ihpaged -osflow -ihflow baud=%d par=none bits=8 stopb=1 < %s\n",ptr->bd_dig,port_str);
system (stty_str);


/*--------------------------------------------------------------------------*/
/* open serial line where digitizer is connected                            */
/*--------------------------------------------------------------------------*/
if((fh=open(port_str,O_RDONLY | O_CACHE))==ERROR)
{
    printf("DIG_ROR: Can't open serial line %s\n",port_str);
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* find out what type of digitizer                                          */
/*--------------------------------------------------------------------------*/

sts_snc=chk_typ();
switch(sts_snc)
{
    case 3:
    close(fh);
    ptr->bd_dig=ptr->bd_dig*2;
    if(ptr->bd_dig <= 19200)
        goto reopen;
    printf("Could not find any known package type at 2400,4800,9600\n");
    exit(0);
    break;
    case 10:
    printf("RESET package recognized at %d\n",ptr->bd_dig);
    close(fh);
    goto reinit;
    break;
    case 20:
    printf("16 BIT package recognized at %d\n",ptr->bd_dig);
    BIT=16;
    BAUD=ptr->bd_dig;
    break;
    case 30:
    printf("INFO package recognized at %d\n",ptr->bd_dig);
    close(fh);
    goto reinit;
    break;
    case 40:
    printf("CALIB package recognized at %d\n",ptr->bd_dig);
    close(fh);
    goto reinit;
    break;
    case 50:
    printf("24 BIT package recognized at %d\n",ptr->bd_dig);
    BIT=24;
    BAUD=ptr->bd_dig;
    break;
    case 60:
    printf("MODEM package recognized at %d\n",ptr->bd_dig);
    BAUD=ptr->bd_dig;
    MODEM=1;
    KAN=1;
    break;
    case 70:
    printf("SLOW package recognized at %d\n",ptr->bd_dig);
    close(fh);
    goto reinit;
    break;
    case 80:
    printf("TIME package recognized at %d\n",ptr->bd_dig);
    TIME=1;
    BAUD=ptr->bd_dig;
    close(fh);
    goto reinit;
    break;
}
printf("BIT:%3d MODEM:%2d BAUD:%5d TIME:%2d KAN:%3d\n",BIT,MODEM,BAUD,TIME,KAN);
sts_snc=chk_pak(0,&teller);
printf("%5d Reset       packets\n",teller);
sts_snc=chk_pak(1,&teller);
printf("%5d 16-bits     packets\n",teller);
sts_snc=chk_pak(2,&teller);
printf("%5d Info        packets\n",teller);
sts_snc=chk_pak(3,&teller);
printf("%5d Calibration packets\n",teller);
sts_snc=chk_pak(4,&teller);
printf("%5d 24-bits     packets\n",teller);
sts_snc=chk_pak(5,&teller);
printf("%5d Modem       packets\n",teller);
sts_snc=chk_pak(10,&teller);
printf("%5d Slow        packets\n",teller);
sts_snc=chk_pak(20,&teller);
printf("%5d Time        packets\n",teller);
sts_snc=chk_pak(80,&teller);
printf("%5d GPS         packets\n",teller);
if(MODEM==1)
    UNIT=17;
if(BIT==16)
    UNIT=1;
if(BIT==24 && ptr->nchan==3)
    UNIT=2;
if(BIT==24 && ptr->nchan==1)
    UNIT=5;
if(BIT==16 && ptr->nchan==3 && GPS==1)
   UNIT=11;
printf("UNIT=%2d\n",UNIT);
/*--------------------------------------------------------------------------*/
/* Read data from GeoSys until one 5 sync character is detected.            */
/*--------------------------------------------------------------------------*/

for(i=0;i<3;i++)
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    switch(UNIT)
    {
        case 17:
        sts_snc=Sync1();
        break;
        case 1:
        sts_snc=Sync2();
        break;
        case 2:
        sts_snc=Sync3();
        break;
        case 5:
        sts_snc=Sync3();
        break;
        case 11:
        sts_snc=Sync5();
        break;
    }
    switch(sts_snc)                    /* test if return value is ok        */
    {
        case 1:                        /* reading byte ok                   */
        break;
        case 0:                        /* timeout looking for sync          */
        break;
        case -1:                       /* error reading data                */
        break;
    }
}
printf("DIG_ROR   : Receiving data from digitizer !\n");
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
 if(UNIT==11)
  Sync5();    // search for start GPS string
    ptr->alive[1]=1;                   /* tel I am alive                    */
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pipe to be ready     */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
///////    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time-tag */
                                       /* signal.                           */
/*--------------------------------------------------------------------------*/
/* read 1 data block and extract time information into send-message         */
/*--------------------------------------------------------------------------*/
    sts_buf=ED_3();
//s=0;
//for(i=0;i<50;i++)
//{
//if(i<1)
//  exit(0); 
//  printf("%2d %2x%2x %2x%2x %2x%2x\n",
//i,byt[s],byt[s+1],byt[s+2],byt[s+3],byt[s+4],byt[s+5]);
//s=s+6;
//}

//printf("sts_buf= %d\n",sts_buf);
    switch(sts_buf)                    /* test if return value is ok        */
    {
        case 1:                        /* reading byte ok                   */
        Send(buf_pip_id,&snd_tim,&rec_msg,32,1); /* let buf_pip get the time-tag */
        break;
        case 0:                        /* timeout reading data buffer       */
        sec_ptr=HEADER;
        ErrBuf();
        goto goon;
        break;
        case -1:                       /* error reading data                */
        sec_ptr=HEADER;
        ErrBuf();
        goto goon;
        break;
    }
/*--------------------------------------------------------------------------*/
/* convert sample values into right format and store in correct buffer      */
/*--------------------------------------------------------------------------*/
    sec_ptr=HEADER;
    switch(UNIT)
    {
        case 17:
        Convert1();
        break;
        case 1:
        Convert1();
        break;
        case 2:
        Convert1();
        break;
        case 5:
        Convert1();
        break;
        case 11:
        Convert1();
        break;

    }
/*--------------------------------------------------------------------------*/
/* send time info to BUF_PIP                                                */
/*--------------------------------------------------------------------------*/
goon:
i=0;
//printf("DIG_ROR finished buffer: %d\n",ptr->index);
    Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
}
}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3()
{
register unsigned short sync_value = 0;
int lo;
int num_byt;
int cnt;
cnt=0;
num_byt=ptr->rate * 2 * ptr->nchan;
buf_ptr=one;
for(;;)
{
  if(UNIT !=11)
    n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
//printf("ED last byte: %2x\n",one[0]);

    switch(n)
    {
        case -1:                           /* ERROR reading data                */
        return(-1);
        break;
        case 0:                            /* TIMEOUT reading data              */
        return(0);
        break;
        case 1:                            /* Reading data ok                   */
        sync_value = sync_value << 8 | *buf_ptr;
        break;
    }
    switch(UNIT)
    {
        case 17:
        num_byt=ptr->rate * 2 * ptr->nchan;
        if(sync_value == 0xaa05)
        {
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
        }
        if(cnt>=num_byt)
            return(1);
        break;
        case 11:
        num_byt=ptr->rate * 2 * ptr->nchan;
cnt=0;
byt[0]=0;
byt[1]=0;

for(kj=0;kj<(num_byt/6);kj++)
{
//if(kj<3)
//  printf(" 1ED %2x%2x\n",byt[0],byt[1]);

//if(kj==0)
//  printf("ED %2d",kj);
min:
   sts_snc=Sync55();      // find next aa01 or aa02
//if(kj<3)
//  printf(" 2ED %2x%2x\n",byt[0],byt[1]);
   switch(sts_snc)
   {
     case 2:             // minute mark
     printf("Minute mark !\n");
     goto min;
     break;
     case 1:          // data block
     buf_ptr=one;
     n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
     byt[cnt]=one[0];
//if(kj==0)
//printf(" %2x",byt[cnt]);
     cnt++;
     n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
     byt[cnt]=one[0];
//if(kj==0)
//printf("%2x",byt[cnt]);
     cnt++;
     n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
     byt[cnt]=one[0];
//if(kj==0)
//printf(" %2x",byt[cnt]);
     cnt++;
     n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
     byt[cnt]=one[0];
//if(kj==0)
//printf("%2x",byt[cnt]);
     cnt++;
     n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
     byt[cnt]=one[0];
//if(kj==0)
//printf(" %2x",byt[cnt]);
     cnt++;
     n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
     byt[cnt]=one[0];
//if(kj==0)
//printf("%2x\n",byt[cnt]);
     cnt++;

     break;
   }
//if(kj<3)
//printf(" indED %2x%2x\n",byt[0],byt[1]);

}
//printf(" endED %2x%2x",byt[0],byt[1]);

//exit(0);
//printf("CNT=%d\n",cnt);
return(1);

        

        case 5:
        num_byt=ptr->rate * 3 * ptr->nchan;
        if(sync_value == 0xaa04)
        {
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
        }
        if(cnt>=num_byt)
            return(1);
        break;
        case 1:
        num_byt=ptr->rate * 2 * ptr->nchan;
        if(sync_value == 0xaa01)
        {
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
        }
        if(cnt>=num_byt)
            return(1);
        break;
        case 2:
        num_byt=ptr->rate * 3 * ptr->nchan;
        if(sync_value == 0xaa04)
        {
//printf("snc\n");
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
            n=dev_read(fh,buf_ptr,1,1,0,11,0,0); /* read 'nbyt' bytes from serial port*/
            byt[cnt]=one[0];
            cnt++;
        }
        if(cnt>=num_byt)
            return(1);
        break;
    }
}
return(-1);
}

/****************************************************************************/
/* Function to convert data buffers from IDS-24                              /
/****************************************************************************/
void Convert1()
{
int tt;
register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register unsigned short vals;
register short verdis;
register int verdi;
register int v1,v2,v3;
static union CHAR_INTE
{
    char cadena[4];
    int numero;
    unsigned long numerol;
}INTEGER_CHAR;

int xx,yy;

unsigned short smp;
int num_byt;
short sample;
tt=0;
    switch(UNIT)
    {
        case 17:
        num_byt=ptr->rate * 2 * ptr->nchan;
        break;
        case 11:
        num_byt=ptr->rate * 2 * ptr->nchan;
        break;

        case 1:
        num_byt=ptr->rate * 2 * ptr->nchan;
        break;
        case 2:
        num_byt=ptr->rate * 3 * ptr->nchan;
        break;
        case 5:
        num_byt=ptr->rate * 3 * ptr->nchan;
        break;
    }
//num_byt=ptr->rate * 2 * ptr->nchan;
buf_ptr= &byt[0];                     /* point to first data byte          */
xx=0;
yy=HEADER;
//printf(" here %2x%2x",byt[0],byt[1]);
while(buf_ptr < &byt[num_byt])
{
  val=0;

  switch(ptr->nchan)
  {
    case 1:                            // 1 CHANNEL
    switch(UNIT)
    {
      case 1:
      lo=(*buf_ptr++);
      hi=(*buf_ptr++);
      vals=hi;
      vals=vals<<8 | lo;
      verdis=vals;
      verdi=verdis;
      v1=verdi;
      ptr->ibuf[ptr->index][sec_ptr]=v1;
            
      sec_ptr=sec_ptr+1;
      break;
      case 5:
      lo=(*buf_ptr++);
      me=(*buf_ptr++);
      hi=(*buf_ptr++);
      INTEGER_CHAR.cadena[2]=hi;
      INTEGER_CHAR.cadena[1]=me;
      INTEGER_CHAR.cadena[0]=lo;
      v1=INTEGER_CHAR.numerol;
      v1-=0x00800000L;
      ptr->ibuf[ptr->index][sec_ptr]=v1;
      sec_ptr=sec_ptr+1;
      break;
    }
    break;
    case 2:                            // 2 CHANNELS
    lo=(*buf_ptr++);
    hi=(*buf_ptr++);
    vals=hi;
    vals=vals<<8 | lo;
    verdis=vals;
    verdi=verdis;
    v1=verdi;
    ptr->ibuf[ptr->index][sec_ptr]=v1;
    sec_ptr=sec_ptr+1;
    lo=(*buf_ptr++);
    hi=(*buf_ptr++);
    vals=hi;
    vals=vals<<8 | lo;
    verdis=vals;
    verdi=verdis;
    v2=verdi;
    ptr->ibuf[ptr->index][sec_ptr]=v2;
    sec_ptr=sec_ptr+1;
    lo=(*buf_ptr++);
    hi=(*buf_ptr++);
    vals=hi;
    vals=vals<<8 | lo;
    verdis=vals;
    verdi=verdis;
    v3=verdi;
    ptr->ibuf[ptr->index][sec_ptr]=v3;
    sec_ptr=sec_ptr+1;

    break;

    case 3:                            // 3 CHANNELS
    if(UNIT==11)    
    {
      lo=(*buf_ptr++);
      hi=(*buf_ptr++);

      lo=byt[xx];
      xx++;
      hi=byt[xx];
      xx++;

      vals=hi;
      vals=vals<<8 | lo;
      verdis=vals;
      verdi=verdis;
      v1=verdi;
//if(tt==0)
//      printf("  CNV %3d l=%2x h=%2x %d\n",yy,lo,hi,v1);
      ptr->ibuf[ptr->index][yy]=v1;
      yy++;
      sec_ptr=sec_ptr+1;
      lo=(*buf_ptr++);
      hi=(*buf_ptr++);

      lo=byt[xx];
      xx++;
      hi=byt[xx];
      xx++;

      vals=hi;
      vals=vals<<8 | lo;
      verdis=vals;
      verdi=verdis;
      v2=verdi;
if(tt<0)
      printf("%3d l=%2x h=%2x %d",yy,lo,hi,v2);
      ptr->ibuf[ptr->index][yy]=v2;
      yy++;
      sec_ptr=sec_ptr+1;
      lo=(*buf_ptr++);
      hi=(*buf_ptr++);

      lo=byt[xx];
      xx++;
      hi=byt[xx];
      xx++;

      vals=hi;
      vals=vals<<8 | lo;
      verdis=vals;
      verdi=verdis;
      v3=verdi;
if(tt<0)
      printf("%3d l=%2x h=%2x %d",yy,lo,hi,v3);
      ptr->ibuf[ptr->index][yy]=v3;
      yy++;
      sec_ptr=sec_ptr+1;
if(tt<0)
      printf(" %5d %5d %5d\n",v1,v2,v3);
      tt++;
//      if(tt>=2)
//        exit(0);   // dgf
//s=0;
//for(i=0;i<50;i++)
//{
//if(i>20)
//  exit(0); 
//  printf("%2d %2x%2x %2x%2x %2x%2x\n",
//i,byt[s],byt[s+1],byt[s+2],byt[s+3],byt[s+4],byt[s+5]);
//s=s+6;
//}

    }else{
      lo=(*buf_ptr++);
      me=(*buf_ptr++);
      hi=(*buf_ptr++);
      INTEGER_CHAR.cadena[2]=hi;
      INTEGER_CHAR.cadena[1]=me;
      INTEGER_CHAR.cadena[0]=lo;
      v1=INTEGER_CHAR.numerol;
      v1-=0x00800000L;
//    printf("%x %d\n",v1,v1);

      ptr->ibuf[ptr->index][sec_ptr]=v1;
      sec_ptr=sec_ptr+1;
      lo=(*buf_ptr++);
      me=(*buf_ptr++);
      hi=(*buf_ptr++);
      INTEGER_CHAR.cadena[2]=hi;
      INTEGER_CHAR.cadena[1]=me;
      INTEGER_CHAR.cadena[0]=lo;
      v2=INTEGER_CHAR.numerol;
      v2-=0x00800000L;
      ptr->ibuf[ptr->index][sec_ptr]=v2;
      sec_ptr=sec_ptr+1;
      lo=(*buf_ptr++);
      me=(*buf_ptr++);
      hi=(*buf_ptr++);
      INTEGER_CHAR.cadena[2]=hi;
      INTEGER_CHAR.cadena[1]=me;
      INTEGER_CHAR.cadena[0]=lo;
      v3=INTEGER_CHAR.numerol;
      v3-=0x00800000L;
      ptr->ibuf[ptr->index][sec_ptr]=v3;
      sec_ptr=sec_ptr+1;
    }
  }
}
lst_val1=v1;
lst_val2=v2;
lst_val3=v3;
}
/****************************************************************************/
/* Function to convert data buffers from IDS-24                              /
/****************************************************************************/
void Convert2()
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register unsigned short vals;
register short verdis;
register int verdi;
register int v1,v2,v3;


unsigned short smp;
int num_byt;
short sample;
num_byt=ptr->rate * 2 * ptr->nchan;
buf_ptr= &byt[0];                     /* point to first data byte          */
while(buf_ptr < &byt[num_byt])
{
val=0;

    switch(ptr->nchan)
    {
        case 1:
        hi=(*buf_ptr++);
        lo=(*buf_ptr++);
        vals=hi;
        vals=vals<<8 | lo;
        verdis=vals;
        verdi=verdis;
        v1=verdi;
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        lo=(*buf_ptr++);
        hi=(*buf_ptr++);
        vals=hi;
        vals=vals<<8 | lo;
        verdis=vals;
        verdi=verdis;
        v1=verdi;
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        lo=(*buf_ptr++);
        hi=(*buf_ptr++);
        vals=hi;
        vals=vals<<8 | lo;
        verdis=vals;
        verdi=verdis;
        v2=verdi;
        ptr->ibuf[ptr->index][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        lo=(*buf_ptr++);
        hi=(*buf_ptr++);
        vals=hi;
        vals=vals<<8 | lo;
        verdis=vals;
        verdi=verdis;
        v1=verdi;
        ptr->ibuf[ptr->index][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        lo=(*buf_ptr++);
        hi=(*buf_ptr++);
        vals=hi;
        vals=vals<<8 | lo;
        verdis=vals;
        verdi=verdis;
        v2=verdi;
        ptr->ibuf[ptr->index][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        lo=(*buf_ptr++);
        hi=(*buf_ptr++);
        vals=hi;
        vals=vals<<8 | lo;
        verdis=vals;
        verdi=verdis;
        v3=verdi;
        ptr->ibuf[ptr->index][sec_ptr]=v3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
lst_val1=v1;
lst_val2=v2;
lst_val3=v3;
}
/****************************************************************************/
/* Function to make zero buffer when error                                  */
/****************************************************************************/
void ErrBuf()
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned int val;
register int verdi;
register int v1,v2,v3;


unsigned short smp;
short sample;
buf_ptr= &byt[0];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v1=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v2=verdi/256;

val=0;
hi=(*buf_ptr++);
me=(*buf_ptr++);
lo=(*buf_ptr++);
val=hi;
val=val<<8 | me;
val=val<<8 | lo;
val=val<<8;
verdi=val;
v3=verdi/256;
buf_ptr++;
buf_ptr++;

    switch(ptr->nchan)
    {
        case 1:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->ibuf[ptr->index][sec_ptr]=lst_val1;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val2;
        sec_ptr=sec_ptr+1;
        ptr->ibuf[ptr->index][sec_ptr]=lst_val3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
}


Sync1()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0xaa05);
//printf("snc\n");
return(1);
}
Sync2()
{
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0xaa01);
//printf("snc5\n");
return(1);
}
Sync55()
{
register unsigned short sync_value = 0;
int retur;
retur=0;
buf_ptr = bytes;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
        switch(sync_value)
        {
          case 0xaa01:
          retur=1;
          break;
          case 0xaa02:
          retur=2;
          break;
        }

    } while(retur==0);
//printf("snc5\n");
return(retur);
}
Sync5()
{
int pl;
register unsigned short sync_value = 0;
int ct;
ct=0;
buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0xaa80); // time tag found
    buf_ptr=one;
    for(pl=0;pl<10;pl++)           // read time tag
    {
      n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
      byt[ct]=one[0];
      ct++;
    }
    SEMANA_GPS.cadenab[0]=byt[5];
    SEMANA_GPS.cadenab[1]=byt[4];

    SEGUND_GPS.cadena[0]=byt[3];
    SEGUND_GPS.cadena[1]=byt[2];
    SEGUND_GPS.cadena[2]=byt[1];
    SEGUND_GPS.cadena[3]=byt[0];

    OFFSET_GPS.cadena[0]=byt[9];
    OFFSET_GPS.cadena[1]=byt[8];
    OFFSET_GPS.cadena[2]=byt[7];
    OFFSET_GPS.cadena[3]=byt[6];

    semana=(int)SEMANA_GPS.semana;
    if(semana > 1024)
      semana -=1024;
    segundos=(int)(SEGUND_GPS.numero - OFFSET_GPS.numero);
    if(segundos < 0)
    {
      segundos += 86400L;
      semana --;
    }
    sx = div(segundos,86400L);
    numero_dias = sx.quot;
    segundos = sx.rem;
    sx=div(segundos,3600L);
    hora_GPS = sx.quot;
    segundos= sx.rem;
    sx=div(segundos,60L);
    minuto_GPS=sx.quot;
    segundo_GPS=sx.rem;
    numero_dias +=7*semana;
    MSECS=86400.0 * numero_dias;
    SECTIM(MSECS+MSEC99,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
    if(SEMANA_GPS.semana < 1000)
    {
      YR=1000;
      DOY=0;
      MTH=0;
      DAY=0;
      HR=0;
      MIN=0;
      SEC=0.0;
    }

    if(SEMANA_GPS.semana >3000)
    {
      YR=1000;
      DOY=0;
      MTH=0;
      DAY=0;
      HR=0;
      MIN=0;
      SEC=0.0;
    }

    if(YR==2001)
    {
//printf("  %4d %3d %2d/%2d %2d:%2d:%2d\n",YR,DOY,MTH,DAY,hora_GPS,minuto_GPS,segundo_GPS);
      sprintf(snd_tim,"%2d:%2d:%6.3f %2d/%2d/%4d %3d SET\n",
hora_GPS,minuto_GPS,(float)segundo_GPS,DAY,MTH,YR,DOY);
      if(snd_tim[0]==' ')snd_tim[0]='0';
      if(snd_tim[3]==' ')snd_tim[3]='0';
      if(snd_tim[6]==' ')snd_tim[6]='0';
      if(snd_tim[13]==' ')snd_tim[13]='0';
      if(snd_tim[16]==' ')snd_tim[16]='0';
      if(snd_tim[24]==' ')snd_tim[24]='0';
      if(snd_tim[25]==' ')snd_tim[25]='0';

//printf("%s",snd_tim);
    }else{
//  printf("%5d",SEMANA_GPS.semana);
//  for(pl=0;pl<10;pl++)
//    printf(" %2x",byt[pl]);
//  printf("\n");
//printf("  %4d %3d %2d/%2d %2d:%2d:%2d\n",YR,DOY,MTH,DAY,HR,MIN,SEC);
      sprintf(snd_tim,"%2d:%2d:%6.3f %2d/%2d/%4d %3d SET\n",
HR,MIN,SEC,DAY,MTH,YR,DOY);
      if(snd_tim[0]==' ')snd_tim[0]='0';
      if(snd_tim[3]==' ')snd_tim[3]='0';
      if(snd_tim[6]==' ')snd_tim[6]='0';
      if(snd_tim[13]==' ')snd_tim[13]='0';
      if(snd_tim[16]==' ')snd_tim[16]='0';
      if(snd_tim[24]==' ')snd_tim[24]='0';
      if(snd_tim[25]==' ')snd_tim[25]='0';

  }
  return(1);
}

dig_typ()
{
register unsigned short sync_value = 0;
int kk;
buf_ptr = byt;
    for(kk=0;kk<50;kk++)
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
        if(sync_value == 0xaa01)
            return(1);
    }
printf("Do not recognize digitizer\n");
    for(kk=0;kk<50;kk++)
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
        if(sync_value == 0xaa05)
            return(2);
    }
    return(3);
}

Sync3()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0xaa04);
printf("snc 24\n");
return(1);
}
chk_typ()
{
register unsigned short sync_value = 0;
int kk;
int c10;
int c20;
int c30;
int c40;
int c50;
int c60;
int c70;
int c80;
c10=c20=c30=c40=c50=c60=c70=c80=0;
buf_ptr = byt;
for(kk=0;kk<100;kk++)
{
    n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
    switch(n)
    {
        case -1:
        printf("ERROR\n");
        return(-1);
        break;
        case 0:
        printf("Timeout\n");
        return(0);
        break;
        case 1:       
        sync_value = sync_value << 8 | *buf_ptr;
        break;
    }
//printf("%2x\n",sync_value);
    switch(sync_value)
    {
        case 0xaa00:
        return(10);
        break;
        case 0xaa01:
        return(20);
        break;
        case 0xaa02:
        return(30);
        break;
        case 0xaa03:
        return(40);
        break;
        case 0xaa04:
        return(50);
        break;
        case 0xaa05:
        return(60);
        break;
        case 0xaa10:
        return(70);
        break;
        case 0xaa20:
        return(80);
        break;
        case 0xaa80:
printf("Time block\n");
        return(80);
        break;
        default:
        break;        
    }     
}
printf("Do not recognize sync characters at: %d\n",ptr->bd_dig);
return(3);
}
chk_pak(sw,cnt)
int sw;
int *cnt;
{
int count;
register unsigned short sync_value = 0;
unsigned short key;
int kk;
count=0;
switch(sw)
{
    case 0:
    key=0xaa00;
    break;
    case 1:
    key=0xaa01;
    break;
    case 2:
    key=0xaa02;
    break;
    case 3:
    key=0xaa03;
    break;
    case 4:
    key=0xaa04;
    break;
    case 5:
    key=0xaa05;
    break;
    case 10:
    key=0xaa10;
    break;
    case 20:
    key=0xaa20;
    break;
    case 80:
    key=0xaa80;
    break;
}
buf_ptr = byt;
for(kk=0;kk<1000;kk++)
{
    n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
    switch(n)
    {
        case -1:
        printf("ERROR\n");
        return(-1);
        break;
        case 0:
        printf("Timeout\n");
        return(0);
        break;
        case 1:       
        sync_value = sync_value << 8 | *buf_ptr;
        break;
    }
    if(sync_value == key)
        count++;
    if(sync_value == key && key == 0xaa80)
      GPS=1;
}
*cnt=count;
return(3);
}
Sync22()
{
register unsigned short sync_value = 0;
unsigned short llave;
int tp;
llave=0xaa02;
buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            llave=0xaa02;
            for(tp=0;tp<153;tp++)
            {
//              if(tp<10)
//                printf("key: %2x\n",llave);
              if(sync_value == llave)
                printf("%2x\n",sync_value);
              llave++;
            }
            break;
        }
    } while(sync_value != 0xaaaa);
//printf("snc5\n");
return(1);
}
