/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:27/01/96                       */
/* Program:DIG_SER                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from serial pipe and to synchronize     */
/* 1 second buffers from the different ports based on timestamp or sequence */
/* number. The buffers are then converted into standard SEISLOG buffers.    */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <conio.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
int      k;
int      indx;                         /* index current channel             */
float    sec;
int      flag;                         /* indicator                         */
int      i;                            /* counter                           */
int      fd;                           /* return value open share           */
int      scans;                        /* counter number samples per buff   */
int      port;                         /* port counter                      */
int      ii;                           /* index                             */
int      cur_ch;                       /* index current channel             */
int      seconds;                      /* index second counter              */
smod     *ptr;                         /* pointer to shared region          */
char     snd_msg[30];                  /* return message containing time    */
char     syscall[100];
main()
{
double   PREVIOUS;
double   LAST;
double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
unsigned int cbyt1,cbyt2;
unsigned int copy;
unsigned char digit;
unsigned char tstamp[30];
int      year;
int      month;
int      day;
int      hour;
int      minute;
int      first_sec;
int      sekunder;
int      dig_ser_id;                   /* Process id for multiserial board  */
int      buf_pip_id;                   /* Process id for BUF_PIP            */
char     rep_msg[10];                  /* reply message                     */
char     rec_msg[10];                  /* receive message                   */
char     timestring[100];
int      pl;
char    time[15],date[15],doy[15],car;
unsigned int  cop;
LAST=0.0;
setprio(getpid(),12);                  /* set the priority of this process  */
dig_ser_id=qnx_name_attach(0,"dig_ser"); /* attach this process as "dig_ser"*/
if(dig_ser_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_SER attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_SER_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_SER_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* read parameters for multi-serial option                                  */
/*--------------------------------------------------------------------------*/
system("/home/seislog/rd_par_ser");
ptr->noser=16;
for(i=0;i<ptr->noser;i++) 
{
    ptr->dqu[i]=-1;
    ptr->squ[i]=-1;                    /* set set all serial pipes to empty */
}
for(i=0;i<ptr->noser;i++)
    ptr->start_sec[i]=-1;
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
/*--------------------------------------------------------------------------*/
/* zero serial line buffers active lines                                    */
/*--------------------------------------------------------------------------*/
for(seconds=0;seconds<SERSIZ;seconds++)
{
    for(ii=0;ii<ptr->noser;ii++)       /* zero serial pipes                 */
    {
        if(ptr->cha_def[ii][1]==1)
        {
            for(k=0;k < ptr->cha_def[ii][15];k++)
                ptr->mulpip[ii][seconds*ptr->cha_def[ii][15]+k]=0;
        }
    }
}
/*--------------------------------------------------------------------------*/
/* start up one reader process for each active line                         */
/*--------------------------------------------------------------------------*/
switch(ptr->unit)
{
    case 33:
    for(k=0;k<ptr->noser;k++)
    {
        if(ptr->cha_def[k][1]==1)
        {
            printf("DIG_SER: start line:%2d\n",k+1);
            sprintf(syscall,"/home/seislog/mul_ser %2d &",k+1);
            system(syscall);
//            sleep(1);
        }
    }
    break;
    case 34:
    for(k=0;k<ptr->noser;k++)
    {
        if(ptr->cha_def[k][1]==1)
        {
            printf("DIG_SER: start line:%2d\n",k+1);
            sprintf(syscall,"/home/seislog/mul_ser2 %2d &",k+1);
            system(syscall);
            sleep(1);
        }
    }
    break;
}
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
vent:
printf("%3d %3d %3d %3d\n",
ptr->start_sec[0],ptr->start_sec[1],ptr->start_sec[2],ptr->start_sec[3]);
if(ptr->start_sec[0] < 0)
{
    sleep(1);
    goto vent;
}else{
    printf("Start second %3d %3d %3d %3d\n",
    ptr->start_sec[0],ptr->start_sec[1],ptr->start_sec[2],ptr->start_sec[3]);
    first_sec=ptr->start_sec[0];
}
for(;;)                                /* indefinite loop collecting data   */
{
    for(seconds=first_sec;seconds<SERSIZ;seconds++)
    {
        ptr->alive[1]=1;               /* tel I am alive                    */
        Receive(buf_pip_id,&rec_msg,1);/* wait for buf_pipe to be ready     */
tst:
        flag=0;
        if(ptr->cha_def[0][1]==1)
        {
           if(ptr->dqu[0] > 1)
                flag=flag+1;
        }
        if(flag == 0)                  /* if not enough data, sleep for 1   */
        {                              /* second and try again.             */
            sleep(1);
            goto tst;
        }
        indx=0;
        for(scans=0;scans< ptr->nsamp;scans++) /* get same scan from all lin*/
        {
            for(port=0;port<ptr->noser;port++) /* one line at the time      */
            {
                if(ptr->cha_def[port][1]==1)
                {
                    for(cur_ch=0;cur_ch < ptr->cha_def[port][3];cur_ch++)
                    {
                        if(port==0 && cur_ch==0 && scans==0)
                        {
                            for(pl=0;pl<HEADER;pl++) /* copy time tag to ibuf*/
                                ptr->ibuf[ptr->index][pl]=ptr->mulpip[0][(ptr->cha_def[0][15]*seconds)+pl];
                            sprintf(timestring,"%s\n",ptr->ibuf[ptr->index]);
                            sscanf(timestring,"%s %s %s",time,date,doy);
                            sscanf(time,"%2d%c%2d%c%f6.3\n",&HR,&car,&MIN,&car,&SEC);
                            sscanf(date,"%2d%c%2d%c%4d\n",&DAY,&car,&MTH,&car,&YR);
                            sscanf(doy,"%3d\n",&DOY);
//                            printf(" OK: %2d:%2d:%6.3f %2d/%2d/%4d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
                            TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
                            if(MSECS < LAST)
                            {
                                printf("BAD:%2d:%2d:%6.3f %2d/%2d/%4d\n",
                                HR,MIN,SEC,DAY,MTH,YR);
                                printf("Lost data\n");
                                MSECS=PREVIOUS+1.0;
                                SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
                                printf("NEW:%2d:%2d:%6.3f %2d/%2d/%4d\n",
                                HR,MIN,SEC,DAY,MTH,YR);
                                printf("COR: %2d:%2d:%6.3f %2d/%2d/%4d %3d",HR,MIN,SEC,DAY,MTH,YR,DOY);
                                sprintf(timestring,"%2d:%2d:%6.3f %2d/%2d/%4d %3d",
                                HR,MIN,SEC,DAY,MTH,YR,DOY);
                                Mark_Time(timestring);
                                PREVIOUS=MSECS;
                            }else{
                                PREVIOUS=MSECS;
                                LAST=MSECS;
                            }                                
//                            printf("%2d:%2d:%6.3f %2d/%2d/%4d\n",
//                            HR,MIN,SEC,DAY,MTH,YR);
                        }
                        ptr->ibuf[ptr->index][indx+HEADER]=ptr->mulpip[port][cur_ch+scans*ptr->cha_def[port][3]+seconds*ptr->cha_def[port][15]+SHDR];
                        ptr->mulpip[port][cur_ch+scans*ptr->cha_def[port][3]+seconds*ptr->cha_def[port][15]+SHDR]=ptr->dc[cur_ch];
                        indx=indx+1;
                    }
                }
            }
        }
        for(port=0;port<ptr->noser;port++)
        {
            if(ptr->cha_def[port][1]==1)
                ptr->dqu[port]=ptr->dqu[port]-1;
        }
        Reply(buf_pip_id,&rep_msg,1);  /* reply to the Send ready from buf_p*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
    }
    first_sec=0;

}
}

Get_Min(cbyt1,cbyt2,ad1,ad2,ad3,ad4,ad5,ad6)
unsigned int cbyt1;
unsigned int cbyt2;
int* ad1;
int* ad2;
int* ad3;
int* ad4;
int* ad5;
int* ad6;
{
int add;
unsigned short ss;
unsigned int ss1;
unsigned short save;
unsigned char byt;
unsigned int copy;
copy=cbyt1;
cbyt1=cbyt1>>16;
cbyt1=cbyt1 & 0x000000ff;
    save=cbyt1;
    ss=save;
    ss=ss & 0x000f;
    *ad1= ss;
    ss=save;
    ss=ss & 0x00f0;
    ss = ss >> 4;
    *ad1=*ad1+ss*10;
    if(*ad1 < 50)
    {
        *ad1=*ad1+2000;
    }else{
        *ad1=*ad1+1900;
    }
cbyt1=copy;
cbyt1=cbyt1>>8;
cbyt1=cbyt1 & 0x000000ff;
    save=cbyt1;
    ss=save;
    ss=ss & 0x000f;
    *ad2= ss;
    ss=save;
    ss=ss & 0x00f0;
    ss = ss >> 4;
    *ad2=*ad2+ss*10;
cbyt1=copy;
cbyt1=cbyt1 & 0x000000ff;
    save=cbyt1;
    ss=save;
    ss=ss & 0x000f;
    *ad3= ss;
    ss=save;
    ss=ss & 0x00f0;
    ss = ss >> 4;
    *ad3=*ad3+ss*10;
copy=cbyt2;
cbyt2=copy;
cbyt2=cbyt2>>24;
cbyt2=cbyt2 & 0x000000ff;
    save=cbyt2;
    ss=save;
    ss=ss & 0x000f;
    *ad4= ss;
    ss=save;
    ss=ss & 0x00f0;
    ss = ss >> 4;
    *ad4=*ad4+ss*10;
cbyt2=copy;
cbyt2=cbyt2>>16;
cbyt2=cbyt2 & 0x000000ff;
    save=cbyt2;
    ss=save;
    ss=ss & 0x000f;
    *ad5= ss;
    ss=save;
    ss=ss & 0x00f0;
    ss = ss >> 4;
    *ad5=*ad5+ss*10;
cbyt2=copy;
cbyt2=cbyt2>>8;
cbyt2=cbyt2 & 0x000000ff;
    save=cbyt2;
    ss=save;
    ss=ss & 0x000f;
    *ad6= ss;
    ss=save;
    ss=ss & 0x00f0;
    ss = ss >> 4;
    *ad6=*ad6+ss*10;
}
Mark_Time(tstring)
char* tstring;
{
char timestamp[100];
int  k;
char str[100];

register char *tmstr;                  /* pointer to time string in buffer  */
register int temp;
register unsigned int shift;
register unsigned int swap;
register unsigned short msw;
register unsigned short lsw;
register unsigned short sw;
int      i;
char     tmpstr[80];

/*           000000000011111111112222222222333333333344444444445555555*/
/*           012345678901234567890123456789012345678901234567890123456*/
/*           00:00:00.000 00/00/0000 000 sts 00:00:00.000 00/00/00 000  "*/

//ti[55]=0x0d;
//ti[56]=0x0a;
for(k=0;k<60;k++)
    timestamp[k]=' ';
timestamp[55]=0x0d;
timestamp[56]=0x0a;
for(k=0;k<27;k++)
    timestamp[k]=tstring[k];
if(ptr->clktyp==8)
{
    timestamp[28]='S';
    timestamp[29]='E';
    timestamp[30]='T';
}
if(ptr->clktyp==0)
{
    timestamp[28]='C';
    timestamp[29]='P';
    timestamp[30]='U';
}
timestamp[31]='\0';
if(timestamp[0]==' ')
    timestamp[0]='0';
if(timestamp[3]==' ')
    timestamp[3]='0';
if(timestamp[6]==' ')
    timestamp[6]='0';
if(timestamp[9]==' ')
    timestamp[9]='0';
if(timestamp[10]==' ')
    timestamp[10]='0';
if(timestamp[13]==' ')
    timestamp[13]='0';
if(timestamp[16]==' ')
    timestamp[16]='0';
if(timestamp[24]==' ')
    timestamp[24]='0';
if(timestamp[25]==' ')
    timestamp[25]='0';
//for(k=0;k<32;k++)
//    ptr->timcpy[k]=timestamp[k];
k=0;
shift=timestamp[3];
shift=shift << 8;
shift=shift | timestamp[2];
shift=shift << 8;
shift=shift | timestamp[1];
shift=shift << 8;
shift=shift | timestamp[0];
ptr->ibuf[ptr->index][k]=(int)shift;
k++;

shift=timestamp[7];
shift=shift << 8;
shift=shift | timestamp[6];
shift=shift << 8;
shift=shift | timestamp[5];
shift=shift << 8;
shift=shift | timestamp[4];
ptr->ibuf[ptr->index][k]=shift;
k++;

shift=timestamp[11];
shift=shift << 8;
shift=shift | timestamp[10];
shift=shift << 8;
shift=shift | timestamp[9];
shift=shift << 8;
shift=shift | timestamp[8];
ptr->ibuf[ptr->index][k]=shift;
k++;

shift=timestamp[15];
shift=shift << 8;
shift=shift | timestamp[14];
shift=shift << 8;
shift=shift | timestamp[13];
shift=shift << 8;
shift=shift | timestamp[12];
ptr->ibuf[ptr->index][k]=shift;
k++;

shift=timestamp[19];
shift=shift << 8;
shift=shift | timestamp[18];
shift=shift << 8;
shift=shift | timestamp[17];
shift=shift << 8;
shift=shift | timestamp[16];
ptr->ibuf[ptr->index][k]=shift;
k++;

shift=timestamp[23];
shift=shift << 8;
shift=shift | timestamp[22];
shift=shift << 8;
shift=shift | timestamp[21];
shift=shift << 8;
shift=shift | timestamp[20];
ptr->ibuf[ptr->index][k]=shift;
k++;

shift=timestamp[27];
shift=shift << 8;
shift=shift | timestamp[26];
shift=shift << 8;
shift=shift | timestamp[25];
shift=shift << 8;
shift=shift | timestamp[24];
ptr->ibuf[ptr->index][k]=shift;
k++;

shift=timestamp[31];
shift=shift << 8;
shift=shift | timestamp[30];
shift=shift << 8;
shift=shift | timestamp[29];
shift=shift << 8;
shift=shift | timestamp[28];
ptr->ibuf[ptr->index][k]=shift;
k++;

}
