/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:27/01/96                       */
/* Program:DIG_SER                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from serial pipe and to synchronize     */
/* 1 second buffers from the different ports based on timestamp or sequence */
/* number. The buffers are then converted into standard SEISLOG buffers.    */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <conio.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/sched.h>
int      k;
int      indx;                         /* index current channel             */
int      i;                            /* counter                           */
int      fd;                           /* return value open share           */
int      scans;                        /* counter number samples per buff   */
int      port;                         /* port counter                      */
int      ii;                           /* index                             */
int      cur_ch;                       /* index current channel             */
int      seconds;                      /* index second counter              */
int      ser_idx;
smod     *ptr;                         /* pointer to shared region          */
char     snd_msg[30];                  /* return message containing time    */
char     syscall[100];
main()
{
double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
int      cnt;
int      dig_ser_id;                   /* Process id for multiserial board  */
int      buf_pip_id;                   /* Process id for BUF_PIP            */
char     rep_msg[10];                  /* reply message                     */
char     rec_msg[10];                  /* receive message                   */
char     timestring[100];
int      pl;
unsigned int  cop;

YR=99;
MTH=1;
DAY=1;
HR=0;
MIN=0;
SEC=0.0;
DOY=1;
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
printf("MSECS %f\n",MSECS);


setprio(getpid(),12);                  /* set the priority of this process  */
dig_ser_id=qnx_name_attach(0,"dig_ser"); /* attach this process as "dig_ser"*/
if(dig_ser_id == -1)                   /* check if OK ?                     */
{
    printf("DIG_SER attach failed\n"); /* its not, EXIT                     */
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_SER_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_SER_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* read parameters for multi-serial option                                  */
/*--------------------------------------------------------------------------*/
system("/home/seislog/rd_par_ser");
ptr->noser=16;
for(i=0;i<ptr->noser;i++) 
    ptr->dqu[i]=-1;
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
/*--------------------------------------------------------------------------*/
/* zero serial line buffers active lines                                    */
/*--------------------------------------------------------------------------*/
for(seconds=0;seconds<SERSIZ;seconds++)
{
    for(ii=0;ii<ptr->active_lines;ii++) /* zero serial pipes                 */
    {
        if(ptr->cha_def[ii][1]==1)
        {
            for(k=0;k < ptr->cha_def[ii][15];k++)
                ptr->mulpip[ii][seconds*ptr->cha_def[ii][15]+k]=0;
        }
    }
}
for(seconds=0;seconds<SERSIZ;seconds++)
{
    for(ii=0;ii<ptr->active_lines;ii++) /* zero serial pipes                 */
    {
        ser_idx=ii;
        ptr->mulpip[ii][seconds*ptr->cha_def[ii][15]+SHDR-1]=8;
        sprintf(timestring,                /* header time tag into timestring   */
        "%2d:%2d:%6.3f %2d/%2d/%4d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
//printf("%s\n",timestring);
        Mark_Time1(timestring,seconds);     /* time tag buffer in memory pipe    */
    }
}
        
/*--------------------------------------------------------------------------*/
/* start up one reader process for each active line                         */
/*--------------------------------------------------------------------------*/
switch(ptr->unit)
{
    case 33:
    for(k=0;k<ptr->noser;k++)
    {
        if(ptr->cha_def[k][1]==1)
        {
            printf("DIG_SER: start line:%2d\n",k+1);
            sprintf(syscall,"/home/seislog/mul_ser %2d &",k+1);
            system(syscall);
        }
    }
    break;
    case 34:
    for(k=0;k<ptr->noser;k++)
    {
        if(ptr->cha_def[k][1]==1)
        {
            printf("DIG_SER: start line:%2d\n",k+1);
            sprintf(syscall,"/home/seislog/mul_ser2 %2d &",k+1);
            system(syscall);
            sleep(1);
        }
    }
    break;
}
/*--------------------------------------------------------------------------*/
/* look for in coming data on any line.                                     */
/* If no lines receives data, the program will hang in this loop until data */
/* again is received.                                                       */
/*--------------------------------------------------------------------------*/
try:
cnt=0;
for(i=0;i<ptr->active_lines;i++)
{
    if(ptr->dqu[i] != -1)
        cnt++;
}
if(cnt < ptr->active_lines)
{
    sleep(1);
    goto try;
}
printf("Data entering system\n");
sleep(10);
Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
   cnt=0;
   for(seconds=0;seconds<SERSIZ;seconds++)
   {
      for(i=0;i<ptr->active_lines;i++)
         ptr->dqu[i]=0;
      if(ptr->clktyp==8)
          ptr->alive[3]=1;
      for(k=0;k<50;k++)
      {
         cnt=0;
         for(i=0;i<ptr->active_lines;i++)
         {
            if(ptr->dqu[i] > 0)
               goto dmx;
         }
         delay(200);
      }
      printf("Not receiving for 10 seconds. Check for incoming data.\n");
      goto try;
dmx:
//      printf("second %d\n",seconds);
      ptr->alive[1]=1;                 /* tel I am alive                    */
      if(ptr->clktyp==8)
          ptr->alive[3]=1;
      Receive(buf_pip_id,&rec_msg,1);  /* wait for buf_pipe to be ready     */
      indx=0;
      for(scans=0;scans< ptr->nsamp;scans++) /* get same scan from all line */
      {
         for(port=0;port<ptr->noser;port++)      /* one line at the time    */
         {
            if(ptr->cha_def[port][1]==1)
            {
               for(cur_ch=0;cur_ch < ptr->cha_def[port][3];cur_ch++)
               {
                  if(port==0 && cur_ch==0 && scans==0)
                  {
                     for(pl=0;pl<HEADER;pl++) /* copy time tag to ibuf      */
                        ptr->ibuf[ptr->index][pl]=ptr->mulpip[0][(ptr->cha_def[0][15]*seconds)+pl];
                  }
if(indx < 12)
{
   ptr->ibuf[ptr->index][indx+HEADER]=ptr->dc[cur_ch];
}else{
                  ptr->ibuf[ptr->index][indx+HEADER]=ptr->mulpip[port][cur_ch+scans*ptr->cha_def[port][3]+seconds*ptr->cha_def[port][15]+SHDR];
}
                  ptr->mulpip[port][cur_ch+scans*ptr->cha_def[port][3]+seconds*ptr->cha_def[port][15]+SHDR]=ptr->dc[cur_ch];
                  indx=indx+1;
               }
            }
         }
      }
      Reply(buf_pip_id,&rep_msg,1);  /* reply to the Send ready from buf_pip*/
                                       /* At this point both processes are  */
                                       /* ready to proceed.                 */
                                       /* BUF_PIP is waiting to tag data    */
   }
}
}

Mark_Time1(tstring,add)
char* tstring;
int  add;
{
char timestamp[100];
int  k;
char str[100];

register char *tmstr;                  /* pointer to time string in buffer  */
register int temp;
register unsigned int shift;
register unsigned int swap;
register unsigned short msw;
register unsigned short lsw;
register unsigned short sw;
int      i;
int      ls;
char     tmpstr[80];

/*           000000000011111111112222222222333333333344444444445555555*/
/*           012345678901234567890123456789012345678901234567890123456*/
/*           00:00:00.000 00/00/0000 000 sts 00:00:00.000 00/00/00 000  "*/

for(k=0;k<60;k++)
    timestamp[k]=' ';
timestamp[55]=0x0d;
timestamp[56]=0x0a;
for(k=0;k<27;k++)
    timestamp[k]=tstring[k];
if(ptr->clktyp==8)
{
    timestamp[28]='S';
    timestamp[29]='E';
    timestamp[30]='T';
}
if(ptr->clktyp==0)
{
    timestamp[28]='C';
    timestamp[29]='P';
    timestamp[30]='U';
}
timestamp[31]='\0';
if(timestamp[0]==' ')
    timestamp[0]='0';
if(timestamp[3]==' ')
    timestamp[3]='0';
if(timestamp[6]==' ')
    timestamp[6]='0';
if(timestamp[9]==' ')
    timestamp[9]='0';
if(timestamp[10]==' ')
    timestamp[10]='0';
if(timestamp[13]==' ')
    timestamp[13]='0';
if(timestamp[16]==' ')
    timestamp[16]='0';
if(timestamp[24]==' ')
    timestamp[24]='0';
if(timestamp[25]==' ')
    timestamp[25]='0';
for(k=0;k<32;k++)
    ptr->timcpy[k]=timestamp[k];
ls=0;
for(k=0;k<64;k=k+4)
{
    shift=timestamp[k+3];
    shift = shift << 8;
    shift = shift | timestamp[k+2];
    shift = shift << 8;
    shift = shift | timestamp[k+1];
    shift = shift << 8;
    shift = shift | timestamp[k];
    ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+ls]=shift;
    ls++;
}
}
