/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:13/08/95                       */
/* Program:DIG_SYN                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to generate 1-second buffers of synthetic data. Signal to        */
/* buf_pipe that data buffer is ready.                                      */
/* The unit number for this "device" is 96. Frequencies can easily be       */
/* changed.                                                                 */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <math.h>

int      i;
int      cnt;
float    f;
int      indx;
int      id;                           /* Buffer indicator                  */
int      byt[BUFFER];
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
float    freq[MAXCHA];                 /* frequency table                   */
float    pi;
float    amp;
int      teller;
float    addon;
float    konst;
main()
{
float    inc_freq;                     /* frequency increment               */
float    frequency;                    /* next frequency                    */
float    start_freq;                   /* start frequency                   */
int      dig_syn_id;                   /* id after attach                   */
int      buf_pip_id;
char     rep_msg[10];
char     snd_msg[30];
char     rec_msg[10];
unsigned int cop;


konst      = 100.0;

addon      = 0.05;
start_freq = 5.0;
inc_freq   = 1.;
pi         = 3.1415*2.;
cnt        = 0;
amp        = 1.0;
teller     = 0;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_SYN_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_SYN_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

f          = ptr->srate;
ptr->factor=0;
/*--------------------------------------------------------------------------*/
/* initialize frequency table                                               */
/*--------------------------------------------------------------------------*/
frequency=start_freq;
for(i=0;i<ptr->nchan;i++)
{
    freq[i]=frequency;
    frequency=frequency + inc_freq;
}
/*--------------------------------------------------------------------------*/
/* attach this program                                                      */
/*--------------------------------------------------------------------------*/
dig_syn_id=qnx_name_attach(0,"dig_syn");
if(dig_syn_id == -1)
{
    printf("DIG_SYN attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);

ptr -> index=0;                        /* Initial buffer pointer            */

Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
    ptr->alive[1]=1;                   /* tell I am alive                   */
    Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pip to be ready      */
    Reply(buf_pip_id,&rep_msg,1);      /* reply to the send ready from buf_p*/
    Sync();                            /* Look for sync characters in       */
                                       /* recived data                      */
    Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time     */
    Convert();

//        ptr->ibuf[ptr->index][0+HEADER]=300;

/*--------------------------------------------------------------------------*/
/* Send signal that one buffer is ready                                     */
/*--------------------------------------------------------------------------*/
    snd_msg[29]='0';                   /* indicate that data is ok          */
    Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
    sleep(1);
}
}
/****************************************************************************/
/* Function to convert data buffers from the SYNT_DAT generator             */
/****************************************************************************/
void Convert()
{
int lg;
int kp;
indx=0;
for(lg=0;lg<ptr->bufsiz/ptr->nchan;lg++)
{
    for(kp=0;kp<ptr->nchan;kp++)
    {
        ptr->ibuf[ptr->index][indx+HEADER]=byt[lg*ptr->nchan+kp];
/*printf("%3d byt[%3d]=%5d\n",indx,lg*ptr->nchan+kp,byt[lg*ptr->nchan+kp]);*/
/*getch();*/
        indx=indx+1;
    }
}
}
void Sync()
{
int   lg;
int   kp;
float s;
for(lg=0;lg<ptr->bufsiz/ptr->nchan;lg++)
{
    for(kp=0;kp<ptr->nchan;kp++)
    {
        s=sin(pi*f*(float)cnt*freq[kp])*konst*amp;
        byt[lg*ptr->nchan+kp]=(int)s+100;
    }
    if(ptr->factor==1)
    {
        amp=amp+addon;
        if(amp <= 1.0)
        {
           ptr->factor=0;
           teller=0;
           addon=0.05;
        }
        teller=teller + 1;
        if(teller == 200)
            addon=-0.005;
    }
    cnt=cnt+1;
    if(cnt >=500000)
         cnt=0;
}
}
