/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:01/03/98                       */
/* Program:DIG_WAV                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to generate 1-second buffers of recorded data. Signal to         */
/* buf_pipe that data buffer is ready.                                      */
/* The unit number for this "device" is 97.                                 */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <math.h>

int      i;
int      cnt;
float    f;
int      j;
int      indx;
int      id;                           /* Buffer indicator                  */
int      byt[BUFFER];
FILE     *inp;
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
main()
{
int      dig_wav_id;                   /* id after attach                   */
int      buf_pip_id;
char     rep_msg[10];
char     snd_msg[30];
char     rec_msg[10];
unsigned int cop;

/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DIG_WAV_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DIG_WAV_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* attach this program                                                      */
/*--------------------------------------------------------------------------*/
dig_wav_id=qnx_name_attach(0,"dig_wav");
if(dig_wav_id == -1)
{
    printf("DIG_WAV attach failed\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* locate process BUF_PIP and get its process id                            */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);

ptr -> index=0;                        /* Initial buffer pointer            */

Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* send to buf_pipe to give id      */
for(;;)                                /* indefinite loop collecting data   */
{
    if((int)(inp=fopen("/home/seislog/wave1","r+"))==ERROR)
    {
        printf("cant open WAVE\n");
        exit(0);
    }
    for(j=0;j<1760;j++)
    {
        fseek(inp,j*664+1328,0);
        fread(byt,664,1,inp);
        ptr->alive[1]=1;                   /* tell I am alive                   */
        Receive(buf_pip_id,&rec_msg,1);    /* wait for buf_pip to be ready      */
        Reply(buf_pip_id,&rep_msg,1);      /* reply to the send ready from buf_p*/
        Send(buf_pip_id,&snd_msg,&rec_msg,1,1); /* let buf_pip get the time     */
        Convert();
/*--------------------------------------------------------------------------*/
/* Send signal that one buffer is ready                                     */
/*--------------------------------------------------------------------------*/
        snd_msg[29]='0';                   /* indicate that data is ok          */
        Send(buf_pip_id,&snd_msg,&rec_msg,30,1);
        sleep(1);
    }
    fclose(inp);
}
}
/****************************************************************************/
/* Function to convert data buffers from the SYNT_DAT generator             */
/****************************************************************************/
void Convert()
{
int lg;
int kp;
indx=0;
for(lg=0;lg<ptr->bufsiz;lg++)
{
        ptr->ibuf[ptr->index][indx+HEADER]=byt[lg+HEADER];
/*printf("%3d byt[%3d]=%5d\n",indx,lg*ptr->nchan+kp,byt[lg*ptr->nchan+kp]);*/
/*getch();*/
        indx=indx+1;
}
}
