/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:08/10/95                       */
/* Program:DPL                                                              */
/*                                                                          */
/* Last update:                                                             */
/* 16/09/97 include help file                                               */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program BUF_PIP sends out a signal each time a data buffer is filled up  */
/* with data. DPL waits for this particular signal and plot data from shared*/
/* memory directly.                                                         */
/* The program can plot one specified channel at the time.                  */
/* The input to the program:                                                */
/* Channel number: The number of the channel you want to plot               */
/* Skip factor   : 0=>all samples, 1=>every second, 2=>every third, etc.    */
/* The signal is plotted in near realtime mode.                             */
/*                                                                          */
/****************************************************************************/  
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/dev.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <graph.h>
#include <conio.h>
#define  SIZ 256
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                             Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program BUF_PIP sends out a signal each time a data buffer is 
             filled up with data. DPL waits for this particular signal and 
             plot data from shared memory directly.
             The program can plot one specified channel at the time.
             The input to the program:
             Channel number: The number of the channel you want to plot 
             Skip factor   : 0=>all samples, 1=>every second, 2=>every third..
             The signal is plotted in near realtime mode. 

Usage:       dpl (program ask for input), will only run when SEISLOG active
#endif

int      kar;
int      n;                            /* font numbers                      */
char     buf[10];
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

void main()
{   
FILE     *fp;                          /* file pointer stdin                */
char     lbuf[SIZ];                    /* input buffer for input_line       */
char*    fontpath=getenv("WINFONTS");  /* define windows fonts              */
pid_t    proxy;
unsigned msg;
int      fnt_no;                       /* font number to use                */
int      id;                           /* buffer switch                     */
float    value;                        /* float of value                    */
int      intval;                       /* integer of value                  */
int      kk;                           /* counter                           */
int      lp;                           /* counter                           */
int      chan;                         /* Channel Number                    */
int      skip;                         /* Sample skip factor                */
short    npage;                        /* New page indicator                */
int      ix;                           /* x-value                           */
int      iy;                           /* y-value                           */
short    jump;                         /* counter to reduce number of       */
                                       /* samples to plot                   */
int      i;                            /* Counter                           */
int      byt;                          /* Return value from test keyboard   */
int      offset;                       /* Offset channel                    */
float    factor;                       /* Gain factor                       */
float    gain;
short    const1;                       /* constant ptr->nchan*2             */
short    const2;                       /* constant HEADER*4                 */
int      con_ser;                      /* switch to tell console or serial  */
int      dpl_id;                       /* process id this process           */
int      buf_pip_id;                   /* process id signalling process     */
float    weight;
float    increm=1.;
unsigned int cop;

int fixed_gain;    /*** DJS 22/08/00 ***/

weight=6.;
fp=stdin;
fnt_no=61;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DPL_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DPL_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* read in COLOR.MAP file                                                   */
/*--------------------------------------------------------------------------*/
system("/home/seislog/rd_color");
/*--------------------------------------------------------------------------*/
/* decide if I am started from console or remote terminal                   */
/*--------------------------------------------------------------------------*/
con_ser=line_typ();
/*--------------------------------------------------------------------------*/
/* locate BUF_PIP                                                           */
/*--------------------------------------------------------------------------*/
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
if(buf_pip_id == -1)
{
    printf("DPL: Can't find BUF_PIP. SEISLOG not active ?\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* compute constants to save time in loop                                   */
/*--------------------------------------------------------------------------*/
const1=ptr->nchan*2;
const2=HEADER*4;
/*--------------------------------------------------------------------------*/
/* Input the channel number you want to plot                                */
/*--------------------------------------------------------------------------*/
printf("Enter Channel Number:");
scanf("%d",&chan);

/*--------------------------------------------------------------------------*/
/* Input the sample skip factor                                             */
/*--------------------------------------------------------------------------*/
printf("Enter Sample-Skip (default=0,press RETURN)  :");
input_line(fp,lbuf,SIZ);
if(lbuf[0]==NULL)                      /* default ?                         */   
{
    skip=0;                            /* set skip to default value         */
    goto nx1;
}
sscanf(lbuf,"%d",&skip);               /* read input                        */
/*--------------------------------------------------------------------------*/
/* Input gain factor                                                        */
/*--------------------------------------------------------------------------*/
nx1:

/**** new code 22/08/00 DJS ***/
printf ("Do you want autoscale (a) or fixed gain (f) - enter 'a' or 'f':");
input_line(fp,lbuf,SIZ);
if (lbuf[0] == 'f' || lbuf[0] == 'F')                      /* default ?                         */   
    fixed_gain = 1;           
else
	fixed_gain = 0;
if (fixed_gain)
    printf("Enter absolute scale in counts (floating point):");
else

    printf("Enter plotting gain (floating point),(default=auto,press RETURN:");
input_line(fp,lbuf,SIZ);
if(lbuf[0]==NULL)                      /* default ?                         */
{
    gain=1.0;                         /* set to default, use automatic     */
    goto nx2;
}
sscanf(lbuf,"%f",&gain);               /* read input                        */

/**** new code 22/08/00 DJS ***/
if (fixed_gain) gain *= -1.0;

nx2:

/*--------------------------------------------------------------------------*/
/* attach proxy for the terminal type in use                                */
/*--------------------------------------------------------------------------*/
if(con_ser == 1)                       /* SERIAL type ?                     */
{
    proxy=qnx_proxy_attach(0,0,0,-1);
    ptr->dpls_proxy=proxy;
}
if(con_ser == 2)                       /* CONSOLE type ?                    */
{
n=_registerfonts(fontpath);
    proxy=qnx_proxy_attach(0,0,0,-1);
    ptr->dplc_proxy=proxy;
}
/*--------------------------------------------------------------------------*/
/* attach this process under the name DPL                                   */
/*--------------------------------------------------------------------------*/
if(con_ser == 1)
{
    dpl_id=qnx_name_attach(0,"dpls");
    if(dpl_id==-1)
    {
        printf("DPL SERIAL attach failed. DPL already active ?\n");
        exit(0);
    }
}
if(con_ser == 2)
{
    dpl_id=qnx_name_attach(0,"dplc");
    if(dpl_id==-1)
    {
        printf("DPL CONSOLE attach failed. DPL already active ?\n");
        exit(0);
    }
}
/*--------------------------------------------------------------------------*/
/* Set indicator for new page                                               */
/*--------------------------------------------------------------------------*/
npage=1;

top1:
/*--------------------------------------------------------------------------*/
/* get last average offset value from shared memory region                  */
/*--------------------------------------------------------------------------*/
offset=(int)ptr->dc[chan-1];           /* use offset computed in detec_01   */
    factor=ptr->to[chan-1];            /* factor based on avg. totals       */
/*--------------------------------------------------------------------------*/
/* Enter graphics mode                                                      */
/*--------------------------------------------------------------------------*/
graphic_mode(con_ser);
for (;;)
{
    buf_pip_id=Receive(0,&msg,sizeof(msg)); /* receive trigger from proxy   */
    if(kbhit())                        /* any key hit, exit                 */
    {
        kar=getch();
        if(kar != 0x68)                /* check for HALT                    */
        {
            clr_scr(con_ser);          /* clear screen                      */
            exit(0);
        }else{                         /* its a HALT                        */
            kar=getch();
            if(kar == 0x68)            /* check for resume or stop          */
            {
                goto cont;      /* clear screen and continue         */
            }else{
                clr_scr(con_ser);
                exit(0);
            }
        }            
    }
cont:
    id=ptr->buf_rdy;
    if(npage == 1)                     /* test for new page (screen)        */
    {
        plt_hdr_box(ptr,con_ser);
        plt_plt_box(ptr,con_ser,fnt_no);

        fill_hdr_box(ptr,offset,chan,increm,skip,con_ser,fnt_no);
        NewPage(ptr,offset,chan,factor,skip,&ix,&iy,con_ser,fnt_no,weight,gain);
    }
    jump=ptr->nchan*(skip+1);          /* reduce number of samples to plot  */
                                       /* according to skip factor          */
    npage=plot_buffer(con_ser,ptr,jump,factor,offset,chan,&ix,&iy,id,gain,weight);
    if(npage==1)
        goto top1;
}   /* End of indefinite loop */
}   /* End of main */




