/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:08/03/95                       */
/* Program:                                                                 */
/*                                                                          */
/* Last update:09/07/97                                                     */
/* 20/06/00  8.25 err_log file extended to 10000 messages to permit links   */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program is used to allocate the number                              */
/* of ring-buffer files specified in the                                    */
/* parameterfile:                              /home/buffer/R0000           */
/*                                             /home/buffer/R0001 etc.....  */
/* The program allocates the detection number                               */
/* file, and initialize the DPX number to 0    /home/seislog/pr/detnr       */
/* The program allocates the sequence number                                */
/* file, and initialize the CONTNR to 0        /home/seislog/pr/contnr      */
/* The program allocates the detection file    /home/buffer/event           */
/* The program allocates the ringbuffer log                                 */
/* file and initialize it to empty status      /home/buffer/idx_log         */
/*                                                                          */
/****************************************************************************/  

#include "shar_mem.h"                  /* shared data- and parameter module */
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "filedef.h"
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include "timexrou.h"
char     idx_log[500];
char     text[200];
char     rngnam[100];
FILE     *in;
FILE     *idx;
/****************************************************************************/
/* RNG_NAM: to make up a dummy ring buffer name for each file               */
/* this procedure makes use of event_name in shar_routines                  */
/****************************************************************************/
void rng_nam(filenum,fnam,ptr)
int      filenum;                      /* number of the buffer file         */
char*    fnam;                         /* returned containing the filename  */
smod*    ptr;
{
struct   date ftim;

/* assign dummy values to the time */
ftim.year = 0;
ftim.month = 0;
ftim.day = 0;
ftim.hour = 0;
ftim.min = 0;
ftim.secs = 0;
Event_Name1(filenum,&ftim,fnam,CONTIN,CONTIN,ptr);
}
/****************************************************************************/
/*       DSK_ALL - generate the continuous record space                     */
/****************************************************************************/
main()
{
time_t   time_of_day;
auto     char buf[26];
char     evnam[DIRNAMLEN+EVNAMLEN+10];
char     fnam[EVNAMLEN+10];
int      PathNo;                       /* Pathnumber to file                */
int      PathNo2;                      /* Pathnumber to lo file             */
int      i;                            /* Counter                           */
int      k;                            /* Counter                           */
int      ret;                          /* Return value allocating the file  */
int      size_file;                    /* filesize each rinbuffer           */
int      rnghdr;                       /* size of header + displ            */
char     head[SIZ_EV_HDR+1];
char     *logptr;
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"DSK_ALL_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"DSK_ALL_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
system("'\'rm -f /home/buffer/R*");     /* remove all ringbuffers           */
system("'\'rm -f /home/buffer/idx_log");/* remove status file ringbuffers   */
system("'\'rm -f /home/buffer/err_log");/* remove error logfile             */

PathNo2     = open("/home/buffer/err_log",O_RDWR | O_CREAT,
             S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
if(PathNo2 != -1)
{
    if( chsize(PathNo2,80* 10000) != 0)
    {
        printf("Error extending /home/buffer/err_log file\n");
    }
    close(PathNo2);
}
printf("/home/buffer/err_log allocated and extended\n");

rnghdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
size_file=rnghdr+ptr->reclength*60*ptr->nmin_per_ring[0];

printf("\nNumber of files to create:%d\n",ptr->nringbuf[0]);
PathNo2     = open("/home/buffer/idx_log",O_RDWR | O_CREAT,
             S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
if(PathNo2 != -1)
{
    if( chsize(PathNo2,320* ptr->nringbuf[0]) != 0)
    {
        printf("Error extending /home/buffer/idx_log file\n");
    }
    close(PathNo2);
}
printf("/home/buffer/idx_log allocated and extended\n");
idx=fopen("/home/buffer/idx_log","r+");
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
for(i=0;i<500;i++)
    idx_log[i]=' ';
for(i=0;i<ptr->nringbuf[0];i++)
{
    for(k=0;k<30;k++)
    {
        evnam[k]='\0';
        rngnam[k]='\0';
    }
rng_nam(i,fnam,ptr);
strcpy(evnam,(char*)ptr->Bufdir);
for(k=0;k<13;k++)
{
    idx_log[k]=evnam[k];
    rngnam[k]=evnam[k];
}
for(k=0;k<34;k++)
    idx_log[k+13]=fnam[k];

for(k=0;k<5;k++)
    rngnam[k+13]=fnam[k];
printf("%s %d %d\n",rngnam,rnghdr,size_file);
sprintf(text,"START:");
for(k=0;k<6;k++)
    idx_log[k+48]=text[k];
time_of_day=time(NULL);
sprintf(text,"%s",_ctime(&time_of_day,buf));
for(k=0;k<26;k++)
    idx_log[k+55]=text[k];
sprintf(text,"%5d",i);
for(k=0;k<5;k++)
    idx_log[k+80]=text[k];
sprintf(text,"0 INI");      /* EMPTY FULL NOT-FULL */
for(k=0;k<5;k++)
    idx_log[k+87]=text[k];
idx_log[96]='0';                       /* zero up time initially            */
idx_log[97]='0';
idx_log[98]='0';
idx_log[100]='0';
idx_log[101]='0';
idx_log[103]='0';
idx_log[104]='0';

idx_log[109]='0';                      /* zero events initially             */
idx_log[110]='0';
idx_log[111]='0';
idx_log[112]='0';
idx_log[113]='0';
sprintf(text,"STOPP:");
for(k=0;k<6;k++)
    idx_log[k+128]=text[k];
sprintf(text,"%s",_ctime(&time_of_day,buf));
for(k=0;k<26;k++)
    idx_log[k+135]=text[k];
idx_log[18]=' ';
idx_log[66]='X';                   /* zero time on initializing         */
idx_log[67]='X';
idx_log[69]='X';
idx_log[70]='X';
idx_log[72]='X';
idx_log[73]='X';
idx_log[146]='X';                   /* zero time on initializing         */
idx_log[147]='X';
idx_log[149]='X';
idx_log[150]='X';
idx_log[152]='X';
idx_log[153]='X';
for(k=0;k<80;k++)
    idx_log[k+160]=' ';
for(k=0;k<80;k++)
    idx_log[k+240]=' ';
idx_log[239]='\n';
idx_log[319]='\n';
fwrite(idx_log,320,1,idx);
/*--------------------------------------------------------------------------*/
/* make files                                                               */
/*--------------------------------------------------------------------------*/
PathNo     = open(rngnam,O_RDWR | O_CREAT,
             S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
if(PathNo != -1)
{
    if( chsize(PathNo,size_file) != 0)
    {
        printf("Error extending file %s\n",rngnam);
    }
    close(PathNo);
    in=fopen(rngnam,"r+");
    head[128]='5';
    head[129]='5';
    fseek(in,0,0);
    fwrite(head,130,1,in);
    fclose(in);
}
}
fclose(idx);
printf("/home/buffer/idx_log initialized\n");
printf("%5d ringbuffer files allocated.\n",i);

PathNo     = open("/home/buffer/event",O_RDWR | O_CREAT,
             S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
if(PathNo != -1)
{
    if( chsize(PathNo,EVTSIZ*EVTREC) != 0)
    {
        printf("Error extending file /home/buffer/event\n");
    }
    close(PathNo);
}
printf("/home/buffer/event allocated and extended\n");
printf("*** DSK_ALL *** All files allocated, extended and initialized\n");
sprintf(ptr->err_msg,"DSK_ALL: REINITIALIZE ALL FILES");
errlog(ptr);
}
/*--------------------------------------------Start: Event_name-------------*/
/* Make file name from time-string                                          */
/*--------------------------------------------------------------------------*/
Event_Name1(seq_no,rec_time, fnam,point,par,ptr)
  int seq_no;
  struct date* rec_time;
  char* fnam;
  int point;
  int par;
  smod* ptr;
{   
int i;
/* The format of this event file is as follows:
   YYMM_DD_HHMM_SST.CCC_NN_P
   year,month,day,hour,mins,secs,T=temporary,station code, num of channels 
   P parameter set */
   
   char cyear[4];   /* help variable in cutting the century off year        */
   char* namptr= fnam;    /* index in fnam string                           */
   
sprintf(namptr,"R");
namptr+=1;
sprintf(namptr,"%4d",seq_no);
namptr+=4; 
sprintf(namptr,"_");
namptr+=1;
sprintf(cyear, "%4d", rec_time->year);

//sprintf(namptr, "%2s", cyear+2);
sprintf(namptr, "%4s_", cyear);
namptr+=5;
sprintf(namptr, "%2d_", rec_time->month);
namptr+=3;
sprintf(namptr, "%2d_", rec_time->day);
namptr+=3;
sprintf(namptr, "%2d%2d_", rec_time->hour, rec_time->min);
namptr+=5;
sprintf(namptr, "%2dT.", (int) rec_time->secs);
namptr+=4;
strncpy(namptr, ptr->name, 3);                   /* station code            */
namptr+=3;
*namptr++ = '_';

sprintf(namptr, "%2d", ptr->nchstr[par]);   

namptr+=2;
switch(point)
{
    case MANUAL:
    sprintf(namptr, "_M");
    break;
    case EXTRACT:
    sprintf(namptr, "_X");
    break;
    case CONTIN:
    sprintf(namptr, "_C");
    break;
    default:
    sprintf(namptr, "_%1d", point);
    break;
}
//printf("%s\n",fnam);
/********************JUST FOR TESTING**********************/
/*sprintf(fnam,"9504_15_1230_30T.WNN_06_C");*/
/**********************************************************/
/* fill fnam with zeros in all blank spaces */
zero_fill1(fnam);

}
/*------------------------------------------End: Event_Name-----------------*/

/*--------------------------------Start: zero_fill-----------------------*/

zero_fill1(string)
  char* string;
  
{
    int i;
    for(i=0; i< strlen(string); i++) 
     if(string[i] == ' ') string[i] = '0';
     
}

/*-----------------------------------End: zero_fill----------------------*/

