//     epicenter location, very primitive     //
//
//     updates:
//     oct 98 by jh:   add date and time, correct to write p and s phases
//     apr 200     : tested with latest version of plt and map
//
//     The program uses a simple halfspace with a constant velocity.
//     The program iterates for location and origin time but not for
//     depth. The depth can optinallt be found by letting the program
//     test a range of depths and selecting the one with the lowest rms.
//
//     How to run program:
//
//     The program uses an input file station. Below is an example
//
//     7.0  3.5
//     5.0  30
//     2.0 0.003 -2.0
//     ODD1 59.912 6.628
//     BLS5 59.423 6.456
//     ASK  60.483 5.195
//     EGD  60.271 5.226
//     OSG  60.479 2.876
//
//     The first line gives the P and S-velocity repspectively. The P
//     velocity will in most cases be between 6 and 7 km/sec 
//     considering that this is an average velocity. The S-velocity
//     should then be the P-velocity divied by 1.73. However, if Lg
//     is a dominating secondary phase, the S- velocity should be set
//     to 3.5 irrespective of the P-velocity.
//
//     The second line gives the initial depth and number of depths
//     to test. If only one depth is tested that is also the depth
//     being used, this means a fixed depth. This might often be
//     advisable. However if a good station coverage is available,
//     test for a range of depths. If e.g. 20 depths are tested for,
//     the program will start with the first depht and try 19 other
//     depths increasing the depth 10 km each time and the last depth
//     tested will be 195 km. The depth giving the smallest rms will be
//     used.
//
//     The third line gives the coda magnitude parameters. Magnitude
//     is calcualted as
//
//     mag = c1*log10(coda) + c2*distance + c3
//
//     coda is the coda length (secs) from the ORIGIN time and not P
//     as is the most usual. This means that compared to other coda
//     magnitude scales, the distance correction term c2 should be
//     reduced a bit.
//
//     The remaining lines give the station code and latitude and 
//     longitude, positive N and E. There should be no blank lines
//     in the file. The first station is the list is used for
//     coordinate system reference and should be near the seismic
//     network center. It can be a dummy station.
//
//     Input of phases comes from a file redirected to standrd input, the
//     following is an example:
//
//     ODD1 40.9 1
//     ODD1 68.9 2
//     BLS5 38.9 1
//     BLS5 69.1 2
//     ASK  25.5 1
//     ASK  43.1 2
//     EGD  25.5 1
//     EGD  42.0 2
//     OSG  11.0 1
//     OSG  21.0 2
//     EGD  160 3
//
//     Each line contains the station code, the phase arrival time and
//     an index giving the phase type. 1 is P, 2 is S and 3 is coda.
//     Program PLT makes such a file when picking phases. If the name is
//     phase, the program is run by typing epi<phase.
//
//     Output on the screen should be self explanatory. In additon to the screen 
//     output, the epicenter and magnitude is written out to a file called
//     epicenter. This file is used by the MAP progran to plot the epicenter.
//
#include <stdio.h>
#include <math.h>
#include <string.h>
FILE *epi,*sta;
main()
{
int n;
int i;
float t[80];          /*  arrival times */
int   phase[80];      /* phase type */
char  phs[4]=" PS "; /* phase type */
int   nstation;       /* number of stations in staion file*/
int   stat_nr[80];    /* station nr */
char  stat[100][5];   /* station names from station file */
char  station[5];     /* one station name with phase */
int  year,month,day,hour,min;       /* date and time*/
float slonmin;
float slonmax;
float slatmin;
float slatmax;
float lat,lon;        /* epicenter */
float slat[100],slon[100];      /* station lat and lon */
float c1,c2,c3;       /* coda scale constants, mc=c1*log10(coda)+c2*distance+c3 */
float xt,yt;
float mag;            /* magnitude */
float rm;             /* rms of residuals */
float tmin;           /* minimum arrival time */
float xmin,ymin,hmin; /* values for best depth solution */
float rmsmin;         /* lowest rms in depth test */
int   ndepth;         /* number of depths to test for */
int   idepth;         /* counter for depht test */
float tcnr[80];       /* temporary save */
float x[80],y[80],tc[80],b[4] [5];
float xx[80],yy[80];  /* station coordinates */
float x0,y0,h0;       /* hypocneter in x,y and z */
float t0;             /* origin time */
float velocity[4];    /* velocity */
float vel[80];        /* velocity each reading */
float res[80];        /* residual */
int ncoda;            /* number of coda readings */
float tr[80],dc[80],d0,dd1,dd2,dd3;
float c[4],a[80] [5], d[80],dd[80];
float xc,yc,r;
int k,l,kk,nmin,kkk;
//
//  
     rmsmin=99999.9;
//
//
//   open and read parameters and station list, read to end of file
//
     sta=fopen("/home/seislog/pr/station","r");
//
//   read velocites
//
     fscanf(sta,"%f %f",&velocity[1],&velocity[2]);
     printf("P-velocity:    %5.1f S-velocity:  %5.1f  \n",velocity[1],velocity[2]);     
//
//  read fixed depth or starting depth and number of depths to test
//  if number of test depths is 1, only the first start depth is used
//
     fscanf(sta,"%f  %d",&h0,&ndepth);
     printf("Depth used %5.1f    Number of dephts to test for %d\n",h0,ndepth);
//
//  read coda scale parameters
//
     fscanf(sta,"%f %f %f",&c1,&c2,&c3);
     printf("Coda paramters %6.2f %6.4f %6.2f\n",c1,c2,c3);
//
//  read min/max long and lat
//
    fscanf(sta,"%f %f %f %f",&slonmin,&slonmax,&slatmin,&slatmax);
//
//  read stations until end of file
//
     i=1;
read_station:
     while(k=fscanf(sta,"%s %f %f",stat[i],&slat[i],&slon[i])!=EOF)
     {
//     printf("%s %f %f\n",stat[i],xx[i],yy[i]);
       i=i+1;
       goto read_station;
     }
     fclose(sta);
     nstation=i-1;
     printf("Number of stations in station file: %d \n",nstation);
//
//   convert from lat lon to x-y, primitive
//
     for(i=1;i<(nstation+1);i++)
       {
        xx[i]=(slon[i]-slon[1])*111.2*cos(slat[1]/57.3);
        yy[i]=(slat[i]-slat[1])*111.2;
       }
      velocity[3]=1.0;                  /* dummy, just to use with coda */
//
//   get readings from redirected input
//
//   first get date and time
//
     scanf("%d %d %d %d %d",&year,&month,&day,&hour,&min);
     
    {
    i=1;
read_more:
     while (k=scanf("%s %f %d",station,&t[i],&phase[i])!=EOF)
     {
//
//  find station number corresponding to station
//
        station[4]='\0';   /* probably not needed */
        stat_nr[i]=-1;     /* no station so far */
        for(k=1;k<nstation+1;k++)  /* find station */
           if(strcmp(stat[k],station)==0) stat_nr[i]=k;
        if(stat_nr[i]==-1)   /* check if station in list */
          {
           printf("Station  %s not found in station list\n",station);
           i=i-1;       /* do not use */
          }
        i=i+1;
        goto read_more;
      }
      n=i-1;
    }
//     for(i=1;i<(n+1);i++)
//        printf(" %d %f %d\n",stat_nr[i],t[i],phase[i]);
//
//   find how many coda readings and put at end of array
//
ncoda=0;
k=0;
for (i=1;i<(n+1);i++)
   {
   if(phase[i]==3)   /* is it a coda */
      {
       ncoda=ncoda+1;
       tc[ncoda]=t[i];
       tcnr[ncoda]=stat_nr[i];
      }
   }
//
//  put at end
//
for  (i=1;i<(n+1);i++)
   {
    if(phase[i]!=3)
    {
        k=k+1;
        phase[k]=phase[i];
        t[k]=t[i];
        stat_nr[k]=stat_nr[i];
    }
   }
n=k;
//printf("ncoda %d\n",ncoda);
for (i=1;i<(ncoda+1);i++)
   { k=k+1;
     phase[k]=3;
     stat_nr[k]=tcnr[i];
     t[k]=tc[i];
    }
//
//  check if enough stations
//
if(n < 3)
     {
     printf(" At least 3 stations needed\n");
     goto end_epi;
     }
//
//   find stations and transfer data
//

for(i=1;i<(n+ncoda+1);i++)
   { 
     x[i]=xx[stat_nr[i]];
     y[i]=yy[stat_nr[i]];
     vel[i]=velocity[phase[i]];
//printf("VEL= %f\n",vel[i]);
   }
exit(0);
//
//   find station with earliest time
//

tmin=9999;
for (i=1;i<(n+1);i++)
   { if(t[i] < tmin)
       {
        nmin=i;
        tmin=t[i];
       }
   }
//
//   set start origin time and location to nearest station
//

x0=x[nmin]+1.0;
y0=y[nmin]+1.0;
t0=t[nmin]-1.0;
//
//////////////////////////////////////////////////////////////////////
//  make ndepth solutions for differnet depths BIG LOOP
//////////////////////////////////////////////////////////////////////
//
final_run:                                       /* get here after depth loop*/
h0=h0-10.0;     /* subtract since always added below */
for(idepth=1;idepth<(ndepth+1);idepth++)
{
h0=h0+10.0;    /* next depth */
//
//   make 15 iterations
//
for (kk=1;kk<16;kk++)
   {
    printf(" x,y,t %f %f %f \n",x0,y0,t0);
    for(i=1;i<(n+ncoda+1);i++) /* loop over readings, coda included to calculate distance*/
       {
        d[i]=sqrt((x[i]-x0)*(x[i]-x0)+(y[i]-y0)*(y[i]-y0)+h0*h0);  /* distance */
        tr[i]=t[i]-t0;                                /* observed travel time */
        dc[i]=vel[i]*tr[i];                           /* distance from time   */
        /*dd[i]=vel[i]/sqrt((1.0-h0/d[i])*(1.0-h0/d[i]));*/          /* ray parameter */
        dd[i]=vel[i];
       }
//
//   location equations
//
    for (i=1;i < (n+1);i++)
       {
        xc=x[i]-x0;
        yc=y[i]-y0;
        r=sqrt(xc*xc+yc*yc);
        a[i] [1]=xc/(r*vel[i]);
        a[i] [2]=yc/(r*vel[i]);
        a[i] [3]=-dd[i]/vel[i];
        a[i] [4]=d[i]/vel[i]-tr[i];
       }
//
//   make normal equations if more than 3 equations
//
    for(i=1;i<4;i++)            /*reset */
       {
        for(k=1;k<5;k++)
          b[i] [k]=0.0;
       }
       
    if (n > 3)
       {
        for(l=1;l<4;l++)
           {
            for(k=1;k<5;k++)
               {
                for(i=1;i<(n+1);i++)
                   b[l] [k] = b[l] [k] + a[i] [l] * a[i] [k];
               }
            }
        } 
     else
        {
         for(i=1;i<4;i++)
            {
             for(k=1;k<5;k++)
                b[i] [k] = a[i] [k];
            }
         }
//
//   solve equations
//
d0 =    b[1] [1]*b[2] [2]*b[3] [3]+b[1] [2]*b[2] [3]*b[3] [1]+b[1] [3]*b[2] [1]*b[3] [2];
d0 =d0 -b[1] [3]*b[2] [2]*b[3] [1]-b[1] [1]*b[2] [3]*b[3] [2]-b[1] [2]*b[2] [1]*b[3] [3];
dd1=    b[1] [4]*b[2] [2]*b[3] [3]+b[1] [2]*b[2] [3]*b[3] [4]+b[1] [3]*b[2] [4]*b[3] [2];
dd1=dd1-b[1] [3]*b[2] [2]*b[3] [4]-b[1] [4]*b[2] [3]*b[3] [2]-b[1] [2]*b[2] [4]*b[3] [3];
dd2=    b[1] [1]*b[2] [4]*b[3] [3]+b[1] [4]*b[2] [3]*b[3] [1]+b[1] [3]*b[2] [1]*b[3] [4];
dd2=dd2-b[1] [3]*b[2] [4]*b[3] [1]-b[1] [1]*b[2] [3]*b[3] [4]-b[1] [4]*b[2] [1]*b[3] [3];
dd3=    b[1] [1]*b[2] [2]*b[3] [4]+b[1] [2]*b[2] [4]*b[3] [1]+b[1] [4]*b[2] [1]*b[3] [2];
dd3=dd3-b[1] [4]*b[2] [2]*b[3] [1]-b[1] [1]*b[2] [4]*b[3] [2]-b[1] [2]*b[2] [1]*b[3] [4];
c[1]=dd1/d0;
c[2]=dd2/d0;
c[3]=dd3/d0;
//
//   calculate the residuals and rms
//
     rm=0.0;
     for(i=1;i<(n+1);i++)
        {
         res[i]=tr[i]-d[i]/vel[i];
         rm=rm+res[i]*res[i];
         }
     rm=sqrt(rm/n);
//
//   new epicenter and origin time
//
     x0=x0+c[1];
     y0=y0+c[2];
     t0=t0+c[3];
 }                            /* end of iteration loop*/
     if(rm<rmsmin)            /* save best depth solution */
      {                     
       xmin=x0;
       ymin=y0;
       hmin=h0;
       tmin=t0;
       rmsmin=rm;
      }
////////////////////////////////////////////////////////////////////////////
}                             /* end of depth loop */
/////////////////////////////////////////////////////////////////////////////
//   
//   if a depth loop, go back and calculate for best depth
//
     if(ndepth>1)
       {
         ndepth=1;            /* only calculate once now */
         x0=xmin;
         y0=ymin;
         h0=hmin;
         t0=tmin;
         goto final_run;       /* backand calculate for best depth */
       }
//
//   calculate magnitude
//
     mag=0.0;
     if(ncoda>0)
       {
        for(i=(n+1);i<(n+ncoda+1);i++)
        {
           mag=mag+c1*log10(tr[i])+c2*d[i]+c3;   
        }
        mag=mag/ncoda;
        }
//
//   calculate lat and lon
//
     lat=slat[1]+y0/111.2;    
     lon=slon[1]+x0/(111.2*cos(slat[1]/57.3));
//
//   print results
//
     printf(" \n");
     printf("Lat= %7.3f  Lon= %8.3f  Depth= %5.1f  mag= %3.1f  rms= %5.2f t0= %5.2f \n",lat,lon,h0,mag,rm,t0);
     printf(" \n");
     printf("STAT  PHASE  DISTAN     TIME     RESI\n");
     for(i=1;i<(n+1);i++)
       printf("%4s      %c  %6.1f   %6.1f    %5.1f\n",stat[stat_nr[i]],phs[phase[i]],d[i],t[i],res[i]);
//
// put in a file to use by program map
//
     epi=fopen("/home/work/epicenter","w");
     fprintf(epi,"%d %d %d %d %d\n",year,month,day,hour,min);
     fprintf(epi,"%f %f %f %f %f\n",lat,lon,h0,mag,rm);
     fclose(epi);
//
end_epi:
     printf(" \n");
//
//   end program
//
}




