/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:17/02/98                       */
/* Program: EXT                                                             */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program checks out the time span you have in your ringbuffer system  */
/* 																			*/
/* Changed timing routines for millenium compliance 26/01/99 DJS			*/
/*                                                                          */
/****************************************************************************/     

#include "shar_mem.h"
#include <sys/mman.h>
#include "filedef.h"
#include <fcntl.h>
#include <errno.h>
//#include "timexrou.c"     /* removed these routines - not 2000 compliant */
#ifdef __USAGE
------------------------------------------------------------------------------
%C                                             Date: 01/10/97     Version 7.00
------------------------------------------------------------------------------
Description: Program to extract data from ringbuffer. 

Usage:       ext (program ask for input), will only run when SEISLOG active
------------------------------------------------------------------------------
#endif


struct date {
             int year;
             int month;
             int day;
             int hour;
             int min;
             int doy;            /* day of year */
             float secs;         /* seconds     */
             double abs_secs;     /* seconds from 1/1/1900 */
            };     /* JE's functions use this structure - although we now
					  use different functions, the structure is needed
					  for reading dates - was in timexrou.c */
double MSECS;


int      fd;
smod     *ptr;
int      cat_started=0;                /* indicator that CATLOG was started */
                                       /* by the program                    */
char     kill_cat[10];                 /* used to hold system command to    */
                                       /* kill CATLOG                       */
int      tophdr;                       /* header each ringbuffer            */

main()
{

int      start_block;                  /* first block that the user requires*/
int      total_blocks;                 /* total number of blocks required   */
int      index;                        /* will be set to the index number of*/
                                       /* the event that is created         */
int      i;                            /* counter                           */
int      rngfil;
char     c;



/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"EXT_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"EXT_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

ptr->man_que=2;

tophdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
/*--------------------------------------------------------------------------*/
/* start program to find time span of current ringbuffer system             */
/*--------------------------------------------------------------------------*/
//system("/home/seislog/cls");
system("/home/seislog/min_max");       /* find time span ringbuffers        */
/*--------------------------------------------------------------------------*/
/* ask user for the start and duration of the event                         */
/*--------------------------------------------------------------------------*/
find_blocks(&start_block,&total_blocks,&rngfil);

printf("%5d blocks will be extracted.\n",total_blocks);
printf("from ringbuffer file %d\n",rngfil);
printf("starting at block %d\n",start_block);
ptr->dur_ext=total_blocks;
ptr->cur_rng=rngfil;
ptr->start_blk=start_block;
printf("Ringbuffer %d Startblock %d\n",ptr->cur_rng,ptr->start_blk);
//printf("EXT %d %d %d\n",ptr->dur_ext,ptr->cur_rng,ptr->start_blk);
/*--------------------------------------------------------------------------*/
/* start up cat_ext program                                                 */
/*--------------------------------------------------------------------------*/
system("/home/seislog/cat_ext");

} /* end of EXT */

/***************************************************************************
This procedure finds the start block that the user requires from
the start time input
****************************************************************************/

find_blocks(start_block,total_blocks,rng_fil)
     int* start_block;
     int* total_blocks;
     int* rng_fil;
{
double   DIFF_SEC1,DIFF_SEC2;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;

struct  date in_dat;
struct  date first_buf_date, sec_buf_date;
int     inputting=0, locating=1;
FILE*   in;                            /* circular buffer                   */
int     first_date;                    /* position of first date found in   */
                                       /* the buffer                        */
int     second_date;                   /* position of second date found in  */
                                       /* the buffer                        */
double  time_gap;                      /* interval between each record in   */
                                       /* the buffer                        */
int     err;
int     kp;
int     place;                         /* file position indicator           */
double  diff;                          /* difference between the first time */
                                       /* in the buffer and the user        */
                                       /* requested time                    */
float   moddiff;                       /* absolute time difference for      */
                                       /* comparison                        */
int     try_pos;                       /* trial date and position for       */
                                       /* finding the right block           */
struct  date try_date;                 /*             "                     */
char    ans,c;                         /* indicate whether block or date    */
                                       /* will be input                     */
err:
printf("Do you still want to extract data Y/N [Y] ?");
ans=getchar();
switch(ans)
{
    case '\n':
    case 'Y':
    case 'y':
    goto input;
    break;
    case 'N':
    case 'n':
    exit(0);
    break;
    default:
    goto err;
    break;
}
input:
printf("\nEnter the date and time you require:\n");
while(inputting==0)
{
    printf("YYYY MM DD HR MN SS- (all whole numbers)\n");
    fflush(stdin);
    err = scanf("%d%d%d%d%d%f",&in_dat.year,&in_dat.month,&in_dat.day,
          &in_dat.hour,&in_dat.min,&in_dat.secs);
    while(getchar()!= '\n');
    if (err <= 5);                     /* input error                       */
        else if(in_dat.month<1 || in_dat.month>12)
            printf("\nMonth %d not reasonable\n",in_dat.month);
        else if(in_dat.day<1 || in_dat.day>31)
            printf("\nDay %d not reasonable\n",in_dat.day);
        else if(in_dat.hour<0 || in_dat.hour>23)
            printf("\nHour %d not reasonable\n",in_dat.hour);
        else if(in_dat.min<0 ||in_dat.min>59)
            printf("\nMinute %d not reasonable\n",in_dat.min);
        else if(in_dat.secs<0 ||in_dat.secs>59.9999)
            printf("\nSecs %f not reasonable\n", in_dat.secs);
        else inputting=1;

    check_for_future_time (&in_dat, 0);   /* DJS 17/11/00 */

    printf("Seeking for: %2d/%2d/%2d %2d:%2d:%6.3f\n",
    in_dat.year,in_dat.month,in_dat.day,in_dat.hour,in_dat.min,in_dat.secs);
/*--------------------------------------------------------------------------*/
/* now work out the date in absolute seconds                                */
/*--------------------------------------------------------------------------*/
	YR=in_dat.year;
	MTH=in_dat.month;
	DAY=in_dat.day;
	HR=in_dat.hour;
	MIN=in_dat.min;
	SEC=in_dat.secs;
	TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
} /* end of if statement, both ways of inputting need the time_gap          */

/*--------------------------------------------------------------------------*/
/* now look in the buffer and read the first date from it                   */
/*--------------------------------------------------------------------------*/
place=0;
for(kp=0;kp<ptr->nringbuf[0];kp++)
{
    rngopen(kp,&in);                   /* opens the next ring file and adds */
    first_date = tophdr;
    second_date =first_date+ptr->reclength;
    if ((err=fseek(in,first_date,place))<0)
    {
         fclose(in);
         printf("Unable to seek in circular buffer\n");
    }
    first_buf_date.year = 0;
    first_buf_date.month= 0;
    first_buf_date.day  = 0;
    first_buf_date.hour = 0;
    first_buf_date.min  = 0;
    first_buf_date.secs = 0;

    read_date(in, &first_buf_date);

//    printf("FIRST_BUF_DATE: %2d/%2d/%2d %2d:%2d:%6.3f  ",
//         first_buf_date.year,first_buf_date.month,first_buf_date.day,
//         first_buf_date.hour,first_buf_date.min,first_buf_date.secs);

    fseek(in, second_date,0);
    read_date(in,&sec_buf_date);

/*--------------------------------------------------------------------------*/
/* work out the time gap between each record                                */
/*--------------------------------------------------------------------------*/
	TIMSEC (first_buf_date.year,first_buf_date.month,first_buf_date.day,
        first_buf_date.hour,first_buf_date.min,first_buf_date.secs,&DIFF_SEC1);
	TIMSEC (sec_buf_date.year,sec_buf_date.month,sec_buf_date.day,
        sec_buf_date.hour,sec_buf_date.min,sec_buf_date.secs,&DIFF_SEC2);
	time_gap = DIFF_SEC2 - DIFF_SEC1;
//    printf("TIME GAP=%f\n",time_gap);
	diff = MSECS - DIFF_SEC1;
//    printf("DIFF=%f\n",diff);
    if( (int)diff >= 0)
    {
        if((int)diff <= ptr->nmin_per_ring[0] * 60)
        {
//            printf("probably in this file\n");
            goto next;
        }
        if( (int)diff >= ptr->nmin_per_ring[0] * 60 && (int)diff < (ptr->nmin_per_ring[0]+1)*60)
        {
//            printf("probably in next file\n");
            fclose(in);
            kp++;
            rngopen(kp,&in);           /* opens the next ring file and adds */
            first_date = tophdr;
next:
            if ((err=fseek(in,first_date,place))<0)
            {
                fclose(in);
                printf("Unable to seek in circular buffer\n");
            }
            read_date(in, &first_buf_date);

//            printf("        FIRST_BUF_DATE: %2d/%2d/%2d %2d:%2d:%6.3f  ", 
//            first_buf_date.year,first_buf_date.month,first_buf_date.day,
//            first_buf_date.hour,first_buf_date.min,first_buf_date.secs);
			  TIMSEC (first_buf_date.year,first_buf_date.month,first_buf_date.day,
        			first_buf_date.hour,first_buf_date.min,first_buf_date.secs,&DIFF_SEC1);
			  diff = MSECS - DIFF_SEC1;
//            printf("%f\n",diff);
            if( diff <= 1.0)
            {
                printf("found time in file R %d block:%d\n",kp,(first_date-tophdr)/ptr->reclength);
                try_pos=first_date;
                goto found;
            }
            first_date=first_date+ptr->reclength;
            goto next;
        }
    }
    fclose(in);
}
printf("Specified time not found in ringbuffer system.\n");
exit(0);
found:
fseek(in,try_pos,0);
read_date(in,&try_date);
options(&in_dat,&try_pos,&try_date,in,time_gap,start_block);

/*now get the required duration of the event                                */

fclose(in);
printf("\nNumber of seconds to extract: ");
while(scanf("%d", total_blocks)<=0)
{
    while(getchar()!='\n');
    printf("\nNumber of seconds to extract: ");
}
while(getchar()!='\n');

check_for_future_time (&try_date, *total_blocks);     /* DJS 17/11/00 */

*total_blocks = (int)(*total_blocks/time_gap) +1;
*rng_fil=kp;
}
/****************************************************************************/
/* This next procedure looks at the time diference found then allows
   the user to go backwards and forwards in the buffer. 21.4.94 JE */

options(in_dat,try_pos,try_date,in,time_gap,start_block)
FILE*   in;                            /* ring buffer file pointer          */
int*    try_pos;                       /* current position in file data     */
int*    start_block;                   /* to be returned as the starting    */
                                       /* block required                    */
struct  date* in_dat;                  /* date required by the user         */
struct  date* try_date;                /* date at try_pos                   */
double  time_gap;                      /* length of each data block         */

{
double  diff;                          /* time difference between the two   */
                                       /* dates                             */
char    ans;                           /* user answer                       */
double  DIFF_SEC;

while(1)
{
	TIMSEC (try_date->year,try_date->month,try_date->day,
        try_date->hour,try_date->min,try_date->secs,&DIFF_SEC);
	diff = MSECS - DIFF_SEC;
    if (diff<0) diff *= -1;
    printf("\nThe date found is: \n");
    printf("%2d/%2d/%2d, %2d:%2d, and %f secs", try_date->day,
           try_date->month, try_date->year, try_date->hour,
           try_date->min, try_date->secs);

    if (diff <= time_gap) 
    {
        *start_block = *try_pos;
        return(0);
    }
    printf("\ndiff:%f time_gap:%f\n",diff,time_gap);
    printf("can't solve the search key\n");
} /* end of while loop */
} /* end of procedure */



/************************************************************************/
 scan_buffer(in,in_dat,try_pos)
    FILE* in;
    struct date* in_dat;
    int* try_pos;

 /* This routine is used on the off-chance that the buffer is fragmented
    and the data required may be in there somewhere. The whole buffer
    is scanned and the data checked to see if it is near the date
    required. The closest date is saved. */

   {
     int i;                        /* counter */
     struct date temp_dat;         /* for reading each date into */
     double diff;                  /* difference in times between the nearest
                                      and current times */
     double nearest_diff;
     char ans;
     int percent;
	 double DIFF_SEC1, DIFF_SEC2;


     printf("\nScanning the whole buffer for your time....\n");
     printf("\nThis may take over 10 minutes, Do you want to proceed? [Y]: ");
     ans = getchar();
     if (ans=='\n') ans = 'Y';
     else while (getchar()!='\n');
     if (ans=='N' || ans=='n') {
       printf("Unable to find start time required.\n");
       exit(0);
      }

    /* initialise start_block and date */
        fseek(in, ptr->reclength,0);
        read_date(in, &temp_dat);
        *try_pos = ptr->reclength;
		TIMSEC (temp_dat.year,temp_dat.month,temp_dat.day,
        temp_dat.hour,temp_dat.min,temp_dat.secs,&DIFF_SEC1);
		nearest_diff = DIFF_SEC1 - MSECS;
        if(nearest_diff<0) nearest_diff *= -1;
   /*           printf("\n nearest_diff: %f", nearest_diff); */

     printf("%2d%%\b", 0);
     for(i=2;i<ptr->maxblk;i++){
        printf("\b\b%2d", (int)(i*100/(ptr->maxblk)));
        fseek(in,i*ptr->reclength,0);
        read_date(in,&temp_dat);
             /* now compare the dates */
		TIMSEC (temp_dat.year,temp_dat.month,temp_dat.day,
        temp_dat.hour,temp_dat.min,temp_dat.secs,&DIFF_SEC2);
		diff = DIFF_SEC2 - MSECS;
        if(diff<0) diff *= -1;
        if(diff < nearest_diff){
        /*  printf(" nearer found ");
            printf("\n diff: %f", diff);
            printf("\n %2d/%2d/%2d %2d:%2d:%f", temp_dat.year, temp_dat.month,
                    temp_dat.day, temp_dat.hour, temp_dat.min, temp_dat.secs);
        */  nearest_diff = diff;
            *try_pos = i*ptr->reclength;
            }
        }


    } /* end of scanning routine */






  /*************************************************************************
  */
      read_date(in, dat)
        FILE* in;
        struct date* dat;

     {
        char bytes[200];
        fread(bytes,200,1,in);
        sscanf(bytes,"%d",&dat->hour);
        sscanf(bytes+3,"%d",&dat->min);
        sscanf(bytes+6,"%f",&dat->secs);
        sscanf(bytes+12,"%d",&dat->day);
        sscanf(bytes+16,"%d",&dat->month);
        sscanf(bytes+19,"%d",&dat->year);

     /*   printf("hour: %d, min %d, secs %f, day %d, month %d, year %d\n",
               dat->hour, dat->min, dat->secs, dat->day, dat->month,
               dat->year); */
    }
rngopen(nxt_rbf,in)
int nxt_rbf;
FILE* *in;
{ 
char rngbuf[50];
char indexrec[50];
int i;
int lr;
static char path[]="/home/buffer/RXXXXl";
for(lr=0;lr<20;lr++)
    rngbuf[lr]=' ';
sprintf(indexrec,"%4d\n",nxt_rbf);
if(indexrec[0]==' ')
    indexrec[0]='0';
if(indexrec[1]==' ')
    indexrec[1]='0';
if(indexrec[2]==' ')
    indexrec[2]='0';
if(indexrec[3]==' ')
    indexrec[3]='0';
path[14]=indexrec[0];
path[15]=indexrec[1];
path[16]=indexrec[2];
path[17]=indexrec[3];
for(lr=0;lr<18;lr++)
{
    rngbuf[lr]=path[lr];
}
rngbuf[18]='\0';
//printf("This is current ringbuffer: %s\n",rngbuf);
/*--------------------------------------------------------------------------*/
/* open the corresponding ringbuffer file                                   */
/*--------------------------------------------------------------------------*/
if((*in=fopen(rngbuf, "r"))==NULL)
{
    fprintf(stderr,"EXT: Not able to open %s, error=%d\n",rngbuf,errno);
    exit(1);
}
}

/*************************************************************************
  Function to check whether the extract requested will go into the future
  DJS 17/11/00
**************************************************************************/

void check_for_future_time (struct date *try_date, int duration)
{
  int hr, min, day, mon, yr, doy;
  float secs;
  double current_secs, requested_secs;


    sscanf (ptr->timcpy, "%d:%d:%f %d/%d/%d %d", &hr, &min, &secs, &day, &mon, &yr, &doy);
    TIMSEC (yr, mon, day, hr, min, secs, &current_secs);
    TIMSEC (try_date->year, try_date->month, try_date->day, try_date->hour,
            try_date->min, try_date->secs, &requested_secs);
    requested_secs += (float) duration;
    if (requested_secs > current_secs)
    {
		fprintf (stderr, "EXT: Requested time not yet recorded\n");
		exit (1);
	}
}



