/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:02/02/95                       */
/* Program:FPL                                                              */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program plot data from SEISLOG in near real time. All channels      */
/* marked for detection are plotted. Data are extracted from the circular   */
/* diskfile and plotted to the screen. When a lot of channels are plotted,  */
/* there will be a delay in the plotting. This is because this program will */
/* have a lower priority than the real time processes.                      */
/* This routine decide what type of digitizing unit used in the             */
/* configuration and start the corresponding plotting program.              */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <graph.h>
#include <conio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <sys/name.h>

int      fnr;                          /* font number                       */
int      n;                            /* number of fonts                   */

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
main()
{
char*    fontpath=getenv("WINFONTS");  /* use fonts from Windows            */
pid_t    proxy;
unsigned msg;
int      fnt_no;                       /* font number to use                */
int      con_ser;                      /* switch to tell console or serial  */
int      upper;                        /* bottom of top header box          */
int      delta;
int      total;
int      intval;
int      va;
int      ret;
int      kan;                          /* channel number to plot            */
int      secs;
int      intvalue;
char     tmp[10];
float    ampli;
float    value;
int      ix;
int      iy;

int      lp;
int      lc;                        /* Counter                              */
int      ll;                        /* Counter                              */
int      kl;                        /* Counter                              */
int      id;
float    weight;                    /* factor determines the amplitude and  */
                                    /* max min scaling on graph.            */
                                    /* higher number: smaller amplitudes    */
                                    /* lower number : higher amplitudes     */
char     as[50];
int      fpl_id;
int      det_evt_id;
unsigned int     cop;
int      c;
int      left;
unsigned int* ch_ptr;
int      sec_no;                    /* buffer number in pipe just processed */
int      k;
int      l3;
int      hopp;
int      kk;
short    w;
unsigned int ver;
int      ixold[MAXCHA];
int      iyold[MAXCHA];
int      i;                            /* Counter                           */
float    ixfl;                         /* float value of ix                 */
int      ystart[MAXCHA]; /* start y values each trace screen koord.      */
float    factor; /* Gain factor                                  */
int      offset;                    /* offsets different channels                   */
int      number;         /* number of channels marked for detection      */
int      skip;           /* Sample skip factor                           */
int      npage;          /* New page indicator                           */
int      sw;             /* Switch to indicate plotting of station       */
float    increm;         /* increment x-axis                             */
int      y_val;
unsigned old_line_style;

fnt_no=61;                             /* font number to use                */
weight=6.0;                            /* amplitude scaling                 */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"FPL_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"FPL_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* read COLOR.MAP file                                                      */
/*--------------------------------------------------------------------------*/
system("/home/seislog/rd_color");
/*--------------------------------------------------------------------------*/
/* check if only one parameterset is active                                 */
/*--------------------------------------------------------------------------*/
if(ptr->active > 1)
{
    system("/home/seislog/cls");
    printf("FPL can only be executed  when 1 parameter set is specified.\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* decide if I am started from console or remote terminal                   */
/*--------------------------------------------------------------------------*/
con_ser=line_typ();
printf("LINE %d\n",con_ser);
/*--------------------------------------------------------------------------*/
/* locate DET_EVT                                                           */
/*--------------------------------------------------------------------------*/
det_evt_id=qnx_name_locate(0,"det_evt",0,&cop);
if(det_evt_id == -1)
{
    printf("FPL: Can't find DET_EVT. SEISLOG not active\n");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Find number of channels specified for detection and which channels       */
/*--------------------------------------------------------------------------*/
number=1;
printf("Enter Channel Number:");
scanf("%d",&kan);
ampli=1.0;
fnt_no=61;                             /* set font number to 61             */
/*--------------------------------------------------------------------------*/
/* attach proxy for the terminal in use                                     */
/*--------------------------------------------------------------------------*/
if(con_ser == 1)                       /* SERIAL type ?                     */
{
    proxy=qnx_proxy_attach(0,0,0,-1);
    ptr->fpls_proxy=proxy;
}
if(con_ser == 2)                       /* CONSOLE type ?                    */
{
n=_registerfonts(fontpath);            /* register availlable fonts         */
    proxy=qnx_proxy_attach(0,0,0,-1);
    ptr->fplc_proxy=proxy;
}
/*--------------------------------------------------------------------------*/
/* attach this process under the name FPL                                   */
/*--------------------------------------------------------------------------*/
if(con_ser == 1)                       /* SERIAL type ?                     */
{
    fpl_id=qnx_name_attach(0,"fpls");
    if(fpl_id == -1)
    {
        printf("FPL: SERIAL attach failed. FPL already active on serial ?\n");
        exit(0);
    }
}
if(con_ser == 2)                       /* CONSOLE type ?                    */
{
    fpl_id=qnx_name_attach(0,"fplc");
    if(fpl_id == -1)
    {
        printf("FPL: CONSOLE attach failed. FPL already active on console ?\n");
        exit(0);
    }
}
/*--------------------------------------------------------------------------*/
/* Set default SKIP value                                                   */
/*--------------------------------------------------------------------------*/
skip=0;
/*--------------------------------------------------------------------------*/
/* Set indicator for new page                                               */
/*--------------------------------------------------------------------------*/
npage=1;
/*--------------------------------------------------------------------------*/
/* Set switch to plot station                                               */
/*--------------------------------------------------------------------------*/
sw=1;
/*--------------------------------------------------------------------------*/
/* Set default time increment                                               */
/*--------------------------------------------------------------------------*/
increm=1.0;
/*--------------------------------------------------------------------------*/
/* Set up start y-values based on number of channels in detector            */
/*--------------------------------------------------------------------------*/
upper=47;
delta=400/2;
ystart[0]=upper+delta/2;
ystart[1]=ystart[0]+delta;
ystart[2]=ystart[1]+delta;


/*--------------------------------------------------------------------------*/
/* enter graphics mode                                                      */
/*--------------------------------------------------------------------------*/
graphic_mode(con_ser);
/*--------------------------------------------------------------------------*/
/* prepare plot area                                                        */
/*--------------------------------------------------------------------------*/
plt_hdr_box(ptr,con_ser);
plt_plt_box(ptr,con_ser,fnt_no);

top:
offset=(int)ptr->dc[kan-1];
factor=ptr->to[kan-1];
/*--------------------------------------------------------------------------*/
/* prepare side label                                                       */
/*--------------------------------------------------------------------------*/
plt_sid_box(ptr,con_ser,fnt_no,upper,delta,number);

for(i=0;i<2;i++)                       /* restart plotting at beginning     */
{
    ixold[i]=80;
    iyold[i]=ystart[i];
}
for(k=0;k<4;k++)                       /* extract station code for channel  */
    tmp[k]=ptr->statio[(kan-1)*4+k];
tmp[4]='\0';

for (;;)                               /* enter infinite loop               */
{
    det_evt_id=Receive(0,&msg,sizeof(msg)); /* wait for buffer ready       */
    sec_no=ptr->buf_rdy_fpl;
    id=sec_no;
    id=id % 2;

    ret=keyread();                     /* anything from key board ?         */
    switch(ret)
    {
        case 10:                       /* arrow UP increase amplification   */
        ampli=ampli+0.08;
        break;
        case 20:                       /* arrow DOWN decrease amplification */
        ampli=ampli-0.08;
        if(ampli < 0.01) ampli=0.01;
        break;
        case 30:                       /* arrow RIGHT increase time step    */
        increm=increm+0.05;
        break;
        case 40:                       /* arrow LEFT decrease time step     */
        increm=increm-0.05;
        if(increm < 0.01) increm=0.01;
        break;
        case 99:
/*        ptr->plt_on_off=0;*/
        exit(0);
        break;
        default:
        break;
    }
    if(npage == 1)                     /* check for new page                */
    {
/*--------------------------------------------------------------------------*/
/* whenever a plot is at the end of screen, a new page is prepared          */
/*--------------------------------------------------------------------------*/
        plt_hdr_box(ptr,con_ser);      /* prepare top header box            */
        fill_hdr_box(ptr,offset,kan,increm,skip,con_ser,fnt_no);
        npage=NewPage2(ptr,con_ser,offset,kan,fnt_no,skip,increm,number,delta,upper);
        fill_sid_lab(ptr,con_ser,number,factor,fnt_no,delta,upper,offset,kan,weight);

        ix=ixold[0];
        iy=iyold[0];
        ixfl=(float)ix;
        _moveto(ix,iy);
    }
    hopp=ptr->nchan*(skip+1);
    for(l3=0;l3<2;l3++)
    {
        ix=ixold[l3];
        iy=iyold[l3];
        if(l3==0)
        {
            _setcolor(ptr->plt_are_bck_col);
            if(ix+(int)(increm*ptr->rate) <637)
            {
                _rectangle(_GFILLINTERIOR,ix+1,upper,ix+(int)(increm*ptr->rate)+10,upper+(number+1)*delta-4);

old_line_style=_getlinestyle();
_setcolor(7);
_setlinestyle(0x000f);
_moveto(ix+1,delta/2+upper);
_lineto(ix+(int)(increm*ptr->rate)+10,delta/2+upper);

_moveto(ix+1,delta/2+delta+upper);
_lineto(ix+(int)(increm*ptr->rate)+10,delta/2+delta+upper);

_setlinestyle(old_line_style);
_setcolor(ptr->sid_lab_txt_col);
            }else{
                _rectangle(_GFILLINTERIOR,ix+1,upper,637,upper+(number+1)*delta-4);
            }
        }
        _moveto(ix,iy);
        ixfl=(float)ix;
        kk=0;
        _setcolor(ptr->plt_are_plt_col);
        for(lp=0;lp<ptr->bufsiz;lp=lp+hopp)
        {
            if(ix > 637)
            {
                npage=1;               /* Indicate new page                 */
                sw=0;                  /* Indicate plot station code        */
                goto top;
            }
            if(l3==0)
            {
                total=(ptr->nsamp*ptr->nchan)+HEADER;
                ch_ptr=ptr->pipe+sec_no*total+HEADER+lp+(kan-1);
                ver=*ch_ptr;
                va=ver;
                value=va;
                value=value-offset;
                value=value * (100./(factor*weight))*ampli*(-1);
                intval=value;
                _lineto(ix,intval+ystart[l3]);
                ixfl=ixfl+increm;
                ix=(int)ixfl;
            }else{
                if(id==0)
                {
                    value=ptr->b1[lp+kan-1];
/*                  value=value-offset[0];*/
                    value=value * (100./(factor*weight))*ampli*(-1);
                    intval=value;
                    _lineto(ix,intval+ystart[l3]);
                    ixfl=ixfl+increm;
                    ix=(int)ixfl;
                }else{
                    value=ptr->b2[lp+kan-1];
/*                    value=value-offset[0];*/
                    value=value * (100./(factor*weight))*ampli*(-1);
                    intval=value;
                    _lineto(ix,intval+ystart[l3]);
                    ixfl=ixfl+increm;
                    ix=(int)ixfl;
                }
            }
        }
        ixold[l3]=ixfl-increm;
        iyold[l3]=intval+ystart[l3];
    }
    sw=0;                            /* Turn off indicator for station code */
}                                /* End of indefinite loop              */
}                                    /* End of main                         */

cls()
{

    putchar(0x1b);
    putchar(0x5b);
    putchar(0x32);
    putchar(0x4a);
    putchar(0x1b);
    putchar(0x5b);
    putchar(0x3b);
    putchar(0x48);
}

