typedef struct
{
    unsigned char   enable;
    unsigned char   gain;
    unsigned char   filterFrequency;
    unsigned char   filterPoles;
    unsigned short  dcOffset;
    float           channelLSB;
    unsigned short  channelPeak;
    unsigned short  trigger;
    char            channelUnit[5];
    char            channelName[5];
    unsigned char   channelDir;
    char            channelReserved[7];
} oneChannelStructure;    
/*****************************************************************************
Gsr 12 header used by the function
*****************************************************************************/

typedef struct
{
// Header & Event Information
char    headerID[4];
unsigned short  contFileNo;
unsigned short  checkSum;
unsigned long   noOfSamples;
unsigned short  noOfBlocks;
unsigned short  triggerMode;
          // bit 8: time table triggering
          // bit 7: X level triggering
          // bit 6: Y level triggering
          // bit 5: Z level triggering
          // bit 4: X LTA/STA
          // bit 3: Y LTA/STA
          // bit 2: Z LTA/STA
          // bit 1: hardware
          // bit 0: software
  unsigned char   preEventTime; // in second
  unsigned char   postEventTime; // in second
  unsigned char   systemTest;
  unsigned char   periodicTest;
  unsigned char   transmited;
  unsigned char   reserved05;
  unsigned short  dayFileNo;
  unsigned char   reserved1[8];
// Time & Date & Sync
  unsigned char   time[4];
  unsigned char   date[4];
  unsigned char   driftTime[4];
  unsigned char   driftDate[4];
  unsigned char   lastSyncTime[4];
  unsigned char   lastSyncDate[4];
  unsigned long   syncOffset;
  unsigned short  secondOffset;
  unsigned char   syncOk;
  unsigned char   timeValid;
  unsigned char   tprUpStat;
  unsigned char   tprUpPer;
  unsigned char   reserved[14];
// Instrument Information
  unsigned char   manName[25];
  unsigned char   instName[10];
  unsigned char   instSN[5];
  unsigned char   samplingModel;
  unsigned char   triggeringModel;
  unsigned short  noOfPowerLosses;
  unsigned short  temp; // at trigger time in tens of Kelvin
  unsigned short  fwv;
  char    rserved21[2];
  unsigned char   adResolution;
  char    latitude[10];
  char    longitude[10];
  char    altitude[6];
  char    rserved22[4];
  char    comment[50];
  unsigned char   reserved3;
  unsigned short  noOfSysRestarts;
  unsigned char   lastPowerLossTime[4];
  unsigned char   lastPowerLossDate[4];
  unsigned char   lastSysResTime[4];
  unsigned char   lastSysResDate[4];
  unsigned short  mainVoltage;
  unsigned short  backupVoltage;
  unsigned short  GNDVoltage;
  unsigned char   errorFlags;
          // bit 7: memory test
          // bit 6: battery test
          // bit 5: RTC test
          // bit 4: hardware test
          // bit 3: analog2 test
          // bit 2: analog1 test
          // bit 1: reserved
          // bit 0: reserved
  unsigned char   warningFlags;  // see above
  unsigned char   testSelected;
          // bit 7: memory test
          // bit 6: battery test
          // bit 5: RTC test
          // bit 4: hardware test
          // bit 3: test pulse
          // bit 2: analog test
          // bit 1: reserved
          // bit 0: reserved
  unsigned char   reserved4;
  unsigned char   testAnalog1;
          // bit 7: noise test channel X
          // bit 6: noise test channel Y
          // bit 5: noise test channel Z
          // bit 4: zero level test channel X
          // bit 3: zero level test channel Y
          // bit 2: zero level test channel Z
          // bit 1: no sensor detected
          // bit 0: reserved
  unsigned char   testAnalog2;
          // bit 7: filter response test channel X
          // bit 6: filter response test channel Y
          // bit 5: filter response test channel Z
          // bit 4: reserved
          // bit 3: reserved
          // bit 2: reserved
          // bit 1: ADC value resolution
          // bit 0: AD converter time too long
  unsigned char   maskAnalog1; // see testAnalog1
  unsigned char   maskAnalog2; // see testAnalog2
  unsigned char   testHardWare;
          // bit 7: EPROM checksum test
          // bit 6: EEPROM checksum test
          // bit 5: EERAM checksum test
          // bit 4: invalid instruction test
          // bit 3: address error test
          // bit 2: stack monitor test
          // bit 1: divide by zero test
          // bit 0: unexpected intrrupt test
  unsigned char   maskHardWare; // see testHardWare
  unsigned char   testClock;
          // bit 7: valid time and date test
          // bit 6: time increment test
          // bit 5: RTC memory test
          // bit 4: RTC oscillator test
          // bit 3: reserved
          // bit 2: reserved
          // bit 1: reserved
          // bit 0: reserved
  unsigned char   maskClock; // see above
  unsigned char   testBatt;
          // bit 7: primary battery error test
          // bit 6: primary battery warning test
          // bit 5: backup battery test
          // bit 4: reserved
          // bit 3: reserved
          // bit 2: reserved
          // bit 1: reserved
          // bit 0: reserved
  unsigned char   maskBatt; // see above
  unsigned char   testMemory;
          // bit 7: memory full test
          // bit 6: memory read/write test
          // bit 5: memory format test
          // bit 4: memory present test
          // bit 3: data checksum test
          // bit 2: data consistancy test
          // bit 1: memory 90% full test
          // bit 0: reserved
  unsigned char   maskMemory; // see above
  unsigned char   acPowerStatus; //0=no ac, 1=ac
  char    reserved5[2];
  unsigned char   noOfBlockSamples1;
  char    secondHeaderID[4];
  char    location[10];
  char    station[4];
  char    reserved6[30];
// Channel Information
  unsigned short   triggChannel;
  unsigned char    sensorType;
  unsigned char    noOfChannels; // each is 32 byte
  char     reserved7[10];
  oneChannelStructure channels[6];  // 192 bytes
  char     reserved8;
  unsigned char    noOfBlockSamples2;
} GSR_12_Header;  // 512 bytes 
