/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      Author: David Scott                 */
/*                                              BGS, Murchison House,       */
/*                                              Edinburgh                   */
/*                                                                          */
/*                                      Date:12/09/97                       */
/* Program: INT.C                                                           */
/*                                                                          */
/* Last update:   Removed condition for starting buffer and changed         */
/*    condition for returning proxy - more robust.  DJS 03/02/98            */
/*                                                                          */
/* Program description: Interrupt routine for code to talk to 24-bit ili    */
/*    Stand-alone application code is in /home/work/ili/code.c              */
/*    SEISLOG code is in ili_seis.c                                         */
/*                                                                          */
/****************************************************************************/


#include "ili_24b.h"
#include "shar_mem.h"

volatile pid_t ili_24b_irq_proxy = (pid_t) 0;    /* proxy for ILI interrupt */
volatile pid_t timer_proxy = (pid_t) 0;        /* proxy for timer interrupt */
extern volatile char buffer[2][MAX_TRANSFER_SIZE];       /* buffer for data */
extern volatile int current_buffer;                    /* buffer [0] or [1] */
extern volatile int index;                            /* position in buffer */
extern int transfers_per_sec;                  /* no of interrupts expected */
volatile int transfer_no;                         /* counter for interrupts */
volatile int ms_counter;                 /* millisecond counter for timeout */


/**********************************************************************
 *  interrupt handler for ILI data
 ***********************************************************************/
#pragma off (check_stack);
pid_t far ili_24b_irq_handler (void)
{
  static volatile char byte1 = (char *)0, 
                       byte2 = (char *)0;



	transfer_no++;                                /* count no of interrupts */

    if (transfer_no > transfers_per_sec)  /* if reached expected no per sec */
    {
    	transfer_no = 0;                                   /* reset counter */
        index = 0;                                         /* and index     */
	    return (pid_t) 0;                               /* and stop ack'ing */
    }


    outp (P8255_PORT_C, 0x00);                       /* set output port low */

	byte1 = inp (P8255_PORT_A);                             /* read lo byte */
	byte2 = inp (P8255_PORT_B);                             /* read hi byte */

	buffer[current_buffer][index++] = byte1;
	buffer[current_buffer][index++] = byte2;

	outp (P8255_PORT_C, 0x02);                                       /* ack */

    if (transfer_no == transfers_per_sec)         /* reached message length */
    {
        current_buffer = (current_buffer + 1) %2;           /* swap buffers */
		return (ili_24b_irq_proxy);    /* signal last buffer ready for read */
    }
	else return (pid_t) 0;
}
#pragma on (check_stack);


/***********************************************************************
 *   timer interrupt handler for determining timeout condition
 ***********************************************************************/
#pragma off (check_stack);
pid_t far timeout_handler (void)
{
	ms_counter++;
	if (ms_counter == 10000)      /* no data for ten seconds */
		return (timer_proxy);
	else return (0);
}
#pragma off (check_stack);
