/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:25/12/94                       */
/* Program:LGS                                                              */
/*                                                                          */
/* Last update:                                                             */
/* sep.97:     LAA fix problem with positive number selection for hours     */
/* sep.97:     LAA parameter to display output continuosly                  */
// nov 4, 97   JH  Wrap around wrong for time interval, check for no wrap
//                 if file not full
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program list out essential information from detection file          */
/* from the specified last detections or the last number of hours.          */
/*                                                                          */
/****************************************************************************/

#include "/home/seislog/shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include "/home/seislog/filedef.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/select.h>
#include <netdb.h>
#include <stdio.h>
#include <unistd.h>


#define PAUSED     0
#define CONTINUOUS 1
#define TRUE 1

FILE     *eve;                  /* Pointer to event-records                 */
int      nb;
int      cur_dpx;               /* Current detection number                 */
int      rec_adr;               /* Record address for given record          */
int      recorded;              /* Detections recorded                      */
int      count;
int      dpx;                   /* Detection number                         */
int      year;                  /* Year                                     */
int      doys;                  /* Day of year                              */
int      month;                 /* Month                                    */
int      day;                   /* Day                                      */
int      hour;                  /* Hour                                     */
int      min;                   /* Minute                                   */
float    ssec;                  /* Seconds, milliseconds                    */
int      beam;                  /* Beam number        Not in use            */
int      filter;                /* Filter                                   */
float    dur;                   /* Duration                                 */
float    rsta;                  /* STA at detection                         */
float    rlta;                  /* LTA at detection                         */
int      ista;                  /* Integer of STA                           */
int      ilta;                  /* Integer of LTA                           */
char     line[50];
char     karakter[2];           /* Character to indicate triggertype ' ','M'*/
float    rampl;
float    freq;
float    vel;
float    azi;
float    pwr;
int      detblk;                 /* Detection disk blocknumber              */
int      flag;                  /* Indicator for type of event              */
int      detsmp;                /* Sample in buffer at detection            */
int      ndur;
char     fnam[50];              /* Filename for this detection              */
int      addr;                  /* Diskaddress to read/write                */
int      number;                /* Negative: Number of detections back      */
/*           in time.                       */
/* Positive: Number of hours back in        */
/*           time.                          */
int     numdet;                 /* Number of records to write               */
char    input[512];             /* Variable for event-record                */
char    otime[15];              /* Current time                             */
char    date[15];               /* Current date                             */
char    doy[5];                 /* Current day of year                      */
/****************************************************************************/
int     dummy;                  /* Dummy                                    */
int     evst;                   /* Event number to start with               */
char    car;
int     hr;
float   sec;
int     sechel;
double  msecs;
double  timnow;
double  timdet;
double  timdif;
int     difference;
int     yr;
int     namp;
int     place;
int     bytes;
int     tnblka;                 /* Number of records in event-file          */
int     tsiz_event;             /* Number of bytes per event header         */
int     kl;
int     i;
int     j;
int     dif;
int     nn;
int     istart;
int     start;
int     k;
int     blk;
int     cnt;
long    position;
int msgsock;
char buf[1024];
int fd;
smod *ptr;

main(int argc, char *argv[])
{
int sock, length;
struct sockaddr_in server;
int rval;
fd_set ready;
struct timeval to;
double   MSECSN;
double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;

	/* Declarations for various names and functions                             */

	int     id;
        int     display;   /* to display output continuosly or pressing return */

        /* check if command line parameter is received */
        display = PAUSED;
        if (argc == 2) 
          if (strcmp("continuous",argv[1]) == 0)
             display = CONTINUOUS;

	/* Other declarations                                                       */
	/*--------------------------------------------------------------------------*/
	/* open shared memory region                                                */
	/*--------------------------------------------------------------------------*/
	fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
	if(fd== -1)
	{
		fprintf(stderr,"LGS_01 open failed:%s\n",strerror(errno));
		exit(1);
	}
	/*--------------------------------------------------------------------------*/
	/* set size of memory region                                                */
	/*--------------------------------------------------------------------------*/
	ltrunc(fd,sizeof(smod),SEEK_SET);
	/*--------------------------------------------------------------------------*/
	/* Compute pointer to shared memory region                                  */
	/*--------------------------------------------------------------------------*/
	ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
	if((int)ptr == -1)
	{
		fprintf(stderr,"LGS_02 mmap failed : %s\n",strerror(errno));
		exit(1);
	}

//nb=28;
nb=39;
/* Create socket */
  sock = socket(AF_INET, SOCK_STREAM, 0);  
if (sock < 0)   
{    
  perror("opening stream socket"); 
   exit(1);  
}
/* Name socket using wildcards */  
server.sin_family = AF_INET;  
server.sin_addr.s_addr = htonl(INADDR_ANY);  
server.sin_port = htons(5022);  
if (bind(sock, (struct sockaddr *)&server, sizeof(server)))   
{   
 perror("binding stream socket");
    exit(1);  
}
/* Find out assigned port number and print it out */ 
 length = sizeof(server); 
 if (getsockname(sock, (struct sockaddr *)&server, &length))   
{    
perror("getting socket name");   
 exit(1);  
}  
printf("Socket has port #%d\n", ntohs(server.sin_port));
/* Start accepting connections */ 
 listen(sock, 5);
  do   
{   
 FD_ZERO(&ready);  
  FD_SET(sock, &ready); 
   to.tv_sec = 5;    
if (select(sock + 1, &ready, 0, 0, &to) < 0)     
{     
 perror("select");   
   return EXIT_FAILURE;    
}    
if (FD_ISSET(sock, &ready))     
{
      msgsock =  accept(sock, (struct  sockaddr *)0, (int *)0);
      if (msgsock == -1)
       {
        perror("accept");
		return EXIT_FAILURE;  
    } else             {



	/* Initialize variables to startvalues etc.                                 */

	tsiz_event = 512;
	place      = 0;

	cur_dpx=Get_Current_DPX();          /* Get current detection number         */

	if(cur_dpx == 0)                    /* Check if current detection number    */
	{                                   /* is 0,  which means that there has    */
		/* been no detections since last        */
		/* reset of this file. If that's the    */
		/* case, write message and exit.        */
		printf("No detections since reset of DETNR\n");
		exit(0);
	}
	/* Print out station name, code and node from data module                   */

	printf("\nStation name: %s\n",ptr->name);

	rec_adr=Get_Adr_Rec(cur_dpx);       /* Get address current detection no.    */

	/* Open detection file                                                      */

	eve = fopen(DETECTION,"r");

	printf("Last detection number was: %d\n",cur_dpx);
	printf("\n");
	/* Input how many detections or how long time window you want               */

	printf("How many detections counted from the last (negative no)\n");
	printf("or number of hours from now (positive no)...");

number=-20;
//	scanf("%d",&number);
//	printf("\n");

	if(number == 0)                     /* In the case we dont want any log     */
		exit(0);                        /* we just stop right here              */

	/* Read current time from buffer 0                                          */

	/*sscanf(ptr->ibuf[0],"%s %s %s\n",otime,date,doy);*/
	sscanf(ptr->timcpy,"%s %s %s\n",otime,date,doy);

	printf("Current time: %s %s %s\n",otime,date,doy);
	printf("\n");
//
//  loop for number of detections ////////////////////////////////////////////
//
	if(number < 0)                      /* If the number specified is           */
	/* negative, the absolute value is      */
	/* the number of detections you want    */
	/* to print out. If recorded ?          */
	{
		numdet = -number;
		recorded = cur_dpx;
		if(numdet > recorded)           /* If the number of recordings is       */
			/* less than the number we want to      */
			/* print, print the number recorded     */
			numdet = recorded;
		if(numdet > ptr->nblka)
			numdet=ptr->nblka;

		printf("  dpx  y doy  date        time dur   sta    lta\
                  filename\n");
		cnt=0;
		do
		{
			addr = Read_Adr();
            read(msgsock,buf,1);      // RECEIVE LINEFEED FROM JRASK_A (R1)
            printf("got linefeed from client\n");
            BlankBuf();
			Det_Rec(addr);
next:

printf("NAME %s\n",buf);
              write(msgsock,buf,nb+1);

			numdet = numdet -1;
printf("NUMDET=%d\n",numdet);
		}        while(numdet > 0);
		goto finito;
	}
//
///  number of hours back  ///////////////////////////////////////////////////
//
	recorded = cur_dpx + 1;
	sscanf(otime,"%2d%*c%2d%*c%f6.3",&hour,&min,&sec);
	sscanf(date,"%2d%*c%2d%*c%4d",&day,&month,&year);
//    printf("YEAR %d\n",year);
	sscanf(doy,"%3d",&doys);
//      year=year+1900;
	sechel=sec;

//      printf("%d %d %d %d %d %d\n", year,month,day,hour,min,sechel);
    YR=year;
    MTH=month;
    DAY=day;
    HR=hour;
    MIN=min;
    SEC=sec;
    TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);

	timnow=MSECS;
//	printf("TIMNOW:%f\n",timnow);
	addr=rec_adr;
	/*printf("ADDR:%d\n",addr);*/
	numdet=0;
more:

//  address of previous event
	addr=addr-tsiz_event;
//  check for wrap around
	if(addr < 0)
             {
               if(cur_dpx < ptr->nblka)
                 {
                   addr=0; /* do not wrap around if first time file fills up*/
                   goto list;
                 }
		addr = addr + ptr->nblka*tsiz_event;
             }
	if(addr > ptr->nblka*tsiz_event)
		addr = 0;
	Read_Tim(addr);
    if(input[233] == 'X')
    {
        numdet++;
		if(numdet >= ptr->nblka) goto list;
        goto more;
    }
//      year=year+1900;
	sechel=sec;
//      printf("%d %d %d %d %d %d\n", year,month,day,hour,min,sechel);
    YR=year;
    MTH=month;
    DAY=day;
    HR=hour;
    MIN=min;
    SEC=sec;
    TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECSN);
	timdet=MSECSN;
//	printf("TIMDET:%f\n",timdet);
	timdif=timnow-timdet;
	difference=timdif/3600;
//      printf("DET:%f DIF:%f DHR:%d\n",timdet,timdif,difference);
	if(difference < number)
//    if(timdif > number*3600)
	{
		numdet=numdet+1;
		if(numdet >= ptr->nblka) goto list;
		goto more;
	}
list:
//      printf("NUMDET:%d\n",numdet);
	if(numdet != 0)
	{
		printf("  dpx  y doy  date        time dur   sta    lta\
                  filename\n");

		/*addr=rec_adr+tsiz_event;*/
		addr=rec_adr-(numdet)*tsiz_event;
		do
		{
			/*printf("ADDR:%d\n",addr);*/
			Det_Rec(addr);
			addr=addr+tsiz_event;
			if(addr < 0)
				addr = addr + ptr->nblka*tsiz_event;
			if(addr > (ptr->nblka-1)*tsiz_event)
				addr = 0;
			numdet=numdet-1;
		}while(numdet != 0);
		goto finito;
	}
	printf("No detections the last %d hours.\n",number);
	dummy=0;
finito:
	dummy=0;



             }idle:             close(msgsock);    } else      printf("JLGS idle\n");
      ptr->servers[6]=1;
  } while (TRUE);



}
/*--------------------------------------------------------------------------*/
/* Compute disk adress for correct detection record                         */
/*--------------------------------------------------------------------------*/
Read_Adr()
{
	int val;

	val=rec_adr-(numdet)*tsiz_event;
	if(val < 0)
		val=val+ptr->nblka*tsiz_event;
	if(val > (ptr->nblka-1)*tsiz_event)
		val=val - ptr->nblka*tsiz_event;
	return(val);
}
/*--------------------------------------------------------------------------*/
/* Read specified detection record and write corresponding values           */
/*--------------------------------------------------------------------------*/
Det_Rec(disc_addr)

int disc_addr;                     /* Disc address to specified det.record */
{
	position=fseek(eve,disc_addr,place);
	bytes=fread(input,tsiz_event,1,eve);
//	line[0]=input[207];
	line[0]=' ';
input[16]=' ';
input[17]=' ';
	sscanf(input,"%d %d %d %d %d %d %d %f %d %d %f %f %f %f\
 %f %f %f %f %d %d %d\n",
	    \
    &dpx, &year, &doys,  &month, &day,   &hour, &min,\
    &ssec,
	    &beam, &filter,&dur,   &rsta,  &rlta, &rampl,\
    &freq,&vel, 
	    &azi,   &pwr,   &detblk,&flag, &detsmp);

	dur=dur+0.5;
	rampl=rampl+0.5;
	ndur=dur;
	j=206;
	for(i=0;i<28;i++)
	{
		fnam[i]=input[j];
		j=j+1;
	}

	ista=rsta;
	ilta=rlta;

	printf("%5d%3d%4d%3d%3d%3d:%2d:%5.2f%4d%6d%7d%1c%s\
\n",dpx,year,doys,month,day,hour,min,ssec,ndur,ista,ilta,line[0],fnam);
	sprintf(buf,"%5d%1c%4d%1c%s\n",dpx,line[0],ndur,line[0],fnam);
        if(buf[0]==' ')buf[0]='0';
        if(buf[1]==' ')buf[1]='0';
        if(buf[2]==' ')buf[2]='0';
        if(buf[3]==' ')buf[3]='0';
        if(buf[6]==' ')buf[6]='0';
        if(buf[7]==' ')buf[7]='0';
        if(buf[8]==' ')buf[8]='0';
        if(buf[9]==' ')buf[9]='0';
//sprintf(buf,"%s\n",fnam);
//printf("%s\n",buf);
}
/*--------------------------------------------------------------------------*/
/* Read time for specified disc address                                     */
/*--------------------------------------------------------------------------*/
Read_Tim(disc_addr)

int disc_addr;                      /* Disc address to specified det.record */
{
	position=fseek(eve,disc_addr,place);
	bytes=fread(input,tsiz_event,1,eve);
//printf("%s\n",input);
	sscanf(input,"%d %d %d %d %d %d %d %f %d %d %f %f %f %f\
    %f %f %f %f %d %d %d\n",
	    \
    &dpx, &year, &doys,  &month, &day,   &hour, &min,\
    &ssec,
	    &beam, &filter,&dur,   &rsta,  &rlta, &rampl,\
    &freq,&vel, 
	    &azi,   &pwr,   &detblk,&flag, &detsmp);
//printf("TYPE=%c\n",input[233]);
}
/*--------------------------------------------------------------------------*/
/* Get current detection number from /dd/parameter/detnr                    */
/*--------------------------------------------------------------------------*/
Get_Current_DPX()
{
	FILE    *det;               /* Pointer to current detection number      */
	int     dpx;                /* Current detection number                 */
	det=fopen(DETNUMBER,"r+");
	fscanf(det,"%d",&dpx);      /* Read current detection number            */
	fclose(det);
	return(dpx);
}
/*--------------------------------------------------------------------------*/
/* Compute disk address for given detection record                          */
/*--------------------------------------------------------------------------*/
Get_Adr_Rec(record)
int record;
{
	return((record-(record/ptr->nblka)*ptr->nblka)*tsiz_event);
}
BlankBuf()
{
  int lt;
  for(lt=0;lt<nb;lt++)
    buf[lt]=' ';
}

