/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:20/09/95                       */
/* Program:MDC                                                              */
/*                                                                          */
/* Last update:                                                             */
/* 01/06/00 8.22 t.u. support for unit 30 DAS-1202 + 64 Earth Data new vers.*/
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program monitors the DC values each channel                         */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                       /* shared memory definitions    */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/qnxterm.h>

main ()
{
int      lines;                        /* lines to allocate on screen       */
int      i;                            /* counter                           */
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) {
    fprintf(stderr,"MDC open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) {
    fprintf(stderr,"MDC mmap failed : %s\n",strerror(errno));
    exit(1);
}
if(term_load() == -1)
{
    fprintf(stderr,"term_load error, errno=%d\n",errno);
    exit(1);
}
term_video_on();
if(term_state.qnx_term)
    fwrite("\033!71\033@71",8,1,stdout);
term_clear(0);
switch(ptr->unit)
{
  case 11:
  break;
  case 21:
  break;
  case 30:
  lines=(ptr->nchan-1)/6;
  term_box_fill(0,0,80,lines+3,TERM_BLUE | TERM_CYAN_BG,1,' ');
  term_type(0,20," DC values in counts Metrabyte 1202 ",36,
  TERM_GREEN | TERM_BLACK_BG);
  switch(lines)
  {
    case 0:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 1:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 2:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 3:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 4:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 5:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 6:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 7:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 8:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 9:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(10,2,"Channel 55-60:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 10:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(10,2,"Channel 55-60:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(11,2,"Channel 61-66:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
  }
  break;
  case 36:
  lines=(ptr->nchan-1)/6;
  term_box_fill(0,0,80,lines+3,TERM_BLUE | TERM_CYAN_BG,1,' ');
  term_type(0,20," DC values in counts Metrabyte 1800 ",36,
  TERM_GREEN | TERM_BLACK_BG);
  switch(lines)
  {
    case 0:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 1:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 2:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 3:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 4:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 5:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 6:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 7:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 8:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 9:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(10,2,"Channel 55-60:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 10:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(10,2,"Channel 55-60:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(11,2,"Channel 61-66:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
  }
  break;
  case 37:
  break;
  case 53:
  term_box_fill(0,0,80,3,TERM_BLUE | TERM_CYAN_BG,1,' ');
  term_type(0,25," DC values in counts Nanometrics ",33,
  TERM_GREEN | TERM_BLACK_BG);
  term_type(1,2,"Channel 01-03:",14,TERM_WHITE | TERM_BLUE_BG);
  break;
  case 56:
  break;
  case 63:
  term_box_fill(0,0,80,3,TERM_BLUE | TERM_CYAN_BG,1,' ');
  term_type(0,25," DC values in counts Earth Data ",33,
  TERM_GREEN | TERM_BLACK_BG);
  term_type(1,2,"Channel 01-03:",14,TERM_WHITE | TERM_BLUE_BG);
  break;
  case 64:
  term_box_fill(0,0,80,3,TERM_BLUE | TERM_CYAN_BG,1,' ');
  term_type(0,25," DC values in counts Earth Data ",32,
  TERM_GREEN | TERM_BLACK_BG);
  term_type(1,2,"Channel 01-03:",14,TERM_WHITE | TERM_BLUE_BG);
  break;
  case 96:
  lines=(ptr->nchan-1)/6;
  term_box_fill(0,0,80,lines+3,TERM_BLUE | TERM_CYAN_BG,1,' ');
  term_type(0,20," DC values in counts Synthetic Data ",36,
  TERM_GREEN | TERM_BLACK_BG);
  switch(lines)
  {
    case 0:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 1:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 2:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 3:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 4:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 5:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 6:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 7:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 8:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 9:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(10,2,"Channel 55-60:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 10:
    term_type(1,2,"Channel 01-06:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(2,2,"Channel 07-12:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(3,2,"Channel 13-18:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(4,2,"Channel 19-24:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(5,2,"Channel 25-30:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(6,2,"Channel 31-36:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(7,2,"Channel 37-42:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(8,2,"Channel 43-48:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(9,2,"Channel 49-54:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(10,2,"Channel 55-60:",14,TERM_WHITE | TERM_BLUE_BG);
    term_type(11,2,"Channel 61-66:",14,TERM_WHITE | TERM_BLUE_BG);
    break;
  }
  break;
}
for(;;)
{
    if(kbhit())
    {
        getch();
        term_cur (3 + (ptr->nchan /6), 0);
        term_restore ();
        term_clear (TERM_CLS_EOS);
        exit(0);
    }
    for(i=0;i<ptr->nchan;i++)
        term_printf(1+i/6,(i%6)*10+15,TERM_BLUE | TERM_CYAN_BG,"%10.3f",ptr->dc[i]);
    term_flush();
sleep(1);
}

}
