/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:20/09/95                       */
/* Program:MDC                                                              */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program monitors the DC values each channel                         */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                       /* shared memory definitions    */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <curses.h>
#include <string.h>

main ()
{
  int      lines;                        /* lines to allocate on screen       */
  int      i;                            /* counter                           */
  int      fd;                                /* return value open share      */
  smod     *ptr;                              /* pointer to shared region     */
  WINDOW *win;							/* curses window for display */
  int xmax, ymax;						/* size of screen display */
  int x, y;
  char *title;

  /*------------------------------------------------------------------------*/
  /* open shared memory region                                              */
  /*------------------------------------------------------------------------*/
  fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
  if(fd== -1) {
    fprintf(stderr,"MDC open failed:%s\n",strerror(errno));
    exit(1);
  }

  /*------------------------------------------------------------------------*/
  /* Compute pointer to shared memory region                                */
  /*------------------------------------------------------------------------*/
  ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
  if((int)ptr == -1) {
    fprintf(stderr,"MDC mmap failed : %s\n",strerror(errno));
    exit(1);
  }

  /* set up the screen - note - if this function fails, the program exits */
  initscr ();
  cbreak ();
  noecho ();
  if (has_colors ())
  {
    start_color ();
    init_pair (1, 1, 3);
    init_pair (2, 2, 0);
    init_pair (3, 7, 1);
  }
  getmaxyx (stdscr, ymax, xmax);

  /* get the number of lines to display and set up the window */
  switch(ptr->unit)
  {
  case 11: break;
  case 21: break;
  case 30: lines=(ptr->nchan-1)/6; title = " DC values in Millivolts Metrabyte 1202 "; break;
  case 36: lines=(ptr->nchan-1)/6; title = " DC values in Millivolts Metrabyte 1800 "; break;
  case 37: break;
  case 53: lines = 0; title = "DC values in Millivolts Nanometrics"; break;
  case 56: break;
  case 63: lines = 0; title = "DC values in Millivolts Earth Data"; break;
  case 96: lines=(ptr->nchan-1)/6; title = " DC values in Millivolts Synthetic Data "; break;
  case 99: lines=0; title = " Test of ILI DC Values "; break;
  }
  win = newwin (lines +3, xmax, 0, 0);
  scrollok (win, FALSE);
  if (has_colors ()) wattron (win, COLOR_PAIR (1));
  for (y=1; y<lines+2; y++)
  {
    for (x=1; x<xmax-1; x++) mvwaddch (win, y, x, ' ');
  }
  if (strstr (termname (), "qnx")) wborder (win, 0, 0, 0, 0, 0, 0, 0, 0);
  else wborder (win, '|', '|', '-', '-', '+', '+', '+', '+');

  /* display the titles */
  if (has_colors ()) wattron (win, COLOR_PAIR (2));
  else wstandout (win);
  mvwprintw(win, 0, 20, title);
  if (has_colors ()) wattron (win, COLOR_PAIR (3));
  else wstandend (win);
  switch(lines)
  {
  case 10: mvwprintw (win,11,2,"Channel 61-66:" );
  case 9: mvwprintw (win,10,2,"Channel 55-60:" );
  case 8: mvwprintw (win,9,2,"Channel 49-54:" );
  case 7: mvwprintw (win,8,2,"Channel 43-48:" );
  case 6: mvwprintw (win,7,2,"Channel 37-42:" );
  case 5: mvwprintw (win,6,2,"Channel 31-36:" );
  case 4: mvwprintw (win,5,2,"Channel 25-30:" );
  case 3: mvwprintw (win,4,2,"Channel 19-24:");
  case 2: mvwprintw (win,3,2,"Channel 13-18:");
  case 1: mvwprintw(win, 2, 2, "Channel 07-12:");
  case 0: mvwprintw(win, 1, 2, "Channel 01-06:");
  }

  /* display the data */
  if (has_colors ()) wattron (win, COLOR_PAIR (1));
  for(;;)
  {
    if(kbhit())
    {
      getch();
	  wstandend (win);
      wclear (win);
      wrefresh (win);
      endwin ();
      exit(0);
    }
    for(i=0;i<ptr->nchan;i++)
        mvwprintw(win, 1+i/6, (i%6)*10+15, "%10.3f", ptr->dc[i]);
    wrefresh (win);
    sleep(1);
  }
}
