/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:17/02/98                       */
/* Program:MIN_MAX                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program is started by EXT to find the current time span in the       */
/* ringbuffer system.                                                       */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <dirent.h>
#include <string.h>

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
char     names[5000][32];              /* storage all ringbuffer names      */
char     ringb[5000][6];
char     sts[5000][2];
double   totsec[5000];
int      reset[50];
int      nrec=0;
int      reco=0;
int      nfin=0;
int      nrst=0;
int      indx=0;
int      start,stopp;
int      cnt;
main(argc,argv)
int argc;
char **argv;
{
FILE     *idx;
DIR      *dirp;                        /* pointer to directory              */
struct   dirent *direntp;              /* structure for directory structure */
int      ind_min;                      /* index oldest ringbuffer           */
int      ind_max;                      /* index newest ringbuffer           */
double   MSECS;                        /* total seconds ringbuffer filename */
double   mint;                         /* smallest value indicating rngfile */
double   maxt;                         /* largest value indicating rngfile  */
double   save;
float    SEC;                          /* seconds floating point            */
int      YR,DOY,MTH,DAY,HR,MIN,SECL;   /* ringbuffer file date              */
char     idx_log[400];
int      hu;
int      lk;                           /* index                             */
int      bytes;                        /* return value after read           */
char     tid[13];                      /* string for uptime                 */
char     minutes[10];                  /* storage for minutes from uptime   */
char     hours[10];                    /* storage for hours from uptime     */
char     days[10];                     /* storage for days from uptime      */
char     name[50];                     /* storage file name                 */
char     name1[50];                    /* storage oldest file name          */
char     fil_min[50];                  /* storage oldest file name          */
char     fil_max[50];                  /* storage newest file name          */
char     syscall[100];                 /* string to hold system command     */
char     dum;
char     arg;
int      no_files;                     /* number of files in directory      */
FILE     *up;                          /* pointer to file                   */
int      min;                          /* minutes uptime                    */
int      hr;                           /* hours uptime                      */
int      dg;                           /* days uptime                       */
int      dur;                          /* uptime in minutes                 */
int      fr;
int      i;                            /* index                             */
int      k;
int      opt;
char     num[10];
if(argc != 2)
{
    opt=1;
}else{
    arg=*argv[1];
strcpy(num,argv[1]);
sscanf(num,"%d",&opt);
}
//printf("OPT=%d\n",opt);
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MIN_MAX_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MIN_MAX_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
no_files=0;
/*--------------------------------------------------------------------------*/
/* check if system has been up at least one ringbuffer                      */
/*--------------------------------------------------------------------------*/
up=fopen("/home/buffer/uptime", "r");  /* open directory file               */
if(up == NULL)
{
    sprintf(ptr->err_msg,"MIN_MAX: Error opening /home/buffer/uptime");
    errlog(ptr);
    sprintf(ptr->err_msg,"MIN_MAX: /home/buffer/uptime does not exist.");
    errlog(ptr);
    sprintf(ptr->err_msg,"MIN_MAX: No data recorded so far.");
    errlog(ptr);
    exit(0);
}
bytes=fread(tid,9,1,up);               /* read uptime                       */
if(bytes==0)
{
    sprintf(ptr->err_msg,"MIN_MAX: Error reading /home/buffer/uptime");
    errlog(ptr);
    exit(0);
}
fclose(up);
minutes[0]=tid[7];                     /* extract minutes so far            */
minutes[1]=tid[8];
minutes[2]=tid[9];
sscanf(minutes,"%2d",&min);            /* extract minutes                   */

hours[0]=tid[4];                       /* extract hours so far              */
hours[1]=tid[5];
hours[2]=tid[9];
sscanf(hours,"%2d",&hr);               /* extract hours                     */

days[0]=tid[0];                        /* extract days so far               */
days[1]=tid[1];
days[2]=tid[2];
days[3]=tid[9];
sscanf(days,"%2d",&dg);                /* extract days                      */
dur=min+hr*60+dg*1440;
//printf("%d %d %d %d\n",dg,hr,min,dur);
if(dur < ptr->nmin_per_ring[0])        /* up less than one ringbuffer ?     */
{
    printf("MIN_MAX: SEISLOG up less than 1 ringbuffer\n");
    sprintf(ptr->err_msg,"MIN_MAX: SEISLOG up less than 1 ringbuffer");
    errlog(ptr);
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Read all filenames in ringbuffer directory and find the oldest           */
/*--------------------------------------------------------------------------*/
idx=fopen("/home/buffer/idx_log","r+");    

for(i=0;i<ptr->nringbuf[0];i++)
{
    fseek(idx,i*320,SEEK_SET);
    fread(idx_log,320,1,idx);
    for(lk=0;lk<29;lk++)
        names[i][lk]=idx_log[lk+19];
    names[i][29]='\0';
    sts[i][0]=idx_log[87];
    sts[i][1]=idx_log[167];
    for(lk=0;lk<5;lk++)
        ringb[i][lk]=idx_log[lk+13];
    ringb[i][5]='\0';
    sscanf(names[i],"%2d%2d%c%2d%c%2d%c%2d%2d%c%2d",&hu,&YR,&dum,&MTH,&dum,&DAY,&dum,&HR,&MIN,&dum,&SECL);
    SEC=(float)SECL;
    TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
    totsec[i]=MSECS;
//    printf("%5d %c %c %s %s %10.1f\n",i,sts[i][0],sts[i][1],names[i],ringb[i],totsec[i]);
//getch();
}
fclose(idx);
mint=totsec[0];
for(i=0;i<ptr->nringbuf[0];i++)
{
    if(sts[i][0]=='2')                 /* finished recording ?              */
    {
        if(totsec[i] < mint)
        {
            mint=totsec[i];
            start=i;
        }
    }
}
maxt=totsec[0];
for(i=0;i<ptr->nringbuf[0];i++)
{
    if(sts[i][0]=='2')                 /* finished recording ?              */
    {
        if(totsec[i] > maxt)
        {
            maxt=totsec[i];
            stopp=i;
        }
    }
}

printf("Time window(s) in ringbuffer system:\n");
printf("\n");
printf("From: %s %s\n",names[start],ringb[start]);
printf("To  : %s %s\n",names[stopp],ringb[stopp]);
printf("\n");
cnt=0;
for(i=0;i<ptr->nringbuf[0];i++)
{
    if(sts[i][0]=='2' && sts[i][1]=='3') /* RESET                           */
    {
        printf("RESET at: %s %s\n",names[i],ringb[i]);
        cnt++;
    }
}
if(cnt==0)
    printf("Ringbuffer system is continous.\n");

if(opt==2)
{
    for(i=0;i<ptr->nringbuf[0];i++)
    {
        if(sts[i][0]=='2')                 /* finished recording ?              */
        {
            sprintf(syscall,"ln -fs /home/buffer/%s /home/rngbuf/%s\n",ringb[i],names[i]);
            system(syscall);
        }
    }
}
}



