/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:05/02/95                       */
/* Program:MON                                                              */
/*                                                                          */
/* Last update:                                                             */
/* 29/05/00 runs every second not synchronised to events. will not hang     */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program monitors SEISLOG variables and const. in the Data module     */
/* continously when the data-acquisition is active. It also list out        */
/* the most important parameters from the current parameterfile.            */
/* If the routine is active when an event occurs, it will signal with       */
/* "bell" on the terminal.                                                  */
/* The routine assumes a VT100 compatible terminal.                         */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <sys/qnxterm.h>
#include <sys/name.h>
#include <sys/kernel.h>
int      bytes;
char     statuscode;
int      con_ser;                      /* console or serial line            */
int      i;                            /* Counter                           */
int      lc;                           /* Counter                           */
int      ll;                           /* Counter                           */
int      nrsta;                        /* Counter for printed sta's         */
int      nrlta;                        /* Counter for printed lta's         */
int      val;                          /* Help variable                     */
int      id;                           /* Return value link to pls_event    */
int      pls_id;                       /* Id for pls_event                  */
int      set;                          /* parameter set to monitor          */
char     arg;                          /* argument starting MON             */
char     as[32];                       /* Time string                       */
int      mon_id;
int      buf_pipe_id;
char     rep_msg[10];
char     rec_msg[10];
char     snd_msg[10];
unsigned int cop;

smod     *ptr;
int      fd;
/*--------------------------------------------------------------------------*/
/* Sett error message                                                       */
/*--------------------------------------------------------------------------*/
Pos_Err()
{
/*    gotoxy(65,0);*/
/*    highvideo();*/
/*    blink();*/
/*    touts("No Time/No SYNC");*/
/*    lowvideo();*/
}
main(argc,argv)
int      argc;
char     **argv;
{
pid_t    proxy;
unsigned msg;
if(argc != 2)
{
    set=0;                             /* Monitor first parameter set       */
}else{
    arg=*argv[1];
    switch(arg)
    {
        case '0':
        set=0;
        break;
        case '1':
        set=0;
        break;
        case '2':
        set=1;
        break;
        case '3':
        set=2;
        break;
        case '4':
        set=3;
        break;
        case '5':
        set=4;
        break;
    }
}
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MON_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MON_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
con_ser=line_typ();
//printf("TYPE %d\n",con_ser);
//if(con_ser ==1)
//    system("termdef -s");
//mon_id=qnx_name_attach(0,"mon");
//if(mon_id == -1)
//{
//    printf("MON attach failed\n");
//    exit(1);
//}
buf_pipe_id=qnx_name_locate(0,"buf_pip",0,&cop);
if(buf_pipe_id == -1)
{
    printf("SEISLOG is not running\n");
    exit(1);
}
proxy=qnx_proxy_attach(0,0,0,-1);
ptr->mon_proxy=proxy;

if(term_load() == -1){
    fprintf(stderr,"term_load error,errno=%d\n",errno);
    exit(1);
}
term_video_on();
if(term_state.qnx_term)
    fwrite("\033!71\033@71",8,1,stdout);
term_clear(0);
term_box_fill(0,0,80,15,TERM_BLUE | TERM_CYAN_BG,1,' ');
Head_1();                              /* Print out header one              */
term_flush();
Sys_Cons();
term_flush();


/*--------------------------------------------------------------------------*/
/* Enter infinite loop.                                                     */
/* Updates screen continously                                               */
/*--------------------------------------------------------------------------*/
for (;;)
{
if(kbhit())
{
    getch();
	term_cur (22, 0);
	term_restore ();
    term_clear(TERM_CLS_EOS);
    ptr->mon_proxy=0;
    exit(0);
}
//    buf_pipe_id=Receive(0,&msg,sizeof(msg));
sleep(1);
    if(ptr -> detflg[set] == 1 )       /* Check for event and send "bell"   */
    {
        putchar(0x07);
        fflush(stdout);
    }
Head_1();                              /* Print out header one              */
term_flush();
LstUpd();                              /* Print last update time            */
term_flush();
Time();                                /* Print out time                    */
term_flush();
Sys_Var();                             /* Print out system variables        */
term_flush();


switch(ptr->unit)
{
    case 10:                           /* UNIT=10, GBV Lowcost              */
    case 30:                           /* UNIT=63, Earth Data ED24          */
    case 31:                           /* UNIT=63, Earth Data ED24          */
    case 32:                           /* UNIT=63, Earth Data ED24          */
    case 33:                           /* UNIT=63, Earth Data ED24          */
    case 34:
    case 35:
    case 37:                          /* UNIT= 3, RBO_3                    */
    case 38:                           /* UNIT=38 Ramon Ortiz               */
    case 39:
    case 36:                           /* UNIT=36, VADC36                   */
    case 40:                           /* UNIT=40, Nanometrics HRD          */
    case 41:                           /* UNIT=40, Nanometrics HRD          */
    case 42:                           /* UNIT=40, Nanometrics HRD          */
    case 53:                           /* UNIT=53, Nanometrics RD3          */
    case 56:                           /* UNIT=56, Nanometrics RD6          */
    case 63:                           /* UNIT=63, Earth Data ED24          */
    case 64:                           /* UNIT=63, Earth Data ED24          */
    case 65:                           /* UNIT=63, Earth Data ED24          */
    case 66:                           /* UNIT=63, Earth Data ED24          */
    case 70:                           /* UNIT=70, RA+CA 3 ch.24 bits 50Hz  */
    case 96:                           /* UNIT=96, Synthetic Data           */
    case 97:                           /* UNIT=56, Nanometrics RD6          */
    case 99:                           /* UNIT=99, 24 bit ILI   *DJS        */
    Line_STA2();                       /* Print out current sta's           */
//term_flush();
    Line_LTA2();                       /* Print out current lta's           */
//term_flush();
      break;

}

Line_CHAFLG();                        /* Print out marked detection channels*/

switch(ptr->unit)
{
    case 10:                           /* UNIT=10 GBV Lowcost               */
    case 30:                           /* UNIT=63 Earth Data ED24           */
    case 31:                           /* UNIT=63 Earth Data ED24           */
    case 32:                           /* UNIT=63 Earth Data ED24           */
    case 33:                           /* UNIT=63 Earth Data ED24           */
    case 34:
    case 35:
    case 37:                           /* UNIT= 3 RBO_3                     */
   case 38:                           /* UNIT=38 Ramon Ortiz               */
    case 39:
    case 36:                           /* UNIT=36 VADC36                    */
    case 40:                           /* UNIT=40 Nanometrics HRD           */
    case 41:                           /* UNIT=40, Nanometrics HRD          */
    case 42:                           /* UNIT=40, Nanometrics HRD          */
    case 53:                           /* UNIT=53 Nanometrics RD3           */
    case 56:                           /* UNIT=56 Nanometrics RD6           */
    case 63:                           /* UNIT=63 Earth Data ED24           */
    case 64:                           /* UNIT=63, Earth Data ED24          */
    case 65:                           /* UNIT=63 Earth Data ED24           */
    case 66:                           /* UNIT=63 Earth Data ED24           */
    case 70:                           /* UNIT=70 RO+CA 24 bits 3 ch. 50Hz  */
    case 96:                           /* UNIT=96 Synthetic Data            */
    case 97:                           /* UNIT=56 Nanometrics RD6           */
    case 99:                           /* UNIT=99 24 bit ILI  *DJS          */
    Line_SAMPLES2();                   /* Print out first 16 samples        */
//term_flush();
    break;
}
}
/*--------------------------------------------------------------------------*/
/* Unlink from parameter module                                             */
/*--------------------------------------------------------------------------*/
}
LstUpd()
{
int pek;
term_type(2,1,"Last update:",12,TERM_BLUE | TERM_CYAN_BG);
for(pek=0;pek<27;pek++)
term_printf(2,pek+13,TERM_BLUE | TERM_CYAN_BG,"%c",ptr->Timsav[pek]);
//term_printf (2,13,TERM_BLUE | TERM_CYAN_BG,"%27s",ptr->Timsav);

/*--------------------------------------------------------------------------*/
/* check all different clocktypes                                           */
/*--------------------------------------------------------------------------*/
switch(ptr->clktyp)
{
    case 0:
    break;
    case 1:                            /*   ***** GARMIN 35-HVS No PPS **** */
    switch(ptr->timcpy[27])
    {
        case 'S':
        term_type(2,45,"  GPS Garmin  : No sync received  ",34,TERM_NORMAL);
        break;
        case 'F':
        term_type(2,45,"  GPS Garmin  : Sync found        ",34,TERM_NORMAL);
        break;
        case 'R':
        term_type(2,45,"  GPS Garmin  : ERROR reading     ",34,TERM_NORMAL);
        break;
        case 'M':
        term_type(2,45,"  GPS Garmin  : TIMEOUT reading   ",34,TERM_NORMAL);
        break;
        case 'C':
        term_type(2,45,"  GPS Garmin  : Receiving time    ",34,TERM_NORMAL);
        break;
        case 'A':
        term_type(2,45,"  GPS Garmin  : Not accepting time",34,TERM_NORMAL);
        break;
        case 'E':
        term_type(2,45,"  GPS Garmin  : Time updated      ",34,TERM_NORMAL);
        break;
    }

    case 2:
    switch(ptr->timcpy[27])
    {
        case 'S':
        term_type(2,45,"  GPS Garmin  : No sync received  ",34,TERM_NORMAL);
        break;
        case 'F':
        term_type(2,45,"  GPS Garmin  : Sync found        ",34,TERM_NORMAL);
        break;
        case 'R':
        term_type(2,45,"  GPS Garmin  : ERROR reading     ",34,TERM_NORMAL);
        break;
        case 'M':
        term_type(2,45,"  GPS Garmin  : TIMEOUT reading   ",34,TERM_NORMAL);
        break;
        case 'C':
        term_type(2,45,"  GPS Garmin  : Receiving time    ",34,TERM_NORMAL);
        break;
        case 'A':
        term_type(2,45,"  GPS Garmin  : Not accepting time",34,TERM_NORMAL);
        break;
        case 'E':
        term_type(2,45,"  GPS Garmin  : Time updated      ",34,TERM_NORMAL);
        break;
    }

    break;
    case 3:
    switch(ptr->timcpy[27])
    {
        case 'S':
        term_type(2,45,"  GPS Garmin  : No sync received  ",34,TERM_NORMAL);
        break;
        case 'F':
        term_type(2,45,"  GPS Garmin  : Sync found        ",34,TERM_NORMAL);
        break;
        case 'R':
        term_type(2,45,"  GPS Garmin  : ERROR reading     ",34,TERM_NORMAL);
        break;
        case 'M':
        term_type(2,45,"  GPS Garmin  : TIMEOUT reading   ",34,TERM_NORMAL);
        break;
        case 'C':
        term_type(2,45,"  GPS Garmin  : Receiving time    ",34,TERM_NORMAL);
        break;
        case 'A':
        term_type(2,45,"  GPS Garmin  : Not accepting time",34,TERM_NORMAL);
        break;
        case 'E':
        term_type(2,45,"  GPS Garmin  : Time updated      ",34,TERM_NORMAL);
        break;
    }
    break;
    case 4:
    switch(ptr->timcpy[27])
    {
        case 'S':
        term_type(2,45,"  GPS Garmin  : No sync received  ",34,TERM_NORMAL);
        break;
        case 'F':
        term_type(2,45,"  GPS Garmin  : Sync found        ",34,TERM_NORMAL);
        break;
        case 'R':
        term_type(2,45,"  GPS Garmin  : ERROR reading     ",34,TERM_NORMAL);
        break;
        case 'M':
        term_type(2,45,"  GPS Garmin  : TIMEOUT reading   ",34,TERM_NORMAL);
        break;
        case 'C':
        term_type(2,45,"  GPS Garmin  : Receiving time    ",34,TERM_NORMAL);
        break;
        case 'A':
        term_type(2,45,"  GPS Garmin  : Not accepting time",34,TERM_NORMAL);
        break;
        case 'E':
        term_type(2,45,"  GPS Garmin  : Time updated      ",34,TERM_NORMAL);
        break;
    }
    break;

    break;
}
}
Time()
{
int pek;
term_type(3,1,"Buffer time:",12,TERM_BLUE | TERM_CYAN_BG);
for(pek=0;pek<31;pek++)
{
	if (isprint(ptr->timcpy[pek]))
    	term_printf(3,pek+13,TERM_BLUE | TERM_CYAN_BG,"%c",ptr->timcpy[pek]);
    else
		term_type(3,pek+13," ",1,TERM_BLUE | TERM_CYAN_BG);
}
//term_printf (3,13,TERM_BLUE | TERM_CYAN_BG,"%30s",&ptr->timcpy[2]);

term_type(3,47,"Rec time:",9,TERM_BLUE | TERM_CYAN_BG);
term_printf(3,56,TERM_BLUE | TERM_CYAN_BG,"%8d",ptr->noktime);

term_type(3,65,"rng file:",9,TERM_BLUE | TERM_CYAN_BG);
term_printf(3,74,TERM_BLUE | TERM_CYAN_BG,"%5d",ptr->ringidx[0]);
}
Line_STA1()
{
nrsta=0;
/*printf("sta:\n");*/          /* Current STA for 12 first detection channels   */
printf("sta:");
for(i=0;i<ptr->nchan;i++)
{
    if(nrsta <= 12)
    {
        if(ptr -> chaflg[set][i] != 0)
        {
            printf(" %4.1f",ptr -> sta[set][i]);
            nrsta=nrsta+1;
        }
}
}
/*printf("\n");*/

}

Line_STA2()
{
nrsta=0;
term_type(7,1,"sta:",4,TERM_BLUE | TERM_CYAN_BG);

for(i=0;i<ptr->nchan;i++)
{
    if(nrsta <= 6)
    {
        if(ptr -> chaflg[set][i] != 0)
        {
 term_printf(7,nrsta*10+5,TERM_BLUE | TERM_CYAN_BG,"%10.1f",ptr->sta[set][i]);

            nrsta=nrsta+1;
        }
}
}
}

Line_LTA1()
{
nrlta=0;
/*printf("lta:\n");*/          /* Current LTA for 12 first detection channels   */
printf("lta:");
for(i=0;i<ptr->nchan;i++)
{
    if(nrlta <= 12)
    {
        if(ptr -> chaflg[set][i] != 0)
        {
            printf(" %4.1f",ptr -> lta[set][i]);
            nrlta=nrlta+1;
        }
}
}
/*printf("\n");*/
}

Line_LTA2()
{
nrlta=0;
term_type(8,1,"lta:",4,TERM_BLUE | TERM_CYAN_BG);

for(i=0;i<ptr->nchan;i++)
{
    if(nrlta <= 6)
    {
        if(ptr -> chaflg[set][i] != 0)
        {
 term_printf(8,nrlta*10+5,TERM_BLUE | TERM_CYAN_BG,"%10.1f",ptr->lta[set][i]);

/*            printf(" %10.1f",ptr -> lta[set][i]);*/
            nrlta=nrlta+1;
        }
}
}
}

Sys_Var()
{
/* Print out intresting variables from parameter module and datamodule      */
/*term_cur(3,0);*/
term_type(4,1,"detflg:",7,TERM_BLUE | TERM_CYAN_BG);
term_type(4,11,"ipoint:",7,TERM_BLUE | TERM_CYAN_BG);
term_type(4,21,"ndet  :",7,TERM_BLUE | TERM_CYAN_BG);
term_type(4,32,"preblk:",7,TERM_BLUE | TERM_CYAN_BG);
term_type(4,44,"To CATLOG  :",12,TERM_BLUE | TERM_CYAN_BG);
term_type(4,65,"Last DPX:",9,TERM_BLUE | TERM_CYAN_BG);

if(ptr->detflg[set] != 0)
{
    term_printf(4,9,TERM_GREEN | TERM_BLACK_BG | TERM_BLINK,"%1d",ptr->detflg[set]);
}else{
    term_printf(4,9,TERM_BLUE | TERM_CYAN_BG,"%1d",ptr->detflg[set]);
}
term_printf(4,18,TERM_BLUE | TERM_CYAN_BG,"%2d",ptr->ipoint);
term_printf(4,29,TERM_BLUE | TERM_CYAN_BG,"%2d",ptr->ndet[set]);
term_printf(4,40,TERM_BLUE | TERM_CYAN_BG,"%2d",ptr->preblk[set]);
term_printf(4,56,TERM_BLUE | TERM_CYAN_BG,"%8d",ptr->ntick);
term_printf(4,74,TERM_BLUE | TERM_CYAN_BG,"%5d",ptr->last_dpx);

term_type(5,1,"recrd :",7,TERM_BLUE | TERM_CYAN_BG);
term_type(5,11,"queue :",7,TERM_BLUE | TERM_CYAN_BG);
term_type(5,21,"ndetw :",7,TERM_BLUE | TERM_CYAN_BG);
term_type(5,32,"postbk:",7,TERM_BLUE | TERM_CYAN_BG);
term_type(5,44,"iblk       :",12,TERM_BLUE | TERM_CYAN_BG);
term_type(5,65,"duration:",9,TERM_BLUE | TERM_CYAN_BG);

if(ptr->recrd[set] != 0)
{
    term_printf(5,9,TERM_GREEN | TERM_BLACK_BG | TERM_BLINK,"%1d",ptr->recrd[set]);
}else{
    term_printf(5,9,TERM_BLUE | TERM_CYAN_BG,"%1d",ptr->recrd[set]);
}
if(ptr->queue+1 != 0)
{
    term_printf(5,18,TERM_GREEN | TERM_BLACK_BG | TERM_BLINK,"%2d",ptr->queue+1);
}else{
    term_printf(5,18,TERM_BLUE | TERM_CYAN_BG,"%2d",ptr->queue+1);
}
term_printf(5,29,TERM_BLUE | TERM_CYAN_BG,"%2d",ptr->ndetw[set]);
term_printf(5,40,TERM_BLUE | TERM_CYAN_BG,"%2d",ptr->postbk[set]);
term_printf(5,56,TERM_BLUE | TERM_CYAN_BG,"%8d",ptr->iblk);
term_printf(5,74,TERM_BLUE | TERM_CYAN_BG,"%5d",ptr->pdur[ptr->ipoint]);

}
Line_CHAFLG()
{
int xx,yy;
term_type(10,1,"chaflg:",7,TERM_BLUE | TERM_CYAN_BG);

xx=7;
yy=9;
for(i=0;i<16;i++)
{
    if(ptr->singdt[set][i]==1)
    {
term_printf(10,i+8,TERM_GREEN | TERM_BLACK_BG | TERM_BLINK,"%1d",ptr->chaflg[set][i]);
        xx=xx+1;
    }else{
term_printf(10,i+8,TERM_BLUE | TERM_CYAN_BG ,"%1d",ptr->chaflg[set][i]);
        xx=xx+1;
    }
}
/*term_type(10,i+8," ",1,TERM_NORMAL | TERM_WHITE_BG);*/

xx=xx+1;
for(i=16;i<32;i++)
{
    if(ptr->singdt[set][i]==1)
    {
term_printf(10,i+9,TERM_GREEN | TERM_BLACK_BG | TERM_BLINK,"%1d",ptr->chaflg[set][i]);
        xx=xx+1;
    }else{
term_printf(10,i+9,TERM_BLUE | TERM_CYAN_BG ,"%1d",ptr->chaflg[set][i]);

        xx=xx+1;
    }
}
xx=xx+1;
for(i=32;i<48;i++)
{
    if(ptr->singdt[set][i]==1)
    {
term_printf(10,i+10,TERM_GREEN | TERM_BLACK_BG | TERM_BLINK,"%1d",ptr->chaflg[set][i]);

        xx=xx+1;
    }else{
term_printf(10,i+10,TERM_BLUE | TERM_CYAN_BG,"%1d",ptr->chaflg[set][i]);

        xx=xx+1;
    }
}
xx=xx+1;
for(i=48;i<64;i++)
{
    if(ptr->singdt[set][i]==1)
    {
term_printf(10,i+11,TERM_GREEN | TERM_BLACK_BG | TERM_BLINK,"%1d",ptr->chaflg[set][i]);

        xx=xx+1;
    }else{
term_printf(10,i+11,TERM_BLUE | TERM_CYAN_BG ,"%1d",ptr->chaflg[set][i]);

        xx=xx+1;
    }
}
/*Navigation();*/
/*printf("\n");*/
}

Head_1()
{
term_type(1,1,"Network:",8,TERM_BLUE | TERM_CYAN_BG);
term_printf(1,9,TERM_BLUE | TERM_CYAN_BG,"%s ",ptr->name);

switch(ptr->clktyp)
{
    case 0:
    term_type(1,14," CPU clock only.    ",20,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 1:
    term_type(1,14," Garmin 35HVS No PPS",20,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 2:
    term_type(1,14," Garmin 35HVS PPS   ",20,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 4:
    term_type(1,14," RCC MSF or RCC GPS ",20,TERM_WHITE | TERM_BLUE_BG);
    break;
    case 8:
    term_type(1,14," Timing by digitizer",20,TERM_WHITE | TERM_BLUE_BG);
    break;
}
term_type(1,47,"GPS     :",9,TERM_BLUE | TERM_CYAN_BG);
    term_printf(1,56,TERM_NORMAL,"%4d%% ",ptr->percent_gps);
term_type(1,62,"PPS: ",5,TERM_BLUE | TERM_CYAN_BG);
    term_printf(1,69,TERM_NORMAL,"%10d ",ptr->average1);
}

Line_SAMPLES1()
{
unsigned int dob_val;
short s_val1,s_val2;
for(i=0;i<32;i++)
{
    dob_val=ptr->ibuf[ptr->index][i+HEADER];
    s_val1=dob_val>>16;
    s_val2=dob_val;
    switch(i)
    {

        case 4:
/*        printf("\n");*/
        break;
        case 8:
/*        printf("\n");*/
        break;
        case 12:
/*        printf("\n");*/
        break;
        case 16:
/*        printf("\n");*/
        break;
        case 20:
/*        printf("\n");*/
        break;
        case 24:
/*        printf("\n");*/
        break;
        case 28:
/*        printf("\n");*/
        break;
    }
    printf("%8d",s_val1);
    printf("%8d",s_val2);
}
/*printf("\n");*/
}

Line_SAMPLES2()
{
term_type(12,1,"Samples:",8,TERM_BLUE | TERM_CYAN_BG);

for(i=0;i<ptr->nchan;i++)
{
    if(i<7)
        term_printf(13,i*10+1,TERM_BLUE | TERM_CYAN_BG,"%10d",ptr->ibuf[ptr->index][i+HEADER]);
}    

}

Sys_Cons()
{   /* return from here is ok */
term_box_fill(16,0,80,6,TERM_CYAN | TERM_BLACK_BG,1,' ');
/*term_color(TERM_YELLOW | TERM_BLUE_BG);*/
term_type(17,2,"ParSet:",7,TERM_NORMAL);
//fflush (stdout);
term_type(17,12,"Ext/Int:",8,TERM_NORMAL);
//fflush (stdout);
term_type(17,27,"ArPrWnd:",8,TERM_NORMAL);
//fflush (stdout);
term_type(17,40,"Min_Rng:",8,TERM_NORMAL);
//fflush (stdout);
term_type(17,52,"TimCor:",7,TERM_NORMAL);
//fflush (stdout);
term_printf(17,9,TERM_NORMAL,"%2d",ptr->active);
//fflush (stdout);
term_printf(17,20,TERM_NORMAL,"%6d",ptr->intclk);
//fflush (stdout);
term_printf(17,35,TERM_NORMAL,"%3d",ptr->premax[set]);
//fflush (stdout);
term_printf(17,48,TERM_NORMAL,"%3d",ptr->maxblk/60);
//fflush (stdout);
term_printf(17,60,TERM_NORMAL,"%8.3f",ptr->delay);
//fflush (stdout);
/* return from here is ok */

term_type(18,2,"Chan  :",7,TERM_NORMAL);
//fflush (stdout);
term_type(18,12,"MaxRec :",8,TERM_NORMAL);
//fflush (stdout);
term_type(18,27,"PreEvt :",8,TERM_NORMAL);
//fflush (stdout);
term_type(18,40,"SmpBfCh:",8,TERM_NORMAL);
//fflush (stdout);
term_type(18,52,"S-Rate:",7,TERM_NORMAL);
//fflush (stdout);
term_printf(18,9,TERM_NORMAL,"%2d",ptr->nchan);
//fflush (stdout);
term_printf(18,20,TERM_NORMAL,"%6d",ptr->maxbuf[set]);
//fflush (stdout);
term_printf(18,35,TERM_NORMAL,"%3d",ptr->prevnt[set]);
//fflush (stdout);
term_printf(18,48,TERM_NORMAL,"%3d",ptr->nsamp);
//fflush (stdout);
term_printf(18,60,TERM_NORMAL,"%8.3f",ptr->srate);
//fflush (stdout);
/* return from here is ok */

term_type(19,2,"ClkTyp:",7,TERM_NORMAL);
//fflush (stdout);
term_type(19,12,"BufSiz :",8,TERM_NORMAL);
//fflush (stdout);
term_type(19,27,"PstEvt :",8,TERM_NORMAL);
//fflush (stdout);
term_type(19,40,"CurPset:",8,TERM_NORMAL);
//fflush (stdout);
term_type(19,52,"CPUcor:",7,TERM_NORMAL);
//fflush (stdout);
term_printf(19,9,TERM_NORMAL,"%2d",ptr->clktyp);
//fflush (stdout);
term_printf(19,20,TERM_NORMAL,"%6d",ptr->bufsiz);
//fflush (stdout);
term_printf(19,35,TERM_NORMAL,"%3d",ptr->postmx[set]);
//fflush (stdout);
term_printf(19,48,TERM_NORMAL,"%3d",set+1);
//fflush (stdout);
term_printf(19,60,TERM_NORMAL,"%8.5f",ptr->timerr);
//fflush (stdout);
/* return from here is ok */

//fflush (stdout);
term_type(20,2,"Unit  :",7,TERM_NORMAL);
//fflush (stdout);
term_type(20,12,"Buf_rng:",8,TERM_NORMAL);
//fflush (stdout);
term_type(20,27,"MinTrg :",8,TERM_NORMAL);
//fflush (stdout);
term_type(20,40,"Min/Rng:",8,TERM_NORMAL);
//fflush (stdout);
term_type(20,52,"Minpip:",7,TERM_NORMAL);
//fflush (stdout);
/* return from here is not ok */

term_printf(20,9,TERM_NORMAL,"%2d",ptr->unit);
//fflush (stdout);
term_printf(20,20,TERM_NORMAL,"%6d",ptr->maxblk);
//fflush (stdout);
term_printf(20,35,TERM_NORMAL,"%3d",ptr->mintrg[set]);
//fflush (stdout);
term_printf(20,48,TERM_NORMAL,"%3d",ptr->nmin_per_ring[0]);
//fflush (stdout);
term_printf(20,60,TERM_NORMAL,"%8d",PIPSIZ*4/ptr->reclength);
//fflush (stdout);
/* return from here is not ok */
}


