/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.8   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:05/02/95                       */
/* Program:MON                                                              */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program monitors SEISLOG variables and const. in the Data module     */
/* continously when the data-acquisition is active. It also list out        */
/* the most important parameters from the current parameterfile.            */
/* If the routine is active when an event occurs, it will signal with       */
/* "bell" on the terminal.                                                  */
/* The routine assumes a VT100 compatible terminal.                         */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <curses.h>
#include <sys/name.h>
#include <sys/kernel.h>
int      bytes;
char     statuscode;
int      i;                            /* Counter                           */
int      lc;                           /* Counter                           */
int      ll;                           /* Counter                           */
int      nrsta;                        /* Counter for printed sta's         */
int      nrlta;                        /* Counter for printed lta's         */
int      val;                          /* Help variable                     */
int      id;                           /* Return value link to pls_event    */
int      pls_id;                       /* Id for pls_event                  */
int      set;                          /* parameter set to monitor          */
char     arg;                          /* argument starting MON             */
char     as[32];                       /* Time string                       */
int      mon_id;
int      buf_pipe_id;
char     rep_msg[10];
char     rec_msg[10];
char     snd_msg[10];
unsigned int cop;
int xmax, ymax;					/* screen size */
WINDOW *win_main, *win_sys;		/* curses windows */

smod     *ptr;
int      fd;


main(argc,argv)
int      argc;
char     **argv;
{

  pid_t    proxy;
  unsigned msg;
  int x, y;

  if(argc != 2) {
    set=0;                             /* Monitor first parameter set       */
  } else {
    arg=*argv[1];
    switch(arg)
    {
    case '0':
      set=0;
      break;
    case '1':
      set=0;
      break;
    case '2':
      set=1;
      break;
    case '3':
      set=2;
      break;
    case '4':
      set=3;
      break;
    case '5':
      set=4;
      break;
    }
  }

  /*------------------------------------------------------------------------*/
  /* open shared memory region                                              */
  /*------------------------------------------------------------------------*/
  fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
  if(fd== -1)
  {
    fprintf(stderr,"MON_01 open failed:%s\n",strerror(errno));
    exit(1);
  }

  /*------------------------------------------------------------------------*/
  /* set size of memory region                                              */
  /*------------------------------------------------------------------------*/
  ltrunc(fd,sizeof(smod),SEEK_SET);

  /*------------------------------------------------------------------------*/
  /* Compute pointer to shared memory region                                */
  /*------------------------------------------------------------------------*/
  ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
  if((int)ptr == -1)
  {
    fprintf(stderr,"MON_02 mmap failed : %s\n",strerror(errno));
    exit(1);
  }

  mon_id=qnx_name_attach(0,"mon");
  if(mon_id == -1)
  {
    printf("MON attach failed\n");
    exit(1);
  }
  buf_pipe_id=qnx_name_locate(0,"buf_pipe",0,&cop);

  proxy=qnx_proxy_attach(0,0,0,-1);
  ptr->mon_proxy=proxy;

  /* set up the screen - note - if this function fails, the program exits */
  initscr ();
  cbreak ();
  noecho ();
  if (has_colors ())
  {
    start_color ();
    init_pair (1, 1, 3);	/* COLOR_PAIR(1): blue on cyan */
    init_pair (2, 2, 0);	/* COLOR_PAIR(2): green on black */
    init_pair (3, 7, 1);	/* COLOR_PAIR(3): white on blue */
    init_pair (4, 3, 0);	/* COLOR_PAIR(4): cyan on black */
    init_pair (5, 7, 0);	/* COLOR_PAIR(5): white on black */
  }
  getmaxyx (stdscr, ymax, xmax);
  win_main = newwin (15, xmax, 0, 0);
  win_sys = newwin (6, xmax, 16, 0);
  scrollok (win_main, FALSE);
  scrollok (win_sys, FALSE);
  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  for (y=1; y<15; y++)
  {
    for (x=1; x<xmax-1; x++) mvwaddch (win_main, y, x, ' ');
  }
  if (strstr (termname (), "qnx")) wborder (win_main, 0, 0, 0, 0, 0, 0, 0, 0);
  else wborder (win_main, '|', '|', '-', '-', '+', '+', '+', '+');
  if (has_colors ()) wattron (win_sys, COLOR_PAIR (4));
  if (strstr (termname (), "qnx")) wborder (win_sys, 0, 0, 0, 0, 0, 0, 0, 0);
  else wborder (win_sys, '|', '|', '-', '-', '+', '+', '+', '+');

  Head_1();                              /* Print out header one              */
  Sys_Cons();
  wrefresh (win_main);
  wrefresh (win_sys);


  /*------------------------------------------------------------------------*/
  /* Enter infinite loop.                                                   */
  /* Updates screen continously                                             */
  /*------------------------------------------------------------------------*/
  for (;;)
  {
    if(kbhit())
    {
      getch();
      ptr->mon_proxy=0;
	  wstandend (win_main);
	  wstandend (win_sys);
      wclear (win_main);
      wclear (win_sys);
      wrefresh (win_main);
      wrefresh (win_sys);
	  endwin ();
      exit(0);
    }
    buf_pipe_id=Receive(0,&msg,sizeof(msg));

    if(ptr -> detflg[set] == 1 )       /* Check for event and send "bell"   */
      beep ();
    Head_1();                              /* Print out header one              */
    LstUpd();                              /* Print last update time            */
    Time();                                /* Print out time                    */
    Sys_Var();                             /* Print out system variables        */

	switch(ptr->unit)
	{
    	case 10:                           /* UNIT=10, GBV Lowcost              */
    	case 30:                           /* UNIT=63, Earth Data ED24          */
	    case 31:                           /* UNIT=63, Earth Data ED24          */
    	case 32:                           /* UNIT=63, Earth Data ED24          */
	    case 33:                           /* UNIT=63, Earth Data ED24          */
    	case 37:                           /* UNIT= 3, RBO_3                    */
	    case 36:                           /* UNIT=36, VADC36                   */
    	case 40:                           /* UNIT=40, Nanometrics HRD          */
	    case 53:                           /* UNIT=53, Nanometrics RD3          */
    	case 56:                           /* UNIT=56, Nanometrics RD6          */
	    case 63:                           /* UNIT=63, Earth Data ED24          */
    	case 65:                           /* UNIT=63, Earth Data ED24          */
	    case 66:                           /* UNIT=63, Earth Data ED24          */
    	case 96:                           /* UNIT=96, Synthetic Data           */
	    case 97:                           /* UNIT=56, Nanometrics RD6          */
    	case 99:                           /* UNIT=99, 24 bit ILI   *DJS        */
	    Line_STA2();                       /* Print out current sta's           */
    	Line_LTA2();                       /* Print out current lta's           */
	    break;

	}

    Line_CHAFLG();                        /* Print out marked detection channels*/

	switch(ptr->unit)
	{
    	case 10:                           /* UNIT=10 GBV Lowcost               */
	    case 30:                           /* UNIT=63 Earth Data ED24           */
    	case 31:                           /* UNIT=63 Earth Data ED24           */
	    case 32:                           /* UNIT=63 Earth Data ED24           */
    	case 33:                           /* UNIT=63 Earth Data ED24           */
	    case 37:                           /* UNIT= 3 RBO_3                     */
	    case 36:                           /* UNIT=36 VADC36                    */
	    case 40:                           /* UNIT=40 Nanometrics HRD           */
	    case 53:                           /* UNIT=53 Nanometrics RD3           */
	    case 56:                           /* UNIT=56 Nanometrics RD6           */
    	case 63:                           /* UNIT=63 Earth Data ED24           */
	    case 65:                           /* UNIT=63 Earth Data ED24           */
	    case 66:                           /* UNIT=63 Earth Data ED24           */
	    case 96:                           /* UNIT=96 Synthetic Data            */
	    case 97:                           /* UNIT=56 Nanometrics RD6           */
	    case 99:                           /* UNIT=99 24 bit ILI  *DJS          */
	    Line_SAMPLES2();                   /* Print out first 16 samples        */
    	break;
	}

    /* send output to display */
    wrefresh (win_main);
  }
}

/*--------------------------------------------------------------------------*/
/* Unlink from parameter module                                             */
/*--------------------------------------------------------------------------*/
LstUpd()
{
  int pek;

  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);
  mvwprintw (win_main, 2, 1, "Last update:");
  for(pek=0;pek<27;pek++)
    mvwprintw(win_main, 2, pek+13, "%c", ptr->Timsav[pek]);

  /*------------------------------------------------------------------------*/
  /* check all different clocktypes                                         */
  /*------------------------------------------------------------------------*/
  if (has_colors ()) wattron (win_main, COLOR_PAIR (5));
  else wstandout (win_main);
  switch(ptr->clktyp)
  {
  case 0:
    break;

  case 1:                            /*   ***** GARMIN 35-HVS No PPS **** */
    switch(ptr->timcpy[27])
    {
    case 'S':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : No sync received  ");
        break;
    case 'F':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : Sync found        ");
        break;
    case 'R':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : ERROR reading     ");
        break;
    case 'M':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : TIMEOUT reading   ");
        break;
    case 'C':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : Receiving time    ");
        break;
    case 'A':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : Not accepting time");
        break;
    case 'E':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : Time updated      ");
        break;
    }

  case 2:
    switch(ptr->timcpy[27])
    {
    case 'S':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : No sync received  ");
      break;
    case 'F':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Sync found        ");
      break;
    case 'R':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : ERROR reading     ");
      break;
    case 'M':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : TIMEOUT reading   ");
      break;
    case 'C':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Receiving time    ");
      break;
    case 'A':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Not accepting time");
      break;
    case 'E':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Time updated      ");
      break;
    }
    break;

  case 3:
    switch(ptr->timcpy[27])
    {
    case 'S':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : No sync received  ");
      break;
    case 'F':
      mvwprintw (win_main, 2,45,"  GPS Garmin  : Sync found        ");
      break;
    case 'R':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : ERROR reading     ");
      break;
    case 'M':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : TIMEOUT reading   ");
      break;
    case 'C':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Receiving time    ");
      break;
    case 'A':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Not accepting time");
      break;
    case 'E':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Time updated      ");
      break;
    }
    break;

  case 4:
    switch(ptr->timcpy[27])
    {
    case 'S':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : No sync received  ");
        break;
    case 'F':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : Sync found        ");
        break;
    case 'R':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : ERROR reading     ");
        break;
    case 'M':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : TIMEOUT reading   ");
        break;
    case 'C':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : Receiving time    ");
        break;
    case 'A':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : Not accepting time");
        break;
    case 'E':
        mvwprintw(win_main, 2,45,"  GPS Garmin  : Time updated      ");
        break;
    }
    break;
 
  case 5:                                         /***** GPS-GARMIN ******/
    switch(ptr->timcpy[27])
    {
    case 'S':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : No sync received  ");
      break;
    case 'F':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Sync found        ");
      break;
    case 'R':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : ERROR reading     ");
      break;
    case 'M':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : TIMEOUT reading   ");
      break;
    case 'C':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Receiving time    ");
      break;
    case 'A':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Not accepting time");
      break;
    case 'E':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Time updated      ");
      break;
    }
    break;

  case 6:                            /*   **** GPS-Lowrance ****          */
    switch(ptr->timcpy[27])
    {
    case 'S':
      mvwprintw(win_main, 2, 45,"  GPS Lowrance: No sync received  ");
      break;
    case 'F':
      mvwprintw(win_main, 2, 45,"  GPS Lowrance: Sync found        ");
      break;
    case 'R':
      mvwprintw(win_main, 2, 45,"  GPS Lowrance: ERROR reading     ");
      break;
    case 'M':
      mvwprintw(win_main, 2, 45,"  GPS Lowrance: TIMEOUT reading   ");
      break;
    case 'C':
      mvwprintw(win_main, 2, 45,"  GPS Lowrance: Receiving time    ");
      break;
    case 'A':
      mvwprintw(win_main, 2, 45,"  GPS Lowrance: Not accepting time");
      break;
    case 'E':
      mvwprintw(win_main, 2, 45,"  GPS Lowrance: Time updated      ");
      break;
    }
    break;

  case 7:                            /*   **** GPS-GARMIN ****            */
    switch(ptr->timcpy[27])
    {
    case 'S':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : No sync received  ");
      break;
    case 'F':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Sync found        ");
      break;
    case 'R':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : ERROR reading     ");
      break;
    case 'M':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : TIMEOUT reading   ");
      break;
    case 'C':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Receiving time    ");
      break;
    case 'A':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Not accepting time");
      break;
    case 'E':
      mvwprintw(win_main, 2, 45,"  GPS Garmin  : Time updated      ");
      break;
    }
  }
}


Time()
{
  int pek;

  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);

  mvwprintw (win_main, 3, 1,"Buffer time:");
  for(pek=0;pek<31;pek++)
    mvwprintw (win_main, 3, pek+13, "%c", ptr->timcpy[pek]);

  mvwprintw (win_main, 3, 47, "Rec time:");
  mvwprintw (win_main, 3, 56, "%8d",ptr->noktime);

  mvwprintw (win_main, 3, 65, "rng file:");
  mvwprintw (win_main, 3, 74, "%5d", ptr->ringidx[0]);
}


Line_STA1()
{
  nrsta=0;

  wprintw (win_main, "sta:");

  for(i=0;i<ptr->nchan;i++)
  {
    if(nrsta <= 6)
    {
      if(ptr -> chaflg[set][i] != 0)
      {
        wprintw (win_main, " %4.1f",ptr -> sta[set][i]);
        nrsta=nrsta+1;
      }
    }
  }
}

Line_STA2()
{
  nrsta=0;

  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);

  mvwprintw (win_main, 7, 1, "sta:");

  for(i=0;i<ptr->nchan;i++)
  {
    if(nrsta <= 6)
    {
      if(ptr -> chaflg[set][i] != 0)
      {
        mvwprintw (win_main, 7, nrsta*10+5, "%10.1f", ptr->sta[set][i]);
        nrsta=nrsta+1;
      }
    }
  }
}


Line_LTA1()
{
  nrlta=0;
  wprintw (win_main, "lta:");
  for(i=0;i<ptr->nchan;i++)
  {
    if(nrlta <= 6)
    {
      if(ptr -> chaflg[set][i] != 0)
      {
        wprintw(win_main, " %4.1f",ptr -> lta[set][i]);
        nrlta=nrlta+1;
      }
	}
  }
}


Line_LTA2()
{
  nrlta=0;

  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);

  mvwprintw(win_main, 8, 1, "lta:");

  for(i=0;i<ptr->nchan;i++)
  {
    if(nrlta <= 6)
    {
      if(ptr -> chaflg[set][i] != 0)
      {
        mvwprintw (win_main, 8, nrlta*10+5, "%10.1f", ptr->lta[set][i]);
        nrlta=nrlta+1;
      }
	}
  }
}


Sys_Var()
{
  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);

  /* Print out intresting variables from parameter module and datamodule      */
  mvwprintw (win_main, 4,1, "detflg:");
  mvwprintw (win_main, 4,11,"ipoint:");
  mvwprintw (win_main, 4,21,"ndet  :");
  mvwprintw (win_main, 4,32,"preblk:");
  mvwprintw (win_main, 4,44,"To CATLOG  :");
  mvwprintw (win_main, 4,65,"errno   :");

  if(ptr->detflg[set] != 0)
  {
    if (has_colors ()) wattron (win_main, COLOR_PAIR (2) | A_BLINK);
    else wattron (win_main, A_STANDOUT | A_BLINK);
    mvwprintw (win_main, 4, 9, "%1d", ptr->detflg[set]);
    if (has_colors ())
    {
 	  wattroff (win_main, A_BLINK);
      wattron (win_main, COLOR_PAIR (1));
    }
    else wstandend (win_main);
  }else{
    mvwprintw (win_main, 4, 9, "%1d", ptr->detflg[set]);
  }

  mvwprintw(win_main, 4,18,"%2d",ptr->ipoint);
  mvwprintw(win_main, 4,29,"%2d",ptr->ndet[set]);
  mvwprintw(win_main, 4,40,"%2d",ptr->preblk[set]);
  mvwprintw(win_main, 4,56,"%8d",ptr->ntick);
  mvwprintw(win_main, 4,74,"%5d",ptr->errorno);

  mvwprintw (win_main, 5,1,"recrd :");
  mvwprintw (win_main, 5,11,"queue :");
  mvwprintw (win_main, 5,21,"ndetw :");
  mvwprintw (win_main, 5,32,"postbk:");
  mvwprintw (win_main, 5,44,"iblk       :");
  mvwprintw (win_main, 5,65,"duration:");

  if(ptr->recrd[set] != 0)
  {
    if (has_colors ()) wattron (win_main, COLOR_PAIR (2) | A_BLINK);
    else wattron (win_main, A_STANDOUT | A_BLINK);
    mvwprintw (win_main, 5, 9, "%1d", ptr->recrd[set]);
    if (has_colors ())
    {
 	  wattroff (win_main, A_BLINK);
      wattron (win_main, COLOR_PAIR (1));
    }
    else wstandend (win_main);
  }else{
    mvwprintw(win_main, 5, 9, "%1d",ptr->recrd[set]);
  }
  if(ptr->queue+1 != 0)
  {
    if (has_colors ()) wattron (win_main, COLOR_PAIR (2) | A_BLINK);
    else wattron (win_main, A_STANDOUT | A_BLINK);
    mvwprintw (win_main, 5, 18, "%2d", ptr->queue+1);
    if (has_colors ())
    {
 	  wattroff (win_main, A_BLINK);
      wattron (win_main, COLOR_PAIR (1));
    }
    else wstandend (win_main);
  }else{
    mvwprintw(win_main,5,18,"%2d",ptr->queue+1);
  }
  mvwprintw(win_main, 5,29, "%2d",ptr->ndetw[set]);
  mvwprintw(win_main, 5,40, "%2d",ptr->postbk[set]);
  mvwprintw(win_main, 5,56, "%8d",ptr->iblk);
  mvwprintw(win_main, 5,74, "%5d",ptr->pdur[ptr->ipoint]);

}


Line_CHAFLG()
{
  int xx,yy;

  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);

  mvwprintw (win_main, 10,1,"chaflg:");

  xx=7;
  yy=9;
  for(i=0;i<16;i++)
  {
    if(ptr->singdt[set][i]==1)
    {
      if (has_colors ()) wattron (win_main, COLOR_PAIR (2) | A_BLINK);
      else wattron (win_main, A_STANDOUT | A_BLINK);
      mvwprintw(win_main,10,i+8,"%1d",ptr->chaflg[set][i]);
      if (has_colors ())
      {
   	    wattroff (win_main, A_BLINK);
        wattron (win_main, COLOR_PAIR (1));
      }
      else wstandend (win_main);
      xx=xx+1;
    }else{
      mvwprintw(win_main,10,i+8,"%1d",ptr->chaflg[set][i]);
      xx=xx+1;
    }
  }

  xx=xx+1;
  for(i=16;i<32;i++)
  {
    if(ptr->singdt[set][i]==1)
    {
      if (has_colors ()) wattron (win_main, COLOR_PAIR (2) | A_BLINK);
      else wattron (win_main, A_STANDOUT | A_BLINK);
      mvwprintw(win_main,10,i+9,"%1d",ptr->chaflg[set][i]);
      if (has_colors ())
      {
 	    wattroff (win_main, A_BLINK);
        wattron (win_main, COLOR_PAIR (1));
      }
      else wstandend (win_main);
      xx=xx+1;
    }else{
      mvwprintw(win_main,10,i+9,"%1d",ptr->chaflg[set][i]);
      xx=xx+1;
    }
  }
  xx=xx+1;
  for(i=32;i<48;i++)
  {
    if(ptr->singdt[set][i]==1)
    {
      if (has_colors ()) wattron (win_main, COLOR_PAIR (2) | A_BLINK);
      else wattron (win_main, A_STANDOUT | A_BLINK);
      mvwprintw(win_main,10,i+10,"%1d",ptr->chaflg[set][i]);
      if (has_colors ())
      {
 	    wattroff (win_main, A_BLINK);
        wattron (win_main, COLOR_PAIR (1));
      }
      else wstandend (win_main);
      xx=xx+1;
    }else{
	  mvwprintw(win_main,10,i+10,"%1d",ptr->chaflg[set][i]);
      xx=xx+1;
    }
  }
  xx=xx+1;
  for(i=48;i<64;i++)
  {
    if(ptr->singdt[set][i]==1)
    {
      if (has_colors ()) wattron (win_main, COLOR_PAIR (2) | A_BLINK);
      else wattron (win_main, A_STANDOUT | A_BLINK);
      mvwprintw(win_main,10,i+11,"%1d",ptr->chaflg[set][i]);
      if (has_colors ())
      {
 	    wattroff (win_main, A_BLINK);
        wattron (win_main, COLOR_PAIR (1));
      }
      else wstandend (win_main);
      xx=xx+1;
    }else{
	  mvwprintw(win_main,10,i+11,"%1d",ptr->chaflg[set][i]);
      xx=xx+1;
    }
  }
}


Head_1()
{
  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);

  mvwprintw(win_main,1,1,"Network:");
  mvwprintw(win_main,1,9,"%s",ptr->name);

  if (has_colors ()) wattron (win_main, COLOR_PAIR (3));
  else wstandout (win_main);
  switch(ptr->clktyp)
  {
  case 0:
    mvwprintw(win_main,1,14," CPU clock only.    ");
    break;
  case 1:
    mvwprintw(win_main,1,14," Garmin 35HVS No PPS");
    break;
  case 2:
    mvwprintw(win_main,1,14," Garmin 35HVS PPS   ");
    break;
  case 3:
    mvwprintw(win_main,1,14," Garmin-31          ");
    break;
  case 4:
    mvwprintw(win_main,1,14," RCC MSF or RCC GPS ");
    break;
  case 5:
    mvwprintw(win_main,1,14," Garmin-35          ");
    break;
  case 6:
    mvwprintw(win_main,1,14," GPS Lowrance       ");
    break;
  case 7:
    mvwprintw(win_main,1,14," GPS Garmin         ");
    break;
  case 8:
    mvwprintw(win_main,1,14," Timing by digitizer");
    break;
  }

  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);

  mvwprintw(win_main,1,47,"GPS     :");
  mvwprintw(win_main,1,56,"%4d%%",ptr->percent_gps);
  mvwprintw(win_main,1,62,"PPS:");
  mvwprintw(win_main,1,69,"%10d",ptr->average1);

}



Line_SAMPLES1()
{
  unsigned int dob_val;
  short s_val1,s_val2;

  for(i=0;i<7;i++)
  {
    dob_val=ptr->ibuf[ptr->index][i+HEADER];
    s_val1=dob_val>>16;
    s_val2=dob_val;
    wprintw(win_main,"%8d",s_val1);
    wprintw(win_main,"%8d",s_val2);
  }
}


Line_SAMPLES2()
{
  int end;

  if (has_colors ()) wattron (win_main, COLOR_PAIR (1));
  else wstandend (win_main);

  mvwprintw(win_main,12,1,"Samples:");

  end = ptr->nchan;
  if (end > 7) end = 7;
  for(i=0;i<end;i++)
    mvwprintw(win_main,13,i*10+1,"%10d",ptr->ibuf[ptr->index][i+HEADER]);
}


Sys_Cons()
{
  if (has_colors ()) wattron (win_sys, COLOR_PAIR (4));
  else wstandend (win_sys);

  mvwprintw(win_sys,1,2,"ParSet:");
  mvwprintw(win_sys,1,12,"Ext/Int:");
  mvwprintw(win_sys,1,27,"ArPrWnd:");
  mvwprintw(win_sys,1,40,"Min_Rng:");
  mvwprintw(win_sys,1,52,"TimCor:");
  mvwprintw(win_sys,1,9,"%2d",ptr->active);
  mvwprintw(win_sys,1,20,"%6d",ptr->intclk);
  mvwprintw(win_sys,1,35,"%3d",ptr->premax[set]);
  mvwprintw(win_sys,1,48,"%3d",ptr->maxblk/60);
  mvwprintw(win_sys,1,60,"%8.3f",ptr->delay);

  mvwprintw(win_sys,2,2,"Chan  :");
  mvwprintw(win_sys,2,12,"MaxRec :");
  mvwprintw(win_sys,2,27,"PreEvt :");
  mvwprintw(win_sys,2,40,"SmpBfCh:");
  mvwprintw(win_sys,2,52,"S-Rate:");
  mvwprintw(win_sys,2,9,"%2d",ptr->nchan);
  mvwprintw(win_sys,2,20,"%6d",ptr->maxbuf[set]);
  mvwprintw(win_sys,2,35,"%3d",ptr->prevnt[set]);
  mvwprintw(win_sys,2,48,"%3d",ptr->nsamp);
  mvwprintw(win_sys,2,60,"%8.3f",ptr->srate);

  mvwprintw(win_sys,3,2,"ClkTyp:");
  mvwprintw(win_sys,3,12,"BufSiz :");
  mvwprintw(win_sys,3,27,"PstEvt :");
  mvwprintw(win_sys,3,40,"CurPset:");
  mvwprintw(win_sys,3,52,"CPUcor:");
  mvwprintw(win_sys,3,9,"%2d",ptr->clktyp);
  mvwprintw(win_sys,3,20,"%6d",ptr->bufsiz);
  mvwprintw(win_sys,3,35,"%3d",ptr->postmx[set]);
  mvwprintw(win_sys,3,48,"%3d",set+1);
  mvwprintw(win_sys,3,60,"%8.5f",ptr->timerr);

  mvwprintw(win_sys,4,2,"Unit  :",7);
  mvwprintw(win_sys,4,12,"Buf_rng:");
  mvwprintw(win_sys,4,27,"MinTrg :");
  mvwprintw(win_sys,4,40,"Min/Rng:");
  mvwprintw(win_sys,4,52,"Minpip:");
  mvwprintw(win_sys,4,9,"%2d",ptr->unit);
  mvwprintw(win_sys,4,20,"%6d",ptr->maxblk);
  mvwprintw(win_sys,4,35,"%3d",ptr->mintrg[set]);
  mvwprintw(win_sys,4,48,"%3d",ptr->nmin_per_ring[0]);
  mvwprintw(win_sys,4,60,"%8d",PIPSIZ*4/ptr->reclength);

}
