/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date: 01/08/98                      */
/* Program: MON_TRG                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program is controlled by BUF_PIP. BUF_PIP sends out a proxy each time*/
/* a buffer has been written to the memory pipe with the time tag from the  */
/* clock. This routine reads this time tag and computes a running average   */
/* of the drift of the time. As the time stamping is a function of the      */
/* stability of the sampling frequency, there will normally be a small drift*/
/* caused by this instability.                                              */
/* When the time-tagging is based only on the reception of the time-string  */
/* from the clock, there will be rather big jumps in the time tagging. This */
/* in accuracy is then averaged over time and is more stable.               */
/* When the time-tagging is based on the reception of the time-string from  */
/* the clock AND the 1 PPS, the time tagging is very accurate, and the      */
/* averaging will give more or less the same value.                         */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <graph.h>

#define XUL1 5                         /* upper (x) left corner window #1   */
#define YUL1 0                         /* upper (y) left corner window #1   */ 
#define XLR1 490                       /* lower (x) right corner window #1  */
#define YLR1 75                       /* lower (y) right corner window #1  */

#define XUL2 5                         /* upper (x) left corner window #2   */
#define YUL2 80                       /* upper (y) left corner window #2   */ 
#define XLR2 490                       /* lower (x) right corner window #2  */
#define YLR2 155                       /* lower (y) right corner window #2  */

#define XUL3 5                         /* upper (x) left corner window #3   */
#define YUL3 160                       /* upper (y) left corner window #3   */ 
#define XLR3 490                       /* lower (x) right corner window #3  */
#define YLR3 235                       /* lower (y) right corner window #3  */

#define XUL11 580                       /* upper (x) left corner window #11 */
#define YUL11 0                         /* upper (y) left corner window #11 */ 
#define XLR11 610                       /* lower (x) right corner window #11*/
#define YLR11 175                       /* lower (y) right corner window #11*/

#define MILLIS 1000
#define RATIO  500
#define PROSENT 100
#define DASHED 0xc0c0
#define SOLID  0xffff

#define BLACK         0                       /* black                      */
#define BLUE          1                       /* dark blue                  */
#define GREEN         2                       /* green                      */
#define DARK_CYAN     3                       /* dark cyan                  */
#define RED           4                       /* red                        */
#define VIOLET        5                       /* violet                     */
#define BROWN         6                       /* brown                      */
#define GRAY          7                       /* gray                       */
#define DARK_GRAY     8                       /* dark gray                  */
#define LIGHT_BLUE    9                       /* light blue                 */
#define LIGHT_GREEN  10                       /* light green                */
#define LIGHT_CYAN   11                       /* light cyan                 */
#define LIGHT_RED    12                       /* light red                  */
#define LIGHT_VIOLET 13                       /* light violet               */
#define YELLOW       14                       /* yellow                     */
#define WHITE        15                       /* white                      */


int      n;
char     bl[10];
char     buf[10];
int      l,k,i,lp,lc;
int      sec_in_pip;
int      cnt;
int      acc;
float    scx;
float    scy;
char     times[10][80];
char     tider[80];
char     heltall[30];
int      mil[1000];


double   totsec[50];
double   sta1;
double   first;
double   diff;
double   drift;
int      start,stopp;
double   MSECS;                        /* total seconds ringbuffer filename */
double   mint;                         /* smallest value indicating rngfile */
double   maxt;                         /* largest value indicating rngfile  */
char     karak[30];
float    SEC;                          /* seconds floating point            */
float    SECL;
double    new,gammel,forskjell;
int      inew,igammel,ifor;
int      YR,DOY,MTH,DAY,HR,MIN;        /* ringbuffer file date              */
int      hu;
int      gpix;
int      npix;
int      dpix;
int      startlevel;
int      teller;
float    rlevel;
int      ilevel;
double   rtlevel;
double   slevel;
char     dum;
int      p;

main()
{
int      R;                            /* ratio sta/lta                     */
int      no_of_times;                  /* total number of buffers received  */
int      no_of_syncs;                  /* total number of synchronized times*/
float    percent_snc;                  /* percent of time synchronized      */
int      xold1;                        /* old x window #1 time              */
int      yold1;                        /* old y window #1 time              */
int      xold2;                        /* old x window #1 average           */
int      yold2;                        /* old y window #1 average           */
int      xold3;                        /* old x window #2 average           */
int      yold3;                        /* old y window #2 average           */
int      xold4;                        /* old x window #3 GPS               */
int      yold4;                        /* old y window #3 GPS               */
int      xold5;                        /* old x window #4 ratio             */
int      yold5;                        /* old y window #4 ratio             */
int      ocol;                         /* old color                         */
char     sync;                         /* good time-bad time S-not S        */
int      snc;                          /* good time-bad time 1-0            */
/*                    make window                                           */
int      xul;                          /* x upper left                      */
int      yul;                          /* y upper left                      */
int      xlr;                          /* x lower right                     */
int      ylr;                          /* y lower right                     */
int      fill_color;                   /* fill color window                 */
int      border_color;                 /* color frame around window         */

/*                     origo window                                         */
int      x0;                           
int      y0;
/*                     grid horizontal                                      */
int      x1;                           /* start point gridline              */
int      x2;                           /* stop point  gridline              */
int      from;                         /* start value                       */
int      to;                           /* stop value                        */
int      step;                         /* gridline every step               */
int      linestyle;                    /* dashed or solid                   */
float    scaling;                      /* pixels per unit                   */
int      color;                        /* color gridlines                   */
/*                     grid vertical                                        */
int      y1;                           /* start point gridline              */
int      y2;                           /* stop point gridline               */
/*                     scaling factor sta/lta windows                       */
float      scx1;                       /* scaling x-axis window             */
float      scy1;                       /* scaling y-axis window             */
/*                     scaling factor GPS window                            */
float      scx11;                      /* scaling x-axis window             */
float      scy11;                      /* scaling y-axis window             */


int i;
pid_t    proxy;
unsigned int cop;
int buf_pip_id;
int sw;
char ka;
int tid_id;
unsigned msg;
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */
char*    fontpath=getenv("WINFONTS");
int fnt_no;
fnt_no=61;
n=_registerfonts(fontpath);
xold1=0;
yold1=0;
xold2=0;
yold2=0;
xold3=0;
yold3=0;
xold4=0;
yold4=0;
xold5=0;
yold5=0;
no_of_times=0;                         /* total number of buffers received  */
no_of_syncs=0;                         /* total number of synchronized times*/


sprintf(buf,"n%d",fnt_no);
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MON_PIPE_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MON_PIPE_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
if(buf_pip_id==-1)
    exit(0);
proxy=qnx_proxy_attach(0,0,0,-1);
ptr->rat_proxy=proxy;
tid_id=qnx_name_attach(0,"mon_trg");
if(tid_id==-1)
    exit(0);
sw=0;
sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;
printf("Seconds in memory pipe: %d\n",sec_in_pip);

/*                      scaling factors sta/lta windows                     */
scx1=(float)(XLR1-XUL1)/(float)sec_in_pip;
scy1=(float)(YLR1-YUL1)/(float)RATIO;
/*                      scaling factors GPS window                          */
scx11=(float)(XLR11-XUL11)/(float)sec_in_pip;
scy11=(float)(YLR11-YUL11)/(float)PROSENT;

printf("scx: %f scy: %f\n",scx,scy);
//printf("scx1 %f scy1 %f scx2 %f scy2 %f\n",scx1,scy1,scx2,scy2);
//getch();
_setfont(buf);
cnt=0;
teller=0;
acc=0;

    _setvideomode(_VRES16COLOR);

top:

/****************************************************************************/
/*                  MAKE WINDOW #1 sta/lts channel 1                        */
/****************************************************************************/

    x0=XUL1;
    y0=YUL1;
    xul=XUL1;
    yul=YUL1;
    xlr=XLR1;
    ylr=YLR1;
    fill_color=BLUE;
    border_color=WHITE;
    make_window(xul,yul,xlr,ylr,fill_color,border_color);

/*--------------------------------------------------------------------------*/
/*                draw a line indicating the trigger ratio level channel 1  */
/*--------------------------------------------------------------------------*/
    x0=XUL1;                       /* origo window #1                   */
    y0=YUL1;
    xlr=XLR1;
    ylr=YLR1;
    color=YELLOW;
    R=RATIO-(int)(ptr->ratio[0][0]*100.);
    trg_level(x0,y0,xlr,ylr,scx1,scy1,R,color);
    x1=XUL1;
    x2=XLR1;
    from=0;
    to=RATIO;
    step=100;
    scaling=scy1;
    linestyle=DASHED;
    color=GRAY;
    grid_horizontal(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);

    x1=YUL1;
    x2=YLR1;
    from=0;
    to=sec_in_pip;
    step=60;
    scaling=scx1;
    linestyle=DASHED;
    color=GRAY;
    grid_vertical(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);
    _moveto(0,(int)  (float)(1000.*scy)  );


/****************************************************************************/
/*                       MAKE WINDOW #2 sta/lta channel 2                   */
/****************************************************************************/

    x0=XUL2;
    y0=YUL2;
    xul=XUL2;
    yul=YUL2;
    xlr=XLR2;
    ylr=YLR2;
    fill_color=BLUE;
    border_color=WHITE;
    make_window(xul,yul,xlr,ylr,fill_color,border_color);


/*--------------------------------------------------------------------------*/
/*                draw a line indicating the trigger ratio level channel 2  */
/*--------------------------------------------------------------------------*/
    x0=XUL2;                       /* origo window #4                   */
    y0=YUL2;
    xlr=XLR2;
    ylr=YLR2;
    color=YELLOW;
    R=RATIO-(int)(ptr->ratio[0][1]*100.);
    trg_level(x0,y0,xlr,ylr,scx1,scy1,R,color);
    x1=XUL2;
    x2=XLR2;
    from=0;
    to=RATIO;
    step=100;
    scaling=scy1;
    linestyle=DASHED;
    color=GRAY;
    grid_horizontal(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);

    x1=YUL2;
    x2=YLR2;
    from=0;
    to=sec_in_pip;
    step=60;
    scaling=scx1;
    linestyle=DASHED;
    color=GRAY;
    grid_vertical(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);
    _moveto(0,(int)  (float)(1000.*scy)  );

/****************************************************************************/
/*                       MAKE WINDOW #4 Recording RATIO                     */
/****************************************************************************/

    x0=XUL3;
    y0=YUL3;
    xul=XUL3;
    yul=YUL3;
    xlr=XLR3;
    ylr=YLR3;
    fill_color=BLUE;
    border_color=WHITE;
    make_window(xul,yul,xlr,ylr,fill_color,border_color);
/*--------------------------------------------------------------------------*/
/*                draw a line indicating the trigger ratio level            */
/*--------------------------------------------------------------------------*/
    x0=XUL3;                       /* origo window #4                   */
    y0=YUL3;
    xlr=XLR3;
    ylr=YLR3;
    color=YELLOW;
    R=RATIO-(int)(ptr->ratio[0][2]*100.);
    trg_level(x0,y0,xlr,ylr,scx1,scy1,R,color);
    x1=XUL3;
    x2=XLR3;
    from=0;
    to=RATIO;
    step=100;
    scaling=scy1;
    linestyle=DASHED;
    color=GRAY;
    grid_horizontal(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);

    x1=YUL3;
    x2=YLR3;
    from=0;
    to=sec_in_pip;
    step=60;
    scaling=scx1;
    linestyle=DASHED;
    color=GRAY;
    grid_vertical(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);
    _moveto(0,(int)  (float)(1000.*scy)  );

/****************************************************************************/
/*                       MAKE WINDOW #3 Indicator for GPS reception         */
/****************************************************************************/

    x0=XUL11;
    y0=YUL11;
    xul=XUL11;
    yul=YUL11;
    xlr=XLR11;
    ylr=YLR11;
    fill_color=BLUE;
    border_color=WHITE;
    make_window(xul,yul,xlr,ylr,fill_color,border_color);

/*--------------------------------------------------------------------------*/
/*                draw a line indicating the trigger ratio level            */
/*--------------------------------------------------------------------------*/
    x0=XUL11;                       /* origo window #4                   */
    y0=YUL11;
    xul=XUL11;
    yul=YUL11;
    xlr=XLR11;
    ylr=YLR11;
    fill_color=BLUE;
    border_color=WHITE;
    make_window(xul,yul,xlr,ylr,fill_color,border_color);
    x1=XUL11;
    x2=XLR11;
    from=0;
    to=100;
    step=10;
    scaling=scy11;
    linestyle=DASHED;
    color=GRAY;
    grid_horizontal(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);

for(;;)
{
    buf_pip_id=Receive(0,&msg,sizeof(msg));
    if(kbhit())
        exit(0);
    k=ptr->ClkFlg_05;
    no_of_times++;                     /* total number of buffers received  */

//    sprintf(times[k],"%3d %s\n",k,ptr->pipe+k*((ptr->nchan*ptr->nsamp)+HEADER));
    sprintf(tider,"%3d %s\n",k,ptr->pipe+k*((ptr->nchan*ptr->nsamp)+HEADER));
    sync=tider[30];
    if(sync=='S')
    {
        snc=1;                         /* Clock in sync                     */
    }else{
        snc=0;                         /* Clock not in sync                 */
    }
    sscanf(tider,"%3d%c%2d%c%2d%c%f6.3d",&hu,&dum,&HR,&dum,&MIN,&dum,&SECL);
    for(i=0;i<9;i++)
        tider[i]=tider[i+17];
    sscanf(tider,"%2d%c%2d%c%2d",&DAY,&dum,&MTH,&dum,&YR);
    TIMSEC(YR,MTH,DAY,HR,MIN,SECL,&MSECS); /* current time in milliseconds  */
    percent_snc=ptr->percent_gps;
    status_line2(snc,percent_snc);
/****************************************************************************/
/*                      draw trend sta/lta channel 1                        */
/****************************************************************************/
        x0=XUL1;                       /* origo window #1                   */
        y0=YUL1;
        color=YELLOW;
        R=RATIO-(int)((ptr->sta[0][0]/ptr->lta[0][0])*100.);
        draw_trend(x0,y0,scx1,scy1,cnt,R,&xold1,&yold1,snc,color);

/****************************************************************************/
/*                      draw trend sta/lta channel 2                        */
/****************************************************************************/
        x0=XUL2;                       /* origo window #2                   */
        y0=YUL2;
        color=YELLOW;
        R=RATIO-(int)((ptr->sta[0][1]/ptr->lta[0][1])*100.);
        draw_trend(x0,y0,scx1,scy1,cnt,R,&xold2,&yold2,snc,color);

/****************************************************************************/
/*                      draw trend sta/lta channel 3                        */
/****************************************************************************/
        x0=XUL3;                       /* origo window #3                   */
        y0=YUL3;
        color=YELLOW;
        R=RATIO-(int)((ptr->sta[0][2]/ptr->lta[0][2])*100.);
        draw_trend(x0,y0,scx1,scy1,cnt,R,&xold3,&yold3,snc,color);

/****************************************************************************/
/*                      draw percentage GPS                                 */
/****************************************************************************/
        x0=XUL11;                       /* origo window #4                   */
        y0=YUL11;
        xul=XUL11;
        yul=YUL11;
        xlr=XLR11;
        ylr=YLR11;
        fill_color=BLUE;
        border_color=WHITE;
        make_window(xul,yul,xlr,ylr,fill_color,border_color);
        x0=XUL11;
        y0=YLR11 - (int)(ptr->percent_gps * scy11);
        xul=XUL11;
        yul=y0;
        xlr=XLR11;
        ylr=YLR11;
        if(snc==1)
        {
            fill_color=GREEN;
        }else{
            fill_color=RED;
        }
        border_color=WHITE;
        make_window(xul,yul,xlr,ylr,fill_color,border_color);
        x0=XUL11;
        y0=YUL11;
        x1=XUL11;
        x2=XLR11;
        from=0;
        to=100;
        step=10;
        scaling=scy11;
        linestyle=DASHED;
        color=GRAY;
        grid_horizontal(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);

/****************************************************************************/

    teller++;
    cnt++;
    if(cnt>=sec_in_pip-1)
    {
        cnt=0;
        goto top;
    }
    if(cnt>=sec_in_pip-1)
    {
        cnt=0;
/*                  MAKE WINDOW #1                                          */

    x0=XUL1;
    y0=YUL1;
    xul=XUL1;
    yul=YUL1;
    xlr=XLR1;
    ylr=YLR1;
    fill_color=BLUE;
    border_color=WHITE;
    make_window(xul,yul,xlr,ylr,fill_color,border_color);
    x1=XUL1;
    x2=XLR1;
    from=0;
    to=MILLIS;
    step=100;
    scaling=scy1;
    linestyle=DASHED;
    color=GRAY;
    grid_horizontal(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);

    x1=YUL1;
    x2=YLR1;
    from=0;
    to=sec_in_pip;
    step=60;
    scaling=scx1;
    linestyle=DASHED;
    color=GRAY;
    grid_vertical(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);

    xold1=cnt;
    yold1=startlevel;
    xold2=cnt;
//    yold2=ilevel;
    yold2=ptr->average1;
    xold3=cnt;
//    yold3=ilevel;
    yold3=ptr->average1;
    

/*                       MAKE WINDOW #3                                     */

    x0=XUL3;
    y0=YUL3;
    xul=XUL3;
    yul=YUL3;
    xlr=XLR3;
    ylr=YLR3;
    fill_color=BLUE;
    border_color=WHITE;
    make_window(xul,yul,xlr,ylr,fill_color,border_color);
    x1=XUL3;
    x2=XLR3;
    from=0;
    to=100;
    step=10;
    scaling=scy1;
    linestyle=DASHED;
    color=GRAY;
    grid_horizontal(x1,x2,x0,y0,from,to,step,scaling,linestyle,color);
    }

}
}    /* End of main */




TIMSEC (YR,MTH,DAY,HR,MIN,SECS,MSECS)                         
int YR;
int MTH;
int DAY;
int HR;
int MIN;
float SECS;
double *MSECS;                         /* total seconds to be returned      */
{
int DYR;                               /* flag for leap year                */
int IYR;                               /* number of leap-years since 1900   */
int YDY;                               /* number of days in current year    */

if(YR <= 50) YR=YR+100;                /* check for 2000 or more in 2 digits*/

if(YR >= 1900) YR=YR - 1900;
DYR = 0;
if((YR % 4) == 0) DYR = 1;
IYR = YR/4 - DYR;
//                -- Seconds to beginning of     
*MSECS = (float)(IYR*366) + (float)((YR-IYR)*365);
//                -- current year                
*MSECS = *MSECS*86400.0;
//                -- January                     
if (MTH == 1) YDY = DAY;
//                -- February                    
if (MTH == 2) YDY = DAY + 31;
//                -- ....                        
if (MTH == 3) YDY = DAY + DYR + 59;       
if (MTH == 4) YDY = DAY + DYR + 90;                                     
if (MTH == 5) YDY = DAY + DYR + 120;                                    
if (MTH == 6) YDY = DAY + DYR + 151;                                    
if (MTH == 7) YDY = DAY + DYR + 181;                                    
if (MTH == 8) YDY = DAY + DYR + 212;                                    
if (MTH == 9) YDY = DAY + DYR + 243;                                    
if (MTH ==10) YDY = DAY + DYR + 273;                                    
if (MTH ==11) YDY = DAY + DYR + 304;                                    
if (MTH ==12) YDY = DAY + DYR + 334;                                    
*MSECS = *MSECS + (float)(YDY*86400 + HR*3600 + MIN*60) + SECS;
}

/*--------------------------------------------------------------------------*/
/* make window                                                              */
/*--------------------------------------------------------------------------*/
/* xul         x-upper-left                                                 */
/* yul         y-upper-left                                                 */
/* xlr         x-lower-right                                                */
/* ylr         y-lower-right                                                */
/* fill_color  background color window                                      */
/* border_color color frame aroud window                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
make_window(xul,yul,xlr,ylr,fill_color,border_color)
int xul;
int yul;
int xlr;
int ylr;
int fill_color;
int border_color;
{
    _setcolor(fill_color);
    _rectangle(_GFILLINTERIOR,xul,yul,xlr,ylr);
    _setcolor(border_color);
    _rectangle(_GBORDER,xul,yul,xlr,ylr);
}
/*--------------------------------------------------------------------------*/
/* grid horizontal                                                          */
/*--------------------------------------------------------------------------*/
/* x1          start point gridline                                         */
/* x2          stop point gridline                                          */
/* x0          origo window                                                 */
/* y0          origo window                                                 */
/* from        start value                                                  */
/* to          stop value                                                   */
/* step        gridline every 'step'                                        */
/* scaling     pixels per unit vertical                                     */
/* linestyle   dashed or solid                                              */
/* color       color gridlines                                              */
/*                                                                          */
/*--------------------------------------------------------------------------*/
grid_horizontal(x1,x2,x0,y0,from,to,step,scaling,linestyle,color)
int   x1;
int   x2;
int   x0;
int   y0;
int   from;
int   to;
int   step;
int   linestyle;
float scaling;
int   color;
{
int   lk;
_setlinestyle(linestyle);
_setcolor(color);
for(lk=from+step;lk<to;lk=lk+step)
{
    _moveto(x0+1,(int) ((float)lk*scaling)+y0);
    _lineto(x2-1, (int) ((float)lk*scaling)+y0);
}
_setlinestyle(SOLID);
}
/*--------------------------------------------------------------------------*/
/* grid vertical                                                            */
/*--------------------------------------------------------------------------*/
/* y1          start point gridline                                         */
/* y2          stop point gridline                                          */
/* x0          origo window                                                 */
/* y0          origo window                                                 */
/* from        start value                                                  */
/* to          stop value                                                   */
/* step        gridline every 'step'                                        */
/* scaling     pixels per unit horizontal                                   */
/* linestyle   dashed or solid                                              */
/* color       color gridlines                                              */
/*                                                                          */
/*--------------------------------------------------------------------------*/
grid_vertical(y1,y2,x0,y0,from,to,step,scaling,linestyle,color)
int   y1;
int   y2;
int   x0;
int   y0;
int   from;
int   to;
int   step;
int   linestyle;
float scaling;
int   color;
{
int   lk;
_setlinestyle(linestyle);
_setcolor(color);
for(lk=from+step;lk<to;lk=lk+step)
{
    _moveto((int) ((float)lk*scaling)+x0,y1+1);
    _lineto( (int) ((float)lk*scaling)+x0,y2-1);
}
_setlinestyle(SOLID);
}
status_line2(snc,percent)
int snc;
float percent;
{
char     as[50];
int      ocol;
    sprintf(as,"GPS: %5.1f %%\n",percent);
ocol=_getcolor();
_setcolor(1);
_rectangle(_GFILLINTERIOR,XUL11-35,YLR11+3,XUL11+55,YLR11+27);
_setcolor(15);
_rectangle(_GBORDER,XUL11-35,YLR11+3,XUL11+55,YLR11+27);
if(snc==1)
{
    _setcolor(WHITE);
}else{
    _setcolor(RED);
}
_moveto(XUL11-32,YLR11+10);
_outgtext(as);
_setcolor(ocol);
}
draw_trend(x0,y0,scx,scy,xnew,ynew,xold,yold,snc,color)
int   x0;
int   y0;
float scx;
float scy;
int   xnew;
int   ynew;
int*  xold;
int*  yold;
int   snc;
int   color;
{
        if(snc==1)
        {
            _setcolor(color);
        }else{
            _setcolor(DARK_CYAN);
        }
        _moveto((int)  ((float)(*xold)*scx+x0),(int)((float)(*yold)*scy+y0));
        _lineto((int)  ((float)(xnew)*scx+x0),(int)((float)(ynew)*scy+y0));
        *xold=xnew;
        *yold=ynew;
}

trg_level(x0,y0,x1,y1,scx,scy,ynew,color)
int   x0;
int   y0;
int   x1;
int   y1;
float scx;
float scy;
int   ynew;
int   color;
{
            _setcolor(color);
    _moveto(x0,(int)((float)(ynew)*scy+y0));
    _lineto(x1,(int)((float)(ynew)*scy+y0));
}
