/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:30/05/00                       */
/* Program:MUL_EDA                                                          */
/* 01/06/00 8.22 t.u. new driver Earth Data PS2400 new model multiserial    */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from multiserial board.                 */
/* Extract sequence number and time stamp and put into serial pipe          */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/proxy.h>
#include <sys/sched.h>
#define numfread(a1,a2) numread = fread(a1,1,a2,in)
static int block_length;
char     nummer[10];
char     line_to_open[100];            /* name of line to open              */
char     line_speed[100];              /* set line speed                    */
int      baud;                         /* line speed                        */
int      kval;
int      err_cnt;
char     syscall[100];                 /* system call string                */
int      ret_val;
int      BPS;                          /* bytes per sample                  */
int      RHDR;                         /* bytes in rest of header after SNC */
long int SEQ_no;
long int LST_gps;
char snd_msg[30];
FILE *in;
int numread;
int      first_time;
int      nr;
int      ser_prt;                      /* serial port number                */
int      ser_idx;                      /* port number minus 1 for table     */
int      sec_ptr;                      /* pointer to pipe next record       */
int      k;                            /* counter                           */
unsigned char byt[2000];               /* Input buffer                      */
int      i;                            /* counter                           */
int      fd;                           /* return value open share           */
int      no_sec;                       /* counter                           */
int      sts_buf;                      /* return status from ED             */
int      sts_snc;                      /* return status from Sync           */
int      n;                            /* return value from dev_read        */
int      fh;                           /* file handler for read             */
int      rdbyt;

unsigned char *buf_ptr;                /* pointer to data buffer            */
int      nbyt;                         /* number of bytes to read from line */
smod     *ptr;                         /* pointer to shared region          */
int      arg[200];

main(argc,argv)
int      argc;
char     **argv;
{
static int time_diff, time_diff2;		/* for when GPS not in sync */
double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
int      year;
int      month;
int      day;
int      hour;
int      minute;
int      sec;
int      old_sek;
int      diff;
int      sekvens;
unsigned int cop;
int      mul_ser_id;
int      only_one;
int      one_shot;
int      address;
char     timestring[100];
setprio(getpid(),12);                  /* set the priority of this process  */
strcpy(nummer,argv[1]);
sscanf(nummer,"%d",&nr);
//printf("nr=%d\n",nr);
ser_prt=nr;

printf("MUL_EDA:Serial port:%2d\n",ser_prt);
ser_idx=ser_prt-1;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MUL_EDA_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MUL_EDA_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
BPS=3;
RHDR=57;
/*--------------------------------------------------------------------------*/
/* bytes to read                                                            */
/*--------------------------------------------------------------------------*/
nbyt=ptr->cha_def[ser_idx][3]*ptr->cha_def[ser_idx][2] * BPS + RHDR; 
//printf("CHA_DEF %d %d\n",ptr->cha_def[ser_idx][3],ptr->cha_def[ser_idx][2]);
//printf("NBYT:%d RDBYT:%d\n",nbyt,rdbyt);
/*--------------------------------------------------------------------------*/
/* Open device for reading data from multiserial board                      */
/*--------------------------------------------------------------------------*/
sprintf(line_to_open,"/dev/ser%d\0",ser_prt);
if(ptr->cha_def[ser_idx][3] == 1)      
{
    baud=9600;                         /* 1 channel format                  */
}else{
    baud=19200;                        /* 3 channel format                  */
}
sprintf(line_speed,"stty baud=%d <%s\0",baud,line_to_open);
//printf("%s\n",line_to_open);
//printf("%s\n",line_speed);
system(line_speed);
in=fopen(line_to_open,"r");
for(i=0;i<5;i++)
{
    ptr->alive[1]=1;
    Sync();
}
do
{
  Sync();
  ret_val= ED();
}while (ret_val== -1);
fclose(in);
//printf("finished header\n");
nbyt=block_length;


if((fh=open(line_to_open,O_RDONLY | O_CACHE))==ERROR)
{
    printf("MUL_EDA: Can't open serial line %s\n",line_to_open);
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Read data from multiserial board until one 5 sync character is detected. */
/*--------------------------------------------------------------------------*/
for(i=0;i<5;i++)
    sts_snc=Sync1();
printf("MUL_EDA: Receiving from line: %2d !\n",ser_prt);





only_one=0;
one_shot=0;

for(;;)                                /* indefinite loop collecting data   */
{
    for(no_sec=0;no_sec<SERSIZ;no_sec++)/* serial pipe is 60 seconds long   */
    {
//more:
        sts_snc=Sync1();               /* find sync char. in received data  */
                                       /* signal.                           */
        switch(sts_snc)                /* check return value after search   */
        {
            case 1:                    /*************************************/
                                       /* SYNC found, all well, go on       */
                                       /*************************************/
            sts_buf=ED_3();            /* read 1 sec of data from line      */
            switch(sts_buf)            /* check return value after read     */
            {
                case 1:                /*************************************/
                                       /* DATA READ ok, convert data        */
                                       /*************************************/
/****************************************************************************/
/* extract timestring and check sequence number against blocknumber for last*/
/* GPS message to compute number of seconds to add to time.                 */
/* Tests below also tests for wrap around on sequence number.               */
/* NOTE: time_diff MUST be static !!                                        */
/****************************************************************************/
                Get_Adr_ED(&address,&sekvens,&YR,&MTH,&DAY,&HR,&MIN,&SEC);
//printf("SEQ_no=%d LST_gps=%d\n",SEQ_no,LST_gps);
                if(SEQ_no != LST_gps)
                {
                  if(SEQ_no > LST_gps)
                  {
                    time_diff = SEQ_no - LST_gps;                
                    diff=time_diff;
                  }else{
                    diff = time_diff + SEQ_no;
                  }
                }else{
                  diff=0;
                }               
//printf("DIFF=%d\n",diff);
                TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
                if(diff>0)
                {
                  ptr->TimFlg[0]=2; /* set indicator NSY gps clock        */
                  MSECS=MSECS+diff;
                }else{
                  ptr->TimFlg[0]=1; /* set indicator SET gps clock        */
                }
                SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
                address=(int)SEC;
                sprintf(timestring,
                "%2d:%2d:%6.3f %2d/%2d/%4d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
//                printf("%2d:%2d:%6.3f %2d/%2d/%4d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
switch(ser_prt)
{
    case 1:
//    printf("PRT 1: %5d %s\n",address,timestring);
    break;
    case 2:
//    printf("PRT 2: %5d %s\n",address,timestring);
    break;
}
//printf("SER_IDX %d\n",ser_idx);
                if(ser_idx==0)
                    Mark_Time(timestring,address);
                if(one_shot==0)
                {
                    ptr->start_sec[ser_idx]=address;
//printf("Line: %d Start sec:%d\n",ser_idx,ptr->start_sec[ser_idx-1]);
                    one_shot=1;
                }
                if(ser_idx==0)
                {
                    diff=sekvens-old_sek;
                    if(diff != 1)
                        ptr->sts_msg2[4]++;
                    old_sek=sekvens;
//   printf("%d\n",sekvens);
                }
                    
/****************************************************************************/
/* compute the the address of first sample after header for current second  */
/****************************************************************************/                            

                sec_ptr=ptr->cha_def[0][15]*address+SHDR; 
//printf("SEC_PTR=%d CHA_DEF[0][15]=%d ADDRESS=%d SHDR=%d\n",sec_ptr,
//ptr->cha_def[0][15],address,SHDR);
                Convert();             /* convert and store in serial pipe  */
                ptr->dqu[ser_idx]++;   /* increment buffer ready this line  */
                ptr->squ[ser_idx]++;   /* increment queue serial pipe       */
                if(ptr->squ[ser_idx] > (SERSIZ -1)) /* check for wrap pipe  */
                    ptr->squ[ser_idx]=0; /* set pointer to start of pipe    */
                break;
                case 0:                /*************************************/
                                       /* DATA READ TIMEOUT, no data        */
                                       /*************************************/
                printf("MUL_EDA: timeout on line: %2d\n",ser_prt);
                ptr->sts_msg2[2]++;
                break;

                case -1:               /*************************************/
                                       /* ERROR reading data                */
                                       /*************************************/
                printf("MUL_EDA: error on line: %2d\n",ser_prt);
                ptr->sts_msg2[3]++;
                break;
            }
            break;

            case 0:                    /*************************************/
                                       /* TIMEOUT looking for SYNC          */
                                       /*************************************/
            printf("MUL_EDA: SNC timeout on line: %2d\n",ser_prt);
            ptr->sts_msg2[0]++;
            break;

            case -1:                   /*************************************/
                                       /* ERROR reading SYNC characters     */
                                       /*************************************/
            printf("MUL_EDA: SNC error on line: %2d\n",ser_prt);
            ptr->sts_msg2[1]++;
            break;
        }
    }
}
}
Mark_Time(tstring,add)
char* tstring;
int  add;
{
char timestamp[100];
int  k;
char str[100];

register char *tmstr;                  /* pointer to time string in buffer  */
register int temp;
register unsigned int shift;
register unsigned int swap;
register unsigned short msw;
register unsigned short lsw;
register unsigned short sw;
int      i;
char     tmpstr[80];

/*           000000000011111111112222222222333333333344444444445555555*/
/*           012345678901234567890123456789012345678901234567890123456*/
/*           00:00:00.000 00/00/0000 000 sts 00:00:00.000 00/00/00 000  "*/

//ti[55]=0x0d;
//ti[56]=0x0a;
for(k=0;k<60;k++)
    timestamp[k]=' ';
timestamp[55]=0x0d;
timestamp[56]=0x0a;
for(k=0;k<27;k++)
    timestamp[k]=tstring[k];
if(ptr->clktyp==8)
{
    if(ptr->TimFlg[0]==1)
    {
      timestamp[28]='S';
      timestamp[29]='E';
      timestamp[30]='T';
    }else{
      timestamp[28]='N';
      timestamp[29]='S';
      timestamp[30]='Y';
    }
}
if(ptr->clktyp==0)
{
    timestamp[28]='C';
    timestamp[29]='P';
    timestamp[30]='U';
}
timestamp[31]='\0';
if(timestamp[0]==' ')
    timestamp[0]='0';
if(timestamp[3]==' ')
    timestamp[3]='0';
if(timestamp[6]==' ')
    timestamp[6]='0';
if(timestamp[9]==' ')
    timestamp[9]='0';
if(timestamp[10]==' ')
    timestamp[10]='0';
if(timestamp[13]==' ')
    timestamp[13]='0';
if(timestamp[16]==' ')
    timestamp[16]='0';
if(timestamp[24]==' ')
    timestamp[24]='0';
if(timestamp[25]==' ')
    timestamp[25]='0';
for(k=0;k<32;k++)
    ptr->timcpy[k]=timestamp[k];
//printf("MUL_SERb: %s\n",timestamp);
k=0;
shift=timestamp[3];
shift=shift << 8;
shift=shift | timestamp[2];
shift=shift << 8;
shift=shift | timestamp[1];
shift=shift << 8;
shift=shift | timestamp[0];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[7];
shift=shift << 8;
shift=shift | timestamp[6];
shift=shift << 8;
shift=shift | timestamp[5];
shift=shift << 8;
shift=shift | timestamp[4];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[11];
shift=shift << 8;
shift=shift | timestamp[10];
shift=shift << 8;
shift=shift | timestamp[9];
shift=shift << 8;
shift=shift | timestamp[8];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[15];
shift=shift << 8;
shift=shift | timestamp[14];
shift=shift << 8;
shift=shift | timestamp[13];
shift=shift << 8;
shift=shift | timestamp[12];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[19];
shift=shift << 8;
shift=shift | timestamp[18];
shift=shift << 8;
shift=shift | timestamp[17];
shift=shift << 8;
shift=shift | timestamp[16];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[23];
shift=shift << 8;
shift=shift | timestamp[22];
shift=shift << 8;
shift=shift | timestamp[21];
shift=shift << 8;
shift=shift | timestamp[20];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[27];
shift=shift << 8;
shift=shift | timestamp[26];
shift=shift << 8;
shift=shift | timestamp[25];
shift=shift << 8;
shift=shift | timestamp[24];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[31];
shift=shift << 8;
shift=shift | timestamp[30];
shift=shift << 8;
shift=shift | timestamp[29];
shift=shift << 8;
shift=shift | timestamp[28];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3()
{
buf_ptr=byt;
n=dev_read(fh,buf_ptr,nbyt,nbyt,0,11,0,0); /* read 'nbyt' bytes from serial port*/
//printf("ED_3: N=%d NBYT=%d\n",n,nbyt);
switch(n)
{
    case -1:
    return(-1);
    break;

    case 0:
    return(0);
    break;

    case 1:       
    break;
}
return(1);
}
/****************************************************************************/
/* Function to convert data buffers from EDR                            /
/****************************************************************************/
void Convert()
{
register unsigned char lo,me,hi;
register unsigned int val;
unsigned short smp;
int count;
short sample;
register int verdi;
register int v1,v2,v3;
buf_ptr= &byt[198];                     /* point to first data byte          */
for(count=0;count<75; count++)
{
    val=0;
    lo=(*buf_ptr++);
    me=(*buf_ptr++);
    hi=(*buf_ptr++);                   /* in this case dummy                */
    val=(hi<< 16) | (me << 8) | lo;
    if (hi & 0x80) v1 = val | 0xff000000;
    else v1 = val;

    val=0;
    lo=(*buf_ptr++);
    me=(*buf_ptr++);
    hi=(*buf_ptr++);
    val=(hi<< 16) | (me << 8) | lo;
    if (hi & 0x80) v2 = val | 0xff000000;
    else v2 = val;

    val=0;
    lo=(*buf_ptr++);
    me=(*buf_ptr++);
    hi=(*buf_ptr++);
    val=(hi<< 16) | (me << 8) | lo;
    if (hi & 0x80) v3 = val | 0xff000000;
    else v3 = val;

    switch(ptr->cha_def[ser_idx][3])
    {
        case 1:
        ptr->mulpip[ser_idx][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->mulpip[ser_idx][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        ptr->mulpip[ser_idx][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->mulpip[ser_idx][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        ptr->mulpip[ser_idx][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        ptr->mulpip[ser_idx][sec_ptr]=v3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
}

Sync1()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0x4d4f);
return(1);
}
/****************************************************************************/
/* Function to data buffers from a RD3                                      */
/****************************************************************************/
int    ED()
{
int error, count, ncomps, sample_rate, bytes_per_sample;
static long int data_block_size;		/* calculated */
long int DAT_size;						/* taken from header */
char *byt_ptr;
char time_inc[7];
if (first_time == 0) block_length = 885; /* assume this for now */

numfread(byt,block_length);            /* read in the header = 192 bytes
									    *   less the 1st 2 bytes (already read)
										*   plus the data header 8 bytes
										*   plus the data (75Hz, 3C, 3bytes/samp) = 675 bytes
									    *   plus the checksum 12 bytes */
if(numread<block_length)               /* read OK ?                         */
{
    clearerr(in);                      /* clear error                       */
}

DAT_size = (long)0;				/* extract data block size from header */

DAT_size = (byt[197] << 24) | (byt[196] << 16) | (byt[195] << 8) | byt[194];


LST_gps = (long)0;				/* extract sequence number from header */
LST_gps = (byt[117] << 24) | (byt[116] << 16) | (byt[115] << 8) | byt[114];

//printf("%d %d \n",SEQ_no,LST_gps);

if (first_time == 0)
{
	if (LST_gps <= 0)
	{
		printf ("Waiting for GPS to lock . . .\n");
		return -1;
	}
	ncomps = (byt[43] << 8) | byt[42];
	sample_rate = (byt[45] << 8) | byt[44];
	/* N.B. Documentation suggests this would be 75 as default,
	 *   but in the header, this is 225 for 3 component set */
	bytes_per_sample = (byt[47] << 8) | byt[46];
	data_block_size = (long)(sample_rate * bytes_per_sample);	/* expected */
	block_length = 190				/* header size minus two bytes read */
 				   + 8				/* data header size */
				   + (int)data_block_size		/* data */
				   + 12; 			/* checksum */
	first_time = 1;
}

/* here we need to check the received data for an error and set the
 * error flag TRUE if we find one */
error = 0;
return 0;
}
void    Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
        } while(sync_value != 0x4d4f);
/*putchar(0x07);*/
}
/****************************************************************************/
/* Function to data buffers from a RD3                                      */
/****************************************************************************/
Get_Adr_ED(address,seq,yrn,mon,day,hrn,min,sec)
int* address;
int* seq;
int* yrn;
int* mon;
int* day;
int* hrn;
int* min;
float* sec;

{
int Address,Seq,Yrn,Mon,Day,Hrn,Min;
float Sec;
int error, count, ncomps, sample_rate, bytes_per_sample;
static long int data_block_size;		/* calculated */
long int DAT_size;						/* taken from header */
static int time_diff, time_diff2;		/* for when GPS not in sync */
char *byt_ptr;
char time_inc[7];


SEQ_no = (long)0;				/* extract sequence number from header */
SEQ_no = (byt[41] << 24) | (byt[40] << 16) | (byt[39] << 8) | byt[38];
*seq = (int)SEQ_no;

LST_gps = (long)0;				/* extract sequence number from header */
LST_gps = (byt[117] << 24) | (byt[116] << 16) | (byt[115] << 8) | byt[114];

//	bytes_per_sample = (byt[47] << 8) | byt[46];
/* extract GPS message */
byt_ptr = &byt [118];
//printf ("%70.70s\n", byt_ptr); 
for (count=0; count<9; count++)
{
  byt_ptr = strchr (byt_ptr, ',') + 1;
  switch (count)
  {
  case 0:
    strcpy (snd_msg, "xx xx xx.000 xx xx xx 0      ");
    snd_msg [0] = *byt_ptr;
    snd_msg [1] = *(byt_ptr +1);
    snd_msg [3] = *(byt_ptr +2);
    snd_msg [4] = *(byt_ptr +3);
    snd_msg [6] = *(byt_ptr +4);
    snd_msg [7] = *(byt_ptr +5);
    break;
  case 8:
    snd_msg [13] = *byt_ptr;
    snd_msg [14] = *(byt_ptr +1);
    snd_msg [16] = *(byt_ptr +2);
    snd_msg [17] = *(byt_ptr +3);
    snd_msg [19] = *(byt_ptr +4);
    snd_msg [20] = *(byt_ptr +5);
    break;
  }
}
sscanf(snd_msg,"%2d %2d %6f %2d %2d %2d",&Hrn,&Min,&Sec,&Day,&Mon,&Yrn);
Address=(int)Sec;
//printf("%2d:%2d:%6.3f %2d/%2d/%2d\n",Hrn,Min,Sec,Day,Mon,Yrn);
*hrn=Hrn;
*min=Min;
*sec=Sec;
*day=Day;
*mon=Mon;
*yrn=Yrn;
*address=Address;
}
