/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:21/01/97                       */
/* Program:MUL_SER                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from multiserial board.                 */
/* Extract sequence number and time stamp and put into serial pipe          */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <conio.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/proxy.h>
#include <sys/sched.h>
char     nummer[10];
char     line_to_open[100];            /* name of line to open              */
char     line_speed[100];              /* set line speed                    */
int      baud;                         /* line speed                        */
char     syscall[100];                 /* system call string                */
int      BPS;                          /* bytes per sample                  */
int      RHDR;                         /* bytes in rest of header after SNC */
int      nr;
int      ser_prt;                      /* serial port number                */
int      ser_idx;                      /* port number minus 1 for table     */
int      sec_ptr;                      /* pointer to pipe next record       */
int      k;                            /* counter                           */
unsigned char byt[2000];               /* Input buffer                      */
int      i;                            /* counter                           */
int      fd;                           /* return value open share           */
int      no_sec;                       /* counter                           */
int      sts_buf;                      /* return status from ED             */
int      sts_snc;                      /* return status from Sync           */
int      n;                            /* return value from dev_read        */
int      fh;                           /* file handler for read             */
int      rdbyt;

unsigned char *buf_ptr;                /* pointer to data buffer            */
int      nbyt;                         /* number of bytes to read from line */
smod     *ptr;                         /* pointer to shared region          */
int      arg[200];

main(argc,argv)
int      argc;
char     **argv;
{
double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
int      year;
int      month;
int      day;
int      hour;
int      minute;
int      sec;
int      old_sek;
int      diff;
int      sekvens;
unsigned int cop;
int      mul_ser_id;
int      only_one;
int      one_shot;
int      address;
char     timestring[100];
setprio(getpid(),12);                  /* set the priority of this process  */
strcpy(nummer,argv[1]);
sscanf(nummer,"%d",&nr);
printf("nr=%d\n",nr);
ser_prt=nr;

printf("MUL_SER:Serial port:%2d\n",ser_prt);
ser_idx=ser_prt-1;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MUL_SER_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MUL_SER_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
BPS=3;
RHDR=57;
/*--------------------------------------------------------------------------*/
/* bytes to read                                                            */
/*--------------------------------------------------------------------------*/
nbyt=ptr->cha_def[ser_idx][3]*ptr->cha_def[ser_idx][2] * BPS + RHDR; 
//printf("NBYT:%d RDBYT:%d\n",nbyt,rdbyt);
/*--------------------------------------------------------------------------*/
/* Open device for reading data from multiserial board                      */
/*--------------------------------------------------------------------------*/
sprintf(line_to_open,"/dev/dty%d\0",ser_prt);
if(ptr->cha_def[ser_idx][3] == 1)      
{
    baud=9600;                         /* 1 channel format                  */
}else{
    baud=38400;                        /* 3 channel format                  */
}
sprintf(line_speed,"stty baud=%d <%s\0",baud,line_to_open);
printf("%s\n",line_to_open);
printf("%s\n",line_speed);
if((fh=open(line_to_open,O_RDONLY | O_CACHE))==ERROR)
{
    printf("MUL_SER: Can't open serial line %s\n",line_to_open);
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* Read data from multiserial board until one 5 sync character is detected. */
/*--------------------------------------------------------------------------*/
for(i=0;i<5;i++)
    sts_snc=Sync1();
printf("MUL_SER: Receiving from line: %2d !\n",ser_prt);

only_one=0;
one_shot=0;

for(;;)                                /* indefinite loop collecting data   */
{
    for(no_sec=0;no_sec<SERSIZ;no_sec++)/* serial pipe is 60 seconds long   */
    {
//more:
        sts_snc=Sync1();               /* find sync char. in received data  */
                                       /* signal.                           */
        switch(sts_snc)                /* check return value after search   */
        {
            case 1:                    /*************************************/
                                       /* SYNC found, all well, go on       */
                                       /*************************************/
            sts_buf=ED_3();            /* read 1 sec of data from line      */
            switch(sts_buf)            /* check return value after read     */
            {
                case 1:                /*************************************/
                                       /* DATA READ ok, convert data        */
                                       /*************************************/
                Get_Adr_GBV(&address,&sekvens,&YR,&MTH,&DAY,&HR,&MIN,&SEC);
                TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
                SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC);
                sprintf(timestring,
                "%2d:%2d:%6.3f %2d/%2d/%4d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
switch(ser_prt)
{
    case 1:
//    printf("PRT 1: %5d %s\n",address,timestring);
    break;
    case 2:
//    printf("PRT 2: %5d %s\n",address,timestring);
    break;
}
                if(ser_idx==0)
                    Mark_Time(timestring,address);
                if(one_shot==0)
                {
                    ptr->start_sec[ser_idx]=address;
printf("Line: %d Start sec:%d\n",ser_idx,ptr->start_sec[ser_idx-1]);
                    one_shot=1;
                }
                if(ser_idx==0)
                {
                    diff=sekvens-old_sek;
                    if(diff != 1)
                        ptr->sts_msg2[4]++;
                    old_sek=sekvens;
//   printf("%d\n",sekvens);
                }
                    
/****************************************************************************/
/* compute the the address of first sample after header for current second  */
/****************************************************************************/                            
                sec_ptr=ptr->cha_def[0][15]*address+SHDR; 
                Convert();             /* convert and store in serial pipe  */
                ptr->dqu[ser_idx]++;   /* increment buffer ready this line  */
                ptr->squ[ser_idx]++;   /* increment queue serial pipe       */
                if(ptr->squ[ser_idx] > (SERSIZ -1)) /* check for wrap pipe  */
                    ptr->squ[ser_idx]=0; /* set pointer to start of pipe    */
                break;
                case 0:                /*************************************/
                                       /* DATA READ TIMEOUT, no data        */
                                       /*************************************/
                printf("MUL_SER:RDB timeout on line: %2d\n",ser_prt);
                ptr->sts_msg2[2]++;
                break;

                case -1:               /*************************************/
                                       /* ERROR reading data                */
                                       /*************************************/
                printf("MUL_SER: RDB error on line: %2d\n",ser_prt);
                ptr->sts_msg2[3]++;
                break;
            }
            break;

            case 0:                    /*************************************/
                                       /* TIMEOUT looking for SYNC          */
                                       /*************************************/
            printf("MUL_SER: SNC timeout on line: %2d\n",ser_prt);
            ptr->sts_msg2[0]++;
            break;

            case -1:                   /*************************************/
                                       /* ERROR reading SYNC characters     */
                                       /*************************************/
            printf("MUL_SER: SNC error on line: %2d\n",ser_prt);
            ptr->sts_msg2[1]++;
            break;
        }
    }
}
}
Get_Adr_GBV(address,seq,ad1,ad2,ad3,ad4,ad5,ad6)
int* address;
int* seq;
int* ad1;
int* ad2;
int* ad3;
int* ad4;
int* ad5;
float* ad6;
{
int add;
unsigned short ss;
unsigned int ss1;
unsigned short save;
int sequence;
int k;
int tusen;
int hundre;
int ti;
int en;
    

    ss1=byt[38];
    ss1=ss1<<8 | byt[39];
    ss1=ss1<<8 | byt[40];
    ss1=ss1<<8 | byt[41];
    sequence=ss1;
    *seq=sequence;                     /* extract sequence number           */

    save=byt[54];
    ss=save;
    ss=ss & 0x000f;
    *address= ss;
    ss=save;
    ss=ss & 0x00f0;
    ss = ss >> 4;
    *address=*address+ss*10;
    add=*address;                      /* extract second to use as address  */

    save=byt[48];                      /* extract thousand in year          */
    ss=save;
    ss=ss & 0x00f0;
    ss=ss >> 4;
    tusen=ss*1000;

    ss=save;                           /* extract hundreds in year          */
    ss=ss & 0x000f;
    hundre=ss*100;

    save=byt[49];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in year              */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in year              */

    *ad1=tusen+hundre+ti+en;

    save=byt[50];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in months            */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in months            */

    *ad2=ti+en;

    save=byt[51];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in day               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in day               */

    *ad3=ti+en;

    save=byt[52];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in hour              */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in hour              */

    *ad4=ti+en;

    save=byt[53];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in minute            */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in minute            */

    *ad5=ti+en;

    save=byt[54];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in seconds           */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in seconds           */

    *ad6=ti+en;


//    ptr->mulpip[ser_idx][ptr->cha_def[0][15]*add]=ss1;

//    ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+1]=ss1;

//    ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+2]=ss1;

}
Mark_Time(tstring,add)
char* tstring;
int  add;
{
char timestamp[100];
int  k;
char str[100];

register char *tmstr;                  /* pointer to time string in buffer  */
register int temp;
register unsigned int shift;
register unsigned int swap;
register unsigned short msw;
register unsigned short lsw;
register unsigned short sw;
int      i;
char     tmpstr[80];

/*           000000000011111111112222222222333333333344444444445555555*/
/*           012345678901234567890123456789012345678901234567890123456*/
/*           00:00:00.000 00/00/0000 000 sts 00:00:00.000 00/00/00 000  "*/

//ti[55]=0x0d;
//ti[56]=0x0a;
for(k=0;k<60;k++)
    timestamp[k]=' ';
timestamp[55]=0x0d;
timestamp[56]=0x0a;
for(k=0;k<27;k++)
    timestamp[k]=tstring[k];
if(ptr->clktyp==8)
{
    timestamp[28]='S';
    timestamp[29]='E';
    timestamp[30]='T';
}
if(ptr->clktyp==0)
{
    timestamp[28]='C';
    timestamp[29]='P';
    timestamp[30]='U';
}
timestamp[31]='\0';
if(timestamp[0]==' ')
    timestamp[0]='0';
if(timestamp[3]==' ')
    timestamp[3]='0';
if(timestamp[6]==' ')
    timestamp[6]='0';
if(timestamp[9]==' ')
    timestamp[9]='0';
if(timestamp[10]==' ')
    timestamp[10]='0';
if(timestamp[13]==' ')
    timestamp[13]='0';
if(timestamp[16]==' ')
    timestamp[16]='0';
if(timestamp[24]==' ')
    timestamp[24]='0';
if(timestamp[25]==' ')
    timestamp[25]='0';
for(k=0;k<32;k++)
    ptr->timcpy[k]=timestamp[k];
//printf("MUL_SERb: %s\n",timestamp);
k=0;
shift=timestamp[3];
shift=shift << 8;
shift=shift | timestamp[2];
shift=shift << 8;
shift=shift | timestamp[1];
shift=shift << 8;
shift=shift | timestamp[0];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[7];
shift=shift << 8;
shift=shift | timestamp[6];
shift=shift << 8;
shift=shift | timestamp[5];
shift=shift << 8;
shift=shift | timestamp[4];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[11];
shift=shift << 8;
shift=shift | timestamp[10];
shift=shift << 8;
shift=shift | timestamp[9];
shift=shift << 8;
shift=shift | timestamp[8];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[15];
shift=shift << 8;
shift=shift | timestamp[14];
shift=shift << 8;
shift=shift | timestamp[13];
shift=shift << 8;
shift=shift | timestamp[12];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[19];
shift=shift << 8;
shift=shift | timestamp[18];
shift=shift << 8;
shift=shift | timestamp[17];
shift=shift << 8;
shift=shift | timestamp[16];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[23];
shift=shift << 8;
shift=shift | timestamp[22];
shift=shift << 8;
shift=shift | timestamp[21];
shift=shift << 8;
shift=shift | timestamp[20];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[27];
shift=shift << 8;
shift=shift | timestamp[26];
shift=shift << 8;
shift=shift | timestamp[25];
shift=shift << 8;
shift=shift | timestamp[24];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

shift=timestamp[31];
shift=shift << 8;
shift=shift | timestamp[30];
shift=shift << 8;
shift=shift | timestamp[29];
shift=shift << 8;
shift=shift | timestamp[28];
ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+k]=shift;
k++;

}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3()
{
buf_ptr=byt;
n=dev_read(fh,buf_ptr,nbyt,nbyt,0,11,0,0); /* read 'nbyt' bytes from serial port*/
switch(n)
{
    case -1:
    return(-1);
    break;

    case 0:
    return(0);
    break;

    case 1:       
    break;
}
return(1);
}
/****************************************************************************/
/* Function to convert data buffers from Lowcost                            /
/****************************************************************************/
void Convert()
{
register unsigned char lo,me,hi;
register unsigned int val;
unsigned short smp;
short sample;
register int verdi;
register int v1,v2,v3;
buf_ptr= &byt[57];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
    val=0;
    hi=(*buf_ptr++);
    me=(*buf_ptr++);
    lo=(*buf_ptr++);                   /* in this case dummy                */
    smp=hi;
    smp=smp<<8 | me;
    sample=smp;
    v1=sample;

    val=0;
    hi=(*buf_ptr++);
    me=(*buf_ptr++);
    lo=(*buf_ptr++);
    smp=hi;
    smp=smp<<8 | me;
    sample=smp;
    v2=sample;

    val=0;
    hi=(*buf_ptr++);
    me=(*buf_ptr++);
    lo=(*buf_ptr++);
    smp=hi;
    smp=smp<<8 | me;
    sample=smp;
    v3=sample;
    switch(ptr->cha_def[ser_idx][3])
    {
        case 1:
        ptr->mulpip[ser_idx][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        break;

        case 2:
        ptr->mulpip[ser_idx][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        ptr->mulpip[ser_idx][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        break;

        case 3:
        ptr->mulpip[ser_idx][sec_ptr]=v1;
        sec_ptr=sec_ptr+1;
        ptr->mulpip[ser_idx][sec_ptr]=v2;
        sec_ptr=sec_ptr+1;
        ptr->mulpip[ser_idx][sec_ptr]=v3;
        sec_ptr=sec_ptr+1;
        break;
    }
}
}

Sync1()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
    do
    {
        n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
        switch(n)
        {
            case -1:
            return(-1);
            break;
            case 0:
            return(0);
            break;
            case 1:       
            sync_value = sync_value << 8 | *buf_ptr;
            break;
        }
    } while(sync_value != 0x4742);
return(1);
}
