/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:21/01/97                       */
/* Program:MUL_SER                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Program to read 1-second buffers from multiserial board.                 */
/* Extract sequence number and time stamp and put into serial pipe          */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dev.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <conio.h>
#include <stdlib.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/proxy.h>
#include <sys/sched.h>
#define  BPS 4                         /* bytes per sample                  */
#define  HDR 24                        /* rest of header                    */
#define  numfread(a1,a2) numread=fread(a1,1,a2,in)

int numread;
int      tmp_buf[8][250];
char     nummer[10];
char     line_to_open[100];            /* name of line to open              */
char     line_speed[100];              /* set line speed                    */
char     syscall[100];                 /* system call string                */
int      nr;                           /* line number                       */
int      ser_prt;                      /* serial port number                */
int      ser_idx;                      /* port number minus 1 for table     */
int      sec_ptr;                      /* pointer to pipe next record       */
int      k;                            /* counter                           */
unsigned char byt[2000];               /* Input buffer                      */
int      i;                            /* counter                           */
int      fd;                           /* return value open share           */
int      sts_buf;                      /* return status from ED             */
int      sts_snc;                      /* return status from Sync           */
int      n;                            /* return value from dev_read        */
int      fh;                           /* file handler for read             */
unsigned char *buf_ptr;                /* pointer to data buffer            */
int      nbyt;                         /* number of bytes to read from line */
smod     *ptr;                         /* pointer to shared region          */

main(argc,argv)
int      argc;
char     **argv;
{
double   MSCUR;
double   MSECS;
double   MSEC70;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN;
int      SECL;
unsigned int cop;
int      mul_ser_id;
int      address;
int      err_ind;
int      next_channel;
int      SEQ;                          /* sequence number                   */
int      KAN;                          /* channel current packet            */
int      CHK;                          /* checksum from unit                */
int      BPP;                          /* bytes per packet                  */
int      SPP;                          /* samples per packet                */
int      FMT;                          /* data format 16 or 32              */
unsigned char CHK_SUM;                 /* checksum computed                 */
char     stty_str[100];                /* port setup                        */
char     port_str[12];                 /* port setup                        */
char     timestring[100];

setprio(getpid(),12);                  /* set the priority of this process  */
strcpy(nummer,argv[1]);                /* save serial line number           */
sscanf(nummer,"%d",&nr);
ser_prt=nr;

//printf("MUL_SER:Serial port:%2d\n",ser_prt);
ser_idx=ser_prt-1;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MUL_SER_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MUL_SER_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/****************************************************************************/
/********************* SIMULATING TIMETAG START *****************************/
/****************************************************************************/
YR=99;
MTH=3;
DAY=29;
HR=0;
MIN=0;
switch(ser_idx)
{
    case 0:
    SEC=50.0;
    break;
    case 1:
    SEC=52.0;
    break;
    case 2:
    SEC=54.0;
    break;
    case 3:
    SEC=56.0;
    break;
}
TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
/****************************************************************************/
/********************* SIMULATING TIMETAG STOPP *****************************/
/****************************************************************************/

/*--------------------------------------------------------------------------*/
/* bytes to read                                                            */
/*--------------------------------------------------------------------------*/
nbyt=HDR-2+(ptr->rate * BPS); 
//printf("NBYT=%d\n",nbyt);
/*--------------------------------------------------------------------------*/
/* Open device for reading data from multiserial board                      */
/*--------------------------------------------------------------------------*/
sprintf(line_to_open,"/dev/ser%d\0",ser_prt);
sprintf(line_speed,"stty baud=%d <%s\0",ptr->bd_dig,line_to_open);
system(line_speed);
if((fh=open(line_to_open,O_RDONLY | O_CACHE))==ERROR)
{
    printf("MUL_SER2: Can't open serial line %s\n",line_to_open);
    exit(0);
}
err_ind=0;                             /* set to NO ERROR initially         */
/*--------------------------------------------------------------------------*/
/* Read data from IDS until 3 sync character is detected.                   */
/*--------------------------------------------------------------------------*/
for(i=0;i<3;i++)
{
//    sts_snc=Sync1();                   /* look for sync pattern             */
//    switch(sts_snc)                    /* test if return value is ok        */
//    {
//        case 1:                        /* reading byte ok                   */
//        break;
//        case 0:                        /* timeout looking for sync          */
//        printf("MUL_SER2: TIMEOUT reading data from line %s\n",line_to_open);
//        break;
//        case -1:                       /* error reading data                */
//        printf("MUL_SER2: ERROR reading data from line %s\n",line_to_open);
//        break;
//    }
  Sync();
}
printf("MUL_SER2   : Receiving data from digitizer ! Port %s\n",line_to_open);

/*--------------------------------------------------------------------------*/
/* search in data until buffer last channel has been read. Next channel to  */
/* read will then be channel 1. This is to get synchronized at startup.     */
/*--------------------------------------------------------------------------*/
next_channel=ptr->cha_def[ser_idx][3]; /* set next_channel to last channel  */
next:
//sts_snc=Sync1();                       /* look for sync characters          */
Sync();
//switch(sts_snc)                        /* test if return value is ok        */
//{
//    case 1:                            /* reading byte ok                   */
//    break;
//    case 0:                            /* timeout looking for sync          */
//    printf("MUL_SER2: TIMEOUT (Sync1) reading data from line %s\n",line_to_open);
//    goto next;
//    break;
//    case -1:                           /* error reading data                */
//    printf("MUL_SER2: ERROR (Sync1) reading data from line %s\n",line_to_open);
//    goto next;
//    break;
//}
sts_buf=ED_3(KAN);                        /* read 1 second of data, one channel*/
switch(sts_buf)                        /* test if return value is ok        */
{
    case 0:                            /* timeout reading data buffer       */
    printf("MUL_SER2: TIMEOUT (ED_3) reading data from line %s\n",line_to_open);
    goto next;
    break;
    case -1:                           /* error reading data                */
    printf("MUL_SER2: ERROR (ED_3) reading data from line %s\n",line_to_open);
    goto next;
    break;
    default:
    if(sts_buf != nbyt)
    {
        printf("MUL_SER2: Less than 1 second of data has been read on %s\n",line_to_open);
        goto next;
    }
    break;
}

//CHKSUM(nbyt,&CHK_SUM);                 /* compute checksum of received data */
HDR_INFO(&SEQ,&KAN,&CHK,&BPP,&SPP,&FMT,&YR,&DOY,&HR,&MIN,&SECL); /* extract header information      */
//printf("NEXT_CH %d KAN:%d\n",next_channel,KAN);
if(KAN != next_channel)                /* if not last channel               */
    goto next;                         /* continue searching                */
//if(CHK != CHK_SUM)                     /* test check sum                    */
//    goto next;                         /* error in checksum, continue search*/
/*--------------------------------------------------------------------------*/
/* we are now receiving correct data, and we found last channel Tell BUF_PIP*/
/*--------------------------------------------------------------------------*/
//printf("OK\n");
for(;;)                                /* indefinite loop collecting data   */
{
top:
/*--------------------------------------------------------------------------*/
/* find error-free sequence of 1->nchan buffers                             */
/* if not error-free, insert zero-buffer                                    */
/*--------------------------------------------------------------------------*/
   for(next_channel=1;next_channel<ptr->cha_def[ser_idx][3]+1;next_channel++)
   {
      sts_snc=Sync1();                 /* find sync char. in received data  */
      switch(sts_snc)                  /* test if return value is ok        */
      {
         case 1:                       /* reading byte ok                   */
         break;
   
         case 0:                       /* timeout looking for sync          */
         printf("MUL_SER2: TIMEOUT (Sync1) reading data from line %s\n",line_to_open);
         err_ind++;                    /* set error                         */
         break;

         case -1:                      /* error reading data                */
         printf("MUL_SER2: ERROR (Sync1) reading data from line %s\n",line_to_open);
         err_ind++;                    /* set error                         */
         break;
      }
/*--------------------------------------------------------------------------*/
/* read 1 data block                                                        */
/*--------------------------------------------------------------------------*/
      sts_buf=ED_3(KAN);
      switch(sts_buf)                  /* test if return value is ok        */
      {
         case 0:                       /* timeout reading data buffer       */
         printf("MUL_SER2: TIMEOUT (ED_3) reading data from line %s\n",line_to_open);
         err_ind++;                    /* set error                         */
         break;
         case -1:                      /* error reading data                */
         printf("MUL_SER2: ERROR (ED_3) reading data from line %s\n",line_to_open);
         err_ind++;                    /* set error                         */
         break;
         default:
         if(sts_buf != nbyt)
         {
            printf("MUL_SER2: Less than 1 second of data has been read on %s\n",line_to_open);
            err_ind++;                 /* set error                         */
         }
         break;
      }
      CHKSUM(nbyt,&CHK_SUM);           /* compute checksum received data    */
      HDR_INFO(&SEQ,&KAN,&CHK,&BPP,&SPP,&FMT,&YR,&DOY,&HR,&MIN,&SECL); /* extract header information      */

//udo
//      printf("1. NEXT=%d SEQ=%d KANAL=%d BPP=%d SPP=%d FMT=%d CHK=%8d %8d\n",next_channel,SEQ,KAN,BPP,SPP,FMT,CHK,CHK_SUM);
//      printf("2. %2x %2x %2x %2x\n",byt[22],byt[23],byt[24],byt[25]);

      if(next_channel != KAN)          /* do we have the right channel ?    */
      {
         printf("Wrong channel sequence\n");
         err_ind++;                    /* set error                         */
      }
//      if(CHK_SUM != CHK)               /* compare recorded & computed checks*/
//      {
//         printf("Checksum error\n");
//         err_ind++;                    /* set error                         */
//      }
/****************************************************************************/
/* DATA DO NOT CONTAIN TIMING, SIMULATE TIMING                              */
/* MUST GO OUT ON REAL VERSION                                              */
/****************************************************************************/
//      if(next_channel==1)              /* extract time only component 1     */
      if(next_channel<10)              /* extract time only component 1     */
      {
//         YR=0;
         DTE(DOY,&DAY,&MTH,YR);
//         printf("1. %2d/%2d/%2d\n",YR,MTH,DAY);
         SEC=(float)SECL;
         TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
         SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC); /*TESTING***/
//         printf("2. %2d/%2d/%2d\n",YR,MTH,DAY);
         address=SEC;                  /******** TESTING ********************/
//         ptr->start_sec[ser_idx]=address;
         sec_ptr=ptr->cha_def[0][15]*address+SHDR;
//printf("cha_def %d SEC_PTR=%d adress %d SHDR %d\n",ptr->cha_def[0][15],sec_ptr,address,SHDR);
      }
      Conv_tempo(next_channel);        /* convert each component and store  */
   }   /********** NCHAN CHANNELS READ, BUT DO WE HAVE ERRORS ? *************/

/*--------------------------------------------------------------------------*/
/* nchan channels are now stored in outputbuffer and the buffer is tagged   */
/* Now check data integrity of the buffer. If errors, insert zero buffer    */
/*--------------------------------------------------------------------------*/
   if(err_ind != 0)                    /* any errors ?                      */
   {
      printf("ErrorBuffer\n");
      err_ind=0;                       /* reset error flag                  */
/*--------------------------------------------------------------------------*/
/* search in data until buffer last channel has been read. Next channel to  */
/* read will then be channel 1. This is to get synchronized at startup.     */
/*--------------------------------------------------------------------------*/
      next_channel=ptr->cha_def[ser_idx][3]; /* set next_channel to last channel  */
next1:
      sts_snc=Sync1();                 /* look for sync characters          */
      switch(sts_snc)                  /* test if return value is ok        */
      {
         case 1:                       /* reading byte ok                   */
         break;
         case 0:                       /* timeout looking for sync          */
         printf("MUL_SER2: TIMEOUT (Sync1) reading data from line %s\n",line_to_open);
         goto next1;
         break;
         case -1:                      /* error reading data                */
         printf("MUL_SER2: ERROR (Sync1) reading data from line %s\n",line_to_open);
         goto next1;
         break;
      }
      sts_buf=ED_3(KAN);                  /* read 1 second of data, one channel*/
      switch(sts_buf)                  /* test if return value is ok        */
      {
         case 0:                       /* timeout reading data buffer       */
         printf("MUL_SER2: TIMEOUT (ED_3) reading data from line %s\n",line_to_open);
         goto next1;
         break;
         case -1:                      /* error reading data                */
         printf("MUL_SER2: ERROR (ED_3) reading data from line %s\n",line_to_open);
         goto next1;
         break;
         default:
         if(sts_buf != nbyt)
         {
            printf("MUL_SER2: Less than 1 second of data has been read on %s\n",line_to_open);
            goto next1;
         }
         break;
      }

      CHKSUM(nbyt,&CHK_SUM);           /* compute checksum of received data */
      HDR_INFO(&SEQ,&KAN,&CHK,&BPP,&SPP,&FMT,&YR,&DOY,&HR,&MIN,&SECL); /* extract header information      */
//         YR=0;
         DTE(DOY,&DAY,&MTH,YR);
//         printf("3. %2d/%2d/%2d\n",YR,MTH,DAY);
         SEC=(float)SECL;
         TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
         SECTIM (MSECS+MSEC70,&YR,&DOY,&MTH,&DAY,&HR,&MIN,&SEC); /*TESTING***/
//         printf("4. %2d/%2d/%2d\n",YR,MTH,DAY);
      if(KAN != next_channel)          /* if not last channel               */
         goto next1;
//      if(CHK != CHK_SUM)               /* test check sum                    */
//         goto next1;
/****************************************************************************/
/********************* SIMULATING TIMETAG START *****************************/
/****************************************************************************/
//      MSECS++;
/****************************************************************************/
/********************* SIMULATING TIMETAG STOPP *****************************/
/****************************************************************************/
      goto top;                        /* re-synchronized                   */

   }else{
//printf("YEAR= %d\n",YR);

      sprintf(timestring,              /* header time tag into timestring   */
      "%2d:%2d:%6.3f %2d/%2d/%4d %3d\n",HR,MIN,SEC,DAY,MTH,YR,DOY);
//      printf("SEQ=%d KANAL=%d BPP=%d SPP=%d FMT=%d CHK=%8d %8d\n",SEQ,KAN,BPP,SPP,FMT,CHK,CHK_SUM);
//printf("%s\n",timestring);
      Mark_Time1(timestring,address);  /* time tag buffer in memory pipe    */
      Convert();                       /* multiplex data into SEISLOG buffer*/
/****************************************************************************/
/********************* SIMULATING TIMETAG START *****************************/
/****************************************************************************/
//      MSECS++;                         /******** TESTING ********************/
/****************************************************************************/
/********************* SIMULATING TIMETAG STOPP *****************************/
/****************************************************************************/
      ptr->dqu[ser_idx]++;             /* increment buffer ready this line  */
   }
}
}
/****************************************************************************/
/* Function to data buffers from a serial port                              */
/****************************************************************************/
ED_3(ch)
int ch;
{
int tt;
buf_ptr=byt;
n=dev_read(fh,buf_ptr,nbyt,nbyt,0,11,0,0); /* read 'nbyt' bytes from serial port*/

    
//    exit(0);
//}
switch(n)
{
    case -1:                           /* ERROR reading data                */
    return(-1);
    break;

    case 0:                            /* TIMEOUT reading data              */
    return(0);
    break;

    default:                           /* Reading data ok, return no. read  */
    return(n);
    break;
}
}
/****************************************************************************/
/* Function to convert data buffers from IDS-24                              /
/****************************************************************************/
void Convert()
{
int ka,ll,kk;
for(ll=0;ll<ptr->rate;ll++)
{
//printf("SER_IDX=%d SEC_PTR=%d\n",ser_idx,sec_ptr);
//exit(0);
    for(ka=0;ka<ptr->cha_def[ser_idx][3];ka++)
    {
        ptr->mulpip[ser_idx][sec_ptr+ka]=tmp_buf[ka][ll];
//        if(ll==0)
//            printf("4. [%d]=tmp_buf[%d][%d]=%d\n",sec_ptr+ka,ka,ll,tmp_buf[ka][ll]);
    }
    sec_ptr=sec_ptr+ptr->cha_def[ser_idx][3];
}
}
void Conv_tempo(cur_kan)
int cur_kan;
{

register unsigned char *buf_ptr;
register unsigned char lo,me,hi;
register unsigned char b1,b2,b3,b4;
register unsigned int val;
register int verdi;
register int v1,v2,v3;
int pp;

unsigned short smp;
short sample;
pp=0;
buf_ptr= &byt[22];                     /* point to first data byte          */
while(buf_ptr < &byt[nbyt])
{
    val=0;
    b1=(*buf_ptr++);
    b2=(*buf_ptr++);
    b3=(*buf_ptr++);
    b4=(*buf_ptr++);
    val=b1;
    val=val<<8 | b2;
    val=val<<8 | b3;
    val=val<<8 | b4;
    verdi=val;
    v1=verdi;
    tmp_buf[cur_kan-1][pp]=v1;
//if(pp==0)
//    printf("3. KAN=%d VAL=%d tmp_buf[%d]=%d\n",cur_kan,v1,cur_kan-1,v1);
//printf("CUR_CHAN=%d\n",cur_kan);
//exit(0);
    pp++;
}
}
Sync1()
{
register unsigned short sync_value = 0;

buf_ptr = byt;
do
{
    n=dev_read(fh,buf_ptr,1,1,0,11,0,0);
    switch(n)
    {
        case -1:
        return(-1);
        break;
        case 0:
        return(0);
        break;
        case 1:       
        sync_value = sync_value << 8 | *buf_ptr;
        break;
    }
} while(sync_value != 0x4454);
//printf("snc\n");
return(1);
}


CHKSUM(no_byt,chksum)
int  no_byt;
unsigned char* chksum;
{
int t;
unsigned char sum;
sum=68+84;
for(t=0;t<no_byt;t++)
{
    if(t!=20)
        sum=sum+byt[t];
}

*chksum=sum;
}

HDR_INFO(seq,ch,chk,bpp,spp,fmt,aar,doy1,timer,minu,seku)
int* seq;
int* ch;
int* chk;
int* bpp;
int* spp;
int* fmt;
int* aar;
int* doy1;
int* timer;
int* minu;
int* seku;
//int* dd;
//int* ma;
//int* hh;
//int* ss;
{
int add;
unsigned short ss;
unsigned int ss1;
unsigned short save;
int sequence;
int k;
int tusen;
int hundre;
int ti;
int en;
//int aar;
//int doy1;
//int timer;
//int minu;
//int seku;
/*--------------------------------------------------------------------------*/
/*                            SEQUENCE NUMBER                               */
/*--------------------------------------------------------------------------*/
    save=byt[12];                      /* extract thousand in sequence      */
    ss=save;
    ss=ss & 0x00f0;
    ss=ss >> 4;
    tusen=ss*1000;

    ss=save;                           /* extract hundreds in sequence      */
    ss=ss & 0x000f;
    hundre=ss*100;

    save=byt[13];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in sequence          */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in sequence          */

    *seq=tusen+hundre+ti+en;
/*--------------------------------------------------------------------------*/
/*                            CHANNEL NUMBER                                */
/*--------------------------------------------------------------------------*/
    save=byt[17];
    *ch=save;                          /* extract channel number            */

/*--------------------------------------------------------------------------*/
/*                            CHECKSUM                                      */
/*--------------------------------------------------------------------------*/
    *chk=byt[20];                      /* extract checksum                  */

/*--------------------------------------------------------------------------*/
/*                            BYTES PER PACKET                              */
/*--------------------------------------------------------------------------*/
    save=byt[10];                      /* extract thousand in bpp           */

    ss=save;                           /* extract bytes per packet          */
    ss=ss & 0x00f0;
    ss=ss >> 4;
    tusen=ss*1000;

    ss=save;                           /* extract hundreds in bpp           */
    ss=ss & 0x000f;
    hundre=ss*100;

    save=byt[11];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in bpp               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in bpp               */

    *bpp=tusen+hundre+ti+en;

/*--------------------------------------------------------------------------*/
/*                            SAMPLES PER PACKET                            */
/*--------------------------------------------------------------------------*/
    save=byt[18];                      /* extract thousand in spp           */

    ss=save;                           /* extract samples per packet        */
    ss=ss & 0x00f0;
    ss=ss >> 4;
    tusen=ss*1000;

    ss=save;                           /* extract hundreds in spp           */
    ss=ss & 0x000f;
    hundre=ss*100;

    save=byt[19];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in spp               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in spp               */

    *spp=tusen+hundre+ti+en;


/*--------------------------------------------------------------------------*/
/*                            DATA FORMAT                                   */
/*--------------------------------------------------------------------------*/
    save=byt[21];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in fmt               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in fmt               */

    *fmt=ti+en;

/*--------------------------------------------------------------------------*/
/*                            TIME                                          */
/*--------------------------------------------------------------------------*/
    save=byt[4];                       /* 1 byte in time                    */
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;                          /* extract tens in fmt               */

    ss=save;
    ss=ss & 0x000f;
    en=ss;                             /* extract ones in fmt               */
//    *fmt=ti+en;
//tider
//printf("%2x %2x %2x %2x %2x %2x %2x\n",byt[1],byt[4],byt[5],byt[6],byt[7],byt[8],byt[9]);
/********************/
    save=byt[1];
    ss=save;
    ss = ss & 0x00f0;
    ss=ss>>4;
    ti=ss*10;
    ss=save;
    ss = ss & 0x000f;
    en=ss;
    *aar=ti+en;
/*****************/
    save=byt[4];
    ss=save;
    ss = ss & 0x00f0;
    ss = ss>>4;
    hundre=ss*100;
    ss=save;
    ss=ss & 0x000f;
    ti=ss*10;

    save=byt[5];
    ss=save;
    ss=ss & 0x00f0;
    ss=ss>>4;
    en=ss;
    *doy1=hundre+ti+en;
/*************************/
    ss=save;
    ss = ss & 0x000f;
    ti=ss*10;

    save=byt[6];
    ss=save;
    ss = ss & 0x00f0;
    ss = ss>>4;
    en=ss;
    *timer=ti+en;
/*************************/
    ss=save;
    ss = ss & 0x000f;
    ti=ss*10;

    save=byt[7];
    ss=save;
    ss = ss & 0x00f0;
    ss = ss>>4;
    en=ss;
    *minu=ti+en;
/************************/
    ss=save;
    ss = ss & 0x000f;
    ti=ss*10;

    save=byt[8];
    ss=save;
    ss = ss & 0x00f0;
    ss = ss>>4;
    en=ss;
    *seku=ti+en;
/***************************/

//printf("ar=%d doy=%d timer=%d min=%d sec=%d\n",aar,doy1,timer,minu,seku);
}
Mark_Time1(tstring,add)
char* tstring;
int  add;
{
char timestamp[100];
int  k;
char str[100];

register char *tmstr;                  /* pointer to time string in buffer  */
register int temp;
register unsigned int shift;
register unsigned int swap;
register unsigned short msw;
register unsigned short lsw;
register unsigned short sw;
int      i;
int      ls;
char     tmpstr[80];

/*           000000000011111111112222222222333333333344444444445555555*/
/*           012345678901234567890123456789012345678901234567890123456*/
/*           00:00:00.000 00/00/0000 000 sts 00:00:00.000 00/00/00 000  "*/

//ti[55]=0x0d;
//ti[56]=0x0a;
for(k=0;k<60;k++)
    timestamp[k]=' ';
timestamp[55]=0x0d;
timestamp[56]=0x0a;
for(k=0;k<27;k++)
    timestamp[k]=tstring[k];
if(ptr->clktyp==8)
{
    timestamp[28]='S';
    timestamp[29]='E';
    timestamp[30]='T';
}
if(ptr->clktyp==0)
{
    timestamp[28]='C';
    timestamp[29]='P';
    timestamp[30]='U';
}
timestamp[31]='\0';
if(timestamp[0]==' ')
    timestamp[0]='0';
if(timestamp[3]==' ')
    timestamp[3]='0';
if(timestamp[6]==' ')
    timestamp[6]='0';
if(timestamp[9]==' ')
    timestamp[9]='0';
if(timestamp[10]==' ')
    timestamp[10]='0';
if(timestamp[13]==' ')
    timestamp[13]='0';
if(timestamp[16]==' ')
    timestamp[16]='0';
if(timestamp[24]==' ')
    timestamp[24]='0';
if(timestamp[25]==' ')
    timestamp[25]='0';
for(k=0;k<32;k++)
    ptr->timcpy[k]=timestamp[k];
//printf("MUL_SERb: address %3d %s\n",add,timestamp);
ls=0;
for(k=0;k<64;k=k+4)
{
    shift=timestamp[k+3];
    shift = shift << 8;
    shift = shift | timestamp[k+2];
    shift = shift << 8;
    shift = shift | timestamp[k+1];
    shift = shift << 8;
    shift = shift | timestamp[k];
    ptr->mulpip[ser_idx][(ptr->cha_def[0][15]*add)+ls]=shift;
    ls++;
}
}
void    Sync()
{
register unsigned char *buf_ptr;
register unsigned short sync_value = 0;
buf_ptr = byt;
    do
    {
        numfread(buf_ptr,1);
        if(numread == 0)
            clearerr(in);
        else
            sync_value = sync_value << 8 | *buf_ptr;
        } while(sync_value != 0x4d4f);
/*putchar(0x07);*/
}
