#include< math.h>
#include<malloc.h>
#include<stdio.h>

static float sqrarg;
#define SQR(a) (sqrarg=(a),sqrarg*sqrarg)

void memcof(data,n,m,pm,cof)
float data[],*pm,cof[];
int n,m;
{
    int k,j,i;
    float p=0.0,*wk1,*wk2,*wkm,*vector();
    void free_vector();

    wk1=vector(1,n);
    wk2=vector(1,n);
    wkm=vector(1,m);
    for(j=1;j<=n;j++) p += SQR(data[j]);
    *pm=p/n;
    wk1[1]=data[1];
    wk2[n-1]=data[n];
    for(j=2;j<=n-1;j++) {
       wk1[j]=data[j];
       wk2[j-1]=data[j];
    };
    for(k=1;k<=m;k++) {
       float num=0.0,denom=0.0;
       for(j=1;j<=(n-k);j++) {
          num += wk1[j]*wk2[j];
          denom += SQR(wk1[j])+SQR(wk2[j]);
       };
       cof[k]=2.0*num/denom;
       *pm *= (1.0-SQR(cof[k]));
       for (i=1;i<=(k-1);i++)
           cof[i]=wkm[i]-cof[k]*wkm[k-i];
       if (k==m){
           free_vector(wkm,1,m);
           free_vector(wk2,1,n);
           free_vector(wk1,1,n);
           return;
       };
       for(i=1;i<=k;i++) wkm[i]=cof[i];
       for(j=1;j<=(n-k-1);j++) {
           wk1[j] -= wkm[k]*wk2[j];
           wk2[j]=wk2[j+1]-wkm[k]*wk1[j+1];
       };
    };
}

void nerror(error_text)
char error_text[];
{
 void exit();

 printf("Numerical recipes run-time error...\n");
 printf("%s\n",error_text);
 printf("...now exiting to system...\n");
 exit(1);

}

float *vector(nl,nh)
int nl,nh;
{
    float *v;
    v=(float *)malloc((unsigned)(nh-nl+1)*sizeof(float));
    if(!v) nerror("allocation failure in vector()");
    return v-nl;
}  


void free_vector(v,nl,nh)
float *v;
int nl,nh;
{ 
  free((char*) (v+nl));
}

typedef struct FCOMPLEX {float r,i;} fcomplex;

fcomplex Cadd(a,b)
fcomplex a,b;
{
   fcomplex c;
   c.r=a.r+b.r;
   c.i=a.i+b.i;
   return c;
}

fcomplex Csub(a,b)
fcomplex a,b;
{
   fcomplex c;
   c.r=a.r-b.r;
   c.i=a.i-b.i;
   return c;
}

fcomplex Cmul(a,b)
fcomplex a,b;
{
   fcomplex c;
   c.r=a.r*b.r-a.i*b.i;
   c.i=a.i*b.r+a.r*b.i;
   return c;
}

fcomplex Cdiv(a,b)
fcomplex a,b;
{
   fcomplex c;
   float r,den;
   if(fabs(b.r)>=fabs(b.i)){
      r=b.i/b.r;
      den=b.r+r*b.i;
      c.r=(a.r+r*a.i)/den;
      c.i=(a.i-r*a.r)/den;
   } else {
     r=b.r/b.i;
     den=b.i+r*b.r;
     c.r=(a.r*r+a.i)/den;
     c.i=(a.i*r-a.r)/den;
   };
   return c;
}


fcomplex Complex(re,im)
float re,im;
{
   fcomplex c;
   c.r=re;
   c.i=im;
   return c;
}

float Cabs(z)
fcomplex z;
{
   float x,y,ans,temp;
   x=fabs(z.r);
   y=fabs(z.i);
   if(x==0.0) ans=y;
   else if(y==0.0) ans=x;
   else if(x > y) {
        temp=y/x;
        ans=x*sqrt(1.0+temp*temp);
   } else {
        temp=x/y;
        ans=x*sqrt(1.0+temp*temp);
   };
   return ans;
}

fcomplex Conjg(z)
fcomplex z;
{
   fcomplex c;
   c.r=z.r;
   c.i= -z.i;
   return c;
}

fcomplex Csqrt(z)
fcomplex z;
{
   fcomplex c;
   float x,y,w,r;
   if((z.r==0.0) && (z.i==0.0)){
     c.r=c.i=0.0;
     return c;
   } else {
     x=fabs(z.r);
     y=fabs(z.i);
     if (x >= y){
        r=y/x;
        w=sqrt(x)*sqrt(0.5*(1.0+sqrt(1.0+r*r)));
     } else {
       r=x/y;
       w=sqrt(y)*sqrt(0.5*(r*sqrt(1.0+r*r)));
     };
     if(z.r>=0.0){
       c.r=w;
       c.i=z.i/(2.0*w);
     } else {
       c.i=(z.i >= 0.0) ? w: -w;
       c.r=z.i/(2.0*c.i);
     };
     return c;
   };
}

fcomplex RCmul(x,a)
float x;
fcomplex a;
{
   fcomplex c;
   c.r=x*a.r;
   c.i=x*a.i;
   return c;
}

#define NPMAX 100
#define ZERO Complex(0.0,0.0)
#define ONE Complex(1.0,0.0)

void fixrts(d,npoles)
float d[];
int npoles;
{
   int i,j,polish;
   fcomplex a[NPMAX],roots[NPMAX];
   void zroots();

   a[npoles]=ONE;
   for (j=npoles-1;j>=0;j--)
       a[j]=Complex(-d[npoles-j],0.0);
   polish=1;
   zroots(a,npoles,roots,polish);
   for(j=1;j<=npoles;j++)
      if(Cabs(roots[j]) >1.0)
        roots[j]=Cdiv(ONE,Conjg(roots[j]));;
   a[0]=Csub(ZERO,roots[1]);
   a[1]=ONE;
   for(j=2;j<=npoles;j++){
       a[j]=ONE;
       for(i=j;i>=2;i--)
          a[i-1]=Csub(ZERO,Cmul(roots[j],a[i-1])) ;
       a[0]=Csub(ZERO,Cmul(roots[j],a[0]));
   };
   for(j=0;j<=npoles-1;j++)
       d[npoles-j]=-a[j].r;
}


void predic(data,ndata,d,npoles,future,nfut)
float data[],d[],future[];
int ndata,npoles,nfut;
{
   int k,j;
   float sum,discrp,*reg,*vector();
   void free_vector();

   reg=vector(1,npoles);
   for(j=1;j<=npoles;j++) reg[j]=data[ndata+1-j];
   for(j=1;j<=nfut;j++){
      discrp=0.0;
      sum=discrp;
      for(k=1;k<=npoles;k++) sum += d[k]*reg[k];
      for(k=npoles;k>=2;k--) reg[k]=reg[k-1];
      future[j]=reg[1]=sum;
   };
   free_vector(reg,1,npoles);
}

#define EPSS 6.e-8
#define MAXIT 100

void laguer(a,m,x,eps,polish)
fcomplex a[],*x;
int m,polish;
float eps;
{
   int j,iter;
   float err,dxold,cdx,abx;
   fcomplex sq,h,gp,gm,g2,g,b,d,dx,f,x1;
   void nerror();

dxold=Cabs(*x);
for(iter=1;iter<=MAXIT;iter++){
   b=a[m];
   err=Cabs(b);
   d=f=Complex(0.0,0.0);
   abx=Cabs(*x);
   for(j=m-1;j>=0;j--) {
       f=Cadd(Cmul(*x,f),d);
       d=Cadd(Cmul(*x,d),b);
       b=Cadd(Cmul(*x,b),a[j]);
       err=Cabs(b)+abx*err;
   }; 
   err *= EPSS;
   if (Cabs(b)<=err) return;
   g=Cdiv(d,b);
   g2=Cmul(g,g);
   h=Csub(g2,RCmul(2.0,Cdiv(f,b)));
   sq=Csqrt(RCmul((float)(m-1),Csub(RCmul((float) m,h),g2)));
   gp=Cadd(g,sq);
   gm=Csub(g,sq);
   if(Cabs(gp) < Cabs(gm))gp=gm;
   dx=Cdiv(Complex((float) m,0.0),gp);
   x1=Csub(*x,dx);
   if (x->r == x1.r && x->i == x1.i) return;
   *x=x1;
   cdx=Cabs(dx);
   dxold=cdx;
   if(!polish)
     if(cdx <= eps*Cabs(*x)) return;
}
nerror("Too many iterations in routine LAGUER");
}

#define EPS 2.0e-6
#define MAXM 100

void zroots(a,m,roots,polish)
fcomplex a[],roots[];
int m,polish;

{
    int jj,j,i;
    fcomplex x,b,c,ad[MAXM];
    void laguer();
   

    for (j=0;j<=m;j++) ad[j]=a[j];
    for (j=m;j>=1;j--){
        x=Complex(0.0,0.0);
        laguer(ad,j,&x,EPS,0);
        if (fabs(x.i) <= (2.0*EPS*fabs(x.r))) x.i=0.0;
        roots[j]=x;
        b=ad[j];
        for(jj=j-1;jj>=0;jj--){
           c=ad[jj];
           ad[jj]=b;
           b=Cadd(Cmul(x,b),c);
        };
    };
    if(polish)
       for(j=1;j<=m;j++)
          laguer(a,m,&roots[j],EPS,1);
    for(j=2;j<=m;j++) {
       x=roots[j];
       for(i=j-1;i>=1;i--){
           if(roots[i].r <= x.r) break;
           roots[i+1]=roots[i];
       };
       roots[i+1]=x;
    };
} 

int ndatat=0;
float *xt=0,*yt=0,aa=0.0,abdevt=0.0;


void medfit(x,y,ndata,a,b,abdev)
float *x,*y,*a,*b,*abdev;
int ndata;
{
	int j;
	float bb,b1,b2,del,f,f1,f2,sigb,temp;
	float sx=0.0,sy=0.0,sxy=0.0,sxx=0.0,chisq=0.0;
        float rofunc();

	ndatat=ndata;
	xt=x;
	yt=y;
	for(j=1;j<=ndata;j++){
		sx +=x[j];
		sy+=y[j];
		sxy+=x[j]*y[j];
		sxx+=x[j]*x[j];
	};
	del=ndata*sxx-sx*sx;
	aa=(sxx*sy-sx*sxy)/del;
	bb=(ndata*sxy-sx*sy)/del;
	for(j=1;j<=ndata;j++)
		chisq+=(temp=y[j]-(aa+bb*x[j]),temp*temp);
	sigb=sqrt(chisq/del);
	b1=bb;
	f1=rofunc(b1);
	b2=bb+((f1>0.0)?fabs(3.0*sigb):-fabs(3.0*sigb));
	f2=rofunc(b2);
	while (f2*f1>0.0){
		bb=2.0*b2-b1;
		b1=b2;
		f1=f2;
		b2=bb;
		f2=rofunc(b2);
	}
	sigb=0.01*sigb;
	while(fabs(b2-b1)>sigb){
		bb=0.5*(b1+b2);
		if(bb==b1 || bb==b2) break;
		f=rofunc(bb);
		if(f*f1>=0.0){
			f1=f;
			b1=bb;	
		} else {
			f2=f;
			b2=bb;
		}
	}
	*a=aa;
	*b=bb;
	*abdev=abdevt/ndata;
}

extern int ndatat;
extern float *xt,*yt,aa,abdevt;

float rofunc(b)
float b;

{
	int j,n1,nmh,nml;
	float *arr,d,sum=0.0,*vector();
	void sort(),free_vector();

	arr=vector(1,ndatat);
	n1=ndatat+1;
	nml=n1/2;
	nmh=n1-nml;
	for(j=1;j<=ndatat;j++) arr[j]=yt[j]-b*xt[j];
	sort(ndatat,arr);
	aa=0.5*(arr[nml]+arr[nmh]);
	abdevt=0.0;
	for(j=1;j<=ndatat;j++){
		d=yt[j]-(b*xt[j]+aa);
		abdevt+=fabs(d);
		sum+=d>0.0?xt[j]:-xt[j];
	}
	free_vector(arr,1,ndatat);
	return sum;
}

void sort(n,ra)
int n;
float ra[];
{
	int l,j,ir,i;
	float rra;
	
	l=(n>>1)+1;
	ir=n;
	for(;;){
		if(l>1)
			rra=ra[--l];
		else{
			rra=ra[ir];
			ra[ir]=ra[1];
			if(--ir==1){
				ra[i]=rra;
				return;
			}
		}
		i=l;
		j=l<<1;
		while(j<=ir){
			if(j<ir && ra[j]<ra[j+1]) ++j;
			if(rra<ra[j]){
				ra[i]=ra[j];
				j+=(i=j);
			}
			else j=ir+1;
		}
		ra[i]=rra;
	}
}

float evlmem(fdt,cof,m,pm)
float fdt,cof[],pm;
int m;
{
	int i;
	float sumr=1.0,sumi=0.0;
	double wr=1.0,wi=0.0,wpr,wpi,wtemp,theta;
	theta=6.28318530717959*fdt;
	wpr=cos(theta);
	wpi=sin(theta);
	for(i=1;i<=m;i++){
		wr=(wtemp=wr)*wpr-wi*wpi;
		wi=wi*wpr*wtemp*wpi;
		sumr-=cof[i]*wr;
		sumi-=cof[i]*wi;
	}
	return pm/(sumr*sumr+sumi*sumi);
}
