/****************************************************************************/
/*                 VME OS-9 & PC QNX SEISLOG Software          Version 6.0  */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Pho +47 55 213408 +47 55 213414     */ 
/*                                      Fax +47 55 233337 +47 55 320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:09/02/95                       */
/* Program:PAR                                                              */
/*                                                                          */
/* Last update:                                                             */
/* 10/05/00 t.u. only clocktype 0,1,2,4 and 8 is allowed                    */
/* 01/06/00 t.u. unit 34 Earth Data Muliserial (preliminary version)        */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* PAR is used to maintain the parameter files for SEISLOG. By entering PAR */
/* the parameter file will be read into memory, and the user may modify any */
/* parameter. The arrow keys are supported. The TAB key will swap between   */
/* three different screens. The ESC key will end the session, but first you */
/* are asked to fill in 2 comments about the changes you have made. The date*/
/* will be filled in automatically.                                         */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"
#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <sys/qnxterm.h>
#include "filedef.h"
#include <sys/mman.h>
#include <fcntl.h>
#include <stdlib.h>


#define spalte1            0    /* start position 1. column                 */
#define spalte2            40   /* start position 2. column                 */
#define field1             29   /* start position values 1. column          */
#define field2             70   /* start position values 2. column          */
#define field3              7   /* start position values from PARAM_2       */
#define field_1             7   /* start position Name                      */
#define field_2            14   /* start position Component                 */
#define field_3            21   /* start position Gain                      */
#define field_4            28   /* start position Filter                    */
#define field_5            35   /* start position STA                       */
#define field_6            42   /* start position LTA                       */
#define field_7            49   /* start position Ratio                     */
#define field_8            56   /* start position AUX_1                     */
#define field_9            63   /* start position AUX_2                     */
#define field_10           70   /* start position AUX_3                     */

#define feltsize_p1        10   /* fieldsize page 1                         */
#define feltsize_p2         7   /* fieldsize page 2                         */
#define feltsize_p3         7   /* fieldsize page 3                         */
#define No_Lines_Par1_File 33   /* number of lines in PARAM_1               */
#define No_Lines_Par2_File 66   /* number of lines in PARAM_2               */

#define KEY_CURSOR_UP       1
#define KEY_CURSOR_DOWN     2
#define KEY_CURSOR_RIGHT    3
#define KEY_CURSOR_LEFT     4
#define KEY_ESC             5
#define KEY_TAB             6
#define KEY_ERROR          -1

time_t time_of_day;

/*struct tm *newtime;*/         /* pointer to structure for time & date     */
/*int     ltime;*/              /* variable for time                        */
int     rnghdr;                 /* size of header ringbuffers               */
int     bryter;                 /* on off switch                            */
int     tast;                   /* copy of key read from keyboard           */
int     counter;                /* counting number of keys                  */
int     nc;                     /* copy of counter                          */
int     lq;
int     i;
int     sq;
int     watch;
FILE    *par;                   /* pointer to parameter file                */

int     stor[20];
char    com_1[80];              /* space for 1. comment                     */
char    com_2[80];              /* space for 2. comment                     */
char    debug[80];

char    linje1[70][80];         /* current values in memory PARAM_1         */
char    linje2[70][80];         /* current values in memory PARAM_2         */
char    com[81];                /* room for one line from parameter file    */
char    character;              /* one character                            */
char    input[80];              /* input string from keyboard               */
int     err;                    /* error indicator from check_par           */
int     xphys;                  /* physical address on screen               */
int     yphys;                  /* physical address on screen               */

int     x;                      /* horizontal movement of cursor            */
int     y;                      /* vertical movement of cursor              */
int     page;                   /* page indicator                           */
int     ypar;                   /* parameter                                */

int     mintrg;                 /* minimum number of triggers               */
int     prevnt;                 /* pre-event-memory                         */
int     premax;                 /* array propagation window                 */
int     postmx;                 /* post-event-memory                        */
int     maxbuf;                 /* maximum recording time in seconds        */
int     nchan;                  /* total number of channels to digitize     */
int     nmin;                   /* number of minutes per ring-buffer        */
int     nrring;                 /* number of ring-buffers                   */
int     cont_dat;               /* switch to turn on/off cont recording     */
float   tim_corr;               /* CPU time correction                      */
int     nsamples;
int     recbytes;
int     recsize;
float   rate;                   /* samplerate                               */
float   smprat;
int     intclk;                 /* source sampling frequency                */
int     rsmp;
int     err_mail;
int     clktyp;                 /* clock type                               */
int     unit;                   /* digitizing unit                          */
int     pip_dsk;                /* indicator to record from pipe or disk    */
int     dsk_level;              /* level to start auto delete of events     */
int     minut_dur;              /* duration of minute pulse in milliseconds */
float   delay;                  /* time correction                          */
int     p_phase;                /* indicator p-phase recording              */
int     web;
int     ad_irq;                 /* A/D board interrupt                      */
int     tot_size;               /* total size to allocate for ringbuffers   */
int     min_per_file;           /* minutes per ring buffer file             */
int     autoproc;               /* auto processing                          */
int     sndmail;                /* send mail                                */
int     base1;                  /* base address 1 a/d board                 */
int     base2;                  /* base address 2 a/d board                 */
char    p_b_1[10];              /* port and baudrate clock                  */
char    p_b_2[10];              /* port and baudrate digitizer              */
char    arg;                    /* argument starting PAR                    */
int     set;                    /* int argument starting PAR                */
int     min_per_rng;
int     no_of_rng;
float   samplerate;
int     channels;
int     present;
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */

int main (argc,argv)
int argc;
char **argv;
{

    register FILE   *infile;
    register int key;
    int con_ser;
    if(argc != 2)
    {
        set=1;
    }else{
        arg=*argv[1];
        switch(arg)
        {
            case '1':
            set=1;
            break;
            case '2':
            set=2;
            break;
            case '3':
            set=3;
            break;
            case '4':
            set=4;
            break;
            case '5':
            set=5;
            break;
        }
    }
stor[0]=3;
stor[1]=4;
stor[2]=4;
stor[3]=4;
stor[4]=8;
stor[5]=8;
stor[6]=5;
stor[7]=5;
stor[8]=10;
stor[9]=10;
stor[10]=2;
stor[11]=2;


/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"PAR01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"PAR_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
con_ser=line_typ();
system("/home/seislog/rd_param");
ptr->reinit_flag=0;
min_per_rng=ptr->nmin_per_ring[0];
no_of_rng=ptr->nringbuf[0];
samplerate=ptr->srate;
channels=ptr->nchan;

if(ptr->unit==33)                      /* multiserial GBV                   */
    system("/home/seislog/rd_par_ser");
if(ptr->unit==34)                      /* multiserial IDS                   */
    system("/home/seislog/rd_par_ser");

if(term_load() == -1) {
    fprintf(stderr,"term_load error. errno=%d\n",errno);
    exit(0);
}
term_clear(TERM_CLS_SCRH);
    page=1;                           /* initiall page                       */
    counter=0;
    nc=0;
    err=0;
    bryter=0;
/*--------------------------------------------------------------------------*/
/* read all parameters from PARAMS_na into memory                           */
/*--------------------------------------------------------------------------*/

    switch(set)
    {
        case 1:
        par=fopen(PARAM_1A,"r"); /* open parameter file    */
        if(par == 0)
        {
            printf("Error opening PARAM_1A\n");
            exit(0);
        }
        break;
        case 2:
        par=fopen(PARAM_2A,"r"); /* open parameter file    */
        break;
        case 3:
        par=fopen(PARAM_3A,"r"); /* open parameter file    */
        break;
        case 4:
        par=fopen(PARAM_4A,"r"); /* open parameter file    */
        break;
        case 5:
        par=fopen(PARAM_5A,"r"); /* open parameter file    */
        break;
    }

    Par1_Mem(No_Lines_Par1_File);        /* copy parameter file into memory */
    fclose(par);                         /* close file                      */
/*--------------------------------------------------------------------------*/
/* read all parameters from PARAMS_nb into memory                           */
/*--------------------------------------------------------------------------*/

    switch(set)
    {
        case 1:
        par=fopen(PARAM_1B,"r");       /* open parameter file               */
        break;
        case 2:
        par=fopen(PARAM_2B,"r");       /* open parameter file               */
        break;
        case 3:
        par=fopen(PARAM_3B,"r");       /* open parameter file               */
        break;
        case 4:
        par=fopen(PARAM_4B,"r");       /* open parameter file               */
        break;
        case 5:
        par=fopen(PARAM_5B,"r");       /* open parameter file               */
        break;
    }

    Par2_Mem(No_Lines_Par2_File);
    fclose(par);
    New_Screen_1();                      /* write text fields to screen     */
    Cur_Val1();                          /* write actual values on screen   */
    x=0;
    y=2;
rd_par_1();
/***term_clear(TERM_CLS_SCRH);***/
    Mark_Field(1);                       /* mark first field in position x,y*/
    do
    {
        switch (key=GetKey (con_ser))
        {
            case KEY_CURSOR_DOWN:
            if(err != 1)
            {
                switch(page)
                {
                    case 1:            /* page #1                           */
                    Mark_Field(0);
                    y=y+1;
                    Mark_Field(1);
                    check_par_1(); 
                    rd_par_1();
                    break;

                    case 2:            /* page #2                           */
                    Mark_Field_2(0);
                    y=y+1;
                    Mark_Field_2(1);
                    break;

                    case 3:            /* page #3                           */
                    Mark_Field_3(0);
                    y=y+1;
                    Mark_Field_3(1);
                    break;

                    case 4:            /* page #4                           */
                    Mark_Field_4(0);
                    y=y+1;
                    Mark_Field_4(1);
                    break;

                    case 5:            /* page #5                           */
                    Mark_Field_5(0);
                    y=y+1;
                    Mark_Field_5(1);
                    break;

                }
            }
                break;
            case KEY_CURSOR_UP:
            if(err != 1)
            {
                switch(page)
                {
                    case 1:            /* page #1                           */
                    Mark_Field(0);
                    y=y-1;
                    Mark_Field(1);
                    check_par_1();
                    rd_par_1();
                    break;
                    case 2:            /* page #2                           */
                    Mark_Field_2(0);
                    y=y-1;
                    Mark_Field_2(1);
                    break;

                    case 3:            /* page #3                           */
                    Mark_Field_3(0);
                    y=y-1;
                    Mark_Field_3(1);
                    break;

                    case 4:            /* page #4                           */
                    Mark_Field_4(0);
                    y=y-1;
                    Mark_Field_4(1);
                    break;

                    case 5:            /* page #5                           */
                    Mark_Field_5(0);
                    y=y-1;
                    Mark_Field_5(1);
                    break;

                }
            }
            break;
            case KEY_CURSOR_LEFT:
            if(err != 1)
            {
                switch(page)
                {
                    case 1:            /* page #1                           */
                    Mark_Field(0);
                    x=x-1;
                    Mark_Field(1);
                    check_par_1();
                    rd_par_1();
                    break;

                    case 2:            /* page #2                           */
                    Mark_Field_2(0);
                    x=x-1;
                    Mark_Field_2(1);
                    break;

                    case 3:            /* page #3                           */
                    Mark_Field_3(0);
                    x=x-1;
                    Mark_Field_3(1);
                    break;

                    case 4:            /* page #4                           */
                    Mark_Field_4(0);
                    x=x-1;
                    Mark_Field_4(1);
                    break;

                    case 5:            /* page #5                           */
                    Mark_Field_5(0);
                    x=x-1;
                    Mark_Field_5(1);
                    break;

                }
            }
            break;
            case KEY_CURSOR_RIGHT:
            if(err != 1)
            {
                switch(page)
                {
                    case 1:            /* page #1                           */
                    Mark_Field(0);
                    x=x+1;
                    Mark_Field(1);
                    check_par_1();
                    rd_par_1();
                    break;

                    case 2:            /* page #2                           */
                    Mark_Field_2(0);
                    x=x+1;
                    Mark_Field_2(1);
                    break;

                    case 3:            /* page #3                           */
                    Mark_Field_3(0);
                    x=x+1;
                    Mark_Field_3(1);
                    break;

                    case 4:            /* page #4                           */
                    Mark_Field_4(0);
                    x=x+1;
                    Mark_Field_4(1);
                    break;

                    case 5:            /* page #5                           */
                    Mark_Field_5(0);
                    x=x+1;
                    Mark_Field_5(1);
                    break;

                }
            }
            break;
            case KEY_ESC:
            if(err != 1)
            {
                if(ptr->reclength == 0) ptr->reclength=1;
                rnghdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
//                ptr->tot_ring_size[0]=recbytes*nmin*60+rnghdr;

                term_clear(TERM_CLS_SCRH);
                Comment_line_1();
                Into_mem_1();
                Comment_line_2();
                Into_mem_2();
                for(sq=0;sq<No_Lines_Par2_File;sq++)
                {
                    for(lq=0;lq<77;lq++)
                        debug[lq]=linje2[sq][lq];
                    debug[77]='\0';
                }
                write_pars();              /* write new parameter files    */
                bryter=1;
            }
            break;

            case KEY_TAB:
            if(err != 1)
            {
                page =page+1;
                if(page > 5)
                    page=1;
                switch(page)
                {
                    case 1:            /* page #1                           */
                    term_clear(TERM_CLS_SCRH);
                    term_flush();
                    New_Screen_1();
                    Cur_Val1();
                    x=0;
                    y=2;
                    Mark_Field(1);
                    break;

                    case 2:            /* page #2                           */
                    term_clear(TERM_CLS_SCRH);
                    term_flush();
                    New_Screen_2();
                    Cur_Val2();
                    x=0;
                    y=0;
                    Mark_Field_2(1);
                    break;

                    case 3:            /* page #3                           */
                    term_clear(TERM_CLS_SCRH);
                    term_flush();
                    New_Screen_3();
                    Cur_Val3();
                    x=0;
                    y=0;
                    Mark_Field_3(1);
                    break;

                    case 4:            /* page #4                           */
                    term_clear(TERM_CLS_SCRH);
                    term_flush();
                    New_Screen_4();
                    Cur_Val4();
                    x=0;
                    y=0;
                    Mark_Field_4(1);
                    break;

                    case 5:            /* page #5                           */
                    term_clear(TERM_CLS_SCRH);
                    term_flush();
                    New_Screen_5();
                    Cur_Val5();
                    x=0;
                    y=0;
                    Mark_Field_5(1);
                    break;
                }
            }
            break;
            default:
                tast=key;
                Catch_Echo();
            break;
                }
            }
    while(bryter==0);
/*    gotoxy(0,23);*/
term_clear(TERM_CLS_SCRH);
term_flush();
term_restore();
system("/home/seislog/rd_param");

if(ptr->nmin_per_ring[0] != min_per_rng) ptr-> reinit_flag=1;
if(ptr->nringbuf[0] != no_of_rng) ptr->reinit_flag=1;
if(ptr->srate !=  samplerate) ptr -> reinit_flag=1;
if(ptr->nchan != channels) ptr-> reinit_flag=1;


if(ptr->unit==33)                      /* Multiserial GBV                   */
    system("/home/seislog/rd_par_ser");
if(ptr->unit==33)
    system("/home/seislog/par_ser");
if(ptr->rsmp==1)                       /* if we want re-sampling, start edit*/
    system("/home/seislog/par_smp");
if(ptr->err_mail==1)                   /* if we want e-mail on error, edit  */
    system("/home/seislog/e_mail_adr");
if(ptr->mail==1)                   /* if we want e-mail on event, edit  */
    system("/home/seislog/e_addr");
if(ptr->mail==2)                   /* if we want e-mail on event, edit  */
    system("/home/seislog/e_addr");
if(ptr->unit==34)                      /* Multiserial IDS                   */
{
    system("/home/seislog/rd_par_ser");
//if(ptr->unit==34)
    system("/home/seislog/par_ser");
    system("/home/seislog/rd_par_ser");
}


system("/home/seislog/chk_par");
    return(0);
}

check_par_1()
{
    int dum;
    err=0;
    term_type(21,15,"                                     ",37,TERM_NORMAL | TERM_FLUSH);

    term_cur(yphys,xphys);
    term_flush();
    switch(ypar)
    {
        case 2:
        break;

        case 3:                        /* minimum number of triggers        */
        sscanf(linje1[ypar],"%66c%d\n",com,&mintrg);
        if(mintrg < 1 )
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        break;

        case 4:                        /* pre-event-memory                  */
        sscanf(linje1[ypar],"%66c%d\n",com,&prevnt);
        if(prevnt < 1 || prevnt > 500)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        break;

        case 5:                        /* array propagation window          */
        sscanf(linje1[ypar],"%66c%d\n",com,&premax);
        if(premax < 1 || premax > 500)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        break;

        case 6:                        /* post-event-memory                 */
        sscanf(linje1[ypar],"%66c%d\n",com,&postmx);
        if(postmx < 1 || postmx > 120)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        break;

        case 7:                        /* maximum recording time in seconds */
        sscanf(linje1[ypar],"%66c%d\n",com,&maxbuf);
        if(maxbuf < 1 || maxbuf > 9999)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        break;

        case 8:                        /* total number of channels          */
        sscanf(linje1[ypar],"%66c%d\n",com,&nchan);
        if(set >=2)                    /* no change on parameter set>=2     */
        {
            if(nchan != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(nchan < 1 || nchan > 64)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        }
        break;

        case 9:                        /* samplerate                        */
        sscanf(linje1[ypar],"%66c%f\n",com,&rate);
        if(set >= 2)                   /* no change on set >=2              */
        {
            if(rate != 0.0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(rate < 0.001 || rate > 1.000)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        }
        break;

        case 10:                       /* Re sampling                       */
        sscanf(linje1[ypar],"%66c%d\n",com,&rsmp);
        if(set >= 2)                   /* no change if set >= 2             */
        {
            if(intclk != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(intclk < 0 || rsmp > 1)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        }
        break;

        case 11:                       /* type of clock                     */
        sscanf(linje1[ypar],"%66c%d\n",com,&clktyp);
        if(set >= 2)                   /* no change if set >= 2             */
        {
            if(clktyp != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(clktyp < 0 || clktyp > 8)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        if(clktyp == 3 || clktyp == 5)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        if(clktyp == 5 || clktyp == 6)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        if(clktyp == 7)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }

        }
        break;

        case 12:                       /* digitizing unit                   */
        sscanf(linje1[ypar],"%66c%d\n",com,&unit);
        if(set >= 2)
        {
            if(unit != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(unit == 10)
            goto ok12;
        if(unit == 30)
            goto ok12;
        if(unit == 31)
            goto ok12;
        if(unit == 32)
            goto ok12;
        if(unit == 33)
            goto ok12;
        if(unit == 34)
            goto ok12;
        if(unit == 35)
            goto ok12;
        if(unit == 37)
            goto ok12;
        if(unit == 38)
            goto ok12;
        if(unit == 39)
            goto ok12;
        if(unit == 11)
            goto ok12;
        if(unit == 21)
            goto ok12;
        if(unit == 36)
            goto ok12;
        if(unit == 40)
            goto ok12;
        if(unit == 41)
            goto ok12;
        if(unit == 42)
            goto ok12;
        if(unit == 53)
            goto ok12;
        if(unit == 56)
            goto ok12;
        if(unit == 63)
            goto ok12;
        if(unit == 64)
            goto ok12;
        if(unit == 65)
            goto ok12;
        if(unit == 66)
            goto ok12;
        if(unit == 70)
            goto ok12;
        if(unit == 95)
            goto ok12;
        if(unit == 96)
            goto ok12;
        if(unit == 97)
            goto ok12;
	    if(unit == 99)
            goto ok12;
        Pos_Err();
        term_cur(yphys,xphys);
        term_flush();
        err=1;
ok12:
        dum=0;
        }
        break;

        case 13:                       /* indicat to record from pipe or dsk*/
        sscanf(linje1[ypar],"%66c%d\n",com,&watch);
        if(set >= 2)
        {
            if(pip_dsk != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(watch < 0 || watch > 1)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        }
        break;

        case 14:                       /* time correction                   */
        break;

        case 15:                       /* indicator p-phase recording       */
        sscanf(linje1[ypar],"%66c%d\n",com,&web);
        if(set >= 2)
        {
            if(web != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(web < 0 || web > 1)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        }
        break;

        case 16:                       /* strapping A/D board               */
        sscanf(linje1[ypar],"%66c%d\n",com,&ad_irq);
        if(set >= 2)
        {
            if(ad_irq != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(ad_irq == 2)
            goto ok16;
        if(ad_irq == 3)
            goto ok16;
        if(ad_irq == 4)
            goto ok16;
        if(ad_irq == 5)
            goto ok16;
        if(ad_irq == 7)
            goto ok16;
        Pos_Err();
        term_cur(yphys,xphys);
        term_flush();
        err=1;
ok16:
        dum=0;
        }
        break;
        case 17:
        break;

        case 18:                       /* Level for auto delete of events   */
        sscanf(linje1[ypar],"%66c%d\n",com,&dsk_level);
        if(set >= 2)
        {
            if(dsk_level != 0 )
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(dsk_level < 0 || dsk_level > 50)
        {
            Pos_Err();
            term_cur(yphys,xphys);
            term_flush();
            err=1;
        }
        }
        break;

        case 19:                       /* duration of minute pulse millisec */
        sscanf(linje1[ypar],"%66c%d\n",com,&minut_dur);
        if(set >= 2)
        {
            if(minut_dur != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(minut_dur != 0)
        {
            if(minut_dur < 0 || minut_dur > 2000)
            {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }
        }
        break;
        case 21:                       /* number of minutes in ringbuffer   */
        sscanf(linje1[ypar],"%66c%d\n",com,&nmin);
        if(set >= 2)
        {
            if(nmin != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
            if(nmin != 0)
            {    
                if(nmin < 1 || nmin > 1000)
                {
                    Pos_Err();
                    term_cur(yphys,xphys);
                    term_flush();
                    err=1;
                }
            }
        }
        break;
        case 22:                       /* number of ring buffer files       */
        sscanf(linje1[ypar],"%66c%d\n",com,&nrring);
        if(set >= 2)
        {
            if(nrring != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(nrring !=0)
        {
            if(nrring < 2 || nrring > MAXNUMRINGBUF)
            {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }
    }
        break;
        case 23:                       /* send mail on error                */
        sscanf(linje1[ypar],"%66c%d\n",com,&err_mail);
        if(set >= 2)
        {
            if(cont_dat != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(cont_dat < 0 || err_mail > 1)
        {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
        }
    }
        break;
        case 24:                       /* CPU time correction               */
        sscanf(linje1[ypar],"%66c%f\n",com,&tim_corr);
        if(set >= 2)
        {
            if(tim_corr != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(tim_corr < 0.0 || tim_corr > 0.0)
        {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
        }
    }
        break;
        case 26:                       /* switch Auto Processing            */
        sscanf(linje1[ypar],"%66c%d\n",com,&autoproc);
        if(set >= 2)
        {
            if(autoproc != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(autoproc < 0 || autoproc > 3)
        {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
        }
    }
         break;


        case 27:                       /* switch Send Mail Automatically    */
        sscanf(linje1[ypar],"%66c%d\n",com,&sndmail);
        if(set >= 2)
        {
            if(sndmail != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(sndmail < 0 || sndmail > 3)
        {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
        }
    }
         break;

        case 28:                       /* Base address 1. a/d board         */
        sscanf(linje1[ypar],"%66c%d\n",com,&base1);
        if(set >= 2)
        {
            if(base1 != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(base1 < 0 || base1 > 500)
        {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
        }
    }
         break;

        case 29:                       /* Base address 2. a/d board         */
        sscanf(linje1[ypar],"%66c%d\n",com,&base2);
        if(set >= 2)
        {
            if(base2 != 0)
            {
                Pos_Err_2();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
            }
        }else{
        if(base2 < 0 || base2 > 500)
        {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
        }
    }
         break;
         case 30:                       /* port & baudrate  clock            */
         sscanf(linje1[ypar],"%66c%s\n",com,&p_b_1);
         present=0;
         for(i=0;i<10;i++)
         {
             if(p_b_1[i]==',')
                 present=1;
         }
         if(present==0)
         {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
         }
         break;

        case 31:                       /* port & baudrate digitizer         */
        sscanf(linje1[ypar],"%66c%s\n",com,&p_b_2);
         present=0;
         for(i=0;i<10;i++)
         {
             if(p_b_2[i]==',')
                 present=1;
         }
         if(present==0)
         {
                Pos_Err();
                term_cur(yphys,xphys);
                term_flush();
                err=1;
         }
         break;



    }
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;

}

Pos_Err()
{
    putchar(0x07);
    term_type(21,21,"*** PARAMETER NOT VALID ***",27,TERM_BLINK | TERM_FLUSH);
}

Pos_Err_2()
{
    putchar(0x07);
    term_type(21,15,"*** FOR PARAMETER SET >=2 ENTER 0 ***",37,TERM_BLINK | TERM_FLUSH);
}

/*--------------------------------------------------------------------------*/
/* print out info about current field page 2                                */
/*--------------------------------------------------------------------------*/
Info_par_2()
{

switch(x)
{
case 0:
term_type(22,0,"STATION NAME: up to 4 letter code. f.ex. BRG or BRG1        \
                   ",79,TERM_INVERSE | TERM_FLUSH);
break;
case 1:
term_type(22,0,"STATION COMPONENT: up to 4 letter code f.ex. S  Z           \
                   ",79,TERM_INVERSE | TERM_FLUSH);
break;
case 2:
term_type(22,0,"GAIN: UNIT 11:1,10,100  21:1,10,100 36:1,2,4,8 40:n.d 53:n.d\
 56:n.d.  (integer)",79,TERM_FLUSH | TERM_INVERSE);
break;
case 3:
term_type(22,0,"LOW-CUT FILTER or DIFFERENCE FILTER                         \
   (floating point)",79,TERM_INVERSE | TERM_FLUSH);
break;
case 4:
term_type(22,0,"HIGH-CUT FILTER                                             \
   (floating point)",79,TERM_INVERSE | TERM_FLUSH);
break;
case 5:
term_type(22,0,"SHORT TERM AVERAGE: Range 0.100 - 60.000 seconds            \
   (floating point)",79,TERM_INVERSE | TERM_FLUSH);
break;
case 6:
term_type(22,0,"LONG TERM AVERAGE: Range 0.500 - 120.000 seconds            \
   (floating point)",79,TERM_INVERSE | TERM_FLUSH);
break;
case 7:
term_type(22,0,"TRIGGER VALUE: 0.0 -> very big. Trigger type dependant.     \
   (floating point)",79,TERM_INVERSE | TERM_FLUSH);
break;
case 8:
term_type(22,0,"De-Trigger value: 0.0 -> very big. Trigger type dependant.  \
   (floating point)",79,TERM_INVERSE | TERM_FLUSH);
break;
case 9:
term_type(22,0,"Trigger Type: 0->NO trigger, 1->Standard, 2-Level, 3-Contino\
us        (integer)",79,TERM_INVERSE | TERM_FLUSH);
break;
case 10:
term_type(22,0,"Channel Selected for recording: 0-> No Recording, 1-> Record\
ing.      (integer)",79,TERM_INVERSE | TERM_FLUSH);
break;

}
}

/*--------------------------------------------------------------------------*/
/* print out info about current field page 1                                */
/*--------------------------------------------------------------------------*/
Info_par_1(no)
int no;
{

switch(no)
{
case 2:
term_type(22,0,"NETWORK name: 4 letter code.                                \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 3:
term_type(22,0,"MIN. NO. OF TRIGGERS: Range 1 - total number of channels spe\
cified.   (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 4:
term_type(22,0,"PRE-EVENT-MEMORY: Range 1 - 120 seconds.                    \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 5:
term_type(22,0,"ARRAY PROPAGATION WINDOW: Range 1 - 120 seconds.            \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 6:
term_type(22,0,"POST-EVENT-MEMORY: Range 1 - 120 seconds.                   \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 7:
term_type(22,0,"MAXIMUM RECORDING TIME: Range 1 - 9999 seconds.             \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 8:
term_type(22,0,"TOTAL NUMBER OF CHANNELS: Range 1 - 64.                     \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 9:
term_type(22,0,"SAMPLE INTERVAL: Range 0.005 - 1.000 seconds.               \
   (floating point)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 10:
term_type(22,0,"Re-sampling (only 3-component station) 0-no resampling 1-\
resampling      ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 11:
term_type(22,0,"TIMING: 0-CPU, 1-G35 No PPS, 2-G35 PPS, 4-RSC8000, 8-Direct\
 Timing Digitizer  ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
    term_printf(24,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 12:
term_type(22,0,"10-GBV      30-DAS1202  31-IDS24    32-GeoSys12 36-DAS1802  38-RO     40-HRD24 \
 ",79,TERM_INVERSE | TERM_FLUSH);

term_type(23,0,"53-RD3      56-RD6      63-ED24     64-ED24_NEW 70-RO_NEW   96-SYNTH. 99-ILIBGS\
                                        ",79,TERM_INVERSE | TERM_FLUSH);
term_type(24,0,"xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\
xxxxxxxxx                               ",79,TERM_INVERSE | TERM_FLUSH);
Del_Info_Line();
break;
case 13:
term_type(22,0,"Automatic restart on error 0 - not active 1 - active        \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
    term_printf(24,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 14:
term_type(22,0,"TIME CORRECTION: Range +/- anything                         \
   (floating point)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 15:
term_type(22,0,"SPARE                                                       \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 16:
term_type(22,0,"A/D BOARD INTERRUPT: 2,3,4,5,7                              \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 17:
term_type(22,0,"Number of active parameter sets to use. Range 1 - 5         \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 18:
term_type(22,0,"Level to start automatic delete of events in % Range 1-50   \
(0=>OFF)  (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 19:
term_type(22,0,"Duration minute pulse in milliseconds. Range 0 - 2000.      \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 20:
term_type(22,0,"Events of less duration than 'x' number of seconds not to be\
 recorded (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 21:
term_type(22,0,"Each ring-buffer file will contain 'x' minutes of data.     \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
Pos_Info2();
break;
case 22:
term_type(22,0,"'x' ring-buffer files will be allocated.                    \
          (integer)",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
Pos_Info2();
break;
case 23:
term_type(22,0,"SPARE                                                       \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 24:
term_type(22,0,"SPARE                                                       \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 25:
term_type(22,0,"Not Defined                                                \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 26:
term_type(22,0,"Auto Processing 1-auto, 2-auto_uch, 3-SAC, 23-2+3          \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 27:
term_type(22,0,"Send Mail when event 0 - no mail 1 - mail                  \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
    term_printf(24,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 28:
term_type(22,0,"Base Address 1. A/D board                                  \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 29:
term_type(22,0,"Base Address 2. A/D board                                  \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 30:
term_type(22,0,"Port and baudrate clock: P,BBBBB P:port B:baudrate         \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;
case 31:
term_type(22,0,"Port and baudrate digitizer: P,BBBBB P:port B:baudrate     \
                   ",79,TERM_INVERSE | TERM_FLUSH);
    term_printf(23,0,TERM_FLUSH,"                                        \
                                        ");
Del_Info_Line();
break;

}
}
/*--------------------------------------------------------------------------*/
/* left shift of buffer                                                     */
/*--------------------------------------------------------------------------*/
left_shift(felt)
int felt;
{
int i;
int ne;
    if(nc != felt)
    {
        ne=felt-nc;
         for(i=nc;i<felt;i++)
            input[i]=' ';
     }
}

int Catch_Echo()
{
char tegn;
check:
    if(tast == 13)
    {
        nc=counter;
        counter=0;
        switch(page)
        {
            case 1:
            right_shift(feltsize_p1);
            rewrite_1(feltsize_p1);
            check_par_1();
            rd_par_1();
            Info_par_1(ypar);
            break;

            case 2:
            if(x >= 2)
            {
                right_shift(stor[x+1]);
                rewrite_2(stor[x+1]);
            }else{
                left_shift(stor[x+1]);
                rewrite_2(stor[x+1]);
            }
            break;

            case 3:
            if(x >= 2)
            {
                right_shift(stor[x+1]);
                rewrite_3(stor[x+1]);
            }else{
                left_shift(stor[x+1]);
                rewrite_3(stor[x+1]);
            }
            break;

            case 4:
            if(x >= 2)
            {
                right_shift(stor[x+1]);
                rewrite_4(stor[x+1]);
            }else{
                left_shift(stor[x+1]);
                rewrite_4(stor[x+1]);
            }
            break;

            case 5:
            if(x >= 2)
            {
                right_shift(stor[x+1]);
                rewrite_5(stor[x+1]);
            }else{
                left_shift(stor[x+1]);
                rewrite_5(stor[x+1]);
            }
            break;

        }
    }else{
/**TESTING****/
        if(tast == 0x7f)
        {
           counter--;   
           term_cur(yphys,xphys+counter);
           term_flush();
            return(0);
        }


        input[counter]=toupper(tast);
        counter=counter+1;

        switch(page)
        {
            case 1:
            if(counter == feltsize_p1)
            {
                tast=13;
                goto check;
            }
            break;
            case 2:
            if(counter == stor[x+1])
            {
                tast=13;
                goto check;
            }
            break;

            case 3:
            if(counter == stor[x+1])
            {
                tast=13;
                goto check;
            }
            break;

            case 4:
            if(counter == stor[x+1])
            {
                tast=13;
                goto check;
            }
            break;

            case 5:
            if(counter == stor[x+1])
            {
                tast=13;
                goto check;
            }
            break;

        }
        tegn=tast;
/*        write(1,&tegn,1);*/
        term_printf(yphys,xphys+counter-1,TERM_INVERSE | TERM_FLUSH,"%c",tegn);
/*        term_cur(yphys,xphys+counter);*/
/*        term_flush();*/
/*        gotoxy(xphys+counter,yphys);*/
/*            dump_key();*/
    }
return(0);
}

/*--------------------------------------------------------------------------*/
/* Rewrite right shifted buffer in same field                               */
/*--------------------------------------------------------------------------*/
rewrite_1(felt)
int felt;
{
int i;
for(i=0;i<felt;i++)
{
    term_printf(yphys,xphys+i,TERM_INVERSE | TERM_FLUSH,"%c",input[i]);
    linje1[ypar][66+i]=input[i];

}
term_color(TERM_NORMAL | TERM_FLUSH);
term_cur(yphys,xphys);
term_flush();
}

/*--------------------------------------------------------------------------*/
/* Rewrite right shifted buffer in same field                               */
/*--------------------------------------------------------------------------*/
rewrite_2(felt)
int felt;
{
int i;
for(i=0;i<felt;i++)
    term_printf(yphys,xphys+i,TERM_INVERSE | TERM_FLUSH,"%c",input[i]);

term_color(TERM_NORMAL | TERM_FLUSH);
for(i=0;i<felt;i++)
    linje2[y+1][xphys+i]=input[i];
/*sscanf(input,"%4c\n",(char*)linje2+(y+1)*80+(x+1)*4);*/
term_cur(yphys,xphys);
term_flush();
}
/*--------------------------------------------------------------------------*/
/* Rewrite right shifted buffer in same field                               */
/*--------------------------------------------------------------------------*/
rewrite_3(felt)
int felt;
{
int i;
for(i=0;i<felt;i++)
    term_printf(yphys,xphys+i,TERM_INVERSE | TERM_FLUSH,"%c",input[i]);

term_color(TERM_NORMAL | TERM_FLUSH);

for(i=0;i<felt;i++)
    linje2[y+1+16][xphys+i]=input[i];
/*sscanf(input,"%7c\n",(char*)linje2+80*16+(y+1)*80+(x+1)*7);*/
term_cur(yphys,xphys);
term_flush();
}
/*--------------------------------------------------------------------------*/
/* Rewrite right shifted buffer in same field                               */
/*--------------------------------------------------------------------------*/
rewrite_4(felt)
int felt;
{
int i;
for(i=0;i<felt;i++)
    term_printf(yphys,xphys+i,TERM_INVERSE | TERM_FLUSH,"%c",input[i]);

term_color(TERM_NORMAL | TERM_FLUSH);

for(i=0;i<felt;i++)
    linje2[y+1+32][xphys+i]=input[i];
/*sscanf(input,"%7c\n",(char*)linje2+80*32+(y+1)*80+(x+1)*7);*/
term_cur(yphys,xphys);
term_flush();
}
/*--------------------------------------------------------------------------*/
/* Rewrite right shifted buffer in same field                               */
/*--------------------------------------------------------------------------*/
rewrite_5(felt)
int felt;
{
int i;
for(i=0;i<felt;i++)
    term_printf(yphys,xphys+i,TERM_INVERSE | TERM_FLUSH,"%c",input[i]);

term_color(TERM_NORMAL | TERM_FLUSH);
for(i=0;i<felt;i++)
    linje2[y+1+48][xphys+i]=input[i];
/*sscanf(input,"%7c\n",(char*)linje2+80*48+(y+1)*80+(x+1)*7);*/
term_cur(yphys,xphys);
term_flush();
}
/*--------------------------------------------------------------------------*/
/* Rigth shift of buffer                                                    */
/*--------------------------------------------------------------------------*/
right_shift(felt)
int felt;
{
int i;
int ne;
    if(nc != felt)
    {
        ne=felt-nc;
        for(i=0;i<nc;i++)
            input[felt-1-i]=input[felt-1-ne-i];
        for(i=0;i<ne;i++)
            input[i]=' ';
     }
}

/*-------------------------------------------------------------------------*/
/* write out new parameter files                                           */
/*-------------------------------------------------------------------------*/
write_pars()
{
int k;

    switch(set)
    {
        case 1:
        par=fopen(PARAM_1A,"r+");      /* open parameter file               */
        break;
        case 2:
        par=fopen(PARAM_2A,"r+");      /* open parameter file               */
        break;
        case 3:
        par=fopen(PARAM_3A,"r+");      /* open parameter file               */
        break;
        case 4:
        par=fopen(PARAM_4A,"r+");      /* open parameter file               */
        break;
        case 5:
        par=fopen(PARAM_5A,"r+");      /* open parameter file               */
        break;
    }

    for(k=0;k< No_Lines_Par1_File; k++)
        fprintf(par,"%s\n",linje1[k]);
    fclose(par);

    switch(set)
    {
        case 1:
        par=fopen(PARAM_1B,"r+");      /* open parameter file               */
        break;
        case 2:
        par=fopen(PARAM_2B,"r+");      /* open parameter file               */
        break;
        case 3:
        par=fopen(PARAM_3B,"r+");      /* open parameter file               */
        break;
        case 4:
        par=fopen(PARAM_4B,"r+");      /* open parameter file               */
        break;
        case 5:
        par=fopen(PARAM_5B,"r+");      /* open parameter file               */
        break;
    }

    for(k=0;k< No_Lines_Par2_File; k++)
        fprintf(par,"%s\n",linje2[k]);
    fclose(par);
}

Comment_line_2()
{
int k;
int lk,ls;
char cm2[85];
register int key;
for(lk=0;lk<85;lk++)
    cm2[lk]='-';
xphys=0;
yphys=14;

/*highvideo();*/
/*drawtext("                                             ",xphys,yphys);*/
/*gotoxy(xphys,yphys);*/
for(lk=0;lk<76;lk++)
    cm2[lk]=linje1[1][lk];
cm2[76]='\0';
for(lk=0;lk<76;lk++)
    com_2[lk]=linje1[1][lk];
com_2[76]='\0';
term_printf(yphys,xphys,TERM_INVERSE | TERM_FLUSH,"%s",cm2);
xphys=29;
/*gotoxy(xphys,yphys);*/
term_cur(yphys,xphys);
term_flush();
counter=0;
for(k=0;k<45;k++)
{
    key=GetKeyEcho();
    if(key != 13)
    {
        if(key > 31 && key < 123 )
        {
            tast=key;
            Catch_comment_2();
         }
    }else{
        goto fini;
    }
}
fini:
k=0; /* kan fjernes */
/*    lowvideo();*/
}
Catch_comment_2()
{
char tegn;
    tegn=tast;
    com_2[counter]=tegn;
    counter=counter+1;
/*    write(1,&tegn,1);*/
/*    gotoxy(xphys+counter,yphys);*/
}
Into_mem_2()
{
int k;
int nb;
int t;
linje1[1][28]='*';
if(counter !=0)
{
    for(t=0;t<counter;t++)
        linje1[1][t+29]=com_2[t];
    for(t=counter+29;t<75;t++)
        linje1[1][t]='.';
    linje1[1][75]='*';
    linje1[1][76]='\0';
}
}

Into_mem_1()
{
int k;
int nb;
int t;                               /* NEW */
auto char buf[26];

    time_of_day=time(NULL);
    sprintf((char*)linje1+3,"%s",_ctime(&time_of_day,buf));
    linje1[0][27]='*';    /* NEW */
    linje1[0][28]='*';    /* NEW */
    if(counter!=0)
    {                                       /* NEW */
        for(t=0;t<counter;t++)              /* NEW */
            linje1[0][t+29]=com_1[t];       /* NEW */
        for (t=counter+29;t<75;t++)            /* NEW */
            linje1[0][t]='.';               /* NEW */
        linje1[0][75]='*';                  /* NEW */
        linje1[0][76]='\0';                 /* NEW */

    }
/*        sscanf(com_1,"%50c",(char*)linje1+28);*/
}

Comment_line_1()
{
int k;
int lk,ls;
char cm1[85];       /* NEW */
register int key;
for(lk=0;lk<85;lk++)
    cm1[lk]='-';
xphys=0;
yphys=12;
term_type(6,15,"Enter comments into parameter file:",35,TERM_NORMAL | TERM_FLUSH);
term_type(8,15,"If old comment is OK, press return.",35,TERM_NORMAL | TERM_FLUSH);
term_type(10,15,"New comment, just start writing.",32,TERM_NORMAL | TERM_FLUSH);

/*highvideo();*/
/*drawtext("                                             ",xphys,yphys);*/
/*gotoxy(xphys,yphys);*/             /* NEW */

sscanf(linje1[0],"%76c",cm1);       /* NEW */
cm1[76]='\0';
sscanf(linje1[0],"%76c\n",com_1);       /* NEW */
com_1[76]='\0';

term_printf(yphys,xphys,TERM_INVERSE | TERM_FLUSH,"%s",cm1);                   /* NEW */
xphys=29;                            /* NEW */
term_cur(yphys,xphys);
term_flush();
/*gotoxy(xphys,yphys);*/
counter=0;
for(k=0;k<45;k++)
{
    key=GetKeyEcho();
    if(key != 13)
    {
        if(key > 31 && key < 123 )
        {
            tast=key;
            Catch_comment_1();
         }
    }else{

        goto fini;
    }
}
fini:
k=0; /* kan fjernes */
/*    lowvideo();*/
}
Catch_comment_1()
{
char tegn;
    tegn=tast;
    com_1[counter]=tegn;
    counter=counter+1;
/*    write(1,&tegn,1);*/
/*    gotoxy(xphys+counter,yphys);*/
}

/*--------------------------------------------------------------------------*/
/* Print out current values from memory buffer PARAM_2  01-16               */
/*--------------------------------------------------------------------------*/
Cur_Val2()
{
int lp;
int ts;
char tegn;

    for(lp=1;lp<17;lp++)
    {
        for(ts=0;ts<72;ts++)
        {
            tegn=linje2[lp][4+ts];
            term_printf(lp+1,4+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
}
/*--------------------------------------------------------------------------*/
/* Print out current values from memory buffer PARAM_2  17-32               */
/*--------------------------------------------------------------------------*/
Cur_Val3()
{
int lp;
int ts;
char tegn;

    for(lp=17;lp<33;lp++)
    {
        for(ts=0;ts<72;ts++)
        {
            tegn=linje2[lp][4+ts];
            term_printf(lp-16+1,4+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
}
/*--------------------------------------------------------------------------*/
/* Print out current values from memory buffer PARAM_2  33-48               */
/*--------------------------------------------------------------------------*/
Cur_Val4()
{
int lp;
int ts;
char tegn;

    for(lp=33;lp<49;lp++)
    {
        for(ts=0;ts<72;ts++)
        {
            tegn=linje2[lp][4+ts];
            term_printf(lp-16*2+1,4+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
}
/*--------------------------------------------------------------------------*/
/* Print out current values from memory buffer PARAM_2  49-64               */
/*--------------------------------------------------------------------------*/
Cur_Val5()
{
int lp;
int ts;
char tegn;

    for(lp=49;lp<65;lp++)
    {
        for(ts=0;ts<72;ts++)
        {
            tegn=linje2[lp][4+ts];
            term_printf(lp-16*3+1,4+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
}
New_Screen_2()
{
term_type(0,0,"Cha Name Comp Gain Filt_Low Filt_Hig STA   LTA   Trig-Lev   De-Trg-Lev TT RC",76,TERM_NORMAL | TERM_FLUSH);
term_type(1,0,"____________________________________________________________________________",76,TERM_NORMAL | TERM_FLUSH);

term_type(2,0,"  1 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(3,0,"  2 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(4,0,"  3 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(5,0,"  4 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(6,0,"  5 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(7,0,"  6 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(8,0,"  7 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(9,0,"  8 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(10,0,"  9 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(11,0," 10 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(12,0," 11 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(13,0," 12 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(14,0," 13 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(15,0," 14 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(16,0," 15 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(17,0," 16 ",4,TERM_NORMAL | TERM_FLUSH);

}

New_Screen_3()
{
term_type(0,0,"Cha Name Comp Gain Filt_Low Filt_Hig STA   LTA   Trig-Lev   De-Trg-Lev TT RC",76,TERM_NORMAL | TERM_FLUSH);
term_type(1,0,"____________________________________________________________________________",76,TERM_NORMAL | TERM_FLUSH);

term_type(2,0," 17 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(3,0," 18 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(4,0," 19 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(5,0," 20 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(6,0," 21 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(7,0," 22 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(8,0," 23 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(9,0," 24 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(10,0," 25 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(11,0," 26 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(12,0," 27 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(13,0," 28 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(14,0," 29 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(15,0," 30 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(16,0," 31 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(17,0," 32 ",4,TERM_NORMAL | TERM_FLUSH);

}

New_Screen_4()
{
term_type(0,0,"Cha Name Comp Gain Filt_Low Filt_Hig STA   LTA   Trig-Lev   De-Trg-Lev TT RC",76,TERM_NORMAL | TERM_FLUSH);
term_type(1,0,"____________________________________________________________________________",76,TERM_NORMAL | TERM_FLUSH);

term_type(2,0," 33 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(3,0," 34 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(4,0," 35 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(5,0," 36 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(6,0," 37 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(7,0," 38 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(8,0," 39 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(9,0," 40 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(10,0," 41 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(11,0," 42 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(12,0," 43 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(13,0," 44 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(14,0," 45 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(15,0," 46 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(16,0," 47 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(17,0," 48 ",4,TERM_NORMAL | TERM_FLUSH);

}

New_Screen_5()
{
term_type(0,0,"Cha Name Comp Gain Filt_Low Filt_Hig STA   LTA   Trig-Lev   De-Trg-Lev TT RC",76,TERM_NORMAL | TERM_FLUSH);
term_type(1,0,"____________________________________________________________________________",76,TERM_NORMAL | TERM_FLUSH);

term_type(2,0," 49 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(3,0," 50 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(4,0," 51 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(5,0," 52 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(6,0," 53 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(7,0," 54 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(8,0," 55 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(9,0," 56 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(10,0," 57 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(11,0," 58 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(12,0," 59 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(13,0," 60 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(14,0," 61 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(15,0," 62 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(16,0," 63 ",4,TERM_NORMAL | TERM_FLUSH);
term_type(17,0," 64 ",4,TERM_NORMAL | TERM_FLUSH);

}
Mark_Field_2(sw)
int sw;
{
int ts;
int s1;
char tegn;
    if(y > 15)
        y=0;
    if(y < 0)
        y=15;
    if(x > 10)   /* check if more than 10 fields RIGHT */
        x=0;
    if(x < 0)
        x=10;
    xphys=0;
    for(s1=0;s1<x+1;s1++)
        xphys=xphys+stor[s1];
    xphys=xphys+x+1;
    yphys=y + 2;
    ypar=y;

    for(ts=0;ts<stor[x+1];ts++)
    {
        tegn=linje2[ypar+1][xphys+ts];
        if(sw==1)
        {
            term_printf(yphys,xphys+ts,TERM_INVERSE | TERM_FLUSH,"%1c",tegn);
        }else{
            term_printf(yphys,xphys+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
    Info_par_2();
      term_cur(yphys,xphys);
      term_flush();
}

Mark_Field_3(sw)
int sw;
{
int ts;
int s1;
char tegn;
    if(y > 15)
        y=0;
    if(y < 0)
        y=15;
    if(x > 10)
        x=0;
    if(x < 0)
        x=10;
    xphys=0;
    for(s1=0;s1<x+1;s1++)
        xphys=xphys+stor[s1];
    xphys=xphys+x+1;
    yphys=y + 2;
    ypar=y+16;

    for(ts=0;ts<stor[x+1];ts++)
    {
        tegn=linje2[ypar+1][xphys+ts];
        if(sw==1)
        {
            term_printf(yphys,xphys+ts,TERM_INVERSE | TERM_FLUSH,"%1c",tegn);
        }else{
            term_printf(yphys,xphys+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
    Info_par_2();
      term_cur(yphys,xphys);
      term_flush();
}

Mark_Field_4(sw)
int sw;
{
int ts;
int s1;
char tegn;
    if(y > 15)
        y=0;
    if(y < 0)
        y=15;
    if(x > 10)
        x=0;
    if(x < 0)
        x=10;
    xphys=0;
    for(s1=0;s1<x+1;s1++)
        xphys=xphys+stor[s1];
    xphys=xphys+x+1;
    yphys=y + 2;
    ypar=y+32;
    for(ts=0;ts<stor[x+1];ts++)
    {
        tegn=linje2[ypar+1][xphys+ts];
        if(sw==1)
        {
            term_printf(yphys,xphys+ts,TERM_INVERSE | TERM_FLUSH,"%1c",tegn);
        }else{
            term_printf(yphys,xphys+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
    Info_par_2();
      term_cur(yphys,xphys);
      term_flush();
}

Mark_Field_5(sw)
int sw;
{
int ts;
int s1;
char tegn;
    if(y > 15)
        y=0;
    if(y < 0)
        y=15;
    if(x > 10)
        x=0;
    if(x < 0)
        x=10;
    xphys=0;
    for(s1=0;s1<x+1;s1++)
        xphys=xphys+stor[s1];
    xphys=xphys+x+1;
    yphys=y + 2;
    ypar=y+48;
    for(ts=0;ts<stor[x+1];ts++)
    {
        tegn=linje2[ypar+1][xphys+ts];
        if(sw==1)
        {
            term_printf(yphys,xphys+ts,TERM_INVERSE | TERM_FLUSH,"%1c",tegn);
        }else{
            term_printf(yphys,xphys+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
    Info_par_2();
      term_cur(yphys,xphys);
      term_flush();
}

Mark_Field(sw)
int sw;
{
int ts;
char tegn;
    if(y >=17)
        y=2;
    if(y <=1)
        y=16;
    if(x <= -1)
        x=x+2;
    if(x >=2)
        x=x-2;
    if(x == 0)
        {
            xphys=field1;
            yphys=y;
            ypar=y;
        }else{
            xphys=field2;
            yphys=y;
            ypar=y+15;
        }
    for(ts=0;ts<feltsize_p1;ts++)
    {
        tegn=linje1[ypar][66+ts];
        if(sw==1)
        {
            term_printf(yphys,xphys+ts,TERM_INVERSE | TERM_FLUSH,"%1c",tegn);
        }else{
            term_printf(yphys,xphys+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
    Info_par_1(ypar);
    term_cur(yphys,xphys);
    term_flush();
}

/*--------------------------------------------------------------------------*/
/* Print out current values from memory buffer PARAM_1                      */
/*--------------------------------------------------------------------------*/
Cur_Val1()
{
int lp;
int ip;
int ts;
char tegn;

    for(lp=2;lp<17;lp++)
    {
        for(ts=0;ts<feltsize_p1;ts++)
        {
            tegn=linje1[lp][66+ts];
            term_printf(lp,field1+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
    for(lp=17;lp<32;lp++)
    {
        for(ts=0;ts<feltsize_p1;ts++)
        {
            tegn=linje1[lp][66+ts];
            term_printf(lp-15,field2+ts,TERM_NORMAL | TERM_FLUSH,"%1c",tegn);
        }
    }
}

/*--------------------------------------------------------------------------*/
/* Print out all text headers on screen                                     */
/*--------------------------------------------------------------------------*/
New_Screen_1()
{
    term_type(2,spalte1,"02 Network Name             :",29,TERM_NORMAL | TERM_FLUSH);
    term_type(3,spalte1,"03 Minim. number of Triggers:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(4,spalte1,"04 Pre-Event_Memory_________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(5,spalte1,"05 Array Propagation Window :",29,TERM_NORMAL | TERM_FLUSH);
    term_type(6,spalte1,"06 Post-Event-Memory________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(7,spalte1,"07 Maximm Recording_________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(8,spalte1,"08 Number of Channels_______:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(9,spalte1,"09 Sample Interval__________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(10,spalte1,"10 Resampling (3-comp only)_:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(11,spalte1,"11 Type of Clock____________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(12,spalte1,"12 Digitizing Unit__________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(13,spalte1,"13 Automatic reboot on error:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(14,spalte1,"14 Time Correction__________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(15,spalte1,"15 Start sockets____________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(16,spalte1,"16 A/D board Interrupt______:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(2,spalte2,"17 Number of Parameter-sets_:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(3,spalte2,"18 Disk space level_________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(4,spalte2,"19 Duration minute pulse____:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(5,spalte2,"20 Event duration low limit_:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(6,spalte2,"21 Minutes per ring-buffer__:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(7,spalte2,"22 Number of ring-buffers___:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(8,spalte2,"23 SPARE____________________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(9,spalte2,"24 SPARE____________________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(10,spalte2,"25 SPARE____________________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(11,spalte2,"26 Auto Processing__________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(11,spalte2,"26 Auto Processing__________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(12,spalte2,"27 Send Mail________________:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(13,spalte2,"28 Base Address 1 A/D_______:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(14,spalte2,"29 Base Address 2 A/D_______:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(15,spalte2,"30 Port/Baudrate clock______:",29,TERM_NORMAL | TERM_FLUSH);
    term_type(16,spalte2,"31 Port/Baudrate digitizer__:",29,TERM_NORMAL | TERM_FLUSH);

}

/*--------------------------------------------------------------------------*/
/* Copy parameter file PARAM_2 into character matrix linje2[50][80]         */
/*--------------------------------------------------------------------------*/
void Par2_Mem(records)
int records;                        /* Number of lines in parameter file    */
{
int ls;                             /* Counter                              */
int ld;                             /* Counter                              */
for(ld=0;ld<records;ld++)           /* Read 'records' lines from file       */
{
    Get_Parameter();                /* Get copy of the line                 */
    for(ls=0;ls<80;ls++)
        linje2[ld][ls]=com[ls];      /* Put into correct position in memory  */
}
}

/*--------------------------------------------------------------------------*/
/* Copy parameter file PARAM_1 into character matrix linje1[50][80]         */
/*--------------------------------------------------------------------------*/
void Par1_Mem(records)
int records;                        /* Number of lines in parameter file    */
{
int ls;                             /* Counter                              */
int ld;                             /* Counter                              */
/*term_clear(TERM_CLS_SCRH);*/

for(ld=0;ld<records;ld++)           /* Read 'records' lines from file       */
{
    Get_Parameter();                /* Get copy of the line                 */
    for(ls=0;ls<80;ls++)
        linje1[ld][ls]=com[ls];      /* Put into correct position in memory  */
}
}
/*--------------------------------------------------------------------------*/
/* Read one line of characters from the parameter file                      */
/*--------------------------------------------------------------------------*/
void Get_Parameter()
{
int i;
int ch;
    for(i=0;(i<80) && ((ch=fgetc(par)) != EOF) && (ch !='\n'); i++)
        com[i]=ch;
    com[i]='\0';
/*printf("%s\n",com);*/
/*getch();*/
}
int GetKeyEcho()
{
int c;
int left;
int kb;
    fflush(stdin);
    fflush(stdout);
    left=kbhit();
    c=getche();                    /* Read 1 character from keyboard        */
    if(c != 0xff && c != 0x1b)
    {
        if(c == 0x0a)
        {
            kb=7;
            return(kb);            /* CR pressed */
        }else{
            kb=c;
            return(kb);           /* Return valid key */
        }
     }
return(-1);
}
Del_Info_Line()
{
    term_printf(17,0,TERM_FLUSH,"                                        \
                                        ");
    term_printf(18,0,TERM_FLUSH,"                                        \
                                        ");
    term_printf(19,0,TERM_FLUSH,"                                        \
                                        ");
    term_printf(20,0,TERM_FLUSH,"                                        \
                                        ");

}
Pos_Info1()
{
    term_printf(18,0,TERM_INVERSE | TERM_FLUSH,"1 second of data =>%7d \
bytes. %6d minutes of data:%10d bytes."\
,recbytes,nmin,recbytes*nmin*60);
}
Pos_Info2()
{
double meg;
float snrring;
float srecbytes;
float snmin;
snrring=(float)nrring;
srecbytes=(float)recbytes;
//printf("%f\n",srecbytes);
snmin=(float)nmin;
//printf("%f\n",snmin);
//meg=(nrring*recbytes*nmin*60)/1000000.0;
meg=(snrring*srecbytes*snmin*60.)/1000000.0;
//printf("%f\n",meg);
ptr->rngsystem=(int)meg;
//printf("%d\n",ptr->rngsystem);
term_printf(17,20,TERM_INVERSE | TERM_FLUSH,"Disk allocation current\
 configuration");

term_printf(18,0,TERM_INVERSE | TERM_FLUSH,"\
Bytes per second of data :%10d \
  Minutes per ringbuffer   :%10d", \
recbytes,nmin);
rnghdr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;

term_printf(19,0,TERM_INVERSE | TERM_FLUSH,"\
Bytes per ringbuffer     :%10d \
  Number of ringbuffers    :%10d",\
recbytes*nmin*60+rnghdr,nrring);


term_printf(20,0,TERM_INVERSE | TERM_FLUSH,"\
Minutes of continous data:%10d \
  Megabytes to allocate    :%12.3f",\
nrring*nmin,meg);

}
rd_par_1()
{
int dum;
sscanf(linje1[3],"%66c%d\n",com,&mintrg);
sscanf(linje1[4],"%66c%d\n",com,&prevnt);
sscanf(linje1[5],"%66c%d\n",com,&premax);
sscanf(linje1[6],"%66c%d\n",com,&postmx);
sscanf(linje1[7],"%66c%d\n",com,&maxbuf);
sscanf(linje1[8],"%66c%d\n",com,&nchan);
sscanf(linje1[9],"%66c%f\n",com,&rate);
smprat=1./rate;
nsamples=(int)smprat;
sscanf(linje1[10],"%66c%d\n",com,&intclk);
sscanf(linje1[11],"%66c%d\n",com,&clktyp);
sscanf(linje1[12],"%66c%d\n",com,&unit);
sscanf(linje1[13],"%66c%d\n",com,&pip_dsk);
sscanf(linje1[15],"%66c%d\n",com,&web);
sscanf(linje1[16],"%66c%d\n",com,&ad_irq);
sscanf(linje1[18],"%66c%d\n",com,&dsk_level);
sscanf(linje1[19],"%66c%d\n",com,&minut_dur);
sscanf(linje1[21],"%66c%d\n",com,&nmin);
sscanf(linje1[22],"%66c%d\n",com,&nrring);
sscanf(linje1[23],"%66c%d\n",com,&cont_dat);
sscanf(linje1[24],"%66c%f\n",com,&tim_corr);

switch(unit)
{
    case 10:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 30:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 31:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 33:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 34:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 35:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 37:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 38:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 39:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 11:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=(recsize-HEADER*4)/2+HEADER*4;
    break;
    case 21:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=(recsize-HEADER*4)/2+HEADER*4;
    break;
    case 36:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 40:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 41:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 42:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 53:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 56:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 63:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 64:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 65:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 70:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 95:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 96:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
    case 97:
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;
    break;
}
    recsize=nsamples*nchan*4+HEADER*4;
    recbytes=recsize;

}





