/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:24/03/96                       */
/* Program:PAR_GPS                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program is started by the current clock program. It attaches a proxy*/
/* and makes this proxy id known to the clock program via the shared memory.*/
/* Then it waites for the clock program to trigger it every valid GPS time. */
/* This trigger let the program continue, and the next statements initiate a*/
/* pulse on the printer port.                                               */
/*                                                                          */
/****************************************************************************/  
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/dev.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <graph.h>
#include <conio.h>
#define  SIZ 256

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

void main()
{   
FILE     *fp;                          /* file pointer stdin                */
pid_t    proxy;
unsigned msg;
int      id;                           /* buffer switch                     */
int      i;                            /* Counter                           */
int      gps_garm_id;                  /* process id signalling process     */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"PAR_GPS_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"PAR_GPS_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* attach proxy to be triggered by clock program                            */
/*--------------------------------------------------------------------------*/
ptr->gps_proxy=0;                      /* initialize to 0                   */
if((proxy=qnx_proxy_attach(0,0,0,-1))==-1)
{
    printf("PAR_GPS Unable to attach proxy.\n");
    return;
}    
ptr->gps_proxy=proxy;                  /* make id known to clock program    */
outp(0x378,0x00);                      /* set d0-d7 on par port to 0        */
for (;;)
{
    gps_garm_id=Receive(0,&msg,sizeof(msg)); /* receive trigger from proxy  */
    outp(0x378,inp(0x378) ^ 0x04);     /* invert bit 0                      */
    delay(500);                        /* duration of pulse in msec         */
    outp(0x378,inp(0x378) ^ 0x04);     /* invert bit 0                      */
//printf("ok\n");
}   /* End of indefinite loop */
}   /* End of main */




