/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:24/03/96                       */
/* Program:PAR_MRK                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program is started by the current clock program. It attaches a proxy*/
/* and makes this proxy id known to the clock program via the shared memory.*/
/* Then it waites for the clock program to trigger it every minute. This    */
/* trigger let the program continue, and the next statements initiate a     */
/* pulse on the printer port. This pulse can be connected to a relay that   */
/* will make minute mark on f.ex. a helicorder.                             */
/*                                                                          */
/****************************************************************************/  
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/dev.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <graph.h>
#include <conio.h>

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

void main()
{   
unsigned bit;                          /* bit on par port 0-9               */
pid_t    proxy;
unsigned msg;
unsigned int cop;
int      i;                            /* Counter                           */
int      gar_min_id;                  /* process id signalling process     */
int      hold;                         /* number of millisecond to hold puls*/
long     hrn;                          /* hours                             */
long     min;                          /* minutes                           */
long     day;                          /* days                              */
long     mon;                          /* months                            */
long     yrn;                          /* year                              */
long     doy;                          /* day of year                       */
float    sec;                          /* seconds                           */
double   msecsn;                       /* total number of seconds           */
double   sectot;                       /* total number of seconds           */
double   secold;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MIN_MRK_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MIN_MRK_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* attach proxy to be triggered by clock program                            */
/*--------------------------------------------------------------------------*/
ptr->mrk_proxy=0;                      /* initialize to 0                   */
if((proxy=qnx_proxy_attach(0,0,0,-1))==-1)
{
    printf("MIN_MRK Unable to attach proxy.\n");
    return;
}    
gar_min_id=qnx_name_locate(0,"clk_pps",0,&cop);
if(gar_min_id == -1)
{
    printf("PAR_MRK: Can't find CLK_GEN. SEISLOG not active ?\n");
    exit(0);
}
ptr->mrk_proxy=proxy;                  /* make id known to clock program    */
bit=0x04;                              /* activate bit D0 on par port       */
secold=0;
for (;;)
{
    gar_min_id=Receive(0,&msg,sizeof(msg)); /* receive trigger from proxy  */
//    sectot=(float)ptr->icmde;
//    sectim(&yrn,&mon,&day,&hrn,&min,&doy,sectot,&sec); /* conv sec back     */
//printf("%f\n",sectot-secold);
//if((sectot-secold) > 60)
//{
//    putchar(0x7);
//    printf("error\n");
//}
//secold=sectot;
//    if(ptr->dbg==1)
//        printf("Minute mark: %2d:%2d:%6.3f\n",hrn,min,sec);
if(ptr->whole_second==0)
{
outp(0x378,0x00);                      /* set d0-d7 on par port to 0        */
outp(0x378,inp(0x378) ^ 0x04);         /* invert bit 2                      */
sleep(1);                             /* duration of pulse in msec         */
outp(0x378,inp(0x378) ^ 0x04);         /* invert bit 2 back to normal       */
printf("MINUTE\n");
}
}   /* End of indefinite loop */
}   /* End of main */




high(bit)
unsigned char bit;
{
    outp(0x378,inp(0x378) | bit);      /* set bit on par port high          */
}
low(bit)
unsigned char bit;
{
unsigned char bitcomp;                 /* 1 complement                      */
    bitcomp=~bit;
    outp(0x378,inp(0x378) & bitcomp);  /* set bit on par port low           */
}
/*--------------------------------------------------------------------------*/
/* called by: BUF_PIPE                                                      */
/*--------------------------------------------------------------------------*/
/* SECTIM                                                                   */
/* convert total number of seconds back to year,month,day,etc...            */
/* input : sectot     total number of seconds                               */
/* output: yrn        year                                                  */
/*         mon        month                                                 */
/*         day        day                                                   */
/*         hrn        hours                                                 */
/*         min        minutes                                               */
/*         doy        day of year                                           */
/*         sec        seconds                                               */
/*--------------------------------------------------------------------------*/
sectim(yrn,mon,day,hrn,min,doy,sectot,secs)
long   *yrn;
long   *mon;
long   *day;
long   *hrn;
long   *min;
long   *doy;
double sectot;
float  *secs;
{
    double secday;
    double secyr;
    long   year;
    long   day_of_year;
    long   month;
    long   days;
    long   i;
    float  ind;
    int    start_year;
    secday=86400.0;
    secyr=31536000.0;
    if(ptr->clktyp==8)
    {
        start_year=0;
    }else{
        start_year=70;    /* change to 70 */
    }
    for(i=start_year;i< 201;i+=1)
    {
        ind=0.0;
        if(i%4==0)ind=1.0;
        if(sectot < (secyr+ind*secday))
        {
           year=i;
           *yrn=year;
           goto found_year;
        }
        else
            sectot=sectot-(secyr+ind*secday);
    }
    year=-1;
    found_year:
    day_of_year=sectot/secday+1;
    *doy=day_of_year;
    dte(year,day_of_year,&month,&days);
    *mon=month;
    *day=days;
    sectot=sectot-(day_of_year-1)*secday;
    *hrn=sectot/3600;
    sectot=sectot-*hrn*3600.0;
    *min=sectot/60;
    *secs=sectot-*min*60.0;
}

/*--------------------------------------------------------------------------*/
/* called by: BUF_PIPE                                                      */
/*--------------------------------------------------------------------------*/
/* DTE                                                                      */
/* Convert Day Of Year to Date                                              */
/* Input : doy                                                              */
/* Output: mon,day                                                          */
/*--------------------------------------------------------------------------*/
dte(yrn,doy,mon,day)
long yrn;
long doy;
long *mon;
long *day;
{
int   mth[13];                         /* days in each month                */
int   j,m,n;                           /* counters                          */
    
/* assign days of month to array                                            */

for (j=1;j< 8; j+=2)
{
    mth[j] = 31;
}
for(j=8;j<13;j+=2)
{
    mth[j] = 31;
}
mth[2]=28;
mth[4]=30;
mth[6]=30;
mth[9]=30;
mth[11]=30;
if(yrn%4==0)
    mth[2]=29;
m=0;
for(j=1;j<13;j++)
{
    m=m+mth[j];
    n=doy-m;
    if(n<=0)
    {
        *mon=j;
        *day=n + mth[j];
        break;
    }
}
}       




