/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:02/08/99                       */
/* Program:PK                                                               */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/*                                                                          */
/****************************************************************************/     

#include <math.h>
#include "shar_mem.h"
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include "filedef.h"
#include <stdio.h>
#include <conio.h>
#include <graph.h>
#include <sys/qnxterm.h>
#include <stdlib.h>
#include <term.h>
#include <sys/qioctl.h>
#include <signal.h>
#include <string.h>
#define  XL 639.
#define  YL 479.
#define  M2(arr,x,y) (*((arr)+(y)*(dimx)+(x))) /* define matrix for data    */
#define  DASHED 0xc0c0                 /* linetype after coherence method   */
#define  SIZ 256                       /* max size of input line            */
#define  HST 7.5
#define  LIK 0.75

FILE     *buf;
FILE     *evt;                         /* pointer to event file to plot     */
FILE     *coh;                         /* pointer results 3-component       */
FILE     *sta;                         /* pointer station file              */
FILE     *phs;                         /* pointer phases                    */
FILE     *pri;                         /* pointer default channels          */
FILE     *mail;                        /* pointer to mail file              */
FILE     *lst;
FILE     *loc;

float prev_max;
float    procoh1();                    /* return type from procoh1          */
float    cross();                      /* return type from cross            */
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
float    *matr;                        /* pointer to data matrix            */

main(argc,argv)
int argc;
char *argv[];
{
int pp;
float p_snitt_sta;
float s_snitt_sta;
float d_snitt;
float Mb;
float p_coher;
float p_coher_peak;
float s_coher_peak;
float s_coher;
int pre;
int fra,til;
int  ne;
int   equalpp;
int   equalss;
float zeroto;
float ffra,ftil;
char  value[80];                       /* text for graphic output           */
char  cur_sta[10];
float percent;                         /* percent of full scale (1.00)      */
float p_prelim;                        /* preliminar p                      */
float s_prelim;                        /* preliminar s                      */
float p_prelim2;                       /* preliminar p                      */
float s_prelim2;                       /* preliminar s                      */
float ph1,ph2;                         /* possible phases                   */
int ntops;                             /* number of tops                    */
int top[2000];
int      nsm;                          /* number of smooth iterations       */
int      bryter;
int      start_indx;                   /* index to start computing average  */
int      stopp_indx;                   /* index to stopp computing average  */
float    cut;                          /* seconds to cut in front of p      */     
float    n_p,n_s;                      /* new p & new s                     */
float    zoom;                         /* seconds to include in front of p  */
float    zooml;                        /* total zoom length in seconds      */
int      ext1,ext2;                    /* extract data from-to              */
int      no_ext;                       /* number of extracted samples       */
int      sh;                           /* counter                           */
int      pkt;                          /* counter                           */
int      each_flt;                     /* pixels per filter                 */
int      no_tre_comp;                  /* number of 3-component station     */
float    pvalid;                       /* valid p phase                     */
float    svalid;
float    avstand;                      /* distance station-event            */
int      dimx;                         /* dimension data matrix             */
int      dimy;                         /* dimension data matrix             */
float    start_sec;                    /* second to start computing average */
float    stopp_sec;                    /* second to stop computing average  */
int      tre_start[MAXCHA];            /* start channels 3-component station*/
int      chan[MAXCHA];                 /* channel number to plot            */
char     stcode[4*MAXCHA];             /* station codes                     */
char     comp[4*MAXCHA];               /* component codes                   */

int      stop;                         /* second to stop coherence anal.    */
float    ma[MAXCHA];                   /* time max amplitude in channel     */
float    pavg;                         /* average p time                    */
float    savg;                         /* average s time                    */
int      lik;                          /* counter valid p and s times       */
int      equalp[MAXCHA];               /* counter equal p phases            */
int      equals[MAXCHA];               /* counter equal s phases            */
float    velocityf[10];                /* velocity from station file        */
int      ndepth;                       /* no. of depths to test (stationfile*/
float    h0;                           /* starting depth (station file)     */
float    c11,c22,c33;                  /* coda parameters (station file)    */
float    flon,flat;
char     stasjon[20];
float    slon1,slon2,slat1,slat2;      /* min,max lon and lat (station file)*/
float    slatt[100],slonn[100];        /* station lat and lon (station file)*/
char     stat[100][8];                 /* station names from station file   */
int      nstation;                     /* number of stations in station file*/
float    x11,y11;                      /* coordinates in kilometers         */
float    slat,slon;                    /* latitude,longitude current station*/
float    lat,lon;                      /* latitude,longitude current event  */
int      r_kan[MAXCHA];                /* channel read from 'threecomp'     */
float    r_fl[MAXCHA];                 /* filter low from 'threecomp'       */
float    r_fh[MAXCHA];                 /* filter high from 'threecomp'      */
int      r_amp[MAXCHA];                /* max amp from 'threecomp'          */
float    r_tamp[MAXCHA];               /* time max amp from 'threecomp'     */
float    r_max_snr[MAXCHA];            /* max SNR from 'threecomp'          */
float    r_t_snr[MAXCHA];              /* time max SNR from 'threecomp'     */
float    r_dur[MAXCHA];                /* duration from 'threecomp'         */
float    r_pp[MAXCHA];                 /* p-phase time from 'threecomp'     */
float    r_ss[MAXCHA];                 /* s-phase time from 'threecomp'     */
float    r_s_p[MAXCHA];                /* s-p time from 'threecomp'         */
int      r_dis[MAXCHA];                /* distance from 'three p'         */
float    coherence;                    /* max coherence from 'three  p'    */
float    t_coher;                      /* time max coherence from 'threecomp*/
float    velocity;                     /* velocity from 'threecomp'         */
float    RMS;                          /* rms from 'threecomp'              */
float    azimuth;                      /* azimuth from 'threecomp'          */
float    c_p_ph;                       /* p-phase 3-comp from 'three p'   */
float    c_s_ph;                       /* s-phase 3-comp from 'threecomp'   */
float    c_s_p;                        /* s-p 3-comp from 'threecomp'       */
int      c_distance;                   /* distance 3-comp from 'threecomp'  */
char     statf[10];                    /* station from header               */
int      arrow;                        /* vertical position of arrows       */
int      dsp;                          /* displacement traces               */
int      no_of_filters;                /* number of filters to use          */
int      threec;                       /* switch to indicate 3-comp anal.   */
float    dura;                         /* duration of event                 */
float    p_ph;                         /* P-phase time in seconds from start*/
float    s_ph;                         /* S-phase time in seconds from start*/
int      color;                        /* color for drawing                 */
int      plot;                         /* switch plot=1 plot plot=0 no plot */
int      irate;                        /* samples per second                */
int      filter;                       /* index filters                     */
float    flow[20],fhig[20];            /* arrays for preset filters         */
int      orden;                        /* filter order                      */
float    next;                         /* displacement in units for next plt*/
float    next2;                        /* displacement in units for next plt*/
float    xsc;                          /* scaling factor x-axis             */
int      ww;                           /* window size in samples coherence  */
int      indx;                         /* number of samples in event        */
float    hvor;                         /* time max amplitude                */
int      from;                         /* index to extract data             */
int      to;                           /* index to extract data             */
float    valfl;                        /* value extracted from data matrix  */
int      smp;                          /* index                             */
char     infile1[100];                 /* input filename or DPX no          */
char     longf[100];                   /* file to look for                  */
char     head[2000];                   /* header of event                   */
FILE     *fp;                          /* pointer to input                  */
char     thead[200];                    /* time string of event header       */
float    c1[20000];                    /* original data for coherece        */
float    c2[20000];                    /* original data for coherence       */
float    c3[20000];                    /* original data for coherence       */
float    pv[20000];                    /* plot vector                       */
float    rat[20000];                   /* array for SNR                     */
int      iworkl[20000];                /* data from one buffer (long)       */
float    data[20000];                  /* original data current trace       */
float    coher[20000];                 /* coherence                         */
float    n4;                           /* coherence                         */
int      ipts;                         /* start sample                      */
int      lpts;                         /* stop sample                       */
float    pvel;                         /* velocity of p wave                */
float    psrat;                        /* Vp Vs ratio                       */
float    oprocoh[10];                  /* results from coherence            */

int      nchtot;                       /* selected number of channels to plt*/
int      dpx;                          /* detection number                  */
int      i;                            /* counter                           */
int      k;                            /* counter                           */
int      j;                            /* counter                           */
int      l;                            /* counter                           */
int      curdpx;                       /* current dpx number                */
int      tot;                          /* total duration of event in sec    */
long     year;                         /* year from header                  */
long     month;                        /* month from header                 */
long     day;                          /* day from header                   */
long     hour;                         /* hour from header                  */
long     min;                          /* minute from header                */
float    sec;                          /* seconds from header               */
int      nchan;                        /* number of channels from header    */
float    srate;                        /* samplerate from header            */
int      ntotal;                       /* ntotal from header                */
int      unit;                         /* unit from header                  */
int      nsamp;                        /* nsamp from header                 */
int      recsize;                      /* from header                       */
int      reclength;                    /* from header                       */
int      ichan;                        /* channel counter                   */
int      isamp;                        /* counter/index  of samples         */
int      kbuf;                         /* buffer counter                    */
float    samp_indx;                    /* time max ratio                    */
float    snr_max;                      /* max ratio                         */
float    p[MAXCHA];                    /* storage p-phases                  */
float    s[MAXCHA];                    /* storage s-phases                  */
float    d[MAXCHA];                    /* storage duration                  */
int      ws,wl;                        /* short & long window stalta        */
int      maxamp2;                      /* maximum amplitude of trace        */
float    ttotal;                       /* total duration of event           */
float    time1;                        /* start time relative to first sampl*/
float    time2;                        /* end time relative to first sample */
int      ret;                          /* return value defchan              */
int      manual;                       /* from console or automatic         */
/*--------------------------------------------------------------------------*/
/* initialize coherence values                                              */
/*--------------------------------------------------------------------------*/
float    n4_max=0.0;
int      smp_indx=0;
float    azi=0.0;
float    vel=0.0;
float    rms=0.0;
/*--------------------------------------------------------------------------*/
/* preset filter values                                                     */
/*--------------------------------------------------------------------------*/
flow[0]=3.0;
fhig[0]=7.0;

flow[1]=3.0;
fhig[1]=6.0;

flow[2]=3.0;
fhig[2]=5.0;

flow[3]=2.0;
fhig[3]=5.0;

flow[4]=2.0;
fhig[4]=7.0;

flow[5]=2.0;
fhig[5]=8.0;

bryter=0;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"PK open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"PK mmap failed : %s\n",strerror(errno));
    exit(1);
}
if (argc==2)
{ 
//    printf("%s\n",argv[1]);
    sprintf(longf,"%s\0",argv[1]);
    plot=0;
    no_of_filters=1;
    manual=0;
    goto cont;
}
/*--------------------------------------------------------------------------*/
/* if no file name in argument, ask for parameters                          */
/*--------------------------------------------------------------------------*/
printf("Enter DPX no. or -1 for filename or -2 for DPX.LIS:\n");
scanf("%d",&dpx);
/*--------------------------------------------------------------------------*/
/* if dpx=0, ask for filename                                               */
/*--------------------------------------------------------------------------*/
switch(dpx)
{
    case -1:
    printf("Enter filename:\n");
    scanf("%s",&infile1);
    sprintf(longf,"/home/events/");
    strcat(longf,infile1);
    break;

    case -2:
    system("rm -f /home/work/dpx.lis");
    system("/home/seislog/fil_dpx");
    system("more /home/work/dpx.lis");
    printf("Enter DPX no. from DPX.LIS:\n");
    scanf("%d",&dpx);

    if((lst=fopen("/home/work/dpx.lis","r"))==NULL)
    {
        printf("Could not open file containing dpx summary\n");
        exit(0);
    }
    bryter=1;
    for(;;)
    {
        if(fscanf(lst,"%5d %s",&i,longf) != -1)
        {
            printf("%5d %s\n",i,longf);
            if(i==dpx)
            {
                printf("Event: %s\n",longf);
                goto fin;              
            }
        }else{
            printf("Number %d is not in DPX.LST\n",dpx);
            fclose(lst);
            exit(0);
        }
    }
fin:
//    fclose(lst);

    break;

    default:
    dpxnam(dpx,infile1,&ttotal);       /* recorded, extract filename        */
    sprintf(longf,"/home/events/");
    strcat(longf,infile1);
    break;
}
manual=1;
printf("Enter PLOT:\n");
scanf("%d",&plot);
//printf("File path: %s\n",longf);
cont:
if((int)(evt=fopen(longf,"r"))!=NULL)  /* try to open the event file        */
{
    goto found;
}else{
    printf("Can not find file: %s\n",longf);
    exit(0);
}
found:
/*--------------------------------------------------------------------------*/
/* Read header part of event                                                */
/*--------------------------------------------------------------------------*/
fseek(evt,0,0);
fread(head,1024,1,evt);
/*--------------------------------------------------------------------------*/
/* extract header information from event header                             */
/*--------------------------------------------------------------------------*/
hdr_info(head,&year,&month,&day,&hour,&min,&sec,&nchan,&srate,&ntotal,&unit,&nsamp,&recsize,&reclength,&pre);
ttotal=(float)(ntotal/nsamp);
//printf("Start time: %2d:%2d:%6.3f %2d/%2d/%4d\n",hour,min,sec,day,month,year);
sprintf(thead,"%2d:%2d:%6.3f %2d/%2d/%4d\n",hour,min,sec,day,month,year);
thead[23]='\0';
statf[0]=head[160];
statf[1]=head[161];
statf[2]=head[162];
statf[3]='\0';
//printf("Total recording (seconds):%7.2f  S-rate: %7.3f Channels: %3d Station: %s\n",
//ttotal,srate,nchan,statf); 
/*--------------------------------------------------------------------------*/
/*   Get station codes and components for NCHAN channels                    */
/*--------------------------------------------------------------------------*/
stacomp(nchan,stcode,comp,head);
//for(i=0;i<nchan;i++)
//    printf("%s %s\n",stcode[i],comp[i]);
/*--------------------------------------------------------------------------*/
/* allocate memory                                                          */
/*--------------------------------------------------------------------------*/
dimx=nchan;
dimy=(int)ttotal*nsamp;
if(!(matr=(float *)malloc(dimx*dimy*sizeof(float)) ))
{
    printf("Out of memory !!");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* read whole event into memory matrix                                      */
/*--------------------------------------------------------------------------*/
ttotal=ttotal-1;
isamp=0;                               /* record counter                    */
for(kbuf = (1024/reclength)+1; kbuf < (int)ttotal+(1024/reclength)+1; kbuf++)
{
    fseek(evt,kbuf*reclength,SEEK_SET);/* position at next record           */
    fread(iworkl,reclength,1,evt);     /* read 1 block of data              */
    dmx_buf(dimx,isamp,nsamp,nchan,(int)ttotal,iworkl); /* demultiplex record    */
    isamp=isamp+1;                     /* icrement record counter           */
}
fclose(evt);
/*--------------------------------------------------------------------------*/
/* get default channels to plot from /home/seislog/pr/pick.def              */
/*--------------------------------------------------------------------------*/
defchan(chan,&nchtot,tre_start,&no_tre_comp);
//for(k=0;k<nchtot;k++)
//    printf("%5d %5d\n",k,chan[k]);
//printf("\n");
//getch();
//printf("No of 3-component stations %5d\n",no_tre_comp);
j=0;
for(k=0;k<no_tre_comp;k++)
{
    for(i=0;i<3;i++)
    {
//        printf("%5d %5d\n",j,tre_start[j]);
        j++;
    }
}
//printf("NCHAN %d NCHTOT %d\n",nchan,nchtot);
//printf("here 2\n");
//getch();
//exit(0);
time1=0.0;                             /* start time                        */
time2=ttotal;                          /* stop time                         */
//time2=110;
from=(int)(time1/srate);               /* start index                       */
to  =(int)(time2/srate);               /* stop index                        */
irate=(int)(1./srate);                 /* samples per second                */
//printf("Samples per second: %d\n",irate);

coh=fopen("/home/work/netw","w");  /* open file for results selected ch */
system("rm -f /home/work/newmail.txt");
mail=fopen("/home/work/newmail.txt","w"); /* open file for results selected ch */

system("rm -f /home/work/phases"); /* remove old file                   */
phs=fopen("/home/work/phases","w"); /* open file for phases             */

/*--------------------------------------------------------------------------*/
/* print heading to screen                                                  */
/*--------------------------------------------------------------------------*/
      printf("COMP CH    AMP  T-AMP MAX-SNR   T-SNR     DUR P-Phase S-Phase    S-P  DIS\n");
fprintf(mail,"\nCOMP CH    AMP  T-AMP MAX-SNR   T-SNR     DUR P-Phase S-Phase    S-P  DIS\n");

orden=3;                               /* filter order                      */
ws=5;                                 /* short window (sta)                */
wl=600;                                /* long window (lta)                 */     


no_of_filters=1;
each_flt=(int)YL/no_of_filters;
each_flt=(int)YL/2;
dsp=each_flt/4;               /* displacement in pixels between    */


/*--------------------------------------------------------------------------*/
/* loop through active channels                                             */
/*--------------------------------------------------------------------------*/
//for(ichan=0;ichan<3;ichan++)   /* channel to be processed           */
for(ichan=0;ichan<nchtot;ichan++)   /* channel to be processed           */
{
    if(plot==1)                        /* plot ?                            */
        GraphicsMode();                /* set to graphics mode              */
    next=dsp;                        /* zero line diplacement in pixels   */
/*--------------------------------------------------------------------------*/
/* start main loop FILTER                                                   */
/*--------------------------------------------------------------------------*/
    for(filter=0;filter<no_of_filters;filter++)
    {
        design(orden,"BP","BU",30.0,0.3,flow[filter],fhig[filter],srate);
        isamp=ntotal;                  /* counter for number of samples     */
        indx=0;
        for(smp=from;smp<to;smp++)     /* extract channel to process vector */
        {
            valfl=M2(matr,chan[ichan]-1,smp);
            data[indx]=valfl;
            indx=indx+1;               /* counting samples                  */
        }
        xsc=XL/(float)indx;            /* scale factor x-axis               */

        REMOVE_DC(indx,data);          /* remove DC                         */
        apply(data,indx,1);            /* Apply FILTER                      */
        for(i=0;i<indx;i++)            /* move to plot vector               */
            pv[i]=data[i]* -1;         /* TURN POLARITY FOR PLOTTING        */
        SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
// PLOT1: ORIGINAL DATA FILTERED
        if(plot==1)                    /* plot filtered data                */
        {    
            color=14;                  /* YELLOW                            */
            PlotVec(color,indx,next,pv,xsc);
            stco_label(chan,ichan,next,stcode,comp,thead);
            arrow=next;
            next=next+dsp*2;
        }
/*--------------------------------------------------------------------------*/
/* find maximum amplitude and sample index in signal                        */
/*--------------------------------------------------------------------------*/
        conv2(indx,data,srate,&hvor,&maxamp2);
        ma[ichan]=hvor;                /* save time max amp each channel    */
printf("Chan: %2d MAX AMP at: %6.2f\n",ichan,hvor);
/*--------------------------------------------------------------------------*/
/* find maximum SNR, time of max. SNR and array of running SNR              */
/* to find an initial start value, a noise level is computed from:          */
/* start_sec -> stopp_sec                                                   */
/*--------------------------------------------------------------------------*/
        start_sec=0.0;
        stopp_sec=ptr->prevnt[0]-5.0;
//        printf("PREEVENT=%d\n",pre);
        stopp_sec=pre-5.0;
        if(stopp_sec < 0.0)
            stopp_sec=2.0;
//stopp_sec=30;
        stalta(start_sec,stopp_sec,srate,indx,data,ichan,ws,wl,&snr_max,&samp_indx,rat);
        conv8(indx,rat,srate,&samp_indx,&snr_max);

        for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
            pv[i]=rat[i];
        SCALETO_1(indx,pv,dsp);        /* scale plot vector                 */
        for(j=0;j<indx;j++)
            pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
// PLOT2: STA/LTA RATIO
        if(plot==1)
        {    
            color=11;                  /* CYAN                              */
            PlotVec(color,indx,next,pv,xsc);
            next=next+dsp;
        }
        nsm=100;
        SMOOTH(indx,rat,c1);
        for(i=0;i<nsm;i++)
        {
            SMOOTH(indx,c1,c2);
            SMOOTH(indx,c2,c1);
        }

        for(j=0;j<indx;j++)
            c1[j]=(c1[j]*1000.)*(c1[j]*1000);

        for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
            pv[i]=c1[i];
        SCALETO_1(indx,pv,dsp);        /* scale plot vector                 */
        for(j=0;j<indx;j++)
            pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
// PLOT3: STA/LTA RATIO SMOOTHED
        if(plot==1)
        {    
            color=11;                  /* CYAN                              */
            PlotVec(color,indx,next,pv,xsc);
            next=next+dsp;
        }
        DERIV(start_sec,stopp_sec,srate,c1,indx);
/*--------------------------------------------------------------------------*/
/* find 2 highest peaks (p and s) and time of occurrence                    */
/*--------------------------------------------------------------------------*/
       MAXONE(c1,c2,indx);
       zeroto=pre/4.0;
       for(j=0;j<(int)(zeroto/srate);j++)
           c2[j]=0.0;
       for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
           rat[i]=c2[i];

       for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
           pv[i]=c2[i];
       SCALETO_1(indx,pv,dsp);        /* scale plot vector                 */
       for(j=0;j<indx;j++)
           pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
// PLOT4: STA/LTA RATIO SMOOTHED DERIVATED
       if(plot==1)
       {    
           color=11;                  /* CYAN                              */
           PlotVec(color,indx,next,pv,xsc);
           next=next+dsp;
       }
       start_sec=0.0;
       stopp_sec=samp_indx+60.;


//printf("1. SAMP_INDX %6.2f START: %6.2f STOPP: %6.2f\n",samp_indx,start_sec,stopp_sec);
       SEARCH(samp_indx,start_sec, stopp_sec,c2,indx,srate,&p_prelim2);
//printf("2. SAMP_INDX %6.2f START: %6.2f STOPP: %6.2f\n",samp_indx,start_sec,stopp_sec);
       SEARCH(samp_indx,start_sec, stopp_sec,c2,indx,srate,&s_prelim2);
//printf("P_PRELIM: %6.2f S_PRELIM: %6.2f\n",p_prelim2,s_prelim2);
       if(s_prelim2 > p_prelim2)
       {
           p[ichan]=p_prelim2;                 /* save time of p-phase each channel */
           s[ichan]=s_prelim2;                 /* save time of s-phase each channel */
           p_ph=p_prelim2;
           s_ph=s_prelim2;
        }else{
           p[ichan]=s_prelim2;                 /* save time of p-phase each channel */
           s[ichan]=p_prelim2;                 /* save time of s-phase each channel */
           p_ph=s_prelim2;
           s_ph=p_prelim2;
       }
       start_sec=ma[ichan]-4.0;
       stopp_sec=ma[ichan]+4.0;
       SEARCH_S(samp_indx,start_sec, stopp_sec,rat,indx,srate,&s_prelim2);
printf("S_PRELIM %6.2f\n",s_prelim2);
       s_ph=s_prelim2;
/*--------------------------------------------------------------------------*/
/* find duration of event from p-phase to end of trigger                    */
/*--------------------------------------------------------------------------*/
        for(i=0;i<zeroto;i++)
            data[i]=0;
        duration(pre,p_ph,s_ph,srate,indx,data,&dura);
        d[ichan]=dura;                 /* save duration each channel        */
        if(plot==1)
        {
            color=12;                  /* RED                               */
            TimScale(arrow,dsp,color,indx,next,xsc,srate,p_ph,s_ph,dura);
            next=next+dsp;  
        }
/*--------------------------------------------------------------------------*/
/* plot from p-10 seconds + 20 seconds                                      */
/*--------------------------------------------------------------------------*/
       zoom=3.0;       
       zooml=s_ph+2.0;
       if(zooml > 5.0)
           zooml=5.0;       
       ext1=(int)((p_ph-zoom)/srate);
       ext2=(int)((p_ph-zoom)/srate)+(int)(zooml/srate);
       no_ext=ext2-ext1;       
       xsc=XL/(float)no_ext;            /* scale factor x-axis               */
       j=0;
       for(i=ext1;i<ext2;i++)            /* move to plot vector               */
       {
           pv[j]=data[i] * -1.0;       /* turn polarity for plotting        */
           j++;
       }
       cut=p_ph-zoom;
       n_p= p_ph-cut;
       n_s= s_ph-cut;
       SCALETO_1(no_ext,pv,dsp);      /* scale to dsp/2                    */
       if(plot==1)                    /* plot filtered data                */
       {    
           color=14;                  /* YELLOW                            */
           PlotVec(color,no_ext,next,pv,xsc);
           arrow=next;
           next=next+dsp;
           color=12;                   /* RED                               */
           TimScale3(arrow,dsp,color,no_ext,next,xsc,srate,n_p,n_s,dura);
       }
/*--------------------------------------------------------------------------*/
/* write results to screen and /home/work/newmail.txt                       */
/*--------------------------------------------------------------------------*/
        output(chan,stcode,ichan,flow[filter],fhig[filter],maxamp2,hvor,snr_max,samp_indx,dura-p_ph,p_ph,s_ph);
/*--------------------------------------------------------------------------*/
/* if no 3-comp analysis, write phases and coda for map to /home/work/phases*/
/*--------------------------------------------------------------------------*/
cur_sta[0]=stcode[(chan[ichan]-1)*4];
cur_sta[1]=stcode[(chan[ichan]-1)*4+1];
cur_sta[2]=stcode[(chan[ichan]-1)*4+2];

        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
        fprintf(phs," %10.2f    1 \n",p_ph);

        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
        fprintf(phs," %10.2f    2 \n",s_ph);

        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
        fprintf(phs," %10.2f    3 \n",dura-p_ph);
/*--------------------------------------------------------------------------*/
/* set filter coeficients back again                                        */
/*--------------------------------------------------------------------------*/
        design(orden,"BP","BU",30.0,0.3,flow[filter],fhig[filter],srate);
    }    /* end of filter loop */
    if(plot==1)    
        getch();
}       /* end of channel loop */ 
fclose(phs);
//        system("rm -f /home/work/epiout"); /* make sure no old epicneter     */
//        system("epi< /home/work/phases>/home/work/epiout");
//        system("cp /home/work/phases /home/work/phase"); /* save phase readings */
//        system("/home/seislog/map3");
//        exit(0);
if(no_tre_comp > 0)
{
    for(pkt=0;pkt<no_tre_comp;pkt++)
    {
        each_flt=(int)YL/5;
        dsp=each_flt/2;               /* displacement in pixels between    */
        if(plot==1)                        /* plot ?                            */
            GraphicsMode();                /* set to graphics mode              */
        next=dsp;                        /* zero line diplacement in pixels   */
        for(ichan=0;ichan<3;ichan++)   /* channel to be processed           */
        {
            isamp=ntotal;                  /* counter for number of samples     */
            indx=0;
            for(smp=from;smp<to;smp++)     /* extract channel to process vector */
            {
                valfl=M2(matr,tre_start[ichan+pkt*3]-1,smp);
                data[indx]=valfl;
                indx=indx+1;               /* counting samples                  */
            }
            xsc=XL/(float)indx;            /* scale factor x-axis               */

            switch(ichan)              /* test for channel                  */
            {
                case 0:
                for(smp=0;smp<indx;smp++)
                    c1[smp]=data[smp]; /* save channel 1 for coherence      */
                break;
                case 1:
                for(smp=0;smp<indx;smp++) /* save channel 2 for coherence   */
                    c2[smp]=data[smp];
                break;
                case 2:
                for(smp=0;smp<indx;smp++) /* save channel 3 for coherence   */
                    c3[smp]=data[smp];
                break;
            }
        }
        design(orden,"BP","BU",30.0,0.3,flow[0],fhig[0],srate);
        REMOVE_DC(indx,c1);        /* remove DC                         */
        apply(c1,indx,1);          /* Apply FILTER                      */
        REMOVE_DC(indx,c2);        /* remove DC                         */
        apply(c2,indx,1);          /* Apply FILTER                      */
        REMOVE_DC(indx,c3);        /* remove DC                         */
        apply(c3,indx,1);          /* Apply FILTER                      */

        for(i=0;i<indx;i++)            /* move to plot vector               */
            pv[i]=c1[i] * -1.0;        /* turn polarity for plotting        */
        SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
        if(plot==1)                    /* plot filtered data                */
        {    
            color=14;                  /* YELLOW                            */
            PlotVec(color,indx,next,pv,xsc);
            ichan=0+pkt*3;
statf[0]=stcode[(tre_start[ichan]-1)*4];
statf[1]=stcode[(tre_start[ichan]-1)*4+1];
statf[2]=stcode[(tre_start[ichan]-1)*4+2];
statf[3]=stcode[(tre_start[ichan]-1)*4]+3;
statf[4]='\0';
            stco_label(tre_start,ichan,next,stcode,comp,thead);
            arrow=next;
            next=next+dsp*2;
        }
        for(i=0;i<indx;i++)            /* move to plot vector               */
            pv[i]=c2[i] * -1.0;        /* turn polarity for plotting        */
        SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
        if(plot==1)                    /* plot filtered data                */
        {    
            color=14;                  /* YELLOW                            */
            PlotVec(color,indx,next,pv,xsc);
            ichan=1+pkt*3;
            stco_label(tre_start,ichan,next,stcode,comp,thead);
            arrow=next;
            next=next+dsp*2;
        }
        for(i=0;i<indx;i++)            /* move to plot vector               */
            pv[i]=c3[i] * -1.0;        /* turn polarity for plotting        */
        SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
        if(plot==1)                    /* plot filtered data                */
        {    
            color=14;                  /* YELLOW                            */
            PlotVec(color,indx,next,pv,xsc);
            ichan=2+pkt*3;
            stco_label(tre_start,ichan,next,stcode,comp,thead);
            arrow=next;
            next=next+dsp*2;
        }

        start_sec=0.0;
        stopp_sec=ptr->prevnt[0]-5.0;

        stopp_sec=pre-5.0;
        if(stopp_sec < 0.0)
            stopp_sec=2.0;

        stalta(start_sec,stopp_sec,srate,indx,c1,ichan,ws,wl,&snr_max,&samp_indx,rat);

        for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
            pv[i]=rat[i];
        stalta(start_sec,stopp_sec,srate,indx,c2,ichan,ws,wl,&snr_max,&samp_indx,rat);

        for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
            pv[i]=pv[i]+rat[i];
        stalta(start_sec,stopp_sec,srate,indx,c3,ichan,ws,wl,&snr_max,&samp_indx,rat);

        for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
            pv[i]=pv[i]+rat[i];
/* Here pv contains the sum of SNR all 3 components                         */


///       SEARCH(samp_indx,start_sec, stopp_sec,pv,indx,srate,&p_prelim2);
///stopp_sec=pre+20.;
//start_sec=(ma[0]+ma[1]+ma[2])/3.0-5.0;
//stopp_sec=(ma[0]+ma[1]+ma[2])/3.0+5.0;
///       SEARCH(samp_indx,start_sec, stopp_sec,pv,indx,srate,&s_prelim2);
///printf("P-Prelim: %6.3f S-Prelim: %6.2f\n",p_prelim2,s_prelim2);
       s_coher_peak=s_prelim2;
       svalid=s_prelim2;


        SCALETO_1(indx,pv,dsp);        /* scale plot vector                 */
        for(j=0;j<indx;j++)
            pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
        if(plot==1)
        {    
            color=11;                  /* CYAN                              */
            PlotVec(color,indx,next,pv,xsc);
            next=next+dsp;
        }



/*--------------------------------------------------------------------------*/
/* find start and stop index for coherence based on previously found        */
/* p-phases and maximum amplitudes.                                         */
/* start 10 seconds before P, stop 10 seconds after max amplitude           */
/*--------------------------------------------------------------------------*/
    for(smp=0;smp<indx;smp++)
        coher[smp]=0.0;
//uth
equalpp=same_phase(p);
switch(equalpp)
{
    case 123:
    printf("P-phase: All equal\n");
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 12:
    printf("P-phase: 0-1 equal\n");
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 13:
    printf("P-phase: 0-2 equal\n");
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 23:
    printf("P-phase: 1-2 equal\n");
    p_snitt_sta=(p[1]+p[2])/2.;
    break;
    case -1:
    printf("P-phase: Not consistent\n");
    break;
}
equalss=same_phase(s);
switch(equalss)
{
    case 123:
    printf("S-phase: All equal\n");
    s_snitt_sta=(s[0]+s[1]+s[2])/3.;
    break;
    case 12:
    printf("S-phase: 0-1 equal\n");
    s_snitt_sta=(s[0]+s[1])/2.;
    break;
    case 13:
    printf("S-phase: 0-2 equal\n");
    s_snitt_sta=(s[0]+s[2])/2.;
    break;
    case 23:
    printf("S-phase: 1-2 equal\n");
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case -1:
    printf("S-phase: Not consistent\n");
    break;
}
d_snitt=(d[0]+d[1]+d[2])/3. - p_snitt_sta;
printf("%6.2f %6.2f %6.2f %6.2f\n",d[0],d[1],d[2],d_snitt);

//if(equalpp==123 && equalss==123)
//{
//    goto process;
//}else{
//    printf("Not Processed\n");
//    exit(0);
//}
//process:


p_snitt_sta=(p[0]+p[1]+p[2])/3.;
s_snitt_sta=(s[0]+s[1]+s[2])/3.;

fra=(int)((p_snitt_sta-1.0)/srate);
til=fra+(int)(1.5/srate);

fra=(int)((p_snitt_sta-0.3)/srate);
til=fra+(int)(0.3/srate);

try_again:
printf("P-SNITT_STA=%6.3f S-SNITT_STA=%6.3f FRA %d TIL %d\n",p_snitt_sta,s_snitt_sta,fra,til);
    ww=50;
    n4_max=0.0;
    smp_indx=0;
    azi=0.0;
    vel=0.0;
    rms=0.0;
    lpts=ipts+ww;
    pvel=6500.0;
    psrat=1.73;

    for(smp=fra;smp<til;smp++)
    {
        n4=procoh1(c1,c2,c3,smp,smp+ww,pvel,psrat,oprocoh);
//printf("%5d %10.2f %10.3f  %10.3f\n",smp,(float)smp*srate,oprocoh[1],n4);
//getch();
        if(n4 < 0.0)
            n4=0.0;
//        coher[smp+ww/2]=n4;
        coher[smp]=n4;
        if(n4 > n4_max)
        {
            n4_max=n4;
            smp_indx=smp;
            azi=oprocoh[1];
            vel=oprocoh[2];
            rms=oprocoh[3];
        }
    }
if(n4_max <=0.0)
{
    printf("No coherence in window\n");
    fra=fra-10;
    if(fra <= 0)
        goto nocoher;
//    til-til+10;
    goto try_again;
}
//getch();
    for(i=0;i<indx;i++)                /* move to plot vector               */
        pv[i]=coher[i];

    for(j=0;j<indx;j++)
        pv[j]=pv[j]* -1.0;


    SCALETO_1(indx,pv,dsp);            /* scale to dsp                      */
    if(plot==1)
    {    
        color=13;                      /* VIOLET                            */
        PlotVec(color,indx,next,pv,xsc); /* Plot COHERE                     */
        next=next+dsp;
    }


        nsm=100;
        SMOOTH(indx,coher,c1);
        for(i=0;i<nsm;i++)
        {
            SMOOTH(indx,c1,c2);
            SMOOTH(indx,c2,c1);
        }
        for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
            pv[i]=c1[i];
        SCALETO_1(indx,pv,dsp);        /* scale plot vector                 */
        for(j=0;j<indx;j++)
            pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
        if(plot==1)
        {    
            color=11;                  /* CYAN                              */
            PlotVec(color,indx,next,pv,xsc); /* Plot COHER SMOOTHED         */
            next=next+dsp;
        }
//        DERIV(start_sec,stopp_sec,srate,c1,indx);
        for(j=0;j<indx;j++)
            c1[j]=c1[j]*10.0*c1[j]*10.0;

//    design(orden,"LP","BU",30.0,0.3,0.4,0.4,srate); /* filter */
//    apply(coher,indx,1);               /* Apply FILTER                      */
    for(i=0;i<indx;i++)                /* move to plot vector               */
        pv[i]=c1[i];
    SCALETO_1(indx,pv,dsp);            /* scale to dsp                      */
    for(j=0;j<indx;j++)
        pv[j]=pv[j]* -1.0;
    if(plot==1)
    {
        color=14;                      /* YELLOW                            */
        PlotVec(color,indx,next,pv,xsc);
    }
//printf("here9\n");
       MAXONE(c1,c2,indx);
       start_sec=(float)fra*srate;
       stopp_sec=(float)til*srate;
       SEARCH2(start_sec, stopp_sec,c2,indx,srate,&p_prelim2);
       SEARCH2(start_sec, stopp_sec,c2,indx,srate,&s_prelim2);
//printf("%6.3f %6.3f\n",p_prelim2,s_prelim2);
        if(s_prelim2 > p_prelim2)
        {
            p[0]=p_prelim2;                 /* save time of p-phase each channel */
            s[0]=s_prelim2;                 /* save time of s-phase each channel */
            p_ph=p_prelim2;
            s_ph=s_prelim2;
        }else{
            p[0]=s_prelim2;                 /* save time of p-phase each channel */
            s[0]=p_prelim2;                 /* save time of s-phase each channel */
            p_ph=s_prelim2;
            s_ph=p_prelim2;
        }
        p_coher_peak=p[0];
        s_coher=s[0];
//printf("P_COHER_PEAK=%6.3f S_COHER    =%6.2f\n",p_coher_peak,s_coher);
    if(plot==1)
    {
        _setlinestyle(DASHED);
        _moveto((int)(fra)*xsc,YL);
        _lineto((int)(fra)*xsc,0);
        _moveto((int)(til)*xsc,YL);
        _lineto((int)(til)*xsc,0);
        color=12;                      /* set color RED                     */
        next=next+dsp/2;
            TimScale1(arrow,dsp,color,indx,next,xsc,srate,p_ph,s_ph,dura);
        next=next+dsp;  
        if(manual==1)
        {
            getch();
        }else{
            sleep(4);
        }
    }
    p_coher=(float)smp_indx*srate;
//    printf("P_COHER    =%6.2f\n",p_coher);
nocoher:
      printf("C:%5.3f at:%6.2f Vel:%7.1f rms:%7.1f azm:%4.1f %7.2f %7.2f %6.2f %4d\n",
n4_max,(float)smp_indx*srate,vel,rms,azi,p_ph,s_ph,s_ph-p_ph,(int)((s_ph-p_ph)*HST));

fprintf(mail,"C:%5.3f at:%6.2f Vel:%7.1f rms:%7.1f azm:%4.1f %7.2f %7.2f %6.2f %4d\n",
n4_max,(float)smp_indx*srate,vel,rms,azi,p_ph,s_ph,s_ph-p_ph,(int)((s_ph-p_ph)*HST));

fprintf(coh,"%5.3f %6.2f %7.1f %7.1f %4.1f %7.2f %7.2f %6.2f %4d\n",
n4_max,(float)smp_indx*srate,vel,rms,azi,p_ph,s_ph,s_ph-p_ph,(int)((s_ph-p_ph)*HST));

//fprintf(mail,"C:%5.3f at:%6.2f Vel:%7.1f rms:%7.1f azm:%4.1f %7.2f %7.2f %6.2f %4d\n",
//n4_max,(float)smp_indx*srate,vel,rms,azi,p_ph,s_ph,s_ph-p_ph,(int)((s_ph-p_ph)*HST));

//fprintf(mail,"%5.3f %6.2f %7.1f %7.1f %4.1f %7.2f %7.2f %6.2f %4d\n",
//n4_max,(float)smp_indx*srate,vel,rms,azi,p_ph,s_ph,s_ph-p_ph,(int)((s_ph-p_ph)*HST));
fclose(coh);

sta=fopen("/home/seislog/pr/station","r");
/*   read velocites*/
     fscanf(sta,"%f %f",&velocityf[1],&velocityf[2]);
//     printf("P-velocity:    %5.1f S-velocity:  %5.1f  \n",velocityf[1],velocityf[2]);     

/*  read fixed depth or starting depth and number of depths to test*/
/*  if number of test depths is 1, only the first start depth is used*/
     fscanf(sta,"%f  %d",&h0,&ndepth);
//     printf("Depth used %5.1f    Number of dephts to test for %d\n",h0,ndepth);

/*  read coda scale parameters*/
     fscanf(sta,"%f %f %f",&c11,&c22,&c33);
//     printf("Coda paramters %6.2f %6.4f %6.2f\n",c11,c22,c33);

//  read min/max long and lat
//
    fscanf(sta,"%f %f %f %f",&slon1,&slon2,&slat1,&slat2);
//    printf("Min/max lon lat:%f %f %f %f\n",slon1,slon2,slat1,slat2);
//

//  read stations until end of file
//
//     fscanf(sta,"%s %f %f",stat[i],&slatt[i],&slonn[i])
fler:
     fscanf(sta,"%s %f %f",stasjon,&flat,&flon);
    cur_sta[3]='\0';
//    printf("%s %s %f %f\n",stasjon,cur_sta,flat,flon);
//getch();
    if(stasjon[0]==cur_sta[0] && stasjon[1]==cur_sta[1] && stasjon[2]==cur_sta[2])
    {
//        printf("%s %f %f\n",stasjon,flat,flon);
        fclose(sta);
    }else{
        goto fler;
    }
/*--------------------------------------------------------------------------*/
/* find channels with equal p readings                                      */
/*--------------------------------------------------------------------------*/
avstand=(s_snitt_sta-p_snitt_sta)*HST;

printf("STA/LTA       : %s P: %8.2f  S: %8.2f ASM: %5.2f DST: %10.2f\n",
stasjon,p_snitt_sta,s_snitt_sta,azi,avstand);
fprintf(mail,"STA/LTA: %s P: %8.2f  S: %8.2f ASM: %5.2f DST: %10.2f\n",
stasjon,p_snitt_sta,s_snitt_sta,azi,avstand);

avstand=(s_snitt_sta-p_coher_peak)*HST;

//printf("COHERENCE PEAK: %s P: %8.2f  S: %8.2f ASM: %5.2f DST: %10.2f\n",
//stasjon,p_coher_peak,s_snitt_sta,azi,avstand);
//fprintf(mail,"COHERENCE PEAK: %s P: %8.2f  S: %8.2f ASM: %5.2f DST: %10.2f\n",
//stasjon,p_coher_peak,s_snitt_sta,azi,avstand);

avstand=(s_coher_peak-p_coher_peak)*HST;

//printf("COHE. S-P PEAK: %s P: %8.2f  S: %8.2f ASM: %5.2f DST: %10.2f\n",
//stasjon,p_coher_peak,s_coher_peak,azi,avstand);
//fprintf(mail,"COHE. S-P PEAK: %s P: %8.2f  S: %8.2f ASM: %5.2f DST: %10.2f\n",
//stasjon,p_coher_peak,s_coher_peak,azi,avstand);

avstand=(s_snitt_sta-p_coher)*HST;
if(avstand < 55.)
{
    Mb=0.23 + 1.71 * log10(d_snitt);
}else{
    Mb=-1.56 + 2.36 * log10(d_snitt);
}
printf("COHERENCE     : %s P: %8.2f  S: %8.2f ASM: %5.2f DST: %10.2f\n",
stasjon,p_coher,s_snitt_sta,azi,avstand);
//printf("Mb=%6.2f\n",Mb);
fprintf(mail,"COHERENCE: %s P: %8.2f  S: %8.2f ASM: %5.2f DST: %10.2f\n",
stasjon,p_coher,s_snitt_sta,azi,avstand);
//fprintf(mail,"Mb=%6.2f\n",Mb);



slat=flat;
slon=flon;
x11=sin(azi*0.01745)*avstand;
y11=cos(azi*0.01745)*avstand;
lat=slat+y11/111.2;
lon=slon+x11/(111.2*cos(slat/57.3));
printf("%10.3f %10.3f ( %s %10.3f %10.3f) %10.2f %6.2f\n",lat,lon,stasjon,slat,slon,avstand,Mb);
fprintf(mail,"%10.3f %10.3f ( %s %10.3f %10.3f) %10.2f %6.2f\n",lat,lon,stasjon,slat,slon,avstand,Mb);
system("rm -f /home/work/coord");
coh=fopen("/home/work/coord","w");
fprintf(coh,"%s %10.3f %10.3f %10.3f %10.3f %10.2f %6.2f\n",statf,slat,slon,lat,lon,avstand,Mb);
loc=fopen("/home/work/location","a"); /* open file for location            */
fprintf(loc,"%s %10.3f %10.3f %10.3f %10.3f %10.2f %6.2f\n",statf,slat,slon,lat,lon,avstand,Mb);
fclose(coh);
fclose(mail);
fclose(loc);
//system("/home/seislog/map2");
if(manual==1)
{
    getch();
//terje


    each_flt=(int)YL/4;
    dsp=each_flt/2;               /* displacement in pixels between    */
    if(plot==1)                        /* plot ?                            */
        GraphicsMode();                /* set to graphics mode              */
    next=dsp;                        /* zero line diplacement in pixels   */
    for(ichan=0;ichan<3;ichan++)   /* channel to be processed           */
    {
        isamp=ntotal;                  /* counter for number of samples     */
        indx=0;
        for(smp=from;smp<to;smp++)     /* extract channel to process vector */
        {
            valfl=M2(matr,tre_start[ichan+pkt*3]-1,smp);
            data[indx]=valfl;
            indx=indx+1;               /* counting samples                  */
        }
        xsc=XL/(float)indx;            /* scale factor x-axis               */

        switch(ichan)              /* test for channel                  */
        {
            case 0:
            for(smp=0;smp<indx;smp++)
                c1[smp]=data[smp]; /* save channel 1 for coherence      */
            break;
            case 1:
            for(smp=0;smp<indx;smp++) /* save channel 2 for coherence   */
                c2[smp]=data[smp];
            break;
            case 2:
            for(smp=0;smp<indx;smp++) /* save channel 3 for coherence   */
                c3[smp]=data[smp];
            break;
        }
    }
    design(orden,"BP","BU",30.0,0.3,flow[0],fhig[0],srate);
    REMOVE_DC(indx,c1);        /* remove DC                         */
    apply(c1,indx,1);          /* Apply FILTER                      */
    REMOVE_DC(indx,c2);        /* remove DC                         */
    apply(c2,indx,1);          /* Apply FILTER                      */
    REMOVE_DC(indx,c3);        /* remove DC                         */
    apply(c3,indx,1);          /* Apply FILTER                      */

    for(i=0;i<indx;i++)            /* move to plot vector               */
        pv[i]=c1[i] * -1.0;        /* turn polarity for plotting        */
    SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,indx,next,pv,xsc);
        ichan=0+pkt*3;
        statf[0]=stcode[(tre_start[ichan]-1)*4];
        statf[1]=stcode[(tre_start[ichan]-1)*4+1];
        statf[2]=stcode[(tre_start[ichan]-1)*4+2];
        statf[3]=stcode[(tre_start[ichan]-1)*4]+3;
        statf[4]='\0';
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    for(i=0;i<indx;i++)            /* move to plot vector               */
        pv[i]=c2[i] * -1.0;        /* turn polarity for plotting        */
    SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,indx,next,pv,xsc);
        ichan=1+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    for(i=0;i<indx;i++)            /* move to plot vector               */
        pv[i]=c3[i] * -1.0;        /* turn polarity for plotting        */
    SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,indx,next,pv,xsc);
        ichan=2+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }

    for(i=0;i<indx;i++)                /* move to plot vector               */
        pv[i]=coher[i];

    for(j=0;j<indx;j++)
        pv[j]=pv[j]* -1.0;


    SCALETO_1(indx,pv,dsp);            /* scale to dsp                      */
    if(plot==1)
    {    
        color=13;                      /* VIOLET                            */
        PlotVec(color,indx,next,pv,xsc); /* Plot COHERE                     */
        next=next+dsp;
    }
    if(plot==1)
    {    
        _setlinestyle(DASHED);
        _setcolor(11);                 /* color CYAN                        */
        _moveto((int)(p_snitt_sta/srate)*xsc,YL);
        _lineto((int)(p_snitt_sta/srate)*xsc,0);
        _setcolor(14);                 /* color YELLOW                      */
        _moveto((int)(p_coher_peak/srate)*xsc,YL);
        _lineto((int)(p_coher_peak/srate)*xsc,0);
        _setcolor(9);                 /* color BLUE                         */
        _moveto((int)(p_coher/srate)*xsc,YL);
        _lineto((int)(p_coher/srate)*xsc,0);
        _setcolor(7);                 /* color GRAY                         */
        _moveto((int)(s_snitt_sta/srate)*xsc,YL);
        _lineto((int)(s_snitt_sta/srate)*xsc,0);
        _setcolor(12);                 /* color RED                         */
        _moveto((int)(fra)*xsc,YL);
        _lineto((int)(fra)*xsc,0);
        _moveto((int)(til)*xsc,YL);
        _lineto((int)(til)*xsc,0);
        color=13;                      /* VIOLET                            */
//        PlotVec(color,no_ext,next,pv,xsc); /* Plot COHERE                     */
        next=next+dsp;
    }
//    printf("%5d\n",(int)(p_snitt_sta/srate));
//    printf("%5d\n",(int)(p_coher_peak/srate));
//    printf("%5d\n",(int)(p_coher/srate));
//    printf("%5d\n",fra);
fra=(int)((p_snitt_sta-2.0)/srate);
getch();
//utheim
    til=(int)((p_snitt_sta+2.0)/srate);
    ne=til-fra;
//    printf("FRA %d TIL %d NE %d\n",fra,til,til-fra);
    xsc=XL/(float)(til-fra);            /* scale factor x-axis               */
    each_flt=(int)YL/4;
    dsp=each_flt/2;               /* displacement in pixels between    */
    if(plot==1)                        /* plot ?                            */
        GraphicsMode();                /* set to graphics mode              */
    next=dsp;                        /* zero line diplacement in pixels   */
    j=0;
    for(i=fra;i<til;i++)            /* move to plot vector               */
    {
        pv[j]=c1[i] * -1.0;        /* turn polarity for plotting        */
        j++;
    }
    SCALETO_1(ne,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,ne,next,pv,xsc);
        ichan=0+pkt*3;
        statf[0]=stcode[(tre_start[ichan]-1)*4];
        statf[1]=stcode[(tre_start[ichan]-1)*4+1];
        statf[2]=stcode[(tre_start[ichan]-1)*4+2];
        statf[3]=stcode[(tre_start[ichan]-1)*4]+3;
        statf[4]='\0';
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    j=0;
    for(i=fra;i<til;i++)            /* move to plot vector               */
    {
        pv[j]=c2[i] * -1.0;        /* turn polarity for plotting        */
        j++;
    }
    SCALETO_1(ne,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,ne,next,pv,xsc);
        ichan=1+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    j=0;
    for(i=fra;i<til;i++)            /* move to plot vector               */
    {
        pv[j]=c3[i] * -1.0;        /* turn polarity for plotting        */
        j++;
    }
    SCALETO_1(ne,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,ne,next,pv,xsc);
        ichan=2+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2+dsp;
    }
    j=0;
    for(i=fra;i<til;i++)                /* move to plot vector               */
    {
        pv[j]=coher[i];
        j++;
    }
    for(j=0;j<ne;j++)
        pv[j]=pv[j]* -1.0;


    SCALETO_1(ne,pv,dsp*2);            /* scale to dsp                      */
    if(plot==1)
    {    
        color=11;                      /* CYAN                              */
        PlotVec(color,ne,next,pv,xsc); /* Plot COHERE                     */
        next=next+dsp*2;
    }
    if(plot==1)
    {    
        color=15;
        TimScale8(dsp,color,ne,105.0,xsc,srate);
        TimScale8(dsp,color,ne,225.0,xsc,srate);
        TimScale8(dsp,color,ne,345.0,xsc,srate);
        _setlinestyle(DASHED);
        _setcolor(6);                 /* color BROWN                        */
        _moveto((int)((p_snitt_sta/srate-(float)fra)*xsc),YL);
        _lineto((int)((p_snitt_sta/srate-(float)fra)*xsc),0);
        pp=(int)((p_snitt_sta/srate-(float)fra)*xsc);
        sprintf(thead,"P-AVG-STA %6.2f\0",p_snitt_sta);
        label1(p_snitt_sta,pp,420,thead);

        _setcolor(15);                 /* color WHITE                      */
        _moveto((int)((p_coher_peak/srate-(float)fra)*xsc),YL);
        _lineto((int)((p_coher_peak/srate-(float)fra)*xsc),0);
        pp=(int)((p_coher_peak/srate-(float)fra)*xsc);
        sprintf(thead,"P-COHER_PEAK %6.2f\0",p_coher_peak);
        label1(p_coher_peak,pp,440,thead);

        _setcolor(9);                 /* color BLUE                         */
        _moveto((int)((p_coher/srate-(float)fra)*xsc),YL);
        _lineto((int)((p_coher/srate-(float)fra)*xsc),0);
        pp=(int)((p_coher/srate-(float)fra)*xsc);
        sprintf(thead,"P-COHER %6.2f\0",p_coher);
        label1(p_coher,pp,460,thead);

        _setcolor(7);                 /* color GRAY                         */
        _moveto((int)((s_snitt_sta/srate-(float)fra)*xsc),YL);
        _lineto((int)((s_snitt_sta/srate-(float)fra)*xsc),0);
        _setcolor(12);                 /* color RED                         */
        _moveto((int)((fra-fra)*xsc),YL);
        _lineto((int)((fra-fra)*xsc),0);
        _moveto((int)((til-fra)*xsc),YL);
        _lineto((int)((til-fra)*xsc),0);
        color=13;                      /* VIOLET                            */
//        PlotVec(color,no_ext,next,pv,xsc); /* Plot COHERE                     */
        next=next+dsp;
    }
getch();
//rotate
        xsc=XL/(float)indx;            /* scale factor x-axis               */
    each_flt=(int)YL/5;
    dsp=each_flt/2;               /* displacement in pixels between    */
    if(plot==1)                        /* plot ?                            */
        GraphicsMode();                /* set to graphics mode              */
    next=dsp;                        /* zero line diplacement in pixels   */

    for(i=0;i<indx;i++)            /* move to plot vector               */
        pv[i]=c1[i] * -1.0;        /* turn polarity for plotting        */
    SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,indx,next,pv,xsc);
        ichan=0+pkt*3;
        statf[0]=stcode[(tre_start[ichan]-1)*4];
        statf[1]=stcode[(tre_start[ichan]-1)*4+1];
        statf[2]=stcode[(tre_start[ichan]-1)*4+2];
        statf[3]=stcode[(tre_start[ichan]-1)*4]+3;
        statf[4]='\0';
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    for(i=0;i<indx;i++)            /* move to plot vector               */
        pv[i]=c2[i] * -1.0;        /* turn polarity for plotting        */
    SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,indx,next,pv,xsc);
//next2=next;
        ichan=1+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    for(i=0;i<indx;i++)
        rat[i]=c2[i]*sin(azi)+c3[i]*sin(90.-azi);
    for(i=0;i<indx;i++)            /* move to plot vector               */
        pv[i]=rat[i] * -1.0;        /* turn polarity for plotting        */
    SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
//    SCALETO_MAX(indx,pv,dsp,prev_max);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=11;                  /* YELLOW                            */
//next=next2;
        PlotVec(color,indx,next,pv,xsc);
        ichan=1+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    
    for(i=0;i<indx;i++)            /* move to plot vector               */
        pv[i]=c3[i] * -1.0;        /* turn polarity for plotting        */
    SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,indx,next,pv,xsc);
        ichan=2+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    for(i=0;i<indx;i++)
        rat[i]=c2[i]*cos(azi) + c3[i]*cos(90-azi);
    for(i=0;i<indx;i++)            /* move to plot vector               */
        pv[i]=rat[i] * -1.0;        /* turn polarity for plotting        */
    SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
//    SCALETO_MAX(indx,pv,dsp,prev_max);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=11;                  /* YELLOW                            */
//next=next2;
        PlotVec(color,indx,next,pv,xsc);
        ichan=1+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }

getch();

}else{
//    sleep(4);
}
}
}
}
DERIV(start,stopp,srate,data,indx)
float    start;                        /* start second                      */
float    stopp;                        /* stopp second                      */
float    srate;                        /* sample rate                       */
float*   data;
int      indx;                         /* number of samples in vector       */
{
int      start_indx;                   /* index to start computing average  */
int      stopp_indx;                   /* index to stopp computing average  */
int      j;

start_indx=(int)start/srate;
stopp_indx=(int)stopp/srate;

/* first x2-x1 */
for(j=start_indx+1;j<indx-1;j++)
{
    data[j]=data[j+1]-data[j];
    if(data[j] < 0.0)
        data[j]=0.0;
}
}
stalta(start,stopp,srate,indx,data,ichan,ws,wl,snr_max,samp_indx,rat)
float    start;                        /* start second                      */
float    stopp;                        /* stopp second                      */
float    srate;                        /* sample rate                       */
int      indx;                         /* number of samples in vector       */
float*   data;                         /* data vector                       */
int      ichan;                        /* channel                           */
int      ws;
int      wl;
float* snr_max;
float* samp_indx;
float* rat;
{
float    sta1;
float    lta1;
float    max;                          /* maximum sta/lta                   */
float    R;                            /* ratio sta/lta                     */
float    sum;                          /* acumulator                        */
float    where;                        /* sample index for max sta/lta ratio*/
int      j;                            /* index                             */
float    pre_avg;                      /* noise level before trigger        */
float    limit;                        /* trigger level to discover p-phase */
int      lpre;                         /* number of samples to compute noise*/
int      start_indx;                   /* index to start computing average  */
int      stopp_indx;                   /* index to stopp computing average  */
float    extra;
float    vk;
float    st;
float    beg;
float    at_beg;
/*--------------------------------------------------------------------------*/
/* compute average of pre-event/2 seconds to determine noise level.         */
/*--------------------------------------------------------------------------*/ 
start_indx=(int)start/srate;
stopp_indx=(int)stopp/srate;
max=0.0;
for(j=start_indx+1;j<stopp_indx;j++) 
{
    extra=data[j]-data[j-1];
    extra=extra*extra;
    sta1=sta1+(fabs(data[j-1]) * fabs(data[j-1])+3.* extra-sta1)/ws;
}

pre_avg=sta1;                          /* average noise level          */
limit=pre_avg+pre_avg * 200./100.;     /* trigger level 150 prcent of noise */
sta1=pre_avg;                          /* initial sta value equal noise lev.*/
lta1=pre_avg;                          /* initial lta value equal noise lev.*/
sta1=0.0;      /*TEST*/
lta1=0.0;      /*TEST*/
start_indx=0;  /*TEST*/
/*--------------------------------------------------------------------------*/
/* compute running sta (squared values)                                     */
/* compute running lta (squared values)                                     */
/* compute running ratio sta/lta                                            */
/* save max ratio and corresponding sample index                            */
/*--------------------------------------------------------------------------*/ 
for(j=start_indx+1;j<indx-1;j++)
{
    extra=data[j]-data[j-1];
    extra=extra*extra;
    sta1=sta1+(fabs(data[j-1]) * fabs(data[j-1])+3.* extra-sta1)/ws;
//    sta1=sta1+(fabs(data[j]) * fabs(data[j])-sta1)/ws;
    lta1=lta1+(fabs(data[j]) * fabs(data[j])-lta1)/wl;
    R=sta1/lta1;
    rat[j]=R;
    if(R > max)
    {
        max=R;                         /* max sta/lta ratio                 */
        where=j;                       /* sample index                      */
    }
}       


*snr_max=max;
*samp_indx=(float)where*srate;
}
conv8(lplot,data,srate,where,maxamp2)
int   lplot;
float *data;
float srate;
float *where;
float *maxamp2;
{
    int i;
    int maxa;
    float fmax;    
    float whmax;
    fmax=0.0;
    for(i=20;i<lplot;i++)               /* find max value                */
    {
        if(fabs(data[i]) > fmax)
        {
            fmax=fabs(data[i]);
            whmax=(float)i*srate;
        }
    }
    *where=whmax;
    *maxamp2=fmax;
}
/*-----------------------------------------------------------------------*/
/* scale data                                                            */
/*-----------------------------------------------------------------------*/
conv2(lplot,data,srate,where,maxamp2)
int   lplot;
float *data;
float srate;
float *where;
int *maxamp2;
{
    float sum;
    float avg;
    float factor;
    int i;
    int maxa;
    float fmax;    
    float whmax;
    sum=0.0;
    fmax=0.0;
    for(i=0;i<lplot;i++)               /* find DC                       */
        sum=sum+data[i];
    avg=sum/(float)lplot;
    for(i=0;i<lplot;i++)                /* remove dc                    */
        data[i]=data[i]-avg;
    for(i=0;i<lplot;i++)               /* find max value                */
    {
        if(fabs(data[i]) > fmax)
        {
            fmax=fabs(data[i]);
            whmax=(float)i*srate;
        }
    }
    if(fmax == 0.0)
        fmax=1.;
    maxa=(int)fmax;

    *where=whmax;
    *maxamp2=maxa;
}
conv3(lplot,c)
int lplot;
float *c;
{
    float sum;
    float avg;
    float factor;
    int i;
    float maxa;
    sum=0.0;
    maxa=0;

    for(i=0;i<lplot;i++)               /* find DC                       */
        sum=sum+c[i];
    avg=sum/(float)lplot;
    for(i=0;i<lplot;i++)                /* remove dc                    */
        c[i]=c[i]-avg;
    for(i=0;i<lplot;i++)               /* find max value                */
    {
        if(fabs(c[i]) > maxa)
            maxa=fabs(c[i]);
    }
    for(i=0;i<lplot;i++)
        c[i]=c[i]/maxa* 50.;

}
conv4(lplot,co)
int lplot;
float *co;
{
    float sum;
    float avg;
    float factor;
    int i;
    float maxa;
    sum=0.0;
    maxa=0;

    for(i=0;i<lplot;i++)               /* find max value                */
    {
        if(co[i] > maxa)
            maxa=co[i];
    }
    for(i=0;i<lplot;i++)
        co[i]=(co[i]/maxa) * 100.;
}
SCALETO_1(lplot,co,fac)
int lplot;
float *co;
int fac;
{
    float sum;
    float avg;
    float factor;
    int i;
    float maxa;
    sum=0.0;
    maxa=0;

    for(i=0;i<lplot;i++)               /* find max value                */
    {
        if(fabs(co[i]) > maxa)
            maxa=fabs(co[i]);
    }
    for(i=0;i<lplot;i++)
        co[i]=co[i]/maxa* (float)fac;
prev_max=maxa;
}
SCALETO_MAX(lplot,co,fac,mmxx)
int lplot;
float *co;
int fac;
float mmxx;
{
    float sum;
    float avg;
    float factor;
    int i;
    float maxa;
    sum=0.0;
    maxa=0;

    for(i=0;i<lplot;i++)
        co[i]=co[i]/mmxx* (float)fac;
}
REMOVE_DC(lplot,c)
int lplot;
float *c;
{
    float sum;
    float avg;
    int i;
    sum=0.0;

    for(i=0;i<lplot;i++)               /* find DC                       */
        sum=sum+c[i];
    avg=sum/(float)lplot;
    for(i=0;i<lplot;i++)                /* remove dc                    */
        c[i]=c[i]-avg;
}
/*-----------------------------------------------------------------------*/
/* demultiplex one channel                                               */
/*-----------------------------------------------------------------------*/
dmx(isamp,nsamp,ichan,nchan,iworkl,data)
int isamp;
int nsamp;
int ichan;
int nchan;
int iworkl[];
float data[];
{
int i;
int bufsiz;

bufsiz=nsamp*nchan+HEADER;
for(i=HEADER+ichan-1;i<bufsiz;i=i+nchan)
{
    data[isamp]=(float)iworkl[i];
    isamp=isamp+1;
}
return(isamp);
}
/*-------------------------------------------------------------------------*/
/* Get filenames on old detections and their duration                      */
/*-------------------------------------------------------------------------*/
dpxnam(i,infile,ttotal)
int i;
char* infile;
float* ttotal;
{
    FILE    *det;
    int     nn;
    int     kl;
    char    detrec[512];
    char    dur[16];
    int     istart;
    float   locdur;
    det=fopen(DETECTION,"r");
    nn=i/EVTSIZ;
    kl=nn*EVTSIZ;
    istart=(i-1-kl)*512;
    fseek(det,istart,0);
    fread(detrec,512,1,det);
    for(kl=0;kl<DIRNAMLEN;kl++)
        infile[kl]='\0';
    for(kl=0;kl<28;kl++)
        infile[kl]=detrec[kl+206];
    for(kl=0;kl<16;kl++)
        dur[kl]=detrec[kl+64];
    sscanf(dur,"%f8.3\n",&locdur);
    *ttotal=locdur;
    fclose(det);
}
/*-------------------------------------------------------------------------*/
/* Get current DPX number                                                  */
/*-------------------------------------------------------------------------*/
Get_Current_DPX()
{
    FILE    *det;                        /* pointer current detection no.  */
    int     curdpx;                      /* current dpx number             */
    det=fopen(DETNUMBER,"r");
    fscanf(det,"%d",&curdpx);
    fclose(det);
    return(curdpx);
}

hdr_info(head,year,month,day,hour,min,sec,nchan,srate,ntotal,unit,nsamp,recsize,reclength,pre)
char*    head;
long    *year;
long     *month;
long     *day;
long     *hour;
long     *min;
float    *sec;
int      *nchan;
float    *srate;
int      *ntotal;
int      *unit;
int      *nsamp;
int      *recsize;
int      *reclength;
int      *pre;
{
char     dum[100];
int      i;
//for(i=0;i<20;i++)
//    printf("%c ",head[i]);
//printf("\n");
sscanf(head,"%14c%4d%5c%2d%c%2d%c%2d%c%2d%c%f6.3\n",
dum,year,dum,month,dum,day,dum,hour,dum,min,dum,sec);
if(head[62]==' ')
{
    dum[0]=head[63];
    sscanf(dum,"%1d",nchan);
}else{
    dum[0]=head[62];
    dum[1]=head[63];
    sscanf(dum,"%2d",nchan);
}
for(i=0;i<16;i++)
    dum[i]=head[i+64];
sscanf(dum,"%f16.7",srate);
for(i=0;i<8;i++)
    dum[i]=head[i+80];
sscanf(dum,"%8d",ntotal);
dum[0]=head[96];
dum[1]=head[97];
sscanf(dum,"%2d",unit);
for(i=0;i<6;i++)
    dum[i]=head[i+122];
dum[i]='\0';
sscanf(dum,"%d",nsamp);
for(i=0;i<8;i++)
    dum[i]=head[i+104];
sscanf(dum,"%8d",recsize);
sscanf(dum,"%8d",reclength);
dum[0]=head[880];
dum[1]=head[881];
dum[2]=head[882];
dum[3]=head[883];
sscanf(dum,"%4d",pre);
/*printf("NCHAN:%2d SRATE:%16.7f NTOTAL:%8d UNIT:%2d NSAMP:%6d\n",\*/
/*nchan,srate,ntotal,unit,nsamp);*/
/*printf("RECSIZE:%8d RECLENGTH:%8d\n",recsize,reclength);*/
}

dmx_buf(dimx,isamp,nsamp,nchan,ntotal,iworkl)
int dimx;
int isamp;
int nsamp;
int nchan;
int ntotal;
int iworkl[];
{
int i;
int j;
int s;
int bufsiz;
bufsiz=nsamp*nchan+HEADER;
for(i=0;i<nchan;i++)                   /* loop channels                     */
{
    for(j=0;j<nsamp;j++)               /* loop samples                      */
    {
        s=j+isamp*nsamp;
        M2(matr,i,s)=(float)iworkl[j*nchan+i+HEADER];
    }
}
}    

/*********************subrutine cross***************************************/

/* this subrotine perform the cross correlation between two set of data(x,y) */


float   cross(x,y,l,ipts)

  float x[],y[];
  int l,ipts;

{

  float a=0.0;
  int i;
  
  for (i=ipts;i<=l+ipts;i++) a=a+x[i]*y[i];
      a=a+x[i]*y[i];
  return a;

}
float   procoh1(channel,channel2,channel3,ipts,lpts,pvel,psrat,oprocoh)

float channel[],channel2[],channel3[];
float pvel,psrat,oprocoh[10];
int ipts,lpts;


/*
  S/R to do 3-component azimuth and velocity estimation for a P-phase
  for a fixed user specified time window.
  Method: predicted coherence (R. Roberts, Uppsala)
*/

{

 
  float raddeg=57.2958;
  int lenwin=0,ii;
  float xx,xy,xz,yy,yx,yz,zz,zx,zy;
  float power,azi,zoverr,a,b,err,cc,ai;
  float svel;
  float coher,azim,velo,rmsamp;


  lenwin=lpts-ipts+1;
  xx=cross(channel2,channel2,lenwin,ipts);
  xy=cross(channel2,channel3,lenwin,ipts); 
  xz=cross(channel2,channel,lenwin,ipts);
  yy=cross(channel3,channel3,lenwin,ipts);
  yz=cross(channel3,channel,lenwin,ipts);
  zz=cross(channel,channel,lenwin,ipts);
  yx=xy;
  zx=xz;
  zy=yz;
  power=(xx+yy+zz)/(float)lenwin;
//printf("(%f + %f + %f)/ %d = %f\n",xx,yy,zz,lenwin,power);
//printf("power=%f\n",power);
  rmsamp=sqrt(power);
  azi=atan2(-yz,-xz);
  azim=azi*raddeg;
  if (azim<0) azim=azim+360.0;
//printf("xz:%f yz:%f\n",xz,yz);
  zoverr=zz/sqrt(xz*xz+yz*yz);
  a=-zoverr*cos(azi);
  b=-zoverr*sin(azi);
   
  err=0.0; 
  for (ii=ipts;ii<=lpts;ii++){
      cc=channel[ii]-a*channel2[ii]-b*channel3[ii];
      err=err+cc*cc;
  };
  coher=1.0-err/zz;
  svel=pvel/psrat;
  ai=atan(1.0/zoverr);
  velo=svel/sin(ai/2.0);

  oprocoh[0]=coher;
  oprocoh[1]=azim;
  oprocoh[2]=velo;
  oprocoh[3]=rmsamp;

  return(coher);
} 

GraphicsMode()
{
    _setvideomode(_VRES16COLOR);
    _setcolor(0);
    _rectangle(_GFILLINTERIOR,0,0,(int)XL,(int)YL);
    _setcolor(13);
    _rectangle(_GBORDER,0,0,(int)XL,(int)YL);
    _setcolor(1);
    _moveto(0,(int)YL);
    _setcolor(14);
}
PlotVec(color,no_of_samples,displacement,array,xscale)
int color;
int no_of_samples;
float displacement;
float* array;
float xscale;
{       
int oldcolor;
int j;
int x,y;
float xx;

oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)(array[j]+displacement);
    _lineto(x,y);
}
_setcolor(oldcolor);
}
TimScale(ar,ds,color,no_of_samples,displacement,xscale,sr,p,s,d)
int ar;
int ds;
int color;
int no_of_samples;
float displacement;
float xscale;
float sr;
float p;
float s;
float d;
{
char karak[10];
int oldcolor;
int j;
int x,y;
int spsec;
int cnt;
int pphase;
int sphase;
int duration;
spsec=(int)(1./sr) * 10;
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
cnt=0;
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)displacement+5;
    _lineto(x,y);
    cnt++;
    if(cnt==spsec)
    {
        cnt=0;
        _lineto(x,y-5);
        _moveto(x,y);
    }
}
pphase=(int)(p/sr)*xscale;
sphase=(int)(s/sr)*xscale;
duration=(int)(d/sr)*xscale;

y=ar;
_setcolor(12);                         /* RED                               */
_moveto(pphase,y);
_lineto(pphase-5,y-5);
_moveto(pphase,y);
_lineto(pphase+5,y-5);
_moveto(pphase,y);
_lineto(pphase,y-ds/2);
karak[0]='P';
karak[1]='\0';
_moveto(pphase-10,y-ds/2-10);
_outgtext(karak);

_setcolor(13);                         /* VIOLETT                           */
_moveto(sphase,y);
_lineto(sphase-5,y-5);
_moveto(sphase,y);
_lineto(sphase+5,y-5);
_moveto(sphase,y);
_lineto(sphase,y-ds/2);
karak[0]='S';
karak[1]='\0';
_moveto(sphase-10,y-ds/2-10);
_outgtext(karak);

_setcolor(10);                         /* GREEN                             */
_moveto(duration,y);
_lineto(duration-5,y-5);
_moveto(duration,y);
_lineto(duration+5,y-5);
_moveto(duration,y);
_lineto(duration,y-ds/2);
karak[0]='D';
karak[1]='\0';
_moveto(duration-10,y-ds/2-10);
_outgtext(karak);

//_lineto(pphase,y-ds);
//_moveto(sphase,y);
//_lineto(sphase,y-ds);
//_moveto(duration,y);
//_lineto(duration,y-ds);

_setcolor(oldcolor);
}
TimScale3(ar,ds,color,no_of_samples,displacement,xscale,sr,p,s,d)
int ar;
int ds;
int color;
int no_of_samples;
float displacement;
float xscale;
float sr;
float p;
float s;
float d;
{
char karak[10];
int oldcolor;
int j;
int x,y;
int spsec;
int cnt;
int pphase;
int sphase;
int duration;
spsec=(int)(1./sr);
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
cnt=0;
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)displacement+5;
    _lineto(x,y);
    cnt++;
    if(cnt==spsec)
    {
        cnt=0;
        _lineto(x,y-5);
        _moveto(x,y);
    }
}
pphase=(int)(p/sr)*xscale;
sphase=(int)(s/sr)*xscale;
duration=(int)(d/sr)*xscale;

y=ar;
_setcolor(12);                         /* RED                               */
_moveto(pphase,y);
_lineto(pphase-5,y-5);
_moveto(pphase,y);
_lineto(pphase+5,y-5);
_moveto(pphase,y);
_lineto(pphase,y-ds/2);
karak[0]='P';
karak[1]='\0';
_moveto(pphase-10,y-ds/2-10);
_outgtext(karak);

_setcolor(13);                         /* VIOLETT                           */
_moveto(sphase,y);
_lineto(sphase-5,y-5);
_moveto(sphase,y);
_lineto(sphase+5,y-5);
_moveto(sphase,y);
_lineto(sphase,y-ds/2);
karak[0]='S';
karak[1]='\0';
_moveto(sphase-10,y-ds/2-10);
_outgtext(karak);

_setcolor(10);                         /* GREEN                             */
_moveto(duration,y);
_lineto(duration-5,y-5);
_moveto(duration,y);
_lineto(duration+5,y-5);
_moveto(duration,y);
_lineto(duration,y-ds/2);

//_lineto(pphase,y-ds);
//_moveto(sphase,y);
//_lineto(sphase,y-ds);
//_moveto(duration,y);
//_lineto(duration,y-ds);

_setcolor(oldcolor);
}
skala1(xscale)
float xscale;
{
printf("XSC: %f\n",xscale);
}
TimScale1(ar,ds,color,no_of_samples,displacement,xscale,sr,p,s,d)
int ar;
int ds;
int color;
int no_of_samples;
float displacement;
float xscale;
float sr;
float p;
float s;
float d;
{
int oldcolor;
int j;
int x,y;
int spsec;
int cnt;
int pphase;
int sphase;
int duration;
spsec=(int)(1./sr) * 10;
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
cnt=0;
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)displacement+10;
    _lineto(x,y);
    cnt++;
    if(cnt==spsec)
    {
        cnt=0;
        _lineto(x,y-10);
        _moveto(x,y);
    }
}
_setcolor(oldcolor);
}
FindTops(samp_indx,start,stopp,rat,no_of_samples,srate,p_ph,s_ph)
float  samp_indx;                      /* time ind seconds max amplitude    */
float  start;                          /* start_second                      */
float  stopp;                          /* stopp second                      */
float* rat;
int    no_of_samples;
float  srate;
float* p_ph;
float* s_ph;
{
int smp_max_amp;                       /* sample maximum amplitude          */
int j;
float diff;
int pos_neg;
int tops;
int top[2000];
float maxtop;
int sample_top;
float big_top;
float min_t;
float max_t;
float tmp_t;
float vk;
float st;
float beg;
float at_beg;
int   sindx;
int   start_indx;                      /* start index                       */
int   stopp_indx;                      /* stopp index                       */
beg=200;
start_indx=(int)(start/srate)+100;
stopp_indx=(int)(stopp-start)/srate;
smp_max_amp=(int)(samp_indx/srate);    /* index maximum amplitude           */
/*--------------------------------------------------------------------------*/
/* zero all negative values                                                 */
/*--------------------------------------------------------------------------*/
for(j=0;j<no_of_samples;j++)
{
    if(rat[j] < 0.0)
        rat[j]=0.0;
}
/*--------------------------------------------------------------------------*/
/* count and locate all tops                                                */
/*--------------------------------------------------------------------------*/
diff=rat[start_indx]-rat[start_indx-1];
if(diff > 0.0)
{
    pos_neg=1;
}else{
    pos_neg=-1;
}
tops=0;
for(j=start_indx;j<(no_of_samples-1);j++)
{
    diff=rat[j+1]-rat[j];
    switch(pos_neg)
    {
        case 1:   /* we are positive */
        if(diff <= 0.0)
        {
            top[tops]=j;
            pos_neg=-1;
            tops++;
        }
        break;
        case -1: /* we are negative */
        if(diff > 0.0)
        {
            pos_neg=1;
        }
        break;
   }
}
//printf("ICHAN %2d FILTER %2d has %5d tops\n",ichan,filter,tops-1);
/*--------------------------------------------------------------------------*/
/* find max top and corresponding sample                                    */
/*--------------------------------------------------------------------------*/
maxtop=rat[top[0]];
for(j=1;j<(tops-1);j++)
{
//    printf("%5d %10.2f\n",top[j],rat[top[j]]);
//    getch();
    if(rat[top[j]] > maxtop)
    {
        maxtop=rat[top[j]];
        sample_top=top[j];
    }
}
sindx=sample_top;                      /* sample index max top              */
big_top=maxtop;                        /* value max top                     */
//printf("Sample biggest top:      %5d  time: %5.2f\n",sample_top,(float)sample_top*srate);
tmp_t=(float)sample_top*srate;         /* time in seconds max top           */
/*--------------------------------------------------------------------------*/
/* find the next biggest top and corresponding sample                       */
/*--------------------------------------------------------------------------*/
maxtop=rat[top[0]];
for(j=1;j<(tops-1);j++)
{
if(j<=(sindx-60) || j>=(sindx+60))
{
    if(rat[top[j]] > maxtop && rat[top[j]] != big_top)
    {
        if(top[j] > smp_max_amp-100 || top[j] < smp_max_amp+100)
        {
        maxtop=rat[top[j]];
        sample_top=top[j];
        }
    }
}
}
//printf("Sample next biggest top: %5d  time: %5.2f\n",sample_top,(float)sample_top*srate);
max_t=(float)sample_top*srate;
if(max_t > tmp_t)
{
    min_t=tmp_t;
}else{
    min_t=max_t;
    max_t=tmp_t;
}
//printf("P-Phase: %10.2f  S-Phase: %10.2f\n",min_t,max_t);
*p_ph=min_t;
*s_ph=max_t;
}

duration(pre,p,s,sr,indx,data, dr)
int pre;
float p;
float s;
float sr;
int indx;
float* data;
float* dr;
{
float sta1,lta1;
float sum;
int where2;
int unt;
int j;
float pre_avg;
float limite;
float extra;
int ws;
float limit;
int lpre;
int start;
unt=(int)((float)pre/sr);
sum=0.0;
//printf("PRE:%5d P:%6.2f S:%6.2f SR:%6.3f INDX:%5d UNT:%4d\n",pre,p,s,sr,indx,unt);
//exit(0);
ws=(int)(10.0/sr);
lta1=0.0;
for(j=1;j<(unt-1);j++) 
{
    extra=data[j]-data[j-1];
    extra=extra*extra;
    lta1=lta1+(fabs(data[j]) * fabs(data[j])-lta1)/ws;
}

pre_avg=lta1;
limite=pre_avg+(pre_avg*2.0)/100.0;
start=(int)((s+2.0) * (1./sr));
//printf("PRE_AVG %10.3f START: %5d LINMIT: %10.3f\n",pre_avg,start,limite);
//getch();
for(j=1;j<indx-2;j++)
{
    extra=data[j]-data[j-1];
    extra=extra*extra;
    lta1=lta1+(fabs(data[j]) * fabs(data[j])-lta1)/ws;
//printf("%5d %10.3f %10.3f %10.3f\n",j,pre_avg,lta1,limite);
    if(lta1 <= limite )
    {
        if(j > start)
        {
//printf("%5d %10.3f %10.3f %10.3f\n",j,pre_avg,lta1,limite);
//getch();
            where2=j;
            goto fin2;
        }
    }
//    R=sta1/lta1;
//    rat[j]=R;
//    if(R > max)
//    {
//        max=R;                         /* max sta/lta ratio                 */
//        where=j;                       /* sample index                      */
//    }
}       

//for(j=0;j<indx;j++)
//{
//    lta1=lta1+(fabs(data[j])-lta1)/50.;
//printf("%5d %10.3f %10.3f\n",j,pre_avg,lta1);
//    printf("%10.2f %10.2f %10.2f %10.2f\n",(float)j*sr,pre_avg,pre_avg+pre_avg*10./100.,lta1);
//    getch();
//    if(lta1 <= (pre_avg+pre_avg*10./100.) && j > start)
//    {
//printf("%5d %10.3f %10.3f\n",j,pre_avg,lta1);
//getch();
//        where2=j;
//        goto fin2;
//    }
//}       
fin2:
//    printf("Duration %10.2f seconds\n",(float)j*0.020);
    *dr=(float)where2*sr;

}
void output(chan,stcode,kanal,fl,fh,max,hv,snr,s_indx,dur,p,s)
int    *chan;
char*   stcode;
int    kanal;
float  fl;
float  fh;
int    max;
float  hv;
float  snr;
float  s_indx;
float  dur;
float  p;
float  s;
{
//CH F_LOW F_HIG  AMP   T-AMP MAX-SNR   T-SNR     DUR P-Phase S-Phase   S-P DIS
//dd fff.f fff.f dddd ffff.ff ffff.ff ffff.ff ffff.ff ffff.ff ffff.ff ff.ff ddd

printf("%c",stcode[ (chan[kanal]-1)*4]);
printf("%c",stcode[ (chan[kanal]-1)*4+1]);
printf("%c",stcode[ (chan[kanal]-1)*4+2]);
printf("%c",stcode[ (chan[kanal]-1)*4+3]);

printf(" %2d %6d %6.2f %7.1f %7.2f %7.2f %7.2f %7.2f %6.2f %4d\n",
chan[kanal],
max,hv,
snr,
s_indx,
dur,
p,
s,
s-p,
(int)((s-p)*HST));
fprintf(coh,"%c",stcode[ (chan[kanal]-1)*4]);
fprintf(coh,"%c",stcode[ (chan[kanal]-1)*4+1]);
fprintf(coh,"%c",stcode[ (chan[kanal]-1)*4+2]);
fprintf(coh,"%c",stcode[ (chan[kanal]-1)*4+3]);

fprintf(coh," %2d %6d %6.2f %7.1f %7.2f %7.2f %7.2f %7.2f %6.2f %4d\n",
chan[kanal],
max,hv,
snr,
s_indx,
dur,
p,
s,
s-p,
(int)((s-p)*HST));


fprintf(mail,"%c",stcode[ (chan[kanal]-1)*4]);
fprintf(mail,"%c",stcode[ (chan[kanal]-1)*4+1]);
fprintf(mail,"%c",stcode[ (chan[kanal]-1)*4+2]);
fprintf(mail,"%c",stcode[ (chan[kanal]-1)*4+3]);
fprintf(mail," %2d %6d %6.2f %7.1f %7.2f %7.2f %7.2f %7.2f %6.2f %4d\n",
chan[kanal],
max,hv,
snr,
s_indx,
dur,
p,
s,
s-p,
(int)((s-p)*HST));

}
/*-------------------------------------------------------------------------*/
/* Pick out station codes and components for corresponding channels        */
/*-------------------------------------------------------------------------*/
stacomp(nchan,stcode,comp,head)
int nchan;
char* stcode;
char* comp;
char* head;
{
int l;
int i;
int k;
k=160;
for(i=0;i<nchan;i++)
{
    for(l=0;l<4;l++)
    {
        stcode[i*4+l]=head[k+l];
        comp[i*4+l]=head[k+l+4];
    }
    k=k+8;
}
}
/*--------------------------------------------------------------------------*/
/* get default channels to process from /home/seislog/pr/pick.def           */
/*--------------------------------------------------------------------------*/
defchan(chan,nchtot,tre_start,no_tre_comp)
int *chan;
int *nchtot;
int *tre_start;
int *no_tre_comp;
{
int val;
int k;
int i;
int j;
FILE *par;
int chnr[MAXCHA];
int type[MAXCHA];
int proc[MAXCHA];
int no_ch_def;
int cnt;
int tel;
if((int)(par=fopen("/home/seislog/pr/pick.def","r"))==NULL)
{
    printf("File: /home/seislog/pr/pick.def must be defined !\n");
    exit(0);
}
for(k=0;k<MAXCHA;k++)
{
    val=fscanf(par,"%d %d %d",&chnr[k],&type[k],&proc[k]);
    if(val==EOF)
    {
//        printf("Number channels defined %d\n",k);
        no_ch_def=k;
        goto nomore;
    }
}
nomore:
/*--------------------------------------------------------------------------*/
/* find how many single channels and which ones to process as single channel*/
/*--------------------------------------------------------------------------*/
cnt=0;
for(k=0;k<no_ch_def;k++)
{
    if((type[k]==1 || type[k]==13) && proc[k]==1)
    {
        chan[cnt]=k+1;
//printf("%2d %2d K= %d CNT= %d chan[%d]=%d\n",type[k],proc[k],k,cnt,cnt,chan[cnt]);
//getch();
        cnt++;
    }
}
*nchtot=cnt;
//for(k=0;k<cnt;k++)
//    printf("%5d %5d\n",k,chan[k]);
/*--------------------------------------------------------------------------*/
/* find if any 3-comp, how many and which channels                          */
/*--------------------------------------------------------------------------*/
tel=0;
cnt=0;
for(k=0;k<no_ch_def;k++)
{
    if(type[k]==3 || type[k]==13)      /* do we have a 3-C ?                */
    {
        if(type[k+1]==3 || type[k+1]==13)
        {
            if(type[k+2]==3 || type[k+2]==13)
            {
                if(proc[k]==1 && proc[k+1]==1 && proc[k+2]==1)
                {
//printf("3-C\n");
                    tel++;
                    tre_start[cnt]=k+1;
                    tre_start[cnt+1]=k+2;
                    tre_start[cnt+2]=k+3;
                    cnt=cnt+3;
                    k=k+3;
                }
            }            
        }
    }
} 
*no_tre_comp=tel;
//printf("No of 3-component stations %5d\n",tel);
j=0;
for(k=0;k<tel;k++)
{
    for(i=0;i<3;i++)
    {
//        printf("%5d %5d\n",j,tre_start[j]);
        j++;
    }
}
fclose(par);
}
FindTops2(start,stopp,rat,no_of_samples,srate,p_ph,s_ph)
float  start;                          /* start_second                      */
float  stopp;                          /* stopp second                      */
float* rat;
int    no_of_samples;
float  srate;
float* p_ph;
float* s_ph;
{
int j;
float diff;
int pos_neg;
int tops;
int top[2000];
float maxtop;
int sample_top;
float big_top;
float min_t;
float max_t;
float tmp_t;
float vk;
float st;
float beg;
float at_beg;
int   sindx;
int   start_indx;                      /* start index                       */
int   stopp_indx;                      /* stopp index                       */
beg=200;
start_indx=(int)(start/srate)+10;
stopp_indx=(int)(stopp-start)/srate;
for(j=0;j<no_of_samples;j++)
{
    if(rat[j] < 0.0)
        rat[j]=0.0;
}
diff=rat[start_indx]-rat[start_indx-1];
if(diff > 0.0)
{
    pos_neg=1;
}else{
    pos_neg=-1;
}
tops=0;
for(j=start_indx;j<(no_of_samples-1);j++)
{
    diff=rat[j+1]-rat[j];
    switch(pos_neg)
    {
        case 1:   /* we are positive */
        if(diff <= 0.0)
        {
            top[tops]=j;
            pos_neg=-1;
            tops++;
        }
        break;
        case -1: /* we are negative */
        if(diff > 0.0)
        {
            pos_neg=1;
        }
        break;
   }
}
//printf("ICHAN %2d FILTER %2d has %5d tops\n",ichan,filter,tops-1);
maxtop=rat[top[0]];
for(j=1;j<(tops-1);j++)
{
//    printf("%5d %10.2f\n",top[j],rat[top[j]]);
//    getch();
    if(rat[top[j]] > maxtop)
    {
        maxtop=rat[top[j]];
        sample_top=top[j];
    }
}




sindx=sample_top;
big_top=maxtop;
//printf("Sample biggest top:      %5d  time: %5.2f\n",sample_top,(float)sample_top*srate);
tmp_t=(float)sample_top*srate;
maxtop=rat[top[0]];
for(j=1;j<(tops-1);j++)
{
if(j<=(sindx-50) || j>=(sindx+50))
{
    if(rat[top[j]] > maxtop && rat[top[j]] != big_top)
    {
        maxtop=rat[top[j]];
        sample_top=top[j];
    }
}
}
//printf("Sample next biggest top: %5d  time: %5.2f\n",sample_top,(float)sample_top*srate);
max_t=(float)sample_top*srate;
if(max_t > tmp_t)
{
    min_t=tmp_t;
}else{
    min_t=max_t;
    max_t=tmp_t;
}
//printf("P-Phase: %10.2f  S-Phase: %10.2f\n",min_t,max_t);
*p_ph=min_t;
*s_ph=max_t;
}
SMOOTH(ne,in,out)
int ne;
float *in;
float *out;
{
int i;
float coeff=1./3.;
for(i=1;i<ne-1;i++)
    out[i]=coeff * (in[i-1] + in[i] + in[i+1]);
out[0]=coeff * (2. * in[0] + in[1]);
out[ne-1]=coeff * (2. * in[ne-1] + in[ne-2]);
}

Find_P(ntops,top,samp_indx,start,stopp,rat,no_of_samples,srate,p_ph)
int   ntops;
int*    top;
float  samp_indx;                      /* time ind seconds max amplitude    */
float  start;                          /* start_second                      */
float  stopp;                          /* stopp second                      */
float* rat;
int    no_of_samples;
float  srate;
float* p_ph;
{
int smp_max_amp;                       /* sample maximum amplitude          */
int j;
float diff;
int pos_neg;
int tops;
float maxtop;
int sample_top;
float big_top;
float min_t;
float max_t;
float tmp_t;
float vk;
float st;
float beg;
float at_beg;
int   sindx;
int   start_indx;                      /* start index                       */
int   stopp_indx;                      /* stopp index                       */
beg=200;
start_indx=(int)(start/srate)+100;
stopp_indx=(int)(stopp-start)/srate;
smp_max_amp=(int)(samp_indx/srate);    /* index maximum amplitude           */
//printf("ICHAN %2d FILTER %2d has %5d tops\n",ichan,filter,tops-1);
/*--------------------------------------------------------------------------*/
/* find max top and corresponding sample                                    */
/*--------------------------------------------------------------------------*/
maxtop=rat[top[0]];
for(j=1;j<(ntops-1);j++)
{
//    printf("%5d %10.2f\n",top[j],rat[top[j]]);
//    getch();
    if(rat[top[j]] > maxtop)
    {
        maxtop=rat[top[j]];
        sample_top=top[j];
    }
}
sindx=sample_top;                      /* sample index max top              */
for(j=sindx-(int)(10.0/srate);j<sindx+(int)(10./srate);j++)
    rat[j]=0.0;
*p_ph=(float)sample_top*srate;         /* time in seconds max top           */
}
Find_S(ntops,top,samp_indx,start,stopp,rat,no_of_samples,srate,s_ph)
int    ntops;
int*    top;
float  samp_indx;                      /* time ind seconds max amplitude    */
float  start;                          /* start_second                      */
float  stopp;                          /* stopp second                      */
float* rat;
int    no_of_samples;
float  srate;
float* s_ph;
{
int smp_max_amp;                       /* sample maximum amplitude          */
int j;
float diff;
int pos_neg;
int tops;
float maxtop;
int sample_top;
float big_top;
float min_t;
float max_t;
float tmp_t;
float vk;
float st;
float beg;
float at_beg;
int   sindx;
int   start_indx;                      /* start index                       */
int   stopp_indx;                      /* stopp index                       */
beg=200;
start_indx=(int)(start/srate)+100;
stopp_indx=(int)(stopp-start)/srate;
smp_max_amp=(int)(samp_indx/srate);    /* index maximum amplitude           */
/*--------------------------------------------------------------------------*/
/* find max top and corresponding sample                                    */
/*--------------------------------------------------------------------------*/
maxtop=rat[top[0]];
for(j=1;j<(ntops-1);j++)
{
//    printf("%5d %10.2f\n",top[j],rat[top[j]]);
//    getch();
if(top[j] > smp_max_amp-50 && top[j] <smp_max_amp+5)
{
    if(rat[top[j]] > maxtop)
    {
        maxtop=rat[top[j]];
        sample_top=top[j];
    }
}
}
sindx=sample_top;                      /* sample index max top              */
big_top=maxtop;                        /* value max top                     */
//printf("Sample biggest top:      %5d  time: %5.2f\n",sample_top,(float)sample_top*srate);
*s_ph=(float)sample_top*srate;         /* time in seconds max top           */
//*s_ph=tmp_t;
}

Find_Peaks(top,start,stopp,rat,no_of_samples,srate)
int*    top;
float  start;                          /* start_second                      */
float  stopp;                          /* stopp second                      */
float* rat;
int    no_of_samples;
float  srate;
{
int j;
float diff;
int pos_neg;
int tops;
int   start_indx;                      /* start index                       */
int   stopp_indx;                      /* stopp index                       */
start_indx=(int)(start/srate)+100;
stopp_indx=(int)(stopp-start)/srate;
/*--------------------------------------------------------------------------*/
/* zero all negative values                                                 */
/*--------------------------------------------------------------------------*/
for(j=0;j<no_of_samples;j++)
{
    if(rat[j] < 0.0)
        rat[j]=0.0;
}
/*--------------------------------------------------------------------------*/
/* count and locate all tops                                                */
/*--------------------------------------------------------------------------*/
diff=rat[start_indx]-rat[start_indx-1];
if(diff > 0.0)
{
    pos_neg=1;
}else{
    pos_neg=-1;
}
tops=0;
for(j=start_indx;j<(no_of_samples-1);j++)
{
    diff=rat[j+1]-rat[j];
    switch(pos_neg)
    {
        case 1:   /* we are positive */
        if(diff <= 0.0)
        {
            top[tops]=j;
            pos_neg=-1;
            tops++;
        }
        break;
        case -1: /* we are negative */
        if(diff > 0.0)
        {
            pos_neg=1;
        }
        break;
   }
}
return(tops);
}
MAXONE(in,out,ne)
float* in;
float* out;
int    ne;
{
float  max;
int    i;
max=0.0;
in[0]=0.0;
for(i=0;i<ne;i++)
{
    if(in[i] > max)
        max=in[i];
}
for(i=0;i<ne;i++)
    out[i]=(in[i]/max);
}
PPHA(in,sr,ne,pc,phase)
float* in;
float  sr;
int    ne;
float  pc;
float *phase;
{
int i;
float lev;
lev=pc/100.0;
for(i=0;i<ne;i++)
{
    if(in[i]>= lev)
        goto pp;
}
pp:
*phase= (float)i*sr;
//printf("I=%d SR=%10.3f phase=%10.3f\n",i,sr,phase);
}
SPHA(in,sr,ne,pc,tmax,phase)
float* in;
float  sr;
int    ne;
float  pc;
float  tmax;
float *phase;
{
int i;
float lev;
int start;
int stopp;
float procent;
procent=pc;
try_again:
lev=procent/100.0;
start=(int)(tmax-2.0)/sr;
stopp=(int)(tmax/sr);
for(i=start;i<stopp;i++)
{
    if(in[i]>= lev)
        goto pp;
}
procent=procent-3.0;
if(procent < 1.0)
    goto pp;
goto try_again;
pp:
*phase= (float)i*sr;
//printf("T_MAX %10.3f\n",tmax);
//printf("I=%d SR=%10.3f phase=%10.3f\n",i,sr,phase);
}
SEARCH(samp_indx,start,stopp,data,ne,sr,peak)
float  samp_indx;                      /* time ind seconds max amplitude    */
float  start;                          /* start_second                      */
float  stopp;                          /* stopp second                      */
float* data;
int    ne;
float  sr;
float* peak;
{
int smp_max_amp;                       /* sample maximum amplitude          */
int j;
float maxtop;
int sample_top;
int   sindx;
int   start_indx;                      /* start index                       */
int   stopp_indx;                      /* stopp index                       */
start_indx=(int)(start/sr)+100;
stopp_indx=(int)(stopp-start)/sr;
smp_max_amp=(int)(samp_indx/sr);    /* index maximum amplitude           */
/*--------------------------------------------------------------------------*/
/* find max top and corresponding sample                                    */
/*--------------------------------------------------------------------------*/

maxtop=0.0;
for(j=start_indx;j<stopp_indx;j++)
{
//printf("%6d %10.3f %10.3f %10.3f\n",j,(float)(j)*sr,maxtop,data[j]);
//getch();
    if(data[j] >= maxtop)
    {
        maxtop=data[j];
        sample_top=j;
//        goto okk;
    }
}
//        printf("MAX=%10.3f Time=%10.3f\n",maxtop,(float)(sample_top)*sr);
//printf("Cant find MAX\n");
//exit(0);
//okk:
sindx=sample_top;
for(j=(sindx-(int)(2.0/sr));j<(sindx+(int)(2.0/sr));j++)
    data[j]=0.0;
//printf("maxtop %10.3f\n",maxtop);
//getch();
//for(j=sample_top-10;j<sample_top+10;j++)
//    data[j]=0.0;
*peak=(float)sample_top*sr;         /* time in seconds max top           */
}
SEARCH_S(samp_indx,start,stopp,data,ne,sr,peak)
float  samp_indx;                      /* time ind seconds max amplitude    */
float  start;                          /* start_second                      */
float  stopp;                          /* stopp second                      */
float* data;
int    ne;
float  sr;
float* peak;
{
int smp_max_amp;                       /* sample maximum amplitude          */
int j;
float maxtop;
int sample_top;
int   sindx;
int   start_indx;                      /* start index                       */
int   stopp_indx;                      /* stopp index                       */
start_indx=(int)(start/sr);
stopp_indx=(int)(stopp)/sr;
smp_max_amp=(int)(samp_indx/sr);    /* index maximum amplitude           */
/*--------------------------------------------------------------------------*/
/* find max top and corresponding sample                                    */
/*--------------------------------------------------------------------------*/

maxtop=0.0;
for(j=start_indx;j<stopp_indx;j++)
{
//printf("%6d %10.3f %10.3f %10.3f\n",j,(float)(j)*sr,maxtop,data[j]);
//getch();
    if(data[j] >= maxtop)
    {
        maxtop=data[j];
        sample_top=j;
//        goto okk;
    }
}
//        printf("MAX=%10.3f Time=%10.3f\n",maxtop,(float)(sample_top)*sr);
//printf("Cant find MAX\n");
//exit(0);
//okk:
sindx=sample_top;
*peak=(float)sample_top*sr;         /* time in seconds max top           */
}
stco_label(chan,ichan,next,stcode,comp,thead)
int* chan;
int ichan;
float next;
char* stcode;
char* comp;
char* thead;
{
char value[80];
        value[0]=stcode[ (chan[ichan]-1)*4];
        value[1]=stcode[ (chan[ichan]-1)*4+1];
        value[2]=stcode[ (chan[ichan]-1)*4+2];
        value[3]=stcode[ (chan[ichan]-1)*4+3];
        value[4]='\0';
        _moveto(10,next-30);
        _outgtext(value);
        value[0]=comp[ (chan[ichan]-1)*4];
        value[1]=comp[ (chan[ichan]-1)*4+1];
        value[2]=comp[ (chan[ichan]-1)*4+2];
        value[3]=comp[ (chan[ichan]-1)*4+3];
        value[4]='\0';
        _moveto(50,next-30);
        _outgtext(value);
        _moveto(400,10);
        _outgtext(thead);
}
ZERO_CROSSING(ne,data)
int ne;
float *data;
{
int j;
int k;
int from,to;
float f1,f2;
int cr;
cr=0;
from=0;
to  =50;
for(k=0;k<(ne-100);k++)
{
    cr=0;
    for(j=from;j<to;j++)
    {
        f1=data[j];
        f2=data[j+1];
        if((f1 < 0.0 && f2 > 0.0) || (f1 > 0.0 && f2 < 0.0))
            cr++;
    }
printf("J: %6d CR: %6d\n",j,cr);
//getch();
    from++;
    to++;
}
}
SEARCH2(start,stopp,data,ne,sr,peak)
float  start;                          /* start_second                      */
float  stopp;                          /* stopp second                      */
float* data;
int    ne;
float  sr;
float* peak;
{
int smp_max_amp;                       /* sample maximum amplitude          */
int j;
float maxtop;
int sample_top;
int   sindx;
int   start_indx;                      /* start index                       */
int   stopp_indx;                      /* stopp index                       */
start_indx=(int)(start/sr);
stopp_indx=(int)(stopp/sr);
//printf("START:%6.2f STOPP:%6.2f START_INDX:%4d STOPP_INDX:%4d\n",start,stopp,start_indx,stopp_indx);
/*--------------------------------------------------------------------------*/
/* find max top and corresponding sample                                    */
/*--------------------------------------------------------------------------*/

maxtop=0.0;
for(j=start_indx;j<stopp_indx;j++)
{
//printf("%6d %10.3f %10.3f %10.3f\n",j,(float)(j)*sr,maxtop,data[j]);
//getch();
    if(data[j] >= maxtop)
    {
        maxtop=data[j];
        sample_top=j;
//        goto okk;
    }
}
//        printf("MAX=%10.3f Time=%10.3f\n",maxtop,(float)(sample_top)*sr);
//printf("Cant find MAX\n");
//exit(0);
//okk:
sindx=sample_top;
for(j=(sindx-(int)(2.0/sr));j<(sindx+(int)(2.0/sr));j++)
    data[j]=0.0;
//printf("maxtop %10.3f\n",maxtop);
//getch();
//for(j=sample_top-10;j<sample_top+10;j++)
//    data[j]=0.0;
*peak=(float)sample_top*sr;         /* time in seconds max top           */
}
label1(tid,p,next,thead)
float tid;
int   p;
int next;
char* thead;
{
char value[80];
        _moveto(p+10,next);
//        sprintf(value,"P-AVG-STA %6.2f\n",tid);
        _outgtext(thead);
}
TimScale8(ds,color,no_of_samples,displacement,xscale,sr)
int ds;
int color;
int no_of_samples;
float displacement;
float xscale;
float sr;
{
char karak[10];
int oldcolor;
int j;
int x,y;
int spsec;
int spsec2;
int cnt;
int cnt2;
int pphase;
int sphase;
int duration;
spsec=(int)(1./sr);
spsec2=(int)(1./sr) * 0.1;
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
cnt=0;
cnt2=0;
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)displacement+10;
    _lineto(x,y);
    cnt++;
    cnt2++;
    if(cnt==spsec)
    {
        cnt=0;
        _lineto(x,y-10);
        _moveto(x,y);
    }
    if(cnt2==spsec2)
    {
        cnt2=0;
        _lineto(x,y-4);
        _moveto(x,y);
    }
}
_setcolor(oldcolor);
}
same_phase(fase)
float fase[];
{
float df;
int truth[3];

df=fase[0]-fase[1];
if(fabs(df) <= LIK)
{
    truth[0]=1;
}else{
    truth[0]=0;
}

df=fase[0]-fase[2];
if(fabs(df) <=LIK)
{
    truth[1]=1;
}else{
    truth[1]=0;
}

df=fase[1]-fase[2];
if(fabs(df) <=LIK)
{
    truth[2]=1;
}else{
    truth[2]=0;
}
//printf("%5.2f %5.2f %5.2f %d %d %d\n",fase[0],fase[1],fase[2],truth[0],truth[1],truth[2]);
if(truth[0]==1 && truth[1]==1 && truth[2]==1)
    return(123);
if(truth[0]==1 )
    return(12);
if(truth[1]==1 )
    return(13);
if(truth[2]==1 )
    return(23);
return(-1);
}