/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:02/08/99                       */
/* Program:PK                                                               */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/*                                                                          */
/****************************************************************************/     

#include <math.h>
#include "shar_mem.h"
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include "filedef.h"
#include <stdio.h>
#include <conio.h>
#include <graph.h>
#include <sys/qnxterm.h>
#include <stdlib.h>
#include <term.h>
#include <sys/qioctl.h>
#include <signal.h>
#include <string.h>
#define  XL 639.
#define  YL 479.
#define  M2(arr,x,y) (*((arr)+(y)*(dimx)+(x))) /* define matrix for data    */
#define  DASHED     0xc0c0             /* linetype after coherence method   */
#define  NEW_DASHED 0xeee
#define  SOLID      0xffff
#define  SIZ 256                       /* max size of input line            */

#define  HST      10.0                 /* velocity km/s                     */
#define  LIK      2.00                 /* phases equal if below this number */
#define  MIN_COH  0.4                  /* coherence good if above this number*/
#define  VAC      13.0                  /* variation coeficient              */
#define  ORDER    5                    /* filter order                      */
#define  FL       1.0
#define  FH       7.0
#define  WIN      0.2                  /* window before p-avg to look for coh*/
#define  KWIN     1.0                  /* window before p_avg to look for kof*/
#define  KOFLIM   2.5                 /* level of kof to decide new P      */
#define  PIX_FULL 50
#define  PIX_HALF 25

FILE     *kff;
FILE     *txt;
FILE     *buf;
FILE     *con;
FILE     *evt;                         /* pointer to event file to plot     */
FILE     *cor;
FILE     *coh;                         /* pointer results 3-component       */
FILE     *sta;                         /* pointer station file              */
FILE     *phs;                         /* pointer phases                    */
FILE     *pri;                         /* pointer default channels          */
FILE     *mail;                        /* pointer to mail file              */
FILE     *lst;
FILE     *loc;
FILE     *net;
float prev_max;
float    procoh1();                    /* return type from procoh1          */
float    cross();                      /* return type from cross            */
char     stasjon[20];

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
float    *matr;                        /* pointer to data matrix            */

main(argc,argv)
int argc;
char *argv[];
{
float deg_lat,deg_lon;
float dst;
float slatt1[100];
float slonn1[100];
char statreg[100][5];
float xx0,xx1,yy1;
float P_KOF;
int   intil;
int   intil0;
float maxkof,maxkof2;
int   ik;
float koff,koff2;
float TIME_MAX_AMP;                    /* time in seconds for maximum ampl. */
int   MAX_AMP;                         /* maximum amplitude                 */
float TIME_MAX_SNR;
float MAX_SNR;
int spick;
float sum;
float avgsum;
float sd;
float t_sd;
int bok1,bok2;
int aavg;
int pp;
float p_snitt_sta;
float s_snitt_sta;
float d_snitt;
float Mb;
int teller;
float p_coher,p_coher2;
float p_coher_peak;
float s_coher_peak;
float s_coher;
int sw0;
int ixc;
int ixc1;
float n4_max2;
float t_n4_max2;
int pre;
int fra,til;
int fra1,til1;
int  ne;
short   equalpp;
short   equalss;
float zeroto;
float ffra,ftil;
char  value[80];                       /* text for graphic output           */
char  cur_sta[10];
float percent;                         /* percent of full scale (1.00)      */
float p_prelim;                        /* preliminar p                      */
float s_prelim;                        /* preliminar s                      */
float p_prelim2;                       /* preliminar p                      */
float s_prelim2;                       /* preliminar s                      */
float ph1,ph2;                         /* possible phases                   */
int ntops;                             /* number of tops                    */
int top[2000];
int      nsm;                          /* number of smooth iterations       */
int      bryter;
int      start_indx;                   /* index to start computing average  */
int      stopp_indx;                   /* index to stopp computing average  */
float    cut;                          /* seconds to cut in front of p      */     
float    n_p,n_s;                      /* new p & new s                     */
float    zoom;                         /* seconds to include in front of p  */
float    zooml;                        /* total zoom length in seconds      */
int      ext1,ext2;                    /* extract data from-to              */
int      no_ext;                       /* number of extracted samples       */
int      sh;                           /* counter                           */
int      pkt;                          /* counter                           */
int      each_flt;                     /* pixels per filter                 */
int      no_tre_comp;                  /* number of 3-component station     */
float    pvalid;                       /* valid p phase                     */
float    svalid;
float    avstand;                      /* distance station-event            */
int      dimx;                         /* dimension data matrix             */
int      dimy;                         /* dimension data matrix             */
float    start_sec;                    /* second to start computing average */
float    stopp_sec;                    /* second to stop computing average  */
float    snr_max[MAXCHA];
float    snr_tim[MAXCHA];
int      tre_start[MAXCHA];            /* start channels 3-component station*/
int      chan[MAXCHA];                 /* channel number to plot            */
char     stcode[4*MAXCHA];             /* station codes                     */
char     comp[4*MAXCHA];               /* component codes                   */

int      stop;                         /* second to stop coherence anal.    */
float    ma[MAXCHA];                   /* time max amplitude in channel     */
int      am[MAXCHA];                   /* max amplitude                     */
float    pavg;                         /* average p time                    */
float    savg;                         /* average s time                    */
int      lik;                          /* counter valid p and s times       */
int      equalp[MAXCHA];               /* counter equal p phases            */
int      equals[MAXCHA];               /* counter equal s phases            */
float    velocityf[10];                /* velocity from station file        */
int      ndepth;                       /* no. of depths to test (stationfile*/
float    h0;                           /* starting depth (station file)     */
float    c11,c22,c33;                  /* coda parameters (station file)    */
float    flon,flat;
float    slon1,slon2,slat1,slat2;      /* min,max lon and lat (station file)*/
float    slatt[100],slonn[100];        /* station lat and lon (station file)*/
char     stat[100][8];                 /* station names from station file   */
int      nstation;                     /* number of stations in station file*/
float    x11,y11;                      /* coordinates in kilometers         */
float    slat,slon;                    /* latitude,longitude current station*/
float    lat,lon;                      /* latitude,longitude current event  */
int      r_kan[MAXCHA];                /* channel read from 'threecomp'     */
float    r_fl[MAXCHA];                 /* filter low from 'threecomp'       */
float    r_fh[MAXCHA];                 /* filter high from 'threecomp'      */
int      r_amp[MAXCHA];                /* max amp from 'threecomp'          */
float    r_tamp[MAXCHA];               /* time max amp from 'threecomp'     */
float    r_max_snr[MAXCHA];            /* max SNR from 'threecomp'          */
float    r_t_snr[MAXCHA];              /* time max SNR from 'threecomp'     */
float    r_dur[MAXCHA];                /* duration from 'threecomp'         */
float    r_pp[MAXCHA];                 /* p-phase time from 'threecomp'     */
float    r_ss[MAXCHA];                 /* s-phase time from 'threecomp'     */
float    r_s_p[MAXCHA];                /* s-p time from 'threecomp'         */
int      r_dis[MAXCHA];                /* distance from 'three p'         */
float    coherence;                    /* max coherence from 'three  p'    */
float    t_coher;                      /* time max coherence from 'threecomp*/
float    velocity;                     /* velocity from 'threecomp'         */
float    RMS;                          /* rms from 'threecomp'              */
float    azimuth;                      /* azimuth from 'threecomp'          */
float    c_p_ph;                       /* p-phase 3-comp from 'three p'   */
float    c_s_ph;                       /* s-phase 3-comp from 'threecomp'   */
float    c_s_p;                        /* s-p 3-comp from 'threecomp'       */
int      c_distance;                   /* distance 3-comp from 'threecomp'  */
char     statf[10];                    /* station from header               */
int      arrow;                        /* vertical position of arrows       */
int      dsp;                          /* displacement traces               */
int      no_of_filters;                /* number of filters to use          */
int      threec;                       /* switch to indicate 3-comp anal.   */
float    dura;                         /* duration of event                 */
float    p_ph;                         /* P-phase time in seconds from start*/
float    s_ph;                         /* S-phase time in seconds from start*/
int      color;                        /* color for drawing                 */
int      plot;                         /* switch plot=1 plot plot=0 no plot */
int      irate;                        /* samples per second                */
int      filter;                       /* index filters                     */
float    next;                         /* displacement in units for next plt*/
float    next2;                        /* displacement in units for next plt*/
float    xsc;                          /* scaling factor x-axis             */
float    xscz;
int      ww;                           /* window size in samples coherence  */
int      indx;                         /* number of samples in event        */
float    hvor;                         /* time max amplitude                */
int      from;                         /* index to extract data             */
int      to;                           /* index to extract data             */
float    valfl;                        /* value extracted from data matrix  */
int      smp;                          /* index                             */
char     infile1[100];                 /* input filename or DPX no          */
char     longf[100];                   /* file to look for                  */
char     head[2000];                   /* header of event                   */
FILE     *fp;                          /* pointer to input                  */
char     thead[200];                    /* time string of event header       */
float    kof[10000];
float    kof2[10000];
float    c1[40000];                    /* original data for coherece        */
float    c2[40000];                    /* original data for coherence       */
float    c3[40000];                    /* original data for coherence       */
float    pv[40000];                    /* plot vector                       */
float    rat[40000];                   /* array for SNR                     */
int      iworkl[40000];                /* data from one buffer (long)       */
float    data[40000];                  /* original data current trace       */
float    coher[40000];                 /* coherence                         */
float    n4;                           /* coherence                         */
int      ipts;                         /* start sample                      */
int      lpts;                         /* stop sample                       */
float    pvel;                         /* velocity of p wave                */
float    psrat;                        /* Vp Vs ratio                       */
float    oprocoh[10];                  /* results from coherence            */

int      nchtot;                       /* selected number of channels to plt*/
int      dpx;                          /* detection number                  */
int      i;                            /* counter                           */
int      k;                            /* counter                           */
int      j;                            /* counter                           */
int      l;                            /* counter                           */
int      curdpx;                       /* current dpx number                */
int      tot;                          /* total duration of event in sec    */
long     year;                         /* year from header                  */
long     month;                        /* month from header                 */
long     day;                          /* day from header                   */
long     hour;                         /* hour from header                  */
long     min;                          /* minute from header                */
float    sec;                          /* seconds from header               */
int      nchan;                        /* number of channels from header    */
float    srate;                        /* samplerate from header            */
int      ntotal;                       /* ntotal from header                */
int      unit;                         /* unit from header                  */
int      nsamp;                        /* nsamp from header                 */
int      recsize;                      /* from header                       */
int      reclength;                    /* from header                       */
int      ichan;                        /* channel counter                   */
int      isamp;                        /* counter/index  of samples         */
int      kbuf;                         /* buffer counter                    */
float    samp_indx;                    /* time max ratio                    */
int      snr_mxx;                      /* max ratio                         */
float    p[MAXCHA];                    /* storage p-phases                  */
float    s[MAXCHA];                    /* storage s-phases                  */
float    d[MAXCHA];                    /* storage duration                  */
int      ws,wl;                        /* short & long window stalta        */
int      maxamp2;                      /* maximum amplitude of trace        */
float    ttotal;                       /* total duration of event           */
float    time1;                        /* start time relative to first sampl*/
float    time2;                        /* end time relative to first sample */
int      ret;                          /* return value defchan              */
int      manual;                       /* from console or automatic         */
float fh1,fl1;
/*--------------------------------------------------------------------------*/
/* initialize coherence values                                              */
/*--------------------------------------------------------------------------*/
float    n4_max=0.0;
float    t_n4_max=0.0;
int      smp_indx=0;
float    azi=0.0;
float    vel=0.0;
float    rms=0.0;

bryter=0;
sw0=0;
teller=-1;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"PK open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"PK mmap failed : %s\n",strerror(errno));
    exit(1);
}
if (argc==2)
{ 
//    printf("%s\n",argv[1]);
    sprintf(longf,"%s\0",argv[1]);
    plot=0;
    no_of_filters=1;
    manual=0;
    goto cont;
}
/*--------------------------------------------------------------------------*/
/* if no file name in argument, ask for parameters                          */
/*--------------------------------------------------------------------------*/
printf("Enter DPX no. or -1 for filename or -2 for DPX.LIS:\n");
scanf("%d",&dpx);
/*--------------------------------------------------------------------------*/
/* if dpx=0, ask for filename                                               */
/*--------------------------------------------------------------------------*/
switch(dpx)
{
    case -1:
    printf("Enter filename:\n");
    scanf("%s",&infile1);
    sprintf(longf,"/home/events/");
    strcat(longf,infile1);
    break;

    case -2:
    system("rm -f /home/work/dpx.lis");
    system("/home/seislog/fil_dpx");
    system("more /home/work/dpx.lis");
    printf("Enter DPX no. from DPX.LIS:\n");
    scanf("%d",&dpx);

    if((lst=fopen("/home/work/dpx.lis","r"))==NULL)
    {
        printf("Could not open file containing dpx summary\n");
        exit(0);
    }
label1:
    bryter=1;
    for(;;)
    {
        if(fscanf(lst,"%5d %s",&i,longf) != -1)
        {
            printf("%5d %s\n",i,longf);
            if(i==dpx)
            {
                printf("Event: %s\n",longf);
                goto fin;              
            }
        }else{
            printf("Number %d is not in DPX.LST\n",dpx);
            fclose(lst);
            exit(0);
        }
    }
fin:
//    fclose(lst);

    break;

    default:
    dpxnam(dpx,infile1,&ttotal);       /* recorded, extract filename        */
    sprintf(longf,"/home/events/");
    strcat(longf,infile1);
    break;
}
manual=1;
printf("Enter PLOT:\n");
scanf("%d",&plot);

//printf("Enter ww:\n");
//scanf("%d",&ww);

//printf("Enter low:\n");
//scanf("%f",&fl1);

//printf("Enter high:\n");
//scanf("%f",&fh1);

//printf("File path: %s\n",longf);
cont:
if((int)(evt=fopen(longf,"r"))!=NULL)  /* try to open the event file        */
{
    goto found;
}else{
    printf("Can not find file: %s\n",longf);
    exit(0);
}
found:
/*--------------------------------------------------------------------------*/
/* Read header part of event                                                */
/*--------------------------------------------------------------------------*/
fseek(evt,0,0);
fread(head,1024,1,evt);
/*--------------------------------------------------------------------------*/
/* extract header information from event header                             */
/*--------------------------------------------------------------------------*/
hdr_info(head,&year,&month,&day,&hour,&min,&sec,&nchan,&srate,&ntotal,&unit,&nsamp,&recsize,&reclength,&pre);
ttotal=(float)(ntotal/nsamp);
//printf("Start time: %2d:%2d:%6.3f %2d/%2d/%4d\n",hour,min,sec,day,month,year);
sprintf(thead,"%2d:%2d:%6.3f %2d/%2d/%4d\n",hour,min,sec,day,month,year);
thead[23]='\0';
statf[0]=head[160];
statf[1]=head[161];
statf[2]=head[162];
statf[3]='\0';
//printf("Total recording (seconds):%7.2f  S-rate: %7.3f Channels: %3d Station: %s\n",
//ttotal,srate,nchan,statf); 
for(i=0;i<MAXCHA;i++)
    p[i]=s[i]=ma[i]=d[i]=-1.0;
/*--------------------------------------------------------------------------*/
/*   Get station codes and components for NCHAN channels                    */
/*--------------------------------------------------------------------------*/
stacomp(nchan,stcode,comp,head);
//for(i=0;i<nchan;i++)
//    printf("%s %s\n",stcode[i],comp[i]);
/*--------------------------------------------------------------------------*/
/* allocate memory                                                          */
/*--------------------------------------------------------------------------*/
//printf("TTOTAL %f NSAMP %d\n",ttotal,nsamp);
dimx=nchan;
dimy=(int)ttotal*nsamp;
if(!(matr=(float *)malloc(dimx*dimy*sizeof(float)) ))
{
    printf("Out of memory !!");
    exit(0);
}
/*--------------------------------------------------------------------------*/
/* read whole event into memory matrix                                      */
/*--------------------------------------------------------------------------*/
ttotal=ttotal-1;
isamp=0;                               /* record counter                    */
for(kbuf = (1024/reclength)+1; kbuf < (int)ttotal+(1024/reclength)+1; kbuf++)
{
    fseek(evt,kbuf*reclength,SEEK_SET);/* position at next record           */
    fread(iworkl,reclength,1,evt);     /* read 1 block of data              */
    dmx_buf(dimx,isamp,nsamp,nchan,(int)ttotal,iworkl); /* demultiplex record    */
    isamp=isamp+1;                     /* icrement record counter           */
}
fclose(evt);
//printf("ISAMP %d\n",isamp);
/*--------------------------------------------------------------------------*/
/* get default channels to plot from /home/seislog/pr/pick.def              */
/*--------------------------------------------------------------------------*/
defchan(chan,&nchtot,tre_start,&no_tre_comp);
j=0;
time1=0.0;                             /* start time                        */
time2=ttotal;                          /* stop time                         */
from=(int)(time1/srate);               /* start index                       */
to  =(int)(time2/srate);               /* stop index                        */
irate=(int)(1./srate);                 /* samples per second                */

//coh=fopen("/home/work/netw","w");  /* open file for results selected ch */
system("rm -f /home/work/newmail.txt");
//mail=fopen("/home/work/newmail.txt","w"); /* open file for results selected ch */

//system("rm -f /home/work/phases"); /* remove old file                   */
phs=fopen("/home/work/phases","a"); /* open file for phases             */


/*--------------------------------------------------------------------------*/
/* read station file                                                        */
/*--------------------------------------------------------------------------*/

ret=Read_Station_File(&velocityf,&h0,&ndepth,&c11,&c22,&c33,&slon1,&slon2,&slat1,&slat2,statreg,&slatt1,&slonn1,&nstation);


ws=5;                                 /* short window (sta)                */
wl=600;                                /* long window (lta)                 */     

dsp=(int)YL/10;
next=(float)PIX_HALF+8;

for(i=0;i<20000;i++)
    c1[i]=0.0;
if(plot==1)                        /* plot ?                            */
    GraphicsMode();                /* set to graphics mode              */
/*--------------------------------------------------------------------------*/
/* loop through active channels                                             */
/*--------------------------------------------------------------------------*/
for(ichan=0;ichan<3;ichan++)   /* channel to be processed           */
{
/*--------------------------------------------------------------------------*/
/* start main loop FILTER                                                   */
/*--------------------------------------------------------------------------*/
    design(ORDER,"BP","BU",30.0,0.3,FL,FH,srate);
    isamp=ntotal;                  /* counter for number of samples     */
    indx=0;
    for(smp=from;smp<to;smp++)     /* extract channel to process vector */
    {
        valfl=M2(matr,ichan,smp);
        data[indx]=valfl;
        indx=indx+1;               /* counting samples                  */
    }

    xsc=XL/(float)indx;            /* scale factor x-axis               */
/*--------------------------------------------------------------------------*/
/* draw time-scale                                                          */
/*--------------------------------------------------------------------------*/
    if(ichan==0)
    {
        if(plot==1)
        {
            color=12;                  /* RED                               */
            X_SCALE_10(color,indx,xsc,srate,sec);
        }
    }

    REMOVE_DC(indx,data);              /* remove DC                         */

    for(i=0;i<indx;i++)                /* sum absolute values of all channels*/
        c1[i]=c1[i]+fabs(data[i]);

    apply(data,indx,1);                /* Apply FILTER                      */
/*--------------------------------------------------------------------------*/
/* PLOT1: ORIGINAL DATA FILTERED                                            */
/*--------------------------------------------------------------------------*/
    for(i=0;i<indx;i++)                /* move to plot vector               */
        pv[i]=data[i]* -1;             /* TURN POLARITY FOR PLOTTING        */
    SCALETO_1(indx,pv,PIX_HALF);          /* scale to dsp/2                    */
    if(plot==1)                        /* plot filtered data                */
    {    
        color=14;                      /* YELLOW                            */
        PlotVec(color,indx,next,pv,xsc);
        stco_label(chan,ichan,next,stcode,comp,thead);
        arrow=next;
    }
/*--------------------------------------------------------------------------*/
/* find maximum SNR, time of max. SNR and array of running SNR              */
/*--------------------------------------------------------------------------*/
    STA_LTA_PH(srate,indx,data,ws,wl,rat);
/*--------------------------------------------------------------------------*/
/* find maximum amplitude and sample index in signal                        */
/*--------------------------------------------------------------------------*/
    MAX_AMPL(indx,data,srate,&TIME_MAX_AMP,&MAX_AMP);
    ma[ichan]=TIME_MAX_AMP;            /* save time max amp each channel    */
    am[ichan]=MAX_AMP;                 /* save max amplitude each channel   */
/*--------------------------------------------------------------------------*/
/* zero x % of pre-event and zero from maximum amplitude+n seconds to end   */
/*--------------------------------------------------------------------------*/
    stopp_sec=((float)pre * 50.0)/100.0;
    for(i=0;i<(int)(stopp_sec/srate);i++)
        rat[i]=0.0;
    if(ichan != 0)
    {
        for(i=(int)((TIME_MAX_AMP+5.0)/srate);i<indx;i++)
            rat[i]=0.0;
    }
/*--------------------------------------------------------------------------*/
/* find maximum amplitude and sample index in signal                        */
/*--------------------------------------------------------------------------*/
    MAX_AMPL(indx,rat,srate,&samp_indx,&snr_mxx);
    snr_max[ichan]=(float)snr_mxx;
    snr_tim[ichan]=samp_indx;
/*--------------------------------------------------------------------------*/
/* smooth result from sta/lta                                               */
/*--------------------------------------------------------------------------*/
    nsm=100;
    SMOOTH(indx,rat,c2);
    for(i=0;i<nsm;i++)
    {
        SMOOTH(indx,c2,c3);
        SMOOTH(indx,c3,c2);
    }

    for(j=0;j<indx;j++)
        c2[j]=(c2[j]*10000.)*(c2[j]*10000);
/*--------------------------------------------------------------------------*/
/* derivate smoothed result from sta/lta                                    */
/*--------------------------------------------------------------------------*/
    start_sec=0.0;
    stopp_sec=ttotal;
    DERIV(start_sec,stopp_sec,srate,c2,indx);
/*--------------------------------------------------------------------------*/
/* find 2 highest peaks (p and s) and time of occurrence                    */
/*--------------------------------------------------------------------------*/
for(i=0;i<indx;i++)
    c3[i]=c2[i];
//    MAXONE(c2,c3,indx); /* debug */
    zeroto=(float)pre/4.0;
    for(j=0;j<(int)(zeroto/srate);j++)
        c3[j]=0.0;
/*--------------------------------------------------------------------------*/
/* PLOT2: STA/LTA RATIO SMOOTHED DERIVATED                                  */
/*--------------------------------------------------------------------------*/
    for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
        pv[i]=c3[i];
    SCALETO_1(indx,pv,PIX_HALF);        /* scale plot vector                 */
    for(j=0;j<indx;j++)
        pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
    if(plot==1)
    {    
        color=11;                  /* CYAN                              */
        next=next+(float)PIX_FULL ;
        PlotVec(color,indx,next,pv,xsc);
    }
    start_sec=0.0;
    stopp_sec=samp_indx+60.;
    stopp_sec=ma[ichan]+60.;           /* until max amplitude+60 seconds    */
    if(stopp_sec > ttotal)
        stopp_sec=ttotal;

    SEARCH(samp_indx,start_sec, stopp_sec,c3,indx,srate,&p_prelim2,TIME_MAX_AMP);
/*--------------------------------------------------------------------------*/
/* PLOT3: STA/LTA RATIO SMOOTHED DERIVATED AFTER FIRST SEARCH               */
/*--------------------------------------------------------------------------*/
    for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
        pv[i]=c3[i];
    SCALETO_1(indx,pv,PIX_HALF);        /* scale plot vector                 */
    for(j=0;j<indx;j++)
        pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
    if(plot==1)
    {    
        color=8;                  /* CYAN                              */
        PlotVec(color,indx,next,pv,xsc);
    }

    SEARCH(samp_indx,start_sec, stopp_sec,c3,indx,srate,&s_prelim2,TIME_MAX_AMP);

    if(s_prelim2 >= p_prelim2)
    {
        p[ichan]=p_prelim2;                 /* save time of p-phase each channel */
        s[ichan]=s_prelim2;                 /* save time of s-phase each channel */
        p_ph=p_prelim2;
        s_ph=s_prelim2;
    }else{
        p[ichan]=s_prelim2;                 /* save time of p-phase each channel */
        s[ichan]=p_prelim2;                 /* save time of s-phase each channel */
        p_ph=s_prelim2;
        s_ph=p_prelim2;
    }
/*--------------------------------------------------------------------------*/
/* find duration of event from p-phase to end of trigger                    */
/*--------------------------------------------------------------------------*/
    for(i=0;i<zeroto;i++)
        data[i]=0;
    coda(pre,p[0],s_ph,srate,indx,data,&dura);
    d[ichan]=dura;                 /* save duration each channel        */
/*--------------------------------------------------------------------------*/
/* PLOT4: PLOT PHASES AND DURATION                                          */
/*--------------------------------------------------------------------------*/
    if(plot==1)
    {
        color=12;                  /* RED                               */
        TimScale(arrow,dsp,color,indx,next,xsc,srate,p_ph,s_ph,dura);
    }
/*--------------------------------------------------------------------------*/
/* plot from p-10 seconds + 20 seconds                                      */
/*--------------------------------------------------------------------------*/
    zoom=3.0;       
    zooml=s_ph+2.0;
    if(zooml > 5.0)
        zooml=5.0;       
    ext1=(int)((p_ph-zoom)/srate);
    ext2=(int)((p_ph-zoom)/srate)+(int)(zooml/srate);
    no_ext=ext2-ext1;       
    xscz=XL/(float)no_ext;            /* scale factor x-axis               */
    j=0;
/*--------------------------------------------------------------------------*/
/* PLOT5: PLOT ZOOMED DATA WITH P-PHASE                                     */
/*--------------------------------------------------------------------------*/
    for(i=ext1;i<ext2;i++)            /* move to plot vector               */
    {
        pv[j]=data[i] * -1.0;       /* turn polarity for plotting        */
        j++;
    }
    cut=p_ph-zoom;
    n_p= p_ph-cut;
    n_s= s_ph-cut;
    SCALETO_1(no_ext,pv,PIX_HALF);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        next=next+(float)PIX_HALF ;
        PlotVec(color,no_ext,next,pv,xscz);
        arrow=next;
        color=12;                   /* RED                               */
        TimScale3(arrow,dsp,color,no_ext,next,xscz,srate,n_p,n_s,dura);
    }
/*--------------------------------------------------------------------------*/
/* if no 3-comp analysis, write phases and coda for map to /home/work/phases*/
/*--------------------------------------------------------------------------*/
    cur_sta[0]=stcode[(chan[ichan]-1)*4];
    cur_sta[1]=stcode[(chan[ichan]-1)*4+1];
    cur_sta[2]=stcode[(chan[ichan]-1)*4+2];
    if(ichan==0)
    {
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
//        fprintf(phs," %10.2f    1 \n",p_ph);

//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
//        fprintf(phs," %10.2f    2 \n",s_ph);
  
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
//        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
//        fprintf(phs," %10.2f    3 \n",dura-p_ph);
    }
    next=next+(float)PIX_FULL;
}       /* end of channel-loop */ 


aavg=(am[0]+am[1]+am[2])/3;            /* average max amplitude             */
if(aavg < 2)
    goto no;
/*--------------------------------------------------------------------------*/
/* PLOT6: PLOT ABSOLUTE SUM OF 3 COMPONENTS                                 */
/*--------------------------------------------------------------------------*/
for(i=0;i<indx;i++)            /* move sum of components to plot vector */
    pv[i]=c1[i];
SCALETO_1(indx,pv,PIX_HALF);        /* scale plot vector                 */
for(j=0;j<indx;j++)
    pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
if(plot==1)
{    
    color=14;                  /* CYAN                              */
    PlotVec(color,indx,next,pv,xsc);
    arrow=next;
}
MAX_AMPL(indx,c1,srate,&TIME_MAX_AMP,&MAX_AMP);
/*--------------------------------------------------------------------------*/
/* find maximum SNR, time of max. SNR and array of running SNR              */
/* to find an initial start value, a noise level is computed from:          */
/* start_sec -> stopp_sec                                                   */
/*--------------------------------------------------------------------------*/
stopp_sec=((float)pre * 50.0)/100.0;
if(stopp_sec < 0.0)
    stopp_sec=2.0;
STA_LTA_PH(srate,indx,c1,ws,wl,rat);
for(i=0;i<(int)(stopp_sec/srate);i++)
    rat[i]=0.0;
/*--------------------------------------------------------------------------*/
/* find maximum amplitude and sample index in signal                        */
/*--------------------------------------------------------------------------*/
MAX_AMPL(indx,rat,srate,&samp_indx,&snr_mxx);
        
nsm=100;
SMOOTH(indx,rat,c2);
for(i=0;i<nsm;i++)
{
    SMOOTH(indx,c2,c3);
    SMOOTH(indx,c3,c2);
}

for(j=0;j<indx;j++)
    c2[j]=(c2[j]*1000.)*(c2[j]*1000);

start_sec=0.0;
stopp_sec=ttotal;
DERIV(start_sec,stopp_sec,srate,c2,indx);
/*--------------------------------------------------------------------------*/
/* find 2 highest peaks (p and s) and time of occurrence                    */
/*--------------------------------------------------------------------------*/
MAXONE(c2,c3,indx);
zeroto=pre/4.0;
for(j=0;j<(int)(zeroto/srate);j++)
    c3[j]=0.0;
/*--------------------------------------------------------------------------*/
/* PLOT7: PLOT DERIVATED, SMOOTHED,STA/LTA RATIO OF SUM OF 3 COMPONENTS     */
/*--------------------------------------------------------------------------*/
for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
    pv[i]=c3[i];
SCALETO_1(indx,pv,PIX_HALF);        /* scale plot vector                 */
for(j=0;j<indx;j++)
    pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
if(plot==1)
{    
    color=11;                  /* CYAN                              */
    next=next+(float)PIX_HALF ;
    PlotVec(color,indx,next,pv,xsc);
}
start_sec=0.0;
stopp_sec=samp_indx+60.;
SEARCH(samp_indx,start_sec, stopp_sec,c3,indx,srate,&p_prelim2,TIME_MAX_AMP);
/*--------------------------------------------------------------------------*/
/* PLOT3: STA/LTA RATIO SMOOTHED DERIVATED AFTER FIRST SEARCH               */
/*--------------------------------------------------------------------------*/
for(i=0;i<indx;i++)            /* move SNR values to plot vector    */
    pv[i]=c3[i];
SCALETO_1(indx,pv,PIX_HALF);        /* scale plot vector                 */
for(j=0;j<indx;j++)
    pv[j]=pv[j] * -1.0;        /* turn polarity for plotting        */
if(plot==1)
{    
    color=8;                  /* CYAN                              */
    PlotVec(color,indx,next,pv,xsc);
}

SEARCH(samp_indx,start_sec, stopp_sec,c3,indx,srate,&s_prelim2,TIME_MAX_AMP);

if(s_prelim2 > p_prelim2)
{
    p[3]=p_prelim2;                 /* save time of p-phase each channel */
    s[3]=s_prelim2;                 /* save time of s-phase each channel */
    p_ph=p_prelim2;
    s_ph=s_prelim2;
}else{
    p[3]=s_prelim2;                 /* save time of p-phase each channel */
    s[3]=p_prelim2;                 /* save time of s-phase each channel */
    p_ph=s_prelim2;
    s_ph=p_prelim2;
}
/*--------------------------------------------------------------------------*/
/* find duration of event from p-phase to end of trigger                    */
/*--------------------------------------------------------------------------*/
s_ph=(s[1]+s[2]+s[3])/3.0;
p_ph=p[0];
coda(pre,p_ph,s_ph,srate,indx,c1,&dura);
d[3]=dura;                 /* save duration each channel        */
/*--------------------------------------------------------------------------*/
/* PLOT4: PLOT PHASES AND DURATION                                          */
/*--------------------------------------------------------------------------*/
if(plot==1)
{
    color=12;                  /* RED                               */
    p_ph=p[3];
    s_ph=s[3];
    TimScale(arrow,dsp,color,indx,next,xsc,srate,p_ph,s_ph,dura);
    getch();
}

for(i=0;i<4;i++)
    printf("%6.2f %6.2f %6.2f\n",p[i],s[i],d[i]);

no_tre_comp=1;
if(no_tre_comp > 0)
{
    for(pkt=0;pkt<no_tre_comp;pkt++)
    {
        each_flt=(int)YL/5;
        dsp=each_flt/2;               /* displacement in pixels between    */
        if(plot==1)                        /* plot ?                            */
            GraphicsMode();                /* set to graphics mode              */
        next=dsp;                        /* zero line diplacement in pixels   */
        for(ichan=0;ichan<3;ichan++)   /* channel to be processed           */
        {
            isamp=ntotal;                  /* counter for number of samples     */
            indx=0;
            for(smp=from;smp<to;smp++)     /* extract channel to process vector */
            {
                valfl=M2(matr,tre_start[ichan+pkt*3]-1,smp);
                data[indx]=valfl;
                indx=indx+1;               /* counting samples                  */
            }
            xsc=XL/(float)indx;            /* scale factor x-axis               */

            switch(ichan)              /* test for channel                  */
            {
                case 0:
                for(smp=0;smp<indx;smp++)
                    c1[smp]=data[smp]; /* save channel 1 for coherence      */
                break;
                case 1:
                for(smp=0;smp<indx;smp++) /* save channel 2 for coherence   */
                    c2[smp]=data[smp];
//if(unit==42)
//    REVERSE_POLARITY(c2,indx);
                break;
                case 2:
                for(smp=0;smp<indx;smp++) /* save channel 3 for coherence   */
                    c3[smp]=data[smp];
//if(unit==42)
//    REVERSE_POLARITY(c3,indx);
                break;
            }
        }
        REMOVE_DC(indx,c1);        /* remove DC                         */
        REMOVE_DC(indx,c2);        /* remove DC                         */
        REMOVE_DC(indx,c3);        /* remove DC                         */
        design(ORDER,"BP","BU",30.0,0.3,FL,FH,srate);
//        design(ORDER,"BP","BU",30.0,0.3,fl1,fh1,srate);
        apply(c1,indx,1);          /* Apply FILTER                      */
        apply(c2,indx,1);          /* Apply FILTER                      */
        apply(c3,indx,1);          /* Apply FILTER                      */

        for(i=0;i<indx;i++)            /* move to plot vector               */
            pv[i]=c1[i] * -1.0;        /* turn polarity for plotting        */
        SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
        if(plot==1)                    /* plot filtered data                */
        {    
            color=14;                  /* YELLOW                            */
            PlotVec(color,indx,next,pv,xsc);
            ichan=0+pkt*3;
            statf[0]=stcode[(tre_start[ichan]-1)*4];
            statf[1]=stcode[(tre_start[ichan]-1)*4+1];
            statf[2]=stcode[(tre_start[ichan]-1)*4+2];
            statf[3]=stcode[(tre_start[ichan]-1)*4]+3;
            statf[4]='\0';
            stco_label(tre_start,ichan,next,stcode,comp,thead);
            arrow=next;
            next=next+dsp*2;
        }
        for(i=0;i<indx;i++)            /* move to plot vector               */
            pv[i]=c2[i] * -1.0;        /* turn polarity for plotting        */
        SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
        if(plot==1)                    /* plot filtered data                */
        {    
            color=14;                  /* YELLOW                            */
            PlotVec(color,indx,next,pv,xsc);
            ichan=1+pkt*3;
            stco_label(tre_start,ichan,next,stcode,comp,thead);
            arrow=next;
            next=next+dsp*2;
        }
        for(i=0;i<indx;i++)            /* move to plot vector               */
            pv[i]=c3[i] * -1.0;        /* turn polarity for plotting        */
        SCALETO_1(indx,pv,dsp);      /* scale to dsp/2                    */
        if(plot==1)                    /* plot filtered data                */
        {    
            color=14;                  /* YELLOW                            */
            PlotVec(color,indx,next,pv,xsc);
            ichan=2+pkt*3;
            stco_label(tre_start,ichan,next,stcode,comp,thead);
            arrow=next;
            next=next+dsp*2;
        }

/*--------------------------------------------------------------------------*/
/* find start and stop index for coherence based on previously found        */
/* p-phases and maximum amplitudes.                                         */
/* start 10 seconds before P, stop 10 seconds after max amplitude           */
/*--------------------------------------------------------------------------*/
    for(smp=0;smp<indx;smp++)
        coher[smp]=0.0;
//uth
p_snitt_sta=s_snitt_sta=d_snitt=0.0;
equalpp=same_phase(p);
printf("EQUALPP %d\n",equalpp);
switch(equalpp)
{
    case 63:
    p_snitt_sta=(p[0]+p[1]+p[2]+p[3])/4.;
    break;
    case 62:
    p_snitt_sta=(p[0]+p[1]+p[2]+p[3])/4.;
    break;
    case 61:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 60:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 59:
    p_snitt_sta=(p[0]+p[1]+p[2]+p[3])/4.;
    break;
    case 58:
    p_snitt_sta=(p[0]+p[1]+p[2]+p[3])/4.;
    break;
    case 57:
    p_snitt_sta=(p[0]+p[1]+p[2]+p[3])/4.;
    break;
    case 56:
    p_snitt_sta=(p[0]+p[1]+p[2]+p[3])/4.;
    break;
    case 55:
    p_snitt_sta=(p[0]+p[1]+p[2]+p[3])/4.;
    break;
    case 54:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 53:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 52:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 51:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 50:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 49:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 48:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 47:
    p_snitt_sta=(p[0]+p[1]+p[3])/3.;
    break;
    case 46:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 45:
    p_snitt_sta=(p[0]+p[1]+p[3])/3.;
    break;
    case 44:
    p_snitt_sta=(p[0]+p[1]+p[3])/3.;
    break;
    case 43:
    p_snitt_sta=(p[0]+p[1]+p[3])/3.;
    break;
    case 42:
    p_snitt_sta=(p[0]+p[1]+p[3])/3.;
    break;
    case 41:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 40:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 39:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 38:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 37:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 36:
    p_snitt_sta=(p[0]+p[1]+p[2])/3.;
    break;
    case 35:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 34:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 33:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 32:
    p_snitt_sta=(p[0]+p[1])/2.;
    break;
    case 31:
    p_snitt_sta=(p[0]+p[2]+p[3])/3.;
    break;
    case 30:
    p_snitt_sta=(p[0]+p[2]+p[3])/3.;
    break;
    case 29:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 28:
    p_snitt_sta=(p[0]+p[2]+p[3])/3.;
    break;
    case 27:
    p_snitt_sta=(p[0]+p[2]+p[3])/3.;
    break;
    case 26:
    p_snitt_sta=(p[0]+p[2]+p[3])/3.;
    break;
    case 25:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 24:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 23:
    p_snitt_sta=(p[0]+p[2]+p[3])/3.;
    break;
    case 22:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 21:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 20:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 19:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 18:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 17:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 16:
    p_snitt_sta=(p[0]+p[2])/2.;
    break;
    case 15:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 14:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 13:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 12:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 11:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 10:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 9:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 8:
    p_snitt_sta=(p[0]+p[3])/2.;
    break;
    case 7:
    p_snitt_sta=(p[1]+p[2]+p[3])/3.;
    break;
    case 6:
    p_snitt_sta=p[1];
    break;
    case 5:
    p_snitt_sta=p[1];
    break;
    case 4:
    p_snitt_sta=p[1];
    break;
    case 3:
    p_snitt_sta=p[1];
    break;
    case 2:
    p_snitt_sta=(p[1]+p[3])/2.;
    break;
    case 1:
    p_snitt_sta=(p[2]+p[3])/2.;
    break;
    case 0:
goto out;
    break;
}
equalss=same_phase(s);
printf("EQUALSS %d\n",equalss);
switch(equalss)
{
    case 63:
    s_snitt_sta=(s[0]+s[1]+s[2]+s[3])/4.;
    break;
    case 62:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 61:
    s_snitt_sta=(s[0]+s[1]+s[2])/3.;
    break;
    case 60:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 59:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 58:
    s_snitt_sta=(s[1]+s[3])/2.;
    break;
    case 57:
    s_snitt_sta=(s[2]+s[3])/2.;
    break;
    case 56:
    s_snitt_sta=(s[0]+s[1]+s[2]+s[3])/4.;
    break;
    case 55:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 54:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 53:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 52:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 51:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 50:
    s_snitt_sta=(s[1]+s[3])/2.;
    break;
    case 49:
    s_snitt_sta=(s[2]+s[3])/2.;
    break;
    case 48:
    s_snitt_sta=(s[0]+s[1]+s[2])/3.;
    break;
    case 47:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 46:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 45:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 44:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 43:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 42:
    s_snitt_sta=(s[1]+s[3])/2.;
    break;
    case 41:
    s_snitt_sta=(s[2]+s[3])/2.;
    break;
    case 40:
    s_snitt_sta=(s[0]+s[1]+s[3])/3.;
    break;
    case 39:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 38:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 37:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 36:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 35:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 34:
    s_snitt_sta=(s[1]+s[3])/2.;
    break;
    case 33:
    s_snitt_sta=(s[2]+s[3])/2.;
    break;
    case 32:
    s_snitt_sta=(s[0]+s[1])/2.;
    break;
    case 31:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 30:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 29:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 28:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 27:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 26:
    s_snitt_sta=(s[1]+s[3])/2.;
    break;
    case 25:
    s_snitt_sta=(s[2]+s[3])/2.;
    break;
    case 24:
    s_snitt_sta=(s[2]+s[3])/3.;
    break;
    case 23:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 22:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 21:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 20:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 19:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 18:
    s_snitt_sta=(s[1]+s[3])/2.;
    break;
    case 17:
    s_snitt_sta=(s[2]+s[3])/2.;
    break;
    case 16:
    s_snitt_sta=(s[0]+s[2])/2.;
    break;
    case 15:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 14:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 13:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 12:
    s_snitt_sta=(s[1]+s[2])/2.;
    break;
    case 11:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 10:
    s_snitt_sta=(s[1]+s[3])/2.;
    break;
    case 9:
    s_snitt_sta=(s[2]+s[3])/2.;
    break;
    case 8:
    s_snitt_sta=(s[0]+s[3])/2.;
    break;
    case 7:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.;
    break;
    case 6:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.0;
    break;
    case 5:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.0;
    break;
    case 4:
    s_snitt_sta=(s[1]+s[2])/2.0;
    break;
    case 3:
    s_snitt_sta=(s[1]+s[2]+s[3])/3.0;
    break;
    case 2:
    s_snitt_sta=(s[1]+s[3])/2.;
    break;
    case 1:
    s_snitt_sta=(s[2]+s[3])/2.;
    break;
    case 0:
goto out;
    break;
}
if(d[0] > s_snitt_sta && d[1] > s_snitt_sta && d[2] > s_snitt_sta && d[3] > s_snitt_sta)
{
    d_snitt=(d[0]+d[1]+d[2]+d[3])/4. - p_snitt_sta;
    goto ok;
}
if(d[0] > s_snitt_sta && d[1] > s_snitt_sta && d[2] > s_snitt_sta)
{
    d_snitt=(d[0]+d[1]+d[2])/3. - p_snitt_sta;
    goto ok;
}
if(d[0] > s_snitt_sta && d[1] > s_snitt_sta)
{
    d_snitt=(d[0]+d[1])/2. - p_snitt_sta;
    goto ok;
}
if(d[1] > s_snitt_sta && d[2] > s_snitt_sta)
{
    d_snitt=(d[1]+d[2])/2. - p_snitt_sta;
    goto ok;
}
if(d[3] > s_snitt_sta)
{
    d_snitt=(d[3]) - p_snitt_sta;
    goto ok;
}
ok:
printf("%6.2f %6.2f %6.2f %6.2f\n",d[0],d[1],d[2],d_snitt);

p[10]=p_snitt_sta;
s[10]=s_snitt_sta;
d[10]=d_snitt;

//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
//    fprintf(phs," %10.2f    1 \n",p_snitt_sta);

//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
//    fprintf(phs," %10.2f    2 \n",s_snitt_sta);

//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
//    fprintf(phs," %10.2f    3 \n",d_snitt-p_snitt_sta);




//fra=(int)((p_snitt_sta-2.0)/srate);
//til=fra+(int)(2.0/srate);

fra=0;
til=(int)(s_snitt_sta/srate);

spick=(int)(s_snitt_sta)/srate;
//printf("P-SNITT_STA=%6.3f S-SNITT_STA=%6.3f FRA %d TIL %d\n",p_snitt_sta,s_snitt_sta,fra,til);
    ww=30;
    n4_max=0.0;
    smp_indx=0;
    azi=0.0;
    vel=0.0;
    rms=0.0;
    lpts=ipts+ww;
    pvel=6500.0;
    psrat=1.73;
    if((txt=fopen("/home/work/coh.lis","w"))==NULL)
    {
        printf("Could not open file containing coh summary\n");
        exit(0);
    }
    if((kff=fopen("/home/work/kof.lis","w"))==NULL)
    {
        printf("Could not open file containing kof summary\n");
        exit(0);
    }

ik=0;    
    for(smp=fra;smp<til;smp++)
    {
        xx0=fabs(c1[smp+1]);
        yy1=fabs(c1[smp+2]);
        xx1=fabs(c1[smp]);
        koff=(xx0 * xx0 + xx1 * xx1)/srate;
//        koff=(xx0 * xx0 * xx0 + xx1 * xx1 * xx1)/srate;
        koff2=(yy1 * yy1 + xx1 * xx1)/srate;
        kof[ik]=koff;
        kof2[ik]=koff2;

        ik++;
       n4=procoh1(c1,c2,c3,smp,smp+ww,pvel,psrat,oprocoh);
//printf("%5d %10.2f %10.3f  %10.3f\n",smp,(float)smp*srate,oprocoh[1],n4);
fprintf(txt,"%5d %10.2f %10.3f %10.3f  %10.3f %8.2f %8.2f\n",smp,(float)smp*srate,oprocoh[1],oprocoh[2],n4,koff,koff2);
//getch();
        if(n4 < 0.0)
            n4=0.0;
        coher[smp]=n4;
        rat[smp]=oprocoh[1];
        if(n4 > n4_max)
        {
            n4_max=n4;
            smp_indx=smp;
            azi=oprocoh[1];
            vel=oprocoh[2];
            rms=oprocoh[3];
        }
    }
fclose(txt);
intil=(int)((p_snitt_sta)/srate);
maxkof=0.0;
maxkof2=0.0;
for(i=0;i<intil;i++)
{
    if(kof[i]>maxkof)
        maxkof=kof[i];
    if(kof2[i]>maxkof2)
        maxkof2=kof2[i];
}
for(i=0;i<intil;i++)
{
    kof[i]=(kof[i]/maxkof)*100.0;
    kof2[i]=(kof2[i]/maxkof2)*100.0;
}
for(i=0;i<intil;i++)
fprintf(kff,"%5d %10.2f %10.3f %10.3f\n",i,(float)i*srate,kof[i],kof2[i]);
intil0=(int)((p_snitt_sta-KWIN)/srate);
for(i=intil0;i<intil;i++)
{
    if(kof[i]>KOFLIM)
        goto lab1;
}
fprintf(kff,"Cant find better P\n");
goto lukk;
lab1:
P_KOF=(float)i*srate;
p[15]=P_KOF;
fprintf(kff,"New P: %8.2f\n",(float)i*srate);
lukk:
fclose(kff);
    for(i=0;i<indx;i++)                /* move to plot vector               */
        pv[i]=coher[i];

    for(j=0;j<indx;j++)
        pv[j]=pv[j]* -1.0;


    SCALETO_1(indx,pv,dsp);            /* scale to dsp                      */
    if(plot==1)
    {    
        color=13;                      /* VIOLET                            */
        PlotVec(color,indx,next,pv,xsc); /* Plot COHERE                     */
        next=next+dsp;
    }


til=(int)((p_snitt_sta)/srate);
fra=(int)((p_snitt_sta-WIN)/srate);
p_coher=p_coher2=0.0;

n4_max=0.0;
ixc=til;
for(smp=fra;smp<til;smp++)
{
    if(coher[ixc] > n4_max)
       n4_max=coher[ixc];
    if(n4_max > 0.4)
        goto hit;
    ixc=ixc-1;
}
//printf("No coherence above %5.2f\n",MIN_COH);
p[11]=-1.0;
p[13]=rat[til];
goto forw;
hit:
t_n4_max=n4_max;
p_coher=(float)(ixc)*srate;
p[11]=p_coher;
p[13]=rat[ixc];
ixc=(int)(p[10]/srate);
p[13]=rat[ixc  ];
forw:
n4_max2=0.0;
ixc1=fra;
for(smp=fra;smp<til;smp++)
{
    if(coher[ixc1] > n4_max2)
       n4_max2=coher[ixc1];
    if(n4_max2 > 0.4)
        goto hit2;
    ixc1=ixc1+1;
}
//printf("No coherence above %5.2f\n",MIN_COH);
p[12]=-1.0;;
goto videre;
hit2:
t_n4_max2=n4_max2;
p_coher2=(float)(ixc1)*srate;

p[12]=p_coher2;
p[14]=rat[ixc1];

videre:

sum=0;
avgsum=0;
for(smp=fra;smp<til;smp++)
    sum=sum+rat[smp];
avgsum=sum/(float)(til-fra);
sum=0;
for(smp=fra;smp<til;smp++)
    sum=sum+(rat[smp]-avgsum)*(rat[smp]-avgsum);
sum=sum/(float)(til-fra-1);
sd=sqrt(sum);
sd=(sd/avgsum)*100.;
t_sd=sd;
    if(plot==1)
    {
        _setlinestyle(DASHED);
        _setcolor(8);
        _moveto((int)(fra)*xsc,YL);
        _lineto((int)(fra)*xsc,0);
        _moveto((int)(til)*xsc,YL);
        _lineto((int)(til)*xsc,0);

        _setcolor(12);
        _moveto((int)(ixc)*xsc,YL);
        _lineto((int)(ixc)*xsc,0);

        _setcolor(12);
        _moveto((int)(ixc1)*xsc,YL);
        _lineto((int)(ixc1)*xsc,0);

        _setcolor(14);
        _moveto((int)(spick)*xsc,YL);
        _lineto((int)(spick)*xsc,0);
        color=12;                      /* set color RED                     */
        next=next+dsp/2;
            TimScale1(arrow,dsp,color,indx,next,xsc,srate,p_ph,s_ph,dura);
        next=next+dsp;  
        getch();
    }



sta=fopen("/home/seislog/pr/station","r");
/*   read velocites*/
     fscanf(sta,"%f %f",&velocityf[1],&velocityf[2]);
//     printf("P-velocity:    %5.1f S-velocity:  %5.1f  \n",velocityf[1],velocityf[2]);     

/*  read fixed depth or starting depth and number of depths to test*/
/*  if number of test depths is 1, only the first start depth is used*/
     fscanf(sta,"%f  %d",&h0,&ndepth);
//     printf("Depth used %5.1f    Number of dephts to test for %d\n",h0,ndepth);

/*  read coda scale parameters*/
     fscanf(sta,"%f %f %f",&c11,&c22,&c33);
//     printf("Coda paramters %6.2f %6.4f %6.2f\n",c11,c22,c33);

//  read min/max long and lat
//
    fscanf(sta,"%f %f %f %f",&slon1,&slon2,&slat1,&slat2);
//    printf("Min/max lon lat:%f %f %f %f\n",slon1,slon2,slat1,slat2);
//

//  read stations until end of file
//
//     fscanf(sta,"%s %f %f",stat[i],&slatt[i],&slonn[i])
fler:
     fscanf(sta,"%s %f %f",stasjon,&flat,&flon);
    cur_sta[3]='\0';
    printf("%s %s %f %f\n",stasjon,cur_sta,flat,flon);
//getch();
    if(stasjon[0]==cur_sta[0] && stasjon[1]==cur_sta[1] && stasjon[2]==cur_sta[2])
    {
//        printf("%s %f %f\n",stasjon,flat,flon);
        fclose(sta);
    }else{
        goto fler;
    }
//printf("I am here\n");

/*--------------------------------------------------------------------------*/
/* find channels with equal p readings                                      */
/*--------------------------------------------------------------------------*/
avstand=(s[10]-p[10])*HST;
if(d[10] > -1.0)
{
    if(avstand < 55.)
    {
        Mb=0.23 + 1.71 * log10(d[10]);
    }else{
        Mb=-1.56 + 2.36 * log10(d[10]);
    }
}else{
    Mb=-1.0;
}

//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
//    fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
//    fprintf(phs," %10.2f    1 \n",p_coher);


slat=flat;
slon=flon;


    if(plot==1)
    {    
        _setlinestyle(DASHED);
        _setcolor(11);                 /* color CYAN                        */
        _moveto((int)(p_snitt_sta/srate)*xsc,YL);
        _lineto((int)(p_snitt_sta/srate)*xsc,0);
        _setcolor(9);                 /* color BLUE                         */
        _moveto((int)(p_coher/srate)*xsc,YL);
        _lineto((int)(p_coher/srate)*xsc,0);
        _setcolor(9);                 /* color BLUE                         */
        _moveto((int)(p_coher2/srate)*xsc,YL);
        _lineto((int)(p_coher2/srate)*xsc,0);
        _setcolor(7);                 /* color GRAY                         */
        _moveto((int)(s_snitt_sta/srate)*xsc,YL);
        _lineto((int)(s_snitt_sta/srate)*xsc,0);
        _setcolor(12);                 /* color RED                         */
        _moveto((int)(fra)*xsc,YL);
        _lineto((int)(fra)*xsc,0);
        _moveto((int)(til)*xsc,YL);
        _lineto((int)(til)*xsc,0);
        color=13;                      /* VIOLET                            */
        next=next+dsp;
    }
    fra=(int)((p_snitt_sta-2.0)/srate);

    til=(int)((p_snitt_sta+2.0)/srate);
    ne=til-fra;
    xsc=XL/(float)(til-fra);            /* scale factor x-axis               */
    each_flt=(int)YL/4;
    dsp=each_flt/2;               /* displacement in pixels between    */
    if(plot==1)                        /* plot ?                            */
        GraphicsMode();                /* set to graphics mode              */
    next=dsp;                        /* zero line diplacement in pixels   */
    j=0;
    for(i=fra;i<til;i++)            /* move to plot vector               */
    {
        pv[j]=c1[i] * -1.0;        /* turn polarity for plotting        */
        j++;
    }
    SCALETO_1(ne,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,ne,next,pv,xsc);
        ichan=0+pkt*3;
        statf[0]=stcode[(tre_start[ichan]-1)*4];
        statf[1]=stcode[(tre_start[ichan]-1)*4+1];
        statf[2]=stcode[(tre_start[ichan]-1)*4+2];
        statf[3]=stcode[(tre_start[ichan]-1)*4]+3;
        statf[4]='\0';
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    j=0;
    for(i=fra;i<til;i++)            /* move to plot vector               */
    {
        pv[j]=c2[i] * -1.0;        /* turn polarity for plotting        */
        j++;
    }
    SCALETO_1(ne,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,ne,next,pv,xsc);
        ichan=1+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2;
    }
    j=0;
    for(i=fra;i<til;i++)            /* move to plot vector               */
    {
        pv[j]=c3[i] * -1.0;        /* turn polarity for plotting        */
        j++;
    }
    SCALETO_1(ne,pv,dsp);      /* scale to dsp/2                    */
    if(plot==1)                    /* plot filtered data                */
    {    
        color=14;                  /* YELLOW                            */
        PlotVec(color,ne,next,pv,xsc);
        ichan=2+pkt*3;
        stco_label(tre_start,ichan,next,stcode,comp,thead);
        arrow=next;
        next=next+dsp*2+dsp;
    }
    j=0;
    for(i=fra;i<til;i++)                /* move to plot vector               */
    {
        pv[j]=coher[i];
        j++;
    }
    for(j=0;j<ne;j++)
        pv[j]=pv[j]* -1.0;


    SCALETO_1(ne,pv,dsp*2-18);            /* scale to dsp                      */
    if(plot==1)
    {    
        color=11;                      /* CYAN                              */
        PlotVec(color,ne,next-13,pv,xsc); /* Plot COHERE                     */
    }
    j=0;
    for(i=fra;i<til;i++)                /* move to plot vector               */
    {
        pv[j]=rat[i];
        j++;
    }
    for(j=0;j<ne;j++)
        pv[j]=pv[j]* -1.0;


    SCALETO_1(ne,pv,dsp*2-18);            /* scale to dsp                      */
    if(plot==1)
    {    
        color=13;                      /* CYAN                              */
        PlotVec(color,ne,next-13,pv,xsc); /* Plot COHERE                     */
        next=next+dsp*2;
    }
    if(plot==1)
    {    
        color=15;
        TimScale8(dsp,color,ne,105.0,xsc,srate);
        TimScale8(dsp,color,ne,225.0,xsc,srate);
        TimScale8(dsp,color,ne,345.0,xsc,srate);
step_win:
     teller++;
     if(sw0==0)
     {
        _setlinestyle(DASHED);
        _setcolor(15);                 /* draw P_SNITT_STA dashed in WHITE  */
        _moveto((int)((p_snitt_sta/srate-(float)fra)*xsc),YL-15);
        _lineto((int)((p_snitt_sta/srate-(float)fra)*xsc),0);
     }else{
        _setlinestyle(SOLID);
        _setcolor(14);                 /* draw new cursor position in YELLOW*/
        _moveto((int)((p_snitt_sta/srate-(float)fra)*xsc),330);
        _lineto((int)((p_snitt_sta/srate-(float)fra)*xsc),340);

     }

/*--------------------------------------------------------------------------*/
/* redraw square indicator for coherence and label coherence                */
/*--------------------------------------------------------------------------*/
//        pp=500;  
//        _setlinestyle(SOLID);
//        _setcolor(11);
//        _rectangle(_GFILLINTERIOR,pp,397,pp+10,407);
//        _setcolor(15);
//        _rectangle(_GBORDER,pp,397,pp+10,407);
//        _setcolor(0);
//        pp=515; /* TU6 */
//        _rectangle(_GFILLINTERIOR,pp,400,pp+100,410);
//        sprintf(thead,"Coherence\0");
//        _setcolor(11);
//        _moveto(pp+3,400);
//        _outgtext(thead);

/*--------------------------------------------------------------------------*/
/* redraw square indicator for azimuth and label azimuth                    */
/*--------------------------------------------------------------------------*/

//        pp=500; /* TU2 */
//        _setlinestyle(SOLID);
//        _setcolor(13);
//        _rectangle(_GFILLINTERIOR,pp,417,pp+10,427);
//        _setcolor(15);
//        _rectangle(_GBORDER,pp,417,pp+10,427);
//        _setcolor(0);
//        pp=515; /* TU5 */
//        _rectangle(_GFILLINTERIOR,pp,420,pp+100,430);
//        sprintf(thead,"Azimuth\0");
//        _setcolor(13);
//        _moveto(pp+3,420);
//        _outgtext(thead);
/*--------------------------------------------------------------------------*/
/* redraw label P coefficient                                               */
/*--------------------------------------------------------------------------*/

        pp=500; /* TU10 */
        _setcolor(7);
        _rectangle(_GFILLINTERIOR,pp,423,pp+135,437);
        sprintf(thead,"P coef   :%6.2f\0",p[15]);
        _setcolor(11);
        _moveto(pp+3,426);
        _outgtext(thead);
/*--------------------------------------------------------------------------*/
/* redraw label Azi Avg and avgsum + label Azi VC and sd                    */
/*--------------------------------------------------------------------------*/

        pp=500; /* TU3 */
        _setcolor(15);
        _rectangle(_GFILLINTERIOR,pp,437,pp+135,459);
        sprintf(thead,"Azi avg  :%6.2f\0",avgsum);
        _setcolor(13);
        _moveto(pp+3,439);
        _outgtext(thead);
        sprintf(thead,"Azi VC   :%6.2f\0",sd);
        _setcolor(13);
        _moveto(pp+3,450);
        _outgtext(thead);

/*--------------------------------------------------------------------------*/
/* redraw label coherence and current coherence                             */
/*--------------------------------------------------------------------------*/
        pp=500; /* TU4 */
        _setcolor(1);
        _rectangle(_GFILLINTERIOR,pp,360,pp+135,375);
        sprintf(thead,"Coherence: %5.2f\0",n4_max);
        _setcolor(11);
        _moveto(pp+3,363);
        _outgtext(thead);
/*--------------------------------------------------------------------------*/
/* redraw label distance and current distance                               */
/*--------------------------------------------------------------------------*/
        pp=500; /* TU4 */
        _setcolor(12);
        _rectangle(_GFILLINTERIOR,pp,375,pp+135,392);
        sprintf(thead,"Distance :%6.2f\0",(s[10]-p[10])*HST);
        _setcolor(11);
        _moveto(pp+3,379);
        _outgtext(thead);
/*--------------------------------------------------------------------------*/
/* redraw label azimuth and current azimuth                                 */
/*--------------------------------------------------------------------------*/
        pp=500; /* TU4 */
        _setcolor(13);
        _rectangle(_GFILLINTERIOR,pp,392,pp+135,407);
        sprintf(thead,"Azimuth  :%6.2f\0",p[13]);
        _setcolor(11);
        _moveto(pp+3,396);
        _outgtext(thead);
/*--------------------------------------------------------------------------*/
/* redraw label p-avg-sta and current p-avg-sts                             */
/*--------------------------------------------------------------------------*/
        pp=500; /* TU4 */
        _setcolor(8);
        _rectangle(_GFILLINTERIOR,pp,407,pp+135,422);
        sprintf(thead,"P-avg-sta:%6.2f\0",p[10]);
        _setcolor(11);
        _moveto(pp+3,412);
        _outgtext(thead);

/*--------------------------------------------------------------------------*/
/* redraw bottom line P-SNITT-STA with WHITE rectangle around it            */
/*--------------------------------------------------------------------------*/

        pp=460;
        _setlinestyle(SOLID);
        _setcolor(0);
        _rectangle(_GFILLINTERIOR,pp,460,pp+150,477);
        _setcolor(15);
        _rectangle(_GBORDER,pp,460,pp+150,475);
        sprintf(thead,"P-AVG-STA %6.2f\0",p_snitt_sta);
        label1(p_snitt_sta,pp+2,463,thead);

        pp=230;
        _setlinestyle(SOLID);
        _setcolor(0);
        _rectangle(_GFILLINTERIOR,pp,460,pp+230,475);
        _setcolor(15);
        _rectangle(_GBORDER,pp,460,pp+230,475);
/*--------------------------------------------------------------------------*/
/* if first time: draw dashed line indicating found P-COHER                 */
/*--------------------------------------------------------------------------*/
    if(sw0==0)
    {
        _setcolor(9);                 /* color BLUE                         */
        _setlinestyle(DASHED);
        _moveto((int)((p_coher/srate-(float)fra)*xsc),YL-20);
        _lineto((int)((p_coher/srate-(float)fra)*xsc),0);
    }
/*--------------------------------------------------------------------------*/
/* if first time: draw dashed line indicating new P-KOF                     */
/*--------------------------------------------------------------------------*/
    if(sw0==0)
    {
        _setcolor(9);                 /* color CYAN                        */
        _setlinestyle(NEW_DASHED);
        _moveto((int)((P_KOF/srate-(float)fra)*xsc),YL-20);
        _lineto((int)((P_KOF/srate-(float)fra)*xsc),0);
    }
/*--------------------------------------------------------------------------*/
/* redraw current P-COH1 and AZIMUTH                                        */
/*--------------------------------------------------------------------------*/
        sprintf(thead,"P-COH1 %6.2f Azim_1 %5.1f\0",p_coher,p[13]);
        label1(p_coher,pp+2,463,thead);

/*--------------------------------------------------------------------------*/
/* if first time: draw DASHED line indicating P-COHER2 in BLUE              */
/*--------------------------------------------------------------------------*/
    if(sw0==0)
    {
        _setlinestyle(DASHED);
        _setcolor(2);                 /* color BLUE                         */
        _moveto((int)((p_coher2/srate-(float)fra)*xsc),YL-20);
        _lineto((int)((p_coher2/srate-(float)fra)*xsc),0);
    }else{
//        _setcolor(14);                 /* color BLUE                         */
//        _moveto((int)((p_coher2/srate-(float)fra)*xsc),330);
//        _lineto((int)((p_coher2/srate-(float)fra)*xsc),340);
    }
/*--------------------------------------------------------------------------*/
/* redraw bottom square P-COH2 and azimuth                                  */
/*--------------------------------------------------------------------------*/
        pp=0;
        _setlinestyle(SOLID);
        _setcolor(0);
        _rectangle(_GFILLINTERIOR,pp,460,pp+230,475);
        _setcolor(15);
        _rectangle(_GBORDER,pp,460,pp+230,475);
        sprintf(thead,"P-COH2 %6.2f Azim_2 %5.1f\0",p_coher2,p[14]);
        _setcolor(2);  /* GREEN */
        label2(p_coher2,pp+2,463,thead);
        _setlinestyle(SOLID);
        color=13;                      /* VIOLET                            */
        next=next+dsp;

       x11=sin(p[13]*0.01745)*(s[10]-p[10])*HST;
       y11=cos(p[13]*0.01745)*(s[10]-p[10])*HST;
       lat=slat+y11/111.2;
       lon=slon+x11/(111.2*cos((slat)/57.3));
avstand=(s[10]-p[10])*HST;
if(d[10] > -1.0)
{
    if(avstand < 55.)
    {
        Mb=0.23 + 1.71 * log10(d[10]);
    }else{
        Mb=-1.56 + 2.36 * log10(d[10]);
    }
}else{
    Mb=-1.0;
}
//printf("%10.3f %10.3f ( %s %10.3f %10.3f) %10.2f %6.2f\n",lat,lon,stasjon,slat,slon,(s[10]-p[10])*HST,Mb);

smap(p,s,lat,lon,slat,slon,Mb,nstation,statreg,slatt1,slonn1);

        bok1=getch();


//printf("BOK1 %x\n",bok1);
        if(bok1==0x71 || bok2==0x71)
            goto out;
        bok2=getch();
        if(bok1==0xff && bok2==0xa6) /* ARROW LEFT OR RIGHT */
        {
            _setlinestyle(SOLID);
            _setcolor(0);              /* draw old cursor position in BLACK */
            _moveto((int)((p_snitt_sta/srate-(float)fra)*xsc),330);
            _lineto((int)((p_snitt_sta/srate-(float)fra)*xsc),340);

            sw0=1;
            p_snitt_sta=p_snitt_sta+srate;
            p[10]=p_snitt_sta;
        }else{
            _setlinestyle(SOLID);
            _setcolor(0);                 /* color WHITE                        */
            _moveto((int)((p_snitt_sta/srate-(float)fra)*xsc),330);
            _lineto((int)((p_snitt_sta/srate-(float)fra)*xsc),340);

            sw0=1;
            p_snitt_sta=p_snitt_sta-srate;
            p[10]=p_snitt_sta;
        }

til1=(int)((p_snitt_sta)/srate);
fra1=(int)((p_snitt_sta-WIN)/srate);

//til1=(int)((p_snitt_sta+WIN)/srate);
//fra1=(int)((p_snitt_sta)/srate);
n4_max=0.0;
ixc=til1;
p_coher=(float)(ixc)*srate;
p[13]=rat[ixc];

n4_max=coher[ixc];
t_n4_max=n4_max;
p[11]=p_coher;
p[13]=rat[ixc];



sum=0;
avgsum=0;
for(smp=fra1;smp<til1;smp++)
    sum=sum+rat[smp];
avgsum=sum/(float)(til1-fra1);
sum=0;
for(smp=fra1;smp<til1;smp++)
    sum=sum+(rat[smp]-avgsum)*(rat[smp]-avgsum);
sum=sum/(float)(til1-fra1-1);
sd=sqrt(sum);
sd=(sd/avgsum)*100.;
t_sd=sd;


goto step_win;

    }


/*--------------------------------------------------------------------------*/
/* write results to screen and /home/work/newmail.txt                       */
/*--------------------------------------------------------------------------*/
out:
    for(i=0;i<4;i++)
        printf("%2d %6.2f %6.2f %6.2f %6.2f %8d\n",i,p[i],s[i],ma[i],d[i],am[i]);
    for(i=10;i<16;i++)
        printf("%2d %6.2f %6.2f %6.2f %6.2f\n",i,p[i],s[i],ma[i],d[i]);
    printf("Equalpp %d Equalss %d\n",equalpp,equalss);


//p[13]=p[13]-20.0;

    output(chan,stcode,am,ma,snr_max,snr_tim,d,p,s,slat,slon,Mb,t_sd,t_n4_max,t_n4_max2);


ichan=0;

        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
        fprintf(phs," %10.2f    1 \n",p[15]);

        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
        fprintf(phs," %10.2f    2 \n",s[10]);
  
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+1]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+2]);
        fprintf(phs,"%c",stcode[ (chan[ichan]-1)*4+3]);
        fprintf(phs," %10.2f    3 \n",d[10]);
fclose(phs);




//loc=fopen("/home/work/location","a+"); /* open file for location            */
if((loc=fopen("/home/work/location","a"))==NULL)
{
    printf("Could not open file containing location summary\n");
    exit(0);
}
Mb=1.;
dst=(s[10]-p[15])*10.0;
for(i=0;i<360;i=i+1)
{
azi=(float)i;
x11=sin(azi*0.01745)* dst;
y11=cos(azi*0.01745)* dst;
deg_lat=y11/111.2;
deg_lon=x11/(111.2*cos(slat/57.3));
lat=slat+deg_lat;
lon=slon+deg_lon;

//printf("DST %6.2f X11 %6.2f Y11 %6.2f AZI %6.2f S-P %6.2f\n",dst,x11,y11,azi,s_p);
//printf("Y11 %6.2f -> %8.4f deg  X11 %6.2f -> %8.4f deg\n",y11,deg_lat,x11,deg_lon);

//printf("SLAT %6.2f SLON %6.2f\n",slat,slon);
//printf("%10.3f %10.3f (station  %10.3f %10.3f) %10.2f %6.2f\n",lat,lon,slat,slon,dst,Mb);

//system("rm -f /home/work/coord");
//coh=fopen("/home/work/coord","w");
//fprintf(coh,"%s %10.3f %10.3f %10.3f %10.3f %10.2f %6.2f\n",stasjon,slat,slon,lat,lon,dst,Mb);
fprintf(loc,"%s %10.3f %10.3f %10.3f %10.3f %10.2f 1.0\n",stasjon,slat,slon,lat,lon,dst);
//fclose(coh);
}
fclose(loc);



// HASTA
no:
i=0;
//ik=getchar();
//if(ik== 0x6e)
//    goto label1;
//getch();
//exit(0);
}
}
}
SCALETO_1(lplot,co,fac)
int lplot;
float *co;
int fac;
{
    int i;
    float maxa;
    maxa=0;
    for(i=0;i<lplot;i++)               /* find max value                */
    {
        if(fabs(co[i]) > maxa)
            maxa=fabs(co[i]);
    }
    for(i=0;i<lplot;i++)
        co[i]=co[i]/maxa* (float)fac;
}
REVERSE_POLARITY(in,ne)
float *in;
int   ne;
{
int   i;
for(i=0;i<ne;i++)
    in[i]=in[i] * -1.0;
}
REMOVE_DC(lplot,c)
int lplot;
float *c;
{
    float sum;
    float avg;
    int i;
    sum=0.0;

    for(i=0;i<lplot;i++)               /* find DC                       */
        sum=sum+c[i];
    avg=sum/(float)lplot;
    for(i=0;i<lplot;i++)                /* remove dc                    */
        c[i]=c[i]-avg;
}
/*-------------------------------------------------------------------------*/
/* Get current DPX number                                                  */
/*-------------------------------------------------------------------------*/
Get_Current_DPX()
{
    FILE    *det;                        /* pointer current detection no.  */
    int     curdpx;                      /* current dpx number             */
    det=fopen(DETNUMBER,"r");
    fscanf(det,"%d",&curdpx);
    fclose(det);
    return(curdpx);
}

hdr_info(head,year,month,day,hour,min,sec,nchan,srate,ntotal,unit,nsamp,recsize,reclength,pre)
char*    head;
long    *year;
long     *month;
long     *day;
long     *hour;
long     *min;
float    *sec;
int      *nchan;
float    *srate;
int      *ntotal;
int      *unit;
int      *nsamp;
int      *recsize;
int      *reclength;
int      *pre;
{
char     dum[100];
int      i;
//for(i=0;i<20;i++)
//    printf("%c ",head[i]);
//printf("\n");
sscanf(head,"%14c%4d%5c%2d%c%2d%c%2d%c%2d%c%f6.3\n",
dum,year,dum,month,dum,day,dum,hour,dum,min,dum,sec);
if(head[62]==' ')
{
    dum[0]=head[63];
    sscanf(dum,"%1d",nchan);
}else{
    dum[0]=head[62];
    dum[1]=head[63];
    sscanf(dum,"%2d",nchan);
}
for(i=0;i<16;i++)
    dum[i]=head[i+64];
sscanf(dum,"%f16.7",srate);
for(i=0;i<8;i++)
    dum[i]=head[i+80];
sscanf(dum,"%8d",ntotal);
dum[0]=head[96];
dum[1]=head[97];
sscanf(dum,"%2d",unit);
for(i=0;i<6;i++)
    dum[i]=head[i+122];
dum[i]='\0';
sscanf(dum,"%d",nsamp);
for(i=0;i<8;i++)
    dum[i]=head[i+104];
sscanf(dum,"%8d",recsize);
sscanf(dum,"%8d",reclength);
dum[0]=head[880];
dum[1]=head[881];
dum[2]=head[882];
dum[3]=head[883];
sscanf(dum,"%4d",pre);
/*printf("NCHAN:%2d SRATE:%16.7f NTOTAL:%8d UNIT:%2d NSAMP:%6d\n",\*/
/*nchan,srate,ntotal,unit,nsamp);*/
/*printf("RECSIZE:%8d RECLENGTH:%8d\n",recsize,reclength);*/
}

dmx_buf(dimx,isamp,nsamp,nchan,ntotal,iworkl)
int dimx;
int isamp;
int nsamp;
int nchan;
int ntotal;
int iworkl[];
{
int i;
int j;
int s;
int bufsiz;
bufsiz=nsamp*nchan+HEADER;
for(i=0;i<nchan;i++)                   /* loop channels                     */
{
    for(j=0;j<nsamp;j++)               /* loop samples                      */
    {
        s=j+isamp*nsamp;
        M2(matr,i,s)=(float)iworkl[j*nchan+i+HEADER];
    }
}
}    

/*********************subrutine cross***************************************/

/* this subrotine perform the cross correlation between two set of data(x,y) */


float   cross(x,y,l,ipts)

  float x[],y[];
  int l,ipts;

{

  float a=0.0;
  int i;
  
  for (i=ipts;i<=l+ipts;i++) a=a+x[i]*y[i];
      a=a+x[i]*y[i];
  return a;

}
float   procoh1(channel,channel2,channel3,ipts,lpts,pvel,psrat,oprocoh)

float channel[],channel2[],channel3[];
float pvel,psrat,oprocoh[10];
int ipts,lpts;


/*
  S/R to do 3-component azimuth and velocity estimation for a P-phase
  for a fixed user specified time window.
  Method: predicted coherence (R. Roberts, Uppsala)
*/

{

 
  float raddeg=57.2958;
  int lenwin=0,ii;
  float xx,xy,xz,yy,yx,yz,zz,zx,zy;
  float power,azi,zoverr,a,b,err,cc,ai;
  float svel;
  float coher,azim,velo,rmsamp;


  lenwin=lpts-ipts+1;
  xx=cross(channel2,channel2,lenwin,ipts);
  xy=cross(channel2,channel3,lenwin,ipts); 
  xz=cross(channel2,channel,lenwin,ipts);
  yy=cross(channel3,channel3,lenwin,ipts);
  yz=cross(channel3,channel,lenwin,ipts);
  zz=cross(channel,channel,lenwin,ipts);
  yx=xy;
  zx=xz;
  zy=yz;
  power=(xx+yy+zz)/(float)lenwin;
//printf("(%f + %f + %f)/ %d = %f\n",xx,yy,zz,lenwin,power);
//printf("power=%f\n",power);
  rmsamp=sqrt(power);
  azi=atan2(-yz,-xz);
  azim=azi*raddeg;
  if (azim<0) azim=azim+360.0;
//printf("xz:%f yz:%f\n",xz,yz);
  zoverr=zz/sqrt(xz*xz+yz*yz);
  a=-zoverr*cos(azi);
  b=-zoverr*sin(azi);
   
  err=0.0; 
  for (ii=ipts;ii<=lpts;ii++){
      cc=channel[ii]-a*channel2[ii]-b*channel3[ii];
      err=err+cc*cc;
  };
  coher=1.0-err/zz;
  svel=pvel/psrat;
  ai=atan(1.0/zoverr);
  velo=svel/sin(ai/2.0);

  oprocoh[0]=coher;
  oprocoh[1]=azim;
  oprocoh[2]=velo;
  oprocoh[3]=rmsamp;

  return(coher);
} 

GraphicsMode()
{
    _setvideomode(_VRES16COLOR);
    _setcolor(0);
    _rectangle(_GFILLINTERIOR,0,0,(int)XL,(int)YL);
    _setcolor(13);
    _rectangle(_GBORDER,0,0,(int)XL,(int)YL);
    _setcolor(1);
    _moveto(0,(int)YL);
    _setcolor(14);
}
PlotVec(color,no_of_samples,displacement,array,xscale)
int color;
int no_of_samples;
float displacement;
float* array;
float xscale;
{       
int oldcolor;
int j;
int x,y;
float xx;

oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)(array[j]+displacement);
    _lineto(x,y);
}
_setcolor(oldcolor);
}
TimScale(ar,ds,color,no_of_samples,displacement,xscale,sr,p,s,d)
int ar;
int ds;
int color;
int no_of_samples;
float displacement;
float xscale;
float sr;
float p;
float s;
float d;
{
char karak[10];
int oldcolor;
int j;
int x,y;
int spsec;
int cnt;
int pphase;
int sphase;
int duration;
spsec=(int)(1./sr) * 10;
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
pphase=(int)(p/sr)*xscale;
sphase=(int)(s/sr)*xscale;
duration=(int)(d/sr)*xscale;

y=ar;
_setcolor(12);                         /* RED                               */
_moveto(pphase,y);
_lineto(pphase-5,y-5);
_moveto(pphase,y);
_lineto(pphase+5,y-5);
_moveto(pphase,y);
_lineto(pphase,y-ds/2);
karak[0]='P';
karak[1]='\0';
_moveto(pphase-10,y-ds/2-10);
_outgtext(karak);

_setcolor(13);                         /* VIOLETT                           */
_moveto(sphase,y);
_lineto(sphase-5,y-5);
_moveto(sphase,y);
_lineto(sphase+5,y-5);
_moveto(sphase,y);
_lineto(sphase,y-ds/2);
karak[0]='S';
karak[1]='\0';
_moveto(sphase-10,y-ds/2-10);
_outgtext(karak);

_setcolor(10);                         /* GREEN                             */
_moveto(duration,y);
_lineto(duration-5,y-5);
_moveto(duration,y);
_lineto(duration+5,y-5);
_moveto(duration,y);
_lineto(duration,y-ds/2);
karak[0]='D';
karak[1]='\0';
_moveto(duration-10,y-ds/2-10);
_outgtext(karak);

//_lineto(pphase,y-ds);
//_moveto(sphase,y);
//_lineto(sphase,y-ds);
//_moveto(duration,y);
//_lineto(duration,y-ds);

_setcolor(oldcolor);
}
TimScale3(ar,ds,color,no_of_samples,displacement,xscale,sr,p,s,d)
int ar;
int ds;
int color;
int no_of_samples;
float displacement;
float xscale;
float sr;
float p;
float s;
float d;
{
char karak[10];
int oldcolor;
int j;
int x,y;
int spsec;
int cnt;
int pphase;
int sphase;
int duration;
spsec=(int)(1./sr);
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
cnt=0;
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)displacement+5;
    _lineto(x,y);
    cnt++;
    if(cnt==spsec)
    {
        cnt=0;
        _lineto(x,y-5);
        _moveto(x,y);
    }
}
pphase=(int)(p/sr)*xscale;
sphase=(int)(s/sr)*xscale;
duration=(int)(d/sr)*xscale;

y=ar;
_setcolor(12);                         /* RED                               */
_moveto(pphase,y);
_lineto(pphase-5,y-5);
_moveto(pphase,y);
_lineto(pphase+5,y-5);
_moveto(pphase,y);
_lineto(pphase,y-ds/2);
karak[0]='P';
karak[1]='\0';
_moveto(pphase-10,y-ds/2-10);
_outgtext(karak);

_setcolor(13);                         /* VIOLETT                           */
_moveto(sphase,y);
_lineto(sphase-5,y-5);
_moveto(sphase,y);
_lineto(sphase+5,y-5);
_moveto(sphase,y);
_lineto(sphase,y-ds/2);
karak[0]='S';
karak[1]='\0';
_moveto(sphase-10,y-ds/2-10);
_outgtext(karak);

_setcolor(oldcolor);
}
skala1(xscale)
float xscale;
{
printf("XSC: %f\n",xscale);
}
TimScale1(ar,ds,color,no_of_samples,displacement,xscale,sr,p,s,d)
int ar;
int ds;
int color;
int no_of_samples;
float displacement;
float xscale;
float sr;
float p;
float s;
float d;
{
int oldcolor;
int j;
int x,y;
int spsec;
int cnt;
int pphase;
int sphase;
int duration;
spsec=(int)(1./sr) * 10;
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
cnt=0;
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)displacement+10;
    _lineto(x,y);
    cnt++;
    if(cnt==spsec)
    {
        cnt=0;
        _lineto(x,y-10);
        _moveto(x,y);
    }
}
_setcolor(oldcolor);
}

/*--------------------------------------------------------------------------*/
/* P[0] : p comp Z     S[0] : s comp Z    D[0] : duration comp Z            */
/* P[1] : p comp N     S[1] : s comp N    D[1] : duration comp N            */
/* P[2] : p comp E     S[2] : s comp E    D[2] : duration comp E            */
/* P[3] : p sum Z,N,E  S[3] : s sumZ,N,E  D[3] : duration sum Z,N,E         */
/* P[10]: p_snitt_sta  S[10]: s_snitt_sta D[10]: duration snitt             */
/* P[11]: p from coherence, backward search                                 */
/* P[12]: p from coherence, forward search                                  */
/* P[13]: azimuth from backward search                                      */
/* P[14]: azimuth from forward search                                       */

void output(chan,stcode,am,ma,snr_max,snr_tim,d,p,s,slat,slon,Mb,sd,n4_1,n4_2)
int    *chan;
char*   stcode;
int    am[];
float  ma[];
float  snr_max[];
float  snr_tim[];
float  d[];
float  p[];
float  s[];
float  slat;
float  slon;
float  Mb;
float  sd;
float  n4_1;
float  n4_2;
{
float varig;
float x11,y11,lat,lon;
int   amp_avg;
int kanal;
int cnt;
char stasjon[10];
stasjon[0]=stcode[0];
stasjon[1]=stcode[1];
stasjon[2]=stcode[2];
stasjon[3]='\0';
net=fopen("/home/work/netw","w");
mail=fopen("/home/work/newmail.txt","w"); /* open file for results selected ch */




/*--------------------------------------------------------------------------*/
/* print heading to screen                                                  */
/*--------------------------------------------------------------------------*/
      printf("COMP CH      AMP   T-AMP MAX-SNR   T-SNR     DUR P-Phase S-Phase    S-P  DIS\n");
fprintf(mail,"COMP CH      AMP   T-AMP MAX-SNR   T-SNR     DUR P-Phase S-Phase    S-P  DIS\n");
for(kanal=0;kanal<3;kanal++)
{
printf("%c",stcode[ (chan[kanal]-1)*4]);
printf("%c",stcode[ (chan[kanal]-1)*4+1]);
printf("%c",stcode[ (chan[kanal]-1)*4+2]);
printf("%c",stcode[ (chan[kanal]-1)*4+3]);

printf(" %2d %8d %7.1f %7.1f %7.2f %7.2f %7.2f %7.2f %6.2f %4d\n",
chan[kanal],
am[kanal],ma[kanal],
snr_max[kanal],
snr_tim[kanal],
d[kanal]-p[kanal],
p[kanal],
s[kanal],
s[kanal]-p[kanal],
(int)((s[kanal]-p[kanal])*HST));

fprintf(net,"%c",stcode[ (chan[kanal]-1)*4]);
fprintf(net,"%c",stcode[ (chan[kanal]-1)*4+1]);
fprintf(net,"%c",stcode[ (chan[kanal]-1)*4+2]);
fprintf(net,"%c",stcode[ (chan[kanal]-1)*4+3]);

fprintf(net," %2d %8d %6.2f %7.1f %7.2f %7.2f %7.2f %7.2f %6.2f %4d\n",
chan[kanal],
am[kanal],ma[kanal],
snr_max[kanal],
snr_tim[kanal],
d[kanal]-p[kanal],
p[kanal],
s[kanal],
s[kanal]-p[kanal],
(int)((s[kanal]-p[kanal])*HST));


fprintf(mail,"%c",stcode[ (chan[kanal]-1)*4]);
fprintf(mail,"%c",stcode[ (chan[kanal]-1)*4+1]);
fprintf(mail,"%c",stcode[ (chan[kanal]-1)*4+2]);
fprintf(mail,"%c",stcode[ (chan[kanal]-1)*4+3]);
fprintf(mail," %2d %8d %7.1f %7.1f %7.2f %7.2f %7.2f %7.2f %6.2f %4d\n",
chan[kanal],
am[kanal],ma[kanal],
snr_max[kanal],
snr_tim[kanal],
d[kanal]-p[kanal],
p[kanal],
s[kanal],
s[kanal]-p[kanal],
(int)((s[kanal]-p[kanal])*HST));
}
printf("\n");
fprintf(mail,"\n");

if(p[10]!=-1.0 && s[10]!=-1.0 && d[10]!=-1.0) /* all P,S and D defined */
{
    if(n4_1 > MIN_COH)                        /* coherence good     */
    {
        if(sd < VAC)                          /* azimuth stable     */
        {
            goto number1;
        }else{
            printf("COH > MIN_COH (%6.3f) but SD > VAC (%6.3f)\n",n4_1,sd);
            goto no_solution;
        }
    }else{
        if(sd < VAC)
        {
            goto number2;
        }else{
            printf("COH < MIN_COH (%6.3f) and SD > VAC (%6.3f)\n",n4_1,sd);
            goto no_solution;
        }
    }
}
printf("P,S,D not defined\n");
no_solution:
printf("No solution\n");
goto hoved;
number1:
amp_avg=(am[0]+am[1]+am[2])/3;
printf("Qual_1:\n");
fprintf(mail,"Qual_1:\n");
printf("P-ph: %6.2f S-ph: %6.2f Coda: %6.2f Azi: %6.3f VC: %6.3f Dst:%6.1f\n",p[10],s[10],d[10],p[13],sd,(s[10]-p[15])*HST);
fprintf(mail,"P-ph: %6.2f S-ph: %6.2f Coda: %6.2f Azi: %6.3f VC: %6.3f Dst:%6.1f\n",p[10],s[10],d[10],p[13],sd,(s[10]-p[15])*HST);


//pluss 180
//p[13]=p[13]+180.;


x11=sin(p[13]*0.01745)*(s[10]-p[15])*HST;
y11=cos(p[13]*0.01745)*(s[10]-p[15])*HST;

lat=slat+y11/111.2;
lon=slon+x11/(111.2*cos(slat/57.3));
printf("%10.3f %10.3f ( %s %10.3f %10.3f) %10.2f %6.2f\n",lat,lon,stasjon,slat,slon,(s[10]-p[15])*HST,Mb);

fprintf(mail,"%10.3f %10.3f ( %s %10.3f %10.3f) %10.2f %6.2f\n",lat,lon,stasjon,slat,slon,(s[10]-p[15])*HST,Mb);


system("rm -f /home/work/coord");
coh=fopen("/home/work/coord","w");
fprintf(coh,"%s %8.3f %8.3f %8.3f %8.3f %8.2f %6.2f %8d Q1\n",stasjon,slat,slon,lat,lon,(s[10]-p[15])*HST,Mb,amp_avg);
loc=fopen("/home/work/location","a+"); /* open file for location            */
if((loc=fopen("/home/work/location","a"))==NULL)
{
    printf("Could not open file containing location summary\n");
    exit(0);
}
fprintf(loc,"%s %8.3f %8.3f %8.3f %8.3f %8.2f %6.2f %8d Q1\n",stasjon,slat,slon,lat,lon,(s[10]-p[15])*HST,Mb,amp_avg);
fclose(loc);
fclose(coh);
goto hoved;

number2:
amp_avg=(am[0]+am[1]+am[2])/3;
printf("Qual_2:\n");
fprintf(mail,"Qual_2:\n");
printf("P-ph: %6.2f S-ph: %6.2f Coda: %6.2f Azi: %6.3f VC: %6.3f Dst:%6.1f\n",p[10],s[10],d[10],p[13],sd,(s[10]-p[15])*HST);
fprintf(mail,"P-ph: %6.2f S-ph: %6.2f Coda: %6.2f Azi: %6.3f VC: %6.3f Dst:%6.1f\n",p[10],s[10],d[10],p[13],sd,(s[10]-p[15])*HST);

//pluss 180
//p[13]=p[13]+180.;

x11=sin(p[13]*0.01745)*(s[10]-p[15])*HST;
y11=cos(p[13]*0.01745)*(s[10]-p[15])*HST;
lat=slat+y11/111.2;
lon=slon+x11/(111.2*cos(slat/57.3));
printf("%10.3f %10.3f ( %s %10.3f %10.3f) %10.2f %6.2f\n",lat,lon,stasjon,slat,slon,(s[10]-p[15])*HST,Mb);
fprintf(mail,"%10.3f %10.3f ( %s %10.3f %10.3f) %10.2f %6.2f\n",lat,lon,stasjon,slat,slon,(s[10]-p[15])*HST,Mb);

system("rm -f /home/work/coord");
coh=fopen("/home/work/coord","w");
fprintf(coh,"%s %8.3f %8.3f %8.3f %8.3f %8.2f %6.2f %8d Q2\n",stasjon,slat,slon,lat,lon,(s[10]-p[155])*HST,Mb,amp_avg);
loc=fopen("/home/work/location","a+"); /* open file for location            */
if((loc=fopen("/home/work/location","a"))==NULL)
{
    printf("Could not open file containing location summary\n");
    exit(0);
}
fprintf(loc,"%s %8.3f %8.3f %8.3f %8.3f %8.2f %6.2f %8d Q2\n",stasjon,slat,slon,lat,lon,(s[10]-p[15])*HST,Mb,amp_avg);
hoved:
fclose(loc);
fclose(coh);

fclose(mail);
fclose(net);
}
/*-------------------------------------------------------------------------*/
/* Pick out station codes and components for corresponding channels        */
/*-------------------------------------------------------------------------*/
stacomp(nchan,stcode,comp,head)
int nchan;
char* stcode;
char* comp;
char* head;
{
int l;
int i;
int k;
k=160;
for(i=0;i<nchan;i++)
{
    for(l=0;l<4;l++)
    {
        stcode[i*4+l]=head[k+l];
        comp[i*4+l]=head[k+l+4];
    }
    k=k+8;
}
}
/*--------------------------------------------------------------------------*/
/* get default channels to process from /home/seislog/pr/pick.def           */
/*--------------------------------------------------------------------------*/
defchan(chan,nchtot,tre_start,no_tre_comp)
int *chan;
int *nchtot;
int *tre_start;
int *no_tre_comp;
{
int val;
int k;
int i;
int j;
FILE *par;
int chnr[MAXCHA];
int type[MAXCHA];
int proc[MAXCHA];
int no_ch_def;
int cnt;
int tel;
if((int)(par=fopen("/home/seislog/pr/pick.def","r"))==NULL)
{
    printf("File: /home/seislog/pr/pick.def must be defined !\n");
    exit(0);
}
for(k=0;k<MAXCHA;k++)
{
    val=fscanf(par,"%d %d %d",&chnr[k],&type[k],&proc[k]);
    if(val==EOF)
    {
//        printf("Number channels defined %d\n",k);
        no_ch_def=k;
        goto nomore;
    }
}
nomore:
/*--------------------------------------------------------------------------*/
/* find how many single channels and which ones to process as single channel*/
/*--------------------------------------------------------------------------*/
cnt=0;
for(k=0;k<no_ch_def;k++)
{
    if((type[k]==1 || type[k]==13) && proc[k]==1)
    {
        chan[cnt]=k+1;
//printf("%2d %2d K= %d CNT= %d chan[%d]=%d\n",type[k],proc[k],k,cnt,cnt,chan[cnt]);
//getch();
        cnt++;
    }
}
*nchtot=cnt;
//for(k=0;k<cnt;k++)
//    printf("%5d %5d\n",k,chan[k]);
/*--------------------------------------------------------------------------*/
/* find if any 3-comp, how many and which channels                          */
/*--------------------------------------------------------------------------*/
tel=0;
cnt=0;
for(k=0;k<no_ch_def;k++)
{
    if(type[k]==3 || type[k]==13)      /* do we have a 3-C ?                */
    {
        if(type[k+1]==3 || type[k+1]==13)
        {
            if(type[k+2]==3 || type[k+2]==13)
            {
                if(proc[k]==1 && proc[k+1]==1 && proc[k+2]==1)
                {
//printf("3-C\n");
                    tel++;
                    tre_start[cnt]=k+1;
                    tre_start[cnt+1]=k+2;
                    tre_start[cnt+2]=k+3;
                    cnt=cnt+3;
                    k=k+3;
                }
            }            
        }
    }
} 
*no_tre_comp=tel;
//printf("No of 3-component stations %5d\n",tel);
j=0;
for(k=0;k<tel;k++)
{
    for(i=0;i<3;i++)
    {
//        printf("%5d %5d\n",j,tre_start[j]);
        j++;
    }
}
fclose(par);
}

MAXONE(in,out,ne)
float* in;
float* out;
int    ne;
{
float  max;
int    i;
max=0.0;
in[0]=0.0;
for(i=0;i<ne;i++)
{
    if(in[i] > max)
        max=in[i];
}
for(i=0;i<ne;i++)
    out[i]=(in[i]/max);
}
stco_label(chan,ichan,next,stcode,comp,thead)
int* chan;
int ichan;
float next;
char* stcode;
char* comp;
char* thead;
{
char value[80];
        value[0]=stcode[ (chan[ichan]-1)*4];
        value[1]=stcode[ (chan[ichan]-1)*4+1];
        value[2]=stcode[ (chan[ichan]-1)*4+2];
        value[3]=stcode[ (chan[ichan]-1)*4+3];
        value[4]='\0';
        _moveto(10,next-20);
        _outgtext(value);
        value[0]=comp[ (chan[ichan]-1)*4];
        value[1]=comp[ (chan[ichan]-1)*4+1];
        value[2]=comp[ (chan[ichan]-1)*4+2];
        value[3]=comp[ (chan[ichan]-1)*4+3];
        value[4]='\0';
        _moveto(50,next-20);
        _outgtext(value);
        _moveto(450,8);  /* TIME */
        _outgtext(thead);
}
label1(tid,p,next,thead)
float tid;
int   p;
int next;
char* thead;
{
char value[80];
        _moveto(p,next);
//        sprintf(value,"P-AVG-STA %6.2f\n",tid);
        _outgtext(thead);
}
label2(tid,p,next,thead)
float tid;
int   p;
int next;
char* thead;
{
char value[80];
        _moveto(p,next);
//        sprintf(value,"P-AVG-STA %6.2f\n",tid);
        _outgtext(thead);
}
TimScale8(ds,color,no_of_samples,displacement,xscale,sr)
int ds;
int color;
int no_of_samples;
float displacement;
float xscale;
float sr;
{
char karak[10];
int oldcolor;
int j;
int x,y;
int spsec;
int spsec2;
int cnt;
int cnt2;
int pphase;
int sphase;
int duration;
spsec=(int)(1./sr);
spsec2=(int)(1./sr) * 0.1;
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
cnt=0;
cnt2=0;
for(j=0;j<no_of_samples;j++)
{
    x=(int)(float)j * xscale;
    y=(int)displacement+10;
    _lineto(x,y);
    cnt++;
    cnt2++;
    if(cnt==spsec)
    {
        cnt=0;
        _lineto(x,y-10);
        _moveto(x,y);
    }
    if(cnt2==spsec2)
    {
        cnt2=0;
        _lineto(x,y-4);
        _moveto(x,y);
    }
}
_setcolor(oldcolor);
}
same_phase(fase)
float fase[];
{
int i;
unsigned short val;
int short verdi;
float df;
int truth[5];

//printf("%5.2f-%5.2f=%5.2f\n",fase[0],fase[1],fabs(fase[0]-fase[1]));
//printf("%5.2f-%5.2f=%5.2f\n",fase[0],fase[2],fabs(fase[0]-fase[2]));
//printf("%5.2f-%5.2f=%5.2f\n",fase[0],fase[3],fabs(fase[0]-fase[3]));
//printf("%5.2f-%5.2f=%5.2f\n",fase[1],fase[2],fabs(fase[1]-fase[2]));
//printf("%5.2f-%5.2f=%5.2f\n",fase[1],fase[3],fabs(fase[1]-fase[3]));
//printf("%5.2f-%5.2f=%5.2f\n",fase[2],fase[3],fabs(fase[2]-fase[3]));
df=fase[0]-fase[1];

if(fabs(df) <= LIK)
{
    truth[0]=1;
val=1;
}else{
    truth[0]=0;
val=0;
}
val=val<<1;
df=fase[0]-fase[2];
if(fabs(df) <=LIK)
{
    truth[1]=1;
val=val | 1;
}else{
    truth[1]=0;
val=val | 0;
}
val=val<<1;
df=fase[0]-fase[3];
if(fabs(df) <=LIK)
{
    truth[2]=1;
val=val | 1;
}else{
    truth[2]=0;
val=val | 0;
}
val=val<<1;
df=fase[1]-fase[2];
if(fabs(df) <=LIK)
{
    truth[3]=1;
val=val | 1;
}else{
    truth[3]=0;
val=val | 0;
}
val=val<<1;
df=fase[1]-fase[3];
if(fabs(df) <=LIK)
{
    truth[4]=1;
val=val | 1;
}else{
    truth[4]=0;
val=val | 0;
}
val=val<<1;
df=fase[2]-fase[3];
if(fabs(df) <=LIK)
{

    truth[5]=1;
val=val | 1;
}else{
    truth[5]=0;
val=val | 0;
}
verdi=val;
printf("%5.2f %5.2f %5.2f  %5.2f %d %d %d %d %d %d\n",fase[0],fase[1],fase[2],fase[3],truth[0],truth[1],truth[2],truth[3],truth[4],truth[5]);
    return(verdi);
}
X_SCALE_10(color,ne,xscale,sr,sekunder)
int color;
int ne;
float xscale;
float sr;
float sekunder;
{
char karak[10];
int oldcolor;
int j;
int x,y;
int spsec;
int sptck;
int cnt;
int milli;
int milli_left;
int ne1;
int first;
float sec_left;
milli=(int)(sekunder * 1000.);
milli_left=10000-(milli % 10000);
sec_left=(float)milli_left/1000.;
ne1=(int)(sec_left/sr);
first=(int)sekunder + (10-(int)sekunder % 10);
oldcolor=_getcolor();
_setcolor(color);
_moveto(0,(int)YL);
sptck=(int)((1./sr) * 10.);
cnt= (int)((10.0-sec_left)/sr );
//printf("MILLI=%d MILLI_LEFT=%d SEC_LEFT=%6.3f NE1=%d CNT=%d SPTCK= %d\n",
//milli,milli_left,sec_left,ne1,cnt,sptck);
for(j=0;j<ne;j++)
{
    x=(int)(float)j * xscale;
    y=(int)YL;
    _lineto(x,y);
    cnt++;
    if(cnt==sptck)
    {
sprintf(karak,"%2d\0",first);
_moveto(x-6,y-11);
_outgtext(karak);
_moveto(x,y);
first=first+10;
if(first == 60) first=0;
        cnt=0;
        _lineto(x,y-5);
        _moveto(x,y);
    }
}
_moveto(0,0);
cnt= (int)((10.0-sec_left)/sr );
for(j=0;j<ne;j++)
{
    x=(int)(float)j * xscale;
    y=0;
    _lineto(x,y);
    cnt++;
    if(cnt==sptck)
    {
        cnt=0;
        _lineto(x,y+5);
        _moveto(x,y);
    }
}
_setcolor(oldcolor);
}
coda(pre,p,s,sr,indx,data, dr)
int pre;
float p;
float s;
float sr;
int indx;
float* data;
float* dr;
{
float sta1,lta1;
float sum;
int where2;
int unt;
int j;
int k;
float pre_avg;
float pst_avg;
float limite;
float extra;
int ws;
float limit;
float t1,t2;
int   t1indx,t2indx;
int lpre;
int start;
sum=0.0;
//printf("PRE:%5d P:%6.2f S:%6.2f SR:%6.3f INDX:%5d \n",pre,p,s,sr,indx);
t1=p-10.0;
if(t1<0.0)
    t1=0.0;
t1indx=(int)(t1/sr);

t2=t1+5.0;
t2indx=(int)(t2/sr);
for(j=t1indx;j<t2indx;j++) 
    sum=sum+fabs(data[j]);
pre_avg=sum/(float)(t2indx-t1indx);
limite=pre_avg+(pre_avg*35.5)/100.0;
//printf("%6d %6d PRE_AVG %6.2f LIM %6.2f\n",t1indx,t2indx,pre_avg,limite);
t1=s;
t1indx=(int)(t1/sr);
t2=t1+5.0;
t2indx=(int)(t2/sr);
for(;;)
{
    sum=0.0;
    for(j=t1indx;j<t2indx;j++) 
       sum=sum+fabs(data[j]);
    pst_avg=sum/(float)(t2indx-t1indx);
//printf("%6d %6d PST_AVG %6.2f LIM %6.2f\n",t1indx,t2indx,pst_avg,limite);
//getch();
    if(pst_avg <= limite)
        goto fin2;
    t1indx++;
    t2indx++;
    if(t2indx > indx)
    {
//        printf("PST_AVG %6.2f PRE_AVG %6.2f\n",pst_avg,pre_avg);
        j=indx;
        goto fin2;
    }
}
fin2:
//    printf("Duration %10.2f seconds\n",(float)j*0.020);
    *dr=(float)j*sr;

}
smap(p,s,latepi,lonepi,latsta,lonsta,MM,ns,statreg,slatt1,slonn1)
float p[];
float s[];
float latepi;
float lonepi;
float latsta;
float lonsta;
float MM;
int   ns;
char  statreg[100][5];
float slatt1[];
float slonn1[];
{
float  xjm[15000],yjm[15000];
float    dummy;
float    siz;
int      ex1,ex2,ey1,ey2;
char     cursta[10];
float    Mb;
float    plon,plat,cplon,cplat;
float    velocity[4];                  /* velocity                          */
int      mov;
int      ndepth;                       /* number of depths to test for      */
int      nstation;
float    h0;
float    c1,c2,c3;
float    slatt[100],slonn[100];        /* station lat and lon               */
char     stat[100][5];                 /* station names from station file   */
float    xf,yf;
int      nco;
int      n;                            /* number of fonts                   */
int      ix0,iy0;                      /* screen coordinates of origin      */
char     symbol;                       /* epi plot symbol                   */
float    xscalejm;                     /* screen coordinates/degree         */
float    yscale;
float    lat0,lon0;
float    slon1,slon2,slat1,slat2;      /* min,max lon and lat               */
char     value[80];                    /* used for plotting text            */
int      klat1,klat2;
int      klat;
float    slat;
int      klon1,klon2;
int      klon;
float    slon;
int      kjm,ljm;
int      ixjm;
int      iyjm;
int      ix1;
int      iy1;
int      ix2;
int      iy2;
int      is;
float    sx,sy;
FILE     *fp;                          /* pointer to input                  */
char     b;                            /* one character                     */
int      fnt_no;                       /* font number to use                */
int      con_ser;                      /* switch to tell cosole or serial   */
int      plot;
int      i;                            /* counter                           */
int      k;                            /* counter                           */
int      j;                            /* counter                           */
int      l;                            /* counter                           */
int      curdpx;                       /* current dpx number                */
int      tot;                          /* total duration of event in sec    */
int      ix,iy;                        /* plot vector                       */
float    xstep;                        /* plot x step scale                 */
float    xscale;                       /* Xplot scale in tekunits pr sec    */
char*    fontpath=getenv("WINFONTS");  /* define windows fonts              */

n=_registerfonts(fontpath);            /* register all fonts                */
fnt_no=61;                             /* choose font number                */
plot=1;
/*--------------------------------------------------------------------------*/
/* decide if I am started from console or remote terminal                   */
/*--------------------------------------------------------------------------*/
con_ser=line_typ();

/* set fixed parameters such as plot scale and plot window                  */
/* lowerleft */
ix0=442;
iy0=320;


slat1=latsta-4.0;
slat2=latsta+3.0;
slon1=lonsta-2.5;
slon2=lonsta+4.0;
lat0=slat1;
lon0=slon1;

/* scalemap */
xscalejm=(630-ix0)/(slon2-slon1);
yscale=(290)/(slat2-slat1);


/* DRAW LATITUDES  */
klat1=(int)(slat1*10.0);
klat2=(int)(slat2*10.0);
/* 0.5 deg distance between lines */

if(plot==1)
{
//    graphic_mode(con_ser);
     
    _setcolor(0);
    _rectangle(_GFILLINTERIOR,400,20,639,350);
    _setcolor(14);
    _rectangle(_GBORDER,400,20,639,350);
    _setcolor(3);
}


for (i=0;i<ns;i++)
{
    sprintf(value,statreg[i]);
    trss(slatt1[i],slonn1[i],lat0,lon0,xscalejm,yscale,&ixjm,&iyjm,ix0,iy0);  

    if(slatt1[i] > slat1 && slatt1[i] < slat2)
    {
        if(plot==1)
        {
            _moveto(ixjm,iyjm);                           
            _outgtext(value);
        }
    }
}


for(klat=klat1;klat<=klat2;klat=klat+10)
{
    slat=klat/10.0;
    trss(slat,slon1,lat0,lon0,xscalejm,yscale,&ix1,&iy1,ix0,iy0);  
    sprintf(value,"%4.1f",slat);
    if(plot==1)
    {
        _moveto(ix1-40,iy1-5);
        _outgtext(value);
        _moveto(ix1,iy1);
    }
    trss(slat,slon2,lat0,lon0,xscalejm,yscale,&ix2,&iy2,ix0,iy0);  
    if(plot==1)
        _lineto(ix2,iy2);
}
//
/* draw longitudes */
//
klon1=(int)(slon1*10.0);
klon2=(int)(slon2*10.0);
/* 2 degrees distance between lines */
for(klon=klon1;klon<=klon2;klon=klon+20)
{
    slon=klon/10.0;
    trss(slat1,slon,lat0,lon0,xscalejm,yscale,&ix1,&iy1,ix0,iy0);  
    sprintf(value,"%5.1f",slon);
    trss(slat2,slon,lat0,lon0,xscalejm,yscale,&ix2,&iy2,ix0,iy0);  
    if(plot==1)
    {
        _moveto(ix1-30,iy1+17);
        _outgtext(value);
        _moveto(ix1,iy1);
        _lineto(ix2,iy2);
    }
}
//
/* PLOT CONTOURS */
//
if(plot==1)
    _setcolor(14);
if(latsta > 50.0 && lonsta > 4.0)
{
    con=fopen("/home/seislog/pr/norway.map","r");
}else{
    con=fopen("/home/seislog/pr/input.map","r");
}
ljm=1;
more:
if(fscanf(con,"%d",&nco)!=EOF)
{
    for(kjm=0;kjm<nco;kjm++)
    {
        fscanf(con,"%f",&xjm[kjm]);
        fscanf(con,"%f",&yjm[kjm]);
    }
if(plot==1)
    _setcolor(11);
mov=0;
for(kjm=0;kjm<=(nco-1);kjm++)
{
    sx=xjm[kjm];
    sy=yjm[kjm];
    if(sx>=slat1 && sx<=slat2 && sy>=slon1 && sy<=slon2) /* INSIDE ?        */
    {
        if(mov==1)                     /* YES, but have we moved inside ?   */
        {                              /* YES, then draw to new point       */
            trss(sx,sy,lat0,lon0,xscalejm,yscale,&ixjm,&iyjm,ix0,iy0);  
            if(plot==1)
                _lineto(ixjm,iyjm);
//            mov=0;
        }else{
            trss(sx,sy,lat0,lon0,xscalejm,yscale,&ixjm,&iyjm,ix0,iy0);  
            if(plot==1)
                _moveto(ixjm,iyjm);
            mov=1;
        }
    }else{
        mov=0;
    }
}
goto more;
}
fclose(con);
//
/* Plot stations on map */
//
if(plot==1)
    _setcolor(14);
for (i=0;i<nstation;i++)
{
    if(slatt[i]<slat1 || slatt[i]>slat2)
       goto out;
    sprintf(value,stat[i]);
    trss(slatt[i],slonn[i],lat0,lon0,xscalejm,yscale,&ixjm,&iyjm,ix0,iy0);  
    if(plot==1)
    {
        _moveto(ixjm,iyjm);                           
        _outgtext(value);
    }
out:

j=0;
}
    sprintf(value,stasjon);
    trss(latsta,lonsta,lat0,lon0,xscalejm,yscale,&ixjm,&iyjm,ix0,iy0);  
    if(plot==1)
    {
        _moveto(ixjm,iyjm);                           
        _outgtext(value);
    }
cor=fopen("/home/work/coord","r");

    trss(latsta,lonsta,lat0,lon0,xscalejm,yscale,&ixjm,&iyjm,ix0,iy0);  
    if(plot==1)
        _moveto(ixjm,iyjm);
    trss(latepi,lonepi,lat0,lon0,xscalejm,yscale,&ixjm,&iyjm,ix0,iy0);  
    if(plot==1)
        _lineto(ixjm,iyjm);
sy=iyjm;
sx=ixjm;

siz=(MM*5.)/2.;
ex1=(int)(sx-siz/2.);
ex2=(int)(sx+siz/2.);
ey1=(int)(sy-siz/2.);
ey2=(int)(sy+siz/2.);
//printf("Mb %6.2f SIZ %6.2f %d %d %d %d\n",Mb,siz,ex1,ex2,ey1,ey2);
_setcolor(12);
_ellipse(_GFILLINTERIOR,ex1,ey1,ex2,ey2);

//getch();



for(i=0;i<360;i++)
{
    xf=sin((float)i*0.01745)*5.;
    yf=cos((float)i*0.01745)*5.;
    ixjm=(int)xf+sx;
    iyjm=(int)yf+sy;

    if(i==0)
    {
        if(plot==1)
            _moveto(ixjm,iyjm);
    }else{
        if(plot==1)
            _lineto(ixjm,iyjm);
    }
}
    sprintf(value,"<- -> cursor control");
    if(plot==1)
    {
        _moveto(10,10);
        _outgtext(value);
    }
//symbol=getch();                            /* read one character from keyboard  */
}

trss(slat,slon,lat0,lon0,xscale,yscale,ix,iy,ix0,iy0)
float slat;
float slon;
float lat0;
float lon0;
float xscale;
float yscale;
int*   ix;
int*   iy;
int   ix0;
int   iy0;
{
*iy=iy0-(float)((slat-lat0)*yscale);
*ix=ix0+(float)((slon-lon0)*xscale);
return;
}
