/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:29/01/97                       */
/* Program:RD_PAR_SER                                                       */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This routine reads the parameters for the multiserial board.             */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
char    com[81];                /* room for one line from parameter file    */
FILE     *in;                          /* input file pointer                */

main ()
{
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
int      no_ser;                       /* serial ports in use               */
int      i,j,k,l;                      /* indexes                           */
int      ser_lin;                      /* serial number definition          */
int      comp;                         /* components defined this serial lin*/
int      cmdef[8];                     /* components defined per serial line*/
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"RD_PAR_SER_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"RD_PAR_SER_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
in=fopen("/home/seislog/pr/par_ser","r"); /* open configuration file        */
for(i=0;i<17;i++)                      /* Read first 17 comments lines      */
    Get_Parameter();
for(i=0;i<16;i++)                  /* zero configuration table          */
{
    for(j=0;j<20;j++)
        ptr->cha_def[i][j]=0;
}
for(i=0;i < 16;i++)
{
    fscanf(in,"%d",&ptr->cha_def[i][0]); /* read the serial line number     */
    fscanf(in,"%d",&ptr->cha_def[i][1]); /* read connected or not           */
    fscanf(in,"%d",&ptr->cha_def[i][2]); /* read samples per second         */
    fscanf(in,"%d",&ptr->cha_def[i][3]); /* number of components this line  */
    fscanf(in,"%d",&ptr->cha_def[i][4]); /* comp 1 active or passive        */
    fscanf(in,"%d",&ptr->cha_def[i][5]); /* comp 2 active or passive        */
    fscanf(in,"%d",&ptr->cha_def[i][6]); /* comp 3 active or passive        */
    fscanf(in,"%d",&ptr->cha_def[i][7]); /* comp 4 active or passive        */
    fscanf(in,"%d",&ptr->cha_def[i][8]); /* comp 5 active or passive        */
    fscanf(in,"%d",&ptr->cha_def[i][9]); /* comp 6 active or passive        */
    fscanf(in,"%d",&ptr->cha_def[i][10]); /* comp 7 active or passive        */
    fscanf(in,"%d",&ptr->cha_def[i][11]); /* comp 8 active or passive        */
/*--------------------------------------------------------------------------*/
/* for connected channels, compute number of samples + header per second    */
/* from protocol (all channels)                                             */
/*--------------------------------------------------------------------------*/
    if(ptr->cha_def[i][1]==1)
        ptr->cha_def[i][15]=ptr->cha_def[i][2]*ptr->cha_def[i][3]+SHDR;
}
/*--------------------------------------------------------------------------*/
/* count number of channels connected                                       */
/*--------------------------------------------------------------------------*/
ptr->noser=0;
ptr->active_lines=0;
ptr->totchan=0;
for(i=0;i<16;i++)
{
    if(ptr->cha_def[i][1]==1)
    {
        ptr->active_lines++;
        ptr->totchan=ptr->totchan+ptr->cha_def[i][3];
    }
}
printf("Number of serial ports in use:%3d Total no. of channels: %d\n",
ptr->active_lines,ptr->totchan);
for(i=0;i<16;i++)
{
    for(k=0;k<5;k++)
        printf("%5d ",ptr->cha_def[i][k]);
    for(k=5;k<16;k++)
        printf("%2d ",ptr->cha_def[i][k]);
    printf("\n");
}
}
/*--------------------------------------------------------------------------*/
/* Read one line of characters from the parameter file                      */
/*--------------------------------------------------------------------------*/
void Get_Parameter()
{
int i;
int ch;
    for(i=0;(i<80) && ((ch=fgetc(in)) != EOF) && (ch !='\n'); i++)
        com[i]=ch;
    com[i]='\0';
printf("%s\n",com);
/*getch();*/
}
