/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:20/12/94                       */
/* Program:RD_PARAM                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program will read different parameter files generated by PAR.       */
/* Each parameter set consists of two files: param_1a and param_1b,         */
/*                                           param_2a and param_2b etc.     */
/*                                                                          */
/* In the first parameter set the parameter: Number of active parametersets */
/* will tell this program how many sets to input.                           */
/* If this parameter is present in another set it will not be used.         */
/* The different set are input and parts of the SHARE_BLOCK is initialized. */
/* Some new values are also computed based on the input values.             */
/*                                                                          */
/****************************************************************************/

#include   "shar_mem.h"            /* shared buffers,constants etc          */
#include   <stdio.h>
#include   <fcntl.h>
#include   <sys/mman.h>
#include   <stdlib.h>
#include   <string.h>
#include   "filedef.h"

FILE       *in;                    /* Pointer to name of file               */

int var;
char nombre[20];

int        err_id;                 /* event id                              */
char       com[800];                /* String text parameterfile             */
char       txt[81];                /* String text parameterfile             */

int     present;
int     port;
int     bd;
char    portbaud[10];
char    dum;
int     rnghdr;
int     i;
int     l;
int     k;
int     ch;
int     base;
int     set;
float   sval;
int     a;
float   temp;
main()
{
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"RD_PARAM_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"RD_PARAM_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

ptr->sec_buffer=1;                 /* set number of seconds/buffer to 1     */
ptr->share_siz=BUFFER;             /* size of shared data buffers           */
sprintf(ptr->Bufdir[0],"/home/buffer/");
sprintf(ptr->Eventdir, "/home/events/");
/*--------------------------------------------------------------------------*/
/* Open Parameter-files                                                     */
/*--------------------------------------------------------------------------*/
    in=fopen(PARAM_1A,"r");         
    for(k=0;k<18;k++)
    {
        Get_Parameter();           /* Get parameter 17 active sets          */
    }    
    fclose(in);
    sscanf(com,"%70c %d\n",txt,&var);
    ptr->active=var;

/*--------------------------------------------------------------------------*/
/* Read parameter-file line by line into temprary storage before            */
/* placing parameters in right place in Parameter-Module                    */
/*--------------------------------------------------------------------------*/

for(a=0;a<ptr->active;a++)
{
    switch(a)
    {
    case 0:    
    in=fopen(PARAM_1A,"r");         
    break;
    case 1:    
    in=fopen(PARAM_2A,"r");         
    break;
    case 2:    
    in=fopen(PARAM_3A,"r");         
    break;
    case 3:    
    in=fopen(PARAM_4A,"r");         
    break;
    case 4:    
    in=fopen(PARAM_5A,"r");         
    break;
    }       

Get_Parameter();                                  /* Read line 00 Comment   */

Get_Parameter();                                  /* Read line 01 Comment   */

Get_Parameter();                                  /* Read line 02 NAME      */
if(a==0)
    sscanf(com,"%70c %s\n",txt,ptr->name);

Get_Parameter();                                  /* Read line 03 MINTRG    */
sscanf(com,"%70c %d\n",txt,&ptr->mintrg[a]);      

Get_Parameter();                                  /* Read line 04 PREVNT    */
sscanf(com,"%70c %d\n",txt,&ptr->prevnt[a]);      /* Store in datamodule    */

Get_Parameter();                                  /* Read line 05 PREMAX    */
sscanf(com,"%70c %d\n",txt,&ptr->premax[a]);      /* Store in datamodule    */

Get_Parameter();                                  /* Read line 06 POSTMAX   */
sscanf(com,"%70c %d\n",txt,&ptr->postmx[a]);      /* Store in datamodule    */

Get_Parameter();                                  /* Read line 07 MAXBUF    */
sscanf(com,"%70c %d\n",txt,&ptr->maxbuf[a]);      /* Store in datamodule    */

Get_Parameter();                                  /* Read line 08 NCHAN     */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->nchan);      /* Store in datamodule    */
Get_Parameter();                                  /* Read line 09 RATE      */
if(a==0)
{
   sscanf(com,"%67c %f\n",txt,&sval);
//printf("SVAL:%f\n",sval);
ptr->srate=sval;                                  /* convert to frequency   */
//printf("SRATE=%f\n",ptr->srate);
temp=1.0/sval+0.0001;
//printf("temp=%f\n",temp);
ptr->rate=(int)temp;                              /* Store in datamodule    */
}
Get_Parameter();                                  /* Read line 10 INTCLK    */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->rsmp);     /* Store in datamodule    */
ptr->intclk=0;   /* PARAMETER NOT IN USE, SET TO ZERO */
Get_Parameter();                                  /* Read line 11 CLKTYP    */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->clktyp);     /* Store in datamodule    */

Get_Parameter();                                  /* Read line 12 UNIT      */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->unit);       /* Store in datamodule    */

Get_Parameter();                                  /* Read line 13 REC_IND   */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->watch);    /* Store in datamodule    */
ptr->pip_dsk=1; /* PARAMETER NOT IN USE, SET TO ONE */

Get_Parameter();                                  /* Read line 14 DELAY     */
if(a==0)
    sscanf(com,"%70c%f\n",txt,&ptr->delay);       /* Store in datamodule    */

Get_Parameter();                                  /* Read line 15 P_PHASE   */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->web);     /* Store in datamodule    */
//printf("WEBBBBB %d\n",ptr->web);
ptr->p_phas=0; /* PARAMETER NOT IN USE, SET TO ZERO */
Get_Parameter();                                  /* Read line 16 A/D IRQ   */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->ADIRQ);     /* Store in datamodule    */

Get_Parameter();                                  /* Read line 17           */

Get_Parameter();                                  /* Read line 18 dsk_lev   */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->dsk_lev);    /* Store in datamodule    */

Get_Parameter();                                  /* Read line 19 min_dur   */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->min_dur);    /* Store in datamodule    */

Get_Parameter();                                  /* Read line 20 DUR lim   */
sscanf(com,"%70c %d\n",txt,&ptr->durlim[a]);      /* Store in datamodule    */

Get_Parameter();                                  /* Read line 21 MINUTES   */
if(a==0)
sscanf(com,"%70c %d\n",txt,&ptr->nmin_per_ring[0]); /* store in datamodule  */

Get_Parameter();                                  /* Read line 22 NO OF RNG */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->nringbuf[0]);/* store in datamodule    */

//printf("No of PAR %d No of files %d\n",ptr->active,ptr->nringbuf[0]);


Get_Parameter();                                  /* Read line 23 CONT_DAT  */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->err_mail);   /* store in datamodule    */
ptr->cont_dat=0; /* PARAMETER NOT IN USE, SET TO ZERO */
Get_Parameter();                                  /* Read line 24 TIMERR    */
if(a==0)
   sscanf(com,"%70c%f\n",txt,&ptr->timerr);      /* store in datamodule    */
ptr->timerr=0; /* PARAMETER NOT IN USE, SET TO ZERO */
Get_Parameter();                                  /* Read line 25 BAUD      */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->baud);       /* store in datamodule    */
//("BAUD=%d\n",ptr->baud);

Get_Parameter();                                  /* Read line 26 AUTOP     */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->autop);      /* store in datamodule    */
//printf("Autoproc=%d\n",ptr->autop);

Get_Parameter();                                  /* Read line 27 MAIL      */
if(a==0)
    sscanf(com,"%70c %d\n",txt,&ptr->mail);       /* store in datamodule    */

Get_Parameter();                                  /* Read line 28 A/D base1 */
if(a==0)
{
    sscanf(com,"%70c %d\n",txt,&ptr->AD_BASE_1);  /* store in datamodule    */
    base=ptr->AD_BASE_1;
    ptr->AD_BASE_1=con(base);

//printf("BASE1=%d\n",ptr->AD_BASE_1);
}
Get_Parameter();                                  /* Read line 29 A/D base2 */
if(a==0)
{
    sscanf(com,"%70c %d\n",txt,&ptr->AD_BASE_2);       /* store in datamodule    */
base=ptr->AD_BASE_2;
ptr->AD_BASE_2=con(base);
//printf("BASE2=%d\n",ptr->AD_BASE_2);
}
Get_Parameter();                                  /* Read line 30 Port/Baud */
if(a==0)
{
    sscanf(com,"%67c %s",txt,portbaud);       /* store in datamodule    */
    present=0;
    for(i=0;i<10;i++)
    {
        if(portbaud[i]==',')
            present=1;
    }
    if(present==0)
    {
        printf("ERROR in parameter for port,baudrate clock !\n");
        printf("      Must have a comma between port and baudrate\n");
        exit(0);
    }
    for(i=0;i<10;i++)
        if(portbaud[i]==',')portbaud[i]=' ';
    sscanf(portbaud,"%d %d",&port,&bd);
    ptr->prt_clk=port;
    ptr->bd_clk=bd;
}
Get_Parameter();                                  /* Read line 31 Port/Baud */
if(a==0)
{
    sscanf(com,"%67c %s\n",txt,portbaud);       /* store in datamodule    */
    sscanf(com,"%67c %s",txt,portbaud);       /* store in datamodule    */
    for(i=0;i<10;i++)
        if(portbaud[i]==',')portbaud[i]=' ';
    sscanf(portbaud,"%d %d",&port,&bd);
    ptr->prt_dig=port;
    ptr->bd_dig=bd;
//("BAUD=%d\n",ptr->bd_dig);
}
fclose(in);
/*--------------------------------------------------------------------------*/
/* Compute number of samples/channel in each buffer                         */
/*--------------------------------------------------------------------------*/

ptr->nsamp =(int)(ptr->rate * ptr-> sec_buffer);

/*--------------------------------------------------------------------------*/
/* Size of one shared memory buffer                                         */
/*--------------------------------------------------------------------------*/

ptr->bufsiz=ptr->nsamp*ptr->nchan;
ptr->nchstr[CONTIN] = ptr->nchan;
/*--------------------------------------------------------------------------*/
/* Compute the recordsize in bytes for different units                      */
/*--------------------------------------------------------------------------*/
    ptr -> reclength = (ptr->nsamp * ptr->nchan)*4+HEADER*4;
/*--------------------------------------------------------------------------*/
/* Compute the recordsize in bytes for each channel one second              */
/*--------------------------------------------------------------------------*/
//for(i=0;i<ptr->nchan;i++)
//    ptr -> chalength[i] = ptr->rate *4+HEADER*4;

//printf("RECLENGTH=%d\n",ptr->reclength);

ptr->maxblk=60*ptr->nmin_per_ring[0];

ptr->nblka=EVTSIZ;

ptr -> maxque   = MAXQUE;                 /* Max nr of events in queue      */
ptr -> recrd[0] = 0;                      /* Recording flag initially zero  */
ptr -> queue    = -1;                     /* Queue counter initially -1     */
ptr -> ntick    = 0;                      /* Nr of buffers to write set zero*/
ptr -> oktime   = 0;                      /* Flag to indicate correct time  */

}
/*--------------------------------------------------------------------------*/
/* Open Parameter-files                                                     */
/*--------------------------------------------------------------------------*/

for(a = 0; a < ptr->active; a++)
{
    ptr->nchdet[a] = 0;         
    ptr->nchstr[a] = 0;
    switch(a)
    {
    case 0:    
    in=fopen(PARAM_1B,"r");         
    break;
    case 1:    
    in=fopen(PARAM_2B,"r");         
    break;
    case 2:    
    in=fopen(PARAM_3B,"r");         
    break;
    case 3:    
    in=fopen(PARAM_4B,"r");         
    break;
    case 4:    
    in=fopen(PARAM_5B,"r");         
    break;
    }       

    Get_Parameter();                                 /* Read line 00 Comment    */
    for(l=0;l<ptr->nchan;l++)
    {
        Get_Parameter();                                 /* Read line 01 Comment    */
        if(a==0)
            sscanf(com,"%4c%4c",txt,(char*)ptr->statio+l*4); /* Read STATION            */
        if(a==0)
            sscanf(com,"%9c%4c",txt,(char*)ptr->compon+l*4); /* Read COMPONENT          */
        if(a==0)
            sscanf(com,"%14c%d",txt,&ptr->gain[l]);       /* Read GAIN              */
        sscanf(com,"%19c%f",txt,&ptr->fltlo[a][l]);       /* Read FILTER or DIFF    */
        sscanf(com,"%28c%f",txt,&ptr->flthi[a][l]);       /* Read FILTER            */
        sscanf(com,"%37c%f",txt,&ptr->o_nsta[a][l]);        /* Read STA (seconds)     */
        ptr->nsta[a][l]=ptr->o_nsta[a][l] * ptr-> rate;     /* Convert to samples     */
        sscanf(com,"%43c%f",txt,&ptr->o_nlta[a][l]);        /* Read LTA (seconds)     */
        ptr->nlta[a][l]=ptr->o_nlta[a][l] * ptr-> rate;     /* Convert to samples     */
        sscanf(com,"%49c%f",txt,&ptr->ratio[a][l]);       /* Read RATIO             */
        sscanf(com,"%60c%f",txt,&ptr->postrt[a][l]);      /* Read Post RATIO        */
        sscanf(com,"%71c%2d",txt,&ptr->chaflg[a][l]);     /* Read Trigger Type      */
        sscanf(com,"%74c%2d",txt,&ptr->store[a][l]);      /* Read Recording Flag    */
/*printf("%s\n",com);*/
        if(ptr->chaflg[a][l] != 0)                       /* count number of channels*/
        {                                                /* set for detection in each*/
            ptr->nchdet[a]=ptr->nchdet[a]+1;             /* set the flag indicator=1*/
        }

/* NB changed this as now RC flag can be > 1       DJS 23-09-97 */
        if(ptr->store[a][l] >= 1)                        /* count number of channels*/
        {                                                /* to store this par set.  */
            ptr->nchstr[a]=ptr->nchstr[a]+1;
        }
        ptr->newrl[a]=(ptr->nsamp*ptr->nchstr[a])*4+HEADER*4;
    }
    fclose(in);
    for(i=0;i<80;i++)
        ptr->Timsav[i]=' ';
//printf("Number of channels to store: %d\n",ptr->nchstr[a]);
//for(i=0;i< ptr->nchan;i++)
//    printf(" %d",ptr->store[a][i]);
//printf("\n");
//printf("NEWRL=%d\n",ptr->newrl[a]);
}
system("/home/seislog/gen_koef");
if(ptr->active==1)
{
system("/home/seislog/res_par");
system("/home/seislog/smp_koef");
}
}
/*-----------------------------------------------Start: Get_Parameter---*/
/* Read one line of characters from parameterfile                       */
/*----------------------------------------------------------------------*/
Get_Parameter()
{
for(i=0; (i<80) && ((ch=fgetc(in)) != EOF) && (ch !='\n'); i++)
    com[i]=ch;
com[i]='\0';        
/*printf("%s\n",com);*/
}
/*-----------------------------------------------End: Get_Parameter----*/
con(ba)
int ba;
{
int ii;
int cnt;
int hex;
char k;
unsigned int add;
unsigned char st[10];
cnt=0;
sprintf(st,"%d\n",ba);
//printf("st: %s\n",st);
for(ii=0;ii<10;ii++)
{
    if(st[ii] != 0xa)
    {
        cnt++;
    }else{
        goto slutt;
    }
}
slutt:
//printf("CNT=%d\n",cnt);  
k=st[0];
k=k & 0x0f;
add=k;
add=add<<4;
k=st[1];
k=k & 0x0f;
add=add | k;
add=add<<4;
k=st[2];
k=k & 0xf;
add=add | k;
//printf("ADD %d\n",add);
return((int)add);
}