/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:25/12/94                       */
/* Program:RE_INITI                                                         */
/*                                                                          */
/* Last update:07/07/97                                                     */
/* 29/05/00 option to re_initialize counters, but not re-allocate ringbuffer*/
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program resets the DPX number to the original startvalue 0.         */
/* The first real or manual event will get DPX number 1 and then upwards    */
/* It also resets the current sequence number of the ringbuffer to 0        */
/* The EVENT record file will also be reinitialized. Be aware that          */
/* direct links to the actual event via the DPX number will be lost,        */
/* however the events will still be stored in /home/events with its full    */
/* filename.                                                                */
/* In other words: You will not see previous recorded events through the    */
/* LGS program, but you can find them by ls /home/events.                   */
/* The errormessage number is also reset to -1 by the program               */
/* The old ringbuffers are removed and new are allocated based on the       */
/* current sizes found in the parameter file.                               */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"               /* This file is stored in DEFS and      */
                                    /* defines the structure of datamod.    */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <errno.h>

FILE     *event;
FILE     *dnr;
FILE     *rng;
FILE     *err;

main(int argc, char *argv[])
{
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

int      i;                            /* Counter                           */
int      k;                            /* Counter                           */
int      j;
int      dpx;                          /* detection number                  */
int      pdpx;                         /* collection number                 */
int      contnr;                       /* sequence number ringbuffers       */
int      rng_number;                   /* sequence number ring buffer file  */
int      err_no;                       /* sequence number error file        */
int no_dsk_all = 0;
if(argc > 1)
{
  if((strcmp(argv[1] , "-n")==0) || (strcmp(argv[1], "-N")==0))
    no_dsk_all=1;
}

dpx=0;                                 /* initial value                     */
pdpx=-1;                               /* initial value                     */
err_no=0;                              /* initial value                     */
contnr=0;                             /* initial value                     */
rng_number=0;                          /* initial value                     */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"RE_INITI_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"RE_INITI_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

if(no_dsk_all == 0)
{
  system("'\'rm -f /home/buffer/*");     /* remove all files from events      */
  system("'\'rm -f /home/rngbuf/*");     /* remove all files from rngbuf      */
  system("rm -f /home/seislog/pr/rng_number");/* del current ringbuffer file  */
}
/*system("'\'rm -f /home/events/*");*/
system("rm -f /home/seislog/pr/detnr");/* remove current detection no. file */
system("rm -f /home/seislog/pr/contnr");
/*--------------------------------------------------------------------------*/
/* establish new detection number file and set first value to 0             */
/*--------------------------------------------------------------------------*/
if((dnr=fopen("/home/seislog/pr/detnr","w"))==NULL)
{
    printf("*** RE_INITI *** Can't open new DETNR file !!!!\n");
    printf("*** RE_INITI *** Exit and investigate\n");
    exit(0);
}
fprintf(dnr,"%8d\n",dpx);
fclose(dnr);
printf("*** RE_INITI *** DETNR reinitialized to 0\n");
/*--------------------------------------------------------------------------*/
/* establish new collection number file and set first value to 0             */
/*--------------------------------------------------------------------------*/
if((dnr=fopen("/home/seislog/pr/pdetnr","w"))==NULL)
{
    printf("*** RE_INITI *** Can't open new PDETNR file !!!!\n");
    printf("*** RE_INITI *** Exit and investigate\n");
    exit(0);
}
fprintf(dnr,"%8d\n",pdpx);
fclose(dnr);
printf("*** RE_INITI *** PDETNR reinitialized to -1\n");
/*--------------------------------------------------------------------------*/
/* establish new ringbuffer number file and set first value to 0             */
/*--------------------------------------------------------------------------*/
if(no_dsk_all == 0)
{
  if((rng=fopen("/home/seislog/pr/rng_number","w"))==NULL)
  {
    printf("*** RE_INITI *** Can't open new RNG_NUMBER file !!!!\n");
    printf("*** RE_INITI *** Exit and investigate\n");
    exit(0);
  }
  fprintf(rng,"%8d\n",rng_number);
  fclose(rng);
  printf("*** RE_INITI *** RNG_NUMBER reinitialized to 0\n");
}
/*--------------------------------------------------------------------------*/
/* establish new error number file and set first value to 0                 */
/*--------------------------------------------------------------------------*/
if((err=fopen("/home/buffer/err_no","w"))==NULL)
{
    printf("*** RE_INITI *** Can't open new ERR_NO file !!!!\n");
    printf("*** RE_INITI *** Exit and investigate\n");
    exit(0);
}
fprintf(err,"%8d\n",err_no);
fclose(err);
printf("*** RE_INITI *** ERR_NO reinitialized to 0\n");

if((dnr=fopen("/home/seislog/pr/contnr","w"))==NULL)
{
    printf("*** RE_INITI *** Can't open new CONTNR file !!!!\n");
    printf("*** RE_INITI *** Exit and investigate\n");
    exit(0);
}
fprintf(dnr,"%8d\n",contnr);
fclose(dnr);
printf("*** RE_INITI *** CONTNR reinitialized to -1\n");
/*--------------------------------------------------------------------------*/
/* allocate ring buffer system on hard disk                                 */
/*--------------------------------------------------------------------------*/
if(no_dsk_all == 0)
{
  system("/home/seislog/dsk_all");
  system("chmod a+rwx /home/buffer/R*");
  system("chmod a+rwx /home/buffer/idx_log");
  system("chmod a+rwx /home/buffer/err_log");
}
}
