/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:28/03/98                       */
/* Program:RES_PAR                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Reads parameter file for resampling                                      */
/*                                                                          */
/****************************************************************************/

#include   "shar_mem.h"            /* shared buffers,constants etc          */
#include   <stdio.h>
#include   <fcntl.h>
#include   <sys/mman.h>
#include   <stdlib.h>
#include   <string.h>

FILE       *in;                    /* Pointer to name of file               */

char       com[800];                /* String text parameterfile             */
char       txt[81];                /* String text parameterfile             */
int ch;
int     i;
int     l;
int     k;
int act, or,rt;
float fl,fh;
char type[10];
char comp1[5];
char comp2[5];
char comp3[5];
int    ch_ind[20];
int    kanal;
main()
{
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"RES_PAR_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"RES_PAR_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

/*--------------------------------------------------------------------------*/
/* Open Parameter-files                                                     */
/*--------------------------------------------------------------------------*/
    in=fopen("/home/seislog/pr/par_smp","r");         
    Get_Parameter();           /* Get parameter 17 active sets          */
    Get_Parameter();
    sscanf(com,"%d %d %f %f %d %s",&act,&or,&fl,&fh,&rt,type);
//    printf("%d %d %f %f %d %s\n",act,or,fl,fh,rt,type);
    Get_Parameter();
    sscanf(com,"%s",comp1);
    Get_Parameter();
    sscanf(com,"%s",comp2);
    Get_Parameter();
    sscanf(com,"%s",comp3);
//    printf("%s %s %s\n",comp1,comp2,comp3);
    fclose(in);
    ptr->actpas=act;
    ptr->order2=or;
    ptr->fltlo2[0][0]=fl;
    ptr->flthi2[0][0]=fh;
    ptr->ftype[0]=type[0];
    ptr->ftype[1]=type[1];
    ptr->ftype[2]=' ';
    ptr->rsrate=rt;
    for(i=0;i<4;i++)
    {
        if(comp1[i]=='_')
            comp1[i]=' ';
        if(comp2[i]=='_')
            comp2[i]=' ';
        if(comp3[i]=='_')
            comp3[i]=' ';
    }
    for(i=0;i<4;i++)
        ptr->cmp1[i]=comp1[i];
    for(i=0;i<4;i++)
        ptr->cmp2[i]=comp2[i];
    for(i=0;i<4;i++)
        ptr->cmp3[i]=comp3[i];
//printf("%s\n",ptr->ftype);
    kanal=0;
    for(l=0;l<8;l++)
    {
        Get_Parameter();
        sscanf(com,"%s %d %d %d %d %d %d %d %d",
            type,&ch_ind[0],&ch_ind[1],&ch_ind[2],&ch_ind[3],
            &ch_ind[4],&ch_ind[5],&ch_ind[6],&ch_ind[7]);
        for(k=0;k<8;k++)
        {
            ptr->ch_res[kanal]=ch_ind[k];
            kanal++;
        }
//        for(k=0;k<8;k++)
//            printf("%d ",ch_ind[k]);
//        printf("\n");
    }
    k=(ptr->rate % ptr->rsrate);
    if(k != 0)
    {
        printf("The specified resample rate of %d Hz is not valid !\n",ptr->rsrate);
        printf("For your original samplerate of %d Hz these rates are allowed:\n",ptr->rate);
        for(i=1;i<500;i++)
        {
            k=(ptr->rate % i );
            if(k==0)
            printf("%4d",ptr->rate/i);
        }
        printf("\n");
    }
}
/*-----------------------------------------------Start: Get_Parameter---*/
/* Read one line of characters from parameterfile                       */
/*----------------------------------------------------------------------*/
Get_Parameter()
{
for(i=0; (i<80) && ((ch=fgetc(in)) != EOF) && (ch !='\n'); i++)
    com[i]=ch;
com[i]='\0';        
//printf("%s\n",com);
}
/*-----------------------------------------------End: Get_Parameter----*/
