/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:03/02/98                       */
/* Program:RNG_VER                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* Verify ringbuffer system                                                 */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <dirent.h>
#include <string.h>

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */
char     names[5000][32];
main()
{
DIR      *dirp;
struct   dirent *direntp;
char     err_msg[100];
char     syscall[100];                 /* string to hold system command     */
char     dum;
int      indx;
double   MSECS;
double   minst;
float    SEC;
int      YR,DOY,MTH,DAY,HR,MIN,SECL;
int      no_files;                     /* number of files in directory      */
FILE     *direc;                       /* pointer to file                   */
FILE     *rng;                         /* pointer to file                   */
int      adr;                          /* adress of timestamp               */
int      feil;                         /* error indicator                   */
int      dur;                          /* minutes uptime                    */
int      i;                            /* index                             */
int      lk;                           /* index                             */
int      lp;
int      idx_start;                    /* start index for sorting           */
int      bytes;                        /* return value after read           */
char     tid[10];                      /* storage for uptime                */
char     name[50];                     /* storage file name                 */
char     name1[50];                    /* storage oldest file name          */
char     filename[50];                 /* storage file name                 */
char     filenamesave[40];
char     timbuf[100];
char     line[200];
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"CLN_LNK_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"CLN_LNK_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
no_files=0;
/*--------------------------------------------------------------------------*/
/* Make new directory listing of all existing link files                    */
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/* Read all filenames in ringbuffer directory and find the oldest           */
/*--------------------------------------------------------------------------*/
dirp=opendir("/home/rngbuf");
if(dirp == NULL)
{
    sprintf(err_msg,"RNG_VER: Err opening directory /home/rngbuf");
    errlog(ptr);
    exit(0);
}
    
lk=0;
for(;;)
{
    direntp=readdir(dirp);
    if(direntp == NULL) break;
    sprintf(name,"%s\n",direntp->d_name);
    if(name[0]=='1' || name[0]=='2')
    {
        for(i=0;i<29;i++)
            names[lk][i]=name[i];
        lk++;
    }
}
no_files=lk;
closedir(dirp);
if(no_files==0) exit(0);               /* no link files                     */
idx_start=0;
for(lp=0;lp<(no_files-1);lp++)         /* find 'oldest' no_files-1 times    */
{
    sscanf(names[idx_start],"%4d%c%2d%c%2d%c%2d%2d%c%2d",&YR,&dum,&MTH,&dum,&DAY,&dum,&HR,&MIN,&dum,&SECL);
    SEC=(float)SECL;
    TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
    minst=MSECS;
    indx=idx_start;
    for(i=idx_start+1;i<lk;i++)        /* search through names              */
    {
        sscanf(names[i],"%4d%c%2d%c%2d%c%2d%2d%c%2d",&YR,&dum,&MTH,&dum,&DAY,&dum,&HR,&MIN,&dum,&SECL);
        SEC=(float)SECL;
        TIMSEC (YR,MTH,DAY,HR,MIN,SEC,&MSECS);
        if(MSECS < minst)              /* find oldest file                  */
        {
            minst=MSECS;
            indx=i;
        }
    }                                  /* indx now points to 'oldest' file  */

    for(lk=0;lk<28;lk++)               /* copy of oldest linkfile name      */
        filename[lk]=names[indx][lk];
    filename[28]=0xa;

    for(lk=0;lk<28;lk++)               /* save filename in location idx_star*/
        filenamesave[lk]=names[idx_start][lk];
    filenamesave[28]=0xa;

    for(lk=0;lk<28;lk++)               /* put 'oldest' into 'first'         */
        names[idx_start][lk]=filename[lk];
    names[idx_start][28]=0xa;

    for(lk=0;lk<28;lk++)               /* restore filename in free location */
        names[indx][lk]=filenamesave[lk];
    names[indx][28]=0xa;
//    printf("Number of files: %d Oldest file: %s",no_files,names[idx_start]);
//getch();
    idx_start++;                       /* start search in next location     */
}
//system("ls  /home/rngbuf/*_C > /home/work/rngchk");
for(lk=0;lk<no_files;lk++)
{
    for(i=0;i<50;i++)                  /* blancs into file name             */
        name[i]=' ';
    for(i=0;i<100;i++)
        line[i]=' ';
    sprintf(name,"/home/rngbuf/");
    for(lp=0;lp<28;lp++)
    {
        name[lp+13]=names[lk][lp];
        line[lp]=names[lk][lp];
    }
    name[41]=0x0;
    rng=fopen(name, "r");              /* open ring file                    */
    if(rng == NULL)
    {
        sprintf(err_msg,"RNG_VER: Error opening ringbuffer file");
        errlog(ptr);
        exit(0);
    }
    adr=(SIZ_EV_HDR/ptr->reclength+1)*ptr->reclength;
    fseek(rng,adr,SEEK_SET);           /* position at time stamp            */
    fread(timbuf,80,1,rng);           /*  read timestamp                   */
    for(i=0;i<30;i++)
        line[i+30]=timbuf[i];
    feil=0;
    if(line[2] != line[49]) feil=1;
    if(line[3] != line[50]) feil=1;
    if(line[5] != line[46]) feil=1;
    if(line[6] != line[47]) feil=1;
    if(line[8] != line[43]) feil=1;
    if(line[9] != line[44]) feil=1;
    if(line[11] != line[30]) feil=1;
    if(line[12] != line[31]) feil=1;
    if(line[13] != line[33]) feil=1;
    if(line[14] != line[34]) feil=1;
    if(line[16] != line[36]) feil=1;
    if(line[17] != line[37]) feil=1;
    if(feil != 0)
    {
        line[60]='*';
        line[61]='\n';
        printf("%s",line);
    }else{
        line[60]=' ';
    }
    line[61]='\n';

    fclose(rng);
}
}

