/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:24/10/98                       */
/* Program: RST_GPS                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This routine re_initializes the GPS clock.                               */
/* The program controls data pin 2 on the parallell port. It pulls the      */
/* signal up for some seconds and then back down.                           */
/* Pin 2 from the parallell port should be connected to the YELLOW cable on */
/* the GPS, and ground to ground. This voltage turns off the power on the   */
/* GPS and when the voltage falls back, it swithces on again.               */
/*                                                                          */
/****************************************************************************/     
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/dev.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <graph.h>
#include <conio.h>
#define  SIZ 256

int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

void main()
{   
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"RST_GPS_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"RST_GPS_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
//printf("Reset GPS clock\n");
outp(0x378,0x00);                      /* set d0-d7 on par port to 0        */
outp(0x378,inp(0x378) ^ 0x02);         /* invert bit 2                      */
sleep(2);                             /* duration of pulse in msec         */
outp(0x378,inp(0x378) ^ 0x02);         /* invert bit 2 back to normal       */
}   /* End of main */




