/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:30/01/95                       */
/* Program:RTP                                                              */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This program plot data from SEISLOG in near real time. All channels      */
/* marked for detection are plotted. Data are extracted from the circular   */
/* diskfile and plotted to the screen. When a lot of channels are plotted,  */
/* there will be a delay in the plotting. This is because this program will */
/* have a lower priority than the real time processes.                      */
/* This routine decide what type of digitizing unit used in the             */
/* configuration and start the corresponding plotting program.              */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <graph.h>
#include <conio.h>
#include <sys/kernel.h>
#include <sys/name.h>

#define NTR	12

#define BLACK     0
#define BLUE      1
#define GREEN     2
#define LBLUE     3
#define RED       4
#define VIOLET    5
#define BROWN     6
#define GRAY      7
#define DGRAY     8
#define CYAN      9
#define LGRAY    10
#define LLBLUE   11
#define LRED     12
#define LVIOLET  13
#define YELLOW   14
#define WHITE    15

int     amp;
int     old_color;
int     upper;
int     delta;
int     bytes;
int     intval;
int     secs;
int     intvalue;
char    txt[81];
char    tmp[10];
float   value;
float   ampli;
int     ix;
int     lcr;
int     iy;


int     lp;
int     lc;                         /* Counter                              */
int     ll;                         /* Counter                              */
int     kl;                         /* Counter                              */
int     id;
char    as[50];
int     nxt;
int     ret;
int c;
int left;


int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */
int      con_ser;						 	/* switch to tell if console or serial */

main()
{
int      rtp_id;
int      buf_pip_id;
char     rep_msg[10];
char     rec_msg[10];
char     snd_msg[10];

unsigned int cop;
int      k;
int      l3;
int      hopp;
int      kk;
short    shval;
short    v1;
short    v2;
short    w;
unsigned int ver;
int      ixold[MAXCHA];
int      iyold[MAXCHA];
int      maxa[MAXCHA];
int      max_ini[MAXCHA];
int      i;                            /* Counter                           */
float    ixfl;                         /* float value of ix                 */
int      ystart[MAXCHA];               /* start y values for each trace in  */
                                       /* screen koord.                     */
float    avg[MAXCHA];
float    factor[MAXCHA];               /* Gain factor                       */
int      offset[MAXCHA];               /* offsets different channels        */
short    chan[MAXCHA];                 /* Channel Number                    */
float    sum[MAXCHA];                  /* accumulator                       */
float    tot[MAXCHA];
int      number;                       /* no. of chan. marked for detection */
int      skip;                         /* Sample skip factor                */
int      npage;                        /* New page indicator                */
int      sw;                           /* Switch to ind. plot. of station   */
float    increm;                       /* increment x-axis                  */

/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"RTP_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"RTP_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
if(buf_pip_id == -1)
{
  printf("SEISLOG not running.\n");
  exit(0);
}

/*--------------------------------------------------------------------------*/
/* decide if I am started from console or remote terminal					*/
/*--------------------------------------------------------------------------*/
con_ser=line_typ();

rtp_id=qnx_name_attach(0,"rtp");
if(rtp_id== -1)
{
    printf("RTP attach failed\n");
    exit(0);
}
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
/*--------------------------------------------------------------------------*/
/* Find number of channels specified for detection and which channels       */
/*--------------------------------------------------------------------------*/
number=nchan_in_det(chan);
printf("Number of channels in detection:%d\n",number);
/*--------------------------------------------------------------------------*/
/* Set default SKIP value                                                   */
/*--------------------------------------------------------------------------*/
skip=0;
ampli=1.0;
/*--------------------------------------------------------------------------*/
/* Set indicator for new page                                               */
/*--------------------------------------------------------------------------*/
npage=1;
/*--------------------------------------------------------------------------*/
/* Set switch to plot station                                               */
/*--------------------------------------------------------------------------*/
sw=1;
/*--------------------------------------------------------------------------*/
/* Set defaulttime increment                                                */
/*--------------------------------------------------------------------------*/
increm=0.2;

/*--------------------------------------------------------------------------*/
/* Zero all accumulators                                                    */
/*--------------------------------------------------------------------------*/
for(i=0;i<MAXCHA;i++)
{
    sum[i]=0.0;
    max_ini[i]=0;
    tot[i]=0.0;
    avg[i]=0.0;
    offset[i]=0;
    ystart[i]=0;
}

/*--------------------------------------------------------------------------*/
/* Set up start y-values based on number of channels in detector            */
/*--------------------------------------------------------------------------*/
if (con_ser == 1)
{
  upper=740;
  delta=740/NTR * -1;
  ystart[0]=upper+delta/2;
  for(i=1;i<number;i++)
  {
    ystart[i]=ystart[i-1]+delta;
    printf("YSTART[%d]=%4d\n",i,ystart[i]);
  }
}
else
{
  upper=20;
  delta=426/NTR;
  ystart[0]=upper+delta/2;
  for(i=1;i<number;i++)
  {
    ystart[i]=ystart[i-1]+delta;
    printf("YSTART[%d]=%4d\n",i,ystart[i]);
  }
}
//ptr->plt_on_off=2;
for(kk=0;kk<number;kk++)
   offset[kk]=(int)ptr->dc[(chan[kk]-1)];
for(kk=0;kk<number;kk++)
    tot[kk]=ptr->to[(chan[kk]-1)];
avg_val(number,tot,factor);            /* Comp. avg. value based on 5 buff  */

/*_setvideomode(_VRES16COLOR);*/
graphic_mode(con_ser);
if (con_ser==1)
{

}else{
old_color=_getcolor();
_setcolor(LLBLUE);
_rectangle(_GFILLINTERIOR,0,upper,80,upper+delta*number);
_setcolor(LBLUE);
for(kk=0;kk<number;kk++)
{
    if(kk>=NTR)
        goto top0;
    _moveto(0,upper+kk*delta);
    _lineto(80,upper+kk*delta);
}
top0:
_setcolor(old_color);
}
top:
if (con_ser == 1)
{
  clr_scr (con_ser);
  newlab(number,chan);
}
for(i=0;i<number;i++)
{
    ixold[i]=80;
    iyold[i]=ystart[i];

    for(k=0;k<4;k++)
        tmp[k]=ptr->statio[(chan[i]-1)*4+k];
    tmp[4]='\0';
}
/*--------------------------------------------------------------------------*/
/* Enter graphics mode                                                      */
/*--------------------------------------------------------------------------*/
for (;;)
{
    buf_pip_id=Receive(0,&rec_msg,1);
    Reply(buf_pip_id,&rep_msg,1);
    id=ptr->index;
    ret=keyread();
    switch(ret)
    {
        case 10:                    /* UP */
        ampli=ampli+0.08;
        break;
        case 20:                    /* DOWN */
        ampli=ampli-0.08;
        if(ampli < 0.01) ampli=0.01;
        break;
        case 30:                    /* RIGHT */
        increm=increm+0.05;
        break;
        case 40:                    /* LEFT */
        increm=increm-0.05;
        if(increm < 0.01) increm=0.01;
        break;
        case 99:
//        ptr->plt_on_off=0;
        exit(0);
        break;
        default:
        break;
    }
    if(npage == 1)
    {
        npage=NewPage(offset,chan,factor,skip,increm,number,delta,ystart);
        ix=ixold[0];
        iy=iyold[0];
        ixfl=(float)ix;
        if (con_ser == 1) move (ix, iy);
        else _moveto(ix,iy);
    }
    hopp=ptr->nchan*(skip+1);
    for(l3=0;l3<number;l3++)
    {
        ix=ixold[l3];
        iy=iyold[l3];
        if(l3==0)
        {
           if (con_ser == 1)
           {
           }
           else
           {
             old_color=_getcolor();
             _setcolor(BLUE);
             _rectangle(_GFILLINTERIOR,ix+1,upper,ix+(int)(increm*ptr->rate)+20,upper+number*delta);
             _setcolor(old_color);
          }
        }
        if (con_ser == 1) move (ix, iy);
        else _moveto(ix,iy);
        ixfl=(float)ix;
        kk=0;
        for(lp=0;lp<ptr->bufsiz;lp=lp+hopp)
        {
            if (con_ser == 1)
            {
              if (ix > 1000)
              {
                npage=1;               /* Indicate new page                 */
                goto top;
              }
            }
            else
            {
              if(sw==0)
              {
                if (ix > 110)
                {
                    sw=1;
                    newlab(number,chan);
                }
              }
              if(ix > 640)
              {
                npage=1;               /* Indicate new page                 */
                sw=0;
                goto top;
              }
            }
            value=ptr->ibuf[id][lp+HEADER+chan[l3]-1];
            value=value-offset[l3];
            value=value*factor[l3]*ampli*0.2;
            if (con_ser == 1)
            {
              intval=value;
              amp=intval+ystart[l3];
              if (amp>715)
                amp=715;
              if (amp < upper+delta*number)
                amp=upper+delta*number;
              draw (ix,amp);
            }
            else
            {
              value=value* -1.0;
              intval=value;
              amp=intval+ystart[l3];
              if(amp<20)
                amp=20;
              if(amp > upper+delta*number)
                amp=upper+delta*number;
              _lineto(ix,amp);
            }
            ixfl=ixfl+increm;
            ix=(int)ixfl;
        }
        ixold[l3]=ixfl-increm;
        iyold[l3]=amp;
    }
    sw=0;                            /* Turn off indicator for station code */

    }                                /* End of indefinite loop              */
}                                    /* End of main                         */

NewPage(offset,chan,factor,skip,increm,number,delta,ystart)
int   *offset;
short *chan;
float *factor;
int   skip;
float increm;
int   number;
int   delta;
int   *ystart;
{
char  as[50];

int   i;
int   k;
int   secs;
int   intvalue;
int   sekhel;
int   middle_y;
float sekflo;
float seconds;
float msec;
float sectot;
float secdot;
    Time(&seconds);
    sekhel=(int)seconds;
    sekflo=(float)sekhel;
    msec=seconds-sekflo;
    sprintf(as,"PRESS ANY KEY TO STOP PROGRAM\n");
    if (con_ser == 1)
    {
      sectot=(625/(ptr->rate*increm))*(skip+1);
      secdot=(sectot/625.);
      text(500,0,as);
      secs=(208/(ptr->rate*increm))*(skip+1);
      sprintf(as,"%2d seconds",secs);
      text(790,760,as);
      small_text (1);
      for(i=0;i<number;i++)
      {
        if(i>=NTR)
            goto end1;
        intvalue=100./(2.*factor[i]);
        middle_y = upper + (i * delta) + (delta /2) -6;
        sprintf(as,"+  %6d",intvalue);
        text (0, middle_y + 17, as);

        sprintf(as,"-  %6d",intvalue);
        text (0, middle_y - 17, as);

        sprintf(as,"DC:%6d\n",offset[i]);
        text (0, middle_y, as);
      }
    }
    else
    {
      sectot=(400/(ptr->rate*increm))*(skip+1);
      secdot=(sectot/400.);
      _setcharsize(16,10);
      _grtext(330,465,as);
      secs=(100/(ptr->rate*increm))*(skip+1);
      _setcolor(BLACK);
      _rectangle(_GFILLINTERIOR,495,0,515,20);
      _setcolor(WHITE);
      sprintf(as,"%2d seconds",secs);
      _setcharsize(16,10);
      _grtext(495,0,as);
      for(i=0;i<number;i++)
      {
        if(i>=NTR)
            goto end1;
        old_color=_getcolor();
        intvalue=100./(2.*factor[i]);
        old_color=_getcolor();
        _setcolor(BLUE);
        sprintf(as,"+%6d",intvalue);
        _setcharsize(12,10);
        _grtext(0,i*delta+upper+2,as);

        sprintf(as,"-%6d",intvalue);
        _setcharsize(12,10);
        _grtext(0,i*delta+upper+delta-12,as);

        _setcolor(RED);
        sprintf(as,"DC:\n");
        _setcharsize(12,8);
        _grtext(0,upper+(i+1)*delta-delta/2-6,as);
        sprintf(as,"%6d\n",offset[i]);
        _setcharsize(12,10);
        _grtext(10,upper+(i+1)*delta-delta/2-6,as);
        _setcolor(old_color);
      }
    }
end1:
    if (con_ser == 1) small_text (0);
    pltscale();
    cntscale(number);
    timscale(msec,secdot,number,secs,sekhel);
    return(0);
}

pltscale()
{
    if (con_ser == 1)
    {
      move(781,760);
      draw(781,750);
      draw(938,750);
      draw(938,760);
    }
    else
    {
      _moveto(500,15);
      _lineto(500,19);
      _lineto(600,19);
      _lineto(600,15);
    }
}

timscale(msec,secdot,number,secs,sekhel)
float msec;
float secdot;
int   number;
int   secs;
int   sekhel;
{
int sectel;
int l;
float xfl;
int  x;
int sw;
float d_sec;
int xst;
int yst;
sectel=sekhel;
xst=85;
yst=upper+number*delta+3;
d_sec=1./secdot;
if (con_ser == 1)
{
  move(xst,yst);
  xfl=(float)xst;
  xfl=xfl+(1.-msec)/secdot;
  x=(int)xfl;
  sectel=sectel+1;
  sw=(sectel % 10);
  draw(x,yst);
  if(sw != 0)
  {
    draw(x,yst+8);
  }else{
    draw(x,yst+12);
    draw(x,yst-4);
  }
  draw(x,yst);
  for(l=1;l<1000;l++)
  {
    sectel=sectel+1;
    sw=(sectel % 10);
    xfl=xfl+d_sec;
    x=(int)xfl;
    if(x>=1000)
        return(0);
    draw(x,yst);
    if(sw != 0)
    {
        draw(x,yst+8);
    }else{
        draw(x,yst+12);
        draw(x,yst-4);
    }
    move(x,yst);
  }
}
else
{
  old_color=_getcolor();
  _setcolor(BLACK);
  _rectangle(_GFILLINTERIOR,xst-4,yst-3,640,yst+15);
  _setcolor(old_color);
  _moveto(xst,yst);
  xfl=(float)xst;
  xfl=xfl+(1.-msec)/secdot;
  x=(int)xfl;
  sectel=sectel+1;
  sw=(sectel % 10);
  _lineto(x,yst);
  if(sw != 0)
  {
    _lineto(x,yst+8);
  }else{
    _lineto(x,yst+12);
    _lineto(x,yst-4);
  }
  _moveto(x,yst);
  for(l=1;l<640;l++)
  {
    sectel=sectel+1;
    sw=(sectel % 10);
    xfl=xfl+d_sec;
    x=(int)xfl;
    if(x>=640)
        return(0);
    _lineto(x,yst);
    if(sw != 0)
    {
        _lineto(x,yst+8);
    }else{
        _lineto(x,yst+12);
        _lineto(x,yst-4);
    }
    _moveto(x,yst);
  }
}
return(0);
}

cntscale(number)
int number;
{
int l;
int xst;
int yst;
xst=75;
yst=upper;
for(l=0;l<number;l++)
{
    if(l>NTR)
        goto end;
    if (con_ser == 1)
    {
      move(xst,yst-2);
      draw(xst+5,yst-2);
      draw(xst+5,yst+delta+2);
      draw(xst,yst+delta+2);
    }
    else
    {
      _moveto(xst,yst+2);
      _lineto(xst+5,yst+2);
      _lineto(xst+5,yst+delta-2);
      _lineto(xst,yst+delta-2);
    }
    yst=yst+delta;
}
end:
l=0;
}

Time(seconds)
float *seconds;
{
int ik;
char time[15];
char as[50];
char car;
int  hour;
int  min;
float sekunder;
for(ik=0;ik<29;ik++)
  as[ik]=ptr->timcpy[ik];
as[ik]='\n';
if (con_ser == 1)
{
  text(0,760,as);
}
else
{
  old_color=_getcolor();
  _setcolor(2);
  _rectangle(_GFILLINTERIOR,0,0,290,16);
  _setcolor(old_color);
  _setcharsize(16,10);
  _grtext(0,0,as);
}
for(ik=0;ik<14;ik++)
    time[ik]=ptr->timcpy[ik+6];
    sscanf(time,"%f6.3",&sekunder);
    *seconds=sekunder;
}


/*--------------------------------------------------------------------------*/
/* Compute average each channel based on 5 buffers                          */
/*--------------------------------------------------------------------------*/
avg_val(number,tot,factor)
int   number;
float *tot;
float *factor;
{
int   il;
for(il=0;il<number;il++)
{
/*    tot[il]=tot[il]/ptr->nsamp;*/
    factor[il]=10./tot[il];
}
}
/*--------------------------------------------------------------------------*/
/* Find number of channels specified for detection and which channels       */
/*--------------------------------------------------------------------------*/
nchan_in_det(chan)
short *chan;
{
int i;
int j;
int number;
number=0;
j=0;
for(i=0;i<ptr->nchan;i++)
{
printf("%d %d\n",i,ptr->chaflg[0][i]);

    if(ptr->chaflg[0][i] != 0)
    {
        number=number+1;
        chan[j]=i+1;
        j=j+1;
    }
}

return(number);
}


newlab(number,chan)
int number;
short *chan;
{

int ks,kp;
char as[20];
        for(kp=0;kp< number;kp++)
        {
            for(ks=0;ks<4;ks++)
                as[ks]=ptr->statio[(chan[kp]-1)*4+ks];
            as[ks]='\0';
            if (con_ser == 1)
            {
              text(85,upper+kp*delta-25,as);
            }
            else
            {
              _setcharsize(13,10);
              _grtext(85,upper+kp*delta+2,as);
            }
          }
}
keyread()
{
    fflush(stdin);
    fflush(stdout);
    left=kbhit();
    if(left==0) return(0);
    c=getch();                    /* Read 1 character from terminal         */
    if(c != 0xff && c != 0x1b)
    {
        if(c == 0x09)
        {
            printf("TAB either terminal\n");
        }else{
 /*           printf("KAR:%c\n",c);*/
              return(99);
        }
     }else{
        left=kbhit();
        if(left == 0)
        {
            printf("ESC on VT100\n");
        }else{
            if(c == 0xff)
            {
                c=getch();
                switch(c)
                {
                    case 0xa1:
 /*                   printf("QNX :UP\n");*/
                    return(10);
                    break;
                    case 0xa9:
/*                    printf("QNX :DOWN\n");*/
                    return(20);
                    break;
                    case 0xa6:
/*                    printf("QNX :RIGHT\n");*/
                    return(30);
                    break;
                    case 0xa4:
/*                    printf("QNX :LEFT\n");*/
                    return(40);
                    break;
                }
            }else{
                c=getch();
                c=getch();
                switch(c)
                {
                    case 0x41:
                    printf("VT100:UP\n");
                    break;
                    case 0x42:
                    printf("VT100:DOWN\n");
                    break;
                    case 0x43:
                    printf("VT100:RIGHT\n");
                    break;
                    case 0x44:
                    printf("VT100:LEFT\n");
                    break;
                }
            }
        }
    }        
return(0);
}
