/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:08/02/95                       */
/* Program:SEI_ACT                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This software is designed to indicate the length of a mains power        */
/* cut by periodically storing the latest time string from the system clock.*/
/* This information is continually overwritten until a mains power failure  */
/* occurs.                                                                  */
/* When power is resumed, the first time from the system clock is stored in */
/* a fresh location. This leaves the last time before the power failure and */
/* the first time after the failure as stored records for reference by an   */
/* operator. The new overwritten times are stored in yest another fresh     */
/* storage area until another power failure occurs                          */
/* When the file contains 20 records of restarts, the file is copied to     */
/* /home/error/uplog_old and a new file is opened. This leaves a history of */
/* 40 restarts at any time.                                                 */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"           /* shared memory definitions                */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include "filedef.h"
#include <time.h>

FILE     *tims;                 /* Pointer to filename                      */
long     position;
int      first;                 /* Switch for first entry                   */

int      pid;                   /* Process id                               */
long     filsize;               /* Filesize                                 */

main()
{
time_t time_of_day;
auto char buf[26];
long int offset;
int      fd;                    /* return value open share                  */
smod     *ptr;                  /* pointer to shared region                 */
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"SEI_ACTI_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"SEI_ACTI_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}

/*--------------------------------------------------------------------------*/
/* Get process id for SEI_ACTI. The id is stored in the parameter           */
/* module for later use by KILLTSK.                                         */
/*--------------------------------------------------------------------------*/
ptr->pid_sei_acti=getpid();     /* Store in datamodule                      */

new_file:
    first=0;                    /* Switch to indicate first entry after     */
                                /* restart                                  */
    for(;;)                     /* Indefinite loop                          */
    {
        if(ptr->pid_sei_acti == 0)
        {
            printf("SEI_ACTI stopped\n");
            exit(0);
        }
        sleep(60);              /* Get new time every 60 seconds            */
        time_of_day=time(NULL);

        if(first==0)            /* Check if restart                         */
        {
            tims=fopen(UPSEILOG,"a"); /* Open old or new file               */
            filsize=ftell(tims); /* Get current filesize                    */
            offset=filsize;
            if(filsize >=1350)
            {
                fclose(tims);   /* Yes, close file                          */
                                /* Save file                                */
                system("cp /home/error/uplog /home/error/uplog_old");
//system("cp /home/error/uplog /usr/local/apache/htdocs/FILES/uplog_old");
                system("rm /home/error/uplog"); /* Delete old file          */
                goto new_file;  /* Open new file                            */
            }
            _ctime(&time_of_day,buf);
            buf[24]=' ';
            buf[25]=' ';
            fprintf(tims,"From : %s",buf);
            fprintf(tims,"Until: %s",_ctime(&time_of_day,buf));
            fclose(tims);
//system("cp /home/error/uplog /usr/local/apache/htdocs/FILES/uplog");
            first=1;            /* Turn switch                              */
        }
        tims=fopen(UPSEILOG,"a");
        filsize=ftell(tims);
        fclose(tims);
        tims=fopen(UPSEILOG,"r+");
        position=fseek(tims,offset+32,SEEK_SET);
        _ctime(&time_of_day,buf);
        buf[24]=' ';
        buf[25]=' ';
        fprintf(tims,"Until: %s\n",buf);
        fclose(tims);
//system("cp /home/error/uplog /usr/local/apache/htdocs/FILES/uplog");
    }
}



