/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:08/02/95                       */
/* Program:SEI_ACT                                                          */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* This software is designed to indicate the length of a mains power        */
/* cut by periodically storing the latest time string from the system clock.*/
/* This information is continually overwritten until a mains power failure  */
/* occurs.                                                                  */
/* When power is resumed, the first time from the system clock is stored in */
/* a fresh location. This leaves the last time before the power failure and */
/* the first time after the failure as stored records for reference by an   */
/* operator. The new overwritten times are stored in yest another fresh     */
/* storage area until another power failure occurs                          */
/* When the file contains 20 records of restarts, the file is copied to     */
/* /home/error/uplog_old and a new file is opened. This leaves a history of */
/* 40 restarts at any time.                                                 */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"           /* shared memory definitions                */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <time.h>

FILE     *web;
long     position;
int      ret;
int      first;                 /* Switch for first entry                   */
int      SERVER;
int      pid;                   /* Process id                               */
long     filsize;               /* Filesize                                 */
int      fd;                    /* return value open share                  */
smod     *ptr;                  /* pointer to shared region                 */

main()
{
time_t time_of_day;
auto char buf[26];
long int offset;
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"SEI_ACTI_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"SEI_ACTI_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
SERVER=0;              // SLINGER
//SERVER=1;              // APACHE
sleep(20);
new_file:
    first=0;                    /* Switch to indicate first entry after     */
                                /* restart                                  */
    for(;;)                     /* Indefinite loop                          */
    {
        ret=active();
        if(ret == -1)
        {
            printf("SEI_ACTW stopped\n");
            exit(0);
        }
        sleep(20);              /* Get new time every 60 seconds            */
        time_of_day=time(NULL);

        if(first==0)            /* Check if restart                         */
        {
            if(SERVER==0)
            {
              web=fopen("/usr/webdoc/uplog1","a");
            }else{
              web=fopen("/usr/local/apache/htdocs/uplog1","a");
            }
            filsize=ftell(web); /* Get current filesize                    */
            offset=filsize;
//printf("FILESIZE %d\n",filsize);
            if(filsize < 5)
            {
              fclose(web);
              header();
              goto new_file;
            }
            if(filsize >=1700)
            {
                fclose(web);   /* Yes, close file                          */
                                /* Save file                                */
                if(SERVER==0)
                {
                  system("cp /usr/webdoc/uplog1 /usr/webdoc/uplog1_old");
                  system("rm /usr/webdoc/uplog1"); /* Delete old file          */
                }else{
                  system("cp /usr/local/apache/htdocs/uplog1 /usr/local/apache/htdocs/uplog1_old");
                  system("rm /usr/local/apache/htdocs/uplog1"); /* Delete old file          */
                }
                header();
                goto new_file;  /* Open new file                            */
            }

            _ctime(&time_of_day,buf);
            buf[24]=' ';
            buf[25]=' ';
            fprintf(web,"<b>From</b> : %s",buf);
            fprintf(web,"<b>   Until</b>: %s<br>",_ctime(&time_of_day,buf));
            fclose(web);
            first=1;            /* Turn switch                              */
        }
        if(SERVER==0)
        {
          web=fopen("/usr/webdoc/uplog1","a");
        }else{
          web=fopen("/usr/local/apache/htdocs/uplog1","a");
        }
        filsize=ftell(web);
        offset=filsize;
        fclose(web);
//printf("SIZE %6d\n",offset);
//exit(0);
        if(SERVER==0)
        {
          web=fopen("/usr/webdoc/uplog1","r+");
        }else{
          web=fopen("/usr/local/apache/htdocs/uplog1","r+");
        }
        position=fseek(web,offset-48,SEEK_SET);
        _ctime(&time_of_day,buf);
        buf[24]=' ';
        buf[25]=' ';
        fprintf(web,"<b>   Until</b>: %s<br>\n",buf);
        fclose(web);
    }
}
active()
{
int i;
    for(i=0;i<7;i++)
        ptr->alive[i]=0;               /* zero all indicators               */
    sleep(2);                         /* sleep for 15 seconds              */
//    printf("%d %d %d %d\n",ptr->alive[0],ptr->alive[1],ptr->alive[2],ptr->alive[3]);               /* check if BUF_PIPE is alive ?      */
    if(ptr->alive[0]==1 && ptr->alive[1]==1 && ptr->alive[2]==1 && ptr->alive[3]==1)               /* check if BUF_PIPE is alive ?      */
    {                                  /* no, do something                  */
        return(1);
    }else{
        return(-1);
    }
}

header()
{
        if(SERVER==0)
        {
          web=fopen("/usr/webdoc/uplog1","a");
        }else{
          web=fopen("/usr/local/apache/htdocs/uplog1","a");
        }

                fprintf(web,"<html>\n");
                fprintf(web,"<head>\n");
                fprintf(web,"<meta http-equiv=""Content-Type"" content=""text/html; charset=iso-8859-1"">\n");
                fprintf(web,"</head>\n");

                fprintf(web,"<body bgcolor=""#000000"" text=""#FFFF00"" >\n");
                fprintf(web,"<p><font face=""Arial, Helvetica, sans-serif""><b><font size=""2"">SEISWEB</font></b></font></p>\n");
                fprintf(web,"</body>\n");
                fprintf(web,"</html>\n");
                fprintf(web,"<p>\n");
                fclose(web);
                if(SERVER==0)
                {
                  system("chmod 777 /usr/webdoc/uplog1");
                }else{
                  system("chmod 777 /usr/local/apache/htdocs/uplog1");

                }
             
}