/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:04/08/95                       */
/* Program:SHAR_MEM                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/*                                                                          */
/****************************************************************************/     

#include  <stdio.h>
#include  "version.h"
#define   ERROR        -1

#define   MODNAM       "shareblock"
#define   ERRNAM       "err_event"
#define   DETNAM       "det_event"
#define   PLSNAM       "pls_event"
#define   RD3NAM       "rd3_event"
#define   PLTNAM       "plt_event"
#define   RDYNAM       "rdy_event"
#define   CATNAM       "cat_event"
#define   CTPNAM       "cat_p_evt"

#define   HEADER       16          /* space for 16 4 byte words in buffers  */
#define   EVTREC       512         /* record size detection records         */
#define   EVTSIZ       1000        /* number of records in detection file   */
#define   SIZ_EV_HDR   1024        /* length of event header (bytes) NEW    */
#define   DIRNAMLEN    40          /* max length of directory path   NEW    */
#define   EVNAMLEN     25          /* length of an event filename    NEW    */
#define   ERRSIZ       256         /* record size errorlog                  */
#define   ERRREC       100         /* number of records in errorlog         */
#define   BUFFER       3216        /* total number of elements in buffers   */
#define   PIPSIZ       128640      /* total number of elements in chain     */
#define   NUMBUF       2           /* number of buffers                     */
#define   MAXPAR       5           /* maximum NUMBER of parameter files     */
#define   NUMCONT      1           /* number of continous buffers    NEW    */
#define   NUM_OTHERS   (NUMCONT+1) /* manual + continous        NEW (MOD)   */
#define   MANUAL       MAXPAR      /* index for manual trigger              */
#define   CONTIN       (MAXPAR+1)  /* index for continous trigger           */
#define   MAXSET       (MAXPAR+NUM_OTHERS) /* total number of indexes needed*/
#define   EXTRACT      (MAXSET+1)  /* index for extract-program             */
#define   EXTBUFSIZE   500         /* maximum number of buffers to extract  */
#define   MAXCHA       64          /* maximum number of channels            */
#define   MAXQUE       20          /* maximum number of events in queue     */
#define   MEGA         1000000     /* one mega                              */
#define   KILO         1000        /* one kilo                              */
#define   MAXPOL       5           /* max number of poles for filter        */
#define   MAXNUMRINGBUF 1500       /* max number of allowed ring bufs NEW   */
#define   SHORT        1           /* these two definitions because   NEW   */
#define   LONG         2           /* the ring buffer is long or short NEW  */
                                   /* depending on the unit type            */
#define   VERS         2
typedef   struct  {
                          int ibuf [NUMBUF][BUFFER]; /* Data-buffers        */ 
                          float b1[BUFFER];
                          float b2[BUFFER];
                          int buff[BUFFER];
                          short vadc[BUFFER];
                          unsigned int pipe [PIPSIZ];
                          short mseconds[1000];
                          long  totsecon[1000];
                          char tider[2000][80];
/*--------------------------------------------------------------------------*/
/*           DATA-STRUCTURE FOR PARAMETER-MODULE                            */
/*--------------------------------------------------------------------------*/
char      Eventdir[40];     /* directory for events                  NEW    */
char      Bufdir[NUMCONT][40]; /* directory to keep continous rec.in NEW    */
char      ringnames[NUMCONT][MAXNUMRINGBUF][EVNAMLEN+17];    /*      NEW    */
                            /* array to hold names of ring buffer files     */
int       nringbuf[NUMCONT]; /* number files in ring buffer          NEW    */
int       ringidx[NUMCONT]; /* pointer to current ringbuffer file    NEW    */
int       rng_idx_old[NUMCONT]; /* pointer to previous ringbuffer file      */
int       tot_ring_size[NUMCONT]; /* total bytes for ring buffer     NEW    */
int       nmin_per_ring[NUMCONT]; /* number of minutes each rng.buf  NEW    */
int       cont_dat;         /* switch to start program for continous record */
int       rng_bck;          /* switch to turn on/off backup routine         */
int       sec_in;           /* Index input into buffer                      */
int       sec_ut;           /* Index output from buffer                     */
int       sec_in_pip;       /* Total number of seconds in pipe              */
int       kueue;            /* Number of buffers queued up, -1 empty        */
char      list[2000];       /* List of flags 0 empty/read 1 full/written to */
int       colors[50];       /* current color selection for plotting         */
int       year;             /* Year, for Kinemetrics                        */
char      name[20];         /* Station name                                 */
int       mintrg[MAXSET];   /* Minimum number of channels with trigger      */

int       prevnt[MAXSET];   /* Preevent memory in buffers                   */
short     maxprevt;         /* variable to hold thw maximum pre-event in par*/
int       premax[MAXSET];   /* array propagation window in buffers          */

int       postmx[MAXSET];   /* Post event memory in buffers                 */

int       maxbuf[MAXSET];   /* Maximum number of buffers to be saved        */

int       maxbuf_HD[MAXSET]; /* Maximum number of buffers to be saved HD     */

int       durlim[MAXSET];   /* Maximum allowed duration                    */
int       nchstr[MAXSET];   /* Number of channels to store each par set     */
int       newrl[MAXSET];    /* New reclength for less channels to store     */

int       nchan;            /* Number of channels used                      */
int       rate;             /* Sample rate in integer frequency             */
float     srate;            /* Samplrate in float seconds                   */
int       intclk;           /* Switch for internal or external clock        */
int       clktyp;           /* Type of clock used in configuration          */
int       unit;             /* Type of A/D unit in configuration            */
int       pip_dsk;          /* Indicator recording from disk or pipe        */
int       sec_buffer;       /* Seconds of data to fill into buffers         */
int       buf_circle;       /* Number of buffers to write hard disk         */
float     delay;            /* Delay factor in seconds for RD3              */
int       maxblk;           /* Number of records in datafile DATA           */
int       maxblk_HD;        /* Number of records in datafile DATA on RAM    */
int       p_phas;           /* Indicator to turn on/off p-phase recording   */
int       adstrap;          /* A/D strapping. 5 or 10 volts                 */
int       active;           /* Active sets of parameters                    */ 
int       clk_chk;          /* switch to turn on/off printout from clock    */
int       dsk_lev;          /* Percentage level where to start auto delete  */
int       bufsiz;           /* Memory buffer size in 2 byte words           */
int       nblka;            /* Number of records in detection file          */

char      statio[MAXCHA*4];     /* Station codes of individual channels     */
char      compon[MAXCHA*4];     /* Component codes of individual channels   */
int       gain[MAXCHA];         /* A/D converter gain in each sample        */
float     fltlo[MAXSET][MAXCHA];  /* Low pass or difference                 */
float     flthi[MAXSET][MAXCHA];  /* Hi pass                                */
int       filter[MAXSET][MAXCHA]; /* Number of points in difference calc.   */
float     dc[MAXCHA];             /* DC offsets each channel                */
float     to[MAXCHA];             /* abs totals each channel                */
float     o_nsta[MAXSET][MAXCHA]; /* Number of seconds in STA               */
int       nsta[MAXSET][MAXCHA];   /* Number of samples in STA               */
float     sta[MAXSET][MAXCHA];    /* Short term average                     */
float     stamx[MAXSET][MAXCHA];  /* max sta in a buffer              NEW   */
   
float     o_nlta[MAXSET][MAXCHA]; /* Number of seconds in LTA               */
int       nlta[MAXSET][MAXCHA];   /* Number of samples in LTA               */
float     lta[MAXSET][MAXCHA];    /* Long term average                      */

float     ratio[MAXSET][MAXCHA];  /* Trigger ratio                          */
int       cntdur[MAXSET];         /* Counting blocks when continous         */
int       preblk[MAXSET];         /* Counting blocks from first trigger     */
int       postbk[MAXSET];         /* Counting blocks from end of trigger    */
int       ndet[MAXSET];       /* Current number of channels with trigger    */
int       ndetw[MAXSET];      /* No. of channels with trigger within window */
int       singdt[MAXSET][MAXCHA]; /* Single channel detection fla           */
int       singdw[MAXSET][MAXCHA]; /* Sing chan. det flags within window     */
int       chaflg[MAXSET][MAXCHA]; /* Channels used by trigger 1-64          */
int       nchdet[MAXSET];         /* Number of channels used by detector    */

float     postrt[MAXSET][MAXCHA]; /* Post event detection ratio             */
int       store[MAXSET][MAXCHA];  /* Number of channels to store            */
float     snc[MAXSET][MAXCHA][16]; /* Filter coefficients                   */
float     sdc[MAXSET][MAXCHA][16]; /* Filter coefficients                   */
int       cset[MAXSET][MAXCHA];    /* Number of sets of coefficients        */

int       flg_RAM;           /* Flag datastream to RAM disk                 */
int       flg_Hdisk;         /* Flag datastream to Hard-disk                */
int       flg_Detect;        /* Flag datastream to Detection algorithm      */
int       flg_Plot;          /* Flag datastream to Plot                     */
int       share_siz;         /* Size of shared memory buffer                */
int       reclength;         /* Recordlength bytes for current configuration*/
int       Data_Size;         /* Size of DATA file in k bytes                */
int       Event_Size;        /* Size of EVENT file in k bytes               */
int       Fast_Size;         /* Size of RAM disk for fast circle buffer     */
int       discspace;         /* Space left on disc                          */
int       a;                 /* counter parameter sets                      */


unsigned  stat[MAXCHA];
unsigned  comp[MAXCHA];
int       rec_size_bytes;    /* The current recordsize for current unit     */
int       header;            /* Size of header in buffer 2-byte words       */
int       reclength_HD;      /* Size of records to harddisk circle          */
int       displm;            /* Displacement of first sample in buffer      */
int       nsamp;             /* Number of samples pr channel in buffer      */
int       maxque;            /* Max number of events in queue               */
int       recrd[MAXSET];     /* Recording flag: 1:ON  0:OFF                 */
int       queue;             /* Number of events in queue                   */
int       pqueue;            /* Number of p-phase events in queue           */
int       ntick;             /* Number of ticks to sleep                    */
short     ptick;
int       oktime;            /* Flag to indicate correct time               */
int       pid_buf_pipe;      /* BUF_PIPE                                    */
int       pid_cat_evts;      /* CAT_EVTS                                    */
int       pid_timing;        /* Timing program                              */
int       pid_detec_01;      /* DETEC_01                                    */
int       pid_messages;      /* MESSAGES                                    */
int       pid_nano_rd3;      /* NANO_RD3                                    */
int       pid_nano_hrd;      /* NANO_HRD                                    */
int       pid_ED24;          /* ED24                                        */
int       pid_sei_acti;      /* SEI_ACTI                                    */
int       pid_cpu_acti;      /* CPU_ACTI                                    */
int       pid_re_start;      /* RE_START                                    */
int       pid_security;      /* SECURITY                                    */

int       detflg[MAXSET];     /* Network event trigger flag                 */

int       mtrig;              /* Manual trigger, 1:ON  0:OFF                */
int       ctrig;              /* Continous trigger, 1:ON 0:OFF              */
int       itmde;              /* Interrupt: counting milliseconds           */
int       icmde;              /* copy of itmde at the time of request       */
int       cur_prv;            /* switch to indicate current or previous     */
long      hrn_c;
long      min_c;
long      sec_c;
long      yrn_c;              /* current and counting                       */
long      mon_c;
long      day_c;
long      doy_c;
long      hrn_l;
long      min_l;
long      sec_l;
long      yrn_l;              /* current and locked                         */
long      mon_l;
long      day_l;
long      doy_l;

int       index;              /* Index to tell which buffer is ready        */
int       buf_rdy;            /* index to buffer just filled with data      */
int       buf_rdy_fpl;        /* index to buffer just filed into pipe       */
int       blkinc;             /* Number of 512b disk blocks pr memory buffer*/
int       nblkb;              /* Number of diskblks in small circular buffer*/

int       idle;               /* Flag to indicate error in parameter        */

int       factor;             /* Amplification factor for synt_dat          */

int       pipoint;            /* Pointer in p-phase event queue             */
int       ipoint;             /* Pointer in event queue                     */
int       extclk;             /* Ind. for ext.clock: 0:NONE, 1:Radio Code   */  
int       iblk;               /* Current large disk buffer writing address  */
int       exrate;             /* External oscillator rate                   */
int       qbuffile[MAXQUE];   /* index of which rng.buf was the trigger NEW */
int       qfblk[MAXQUE];      /* Disk blk # of first blk of buffer with det.*/
int       qHDblk[MAXQUE];     /* Disk blk # of first blk of buffer with det.*/
                              /* for the hard disk circle buffer            */
short     qpsec[MAXQUE];
short     qfsec[MAXQUE];
short     bgblk[MAXQUE];
double    secsrt[MAXQUE];
int       qdur[MAXQUE];       /* Duration in buffers or samples of event    */
int       qfsmp[MAXQUE];      /* Det. time in samples from start of buffer  */
short     qpsmp[MAXQUE];
int       qamp[MAXQUE];       /* Max amplitude of event , perhaps..         */
float     qsta[MAXQUE];       /* Max STA of event                           */
float     qlta[MAXQUE];       /* Max LTA of event                           */

int       timcor;             /* Clock time corr. sec to be added to clock  */
float     timerr;             /* estimated clock max error in seconds       */

int       time[32];           /* Timestring from Radiocode clock            */
char      errbuf[512];        /* Buffer cont. last message from application */

int       errorno;            /* Record number next message                 */
int       reset_system;
int       noktime;            /* No. of ok times in sequence since startup  */
int       sinceok[5];         /* Number of ok times at detection            */
char      timcpy[64];         /* Copy of raw time rec. from Radiocode clock */
int       errind[64];         /* Array to ind. position of error in timestr.*/
short     lost;               /* Lost seconds when reading from RD3         */
int       windx;              /* Index of current buffer                    */
int       good_times;         /* No. of correct times received in initialize*/
int       TimFlg[20];         /* Flgs to for track of time received from clk*/
short     ClkFlg_01;
short     ClkFlg_02;
short     ClkFlg_03;
short     ClkFlg_04;
short     ClkFlg_05;
char      Timsav[80];         /* Storage for copy of timestring             */
short     alive[20];          /* Each process updates to say I'm alive      */
unsigned  int next_second;    /* next synchronized second                   */
unsigned  int sync_clock;     /* software clock counting synchronized secs  */
int       latgr;              /* Latitude grades                            */
float     latm;               /* Latitude minutes                           */
char      n_s;                /* North-South                                */
int       longr;              /* Longitude grades                           */
float     lonm;               /* Longitude minutes                          */ 
char      e_w;                /* East-West                                  */
int       pdur[MAXQUE];       /* For monitoring duration                    */
int       dplc_proxy;         /* proxy for DPL Console                      */
int       dpls_proxy;         /* proxy for DPL Serial                       */
int       rtpc_proxy;         /* proxy for RTP Console                      */
int       rtps_proxy;         /* proxy for RTP Serial                       */
int       rat_proxy;          /* proxy for RAT                              */
int       mon_proxy;          /* proxy for MON                              */
int       fplc_proxy;         /* proxy for FPL Console                      */
int       fpls_proxy;         /* proxy for FPL Serial                       */
int       top_hdr_bck_col;    /*Background top header box                   */
int       top_hdr_txt_col;    /*Text       top header box                   */
int       top_hdr_brd_col;    /*Border     top header box                   */
int       top_hdr_plt_col;    /*Plot       top header box                   */
int       sid_lab_bck_col;    /*Background side label box                   */
int       sid_lab_txt_col;    /*Text       side label box                   */
int       sid_lab_brd_col;    /*Border     side label box                   */
int       sid_lab_plt_col;    /*Plot       side label box                   */
int       plt_are_bck_col;    /*Background plot area  box                   */
int       plt_are_txt_col;    /*Text       plot area  box                   */
int       plt_are_brd_col;    /*Border     plot area  box                   */
int       plt_are_plt_col;    /*Plot       plot area  box                   */
int       tim_sca_bck_col;    /*Background timing line                      */
int       tim_sca_txt_col;    /*Text       timing line                      */
int       tim_sca_brd_col;    /*Border     timing line                      */
int       tim_sca_plt_col;    /*Plot       timing line                      */
} smod;

/* Typedef for the shared module - combines common module elements with     */
/* an entry point for the start of the data area */

/* End of header file */
