/*
	Program GURALP2sac
 	Written by Raul Madariaga ENS
		September  10, 1999

	from segy2sac by  Tim Ahern, IRIS and Jim Fowler
	        1988 vintage
       

*/


#include "shar_mem.h"
#include <stdio.h>	/* Standard I/O header file			*/
#include <ctype.h>
#include <string.h>
#include <stddef.h>
#include <fcntl.h>
#include "sac.h"
#define M2(arr,x,y)(*((arr)+(y)*(dimx)+(x)))

/*
  DEFINE STATIC DATA FOR THE HEADER
  this should normally be read from the header but it does not work yet 
*/
int year, julian, hour, minute, second, millisec;
float latitude, longitude;
float *matr;
//void wrtsac(char *, char *, float, int,float *,int);
main (argc, argv)    
    int		argc;
char		*argv[];
{
char head[1200];
long s_year,s_doy,s_month,s_day,s_hour,s_min;
int kbuf;
float valfl;
int isamp;
int iworkl[20000];
int pp;
int ttotal;
float secf;
float s_sec;
int s_nchan;
float s_srate;
int s_ntotal;
int s_unit;
int s_nsamp;
int s_reclength;
int s_recsize;
int s_pre;
int dimx,dimy;
   FILE	*fpin, *fpout;			/* Pointer to the input file  */

   int i, j, l, int_hold, num_sam, num_lin,sam_rate;
   short short_hold;
   unsigned char *line,*cptr, *cptr1,*cptr2,*cptr3, *cptr4, *filename, *fileout;
   unsigned char unit[10];
   unsigned char FileIN[64], FileOUT[64], linea[80];
   float dt, ar[50000]; 
   int ifile, valor;
   float scale ;
   int gain, sclflg ;
   
   /* check options flags */
   sclflg = 0 ;   /* default to scale data */
   while (argc > 1 && argv[1][0] == '-'){
       switch (argv[1][1]){
       case 'g' :  /* -g no gain to be applied */
	   sclflg = 1 ;
	   break ;
       }
       argc-- ;
       argv++ ;
   }
   
for (i=0 ; i<64 ; i++) FileIN[i] = 0;  /* zero input filename               */
if(argc == 1)
{
    fprintf(stderr," Usage %s [options] file_name(s) \n", argv[0]);
    exit(1);
}
//  process input files one at a time

for (ifile =1; ifile <argc; ifile++)
{
    strcpy( FileIN, &argv[ifile][0] );
printf("Input filename: %s\n",FileIN);
    if( strcmp(FileIN,"sac") == 0)continue; /* don't work on a previous directORY */
    fprintf(stderr,"process %s \n",FileIN);       

//  open the input file and go to the output directory	

    if (( fpin = fopen(FileIN, "rb")) == NULL )
    {
	    printf( "\n ERROR: could not  open %s for input", FileIN);
        fclose(fpin);
        exit(1);
    }

//  Fill up the header variables by hand 
    fseek(fpin,0,0);
    fread(head,1024,1,fpin);
    hdr_info(head,&s_year,&s_doy,&s_month,&s_day,&s_hour,&s_min,&s_sec,&s_nchan,&s_srate,&s_ntotal,&s_unit,&s_nsamp,&s_recsize,&s_reclength,&s_pre);

    printf("%4d/%2d/%2d %2d:%2d:%6.3f doy:%3d chan:%2d sr:%6.3f ntot:%6d recl:%5d\n",
    s_year,s_month,s_day,s_hour,s_min,s_sec,s_doy,s_nchan,s_srate,s_ntotal,s_reclength);

    year=s_year;                       /* the year                          */
    julian=s_doy;                      /* the julian day                    */
    hour=s_hour;                       /* the hour                          */
    minute=s_min;                      /* the minute                        */
    second=(int)s_sec;                 /* the seconds                       */
    secf=(float)second;
    millisec=(int)((s_sec-secf)*1000); /* the milliseconds                  */
    latitude = 33.457;
    longitude = 70.6639;
    dt=1./s_srate;
    ttotal=s_ntotal/s_nsamp;         /* total number of seconds             */
    dimx=s_nchan;
    dimy=s_ntotal;
    if(!(matr=(float *)malloc(dimx*dimy*sizeof(float)) ))
    {
        printf("Out of memory\n");
        exit(0);
    }

    isamp=0;
    for(kbuf=(1024/s_reclength)+1;kbuf < ttotal+(1024/s_reclength)+1;kbuf++)
    {
        fseek(fpin,kbuf*s_reclength,SEEK_SET);
        fread(iworkl,s_reclength,1,fpin);
        dmx_buf(dimx,isamp,s_nsamp,s_nchan,ttotal,iworkl);
        isamp++;
    }
    fclose(fpin);
//printf("ISAMP= %d NSAMP=%5d\n",isamp,s_nsamp);
    
    for(j=0;j<s_nchan;j++)
    {
        pp=0;
        for(i=0;i<(isamp*s_nsamp);i++)
        {
            valfl=M2(matr,j,i);
            ar[pp]=valfl;
            pp++;
        }

        scale = 1. ;                       /* counts to cm/s  */
        num_sam = pp; 
        switch(s_unit)
        {
            case 10:
            sprintf(unit,"GBV-316\0");
            break;
            case 30:
            sprintf(unit,"DAS-1202\0");
            break;
            case 31:
            sprintf(unit,"IDS-24\0");
            break;
            case 32:
            sprintf(unit,"GeoSys-12\0");
            break;
            case 36:
            sprintf(unit,"DAS-1802\0");
            break;
            case 38:
            sprintf(unit,"R.Ortiz\0");
            break;
            case 40:
            sprintf(unit,"HRD-24\0");
            break;
            case 41:
            sprintf(unit,"Guralp\0");
            break;
            case 53:
            sprintf(unit,"RD-3\0");
            break;
            case 56:
            sprintf(unit,"RD-6\0");
            break;
            case 63:
            sprintf(unit,"Earth Data\0");
            break;
            case 64:
            sprintf(unit,"Earth Data\0");
            break;
            case 96:
            sprintf(unit,"SYNTH.\0");
            break;
            case 99:
            sprintf(unit,"ILI\0");
            break;

        }
for(i=0;i<50;i++)
    fileout[i]=0x20; 
sprintf(fileout,"/home/work/%4d_%3d_%2d%2d%2d_%3d.sac\0",
year,julian,hour,minute,second,j);
for(i=11;i<30;i++)
    if(fileout[i]==0x20) 
        fileout[i]=0x30;
        printf("output file %s\n",fileout); 
//        wrtsac(fileout,"GURALP",dt,num_sam,ar,0); 
        wrtsac(fileout,unit,dt,num_sam,ar,0); 
    }
}
}



/*********************************************************************/
/*                   wrtsac (FO,sta,dt,ns,ar,flg)                    */
/*********************************************************************/
/*          Routine to write SAC files                               */
/*		FO = output file name                                */
/*              sta = station name                                   */
/*		dt = sample rate in samples per second               */
/*		ns = number of data samples                          */
/*		ar = data array                                      */
/*		flg = output flag                                    */
/*		    = 0 output in binary format                      */
/*		    = 1 output in asci format                        */
/*                                                                   */
/*	    Program will add ".sac" or ".asc" extension to output    */
/*	    File name					             */

void wrtsac (FO,sta,dt,ns,ar,flg)
    char  FO[],*sta;
//    char  FO[],sta[];
int	ns, flg ;
float	dt, ar[] ;
{
    struct  sac	sp, *sp_hdr ;
    int	j, n0 ;
    float	zero = 0.0 ;
    float   t_beg, t_event, t_calc;
    FILE	*iopo ;
//    if (flg == 0){
//	strcat (FO,".sac") ;
//    }else{
//	strcat (FO,".asc") ;
//    }
    if((iopo = fopen(FO,"w")) == NULL){
	printf("cannot open output file %s\n",FO) ;
	exit (1) ;
    }
    /* 
   convert parameters to sac header parameters 
    */
    sp = sac_null;
    sp_hdr = &sp;
    strcpy(sp_hdr->kstnm, sta);
    /* floats */
    sp.delta = 1.0/dt ;
    sp.b = 0.0 ;
    sp.e = ns * sp.delta ;
    
    /* integers */
    sp.internal4 = 6;	/* LLL sets it */
    sp.internal5 = 0;	/* LLL sets it */
    sp.internal6 = 0;	/* LLL sets it */
    sp.npts = ns ;
    sp.iftype = ITIME ;
    sp.iztype = IB ;
    
    /* logicals  */
    sp.leven = TRUE ;
    sp.lpspol = FALSE ;     /* LLL sets it */
    sp.lcalda = TRUE ;      /* LLL sets it */
    sp.unused27 = FALSE ;   /* LLL sets it */
    
    /* add some SEGY stuff   */
    /* 				first some time information */
    
    sp.nzyear = year;	/* the year */
    sp.nzjday = julian;	/* the julian day */	
    sp.nzhour = hour;	/* the hour */	
    sp.nzmin =  minute;	/* the minute */	
    sp.nzsec =  second;	/* the second */	
    sp.nzmsec = millisec;	/* the milliseconds */	
    
    /*				get the trigger time now  */
    t_beg = hour* 3600.0 + minute*60.0 + (float) second + (float) millisec/ 1000.0;
    /* T_event should be defined from input data */
    t_event = t_beg; 
    t_calc = t_event - t_beg;
    if(t_calc < 0){ t_calc += 24.0 * 3600.0;}  /*handle the jump over a day */   
    sp.a = t_calc;
    
    /*
      now some coordinate information   
    */
    sp.stla = (float) ( latitude);
    sp.stlo = (float) ( longitude);
    if (flg == 0) {/* write binary sac file header */
	if (fwrite(&sp, sizeof(struct sac), 1, iopo)!=1){
	    printf("FATAL ERROR: bad  write %s, header\n",FO);
	    exit(4);
	}
	    
	/* write floating point data */
	if (fwrite(&ar[0],ns*sizeof(float),1,iopo)!=1){
	    printf("FATAL ERROR: bad  write %s, data\n",FO);
	    exit(5);
	}
	
	/* zero pad to 512 byte blocks (128 4 byte floats) */
	n0 = 128-(sizeof(struct sac)/4+ns)%128;
	for (j=0; j<n0; j++){
	    if (fwrite(&zero, sizeof(float), 1, iopo)!=1){
		printf("FATAL ERROR: bad  write %s, padding\n"
		       ,FO);
		exit(6);
	    }
	}
    }else{
	
	/* print header in ascii */
	fprintf(iopo, FCS, sp.delta, sp.depmin, sp.depmax, sp.scale, 
		sp.odelta);
	fprintf(iopo, FCS, sp.b, sp.e, sp.o, sp.a, sp.internal1);
	fprintf(iopo, FCS, sp.t0, sp.t1, sp.t2, sp.t3, sp.t4);
	fprintf(iopo, FCS, sp.t5, sp.t6, sp.t7, sp.t8, sp.t9);
	fprintf(iopo, FCS, sp.f, sp.resp0, sp.resp1, sp.resp2, 
		sp.resp3);
	fprintf(iopo, FCS, sp.resp4, sp.resp5, sp.resp6, sp.resp7, 
		sp.resp8);
	fprintf(iopo, FCS, sp.resp9, sp.stla, sp.stlo, sp.stel, 
		sp.stdp);
	fprintf(iopo, FCS, sp.evla, sp.evlo, sp.evel, sp.evdp, 
		sp.unused1);
	fprintf(iopo, FCS, sp.user0, sp.user1, sp.user2, sp.user3, 
		sp.user4);
	fprintf(iopo, FCS, sp.user5, sp.user6, sp.user7, sp.user8, 
		sp.user9);
	fprintf(iopo, FCS, sp.dist, sp.az, sp.baz, sp.gcarc, 
		sp.internal2);
	fprintf(iopo, FCS, sp.internal3, sp.depmen, sp.cmpaz, 
		sp.cmpinc, sp.unused2);
	fprintf(iopo, FCS, sp.unused3, sp.unused4, sp.unused5, 
		sp.unused6, sp.unused7);
	fprintf(iopo, FCS, sp.unused8, sp.unused9, sp.unused10, 
		sp.unused11, sp.unused12);
	fprintf(iopo, ICS, sp.nzyear, sp.nzjday, sp.nzhour, sp.nzmin, 
		sp.nzsec);
	fprintf(iopo, ICS, sp.nzmsec, sp.internal4, sp.internal5, 
		sp.internal6, sp.npts);
	fprintf(iopo, ICS, sp.internal7, sp.internal8, sp.unused13, 
		sp.unused14, sp.unused15);
	fprintf(iopo, ICS, sp.iftype, sp.idep, sp.iztype, sp.unused16,
		sp.iinst);
	fprintf(iopo, ICS, sp.istreg, sp.ievreg, sp.ievtyp, sp.iqual, 
		sp.isynth);
	fprintf(iopo, ICS, sp.unused17, sp.unused18, sp.unused19, 
		sp.unused20, sp.unused21);
	fprintf(iopo, ICS, sp.unused22, sp.unused23, sp.unused24, 
		sp.unused25, sp.unused26);
	fprintf(iopo, ICS, sp.leven, sp.lpspol, sp.lovrok, sp.lcalda, 
		sp.unused27);
	fprintf(iopo, CCS2, sp.kstnm, sp.kevnm);
	fprintf(iopo, CCS1, sp.khole, sp.ko, sp.ka);
	fprintf(iopo, CCS1, sp.kt0, sp.kt1, sp.kt2);
	fprintf(iopo, CCS1, sp.kt3, sp.kt4, sp.kt5);
	fprintf(iopo, CCS1, sp.kt6, sp.kt7, sp.kt8);
	fprintf(iopo, CCS1, sp.kt9, sp.kf, sp.kuser0);
	fprintf(iopo, CCS1, sp.kuser1, sp.kuser2, sp.kcmpnm);
	fprintf(iopo, CCS1, sp.knetwk, sp.kdatrd, sp.kinst);
	
	/* print data in ascii */
	for (j=0; j<ns; j++){
	    fprintf(iopo,"%15.7e", ar[j]);
	    if (j != 0 && (j+1)%5 == 0) fprintf(iopo, "\n");
	}
	fprintf(iopo, "\n\n");
    }
    
    /*    close file      */
    fclose (iopo) ;
//	fprintf(stderr,"return from wrtsac \n");    
}
hdr_info(head,year,doy,month,day,hour,min,sec,nchan,srate,ntotal,unit,nsamp,recsize,reclength,pre)
char*    head;
long    *year;
long     *doy;
long     *month;
long     *day;
long     *hour;
long     *min;
float    *sec;
int      *nchan;
float    *srate;
int      *ntotal;
int      *unit;
int      *nsamp;
int      *recsize;
int      *reclength;
int      *pre;
{
char     dum[100];
int      i;
//for(i=0;i<20;i++)
//    printf("%c ",head[i]);
//printf("\n");
sscanf(head,"%14c%4d%c%3d%c%2d%c%2d%c%2d%c%2d%c%f6.3\n",
dum,year,dum,doy,dum,month,dum,day,dum,hour,dum,min,dum,sec);
if(head[62]==' ')
{
    dum[0]=head[63];
    sscanf(dum,"%1d",nchan);
}else{
    dum[0]=head[62];
    dum[1]=head[63];
    sscanf(dum,"%2d",nchan);
}
for(i=0;i<16;i++)
    dum[i]=head[i+64];
sscanf(dum,"%f16.7",srate);
for(i=0;i<8;i++)
    dum[i]=head[i+80];
sscanf(dum,"%8d",ntotal);
dum[0]=head[96];
dum[1]=head[97];
sscanf(dum,"%2d",unit);
for(i=0;i<6;i++)
    dum[i]=head[i+122];
dum[i]='\0';
sscanf(dum,"%d",nsamp);
for(i=0;i<8;i++)
    dum[i]=head[i+104];
sscanf(dum,"%8d",recsize);
sscanf(dum,"%8d",reclength);
dum[0]=head[880];
dum[1]=head[881];
dum[2]=head[882];
dum[3]=head[883];
sscanf(dum,"%4d",pre);
/*printf("NCHAN:%2d SRATE:%16.7f NTOTAL:%8d UNIT:%2d NSAMP:%6d\n",\*/
/*nchan,srate,ntotal,unit,nsamp);*/
/*printf("RECSIZE:%8d RECLENGTH:%8d\n",recsize,reclength);*/
}

/*-----------------------------------------------------------------------*/
/* demultiplex one channel                                               */
/*-----------------------------------------------------------------------*/
dmx(isamp,nsamp,ichan,nchan,iworkl,data)
int isamp;
int nsamp;
int ichan;
int nchan;
int iworkl[];
float data[];
{
int i;
int bufsiz;

bufsiz=nsamp*nchan+HEADER;
for(i=HEADER+ichan-1;i<bufsiz;i=i+nchan)
{
    data[isamp]=(float)iworkl[i];
    isamp=isamp+1;
}
return(isamp);
}

dmx_buf(dimx,isamp,nsamp,nchan,ntotal,iworkl)
int dimx;
int isamp;
int nsamp;
int nchan;
int ntotal;
int iworkl[];
{
int i;
int j;
int s;
int bufsiz;
bufsiz=nsamp*nchan+HEADER;
for(i=0;i<nchan;i++)                   /* loop channels                     */
{
    for(j=0;j<nsamp;j++)               /* loop samples                      */
    {
        s=j+isamp*nsamp;
        M2(matr,i,s)=(float)iworkl[j*nchan+i+HEADER];
    }
}
}    

