/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 6.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55 213408 +47 55213419     */
/*                                      Fax  +47 55 233337 +47 55320009     */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date:23/01/95                       */
/* Program:SRT                                                              */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The SRT start data acquisition processes                                 */
/*                                                                          */
/****************************************************************************/
#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
FILE     *dnr;
int      fd;                           /* return value open share           */
smod     *ptr;                         /* pointer to shared region          */

main()
{
int contnr;
int flag;
contnr=0;

shm_unlink("SEISLOG");
/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"SRT_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"SRT_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
sprintf(ptr->err_msg,"-------------------------------------------");
errlog(ptr);
sprintf(ptr->err_msg,"SRT    : **** Data acquisition started ****");
errlog(ptr);
sprintf(ptr->err_msg,"-------------------------------------------");
errlog(ptr);
/*--------------------------------------------------------------------------*/
/* Stop data-acquisition if running                                         */
/*--------------------------------------------------------------------------*/
//system("on -t /dev/con1 /home/seislog/stp");
system("on -t /dev/con6 /home/seislog/stp");
//system("/home/seislog/stp");
/* put in a longer delay for slow machines                                  */
sleep(8);
system("/home/seislog/cls");
printf("\n");
printf("\n");
printf("\n");
printf("\n");
printf("\n");
printf("\n");
printf("                        SEISLOG Data Acquisition Started\n");
printf("                        --------------------------------\n");
system("on -t /dev/con1 /home/seislog/begynn");
/*--------------------------------------------------------------------------*/
/* Read the parameterfile and initialize some values in shared data module  */
/*--------------------------------------------------------------------------*/
system("on -t /dev/con6 /home/seislog/rd_param");
sleep(3);
if(ptr->web==1)
{
//  system("on -t /dev/con6 /home/seislog/jmon_s&");
//  system("on -t /dev/con6 /home/seislog/jall_s&");
//  system("on -t /dev/con6 /home/seislog/jrtp_s&");
//  system("on -t /dev/con6 /home/seislog/jlgs_s&");
  system("on -t /dev/con6 /home/seislog/sndfile&");
//  system("on -t /dev/con6 /home/seislog/jsts_s&");
  system("on -t /dev/con6 /home/seislog/jlgf_s&");
//  system("on -t /dev/con6 /home/seislog/fast&");
}
/*--------------------------------------------------------------------------*/
/* Start time correction routine                                            */
/*--------------------------------------------------------------------------*/
sleep(1);
system("rm -f /home/seislog/pr/contnr");
if((dnr=fopen("/home/seislog/pr/contnr","w"))==NULL)
{
    printf("*** RE_INITI *** Can't open new CONTNR file !!!!\n");
    printf("*** RE_INITI *** Exit and investigate\n");
    exit(0);
}
fprintf(dnr,"%8d\n",contnr);
fclose(dnr);


switch(ptr->clktyp)
{
    case 0:                            /* CPU timing only                   */
    system("on -t /dev/con6 /home/seislog/clk_cpu &");
    break;
    case 1:                            /* GARMIN 35-HVS No PPS              */
    system("on -t /dev/con6 /home/seislog/clk_gnp &");
    sleep(1);
    break;
    case 2:                            /* GARMIN 35-HVS PPS                 */
    system("on -t /dev/con6 /home/seislog/clk_gpp &");
    break;
    case 4:
    system("on -t /dev/con6 /home/seislog/clk_ms1 &");
    break;
    case 8:
    break;
}
sleep(5);
//if(ptr->unit==64 && ptr->clktyp==8)
//{
//    system("on -t /dev/con6 /home/seislog/clk_cpu&");
//}

system("on -t /dev/con6 /home/seislog/cat_evt &");
if(ptr->p_phas == 1)
    system("on -t /dev/con6 /home/seislog/cat_pph&");
if(ptr->cont_dat == 1)
    system("on -t /dev/con6 /home/seislog/tap_bck &");
system("on -t /dev/con6 /home/seislog/det_evt &");
system("on  -t /dev/con6 /home/seislog/buf_pip &");
//system("/home/seislog/buf_pip &");

//system("on -t /dev/con6 /home/seislog/tim_avg &");
//system("on -t /dev/con6 /home/seislog/sys_wat &");
if(ptr->unit==41)
    system("on -t /dev/con6 /home/seislog/gur_rst &");

system("on -t /dev/con6 /home/seislog/sei_act &");
if(ptr->web==1)
    system("on -t /dev/con6 /home/seislog/sei_acw&");
if(ptr->web==1)
    system("on -t /dev/con6 /home/seislog/web_mon&");
//system("on -t /dev/con6 /home/seislog/par_mrk &");
if(ptr->err_mail==1 || ptr->watch==1)
    system("on -t /dev/con6 /home/seislog/watchdog &");
system("on -t /dev/con6 /home/seislog/chk_par &");

}

sjekk()
{
char buf,buf2;
int dum;

int i;
FILE *ps;
ps=fopen("/home/seislog/ps.txt","r");
for(i=0;i<2000;i++)
{
  fread(&buf,1,1,ps);
  if(buf=='j')
  {
    fread(&buf2,1,1,ps);
    if(buf2=='m')
    {
      return(1);
    }else{
      goto goon;
    }
  }
goon:
dum=0;
}
return(0);
}