/****************************************************************************/
/*                       PC QNX SEISLOG Software              Version 7.0   */
/****************************************************************************/
/*                                                                          */
/*                                      University of Bergen                */
/*                                      Institute of Solid Earth Physics    */
/*                                      Seismological Observatory           */
/*                                      Allegaten 41, 5007 Bergen, Norway   */
/*                                      Tlf. +47 55583408 +47 55583419      */
/*                                      Fax  +47 55589669                   */
/*                                      Author: Terje Utheim, Jens Havskov  */
/*                                      E-mail: terje.utheim@ifjf.uib.no    */
/*                                              jens.havskov@ifjf.uib.no    */
/*                                                                          */
/*                                      Date: 01/08/98                      */
/* Program: TIM_AVG                                                         */
/*                                                                          */
/* Last update:                                                             */
/*                                                                          */
/* Program description:                                                     */
/*                                                                          */
/* The program is controlled by BUF_PIP. BUF_PIP sends out a proxy each time*/
/* a buffer has been written to the memory pipe with the time tag from the  */
/* clock. This routine reads this time tag and computes a running average   */
/* of the drift of the time. As the time stamping is a function of the      */
/* stability of the sampling frequency, there will normally be a small drift*/
/* caused by this instability.                                              */
/* When the time-tagging is based only on the reception of the time-string  */
/* from the clock, there will be rather big jumps in the time tagging. This */
/* in accuracy is then averaged over time and is more stable.               */
/* When the time-tagging is based on the reception of the time-string from  */
/* the clock AND the 1 PPS, the time tagging is very accurate, and the      */
/* averaging will give more or less the same value.                         */
/*                                                                          */
/****************************************************************************/

#include "shar_mem.h"                  /* shared memory definitions         */
#include <sys/mman.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <graph.h>
int      n;
char     bl[10];
char     buf[10];
int      l,k,i,lp,lc;
int      sec_in_pip;
//int      cnt;
int      acc;
float    scx;
float    scy;
char     times[10][80];
char     tider[80];
char     heltall[30];
int      mil[1000];


double   totsec[50];
double   sta1;
double   sta2;
double   sta3;
double   sta4;
double   first;
double   diff;
double   drift;
int      start,stopp;
double   MSECS;                        /* total seconds ringbuffer filename */
double   mint;                         /* smallest value indicating rngfile */
double   maxt;                         /* largest value indicating rngfile  */
char     karak[30];
float    SEC;                          /* seconds floating point            */
float    SECL;
double    new,gammel,forskjell;
int      inew,igammel,ifor;
int      YR,DOY,MTH,DAY,HR,MIN;        /* ringbuffer file date              */
int      hu;
//int      gpix;
//int      npix;
//int      dpix;
int      startlevel;
int      teller;
float    rlevel;
int      ilevel;
double   rtlevel;
double   slevel;
char     dum;
int      p;

main()
{
int      R;                            /* ratio sta/lta                     */
int      no_of_times;                  /* total number of buffers received  */
int      no_of_syncs;                  /* total number of synchronized times*/
//float    percent_snc;                  /* percent of time synchronized      */
char     sync;                         /* good time-bad time S-not S        */
int      snc;                          /* good time-bad time 1-0            */
int i;
pid_t    proxy;
unsigned int cop;
int buf_pip_id;
int sw;
char ka;
int tid_id;
unsigned msg;
int      fd;                                /* return value open share      */
smod     *ptr;                              /* pointer to shared region     */

no_of_times=0;                         /* total number of buffers received  */
no_of_syncs=0;                         /* total number of synchronized times*/

/*--------------------------------------------------------------------------*/
/* open shared memory region                                                */
/*--------------------------------------------------------------------------*/
fd=shm_open("SEISLOG",O_RDWR | O_CREAT, 0777);
if(fd== -1) 
{
    fprintf(stderr,"MON_PIPE_01 open failed:%s\n",strerror(errno));
    exit(1);
}
/*--------------------------------------------------------------------------*/
/* set size of memory region                                                */
/*--------------------------------------------------------------------------*/
ltrunc(fd,sizeof(smod),SEEK_SET);
/*--------------------------------------------------------------------------*/
/* Compute pointer to shared memory region                                  */
/*--------------------------------------------------------------------------*/
ptr=mmap(0,sizeof(smod),PROT_READ | PROT_WRITE,MAP_SHARED,fd,0);
if((int)ptr == -1) 
{
    fprintf(stderr,"MON_PIPE_02 mmap failed : %s\n",strerror(errno));
    exit(1);
}
buf_pip_id=qnx_name_locate(0,"buf_pip",0,&cop);
if(buf_pip_id==-1)
    exit(0);
proxy=qnx_proxy_attach(0,0,0,-1);
ptr->avg_proxy=proxy;
tid_id=qnx_name_attach(0,"tim_avg");
if(tid_id==-1)
    exit(0);
sw=0;
sec_in_pip=PIPSIZ/(ptr->nsamp*ptr->nchan+HEADER)-1;

teller=0;
acc=0;
for(;;)
{
printf("here I am\n");
    buf_pip_id=Receive(0,&msg,sizeof(msg));
///    if(kbhit())
//        exit(0);
    k=ptr->ClkFlg_05;                  /* the second number in memory pipe  */
    no_of_times++;                     /* total number of buffers received  */

//    sprintf(times[k],"%3d %s\n",k,ptr->pipe+k*((ptr->nchan*ptr->nsamp)+HEADER));
/*         Extract the time-string of the k-th buffer                       */
//    printf("%3d %s\n",k,ptr->pipe+k*((ptr->nchan*ptr->nsamp)+HEADER));
    sprintf(tider,"%3d %s\n",k,ptr->pipe+k*((ptr->nchan*ptr->nsamp)+HEADER));
    sync=tider[32];                    /* extract the sync/nosync indicator */
    if(sync=='S')                      /* check if synchronized or not      */
    {
        snc=1;                         /* clock synchronized                */
        no_of_syncs++;                 /* total number of synchronized times*/
    }else{
        snc=0;                         /* clock not synchronized            */
    }

/* meassure in percent reception of good GPS signal since last startup      */
printf("No of syncs %d No of times %d\n",no_of_syncs,no_of_times);

    ptr->percent_gps=((float)no_of_syncs * 100.)/(float)no_of_times;
//    ptr->percent_gps=percent_snc;
/* extract time and date of the k-th buffer                                 */

    sscanf(tider,"%3d%c%2d%c%2d%c%f6.3d",&hu,&dum,&HR,&dum,&MIN,&dum,&SECL);
    for(i=0;i<9;i++)
        tider[i]=tider[i+17];
    sscanf(tider,"%2d%c%2d%c%2d",&DAY,&dum,&MTH,&dum,&YR);
/* convert time and date to total number of seconds since 1.jan. 1970       */

    TIMSEC(YR,MTH,DAY,HR,MIN,SECL,&MSECS); /* current time in milliseconds  */
    if(sw==0)                          /* check if first time               */
    {
        rlevel=SECL;                   /* copy of seconds    ss.mmm         */
        ilevel=(int)rlevel;            /* take integer part: ss             */
/*                                        get milliseconds:  mmm            */
        startlevel= (int) (( rlevel-(float)ilevel)*1000);
//        gpix=startlevel;
//        printf("milliseconds of first buffer: %d\n",startlevel);
//sta1=(double)startlevel;
        sta1=MSECS;                    /* set average initially to seconds  */
sta2=MSECS;
sta3=MSECS;
sta4=MSECS;
                                       /* since 1.jan.1970 from first buffer*/
                                       /*          f.ex. 311234500.234      */
        gammel=MSECS;                  /* copy to gammel 311234500.234      */
//        cnt++;
        teller++;                      /* first time: 1                     */
        first=MSECS;                   /* first time     311234500.234      */
        sw=1;
        goto end;
    }
/*   not first time                                                         */

    rlevel=SECL;                       /* copy of seconds    ss.mmm         */
    ilevel=(int)rlevel;                /* take integer part: ss             */
/*                                     /* get milliseconds:  mmm            */
    startlevel=(int)((rlevel-(float)ilevel)*1000);

//    npix=startlevel;
//    dpix=gpix-npix;
    new=MSECS;                         /*copy of last seconds seconds since */
    forskjell=(double)(teller*1.);
    new=new-forskjell;
    sta1=sta1+(new-sta1)/100.;
    sta2=sta2+(new-sta2)/200.;
    sta3=sta3+(new-sta3)/400.;
    sta4=sta4+(new-sta4)/600.;
//printf("%f %f %f %f\n",sta1,sta2,sta3,sta4);
/******************* 1. STA ***************************/
    rtlevel=sta1;            /* 312345678.230 */
    sprintf(karak,"%12.3f\n",rtlevel);
for(i=0;i<20;i++)
{
    ka=karak[i];
    if(ka=='.')
        goto fin1;
    heltall[i]=ka;
    goto more1;
fin1:
    heltall[i]='\0';
    goto cont1;
more1:
    k=0;
}
cont1:
    sscanf(heltall,"%lf",&slevel);
    rtlevel=rtlevel-slevel;
    rtlevel=rtlevel*1000.;
    ilevel=(int)rtlevel;     /* 230           */
//    ptr->average1=(short)ilevel;

/******************* 2. STA ***************************/
    rtlevel=sta2;            /* 312345678.230 */
    sprintf(karak,"%12.3f\n",rtlevel);
for(i=0;i<20;i++)
{
    ka=karak[i];
    if(ka=='.')
        goto fin2;
    heltall[i]=ka;
    goto more2;
fin2:
    heltall[i]='\0';
    goto cont2;
more2:
    k=0;
}
cont2:
    sscanf(heltall,"%lf",&slevel);
    rtlevel=rtlevel-slevel;
    rtlevel=rtlevel*1000.;
    ilevel=(int)rtlevel;     /* 230           */
    ptr->average2=(short)ilevel;
/******************* 3. STA ***************************/
    rtlevel=sta3;            /* 312345678.230 */
    sprintf(karak,"%12.3f\n",rtlevel);
for(i=0;i<20;i++)
{
    ka=karak[i];
    if(ka=='.')
        goto fin3;
    heltall[i]=ka;
    goto more3;
fin3:
    heltall[i]='\0';
    goto cont3;
more3:
    k=0;
}
cont3:
    sscanf(heltall,"%lf",&slevel);
    rtlevel=rtlevel-slevel;
    rtlevel=rtlevel*1000.;
    ilevel=(int)rtlevel;     /* 230           */
    ptr->average3=(short)ilevel;
/******************* 4. STA ***************************/
    rtlevel=sta4;            /* 312345678.230 */
    sprintf(karak,"%12.3f\n",rtlevel);
for(i=0;i<20;i++)
{
    ka=karak[i];
    if(ka=='.')
        goto fin4;
    heltall[i]=ka;
    goto more4;
fin4:
    heltall[i]='\0';
    goto cont4;
more4:
    k=0;
}
cont4:
    sscanf(heltall,"%lf",&slevel);
    rtlevel=rtlevel-slevel;
    rtlevel=rtlevel*1000.;
    ilevel=(int)rtlevel;     /* 230           */
    ptr->average4=(short)ilevel;

//    printf("%5d N: %12.3f G: %12.3f AVG: %12.3f Count: %5d\n",startlevel,new,gammel,sta1,ilevel);
//    gammel=new;
    teller++;
end:
    i=0;
}
}    /* End of main */
TIMSEC (YR,MTH,DAY,HR,MIN,SECS,MSECS)                         
int YR;
int MTH;
int DAY;
int HR;
int MIN;
float SECS;
double *MSECS;                         /* total seconds to be returned      */
{
int DYR;                               /* flag for leap year                */
int IYR;                               /* number of leap-years since 1900   */
int YDY;                               /* number of days in current year    */

if(YR <= 50) YR=YR+100;                /* check for 2000 or more in 2 digits*/

if(YR >= 1900) YR=YR - 1900;
DYR = 0;
if((YR % 4) == 0) DYR = 1;
IYR = YR/4 - DYR;
//                -- Seconds to beginning of     
*MSECS = (float)(IYR*366) + (float)((YR-IYR)*365);
//                -- current year                
*MSECS = *MSECS*86400.0;
//                -- January                     
if (MTH == 1) YDY = DAY;
//                -- February                    
if (MTH == 2) YDY = DAY + 31;
//                -- ....                        
if (MTH == 3) YDY = DAY + DYR + 59;       
if (MTH == 4) YDY = DAY + DYR + 90;                                     
if (MTH == 5) YDY = DAY + DYR + 120;                                    
if (MTH == 6) YDY = DAY + DYR + 151;                                    
if (MTH == 7) YDY = DAY + DYR + 181;                                    
if (MTH == 8) YDY = DAY + DYR + 212;                                    
if (MTH == 9) YDY = DAY + DYR + 243;                                    
if (MTH ==10) YDY = DAY + DYR + 273;                                    
if (MTH ==11) YDY = DAY + DYR + 304;                                    
if (MTH ==12) YDY = DAY + DYR + 334;                                    
*MSECS = *MSECS + (float)(YDY*86400 + HR*3600 + MIN*60) + SECS;
}
